set define off

create or replace package body abstract_maint as

rpt_title$	constant varchar2(50)	:= 'Abstract Title/Author Maintenance';
rpt_title2$	constant varchar2(50)	:= 'Abstract Reclassification';
maint$		constant varchar2(5)	:= 'maint';
js$		constant varchar2(500)	:= '<script language="JavaScript" src="' ||
						common.server ||
						common.vdir$  || 'abstract_maint.js"></script>';
role$		constant function_codes.function_code%TYPE := 'RECL';
Q$		constant varchar2(1)	:= chr(39);

-- ****************************************************************************************
-- ****************************************************************************************

procedure done is

begin

   if common.header (rpt_title$) then
      htp.header (3, 'Update Complete');
      htp.tableOpen;
      htp.tableRowOpen ('LEFT', 'TOP');

      if common.has_privs_to_page (common.sort_file_page$, false) then
         htp.tabledata (common.space2$);
         htp.p ('<TD>');
         htp.formOpen ('abstract_maint.html');
         htp.formSubmit (null, 'Return to Abstract List');
         htp.formClose;
         htp.p ('</TD>');
      end if;

      htp.tableRowClose;
      htp.tableClose;
      common.footer;
   end if;

end done;

-- ****************************************************************************************

procedure save (ID integer, this_title varchar2, qa common.ident_arr,
			this_status varchar2, this_referee varchar2, this_comment varchar2) is

curr_estatus	constant abstracts.status_code%TYPE := common.decode_value (
			'status_code', 'abstracts', 'abstract_id', ID);
curr_rstatus	constant abstracts.referee_code%TYPE := common.decode_value (
			'referee_code', 'abstracts', 'abstract_id', ID);

begin

   common.get_gbl_pid;
   if common.has_privs_to_page (common.sort_file_page$) then

      update	abstracts
      set	title		= this_title,
	  	status_code	= this_status,
		referee_code	= this_referee
      where	abstract_id	= ID;

      for i in 2..qa.count LOOP
         insert into logs (abstract_id, log_code) values (ID, qa(i));
      end loop;

      if nvl (curr_estatus, '~') <> this_status OR
	 trim (this_comment) is not null then
         insert into logs (abstract_id, log_code, status_code, comments)
         values (ID, common.dot_reassignment$, this_status, trim (this_comment) );
      end if;

      if this_referee is not null then
         if nvl (curr_rstatus, '~') <> nvl (this_referee, '~') OR
		trim (this_comment) is not null then
            insert into logs (abstract_id, log_code, referee_code, comments)
            values (ID, common.dot_reassignment$, this_referee, 'Referee: ' || this_comment);
         end if;
      end if;

      common.redirect_URL ('abstract_maint.done');
   end if;

end save;

-- ****************************************************************************************

procedure show_srtf_log_items is

cursor	driver is
select	*
from	log_codes
where	srtf_function = 'Y'
order by
	log_descr;

begin

   htp.tableRowOpen ('LEFT', 'TOP');
   htp.tableHeader ('QA', 'RIGHT');

   htp.p ('<TD>');
   htp.formHidden ('qa', 'dummy');

   for driver_rec in driver LOOP
      htp.formCheckBox ('qa', driver_rec.log_code);
      htp.p (driver_rec.log_descr);
      htp.br;
   end loop;

   htp.p ('</TD>');
   htp.tableRowClose;

end show_srtf_log_items;

-- ****************************************************************************************

procedure abs_title_form (the_title varchar2) is

begin

   common.show_data_row ('Abstract Title', htf.formText ('this_title', 75, 200, htf.escape_sc (the_title)) ||
	common.space2$ ||
	'<input type="button" name="Title Case" value="TitleCase" '	||
	'onClick="document.ABSTRACT.this_title.value=document.ABSTRACT.this_title.value.toLowerCase();'	||
		 'document.ABSTRACT.this_title.value=document.ABSTRACT.this_title.value.toTitleCase();"' );

end abs_title_form;

-- ****************************************************************************************

procedure edit (ID integer) is

cursor	driver is
select	*
from	abstracts
where	abstract_id = ID;

driver_rec	driver%ROWTYPE;

begin

   common.get_gbl_pid;
   if common.header (rpt_title$) then
      htp.p (js$);
      htp.p (profile.js$);
      htp.p (editor.js$);
      htp.anchor ('abstract_maint.html', 'Return to the Paper List');
      htp.para;

      open driver;
      fetch driver into driver_rec;
      close driver;

      htp.formOpen ('abstract_maint.save', 'POST', cattributes=>'NAME="ABSTRACT"');
      htp.formHidden ('id', ID);

      if common.referee_conf$ = 'N' then
         htp.formHidden ('this_referee', '');
      end if;

      htp.tableOpen;

      common.show_data_row ('Paper ID',	common.get_paper_id (ID));
      abs_title_form (driver_rec.title);
      show_srtf_log_items;

      common.show_lookup_table ('status', 'Editor Status', driver_rec.status_code);
      if common.referee_conf$ = 'Y' then
         common.show_lookup_table ('referee', 'Referee Status', driver_rec.referee_code);
      end if;

      common.show_data_row ('Comments',	htf.formText ('this_comment', 75, 4000) );
      common.show_data_row ('Authors',	common.get_author_list2 (ID));
      common.show_data_row (null,	htf.formSubmit (null, 'Save') );

      htp.tableClose;
      htp.formClose;

      htp.hr;
      common.show_abstract_log (id, true);

      common.footer;
   end if;

end edit;

-- ****************************************************************************************

procedure show_last_qa_date (aid integer) is

cursor	driver is
select	max (log_date)
from	logs
where	abstract_id	= aid
and	log_code	= 'AB';

last_qa	date;

begin

   open driver;
   fetch driver into last_qa;
   close driver;

   htp.tableData (common.local_time (last_qa), cnowrap=>'NOWRAP');

end show_last_qa_date;

-- ****************************************************************************************

procedure html is

cursor	driver is
select	a.abstract_id, sa.paper_id, a.title, a.editor_id
from	sessions		s,
	abstract_sessions	sa,
	abstracts		a
where	sa.abstract_id		= a.abstract_id
--and	sa.primary		is not null
and	s.session_id		= sa.session_id
order by
	s.sort_order, s.session_date, s.beg_time, s.session_id, sa.paper_id;

begin

   common.get_gbl_pid;
   if common.header (rpt_title$) then
      htp.anchor ('editor.qa', 'Paper Final QA');
      htp.p (common.space2$);
      htp.p (common.space2$);
      htp.anchor ('editor.html', 'Return to the Editors'' Page');
      htp.p (common.space$);
      htp.bold ('Click on the Paper ID to edit.');
      htp.para;
      htp.tableOpen ('BORDER');

      htp.tableRowOpen ('LEFT', 'BOTTOM');
      htp.tableHeader ('Paper ID', cnowrap=>'NOWRAP');
      htp.tableHeader ('Initial QA Date');
      htp.tableHeader ('Title');
      htp.tableHeader ('Author(s)');
      htp.tableHeader ('Editor');
      htp.tableRowClose;

      for driver_rec in driver LOOP
         htp.tableRowOpen ('LEFT', 'TOP');
         htp.tableData (htf.anchor2 ('abstract_maint.edit?id=' || driver_rec.abstract_id,
				nvl (driver_rec.paper_id, driver_rec.abstract_id)) );
         show_last_qa_date (driver_rec.abstract_id);
         htp.tableData (driver_rec.title);
         htp.tableData (common.get_author_list2 (driver_rec.abstract_id));
         htp.tableData (common.get_user_name (driver_rec.editor_id), cnowrap=>'NOWRAP');
         htp.tableRowClose;
      end loop;

      htp.tableClose;

      common.footer;
   end if;

end html;

-- ****************************************************************************************

end abstract_maint;
/
show error

set define on

--	Copyright (C) 2004 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>

-- ****************************************************************************************
