set define off

create or replace package body affil_reqs as

rpt_title$	constant varchar2(25)	:= 'New Affiliation Requests';
rejected_addr$	constant integer	:= -2;

-- **********************************************************************************
-- **********************************************************************************

procedure show_links is

begin

   htp.anchor ('repository.html', 'Back to Repository Home Page');
   htp.p (common.space2$);
   htp.anchor ('repository.affiliations', 'Affiliation Maintenance');
   htp.para;

end show_links;

-- **********************************************************************************
-- **********************************************************************************

function ifnotnull (buffer1 varchar2, buffer2 varchar2, buffer3 varchar2,
			bold boolean default true) return varchar2 is

begin

   if buffer1 is null then
      return null;
   end if;

   if bold then
      return htf.bold(buffer2) || buffer1 || buffer3;
   else
      return buffer2 || buffer1 || buffer3;
   end if;

end ifnotnull;

-- **********************************************************************************
-- **********************************************************************************

procedure merge (from_id integer, to_id integer, req_id integer) is

merge_list	common.ident_arr;

begin

   merge_list(1) := 'dummy';
   merge_list(2) := from_id;
   merge_list(3) := to_id;

   repository.merge (merge_list, null, null, req_id);

end merge;

-- **********************************************************************************
-- **********************************************************************************

procedure save_req (req_id integer, txt varchar2) is

begin

   common.get_gbl_pid;
   if common.has_privs_to_page then
      update	affiliations
      set	name1	= txt
      where	affiliation_id =
	(	select	affiliation_id
		from	addresses
		where	address_id =
		(	select	address_id
			from	affil_requests
			where	affil_request_id = req_id
	)	);

      profile.check_CTX_index;
      common.redirect_URL ('affil_reqs.affil_search?req_id=' || req_id);
   end if;
   
end save_req;

-- **********************************************************************************

procedure edit_req (req_id integer) is

cursor	driver is
select	affil.affiliation_id, affil.name1
from	affiliations		affil,
	addresses		addr,
	affil_requests		req
where	req.affil_request_id	= req_id
and	addr.address_id		= req.address_id
and	affil.affiliation_id	= addr.affiliation_id;

driver_rec	driver%ROWTYPE;

begin

   common.get_gbl_pid;
   if common.has_privs_to_page then
      open driver;
      fetch driver into driver_rec;
      close driver;

      common.blank_header (rpt_title$);
      htp.bold (rpt_title$);
      htp.br;
      htp.p (common.get_affil_name (driver_rec.affiliation_id, htf.br));

      htp.formOpen ('affil_reqs.save_req', 'POST', '_top', cattributes=>'NAME="SPMS"');
      htp.formhidden ('req_id', req_id);
      htp.formText ('txt', 75, 200, htf.escape_sc (driver_rec.name1));
      htp.br;
      htp.p ('Enter a new name that will result in a better search for the merge function.');
      htp.br;

      htp.formSubmit;
      htp.formClose;
      common.form_focus ('SPMS', 'txt');
      common.blank_footer;
   end if;

end edit_req;

-- **********************************************************************************
-- **********************************************************************************

procedure del_req2 (addr_id integer) is

begin

   if common.header (rpt_title$, test_only=>true) then
      update	profiles
      set	address_id	= rejected_addr$
      where	address_id	= addr_id;

      delete from affil_requests where address_id = addr_id;
      common.redirect_URL ('affil_reqs.html');
   end if;

end del_req2;

-- **********************************************************************************
-- **********************************************************************************

procedure del_req (req_id integer) is

cursor	driver is
select	addr.address_id,
	common.get_affil_name (addr.affiliation_id, htf.br)	affil_name,
	common.get_address (addr.address_id, htf.br)		address
from	addresses		addr,
	affil_requests		r
where	r.affil_request_id	= req_id
and	addr.address_id		= r.address_id;

driver_rec	driver%ROWTYPE;

begin

   if common.header (rpt_title$) then
      open driver;
      fetch driver into driver_rec;
      close driver;

      htp.header (3, 'Are you sure you wish to delete this New Affiliation Request?');
      htp.bold (driver_rec.affil_name);
      htp.bold (driver_rec.address);
      htp.para;

      htp.tableOpen;
      htp.tableRowOpen;

      htp.p ('<TD>');
         htp.formOpen ('affil_reqs.del_req2', 'POST');
         htp.formHidden ('addr_id', driver_rec.address_id);
         htp.formSubmit (null, 'Yes');
         htp.formClose;
      htp.p ('</TD>');
      htp.tableData (common.space2$);
      htp.tableData (common.space2$);
      htp.tableData (common.space2$);
      htp.p ('<TD>');
         htp.formOpen ('affil_reqs.affil_search', 'POST');
         htp.formHidden ('req_id', req_id);
         htp.formSubmit (null, 'No');
         htp.formClose;
      htp.p ('</TD>');

      htp.tableRowClose;
      htp.tableClose;

      common.footer;
   end if;

end del_req;

-- **********************************************************************************
-- **********************************************************************************

procedure affil_search3 (string1 varchar2, string2 varchar2, ccode varchar2,
			addr_id integer, req_id integer) is

cursor	driver is
select	/*+ INDEX(addr) INDEX(affil) */
	score(1) + score(2)	iscore,
	common.get_affil_name (affil.affiliation_id, htf.br)	affil_name,
	common.get_address (addr.address_id, htf.br)		address,
	affil.affiliation_id
from	addresses		addr,
	affiliations		affil
where	0			< contains (affil.search_text, common.fmt_search_string (string1), 1) +
				  contains (addr.search_text,  common.fmt_search_string (string2), 2)
and	addr.affiliation_id	= affil.affiliation_id
and	addr.address_id		<> addr_id
and	nvl (addr.country_code,'~')	like nvl (ccode, '%')
order by
	(score(1) * 10) + score(2) desc;

icount		integer := 0;
code		mail_blats_errors.error_code%TYPE;
emessage	mail_blats_errors.error_message%TYPE;
affil_id$	constant integer := common.decode_value ('affiliation_id', 'addresses', 'address_id', addr_id);

begin

   for driver_rec in driver LOOP
      htp.p ('Score:');
      htp.bold (driver_rec.iscore);
      htp.p (common.space2$);
      htp.anchor2 ('affil_reqs.merge'	||
		'?from_id='	|| affil_id$ ||
		'&to_id='	|| driver_rec.affiliation_id ||
		'&req_id='	|| req_id, 'Merge', ctarget=>'_top');

      htp.br;
      htp.p (driver_rec.affil_name);
      htp.br;

      htp.p (driver_rec.address);
      htp.para;

      icount := icount + 1;
   end loop;
   htp.tableClose;

   if icount = 0 then
      htp.header (3, 'No Affiliations Matched Your Search String', cattributes=>common.red$);
   end if;

exception
   when others then
      code	:= sqlcode;
      emessage	:= sqlerrm;

      htp.p ('-->');
      htp.p ('</SCRIPT>');
      htp.header (3, 'An error occured when searching the database.', cattributes=>common.red$);

      if trim (string1 || string2) is null then
         htp.bold ('You must enter a search string.');
      else
         htp.bold ('Please try another search string.');
         send_mail ('arenam@fnal.gov', 'SPMS Search Error', 
		'  Original String = '	|| string1 || '<=>' || string2	|| common.eol$ ||
		'Formattted String = '	|| common.fmt_search_string (string1) || '<=>' ||
					   common.fmt_search_string (string2) || common.eol$ ||
		'       Error Code = '	|| code		|| common.eol$ ||
		'    Error Message = '	|| emessage, 'arenam@fnal.gov');

      end if;

      htp.br;
      htp.p (common.click_back$);

end affil_search3;

-- **********************************************************************************

procedure affil_search2 (req_id integer) is

cursor	driver is
select	affil.search_text	affil_search_text,
	addr.search_text	addr_search_text,
	addr.country_code,	addr.address_id
from	affiliations		affil,
	addresses		addr,
	affil_requests		req
where	req.affil_request_id	= req_id
and	addr.address_id		= req.address_id
and	affil.affiliation_id	= addr.affiliation_id;

driver_rec	driver%ROWTYPE;

begin

   common.blank_header (rpt_title$);

   open driver;
   fetch driver into driver_rec;
   close driver;

   affil_search3 (driver_rec.affil_search_text, driver_rec.addr_search_text,
			driver_rec.country_code, driver_rec.address_id, req_id);
   common.footer;

end affil_search2;

-- **********************************************************************************

procedure affil_search1 (req_id integer) is

cursor	driver is
select	affiliation_id, address_id
from	addresses
where	address_id =
(	select address_id from affil_requests where affil_request_id = req_id);

driver_rec	driver%ROWTYPE;

begin

   open driver;
   fetch driver into driver_rec;
   close driver;

   common.blank_header (rpt_title$);

   htp.tableOpen;
   htp.tableRowOpen ('LEFT', 'TOP');
   htp.p ('<TD>');
      htp.header (3, rpt_title$);
      htp.tableOpen;
      htp.tableRowOpen ('LEFT', 'TOP');
      htp.tableHeader (	htf.anchor2 ('affil_reqs.accept?req_id=' || req_id, 'Accept', null, '_top')	||
			htf.br || htf.anchor  ('affil_reqs.edit_req?req_id='	 || req_id, 'Edit')	||
			htf.br || htf.anchor2  ('affil_reqs.del_req?req_id='	 || req_id, 'Delete', null, '_top') );
      htp.tableHeader (common.get_affil_name (driver_rec.affiliation_id, htf.br) || htf.br ||
			common.get_address (driver_rec.address_id, htf.br), cnowrap=>'NOWRAP',
			cattributes=>common.red$);
      htp.tableRowClose;
      htp.tableClose;
   htp.p ('</TD>');

   htp.p ('<TD>');
      htp.ulistOpen;
      htp.listItem ('This search is based on every word entered into the acronyn, name, deaprtment '	||
		'and address fields by the user.' || common.space2$ || 'The search results '		||
		'is likely a very long list however, the most likley matches are sorted to the top.');
      htp.listItem ('If the affiliation request is a new affiliation then click accept.'		||
		common.space2$ || 'You will have an opportunity to edit the data to make any '		||
		'necessary corrections.');
      htp.listItem ('If the affiliation request is not valid then search for the corrrect affiliation '	||
		'in the lower section of the screen.');
      htp.listItem ('When you find the correct existing affiliation, click the merge link.'		||
		common.space2$ || 'This is effictivly deletes the incorrect affiliation request.');
      htp.listItem ('If the affliation request does not contain sufficient commonality with an '	||
		'existing affiliation then edit the request and change the affiliation name to '	||
		'match an existing affiliation.');
      htp.ulistClose;
   htp.p ('</TD>');

   htp.tableRowClose;
   htp.tableClose;

   common.blank_footer;

end affil_search1;

-- **********************************************************************************

procedure affil_search (req_id integer) is

begin

   common.get_gbl_pid;
   if common.has_privs_to_page then
      htp.framesetOpen ('30%,*');
      htp.frame (common.server || common.vdir$ || 'affil_reqs.affil_search1?req_id=' || req_id);
      htp.frame (common.server || common.vdir$ || 'affil_reqs.affil_search2?req_id=' || req_id);
      htp.framesetClose;			
   end if;

end affil_search;

-- **********************************************************************************
-- **********************************************************************************

procedure save2 (req_id integer, pid integer, affil common.ident_arr, this_affil_type varchar2,
			addr common.ident_arr, this_country varchar2, comments varchar2) is

cursor	driver is
select	address_id, affiliation_id
from	addresses
where	address_id = (select address_id from affil_requests
			where affil_request_id = req_id);

rec	driver%ROWTYPE;

begin

   open driver;
   fetch driver into rec;
   close driver;

   update	affiliations
   set		affil_abbrev		= trim(affil(1)),
		name1			= trim(affil(2)),
		name2			= trim(affil(3)),
		department		= trim(affil(4)),
		web_site		= trim(affil(5)),
		affil_type_code		= this_affil_type
   where	affiliation_id		= rec.affiliation_id;

   update	addresses
   set		address1		= trim(addr(1)),
		address2		= trim(addr(2)),
		address3		= trim(addr(3)),
		po_box			= trim(addr(4)),
		town			= trim(addr(5)),
		postal_code		= trim(addr(6)),
		zip_code		= trim(addr(7)),
		country_code		= this_country
   where	address_id		= rec.address_id;

   delete from affil_requests where affil_request_id = req_id;

end save2;

-- **********************************************************************************

procedure save (req_id integer, pid integer, affil common.ident_arr, this_affil_type varchar2,
			addr common.ident_arr, this_country varchar2, comments varchar2) is

continue	boolean := true;
country		country_codes.country_descr%TYPE;
recipient	mail_blats.recipient%TYPE;
email_address	mail_blats.recipient%TYPE;
cc_address	mail_blats.recipient%TYPE := null;
user_email$	constant mail_blats.recipient%TYPE := common.get_user_email (common.gbl_pid);

new_comments	varchar2(4000);

begin

   common.get_gbl_pid;

   if common.header (rpt_title$) then
      show_links;

      if affil(2) is null then -- Name1 is null
         continue := false;
         htp.header (3, 'You must supply a value for Name1', cattributes=>common.red$);
      end if;

      if this_country is null then -- Country is null
         continue := false;
         htp.header (3, 'You must supply a value for Country', cattributes=>common.red$);
      end if;

      if continue then
         save2 (req_id, pid, affil, this_affil_type, addr, this_country, comments);
         recipient	:= common.get_user_email (pid);
         country	:= common.decode_value ('country_descr', 'country_codes',
						'country_code', this_country);

         new_comments := trim (comments);
         new_comments := replace (new_comments, chr(09), null);
         new_comments := replace (new_comments, chr(10), null);
         new_comments := replace (new_comments, chr(13), null);

         if recipient is not null and new_comments is not null then
            if common.cc_new_affil_reqs$ = 'Y' then
               cc_address := common.editor_email$;
            end if;

            send_mail (recipient, common.conf_title$ || ' New Affiliation Request for ' || common.conf_title$,
			'The ' || common.conf_title$ || ' administrator sends the following comment regarding your request.'   ||
			common.eol2$ || comments || common.eol2$ ||
			ifnotnull (affil(1),	'Abbrev: ',	common.eol$, false) ||
			ifnotnull (affil(2),	'Name1: ',	common.eol$, false) ||
			ifnotnull (affil(3),	'Name2: ',	common.eol$, false) ||
			ifnotnull (affil(4),	'Department: ',	common.eol$, false) ||
			ifnotnull (addr(1),	'Address: 1 ',	common.eol$, false) ||
			ifnotnull (addr(2), 	'Address: 2 ',	common.eol$, false) ||
			ifnotnull (addr(3), 	'Address: 3 ',	common.eol$, false) ||
			ifnotnull (addr(4), 	'PO Box: ',	common.eol$, false) ||
			ifnotnull (addr(5), 	'Town: ',	common.eol$, false) ||
			ifnotnull (addr(6),	'State/Providence/Postal Code: ',common.eol$, false) ||
			ifnotnull (addr(7),	'Zip Code: ',	common.eol$, false) ||
			ifnotnull (country,	'Country: ',	common.eol$, false),
			user_email$, cc=>cc_address, pid=>common.gbl_pid);
         end if;

         htp.header (3, 'New Affiliation Saved/Updated');
         htp.formOpen ('affil_reqs.html');
         htp.formSubmit (null, 'Continue');
         htp.formClose;

         profile.check_CTX_index;
      else
         htp.bold (common.click_back$);
      end if;

      common.footer;
   end if;

end save;

-- **********************************************************************************
-- **********************************************************************************

procedure accept (req_id integer) is

cursor	driver is
select	affil.affil_abbrev, affil.name1, affil.name2, affil.department,
	affil.web_site, affil.affil_type_code, req.requestor_id, addr.*
from	affiliations	affil,
	addresses	addr,
	affil_requests	req
where	req.affil_request_id	= req_id
and	addr.address_id		= req.address_id
and	affil.affiliation_id	= addr.affiliation_id;

driver_rec	driver%ROWTYPE;

begin

   if common.header (rpt_title$) then
      open driver;
      fetch driver into driver_rec;
      close driver;

      show_links;

      htp.formOpen ('affil_reqs.save', 'POST');
      htp.formHidden ('req_id', req_id);
      htp.formHidden ('pid', driver_rec.requestor_id);
      htp.tableOpen;

      common.show_data_row ('Abbreviation',	htf.formText ('affil',	25,  25, htf.escape_sc (driver_rec.affil_abbrev)));
      common.show_data_row ('Name1',		htf.formText ('affil',	50, 200, htf.escape_sc (driver_rec.name1)));
      common.show_data_row ('Name2',		htf.formText ('affil',	50, 200, htf.escape_sc (driver_rec.name2)));
      common.show_data_row ('Department',	htf.formText ('affil',	50, 200, htf.escape_sc (driver_rec.department)));
      common.show_data_row ('Web Site',		htf.formText ('affil',	50, 100, htf.escape_sc (driver_rec.web_site)));
      common.show_lookup_table ('affil_type', 'Type', driver_rec.affil_type_code);

      common.show_data_sep (2);

      common.show_data_row ('Address1',		htf.formText ('addr',	50, 80, htf.escape_sc (driver_rec.address1)));
      common.show_data_row ('Address2',		htf.formText ('addr',	50, 80, htf.escape_sc (driver_rec.address2)));
      common.show_data_row ('Address3',		htf.formText ('addr',	50, 80, htf.escape_sc (driver_rec.address3)));
      common.show_data_row ('PO Box',		htf.formText ('addr',	50, 80, htf.escape_sc (driver_rec.po_box)));
      common.show_data_row ('Town',		htf.formText ('addr',	50, 80, htf.escape_sc (driver_rec.town)));
      common.show_data_row ('Postal Code',	htf.formText ('addr',	50, 80, htf.escape_sc (driver_rec.postal_code)));
      common.show_data_row ('Zip Code',		htf.formText ('addr',	50, 80, htf.escape_sc (driver_rec.zip_code)));
      common.show_lookup_table ('country', 'Country', driver_rec.country_code);

      common.show_data_sep (2);
      common.show_data_row ('Comments to Requestor' || htf.br || 'Limit to 4000 Characters',
						htf.formTextArea ('comments', 3, 50));
      common.show_data_row (null,		htf.formSubmit (null, 'Save'));

      htp.tableClose;
      htp.formClose;
      common.footer;
   end if;

end accept;

-- **********************************************************************************
-- **********************************************************************************

procedure html is

cursor	driver is
select	common.get_affil_name (affil.affiliation_id, htf.br)	affil_name,
	common.get_address (addr.address_id, htf.br)		addr_text,
	req.affil_request_id, req.requestor_id, req.request_date
from	affiliations	affil,
	addresses	addr,
	affil_requests	req
where	addr.address_id		= req.address_id
and	affil.affiliation_id	= addr.affiliation_id
order by req.request_date;

begin

   if common.header (rpt_title$) then
      htp.p (profile.js$);
      show_links;
      htp.tableOpen ('BORDER');

      htp.tableRowOpen ('LEFT', 'BOTTOM');
      htp.tableHeader (common.space$);
      htp.tableHeader ('New Affiliation');
      htp.tableHeader ('New Address');
      htp.tableHeader ('Requested');
      htp.tableHeader ('Requestor');
      htp.tableRowClose;

      for driver_rec in driver LOOP
         htp.tableRowOpen ('LEFT', 'TOP');
         htp.tableData (htf.anchor ('affil_reqs.affil_search?req_id=' ||
				driver_rec.affil_request_id, 'Search') );

         htp.tableData (driver_rec.affil_name);
         htp.tableData (driver_rec.addr_text);
         htp.tableData (common.local_time (driver_rec.request_date));
         htp.tableData (common.get_user_name (driver_rec.requestor_id) || htf.br ||
			common.get_user_affil_list (driver_rec.requestor_id, ', ') ||
			htf.br || htf.mailto (	common.get_user_email (driver_rec.requestor_id),
						common.get_user_email (driver_rec.requestor_id) ) );

         htp.tableRowClose;
      end loop;

      htp.tableClose;
      common.footer;
   end if;

end html;


-- **********************************************************************************

end affil_reqs;
/
show error

set define on

--	Copyright (C) 2004 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>
