set define off

create or replace package body agenda as

rpt_title$	constant char(6)	:= 'Agenda';
style_header$	constant char(45)	:= 'STYLE="BACKGROUND-COLOR:#1A64A0; COLOR:WHITE"';
style_dow$	constant char(32)	:= 'STYLE="BACKGROUND-COLOR:#D3DCE3"';
style_largeTxt$	constant char(23)	:= 'STYLE="FONT-SIZE: 150%"';
max_cols$	constant integer	:= 5;

type	arecs_type	is table of abstracts%ROWTYPE index by binary_integer;

type	abs_type	is record
	(	abs	abstracts%ROWTYPE,
		sa	abstract_sessions%ROWTYPE);

type	abs_types	is table of abs_type index by binary_integer;

type	rec_type is record
	(	sdata	sessions%ROWTYPE,
		adata	abs_types);

type	recs_type	is table of rec_type index by binary_integer;
recs	recs_type;

-- ****************************************************************************
-- ****************************************************************************

procedure show_row (txt varchar2, style varchar2 default null) is

begin

   htp.tableRowOpen ('LEFT', 'TOP', cattributes=>style);
   htp.tableData (common.space2$);
   htp.tableData (txt);
   htp.tableRowClose;

end show_row;

-- ****************************************************************************

procedure show_spacer (style varchar2 default null) is

begin

   htp.tableRowOpen;
   htp.tableData (common.space$, ccolspan=>max_cols$, cattributes=>style);
   htp.tableRowClose;

end show_spacer;

-- ****************************************************************************

procedure show_empty_cell is

begin

   htp.tableData (common.space2$ || common.space2$ || common.space2$ || common.space2$ || common.space2$);

end show_empty_cell;

-- ****************************************************************************

procedure show_header is

begin

   htp.tableOpen (cattributes=>style_header$ || ' WIDTH=100%');
   show_row (common.space$);
   show_row (common.conf_title$ || ' ' || rpt_title$, style_largeTxt$);
   show_row (common.conf_location$);
   show_row (to_char (common.conf_start_monday$, 'DD-Month-YYYY')	||
		common.space$ || common.mdash$ || common.space$		||
		to_char (common.conf_start_monday$+common.conf_len_days$-1, 'DD-Month-YYYY') );
   show_row (common.space$);
   htp.tableClose;
   htp.para;

end show_header;

-- ****************************************************************************
-- ****************************************************************************

function get_agenda_anchors (pid varchar2) return long is

cursor	driver is
select	f.file_name, ft.file_type_descr, pc.platform_descr
from	file_type_codes		ft,
	platform_codes		pc,
	files			f
where	f.abstract_id		=  (select abstract_id from abstract_sessions where paper_id = pid)
and	f.deleted		is null
and	f.uploaded_date		=
	(	select	max (uploaded_date)
		from	files	f2
		where	f2.file_name	= f.file_name
		and	f2.deleted	is null)
and	pc.platform_code	= f.platform_code
and	ft.file_type_code	= f.file_type_code
and	ft.transparency_flag	= 'Y'
order by ft.file_type_descr;

rec_found	boolean;
buffer		long := null;

begin

   if common.editor_download_script$ is null then
      return null;
   end if;

   for rec in driver LOOP
      buffer := buffer || rec.file_type_descr || ': ' ||
		htf.anchor2 (common.wanted_file_URL (pid, rec.file_name), rec.file_name,
				ctarget=>'_blank') || ' (' || rec.platform_descr || ');' || common.space2$;
   end loop;

   return buffer;

end get_agenda_anchors;

-- ****************************************************************************

procedure show_recs is

anchors		long;
has_privs	constant boolean := common.has_privs_to_page ('transparencies.html', show_err_msg=>false);
prev_day	date := '01-jan-00';

begin

   common.blank_header (rpt_title$);
   show_header;
   htp.tableOpen (cattributes=>'WIDTH="100%" CELLSPACING="0"');

   for i in 1..recs.COUNT loop
      if prev_day <> recs(i).sdata.session_date then
         htp.tableRowOpen ('LEFT', 'TOP', cattributes=>style_dow$);
         htp.tableHeader (to_char (recs(i).sdata.session_date, 'FMDay, dd Month YYYY'), ccolspan=>max_cols$);
         htp.tableRowClose;
         prev_day := recs(i).sdata.session_date;

         show_spacer;
      end if;

      htp.tableRowOpen ('LEFT', 'TOP', cattributes=>style_dow$);
      htp.tableHeader (recs(i).sdata.session_id);
      htp.tableHeader (recs(i).sdata.title, ccolspan=>max_cols$-1);
      htp.tableRowClose;

      htp.tableRowOpen ('LEFT', 'TOP', cattributes=>style_dow$);
      show_empty_cell;
      htp.tableData ('Chair: ' ||
			common.get_user_name_affiliation (recs(i).sdata.chair_profile_id) || htf.br ||
			common.decode_value ('location_descr', 'location_codes', 'location_code',
							recs(i).sdata.location_code)	  || htf.br ||
			common.format_time (recs(i).sdata.beg_time) || ' - ' || common.format_time(recs(i).sdata.end_time),
			ccolspan=>max_cols$-1);
      htp.tableRowClose;
      show_spacer (style_dow$);


      recs(i).sdata.session_date := common.add_time (recs(i).sdata.session_date, recs(i).sdata.beg_time);
      for j in 1..recs(i).adata.COUNT loop
         htp.tableRowOpen ('LEFT', 'TOP');
         show_empty_cell;
         htp.tableData (to_char(recs(i).sdata.session_date, 'hh24:mi'));
         htp.tableData (htf.bold (recs(i).adata(j).sa.paper_id) || common.space$ || recs(i).adata(j).abs.title, ccolspan=>max_cols$-2);
         htp.tableRowClose;

         htp.tableRowOpen ('LEFT', 'TOP');
         show_empty_cell;
         show_empty_cell;
         htp.tableData ('Speaker: ' ||
				common.get_user_name_affiliation (
				common.get_speaker_profile_id (recs(i).adata(j).abs.abstract_id) ), ccolspan=>max_cols$-3);
         htp.tableRowClose;


         if has_privs then
            anchors := get_agenda_anchors (recs(i).adata(j).sa.paper_id);
            if anchors is not null then
               htp.tableRowOpen ('LEFT', 'TOP');
               show_empty_cell;
               show_empty_cell;
               htp.tableData (anchors, ccolspan=>max_cols$-3);
               htp.tableRowClose;
            end if;
         end if;

         recs(i).sdata.session_date := common.add_time (recs(i).sdata.session_date, recs(i).adata(j).sa.duration);
      end loop;
   end loop;

   htp.tableClose;
   common.blank_footer;

end show_recs;

-- ****************************************************************************

function get_abs_recs (sid varchar2) return abs_types is

cursor	driver is
select	a.*
from	abstracts		a,
	abstract_sessions	sa
where	sa.session_id		= sid
and	a.abstract_id		= sa.abstract_id
order by sa.session_seq;

cursor	SA (aid$ integer) is
select	*
from	abstract_sessions
where	abstract_id	= aid$
and	session_id	= sid;

arecs	arecs_type;
recs	abs_types;

begin

   open driver;
   fetch driver BULK COLLECT into arecs;
   close driver;

   for i in 1..arecs.COUNT loop
      recs(i).abs	:= arecs(i);

      open SA (arecs(i).abstract_id);
      fetch SA into recs(i).sa;
      close SA;
   end loop;

   return recs;

end get_abs_recs;

-- ****************************************************************************

procedure by_day (dte varchar2) is

cursor	driver is
select	*
from	sessions
where	session_date = to_date (dte)
and	presentation_option_code in
	(	select	presentation_option_code
		from	presentation_option_codes
		where	presentation_type_code = common.oral$)
order by session_date, beg_time, end_time, session_id;

type	lrec_type is table of driver%ROWTYPE index by binary_integer;
lrecs	lrec_type;

begin

   open driver;
   fetch driver BULK COLLECT into lrecs;
   close driver;

   for i in 1..lrecs.COUNT loop
      recs(i).sdata := lrecs(i);
      recs(i).adata := get_abs_recs (lrecs(i).session_id);
   end loop;

   common.get_gbl_pid (false);
   if common.has_privs_to_page (show_err_msg=>false) or common.agenda_public_access$ = 'Y' then
      show_recs;
   else
      common.access_denied;
   end if;

end by_day;

-- ****************************************************************************

procedure by_loc (code varchar2) is

cursor	driver is
select	*
from	sessions
where	location_code = code
and	presentation_option_code in
	(	select	presentation_option_code
		from	presentation_option_codes
		where	presentation_type_code = common.oral$)
order by session_date, beg_time, end_time, session_id;

type	lrec_type is table of driver%ROWTYPE index by binary_integer;
lrecs	lrec_type;

begin

   open driver;
   fetch driver BULK COLLECT into lrecs;
   close driver;

   for i in 1..lrecs.COUNT loop
      recs(i).sdata := lrecs(i);
      recs(i).adata := get_abs_recs (lrecs(i).session_id);
   end loop;

   common.get_gbl_pid (false);
   if common.has_privs_to_page (show_err_msg=>false) or common.agenda_public_access$ = 'Y' then
      show_recs;
   else
      common.access_denied;
   end if;

end by_loc;

-- ****************************************************************************

procedure by_session (sid varchar2) is

cursor	driver is
select	*
from	sessions
where	session_id = sid;

type	lrec_type is table of driver%ROWTYPE index by binary_integer;
lrecs	lrec_type;

begin

   open driver;
   fetch driver BULK COLLECT into lrecs;
   close driver;

   for i in 1..lrecs.COUNT loop
      recs(i).sdata := lrecs(i);
      recs(i).adata := get_abs_recs (lrecs(i).session_id);
   end loop;

   common.get_gbl_pid (false);
   if common.has_privs_to_page (show_err_msg=>false) or common.agenda_public_access$ = 'Y' then
      show_recs;
   else
      common.access_denied;
   end if;

end by_session;

-- ****************************************************************************

procedure full is

cursor	driver is
select	*
from	sessions
where	presentation_option_code in
	(	select	presentation_option_code
		from	presentation_option_codes
		where	presentation_type_code = common.oral$)
order by session_date, beg_time, end_time, session_id;

type	lrec_type is table of driver%ROWTYPE index by binary_integer;
lrecs	lrec_type;

begin

   open driver;
   fetch driver BULK COLLECT into lrecs;
   close driver;

   for i in 1..lrecs.COUNT loop
      recs(i).sdata := lrecs(i);
      recs(i).adata := get_abs_recs (lrecs(i).session_id);
   end loop;

   common.get_gbl_pid (false);
   if common.has_privs_to_page (show_err_msg=>false) or common.agenda_public_access$ = 'Y' then
      show_recs;
   else
      common.access_denied;
   end if;

end full;

-- ****************************************************************************
-- ****************************************************************************

procedure show_by_location is

cursor	driver is
select	distinct location_code, location_descr
from	location_codes
where	location_code in
	(	select	location_code
		from	sessions
		where	presentation_option_code in
			(	select	presentation_option_code
				from	presentation_option_codes
				where	presentation_type_code = common.oral$
	)		)
order by location_descr;

begin

   htp.p ('<TD>');
   for rec in driver LOOP
      htp.anchor ('agenda.by_loc?code=' || rec.location_code, rec.location_descr);
      htp.br;
   end loop;
   htp.p ('</TD>');

end show_by_location;

-- ****************************************************************************

procedure show_by_session is

cursor	driver is
select	*
from	sessions
where	presentation_option_code in
	(	select	presentation_option_code
		from	presentation_option_codes
		where	presentation_type_code = common.oral$)
order by session_date, beg_time, end_time, session_id;

prev_day	date := '01-jan-00';

begin

   htp.p ('<TD>');
   for rec in driver LOOP
      if prev_day <> rec.session_date then
         htp.anchor ('agenda.by_day?dte=' || rec.session_date,
			to_char (rec.session_date, 'Day') );
         htp.br;
         prev_day := rec.session_date;
      end if;

      htp.p (common.format_time (rec.beg_time));
      htp.p ('-');
      htp.p (common.format_time (rec.end_time));
      htp.anchor ('agenda.by_session?sid=' || rec.session_id, rec.title);
      htp.br;
   end loop;
   htp.p ('</TD>');

end show_by_session;

-- ****************************************************************************

procedure html is

begin

   common.get_gbl_pid (false);
   if common.has_privs_to_page (show_err_msg=>false) or common.agenda_public_access$ = 'Y' then
      common.blank_header (rpt_title$);
      show_header;

      htp.anchor ('agenda.full', 'Full Conference');
      htp.para;

      htp.tableOpen (cattributes=>'WIDTH="100%"');

      htp.tableRowOpen ('LEFT', 'TOP');
      htp.tableData ('By Location',	cattributes=>style_dow$);
      htp.tableData ('By Day',		cattributes=>style_dow$);
      htp.tableRowClose;

      htp.tableRowOpen ('LEFT', 'TOP');
      show_by_location;
      show_by_session;
      htp.tableRowClose;

      htp.tableClose;
      common.blank_footer;
   else
      common.access_denied;
   end if;

end html;

-- ****************************************************************************

end agenda;
/
show error

set define on

--	Copyright (C) 2004 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>

