set define off

create or replace package body ajaxMgr as

procedure js is

begin

htp.p ('

AjaxMgr.__instance__ = null;

// ********************************************************************
AjaxMgr.exception = function (message) {
    this.name	 = "AjaxMgrException";
    this.message = message;
};
    
AjaxMgr.exception.prototype = new Error;

// ********************************************************************

AjaxMgr.getInstance = function () {
    
    if (null == this.__instance__ ) {
	this.__instance__ = new AjaxMgr();
    };
    
    return this.__instance__;
};
    
// ********************************************************************
    
function AjaxMgr () {
	
    if (AjaxMgr.caller != AjaxMgr.getInstance) {
	throw new AjaxMgr.exception ("There is no public contructor for AjaxMgr.\n" +
				     "Syntax: var myAjaxMgr = AjaxMgr.getInstance();" );
    };
    this.openReqCount = 0;
};

// ********************************************************************

AjaxMgr.prototype.getOpenReqCount = function () {
    return this.openReqCount;
};
	
// ********************************************************************
AjaxMgr.prototype.incrementOpenReqCount = function() {
    this.openReqCount += 1;
};
// ********************************************************************
AjaxMgr.prototype.decrementOpenReqCount = function() {
    this.openReqCount -= 1;
};
// ********************************************************************
	
AjaxMgr.prototype.__createXMLobject__ = function () {

// private method to create the XML object
// end users should use the callURL method
    
    var XMLobject = null;
    if (window.XMLHttpRequest) {
	XMLobject = new XMLHttpRequest();
    } else if (window.ActiveXObject) {
	XMLobject = new ActiveXObject ("Microsoft.XMLHTTP");
    };
    return XMLobject;
};

// ********************************************************************
AjaxMgr.prototype.callURL = function(url, callBack, args, method, userArgs) {
    
    // Function parameters
    // 1) - STRING: relative or full URL to call
    // 2) - Reference: function to call when data is returned
    // 3) - STRING: Optional URL name/value pairs
    // 4) - STRING: POST or GET
    // 5) - any user-defined object to be passed to the user''s callback function
    
    var theURL      = url;
    var theCallback = callBack;
    var theArgs	    = args == undefined ? "" : args;
    var theMethod;
    try {
	theMethod = method == undefined ? "POST" : method.toUpperCase();
    } catch (e) {
	theMethod = "garbage";
    }

    // arg validation:
    if (theURL == undefined) {
	throw new AjaxMgr.exception("You must supply a URL.");
    };

    if (theCallback == undefined) {
	throw new AjaxMgr.exception("You must supply a callback function.");
    } else if (typeof(theCallback) != "function") {
	throw new AjaxMgr.exception("Invalid callback: not a function.");
    };

    if (theMethod != "POST" && theMethod != "GET") {
	throw new AjaxMgr.exception("Invalid method (" + method + ").  Method must be POST or GET.");
    };

    // manipulation of url/args:
    if (theArgs != null) {
	var argFirstByte = theArgs.substr (0,1);
	
	switch (theMethod) {
	case "POST" : 
            if ("?" == argFirstByte) {
		theArgs = theArgs.substr (1);
            };
            break;
	    
	case "GET" :
            if ("?" != argFirstByte) {
		theArgs = "?" + theArgs;
            };
	    theURL = theURL + theArgs;
	    theArgs = "";
            break;
	};
    };

    // create a new request object:
    var XMLobject = this.__createXMLobject__();
    if (XMLobject) {
	XMLobject.open(theMethod, theURL, true);
	if ("POST" == theMethod) {
	    XMLobject.setRequestHeader("Content-Type", "application/x-www-form-urlencoded");
	};

	XMLobject.onreadystatechange = function() {
	    // status not necessarily assigned to object at all readyStates:
	    var this_status;
	    try {
		this_status = XMLobject.status;
	    } catch (e) {
		this_status = undefined;
	    };

	    // we are only concerned about the final state:
	    if (4 == XMLobject.readyState) {
		if (200 == this_status) {
		    // The ajax call succeeded and received valid data; proceed to call the user''s callback function:
		    try {
			theCallback (XMLobject, userArgs);
		    } catch (e) {
			// Bad user.
			alert("CODING ERROR: Unhandled Exception in Ajax callback handler:" +
			      "\nurl = " + url +
			      "\nargs = " + args +
			      "\nexception = " + e +
			      "\nerror message = " + e.message);
		    };
		    
		    // cleanup:
		    AjaxMgr.getInstance().decrementOpenReqCount();

		} else {
		    // The ajax call FAILED.
		    // Do our cleanup:
		    AjaxMgr.getInstance().decrementOpenReqCount();

		    // Throwing an exception or returning a status does no good whatsoever.
		    // The only way to notify the user that a problem occurred is to ALERT.
		    var this_text;
		    try {
			this_text = XMLobject.responseText.substring(0, 1024) + "\n...";
		    } catch (e) {
			this_text = undefined;
		    };
		    
		    alert("SOMETHING WENT VERY WRONG IN AJAX CALL:" +
			  "\nurl = " + url +
			  "\nargs = " + args + 
			  "\n" +
			  "\nreadyState = " + XMLobject.readyState +
			  "\nstatus = " + this_status + 
			  "\n" + 
			  "\nresponseText = \n" + this_text);
		};
	    };
	};  // end implicit function onreadystatechage

	AjaxMgr.getInstance().incrementOpenReqCount();
	XMLobject.send (theArgs);
    };
};
// ********************************************************************

AjaxMgr.prototype.getResponseXML= function (XMLobject) {

    // There is a bug in firefox:
    //  If you have "document.domain" set, you will NOT be able to directly
    //  access the responseXML (Permission Denied errors).  Stupid Firefox.
    // So, in Firefox, we need to do it the old fashioned way, by parsing
    //  the responseText.
    
    // So: see if we can read the responseXML:
    var responseXML = XMLobject.responseXML;
    try {
	var dummy = responseXML.getElementsByTagName("dummy");
	// ok, we can read it, and we just use it as-is.

    } catch (e) {
	// Ok, now here''s another funky thing:
	//  Ideally, we''d like to check the error to make sure it
	//  is the permission error from Firefox.  BUT, the Firefox 
	//  bug includes other things like:
	//    a) the exception has null name
	//    b) the exception has null message
	//    c) the exception is NOT derived from one of the "well-known"
	//       exception types 
	//  What''s left?  String parsing to look for "Permission denied."
	//  Except that this only works for people using English and normal
	//   fonts in their browser -- chinese glyphs will NOT match on
	//   "Permission denied" strings, for example.  Sigh.
	//  So, just go forth and ASSUME that we''re handling the Firefox
	//   permission bug.  And hope for the best.

	var responseText = XMLobject.responseText;
	var parser = null;

	try {
	    // can we get a parser?  Not in IE, at least not so far as I know...
	    parser = new DOMParser();
	} catch (another_e) {
	    // Now what?!  We cannot get a parser! Ouch!
	    alert("ABORT: Your browser is not letting us read XML responses from the server." +
		  "\nWe tried using a workaround for Firefox, but it is not working either." +
		  "\nPlease report this to ids-group@fnal.gov; include your browser version." +
		  "\nThank you in advance for your help in debugging this situation.");
	};

	if (parser != null) {
	    responseXML = parser.parseFromString(responseText, "text/xml");
	};
    };

    // NB: originally I tried to set the XMLobject.responseXML to the new responseXML
    //     that we just parsed, but by the time you get back to the user''s callback
    //     function, the permission error reappears.  So user will have to call this
    //     routine -- not quite as transparent as hoped...

    // return the useable responseXML:
    return responseXML;
};

');

end js;

end ajaxMgr;
/
show error

set define on

--	Copyright (C) 2004 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>

