set define off

create or replace package body app_maint as

rpt_title$	constant char(30)	:= 'Application Privilege Mappings';
map_msg$	constant char(27)	:= '*** Map a New Privilege ***';

-- *******************************************************************************
-- *******************************************************************************

procedure seed (app_name$ varchar2, priv$ varchar2) is

app_id    integer;

begin

   insert into applications
   select application_id_seq.nextval, lower (app_name$)
   from      dual
   where  not exists
    (    select    1
        from    applications
        where    application_name = app_name$);

   insert into application_privs
   select application_id, priv$
   from      applications	a1
   where  application_name = app_name$
   and	 not exists
	(	select	1
		from	application_privs	a2
		where	a2.application_id	= a1.application_id
		and	a2.privilege_code	= priv$);

end seed; 

-- *******************************************************************************
-- *******************************************************************************

procedure add (app_id integer, pcode common.ident_arr) is

begin

   if common.header (rpt_title$, test_only=>true) then
      delete from application_privs where application_id = app_id;

      for i in 3..pcode.COUNT loop
         insert into application_privs
		(application_id, privilege_code)
		values (app_id, pcode(i));

      end loop;
      common.redirect_URL ('app_maint.edit?app_id=' || app_id);
   end if;

end add;

-- *******************************************************************************
-- *******************************************************************************

procedure edit (app_id integer) is

cursor	driver is
select	p.*, (	select	'CHECKED'
		from	application_privs	ap
		where	application_id		= app_id
		and	privilege_code		= p.privilege_code) checked
from	privilege_codes		p
order by p.privilege_descr;

type	rec_type is table of driver%ROWTYPE index by binary_integer;
recs	rec_type;

begin

   if common.header (rpt_title$, cache_timeout=>0) then
      open driver;
      fetch driver BULK COLLECT into recs;
      close driver;

      htp.header (3, 'Application: ' || common.decode_value ('application_name', 'applications',
					'application_id', app_id) );

      htp.anchor ('app_maint.html', 'Return to main application menu');
      htp.para;

      htp.p ('Select all privileges authorized to access this web page application.');
      htp.para;

      htp.formOpen ('app_maint.add', 'POST');
      htp.formHidden ('app_id', app_id);
      htp.formHidden ('pcode', 'dummy');
      htp.formHidden ('pcode', 'dummy');

      for i in 1..recs.COUNT LOOP
         htp.formCheckBox ('pcode', recs(i).privilege_code, recs(i).checked);
         htp.p (recs(i).privilege_descr);
         htp.br;
      end loop;

      htp.para;
      htp.formSubmit;
      htp.formClose;
      common.footer;
   end if;

end edit;

-- *******************************************************************************
-- *******************************************************************************

function app_wildcard_exists (pkg_name varchar2) return boolean is

cursor	driver is
select	1
from	applications
where	application_name = lower (pkg_name) || '.%';

dummy		integer;
rec_found	boolean;

begin

   open driver;
   fetch driver into dummy;
   rec_found := driver%FOUND;
   close driver;

   return rec_found;

end app_wildcard_exists;

-- *******************************************************************************

procedure new_app (pkg varchar2) is

cursor	driver is
select	distinct lower (object_name) object_name
from	user_arguments
where	package_name	= upper (pkg)
and	not exists
(	select	1
	from	applications
	where	application_name = lower (pkg) || '.' || lower (object_name)
)
order by 1;

type	rec_type is table of driver%ROWTYPE index by binary_integer;
recs	rec_type;

begin

   if common.header (rpt_title$, cache_timeout=>0) then
      open driver;
      fetch driver BULK COLLECT into recs;
      close driver;

      htp.header (3, 'Add the URL ' || pkg || ' plus the portion below.');
      htp.formOpen ('app_maint.add_app', 'POST');
      htp.formHidden ('pkg', pkg);
      htp.formSelectOpen ('app_name');

      if not app_wildcard_exists(pkg) then
         htp.formSelectOption ('*** Any URL starting with ' || pkg, 'SELECTED',
		cattributes=>'VALUE="%"');
      end if;

      for i in 1..recs.COUNT LOOP
         htp.formSelectOption (recs(i).object_name);
      end loop;

      htp.formSelectClose;
      htp.formSubmit;
      htp.formClose;

      common.footer;
   end if;

end new_app;

-- *******************************************************************************
-- *******************************************************************************

procedure remove (id integer) is

begin

   if common.header (rpt_title$, test_only=>true) then
      delete from applications
      where	application_id	= id;

      common.redirect_URL ('app_maint.html');
   end if;

end remove;

-- *******************************************************************************

procedure add_app (pkg varchar2, app_name varchar2) is

app_id	integer;

begin

   if common.header (rpt_title$, test_only=>true) then
      select application_id_seq.nextval into app_id from dual;

      insert into applications (application_id, application_name)
         values (app_id, pkg || '.' || app_name);

      common.redirect_URL ('app_maint.edit?app_id=' || app_id);
   end if;

end add_app;

-- *******************************************************************************

procedure new_app_popup is

begin

   htp.formOpen ('app_maint.new_app', 'POST');
   common.show_popup_table ('lower(object_name)', 'lower(object_name)', 'user_objects',
				'Add a URL ', where_claws=>'where object_type=''PACKAGE''',
				var_name=>'pkg');
   htp.formSubmit;
   htp.formClose;

end new_app_popup;

-- *******************************************************************************

procedure html is

cursor	C1 is
select	*
from	applications
order by application_name;

cursor	C2 is
select	ap.application_id, pc.privilege_code, pc.privilege_descr
from	privilege_codes		pc,
	application_privs	ap
where	pc.privilege_code	= ap.privilege_code
order by ap.application_id, pc.privilege_descr;

type	app_type	is table of C1%ROWTYPE index by binary_integer;
type	app_priv_type	is table of C2%ROWTYPE index by binary_integer;

app_recs	app_type;
app_priv_recs	app_priv_type;
sepSTR		varchar2(4);
anchor		varchar2(200);

begin

   if common.header (rpt_title$, cache_timeout=>0) then
      open C1;
      fetch C1 BULK COLLECT into app_recs;
      close C1;

      open C2;
      fetch C2 BULK COLLECT into app_priv_recs;
      close C2;

      htp.para;
      htp.p ('All applications that appear on this page are restricted to specific privileges.');
      htp.p ('You may assign applications to privileges, which are assigned to functional roles on the');
      htp.anchor ('authorize.html', 'Authorize Page.');
      htp.p ('You may grant and revoke privileges from an application.');
      htp.bold ('Warning:');
      htp.p ('Removing an application from this list grants full access to the public.');
      htp.para;

      new_app_popup;

      htp.tableOpen ('BORDER');
      htp.tableRowOpen ('LEFT', 'BOTTOM');
      htp.tableHeader ('Action');
      htp.tableHeader ('Application Name');
      htp.tableHeader ('Privilege Required');
      htp.tableRowClose;

      for i in 1..app_recs.COUNT LOOP
         anchor := common.space2$ ||
			htf.anchor ('app_maint.remove?id=' || app_recs(i).application_id,
					'Remove');
         for j in 1..app_priv_recs.COUNT LOOP
            if app_recs(i).application_id = app_priv_recs(j).application_id then
               anchor := null;
            end if;
            exit when anchor is null;
         end loop;

         htp.tableRowOpen;
         htp.tableData (htf.anchor ('app_maint.edit?app_id=' || app_recs(i).application_id, 'Edit') || anchor);
         htp.tableData (app_recs(i).application_name);

-- **********

         sepSTR := null;
         htp.p ('<TD>');
         for j in 1..app_priv_recs.COUNT LOOP
            if app_recs(i).application_id = app_priv_recs(j).application_id then
               htp.p (sepSTR || app_priv_recs(j).privilege_descr);
               sepSTR	:= htf.br;
            end if;

            exit when app_priv_recs(j).application_id > app_recs(i).application_id;
         end loop;
         htp.p ('</TD>');

         htp.tableRowClose;
      end loop;

      htp.tableClose;
      common.footer;
   end if;

end html;

-- *******************************************************************************

end app_maint;
/
show error

set define on

--	Copyright (C) 2004-2008 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>