set define off

create or replace package body authorize as

rpt_title$	constant varchar2(30)	:= 'Authorize Users &amp; Roles';

-- ******************************************************************************
-- ******************************************************************************

procedure put (buffer varchar2, newLine boolean default false) is

begin

   htp.prn (buffer);
   if newLine then
      htp.prn (common.crlf$);
   else
      htp.prn (common.tab$);
   end if;

end put;

-- ******************************************************************************

procedure header is

begin

   htp.prn (common.crlf$);
   put ('First');
   put ('Last');
   put ('Email');
   put ('Affiliation');
   put ('Name1');
   put ('Name2');
   put ('Department');
   put ('Address 1');
   put ('Address 2');
   put ('Address 3');
   put ('PO Box');
   put ('Town');
   put ('Postal Code');
   put ('Zip');
   put ('Country', true);

end header;

-- ******************************************************************************

procedure address (pid integer) is

cursor	driver is
select	affil.affil_abbrev, affil.name1, affil.name2, affil.department,
	addr.*
from	affiliations		affil,
	addresses		addr
where	addr.address_id		= common.get_user_primary_addr_id (pid)
and	affil.affiliation_id	= addr.affiliation_id;

driver_rec	driver%ROWTYPE;

begin

   open driver;
   fetch driver into driver_rec;
   close driver;

   put (driver_rec.affil_abbrev);
   put (driver_rec.name1);
   put (driver_rec.name2);
   put (driver_rec.department);
   put (driver_rec.address1);
   put (driver_rec.address2);
   put (driver_rec.address3);
   put (driver_rec.po_box);
   put (driver_rec.town);
   put (driver_rec.postal_code);
   put (driver_rec.zip_code);
   put (driver_rec.country_code, true);

end address;

-- ******************************************************************************

procedure xls (fcode varchar2) is

cursor	driver is
select	person_id, fname, lname
from	persons	p
where	person_id	in
(	select	person_id
	from	user_functions
	where	function_code	= fcode)
order by sort_lname, sort_fname;

begin

   common.get_gbl_pid;
   if common.has_privs_to_page then
      owa_util.mime_header('application/vnd.ms-excel', TRUE);
      header;

      for driver_rec in driver LOOP
         put (driver_rec.fname);
         put (driver_rec.lname);
         put (common.get_user_email (driver_rec.person_id));
         address (driver_rec.person_id);
      end loop;

   end if;

end xls;

-- ******************************************************************************
-- ******************************************************************************

procedure add_user (this_function varchar2, uid varchar2) is

PID		integer;

begin

   common.get_gbl_pid;
   if common.has_privs_to_page then

      if common.validate_number (uid) then
         PID := uid;
      else
         PID := common.decode_value ('person_id', 'persons', 'user_id', upper (UID) );
      end if;

      insert into user_functions (function_code, person_id)
      select this_function, pid
      from   dual
      where  PID is not null
      and  NOT exists
	(	select	1
		from	user_functions
		where	function_code	= this_function
		and	person_id	= pid
	);

      common.redirect_URL ('authorize.html');
   end if;

end add_user;

-- ******************************************************************************
-- ******************************************************************************

procedure add_priv (this_function varchar2, this_privilege varchar2) is

begin

   common.get_gbl_pid;
   if common.has_privs_to_page then
      insert into function_privileges (function_code, privilege_code)
      select this_function, this_privilege
      from   dual
      where  NOT exists
	(	select	1
		from	function_privileges
		where	function_code	= this_function
		and	privilege_code	= this_privilege
	);

      common.redirect_URL ('authorize.html');
   end if;

end add_priv;

-- ******************************************************************************
-- ******************************************************************************

procedure del_priv (fcode varchar2, pcode varchar2) is

begin

   common.get_gbl_pid;
   if common.has_privs_to_page then
      delete from function_privileges
      where  function_code	= fcode
      and    privilege_code	= pcode;


      common.redirect_URL ('authorize.html');
   end if;

end del_priv;

-- ******************************************************************************
-- ******************************************************************************

procedure del_user (fcode varchar2, pid number) is

begin

   common.get_gbl_pid;
   if common.has_privs_to_page then
      delete from user_functions
      where  person_id		= pid
      and    function_code	= fcode;

      common.redirect_URL ('authorize.html');
   end if;

end del_user;

-- ******************************************************************************
-- ******************************************************************************

procedure bulk_save (this_function varchar2, emails varchar2) is

cursor	driver (email$ varchar2) is
select	distinct person_id
from	profiles
where	email_address	= email$;

email_list	long	:= emails;
email_addr	profiles.email_address%TYPE;
fail_safe	integer	:= 0;  -- prevent an infinite loop
IDs		common.int_arr;

err1		long := null;
err2		long := null;
msg		long := null;

begin

   if common.header (rpt_title$) then

--	Format the list separators to one comma between each address

      email_list := trim (email_list);
      email_list := replace (email_list, ' ', ',');
      email_list := replace (email_list, ';', ',');
      email_list := replace (email_list, common.eol$, ',');
      email_list := replace (email_list, common.cr$, ',');
      email_list := replace (email_list, common.tab$, ',');

      LOOP
         exit when instr (email_list, ',,') = 0;
         email_list := replace (email_list, ',,', ',');

         fail_safe := fail_safe + 1;
         exit when fail_safe > 100;
      end loop;

      fail_safe := 0;
      LOOP
         exit when email_list is null;

         email_addr := common.get_next_token (email_list, ',');
         open driver (email_addr);
         fetch driver BULK COLLECT into IDs;
         close driver;

         if IDs.COUNT = 0 then
            err1 := err1 || htf.br || email_addr;

         elsif IDs.COUNT = 1 then
            insert into user_functions (person_id, function_code)
            select IDs(1), this_function
            from   dual
            where  not exists
		(	select	1
			from	user_functions
			where	person_id	= IDs(1)
			and	function_code	= this_function
		);

           msg := msg || htf.br || email_addr || ' &mdash; ' ||
			common.get_user_name(IDs(1)) || ', ' ||
			common.get_user_affil_abbrev_list(IDs(1));

         else
            err2 := err2 || htf.br || htf.bold (email_addr, cattributes=>common.red$);
            for i in 1..IDs.COUNT LOOP
               err2 := err2 || ', ' ||	common.get_user_name(IDs(i)) || ' [' ||
					common.get_user_affil_abbrev_list(IDs(i)) || ']';
            end loop;
         end if;

         fail_safe := fail_safe + 1;
         exit when fail_safe > 100;
      end loop;

      if err1 is not null then
         err1 := substr (err1, length(htf.br)+1);
         htp.header (3, 'ERROR &mdash Emails Not Found', cattributes=>common.red$);
         htp.p (err1);
      end if;

      if err2 is not null then
         err2 := substr (err2, length(htf.br)+1);
         htp.header (3, 'ERROR &mdash Multiple Emails found', cattributes=>common.red$);
         htp.p (err2);
      end if;

      if msg is not null then
         msg := substr (msg, length(htf.br)+1);
         htp.header (3, 'Persons Added to Functional Role ' ||
		common.decode_value ('function_descr', 'function_codes', 'function_code', this_function) );
         htp.p (msg);
      end if;

      htp.para;
      htp.anchor ('authorize.html', 'Back to Main Authorize Page');
      htp.p (common.space2$);
      htp.anchor ('authorize.bulk_roles', 'Back to Bulk Functional Role Upload');

      common.footer;
   end if;

end bulk_save;

-- ******************************************************************************
-- ******************************************************************************

procedure bulk_roles is

fwhere	varchar2(100);

begin

   if common.header (rpt_title$) then
      htp.header (3, 'Bulk add persons to Functional Roles');
      fwhere := 'where nvl (vendor_flag, ''N'')	= ''N'' and nvl (abstract_flag, ''N'') = ''N''';

      htp.formOpen ('authorize.bulk_save', 'POST');
      common.show_lookup_table ('function', 'Functional Role', where_claws=>fwhere);
      htp.para;
      htp.p ('Email address list.  Separate by comma, semi-colon or line feed.');
      htp.br;

      htp.formTextArea ('emails', 20, 50);
      htp.br;
      htp.formSubmit;
      htp.formClose;

      common.footer;
   end if;

end bulk_roles;

-- ******************************************************************************

procedure get_applications (code varchar2) is

cursor	driver is
select	apps.application_name
from	applications		apps,
	application_privs	ap
where	ap.privilege_code	= code
and	apps.application_id	= ap.application_id
order by apps.application_name;

begin

   for driver_rec in driver LOOP
      htp.listItem (driver_rec.application_name);
   end loop;

end get_applications;

-- ******************************************************************************

procedure show_priv_items is

cursor	driver is
select	*
from	privilege_codes
order by privilege_descr;

begin

   htp.tableOpen ('BORDER');
   htp.tableRowOpen ('LEFT', 'BOTTOM');
   htp.tableHeader ('SPMS Privilege');
   htp.tableHeader ('SPMS Module');
   htp.tableRowClose;

   for driver_rec in driver LOOP
      htp.tableRowOpen ('LEFT', 'TOP');
      htp.tableData (driver_rec.privilege_descr);
      htp.p ('<TD>');
         get_applications (driver_rec.privilege_code);
      htp.p ('</TD>');
      htp.tableRowClose;
   end loop;

   htp.tableClose;

end show_priv_items;

-- ******************************************************************************

function get_priv_list (fcode varchar2, sepchr varchar2 default htf.br) return varchar2 is

cursor	driver is
select	*
from	privilege_codes
where	privilege_code in
(	select	privilege_code
	from	function_privileges
	where	function_code = fcode)
order by privilege_descr;

buffer	long := null;
URL	varchar2(100);

begin

   for driver_rec in driver LOOP
      URL	:= htf.anchor ('authorize.del_priv' ||
			'?fcode=' || fcode ||
			'&pcode=' || driver_rec.privilege_code, '(Remove)');
      buffer	:= buffer || sepchr || driver_rec.privilege_descr || ' ' || URL;
   end loop;

   return substr (buffer, length(sepchr)+1 );

end get_priv_list;

-- ******************************************************************************

procedure get_user_list (fcode varchar2, sepchr varchar2 default htf.br) is

cursor	driver is
select	person_id, common.get_user_name (person_id) name,
	common.get_user_affil_abbrev_list (person_id) affils
from	persons
where	person_id in
(	select	person_id
	from	user_functions
	where	function_code = fcode)
order by sort_lname, sort_fname, sort_mname;

type	rec_type is table of driver%ROWTYPE index by binary_integer;
recs	rec_type;

begin

   open driver;
   fetch driver BULK COLLECT into recs;
   close driver;

   htp.p ('<TD>');

   for i in 1..recs.COUNT loop
      if i > 1 then
         htp.p (sepchr);
      end if;

      htp.p (recs(i).name || ',');
      htp.p (recs(i).affils);
      htp.anchor ('authorize.del_user' || '?fcode=' || fcode || '&pid=' || recs(i).person_id, '(Remove)');
   end loop;

   if recs.COUNT = 0 then
      htp.p (common.space$);
   end if;

   htp.p ('</TD>');

end get_user_list;

-- ******************************************************************************

procedure html is

cursor	driver is
select	*
from	function_codes
where	nvl (vendor_flag, 'N')		= 'N'
and	nvl (abstract_flag, 'N')	= 'N'
order by function_descr;

URL	varchar2(200) := htf.anchor ('javascript:newWindow(''profile.find_account'')', 'Search for an Account');
fwhere	varchar2(200);

begin

   if common.header (rpt_title$) then
      htp.p (profile.js$);
      fwhere := 'where nvl (vendor_flag, ''N'')	= ''N'' and nvl (abstract_flag, ''N'') = ''N''';

      htp.tableOpen ('BORDER', cattributes=>'WIDTH="100%"');
      htp.tableRowOpen ('LEFT', 'TOP', cattributes=>'STYLE="background-color: #EEEEEE"');

      htp.p ('<TD>');
         htp.formOpen ('authorize.add_priv', 'POST');
         htp.tableopen;
         common.show_lookup_table ('function',	'Functional Role', where_claws=>fwhere);
         common.show_lookup_table ('privilege',	'Privilege');
         common.show_data_row (null, htf.formSubmit (null, 'Authorize') );
         htp.tableClose;
         htp.formClose;
      htp.p ('</TD>');

      htp.tableData (htf.anchor ('maint.tab?tab_name=function_codes&order_by=function_descr',
			'Functional Role Maintainance'), 'CENTER');

      htp.p ('<TD>');
         htp.formOpen ('authorize.add_user', 'POST', cattributes=>'NAME="SPMS"');
         htp.tableopen;
         common.show_lookup_table ('function', 'Functional Role', where_claws=>fwhere);
         common.show_data_row ('User ID', htf.formText ('uid', 30, 30)  || htf.br || URL ||
			common.space2$ || htf.anchor ('authorize.bulk_roles', 'Bulk Upload'));
         common.show_data_row (null, htf.formSubmit (null, 'Add User') );
         htp.tableClose;
         htp.formClose;
      htp.p ('</TD>');

      htp.tableRowClose;

-- **********

      common.show_data_sep (3, common.space$);
      htp.tableRowOpen ('LEFT', 'BOTTOM', cattributes=>common.red$);
      htp.tableHeader ('Privilege');
      htp.tableHeader ('Functional Role', 'CENTER');
      htp.tableHeader ('Users');
      htp.tableRowClose;

-- **********

      for driver_rec in driver LOOP
         URL := htf.anchor ('authorize.xls?fcode=' || driver_rec.function_code,
				driver_rec.function_descr);

         htp.tableRowOpen ('LEFT', 'TOP');
         htp.tableData (nvl (get_priv_list  (driver_rec.function_code), common.space$) );
         htp.tableHeader (URL, 'CENTER');
         get_user_list (driver_rec.function_code);
         htp.tableRowClose;
      end loop;

      htp.tableClose;
      htp.para;
      show_priv_items;
      common.footer;
   end if;

end html;

-- ******************************************************************************
-- ******************************************************************************

end authorize;
/
show error

set define on

--	Copyright (C) 2004 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>

