set define off

Create or Replace package body cc_vendor as

rpt_title$	constant varchar2(30)	:= 'Online Payment Processing';

amount$		constant SPMS_var_codes.SPMS_var_code%type := 'AMT';
invoice$	constant SPMS_var_codes.SPMS_var_code%type := 'INV';
ID_list$	constant SPMS_var_codes.SPMS_var_code%type := 'IDS';
status$		constant SPMS_var_codes.SPMS_var_code%type := 'STAT';
receipt$	constant SPMS_var_codes.SPMS_var_code%type := 'PCI';
status_text$	constant SPMS_var_codes.SPMS_var_code%type := 'PST';
person_id$	constant SPMS_var_codes.SPMS_var_code%type := 'PID';
username$	constant SPMS_var_codes.SPMS_var_code%type := 'UN';
last_name$	constant SPMS_var_codes.SPMS_var_code%type := 'LN';
first_name$	constant SPMS_var_codes.SPMS_var_code%type := 'FN';
email_address$	constant SPMS_var_codes.SPMS_var_code%type := 'EA';
external_URL$	constant varchar2(8) := 'external';

hidden$		constant var_type_codes.var_type_code%type := 'H';
return$		constant var_type_codes.var_type_code%type := 'R';
validation$	constant var_type_codes.var_type_code%type := 'V';
confirmation$	constant var_type_codes.var_type_code%type := 'C';

regist_id$	constant varchar2(1) := 'R';
amt$	constant varchar2(1) := 'A';
id_sepchar$	constant varchar2(1) := ',';




type	IPN_rec_type is record (
		person_id		integer,
		amount			number,
		invoice			payment_vendor_vars.var_text%TYPE,
		receipt_id		payment_vendor_vars.var_text%TYPE,
		status			payment_vendor_vars.var_text%TYPE,
		status_text		payment_vendor_vars.var_text%TYPE,
		payment_succeeded	boolean := false,
		external_registration	boolean := false,
		page_ID			integer,
		regist_id		common.int_arr,
		amt				common.int_arr);



-- *************************************************************************
-- *************************************************************************

function get_vendor_rec return payment_vendor_codes%ROWTYPE is

cursor	driver is
select	*
from	payment_vendor_codes
where	payment_vendor_code = common.cc_vendor$;

vendor_rec	payment_vendor_codes%ROWTYPE;

begin

   open driver;
   fetch driver into vendor_rec;
   close driver;

   return vendor_rec;

end get_vendor_rec;

-- *************************************************************************
-- *************************************************************************

function get_IDs (ID_array varchar2, flag varchar2) return common.int_arr is

ID_list		common.int_arr;
buffer		varchar2(4000) := ID_array;
ipos		integer;
ilen		integer;

begin

	LOOP
		exit when buffer is null;

		ipos := instr (buffer, id_sepchar$);
		if ipos = 0 then -- at last token
			ilen := length (buffer);
		else
			ilen := ipos - 2;  -- Adjust for separation character and skip the first byte
		end if;

		--	Don't return the ID unless it is the one we want (flag)

		if substr (buffer, 1, 1) = flag then  -- get the numeric portion
			ID_list(ID_list.count+1) := substr (buffer, 2, ilen);
		end if;

		if ipos = 0 then
			buffer := null;
		else
			buffer := substr (buffer, ipos+1);
		end if;

	end loop;

	return ID_list;

end get_IDS;

-- *************************************************************************
-- *************************************************************************

procedure fmt_URL (redirect_URL boolean, buffer in out varchar2,
		sepchr varchar2, var_name varchar2, var_text varchar2) is

begin

   if redirect_URL then
      buffer := buffer || sepchr || var_name || '=' || var_text;
   else
      htp.formHidden (var_name, var_text);
   end if;

end fmt_URL;

-- *************************************************************************

function build_query_string (redirecting boolean, var_type varchar2,
				IPN_rec IPN_rec_type, hh boolean default false, page_id number default null) return varchar2 is

cursor	driver is
select	*
from	payment_vendor_vars
where	payment_vendor_code	= common.cc_vendor$
and	var_type_code		= var_type;

sepchr	varchar2(1)	:= '?';
buffer	long		:= null;
ID_list	varchar2(4000)	:= null;
hh_bool boolean := hh;

begin

   for driver_rec in driver loop
      if driver_rec.var_text is not null then
	 	fmt_URL (redirecting, buffer, sepchr, driver_rec.var_name, driver_rec.var_text);

      elsif driver_rec.SPMS_var_code = invoice$ then
		 if(hh_bool) then

		    fmt_URL (redirecting, buffer, sepchr, driver_rec.var_name,
				'hh'|| '.' || sqid || '.'  || IPN_rec.person_id || '.' || to_char (sysdate, 'yyyymmddhh24miss') );
		 else
		    if(page_id is not null) then
			fmt_URL (redirecting, buffer, sepchr, driver_rec.var_name,
				page_id || '.' || IPN_rec.person_id || '.' || to_char (sysdate, 'yyyymmddhh24miss') );
		    else
			fmt_URL (redirecting, buffer, sepchr, driver_rec.var_name,
				IPN_rec.person_id || '.' || to_char (sysdate, 'yyyymmddhh24miss') );
		    end if;
		 end if;
      elsif driver_rec.SPMS_var_code = amount$ then
		 fmt_URL (redirecting, buffer, sepchr, driver_rec.var_name, IPN_rec.amount);

      elsif driver_rec.SPMS_var_code = ID_list$ then
		 for i in 1..IPN_rec.regist_id.COUNT LOOP
		    ID_list := ID_list || id_sepchar$ || regist_id$ || IPN_rec.regist_id(i);
		 end loop;

		 for i in 1..IPN_rec.amt.COUNT LOOP
		    ID_list := ID_list || id_sepchar$ || amt$ || IPN_rec.amt(i);
		 end loop;
		 ID_list := substr (ID_list, 2);

		 fmt_URL (redirecting, buffer, sepchr, driver_rec.var_name, ID_list);

      elsif driver_rec.SPMS_var_code = person_id$ then
		 fmt_URL (redirecting, buffer, sepchr, driver_rec.var_name, IPN_rec.person_id);

      elsif driver_rec.SPMS_var_code = username$ then
		 fmt_URL (redirecting, buffer, sepchr, driver_rec.var_name,
			common.decode_value ('user_id', 'persons', 'person_id', IPN_rec.person_id) );

      elsif driver_rec.SPMS_var_code = last_name$ then
		 fmt_URL (redirecting, buffer, sepchr, driver_rec.var_name,
			common.decode_value ('lname', 'persons', 'person_id', IPN_rec.person_id) );

      elsif driver_rec.SPMS_var_code = first_name$ then
		 fmt_URL (redirecting, buffer, sepchr, driver_rec.var_name,
			common.decode_value ('fname', 'persons', 'person_id', IPN_rec.person_id) );

      elsif driver_rec.SPMS_var_code = email_address$ then
		 fmt_URL (redirecting, buffer, sepchr, driver_rec.var_name,
			common.get_user_email(IPN_rec.person_id) );

      end if;

      sepchr := '&';
   end loop;

   return buffer;

end build_query_string;

-- *************************************************************************

procedure call_confirm_failure_url (IPN_rec IPN_rec_type) is

	cursor	driver is
		select	confirmation_URL, failure_URL
		from	payment_vendor_codes
		where	payment_vendor_code = common.cc_vendor$;

	driver_rec	driver%ROWTYPE;
	buffer		varchar2(2000);

begin

   open driver;
   fetch driver into driver_rec;
   close driver;

   if driver_rec.confirmation_URL || driver_rec.failure_URL is not null then
      buffer := build_query_string (true, confirmation$, IPN_rec,hh$,sqid);
   end if;

   if IPN_rec.payment_succeeded then
      if driver_rec.confirmation_URL is not null then
	 	buffer := UTL_HTTP.REQUEST (driver_rec.confirmation_URL || buffer);
      end if;
   elsif driver_rec.failure_URL is not null then
      buffer := UTL_HTTP.REQUEST (driver_rec.failure_URL || buffer);
   end if;

end call_confirm_failure_url;

-- *************************************************************************

function get_IPN_data (name_array owa.vc_arr, value_array owa.vc_arr) return IPN_rec_type is

	cursor	driver is
		select	var_name, SPMS_var_code, var_text
		from	payment_vendor_vars
		where	var_type_code in (return$, validation$);

	type		var_type is table of driver%ROWTYPE index by binary_integer;
	var_names	var_type;
	IPN_rec		IPN_rec_type;
	ipos		integer;
	y			integer :=0;
	token$		varchar2(10000);

begin

	open driver;
	fetch driver BULK COLLECT into var_names;
	close driver;

	for i in 1..var_names.COUNT LOOP  -- database variables mapped to registration values

		for j in 1..name_array.COUNT LOOP  -- web variables returned by vendor

			if lower (name_array(j)) = external_URL$ then
				IPN_rec.external_registration	:= true;
				IPN_rec.page_ID			:= value_array(j);

			elsif var_names(i).var_name = name_array(j) then
				if(var_names(i).SPMS_var_code = invoice$) then
					if(substr(value_array(j),1,2) = 'hh') then
						hh$ := true;
						y:=1;
						loop
							token$ := common.stringTokenizer(value_array(j), y , '.');
					    exit when token$ is	null;
					if(y = 2) then
								sqid := token$;
							elsif(y = 3) then
								IPN_rec.person_id := token$;
							elsif(y = 4) then
								IPN_rec.invoice := token$;
							end if;
							y :=y+1;
						end loop;
					else
						y:=1;
						loop
							token$ := common.stringTokenizer(value_array(j), y , '.');
							exit when token$ is	null;
							if(y = 1) then
								IPN_rec.page_id := token$;
							elsif(y = 2) then
								IPN_rec.person_id := token$;
							elsif(y = 3) then
								IPN_rec.invoice := token$;
							end if;
							y :=y+1;
						end loop;
					end if;
				elsif var_names(i).SPMS_var_code = amount$ then
					IPN_rec.amount := value_array(j);
				elsif var_names(i).SPMS_var_code = receipt$ then
					IPN_rec.receipt_id := value_array(j);
				elsif var_names(i).SPMS_var_code = status$ then
					IPN_rec.status := value_array(j);
					if var_names(i).var_text = value_array(j) then
						IPN_rec.payment_succeeded := true;
					end if;
				elsif var_names(i).SPMS_var_code = status_text$ then
					IPN_rec.status_text := value_array(j);
				elsif var_names(i).SPMS_var_code = ID_list$ then
					IPN_rec.regist_id := get_IDs (value_array(j), regist_id$);
					IPN_rec.amt := get_IDs (value_array(j), amt$);
				end if;
			end if;
		end loop;
	end loop;

	-- All vendors return a status but not all return the text.  For example, PayPal's returned
	-- stauts is "Complete" so that is sufficient.	Whereas Authorize.NET returns "1" and the text
	-- "Payment succeeded" in another string.  We want to dispaly the string to the user.

	IPN_rec.status_text := nvl (IPN_rec.status_text, IPN_rec.status);

	return IPN_rec;

end get_IPN_data;

-- *************************************************************************

function record_payment (IPN_rec IPN_rec_type) return integer is

	cursor	driver (pid$ integer, inv$ varchar2) is
		select	payment_id
		from	payments
		where	person_id	= pid$
		and 	invoice		= inv$;

	pay_id		integer default null;
	now$		constant date := common.get_UTC_time;
	rec_found	boolean;
	reg_upd_cnt	integer := 0;
	dummy		varchar2(10000);


begin


	if(hh$ = true and sqid is not null) then
		dummy := 'hh' || '.'|| sqid || '.' || IPN_rec.person_id || '.' || IPN_rec.invoice;
		save_ipn_hh_data;
		update registrants_choices set invoice = dummy, ipn_status = IPN_rec.status, receipt = IPN_rec.receipt_id, payment_date = sysdate
		where registrant_id = IPN_rec.person_id and seq_id = sqid;
	end if;

   	if(hh$ = false) then
	    select payment_id_seq.nextval into pay_id from dual;

	    insert into payments (person_id, timestamp, processed, amount,
			      payment_code, IPN_status, invoice, receipt, payment_id)
		      values (IPN_rec.person_id, now$, now$, IPN_rec.amount,
			      'CC', IPN_rec.status_text, IPN_rec.invoice, IPN_rec.receipt_id, pay_id);

		for i in 1..IPN_rec.regist_id.COUNT LOOP
			Insert into registrations_payments (registration_id,payment_id, amount)
						values (IPN_rec.regist_id(i),pay_id,IPN_rec.amt(i));
		end loop;

  	end if;

    return pay_id;

end record_payment;

-- *************************************************************************

procedure record_IPN_values (pay_id integer, name_array owa.vc_arr, value_array owa.vc_arr) is

now$	constant date := common.get_UTC_time;

begin

 if(hh$ = false) then
   for i in 1..name_array.COUNT LOOP
      insert into payment_vendor_IPNs (payment_id, var_name, var_values, timestamp)
				values (pay_id, name_array(i), value_array(i), now$);
   end loop;
 end if;

end record_IPN_values;

-- *************************************************************************

procedure IPN (name_array owa.vc_arr, value_array owa.vc_arr) is

IPN_rec		IPN_rec_type;
pay_id		integer;
display_invoice	boolean := common.decode_value ('IPN_DISPLAYS_INVOICE', 'payment_vendor_codes',
						'payment_vendor_code', common.cc_vendor$) = 'Y';
msg		varchar2(10000);
begin

   IPN_rec := get_IPN_data (name_array, value_array);

   if IPN_rec.payment_succeeded then
      pay_id := record_payment (IPN_rec);
      if(hh$ = false) then
			record_IPN_values (pay_id, name_array, value_array);
      end if;
      if IPN_rec.external_registration then -- record registration in SPMS
		 	insert into registrants (page_id, person_id)
		 	values (IPN_rec.page_ID, IPN_rec.person_ID);
      end if;
   end if;

   call_confirm_failure_url (IPN_rec);

   if display_invoice then
      if IPN_rec.payment_succeeded then
		if(hh$) then

	  		send_hh_cc_payment_mail(sqid);
	  		common.redirect_URL (common.server || common.vdir$ ||
				    'hotel_handling.ipn_hotel_redirected_receipt?seq=' || sqid);
       	else
	   		common.redirect_URL (common.server || common.vdir$ ||
				    'cc_vendor.ipn_redirected'	||
				    '?page_id='|| IPN_rec.page_id ||'&pid=' || IPN_rec.person_id);
		end if;
      else
		 common.public_header (rpt_title$);
	 	 htp.header (3, 'Payment Transaction Failed', cattributes=>common.red$);
		 htp.tableOpen;
		 common.show_data_row ('Invoice',	IPN_rec.person_id || '.' || IPN_rec.invoice);
		 common.show_data_row ('Receipt',	IPN_rec.receipt_id);
		 common.show_data_row ('Status',	IPN_rec.status || common.space$ || IPN_rec.status_text);
		 htp.tableClose;
		 common.footer;
		 hh$ := false;
      end if;
   end if;
end IPN;

-- *************************************************************************

procedure IPN_redirected (pid varchar2,page_id number default null) is

begin

   common.get_gbl_pid;
    registration.invoice2 (page_id,common.ENCRYPT(pid),
      	htf.header (3, 'Your payment was processed by ' ||
			common.decode_value ('PAYMENT_VENDOR_DESCR', 'payment_vendor_codes',
			'payment_vendor_code', common.cc_vendor$)	|| '.',
			cattributes=>common.red$) );
end IPN_redirected;

-- *************************************************************************
-- *************************************************************************

procedure forwarder (pid integer, total_amt number, reg_id common.ident_arr,
			amt common.ident_arr,
			return_URL in out varchar2, hh boolean default false, page_id number default null) is

vendor_rec	payment_vendor_codes%ROWTYPE;
buffer		long		:= null;
redirect_URL	boolean		:= false;
IPN_rec		IPN_rec_type;
hh_bool boolean := hh;
begin

   vendor_rec := get_vendor_rec;

   if return_URL is not null then
      redirect_URL := true;  -- if set the URL string is returned.
   elsif nvl (vendor_rec.action_URL_override, 'N') = 'Y' then
      redirect_URL := true;
   end if;

   if NOT redirect_URL then
      common.public_header (rpt_title$);
      htp.p (vendor_rec.payment_msg);
      htp.formOpen (vendor_rec.action_URL, 'POST', '_new');
   end if;

   IPN_rec.person_id	:= pid;
   IPN_rec.amount	:= total_amt;
   IPN_rec.invoice	:= pid || '.' || sysdate;

   for i in 1..reg_id.COUNT LOOP
      if common.validate_number (reg_id(i)) then
	 		IPN_rec.regist_id(IPN_rec.regist_id.COUNT+1) := reg_id(i);
	 		IPN_rec.amt(IPN_rec.amt.COUNT+1) := amt(i);
      end if;
   end loop;

   hh$ := hh_bool;

   buffer := build_query_string (redirect_URL, hidden$, IPN_rec, hh_bool, page_id);

   if redirect_URL then
      if return_URL is not null then
	 	return_URL := buffer;
      else
	 	common.redirect_URL (vendor_rec.action_URL || buffer);
      end if;
   else
      htp.p (nvl (vendor_rec.input_tag_text, htf.formSubmit));
      htp.formClose;
      htp.para;
      common.footer;
   end if;

end forwarder;

procedure set_ipn_hh_data(pid integer ,start_Date date,end_Date date,config varchar2,
		      hotel_id integer,confirmed varchar2,deposit number,room_id integer,choice_order varchar2,date_Record date, paycode varchar2 default null,
		      status varchar2 default null) is

	seq_value number;

begin
    if(choice_order = 'fc') then
	    select choice_sequence_seq.nextval into seq_value from dual;
	    insert into temp_registrants_choices (registrant_id,start_date,end_date,config,hotel_id,confirmed,deposit,room_id,choice_order,date_record,payment_code,status,booking_date,seq_id)
	       values (
	       pid,
	       start_Date,
	       end_Date,
	       config,
	       hotel_id,
	       confirmed,
	       deposit,
	       room_id,
	       choice_order,
	       date_Record,
	       paycode,
	       status,
	       date_Record,
	       seq_value
	    );
	    sqid := seq_value;
     else
	      insert into temp_registrants_choices (registrant_id,start_date,end_date,config,hotel_id,confirmed,deposit,room_id,choice_order,date_record,seq_id,payment_code,status,booking_date)
	       values (
	       pid,
	       start_Date,
	       end_Date,
	       config,
	       hotel_id,
	       confirmed,
	       deposit,
	       room_id,
	       choice_order,
	       date_Record,
	       sqid,
	       paycode,
	       status,
	       date_Record
	    );
     end if;


end set_ipn_hh_data;

 procedure save_ipn_hh_data is

	 cursor driver(dummy number) is
		 select * from temp_registrants_choices
		 where seq_id = dummy
		 order by choice_id;

	 driver_rec driver%ROWTYPE;

	 seq_value number;
	 systime timestamp;
 begin
    open driver(sqid);
    loop
	    fetch driver into driver_rec;
	    exit when driver%NOTFOUND;
	    if driver_rec.choice_order = 'fc' then
			systime := systimestamp;
			select choice_sequence_seq.nextval into seq_value from dual;
			insert into registrants_choices (registrant_id,start_date,end_date,config,hotel_id,confirmed,deposit,room_id,choice_order,date_record,status,payment_code,booking_date,payment_date,deleted,seq_id)
			values (
				   driver_rec.registrant_id,
				   driver_rec.start_date,
				   driver_rec.end_date,
				   driver_rec.config,
				   driver_rec.hotel_id,
				   driver_rec.confirmed,
				   driver_rec.deposit,
				   driver_rec.room_id,
				   driver_rec.choice_order,
				   driver_rec.date_record,
				   driver_rec.status,
				   driver_rec.payment_code,
				   driver_rec.booking_date,
				   systime,
				   'N',
				   seq_value
				);
	     elsif driver_rec.choice_order = 'sc' then
			 insert into registrants_choices (registrant_id,start_date,end_date,config,hotel_id,confirmed,deposit,room_id,choice_order,date_record,seq_id,status,payment_code,booking_date,payment_date,deleted)
			 values (
				   driver_rec.registrant_id,
				   driver_rec.start_date,
				   driver_rec.end_date,
				   driver_rec.config,
				   driver_rec.hotel_id,
				   driver_rec.confirmed,
				   driver_rec.deposit,
				   driver_rec.room_id,
				   driver_rec.choice_order,
				   driver_rec.date_record,
				   seq_value,
				   driver_rec.status,
				   driver_rec.payment_code,
				   driver_rec.booking_date,
				   systime,
				   'N'
				);
	     elsif driver_rec.choice_order = 'tc' then
			  insert into registrants_choices (registrant_id,start_date,end_date,config,hotel_id,confirmed,deposit,room_id,choice_order,date_record,seq_id,status,payment_code,booking_date,payment_date,deleted)
			  values (
				   driver_rec.registrant_id,
				   driver_rec.start_date,
				   driver_rec.end_date,
				   driver_rec.config,
				   driver_rec.hotel_id,
				   driver_rec.confirmed,
				   driver_rec.deposit,
				   driver_rec.room_id,
				   driver_rec.choice_order,
				   driver_rec.date_record,
				   seq_value,
				   driver_rec.status,
				   driver_rec.payment_code,
				   driver_rec.booking_date,
				   systime,
				   'N'
				);
	    end if;
    end loop;
	close driver;
    sqid := seq_value;

 end save_ipn_hh_data;

-- *************************************************************************

procedure send_hh_cc_payment_mail(seq number default -1) is

cursor driver is
select * from registrants_choices
where seq_id = seq
and deleted = 'N'
order by date_record desc
;

driver_rec driver%ROWTYPE;

msg		varchar2(10000);

begin
	    open driver;
		fetch driver into driver_rec;
	    close driver;
	    msg := 'Your hotel deposit has been paid' || common.eol$;
	    msg := msg || 'Order nr: ' || driver_rec.invoice || common.eol$;
	    msg := msg || 'Receipt: '	|| driver_rec.receipt || common.eol$;
	    msg := msg || 'Status: '  || driver_rec.ipn_status	|| common.eol$;
	    msg := msg || 'Amount: '  || driver_rec.deposit || common.eol$;
	    msg := msg || 'Keep this e-mail for your records' || common.eol$;
	    hotel_handling_maint.send_payment_recorded_mail(common.ENCRYPT(driver_rec.registrant_id),'ccpay', driver_rec.seq_id, msg);

end send_hh_cc_payment_mail;

-- *************************************************************************

end cc_vendor;

/
show error

set define on

--	Copyright (C) 2004-2008 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>
