set define off

create or replace package body calendar as

max_len$	constant integer := 75;
new_line$	constant varchar2(2) := '\n';

diacritical	common.ident_arr;
std_ascii	common.ident_arr;
name		common.ident_arr;

-- ****************************************************************************************

procedure load_ical_chars is

cursor	driver is
select	*
from	ical_chars;

begin

   open driver;
   fetch driver BULK COLLECT into diacritical, std_ascii;
   close driver;

end load_ical_chars;

-- ****************************************************************************************

procedure write (buffer in out long) is

text		varchar2(75);
max_len		integer := max_len$;

begin

-- Don't split a line feed across lines
   if substr (buffer, max_len, 2) = new_line$ then
      max_len := max_len - 1;
   end if;

   text		:= substr (buffer, 1, max_len);
   buffer	:= substr (buffer, max_len+1);

for i in 1..length(text) LOOP
   if ascii (substr (text, i, 1)) NOT between 32 and 126 then
      htp.prn (common.crlf$ || '=======================' ||
		ascii (substr (text, i, 1)) || ' @ pos: ' || i || common.crlf$);
   end if;
end LOOP;

   htp.prn (text);
   htp.prn (common.crlf$);

   if buffer is not null then
      buffer := ' ' || buffer;
      write (buffer);
   end if;

end write;

-- ****************************************************************************************

procedure put (buffer varchar2) is

text		long := buffer;

begin

   for i in 1..diacritical.COUNT LOOP
      text := replace (text, diacritical(i), std_ascii(i) );
   end loop;

   write (text);

end put;

-- ****************************************************************************************

procedure put (prompt varchar2, buffer common.ident_arr) is

begin

   put (prompt || buffer(1));
   for i in 2..buffer.COUNT LOOP
      put (' ' || buffer(i));
   end loop;

end put;

-- ****************************************************************************************

function fmt_time (dte date) return varchar2 is

begin

   return to_char (dte,   'yyyymmdd') || 'T' ||
	  to_char (dte,   'HH24MISS') || 'Z';

end fmt_time;

-- ****************************************************************************************

function new_date (dte date, tme integer) return date is

new_date	date := dte;

begin

   new_date := new_date + ((round (tme, -2) / 100) * 60) / 1440;
   new_date := new_date + (mod (tme, 100) / 1440);

   return new_date;

end new_date;

-- ****************************************************************************************

function convert_UTC_time (dte timestamp) return date is

new_date	timestamp;

begin

   select SYS_EXTRACT_UTC (dte) into new_date from dual;
   new_date := new_date + (common.tz_offset$ * 60) / 1440;
   return new_date;

end convert_UTC_time;

-- ****************************************************************************************

function get_person_info (profile_id integer) return varchar2 is

name	varchar2(500);
pid$	constant integer := common.decode_value ('person_id', 'profiles', 'profile_id', profile_id);

begin

   name	:= common.get_user_name (pid$, translate=>'N');
   name	:= replace (name, ' - ', ', ');

   return name;

end get_person_info;

-- ****************************************************************************************

procedure open_iCAL is

begin

   owa_util.mime_header('text/calendar', TRUE, common.charset$);

   put ('BEGIN:VCALENDAR');
   put ('VERSION:2.0');
   put ('X-WR-CALNAME:' || common.conf_title$);
   put ('PRODID:-//Apple Computer\, Inc//iCal 2.0//EN');

-- put ('X-WR-TIMEZONE:America/Detroit');
   put ('CALSCALE:GREGORIAN');
   put ('METHOD:PUBLISH');

end open_iCAL;

-- ****************************************************************************************

procedure close_iCAL is

begin

   put ('END:VCALENDAR');

end close_iCAL;

-- ****************************************************************************************

procedure create_event (summary varchar2, location varchar2, uid varchar2,
			dtstart date, dtend date, organizer varchar2,
			description common.ident_arr) is

bdate	date := convert_UTC_time (dtstart);
edate	date := convert_UTC_time (dtend);
now	date := common.get_UTC_time;

begin

   put ('BEGIN:VEVENT');
   put ('DTSTART:'	|| fmt_time (bdate) );
   put ('DTEND:'	|| fmt_time (edate) );
   put ('SUMMARY:'	|| summary);
   put ('UID:'		|| replace (common.conf_title$ || '-' || uid, ' ', '-') );
   put ('DTSTAMP:'	|| fmt_time (now) );

   put ('ORGANIZER;CN='	|| nvl (organizer, 'Unknown') );
   put ('DESCRIPTION:', description);
   put ('LOCATION:'	|| location);
   put ('END:VEVENT');

end create_event;

-- ****************************************************************************************

procedure create_event (summary varchar2, location varchar2, uid varchar2,
			dtstart date, dtend date, organizer varchar2, description varchar2) is

descr_arr	common.ident_arr;

begin

   descr_arr(1)	:= description;
   create_event (summary, location, uid, dtstart, dtend, organizer, descr_arr);

end create_event;

-- ****************************************************************************************
-- ****************************************************************************************

procedure orals (sid varchar2, sdate date, loc varchar2, descr varchar2) is

cursor	driver is
select	sa.paper_id, a.title, a.description, sa.start_time, sa.duration,
	common.get_author_profile_id (a.abstract_id, common.speaker$) spk_profile_id
from	abstracts		a,
	abstract_sessions	sa
where	sa.session_id		= sid
and	a.abstract_id		= sa.abstract_id
order by sa.session_seq;

beg_date	date;
end_date	date;
speaker		varchar2(500);

begin

   for driver_rec in driver LOOP
      beg_date	:= new_date (sdate, driver_rec.start_time);
      end_date	:= new_date (sdate, driver_rec.start_time) + (driver_rec.duration/1440);
      speaker	:= get_person_info (driver_rec.spk_profile_id);

      create_event (driver_rec.paper_id || ' ' || driver_rec.title,
		loc, driver_rec.paper_id, beg_date, end_date, speaker,
		descr || new_line$ || driver_rec.description);


   end loop;

end orals;

-- ****************************************************************************************

function posters (sid varchar2) return common.ident_arr is

cursor	driver is
select	sa.paper_id, a.title,
	common.get_author_profile_id (a.abstract_id, common.speaker$) spk_profile_id
from	abstracts		a,
	abstract_sessions	sa
where	sa.session_id		= sid
and	a.abstract_id		= sa.abstract_id
order by sa.session_seq;

buffer	common.ident_arr;
i	integer := 1;
speaker	varchar2(500);

begin

   for driver_rec in driver LOOP
      speaker:= get_person_info (driver_rec.spk_profile_id);
      buffer(i) := driver_rec.paper_id || ' - ' || driver_rec.title || new_line$ || 
			'Presenter: ' || speaker || new_line$ || new_line$;
      i := i + 1;
   end loop;

   return buffer;

end posters;

-- ****************************************************************************************

procedure ics is

cursor	driver is
select	*
from	sessions	s
where	exists
(
	select	1
	from	abstract_sessions	a
	where	a.session_id	= s.session_id
)
order by
	sort_order nulls last,
	session_date, beg_time, session_id;

bdate		date;
edate		date;

loc_descr	location_codes.location_descr%TYPE;
pid		integer;
organizer	varchar2(500);
descr		varchar2(5000);

I integer := 0;

begin

   load_ical_chars;
   open_iCAL;
   for driver_rec in driver LOOP
      pid	:= common.decode_value ('person_id', 'profiles', 'profile_id', driver_rec.chair_profile_id);
      loc_descr	:= common.decode_value ('location_descr', 'location_codes', 'location_code', driver_rec.location_code);
      organizer := get_person_info (driver_rec.chair_profile_id);

      descr	:= driver_rec.session_id || ' ' || driver_rec.title	|| new_line$ ||
			'Chair: ' || organizer || new_line$;

      if common.get_presentation_type2 (driver_rec.presentation_option_code) = common.oral$ then
         orals (driver_rec.session_id, trunc (driver_rec.session_date), loc_descr, descr);
      else
         bdate	:= new_date (driver_rec.session_date, driver_rec.beg_time);
         edate	:= new_date (driver_rec.session_date, driver_rec.end_time);
         create_event (driver_rec.session_id || ' ' || driver_rec.title,
			loc_descr, driver_rec.session_id, bdate, edate, organizer,
			posters (driver_rec.session_id) );

      end if;

   end loop;
   close_iCAL;

end ics;

-- ****************************************************************************************
-- ****************************************************************************************

procedure publish is

begin

   ics;

end publish;

-- ****************************************************************************************
-- ****************************************************************************************

end calendar;
/
show error

--	Copyright (C) 2004 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>
