set define off

create or replace package body class_maint as

rpt_title$		constant varchar2(80)	:= 'Classification Maintenance';
action_new_combo$	constant varchar2(30)	:= 'Save New Combination';

-- *********************************************************************************************
-- *********************************************************************************************

procedure import2 (conf_id integer) is

cursor	cdriver is
select	*
from	xml_classification_codes
where	conference_id = conf_id;

cursor	sdriver is
select	*
from	xml_sub_class_codes
where	conference_id = conf_id;

cursor	edriver is
select	*
from	xml_eclass_codes
where	conference_id = conf_id;

type	crecs_type is table of cdriver%ROWTYPE index by pls_integer;
type	srecs_type is table of sdriver%ROWTYPE index by pls_integer;
type	erecs_type is table of edriver%ROWTYPE index by pls_integer;

crecs	crecs_type;
srecs	srecs_type;
erecs	erecs_type;

begin

   if common.header (rpt_title$) then
      open cdriver;
      fetch cdriver BULK COLLECT into crecs;
      close cdriver;

      open sdriver;
      fetch sdriver BULK COLLECT into srecs;
      close sdriver;

      open edriver;
      fetch edriver BULK COLLECT into erecs;
      close edriver;

      for i in 1..crecs.COUNT loop
         insert into classification_codes values (crecs(i).classification_code, crecs(i).classification_descr);
      end loop;

      for i in 1..srecs.COUNT loop
         insert into sub_class_codes values (srecs(i).sub_class_code, srecs(i).sub_class_descr);
      end loop;

      for i in 1..erecs.COUNT loop
         insert into eclass_codes (classification_code, sub_class_code) values (erecs(i).classification_code, erecs(i).sub_class_code);
      end loop;

      htp.header (3, 'The Classification Data Imported Successfully');

      common.footer;
   end if;

end import2;

-- *********************************************************************************************

procedure show_import_form is

cursor	driver is
select	c.conference_id, c.conference_title
from	repo_conf_conferences	c
where	conference_id		in
(	select	DISTINCT conference_id
	from	xml_classification_codes)
order by to_char(c.start_date,'yyyy') desc, c.conference_title;

recs	common.popups_type;

begin

   open driver;
   fetch driver BULK COLLECT into recs;
   close driver;

   htp.header (3, 'Select the confernece you wish to use as the Classification Code source.');
   htp.formOpen ('class_maint.import2', 'POST');
   htp.tableOpen;

   common.show_popup_array_row ('Conference', 'conf_id', recs);
   common.show_data_row (null, htf.formSubmit);

   htp.tableClose;
   htp.formClose;
   htp.para;

end show_import_form;

-- *********************************************************************************************

function class_data_exists return boolean is

cursor	driver is
select	1 from classification_codes
UNION
select	1 from sub_class_codes
UNION
select	1 from eclass_codes;

rec_found	boolean;
dummy		integer;

begin

   open driver;
   fetch driver into dummy;
   rec_found := driver%FOUND;
   close driver;

   return rec_found;

end class_data_exists;

-- *********************************************************************************************

procedure import is

begin

   if common.header (rpt_title$) then
      if class_data_exists then
         htp.header (3, 'Unable to Import Classification Data', cattributes=>common.red$);
         htp.bold ('Classification Data Already Exists');
      else
         show_import_form;
      end if;

      common.footer;
   end if;

end import;

-- *********************************************************************************************
-- *********************************************************************************************

procedure redirect (fname integer) is

begin

   if fname = ccode$ then
      common.redirect_URL ('maint.tab?tab_name=classification_codes&order_by=classification_descr');
   elsif fname = scode$ then
      common.redirect_URL ('maint.tab?tab_name=sub_class_codes&order_by=sub_class_descr');
   else
      common.redirect_URL ('class_maint.html');
   end if;

end redirect;

-- *********************************************************************************************

procedure del_session (ID integer, fname integer) is

begin

   delete from class_sessions where class_session_id = ID;
   redirect (fname);

end del_session;

-- *********************************************************************************************

procedure save_session (fname integer, fvalue varchar2, this_session varchar2) is

cursor	driver is
select	nvl (max (sort_order), 0) + 10
from	class_sessions
where	classification_code	= decode (fname, ccode$,fvalue, null)
or	sub_class_code		= decode (fname, scode$,fvalue, null)
or	eclass_code		= decode (fname, ecode$,fvalue, null);

iorder	integer;

begin

   common.get_gbl_pid;
   if common.has_privs_to_page then
      open driver;
      fetch driver into iorder;
      close driver;

      insert into class_sessions (classification_code, sub_class_code, eclass_code,
				session_id, sort_order)
      values (	decode (fname, ccode$,fvalue, null),
		decode (fname, scode$,fvalue, null),
		decode (fname, ecode$,fvalue, null),
		this_session, iorder);

      redirect (fname);
   end if;

end save_session;

-- *********************************************************************************************

procedure add_session (fname integer, fvalue varchar2) is

buffer	varchar2(500);
Q$	constant varchar2(1)	:= chr(39);

begin

   if common.header (rpt_title$) then
      buffer := 'where presentation_option_code in '	||
		'(select presentation_option_code'	||
		' from presentation_option_codes'	||
		' where presentation_type_code = '	||
			Q$ || common.poster$ || Q$	|| ') ' ||
		'and session_code NOT in '	||
		'(select session_id'		||
		' from class_sessions'		||
		' where ~ = '			|| Q$ || fvalue || Q$ || ')';

      if fname = ccode$ then
         buffer := replace (buffer, '~', 'classification_code');
      elsif fname = scode$ then
         buffer := replace (buffer, '~', 'sub_class_code');
      elsif fname = ecode$ then
         buffer := replace (buffer, '~', 'eclass_code');
      end if;

      htp.formOpen ('class_maint.save_session', 'POST', cattributes=>'NAME="SPMS"');
      htp.formHidden ('fname',  fname);
      htp.formHidden ('fvalue', fvalue);

      htp.header (3, 'Select a New Session');
      common.show_popup_list ('session', where_claws=>buffer);
      htp.para;
      htp.formSubmit;
      htp.formClose;
      common.form_focus ('SPMS', 'this_session');
   end if;
   common.footer;

end add_session;

-- *********************************************************************************************

function session_list (fname integer, fvalue varchar2) return varchar2 is

cursor	driver is
select	cs.*
from	sessions	s,
	class_sessions	cs
where	fvalue		= decode (fname,ccode$, cs.classification_code,
					scode$, cs.sub_class_code,
					ecode$, cs.eclass_code)
and	s.session_id	= cs.session_id
order by
	cs.sort_order nulls last;

buffer	long := null;

begin

   for driver_rec in driver LOOP
      buffer := buffer || htf.hr ||
		htf.anchor ('class_maint.del_session'				||
				'?ID='	  || driver_rec.class_session_id	||
				'&fname=' || fname,
			'Remove') || htf.br ||
		common.show_session_summary (driver_rec.session_id);
   end loop;

   return htf.anchor ('class_maint.add_session'					||
				'?fname='	|| fname			||
				'&fvalue='	|| fvalue, 'New Session')	||
			htf.br || buffer;

end session_list;

-- *********************************************************************************************
-- *********************************************************************************************

procedure show_error (msg varchar2) is

begin

   common.public_header (rpt_title$);
   htp.header (3, msg);
   htp.p (common.click_back$);
   htp.para;
   common.footer;

end show_error;

-- *********************************************************************************************
-- *********************************************************************************************

procedure del (code number) is

cursor	driver is
select	1
from	abstracts
where	eclass_code = code;

dummy		integer;
rec_found	boolean;

begin

   open driver;
   fetch driver into dummy;
   rec_found := driver%FOUND;
   close driver;

   if rec_found then
      show_error ('Unable to Delete. This Classification/Sub-classification is in use on one or more abstacts.');
   else
      delete from class_sessions where eclass_code = code;
      delete from eclass_codes	 where eclass_code = code;
      common.redirect_URL ('class_maint.html');
   end if;

end del;

-- *********************************************************************************************
-- *********************************************************************************************

procedure save_ccode (this_code varchar2, descr varchar2) is

begin

   if trim (descr) is not null then
      update	classification_codes
      set	classification_descr	= trim (descr)
      where	classification_code	= this_code;
   end if;

   common.redirect_URL ('class_maint.html');

end save_ccode;

-- *********************************************************************************************
-- *********************************************************************************************

procedure save_scode (this_code varchar2, descr varchar2) is

begin

   if trim (descr) is not null then
      update	sub_class_codes
      set	sub_class_descr	= trim (descr)
      where	sub_class_code	= this_code;
   end if;

   common.redirect_URL ('class_maint.html');

end save_scode;

-- *********************************************************************************************
-- *********************************************************************************************

procedure action (this_classification varchar2, this_sub_class varchar2) is

cursor	driver is
select	1
from	eclass_codes
where	classification_code	= upper (trim (this_classification))
and	sub_class_code		= upper (trim (this_sub_class));

dummy		integer;
rec_found	boolean;

begin

   common.get_gbl_pid;

   if common.has_privs_to_page then
      open driver;
      fetch driver into dummy;
      rec_found := driver%FOUND;
      close driver;

      if rec_found then
         show_error ('This Classification/Sub-classification already exists.');

      elsif this_classification is null then
         show_error ('You must select a Classification');

      elsif this_sub_class is null then
         show_error ('You must select a Sub-classification');

      else
         insert into eclass_codes (eclass_code, classification_code, sub_class_code)
         values (eclass_code_seq.nextval, this_classification, this_sub_class);

         common.redirect_URL ('class_maint.html');
      end if;
   end if;

end action;

-- *********************************************************************************************
-- *********************************************************************************************

procedure new_codes (ccode varchar2, cdescr varchar2, scode varchar2, sdescr varchar2) is

cursor	c1 is select 1 from classification_codes where classification_code = upper (trim (ccode));
cursor	c2 is select 1 from sub_class_codes where sub_class_code = upper (trim (scode));

dummy		integer;
rec_found	boolean;
err_msg1	varchar2(100) := null;
err_msg2	varchar2(100) := null;

begin

   open c1;
   fetch c1 into dummy;
   rec_found := c1%FOUND;
   close c1;

   if rec_found then
      err_msg1 := 'A classification with this code already exists: ' || ccode;

   elsif trim (ccode) is NOT null OR trim (cdescr) is NOT null then

      if trim (ccode) is null then
         err_msg1 := 'You must enter a classification code';

      elsif trim (cdescr) is null then
         err_msg1 := 'You must enter the classification description';

      else
         insert into classification_codes (classification_code, classification_descr)
         values (upper(trim (ccode)), trim (cdescr));

      end if;
   end if;

-- **********

   open c2;
   fetch c2 into dummy;
   rec_found := c2%FOUND;
   close c2;

   if rec_found then
      err_msg2 := 'A sub-classification with this code already exists: ' || scode;

   elsif trim (scode) is NOT null OR trim (sdescr) is NOT null then

      if trim (scode) is null then
         err_msg2 := 'You must enter a sub-classification code';

      elsif trim (sdescr) is null then
         err_msg2 := 'You must enter the sub-classification description';

      else
         insert into sub_class_codes (sub_class_code, sub_class_descr)
         values (upper(trim (scode)), trim (sdescr));

      end if;
   end if;

-- **********

   if err_msg1 is null AND err_msg2 is null then
      common.redirect_URL ('class_maint.html');

   elsif err_msg1 is null then
      show_error (err_msg2);

   elsif err_msg2 is null then
      show_error (err_msg1);

   else
      show_error (err_msg1 || htf.br || err_msg2);

   end if;

end new_codes;

-- *********************************************************************************************
-- *********************************************************************************************

procedure html is

cursor	driver is
select	c.classification_code, c.classification_descr,
	s.sub_class_code, s.sub_class_descr,
	e.eclass_code, (select	count(1)
			from	abstracts	abs
			where	abs.eclass_code	= e.eclass_code) abs_count
from	classification_codes	c,
	sub_class_codes		s,
	eclass_codes		e
where	c.classification_code	= e.classification_code
and	s.sub_class_code	= e.sub_class_code
order by
	c.classification_code,
	s.sub_class_code;

prev_class_code		classification_codes.classification_code%TYPE := '~';

begin

   if common.header (rpt_title$) then
      htp.formOpen ('class_maint.action', 'POST');
      htp.tableOpen;

      htp.tableRowOpen ('LEFT', 'TOP');
      htp.tableHeader ('Classification', 'RIGHT');
      htp.p ('<TD>');
      common.show_popup_list ('classification');
      htp.p ('</TD>');
      htp.tableRowClose;

      htp.tableRowOpen ('LEFT', 'TOP');
      htp.tableHeader ('Sub-Classification', 'RIGHT');
      htp.p ('<TD>');
      common.show_popup_list ('sub_class');
      htp.p ('</TD>');
      htp.tableRowClose;

      common.show_data_row (common.space$, htf.formSubmit (null, action_new_combo$));
      htp.tableClose;
      htp.formClose;

-- **********

      htp.tableOpen ('BORDER');
      htp.tableCaption ('Note: Session assignment is for Posters only.  Assignment does not affect Orals.',
			cattributes=>common.red$);

      htp.tableRowOpen ('LEFT', 'BOTTOM');
      htp.tableHeader (common.space$);
      htp.tableHeader ('Classification');
      htp.tableHeader ('Sub Classification');
      htp.tableHeader ('Session');
      htp.tableRowClose;

      for driver_rec in driver LOOP
         htp.tableRowOpen ('LEFT', 'TOP');
         htp.tableData (htf.anchor ('class_maint.del?code=' || driver_rec.eclass_code, 'DELETE') ||
			htf.br || 'Abstracts: ' || driver_rec.abs_count, cnowrap=>'NOWRAP');

         if prev_class_code = driver_rec.classification_code then
            htp.tableData (common.space$);
         else
            htp.tableData (driver_rec.classification_descr);
         end if;

         htp.tableData (driver_rec.sub_class_descr);
         htp.tableData (session_list (ecode$, driver_rec.eclass_code), cnowrap=>'NOWRAP' );
         htp.tableRowClose;
         prev_class_code := driver_rec.classification_code;
      end loop;

      htp.tableClose;
      common.footer;
   end if;

end html;

-- *********************************************************************************************

end class_maint;
/
show error

set define on

--	Copyright (C) 2004 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>
