set define off

create or replace package body conf_list as

-- *************************************************************************

-- *************************************************************************

procedure html is

cursor	driver is
select	c.*, cc.conference_descr
from	conference_codes	cc,
	conf_conferences	c
where	c.start_date		<> '01-jan-00'
and	c.conference_code	= cc.conference_code
order by to_char (c.start_date, 'YYYY') desc, c.start_date desc;

now$		constant date		:= trunc(sysdate);
mdash$		constant char(7)	:= '&mdash;';
ndash$		constant char(7)	:= '&ndash;';
prev_year	integer := 0;
curr_year	integer;
first_past	boolean	:= true;

conf_now$	constant char(32)	:= 'STYLE="BACKGROUND-COLOR:#FF8282"';
conf_soon$	constant char(31)	:= 'STYLE="BACKGROUND-COLOR:YELLOW"';
ID_warning$	constant char(18)	:= 'STYLE="COLOR:BLUE"';
style		varchar2(32);

type	rec_type is table of driver%ROWTYPE index by binary_integer;
recs	rec_type;
buffer	varchar2(100);
max_Id	integer;

begin

if common.header ('', body_open=>false) then
   select max(last_sync_id) into max_ID from conf_conferences;

   htp.bodyOpen;
   htp.header (3, 'SPMS Conference List (' || max_ID || ')');
   htp.tableOpen (cattributes=>'CELLPADDING="3"');

   open driver;
   fetch driver BULK COLLECT into recs;
   close driver;

   for i in 1..recs.count LOOP
      curr_year	:= to_number (to_char (recs(i).start_date, 'YYYY'));

      if recs(i).start_date < now$ AND first_past then
         common.show_data_sep (7, htf.hr);
         first_past := false;
      elsif prev_year <> curr_year then
         common.show_data_sep (7, common.space$);
      end if;
      prev_year := curr_year;

      style := null;
      if now$ between recs(i).start_date and recs(i).end_date then
         style := conf_now$;
      elsif now$ between recs(i).start_date-7 and recs(i).start_date then
         style := conf_soon$;
      end if;

      htp.tableRowOpen ('LEFT', 'TOP', cattributes=>style);
      htp.tableData (recs.COUNT - i + 1 || ')');
      htp.tableData (common.space2$);
      htp.tableData (recs(i).conference_code || ' ' || to_char (recs(i).start_date, 'yyyy'), cnowrap=>'NOWRAP');

      style	:= null;
      buffer	:= null;
      if recs(i).repatriated is not null then
         buffer := '&#10004;'; -- check mark
      end if;
      if recs(i).last_sync_id is not null then
         buffer := '(' || trim (recs(i).last_sync_id) || ')' || common.space$ || buffer;

         if recs(i).last_sync_id <> max_ID then
            style := ID_warning$;
         end if;
      end if;
      htp.tableData (buffer, 'CENTER', cattributes=>style);

      htp.tableData (recs(i).location, cnowrap=>'NOWRAP');

      buffer := to_char (recs(i).start_date, 'Mon') || ' ';
      buffer := buffer ||
			to_char (recs(i).start_date, 'dd');

      if to_char (recs(i).start_date, 'mm') = to_char (recs(i).end_date, 'mm') then
         buffer := buffer || ndash$ || to_char (recs(i).end_date, 'dd');
      else
         buffer := buffer || mdash$ ||
			to_char (recs(i).end_date, 'Mon')	|| ' ' ||
			to_char (recs(i).end_date, 'dd');
      end if;
      htp.tableData (buffer, cnowrap=>'NOWRAP');

      htp.tableData (recs(i).conference_descr);
      htp.tableRowClose;
   end loop;

   htp.tableClose;
   common.footer;
end if;

end html;

-- *************************************************************************

end conf_list;
/
show error

--	Copyright (C) 2004 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>

set define on
