--	Copyright (C) 2004 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>

-- ****************************************************************************************

spool Create_FKeys.log

prompt Creating Foreign Keys

alter table persons
	add constraint persons_country_code_fk
	foreign key (country_code)
	references country_codes (country_code);

alter table persons
	add constraint persons_date_format_fk
	foreign key (date_format_code)
	references date_format_codes (date_format_code);

alter table persons
	add constraint persons_tz_fk
	foreign key (tz_name)
	references time_zone_codes (time_zone_code);

-- *****

alter table profiles
	add constraint profiles_person_fk
	foreign key (person_id)
	references persons (person_id);

alter table profiles
	add constraint profiles_address_fk
	foreign key (address_id)
	references addresses (address_id);

-- *****

alter table addresses
	add constraint addresses_country_code_fk
	foreign key (country_code)
	references country_codes (country_code);

alter table addresses
	add constraint addresses_affiliation_fk
	foreign key (affiliation_id)
	references affiliations (affiliation_id);

-- *****

alter table affiliations
	add constraint affil_affil_type_codes_fk
	foreign key (affil_type_code)
	references affil_type_codes (affil_type_code);

-- *****

alter table authors
	add constraint authors_abstracts_fk
	foreign key (abstract_id)
	references abstracts (abstract_id);

alter table authors
	add constraint authors_profiles_fk
	foreign key (profile_id)
	references profiles (profile_id)
	deferrable;

-- *****

alter table abstracts
	add constraint abstracts_persons_fk
	foreign key (person_id)
	references persons (person_id);

alter table abstracts
	add constraint abstracts_editor_fk
	foreign key (editor_id)
	references persons (person_id);

alter table abstracts
	add constraint abstracts_status_code_fk
	foreign key (status_code)
	references status_codes (status_code);

alter table abstracts
	add constraint abstracts_referee_code_fk
	foreign key (referee_code)
	references referee_codes (referee_code);

alter table abstracts
	add constraint abstracts_poster_code_fk
	foreign key (poster_status)
	references status_codes (status_code);

alter table abstracts
	add constraint abstracts_presentation_code_fk
	foreign key (presentation_code)
	references presentation_codes (presentation_code);

alter table abstracts
	add constraint abstracts_class_code_fk
	foreign key (classification_code)
	references classification_codes (classification_code);

alter table abstracts
	add constraint abstracts_subclass_code_fk
	foreign key (sub_class_code)
	references sub_class_codes (sub_class_code);

alter table abstracts
	add constraint abstracts_eclass_code_fk
	foreign key (eclass_code)
	references eclass_codes (eclass_code);

-- *****

alter table files
	add constraint files_abstracts_fk
	foreign key (abstract_id)
	references abstracts (abstract_id);

alter table files
	add constraint files_file_type_codes_fk
	foreign key (file_type_code)
	references file_type_codes (file_type_code);

alter table files
	add constraint files_platform_codes_fk
	foreign key (platform_code)
	references platform_codes (platform_code);

-- *****

alter table abstract_keywords
	add constraint abstract_keyword_abstracts_fk
	foreign key (abstract_id)
	references abstracts (abstract_id);

alter table abstract_keywords
	add constraint abstract_keyword_keywords_fk
	foreign key (keyword_code) 
	references keyword_codes (keyword_code);

-- *****

alter table presentation_codes
	add constraint presentation_code_pres_type_fk
	foreign key (presentation_type_code)
	references presentation_type_codes (presentation_type_code);

-- *****

alter table presentation_option_codes
	add constraint presentation_opt_pres_type_fk
	foreign key (presentation_type_code)
	references presentation_type_codes (presentation_type_code);

-- *****

alter table sessions
	add constraint sessions_chair_fk
	foreign key (chair_profile_id)
	references profiles (profile_id)
	deferrable;

alter table sessions
	add constraint session_presentation_opt_fk
	foreign key (presentation_option_code)
	references presentation_option_codes (presentation_option_code);

alter table sessions
	add constraint sessions_location_codes_fk
	foreign key (location_code)
	references location_codes (location_code);

-- *****

alter table logs
	add constraint logs_abstract_id_fk
	foreign key (abstract_id)
	references abstracts (abstract_id);

alter table logs
	add constraint logs_person_id_fk
	foreign key (person_id)
	references persons (person_id);

alter table logs
	add constraint logs_code_fk
	foreign key (log_code)
	references log_codes (log_code);

alter table logs
	add constraint logs_status_code_fk
	foreign key (status_code)
	references status_codes (status_code);

alter table logs
	add constraint logs_referee_code_FK
	foreign key (referee_code)
	references referee_codes (referee_code);

-- *****

alter table parameters
	add constraint parameters_categories_fk
	foreign key (category_code)
	references param_categories (category_code);

-- *****

alter table person_memberships
	add constraint person_memberships_person_fk
	foreign key (person_id)
	references persons (person_id);

alter table person_memberships
	add constraint person_memberships_mbr_code_fk
	foreign key (membership_code)
	references membership_codes (membership_code);

-- *****

alter table JN_log_errors
	add constraint JN_log_errors_error_code_fk
	foreign key (error_code)
	references error_codes (error_code);

-- *****

alter table log_errors
	add constraint log_errors_logs_fk
	foreign key (log_id)
	references logs (log_id);

alter table log_errors
	add constraint log_errors_error_code_fk
	foreign key (error_code)
	references error_codes (error_code);

-- *****

alter table favorite_authors
	add constraint favorite_authors_persons_fk
	foreign key (person_id)
	references persons (person_id);

alter table favorite_authors
	add constraint favorite_authors_profiles_fk
	foreign key (profile_id)
	references profiles (profile_id)
	deferrable;

alter table control
	add constraint control_conf_code_fk
	foreign key (conference_code)
	references conference_codes (conference_code);

-- *****

alter table mailing_lists
	add constraint mailing_lists_person_fk
	foreign key (person_id)
	references persons (person_id);

alter table mailing_lists
	add constraint mailing_lists_conf_code_fk
	foreign key (conference_code)
	references conference_codes (conference_code);

-- *****

alter table activities
	add constraint activities_person_fk
	foreign key (person_id)
	references persons (person_id);


alter table activities
	add constraint activities_updated_fk
	foreign key (updated_by)
	references persons (person_id);

alter table activities
	add constraint activities_activty_fk
	foreign key (activity_code)
	references activity_codes (activity_code);

-- *****

alter table eclass_codes
	add constraint eclass_codes_class_fk
	foreign key (classification_code)
	references classification_codes (classification_code);

alter table eclass_codes
	add constraint eclass_codes_subclass_fk
	foreign key (sub_class_code)
	references sub_class_codes (sub_class_code);

-- *****

alter table affil_requests
	add constraint affil_requests_requestor_fk
	foreign key (requestor_id)
	references persons (person_id);

alter table affil_requests
	add constraint affil_requests_address_fk
	foreign key (address_id)
	references addresses (address_id);

-- *****

alter table country_codes
	add constraint country_codes_region_codes_fk
	foreign key (region_code)
	references region_codes (region_code);

-- *****

alter table merge_logs
	add constraint merge_logs_persons_fk
	foreign key (merged_by)
	references persons (person_id);

-- *****

alter table keyword_codes
	add constraint keyword_codes_conf_fk
	foreign key (conference_code)
	references conference_codes (conference_code);

-- *****

alter table poster_logs
	add constraint poster_logs_abstracts_fk
	foreign key (abstract_id)
	references abstracts (abstract_id);

-- *****

alter table JN_abstracts
	add constraint JN_abstracts_persons_fk
	foreign key (person_id)
	references persons (person_id);

alter table JN_abstracts
	add constraint JN_abstracts_editor_fk
	foreign key (editor_id)
	references persons (person_id);

alter table JN_abstracts
	add constraint JN_abstracts_class_code_fk
	foreign key (classification_code)
	references classification_codes (classification_code);

alter table JN_abstracts
	add constraint JN_abstracts_status_code_fk
	foreign key (status_code)
	references status_codes (status_code);

alter table JN_abstracts
	add constraint JN_abstracts_poster_code_fk
	foreign key (poster_status)
	references status_codes (status_code);

alter table JN_abstracts
	add constraint JN_abstracts_referee_code_fk
	foreign key (referee_code)
	references referee_codes (referee_code);

alter table JN_abstracts
	add constraint JN_abstracts_eclass_code_fk
	foreign key (eclass_code)
	references eclass_codes (eclass_code);

alter table JN_abstracts
	add constraint JN_abstracts_withdrawn_by_fk
	foreign key (withdrawn_by)
	references persons (person_id);

-- *****

alter table JN_authors
	add constraint JN_authors_JN_abs_fk
	foreign key (abstract_id)
	references JN_abstracts (abstract_id);

alter table JN_authors
	add constraint JN_authors_profiles_fk
	foreign key (profile_id)
	references profiles (profile_id);

-- **********

alter table email_logs
	add constraint email_logs_person_id_fk
	foreign key (person_id)
	references persons (person_id);

alter table email_logs
	add constraint email_logs_email_id_fk
	foreign key (email_id)
	references emails (email_id);

-- **********

alter table mail_blats
	add constraint mail_blats_person_fk
	foreign key (person_id)
	references persons (person_id);

alter table mail_blats
	add constraint mail_blats_abstract_fk
	foreign key (abstract_id)
	references abstracts (abstract_id);

-- **********

alter table mail_sent_blats
	add constraint mail_blats_sent_person_fk
	foreign key (person_id)
	references persons (person_id);

alter table mail_sent_blats
	add constraint mail_blats_sent_abstract_fk
	foreign key (abstract_id)
	references abstracts (abstract_id);

-- **********

alter table mail_blats_errors
	add constraint mail_blats_errors_person_fk
	foreign key (person_id)
	references persons (person_id);

alter table mail_blats_errors
	add constraint mail_blats_errors_abstract_fk
	foreign key (abstract_id)
	references abstracts (abstract_id);

-- *********

alter table page_data
	add constraint page_data_page_fk
	foreign key (page_id)
	references pages (page_id);

alter table page_data
	add constraint page_data_meta_fk
	foreign key (meta_data_code)
	references meta_data_codes (meta_data_code);

-- *********

alter table registrations
	add constraint registrations_persons_fk
	foreign key (person_id)
	references persons (person_id);

alter table registrations
	add constraint registrations_meta_fk
	foreign key (meta_data_code)
	references meta_data_codes (meta_data_code);

alter table registrations
	add constraint registrations_pages_fk
	foreign key (page_id)
	references pages (page_id);

-- **********

alter table registrations
	add constraint registrations_payment_id
	foreign key (payment_id)
	references payments (payment_id);

-- *********

alter table meta_data_codes
	add constraint meta_data_type_fk
	foreign key (data_type_code)
        references data_type_codes (data_type_code);

alter table meta_data_codes
	add constraint meta_data_display_fk
	foreign key (display_code)
        references display_codes (display_code);

-- *********

alter table poplist_items
	add constraint poplist_items_meta_data_fk
	foreign key (meta_data_code)
	references meta_data_codes (meta_data_code);

-- *********

alter table registrations
	add constraint registrations_poplist_item_fk
	foreign key (poplist_item_id)
	references poplist_items (poplist_item_id);

-- **********

alter table menu_items
	add constraint menu_items_fk
	foreign key (parent_item_id)
	references menu_items (item_id)
	deferrable;

-- **********

alter table abstract_presentations
	add constraint abstract_presentations_fk1
	foreign key (abstract_id)
	references abstracts (abstract_id);

alter table abstract_presentations
	add constraint abstract_presentations_fk2
	foreign key (presentation_code)
	references presentation_codes (presentation_code);

-- **********

alter table abstract_classifications
	add constraint abstract_classifications_fk
	foreign key (abstract_id)
	references abstracts (abstract_id);

alter table abstract_classifications
	add constraint abstract_class_code_fk
	foreign key (classification_code)
	references classification_codes (classification_code);

alter table abstract_classifications
	add constraint abs_class_subclass_code_fk
	foreign key (sub_class_code)
	references sub_class_codes (sub_class_code);

alter table abstract_classifications
	add constraint abstract_class_pid_fk
	foreign key (person_id)
	references persons (person_id);

-- **********

alter table fee_matrix
	add constraint fee_matrix_meta_data1
	foreign key (poplist_item_id1)
	references poplist_items (poplist_item_id);

alter table fee_matrix
	add constraint fee_matrix_meta_data2
	foreign key (poplist_item_id2)
	references poplist_items (poplist_item_id);

-- **********

alter table conditional_requirements
	add constraint conditional_requirements_fk1
	foreign key (yesno_meta_code)
	references meta_data_codes (meta_data_code);

alter table conditional_requirements
	add constraint conditional_requirements_fk2
	foreign key (meta_data_code)
	references meta_data_codes (meta_data_code);

-- **********

alter table payments
	add constraint payments_person_fk
	foreign key (person_id)
	references persons (person_id);

alter table payments
	add constraint payments_credit_card_id_fk
	foreign key (credit_card_id)
	references credit_cards (credit_card_id);

-- **********

alter table registrations
	add constraint registrations_fee_matrix_fk
	foreign key (fee_matrix_id)
	references fee_matrix (fee_matrix_id);

-- **********

alter table registrants
	add constraint registrants_page_id_fk
	foreign key (page_id)
	references pages (page_id);

alter table registrants
	add constraint registrants_person_id_fk
	foreign key (person_id)
	references persons (person_id);

-- **********

alter table credit_cards
	add constraint credit_cards_payment_fk
	foreign key (payment_code)
	references payment_codes (payment_code);

alter table credit_cards
	add constraint credit_cards_country_fk
	foreign key (country_code)
	references country_codes (country_code);

-- **********

alter table payments
	add constraint payments_payment_code_fk
	foreign key (payment_code)
	references payment_codes (payment_code);

-- **********

alter table person_schedules
	add constraint person_scedules_FK1
	foreign key (person_id)
	references persons (person_id);

alter table person_schedules
	add constraint person_scedules_FK2
	foreign key (abstract_id)
	references abstracts (abstract_id);

-- **********

alter table experts
	add constraint experts_person_FK
	foreign key (person_id)
	references persons (person_id);

alter table experts
	add constraint experts_class_FK
	foreign key (classification_code)
	references classification_codes (classification_code);

-- **********

alter table referee_abstracts
	add constraint referee_abstracts_person_fk
	foreign key (person_id)
	references persons (person_id);

alter table referee_abstracts
	add constraint referee_abstracts_abstract_fk
	foreign key (abstract_id)
	references abstracts (abstract_id);

-- **********

alter table checkbox_items
	add constraint checkbox_items_fk1
	foreign key (poplist_item_id)
	references poplist_items (poplist_item_id);

alter table checkbox_items
	add constraint checkbox_items_persons
	foreign key (person_id)
	references persons (person_id);

alter table checkbox_items
	add constraint checkbox_items_payment_id
	foreign key (payment_id)
	references payments (payment_id);

-- **********

alter table meta_data_codes
	add constraint meta_data_reg_cat_code_fk
	foreign key (reg_cat_code)
	references reg_cat_codes (reg_cat_code);

-- **********

alter table vendor_delegates
	add constraint vendor_delegates_person_FK
	foreign key (person_id)
	references persons (person_id);

alter table vendor_delegates
	add constraint vendor_delegates_vendor_FK
	foreign key (vendor_id)
	references persons (person_id);

alter table vendor_delegates
	add constraint vendor_delegates_funct_fk
	foreign key (function_code)
	references function_codes (function_code);

-- **********

alter table presentation_file_codes
	add constraint present_file_codes_present_FK
	foreign key (presentation_code)
	references presentation_codes (presentation_code);

alter table presentation_file_codes
	add constraint present_file_codes_file_FK
	foreign key (file_type_code)
	references file_type_codes (file_type_code);

-- **********

alter table abs_attributes
	add constraint abs_attributes_abs_id_FK
	foreign key (abstract_id)
	references abstracts (abstract_id);

alter table abs_attributes
	add constraint abs_attributes_attrib_code_FK
	foreign key (attribute_code)
	references attribute_codes (attribute_code);

-- **********

alter table class_sessions
	add constraint class_session_ccode_FK
	foreign key (classification_code)
	references classification_codes (classification_code);

alter table class_sessions
	add constraint class_session_scode_FK
	foreign key (sub_class_code)
	references sub_class_codes (sub_class_code);

alter table class_sessions
	add constraint class_session_ecode_FK
	foreign key (eclass_code)
	references eclass_codes (eclass_code);

alter table class_sessions
	add constraint class_session_session_id_FK
	foreign key (session_id)
	references sessions (session_id);

-- **********

alter table person_attributes
	add constraint person_attributes_person_PK
	foreign key (person_id)
	references persons (person_id);

alter table person_attributes
	add constraint person_attributes_attribute_PK
	foreign key (attribute_code)
	references attribute_codes (attribute_code);

-- **********

alter table payment_vendor_vars
	add constraint payment_vendor_vars_fk
	foreign key (payment_vendor_code)
	references payment_vendor_codes (payment_vendor_code);

alter table payment_vendor_vars
	add constraint payment_vendor_SPMS_fk
	foreign key (SPMS_var_code)
	references SPMS_var_codes (SPMS_var_code);

alter table payment_vendor_vars
	add constraint payment_vendor_var_type_fk
	foreign key (var_type_code)
	references var_type_codes (var_type_code);

-- **********

alter table payment_vendor_IPNs
	add constraint payment_vendor_IPNs_Pay_ID_FK
	foreign key (payment_id)
	references payments (payment_id);

-- **********

alter table payment_vendor_vars
	add constraint payment_vendor_vars_check
	check ( (SPMS_var_code is null AND var_text is NOT null AND var_type_code='H') OR
		(SPMS_var_code is NOT null AND var_text is null AND var_type_code='H') OR
		NOT (SPMS_var_code is null AND var_type_code = 'R') );

-- **********

alter table function_privileges
	add constraint function_privileges_fk1
	foreign key (function_code)
	references function_codes (function_code);

alter table function_privileges
	add constraint function_privileges_fk2
	foreign key (privilege_code)
	references privilege_codes (privilege_code);

-- **********

alter table user_functions
	add constraint user_functions_pid_FK
	foreign key (person_id)
	references persons (person_id);

alter table user_functions
	add constraint user_functions_fcode_FK
	foreign key (function_code)
	references function_codes (function_code);

-- **********

alter table person_privs
	add constraint person_privs_person_fk
	foreign key (person_id)
	references persons (person_id);

alter table person_privs
	add constraint person_privs_priv_fk
	foreign key (privilege_code)
	references privilege_codes (privilege_code);

-- **********

alter table author_codes
	add constraint author_codes_types_FK
	foreign key (author_type)
	references author_types (author_type);

alter table authors
	add constraint authors_author_code_fk
	foreign key (author_code)
	references author_codes (author_code);

alter table authors
	add constraint authors_author_type_fk
	foreign key (author_type)
	references author_types (author_type);

alter table jn_authors
	add constraint jn_authors_author_code_fk
	foreign key (author_code)
	references author_codes (author_code);

alter table jn_authors
	add constraint jn_authors_author_type_fk
	foreign key (author_type)
	references author_types (author_type);

-- **********

alter table hotel_payments
	add CONSTRAINT "HOTEL_PAYMENTS_PAYMENT_CO_FK1"
	FOREIGN KEY ("PAYMENT_CODE")
	REFERENCES "PAYMENT_CODES" ("PAYMENT_CODE") 

alter table hotel_roomtype_fee
	add CONSTRAINT "HOTEL_ROOMTYPE_FEE_HOTELS_FK1"
	FOREIGN KEY ("HOTEL_ID")
	REFERENCES "HOTELS" ("HOTEL_ID");

alter table hotel_roomtype_fee
	add CONSTRAINT "HOTEL_ROOMTYPE_FEE_ROOM_T_FK1"
	FOREIGN KEY ("ROOMTYPE_ID")
	REFERENCES "ROOM_TYPES" ("ROOM_TYPE_ID");

alter table hotels
	add CONSTRAINT "HOTELS_HOTEL_CATEGORIES_FK1"
	FOREIGN KEY ("HOTEL_CAT_ID")
	REFERENCES "HOTEL_CATEGORIES" ("HOTEL_CAT_ID");

alter table room_deposit
	add CONSTRAINT "ROOM_DEPOSIT_HOTEL_CATEGO_FK1" FOREIGN KEY ("HOTEL_CAT_ID")
	REFERENCES "HOTEL_CATEGORIES" ("HOTEL_CAT_ID");

alter table room_deposit
	add CONSTRAINT "ROOM_DEPOSIT_ROOM_TYPES_FK1" FOREIGN KEY ("ROOM_TYPE_ID")
	REFERENCES "ROOM_TYPES" ("ROOM_TYPE_ID");

alter table rooms_hotel
	add CONSTRAINT "ROOMS_HOTEL_HOTELS_FK1" FOREIGN KEY ("HOTEL_ID")
	REFERENCES "HOTELS" ("HOTEL_ID");

-- **********

alter table regist_bypass_vars
	add constraint regist_bypass_vars_bcode_fk
	foreign key (bypass_var_code)
	references bypass_var_codes (bypass_var_code);

alter table regist_bypass_vars
	add constraint regist_bypass_vars_page_fk
	foreign key (page_id)
	references pages (page_id);

alter table regist_bypass_vars
	add constraint regist_bypass_vars_check
	check (
		(bypass_var_code is null AND value is NOT null)
		or
		(bypass_var_code is NOT null AND value is null)
	      );

-- **********

alter table registrations_payments
	add constraint registrations_payments_reg_FK
	FOREIGN KEY (registration_id)
	REFERENCES registrations (registration_id);

alter table registrations_payments
	add constraint registrations_payments_pay__fk
	FOREIGN KEY (payment_id)
	REFERENCES payments (payment_id);

-- **********

ALTER TABLE TYPE_DISPLAY_DATA
      ADD CONSTRAINT tdisplay_data_type_code_fk
      FOREIGN KEY (data_type_code)
      REFERENCES data_type_codes (data_type_code);

ALTER TABLE TYPE_DISPLAY_DATA
      ADD CONSTRAINT tdisplay_data_display_code_fk
      FOREIGN KEY (display_code)
      REFERENCES display_codes (display_code);

-- **********

alter table jn_logs
	add constraint jn_logs_abs_fk
	foreign key (abstract_id)
	references jn_abstracts (abstract_id);

-- **********

alter table application_privs
	add constraint application_privs_FK1
	foreign key (application_id)
	references applications (application_id);

alter table application_privs
	add constraint application_privs_FK2
	foreign key (privilege_code)
	references privilege_codes (privilege_code);

-- **********

alter table upload_status_codes
	add constraint upload_status_codes_fk1
	foreign key (editor_status_code)
	references editor_status_codes (editor_status_code);

alter table upload_status_codes
	add constraint upload_status_codes_fk2
	foreign key (referee_status_code)
	references referee_status_codes (referee_status_code);

-- **********

alter table abstracts
	add constraint abstracts_transparency_FK
	foreign key (transparency_code)
	references transparency_codes (transparency_code);

alter table abstracts
	add constraint abstracts_trans_editor_FK
	foreign key (trans_editor_id)
	references persons (person_id);

alter table jn_abstracts
	add constraint jn_abstracts_transparency_FK
	foreign key (transparency_code)
	references transparency_codes (transparency_code);

alter table jn_abstracts
	add constraint jn_abstracts_trans_editor_FK
	foreign key (trans_editor_id)
	references persons (person_id);

-- **********

alter table status_codes
	add constraint status_codes_user_code_FK
	foreign key (user_validate_status)
	references status_codes (status_code);

-- **********

alter table log_codes
	add constraint log_codes_status_code_fk
	foreign key (status_code)
	references status_codes (status_code);

-- **********

alter table abstract_proposals
	add constraint abstract_proposals_abs_id_FK
	foreign key (abstract_id)
	references abstracts (abstract_id);

alter table abstract_proposals
	add constraint abstract_proposals_priority_FK
	foreign key (priority_code)
	references priority_codes (priority_code);

alter table abstract_proposals
	add constraint abstract_proposals_status_FK
	foreign key (proposal_status_code)
	references proposal_status_codes (proposal_status_code);

-- **********

alter table abstract_sessions
	add constraint abstract_sessions_abstract_FK
	foreign key (abstract_id)
	references abstracts (abstract_id);

alter table abstract_sessions
	add constraint abstract_sessions_present_FK
	foreign key (presentation_code)
	references presentation_codes (presentation_code);

alter table abstract_sessions
	add constraint abstract_sessions_session_FK
	foreign key (session_id)
	references sessions (session_id);

alter table region_codes
	add constraint region_codes_super_region_FK
	foreign key (super_region_code)
	references super_region_codes (super_region_code);

-- **********

-- ** Sync Tables -- Not used in conference instances
/*

alter table sync_persons
	add constraint sync_persons_FK
	foreign key (sync_id)
	references sync_data (sync_id);

alter table sync_profiles
	add constraint sync_profiles_FK
	foreign key (sync_id)
	references sync_data (sync_id);

alter table sync_addresses
	add constraint sync_addresses_FK
	foreign key (sync_id)
	references sync_data (sync_id);

alter table sync_affiliations
	add constraint sync_affiliations_FK
	foreign key (sync_id)
	references sync_data (sync_id);

alter table sync_person_memberships
	add constraint sync_person_memberships_FK
	foreign key (sync_id)
	references sync_data (sync_id);

alter table sync_mailing_lists
	add constraint sync_mailing_lists_FK
	foreign key (sync_id)
	references sync_data (sync_id);

alter table sync_merge_persons
	add constraint sync_merge_persons_FK
	foreign key (sync_id)
	references sync_data (sync_id);

alter table sync_merge_profiles
	add constraint sync_merge_profiles_FK
	foreign key (sync_id)
	references sync_data (sync_id);
*/
spool off
