cl scr

drop table menu_roles;
drop table menu_items;

select tablespace_name from user_tablespaces;

accept pac_data_small  prompt 'Tablespace for Small Tables:  '
accept PAC_Index_Small prompt 'Tablespace for Small Indexes: '

create table menu_items (
item_id		number,
URL		varchar2(100),
link_name	varchar2(50)	not null,
repo_only	varchar2(1),
parent_item_id	number)
tablespace &pac_data_small;

create table menu_roles (
item_id		number		not null,
role_code	varchar2(4)	not null)
tablespace &pac_data_small;

-- **********

create unique index menu_roles_uq
	on menu_roles (item_id, role_code)
	tablespace &pac_index_small;

-- **********

alter table menu_items
	add constraint menu_items_pk
	primary key (item_id)
	using index
	tablespace &pac_index_small;

alter table menu_items
	add constraint menu_items_fk
	foreign key (parent_item_id)
	references menu_items (item_id);

-- **********

alter table menu_roles
	add constraint menu_roles_item_id_fk
	foreign key (item_id)
	references menu_items (item_id);

alter table menu_roles
	add constraint menu_roles_role_fk
	foreign key (role_code)
	references role_codes (role_code);

-- **********

drop sequence item_id_seq;
create sequence item_id_seq;


insert into menu_items values (0, null, 'Root',	'N', null);
insert into menu_items values (item_id_seq.nextval, null, 'Maintenance Tables',	'N', 0);
insert into menu_items values (item_id_seq.nextval, null, 'Editor Links',	'N', 0);
insert into menu_items values (item_id_seq.nextval, 'reports.html', 'Reports',	'N', 0);
insert into menu_items values (item_id_seq.nextval, null, 'Contribution Counts','N', 3);
insert into menu_items values (item_id_seq.nextval, null, 'Profiles',		'N', 1);
insert into menu_items values (item_id_seq.nextval, null, 'Classifications',	'N', 1);
insert into menu_items values (item_id_seq.nextval, null, 'Invitees',		'N', 0);

insert into menu_items
select	item_id_seq.nextval, script_name, link_name, repository_only, 1
from	maintenance_links
where (	link_name like '%Maintenance%'
or	link_name like '%Code%'
or	link_name like '%Type%'
or	link_name in ('Time Zones')
)
and	link_name NOT like 'Profile Maintenance%'
and	link_name NOT like 'Classification%'
and	link_name not in ('Reports', 'Abstract Reclassification');

insert into menu_items
select	item_id_seq.nextval, script_name, link_name, repository_only, 5
from	maintenance_links
where	link_name NOT in (select link_name from menu_items)
and	link_name like 'Profile Maintenance%';

insert into menu_items
select	item_id_seq.nextval, script_name, link_name, repository_only, 6
from	maintenance_links
where	link_name NOT in (select link_name from menu_items)
and	link_name like 'Classification%';

-- **********

insert into menu_items
select	item_id_seq.nextval, script_name, link_name, repository_only, 0
from	maintenance_links
where	link_name NOT in (select link_name from menu_items)
and	link_name NOT like 'Classification Maintenance%'
and	link_name NOT like 'Profile Maintenance%'
and	link_name <> 'Reports';

-- **********

insert into menu_items
select	item_id_seq.nextval, script_name, report_name, 'N', 3
from	jacow_reports
where	report_name NOT like 'Contributions%';

insert into menu_items
select	item_id_seq.nextval, script_name, report_name, 'N', 4
from	jacow_reports
where	report_name like 'Contributions%';

-- **********

insert into menu_items
select	item_id_seq.nextval, script_name, link_name, 'N', 2
from	editor_links
where	link_name not in ('Reports and Statistics', 'Data Extracts', 'Maintenance Tables', 'Validate Paper ID Sequences');

-- **********
-- **********

update	menu_items
set	link_name = decode (link_name,
			'Profile Maintenance',				'Search/Edit',
			'Profile Maintenance (Alphabetical List)',	'Alphabetical List',
			'Profile Maintenance (Create New Profile)',	'Create New Profile',
			link_name);

update	menu_items
set	link_name = replace (link_name, 'Classification Maintenance ', null);

update	menu_items
set	link_name = replace (link_name, 'Style)', 'Style');

update	menu_items
set	link_name = replace (link_name, ' Maintenance', null)
where	parent_item_id is not null;

update	menu_items
set	link_name = replace (link_name, 'Contributions by ', null);

update	menu_items
set	link_name = replace (link_name, 'Code', 'Codes');

update	menu_items
set	link_name = replace (link_name, 'Codess', 'Codes');

update	menu_items
set	link_name = replace (link_name, '(EPAC', 'EPAC');

update	menu_items
set	link_name = replace (link_name, '(PAC', 'PAC');

update	menu_items
set	link_name = 'Affiliaitons'
where	link_name = 'Affiliaiton';

update	menu_items
set	link_name	= 'Invitees'
where	link_name	= 'Specify Conference Invitees';

-- **********

insert	into menu_roles
select	item_id, 'ADMN'
from	menu_items
where	parent_item_id = 1;

insert	into menu_roles
select	item_id, 'ADMN'
from	menu_items
where	link_name	in (select link_name from maintenance_links)
and	item_id		not in (select item_id from menu_roles where role_code = 'ADMN');

insert	into menu_roles
select	item_id, 'PP'
from	menu_items
where	link_name	= 'Poster Police';

insert	into menu_roles
select	item_id, 'RECL'
from	menu_items
where	link_name	= 'Abstract Reclassification';

insert	into menu_roles
select	item_id, 'EDIT'
from	menu_items
where	parent_item_id = 2;

delete from menu_roles
where	item_id in
(	select	item_id
	from	menu_items
	where	link_name in ('My Contributions', 'My Profile', 'Abstract Search/Edit', 'Reports')
);

delete from menu_roles where item_id =
(	select	item_id
	from	menu_items
	where	link_name = 'Generate Paper TOC Values');

insert	into menu_roles
select	item_id, 'ADMN'
from	menu_items
where	link_name = 'Generate Paper TOC Values';

-- **********

select	lpad (' ', 2*(level-1)) ||
	link_name link_name
from	menu_items
start with item_id = 0
connect by prior item_id = parent_item_id;
