-- ********** Post Conference Tables (SYNC back to main repository)

spool Create_Repatriate.log

create sequence conference_id_seq;
create sequence keyword_id_seq;
create sequence presentation_id_seq;
create sequence classification_id_seq;
create sequence sub_class_id_seq;
create sequence eclass_id_seq;
create sequence function_id_seq;

create table conf_conferences (
conference_id		number,
conference_code		varchar2(10)	not null,
conference_title	varchar2(100)	not null,
start_date		date		not null,
end_date		date		not null,
location		varchar2(200)	not null,
repatriated		date		not null,
last_sync_id		number,
spms_URL		varchar2(100),
ISBN			varchar2(20),
ISSN			varchar2(10),
editor_id		number )
tablespace &&pac_data;

create table conf_roles (
function_id		number		not null,
person_id		number		not null,
profile_id		number,
abstract_id		number )
tablespace &&PAC_Data;

create table conf_abstracts (
abstract_id		number,
conference_id		number		not null,
paper_id		varchar2(13)	not null,
title			varchar2(200)	not null,
presentation_id		number		not null,
classification_id	number		not null,
sub_class_id		number,
eclass_id		number)
tablespace &&PAC_Data;

create table conf_keywords (
keyword_id		number,
conference_id		number		not null,
keyword_code		varchar2(4)	not null,
keyword_descr		varchar2(80)	not null,
regular_expression	varchar2(40) )
tablespace &&pac_data;

create table conf_abs_keywords (
keyword_id		number		not null,
abstract_id		number		not null )
tablespace &&pac_data;

create table conf_presentations (
presentation_id		number,
conference_id		number		not null,
presentation_code	varchar2(2)	not null,
presentation_descr	varchar2(80)	not null,
presentation_type	varchar2(1)	not null )
tablespace &&pac_data;

create table conf_classifications (
classification_id	number,
conference_id		number		not null,
class_code		varchar2(8)	not null,
class_descr		varchar2(100)	not null )
tablespace &&pac_data;

create table conf_sub_classes (
sub_class_id		number,
conference_id		number		not null,
sub_class_code		varchar2(4)	not null,
sub_class_descr		varchar2(80)	not null )
tablespace &&pac_data;

create table conf_eclasses (
eclass_id		number,
conference_id		number		not null,
classification_id	number		not null,
sub_class_id		number		not null )
tablespace &&pac_data;

create table conf_functions (
function_id		number,
conference_id		number		not null,
function_code		varchar2(8)	not null,
function_descr		varchar2(80)	not null )
tablespace &&pac_data;

create table conf_registrants (
person_id		number		not null,
conference_id		number		not null)
tablespace &&pac_data;

-- **********
-- **********

alter table conf_conferences
	add constraint conf_conferences_PK
	primary key (conference_id)
	using index tablespace &&pac_index;

alter table conf_abstracts
	add constraint conf_abstracts_PK
	primary key (abstract_id)
	using index tablespace &&pac_index;

alter table conf_keywords
	add constraint conf_keywords_PK
	primary key (keyword_id)
	using index tablespace &&pac_index;

alter table conf_presentations
	add constraint conf_presentation_PK
	primary key (presentation_id)
	using index tablespace &&pac_index;

alter table conf_classifications
	add constraint conf_classifications_PK
	primary key (classification_id)
	using index tablespace &&pac_index;

alter table conf_sub_classes
	add constraint conf_sub_classes_PK
	primary key (sub_class_id)
	using index tablespace &&pac_index;

alter table conf_functions
	add constraint conf_functions_PK
	primary key (function_id)
	using index tablespace &&pac_index;

alter table conf_eclasses
	add constraint conf_eclasses_PK
	primary key (eclass_id)
	using index tablespace &&pac_index;

-- **********
-- **********

alter table conf_conferences
	add constraint conf_conferences_person_FK
	foreign key (editor_id)
	references persons (person_id);

alter table conf_abstracts
	add constraint conf_abstracts_conf_id_FK
	foreign key (conference_id)
	references conf_conferences (conference_id);

alter table conf_abstracts
	add constraint conf_abstracts_presentation_FK
	foreign key (presentation_id)
	references conf_presentations (presentation_id);

alter table conf_abstracts
	add constraint conf_abstracts_class_code_FK
	foreign key (classification_id)
	references conf_classifications (classification_id);

alter table conf_abstracts
	add constraint conf_abs_sub_class_code_FK
	foreign key (sub_class_id)
	references conf_sub_classes (sub_class_id);

alter table conf_abstracts
	add constraint conf_abs_eclass_code_FK
	foreign key (eclass_id)
	references conf_eclasses (eclass_id);

-- **********

alter table conf_roles
	add constraint conf_roles_profile_FK
	foreign key (profile_id)
	references profiles (profile_id);

alter table conf_roles
	add constraint conf_roles_person_FK
	foreign key (person_id)
	references persons (person_id);

alter table conf_roles
	add constraint conf_roles_function_FK
	foreign key (function_id)
	references conf_functions (function_id);

alter table conf_roles
	add constraint conf_roles_abs_FK
	foreign key (abstract_id)
	references conf_abstracts (abstract_id);

-- **********

alter table conf_keywords
	add constraint conf_keywords_conf_FK
	foreign key (conference_id)
	references conf_conferences (conference_id);

-- **********

alter table conf_abs_keywords
	add constraint conf_abs_keywords_keyword_FK
	foreign key (keyword_id)
	references conf_keywords (keyword_id);

alter table conf_abs_keywords
	add constraint conf_abs_keywords_abs_FK
	foreign key (abstract_id)
	references conf_abstracts (abstract_id);

-- **********

alter table conf_classifications
	add constraint conf_classifications_conf_FK
	foreign key (conference_id)
	references conf_conferences (conference_id);

-- **********

alter table conf_sub_classes
	add constraint conf_sub_classes_conf_FK
	foreign key (conference_id)
	references conf_conferences (conference_id);

-- **********

alter table conf_presentations
	add constraint conf_presentations_conf_FK
	foreign key (conference_id)
	references conf_conferences (conference_id);


-- **********

alter table conf_registrants
	add constraint conf_registrants_conf_FK
	foreign key (conference_id)
	references conf_conferences (conference_id);

alter table conf_registrants
	add constraint conf_registrants_person_FK
	foreign key (person_id)
	references persons (person_id);

-- **********
-- **********

create index conf_abstracts_conference_id
	on conf_abstracts (conference_id)
	tablespace &&pac_index;

create index conf_abstracts_class_id
	on conf_abstracts (classification_id)
	tablespace &&pac_index;

create index conf_abstracts_sub_class_id
	on conf_abstracts (sub_class_id)
	tablespace &&pac_index;

create index conf_abstracts_paper_id
	on conf_abstracts (paper_id, conference_id)
	tablespace &&pac_index;

-- **********

create index conf_roles_function_id on 
	conf_roles (function_id)
	tablespace &&pac_index;

create index conf_roles_person_id on 
	conf_roles (person_id)
	tablespace &&pac_index;

create index conf_roles_profile_id on 
	conf_roles (profile_id)
	tablespace &&pac_index;

create index conf_roles_abstract_id on 
	conf_roles (abstract_id)
	tablespace &&pac_index;

create UNIQUE index conf_abstract_roles_UQ on
	conf_roles (function_id, person_id, profile_id, abstract_id)
	tablespace &&pac_index;

-- **********

create UNIQUE index conf_abs_keywords_UQ
	on conf_abs_keywords (keyword_id, abstract_id)
	tablespace &&pac_index;

create index conf_abs_keywords_abs_id
	on conf_abs_keywords (abstract_id)
	tablespace &&pac_index;

-- **********

create index conf_class_conference_id
	on conf_classifications (conference_id)
	tablespace &&pac_index;

create UNIQUE index conf_classifications_UQ on
	conf_classifications (class_code, conference_id)
	tablespace &&pac_index;

-- **********

create index conf_subclass_conference_id
	on conf_sub_classes (conference_id)
	tablespace &&pac_index;

create UNIQUE index conf_sub_classes_UQ on
	conf_sub_classes (sub_class_code, conference_id)
	tablespace &&pac_index;

-- **********

create UNIQUE index conf_eclasses_UQ
	on conf_eclasses (classification_id, sub_class_id, conference_id)
	tablespace &&pac_index;

-- **********

create index conf_present_conference_id
	on conf_presentations (conference_id)
	tablespace &&pac_index;

create UNIQUE index conf_presentations_UQ on
	conf_presentations (presentation_code, conference_id)
	tablespace &&pac_index;

-- **********

create index conf_functions_conference_id
	on conf_functions (conference_id)
	tablespace &&pac_index;

create UNIQUE index conf_functions_UQ on
	conf_functions (function_code, conference_id)
	tablespace &&pac_index;

-- **********

create index conf_keywords_conference_id
	on conf_keywords (conference_id)
	tablespace &&pac_index;

create index conf_keywords_descr_indx on
	conf_keywords (keyword_descr)
	tablespace &&pac_index;

-- **********

create index conf_class_descr_indx on
	conf_classifications (class_descr)
	tablespace &&pac_index;

-- **********

create index conf_sub_class_descr_indx on
	conf_sub_classes (sub_class_descr)
	tablespace &&pac_index;

-- **********

create UNIQUE index conf_conference_UQ on
	conf_conferences (conference_code, start_date)
	tablespace &&pac_index;

-- **********

create UNIQUE index conf_registrants_UQ on
	conf_registrants (person_id, conference_id)
	tablespace &&pac_index;

-- **********

create or replace trigger conf_abstracts_b4_insert
	before insert on conf_abstracts
	for each row

begin

   if :new.abstract_id is null then
      select abstract_id_seq.nextval
      into :new.abstract_id
      from dual;
   end if;

end;
/
show error

-- **********

create or replace trigger conf_conferences_b4_insert
	before insert on conf_conferences
	for each row

begin

   if :new.conference_id is null then
      select conference_id_seq.nextval
      into :new.conference_id
      from dual;
   end if;

end;
/
show error

-- **********

create or replace trigger conf_presentations_b4_insert
	before insert on conf_presentations
	for each row

begin

   if :new.presentation_id is null then
      select presentation_id_seq.nextval
      into :new.presentation_id
      from dual;
   end if;

end;
/
show error

-- **********

create or replace trigger conf_classifications_b4_insert
	before insert on conf_classifications
	for each row

begin

   if :new.classification_id is null then
      select classification_id_seq.nextval
      into :new.classification_id
      from dual;
   end if;

end;
/
show error

-- **********

create or replace trigger conf_sub_classes_b4_insert
	before insert on conf_sub_classes
	for each row

begin

   if :new.sub_class_id is null then
      select sub_class_id_seq.nextval
      into :new.sub_class_id
      from dual;
   end if;

end;
/
show error

-- **********

create or replace trigger conf_eclasses_b4_insert
	before insert on conf_eclasses
	for each row

begin

   if :new.eclass_id is null then
      select eclass_id_seq.nextval
      into :new.eclass_id
      from dual;
   end if;

end;
/
show error

-- **********

create or replace trigger conf_keywords_b4_insert
	before insert on conf_keywords
	for each row

begin

   if :new.keyword_id is null then
      select keyword_id_seq.nextval
      into :new.keyword_id
      from dual;
   end if;

end;
/
show error

-- **********

create or replace trigger conf_functions_b4_insert
	before insert on conf_functions
	for each row

begin

   if :new.function_id is null then
      select function_id_seq.nextval
      into :new.function_id
      from dual;
   end if;

end;
/
show error

-- **********

spool off
