/*
drop table stakeholders;
drop table stakeholder_codes;
drop table stakeholder_group_codes;
drop table stakeholder_codes_TEMP;

delete from menu_items where item_id between 252 and 256;
delete from maint_yes_no_flags where table_column_name like 'STAKEHOLDER_CODES%';

commit;
*/
select tablespace_name from user_tablespaces;

create table stakeholder_group_codes (
stakeholder_group_code	varchar2(4),
stakeholder_group_descr	varchar2(80)	not null )
tablespace &&pac_data;

comment on table stakeholder_group_codes is 'Groupings for stakeholders.';

alter table stakeholder_group_codes
	add constraint stakeholder_group_codes_PK
	primary key (stakeholder_group_code)
	using index
	tablespace &&PAC_Index;

insert into stakeholder_group_codes values ('REP',	'Representative');
insert into stakeholder_group_codes values ('OFF',	'JACoW Officer');
insert into stakeholder_group_codes values ('TEAM',	'JACoW Team Member');
insert into stakeholder_group_codes values ('BOD',	'Board of Directors');

-- **********

create table stakeholder_codes (
stakeholder_code	varchar2(4),
stakeholder_descr	varchar2(80)	not null,
stakeholder_group_code	varchar2(4)	not null,
require_affiliation	char(1)		default 'N' not null,
require_conference	char(1)		default 'N' not null,
require_series		char(1)		default 'N' not null,
require_dates		char(1)		default 'N' not null,
sort_order		number )
tablespace &&pac_data;

comment on column stakeholder_codes.require_affiliation is 'Set to require members'' affiliation on maintenance screen.';
comment on column stakeholder_codes.require_conference is 'Set to require members'' conference on maintenance screen.';
comment on column stakeholder_codes.require_series is 'Set to require members'' conference series on maintenance screen.';
comment on column stakeholder_codes.require_dates is 'Set to require members'' starting and ending dates on maintenance screen.';

alter table stakeholder_codes
	add constraint stakeholder_codes_PK
	primary key (stakeholder_code)
	using index
	tablespace &&PAC_Index;

alter table stakeholder_codes
	add constraint stakeholder_codes_FK
	foreign key (stakeholder_group_code)
	references stakeholder_group_codes (stakeholder_group_code);

insert into stakeholder_codes values ('CHAR',	'Board of Directors, Chair',					'BOD', 'N', 'N', 'N', 'Y', 10);
insert into stakeholder_codes values ('DEP',	'Board of Directors, Deputy Chair',				'BOD', 'N', 'N', 'N', 'Y', 20);
insert into stakeholder_codes values ('COOR',	'Board of Directors, Coordinator',				'BOD', 'N', 'N', 'N', 'Y', 30);
insert into stakeholder_codes values ('DIR',	'Board of Directors, Director',					'BOD', 'N', 'N', 'N', 'Y', 40);

insert into stakeholder_codes values ('RM',	'Repository Manager',						'OFF', 'N', 'N', 'N', 'N', 100);
insert into stakeholder_codes values ('WM',	'Web Master',							'OFF', 'N', 'N', 'N', 'N', 100);
insert into stakeholder_codes values ('SPMS',	'SPMS Developer',						'OFF', 'N', 'N', 'N', 'N', 100);
insert into stakeholder_codes values ('RSCA',	'Regional Support Center Manager, Asia',			'OFF', 'N', 'N', 'N', 'N', 100);
insert into stakeholder_codes values ('RSCE',	'Regional Support Center Manager, Europe, ',			'OFF', 'N', 'N', 'N', 'N', 100);
insert into stakeholder_codes values ('RSCN',	'Regional Support Center Manager, North America',		'OFF', 'N', 'N', 'N', 'N', 100);

insert into stakeholder_codes values ('EDIC',	'Editor-in-Chief',						'TEAM',	'N', 'Y', 'N', 'Y', 200);
insert into stakeholder_codes values ('COOP',	'Co-Opted Team Member',						'TEAM', 'N', 'N', 'N', 'N', 200);

insert into stakeholder_codes values ('SPCC',	'Scientific Program Committee Chair',				'REP', 'N', 'Y', 'N', 'Y', 300);
insert into stakeholder_codes values ('LABR',	'Laboratory Representative',					'REP', 'Y', 'N', 'N', 'N', 300);
insert into stakeholder_codes values ('CSR',	'Conference Series Representative',				'REP', 'N', 'N', 'Y', 'N', 300);

-- **********

create table stakeholders (
stakeholder_code	varchar2(4)	not null,
person_id		number		not null,
affiliation_id		number,
conference_id		number,
conference_code		varchar2(10),
start_date		date,
end_date		date )
tablespace &&pac_data;

alter table stakeholders
	add constraint stakeholders_person_id_FK
	foreign key (person_id)
	references persons (person_id);

alter table stakeholders
	add constraint stakeholders_affiliation_id_FK
	foreign key (affiliation_id)
	references affiliations (affiliation_id);

alter table stakeholders
	add constraint stakeholders_conference_id_FK
	foreign key (conference_id)
	references conf_conferences (conference_id);

alter table stakeholders
	add constraint stakeholders_conf_code_FK
	foreign key (conference_code)
	references conference_codes (conference_code);

alter table stakeholders
	add constraint stakeholders_SH_code_FK
	foreign key (stakeholder_code)
	references stakeholder_codes (stakeholder_code);

create UNIQUE index stakeholders_UQ
	on stakeholders (person_id, stakeholder_code, conference_id, start_date, end_date)
	tablespace &&PAC_Index;

-- **********

insert into menu_items values (252, null, 'Stakeholders', 'Y', 1, null);

set define off
insert into menu_items values (253, 'maint.tab?tab_name=stakeholder_codes&order_by=sort_order,stakeholder_descr', 'Stakeholder Codes', 'Y', 252, null);
insert into menu_items values (254, 'maint.tab?tab_name=stakeholder_group_codes&order_by=stakeholder_group_descr', 'Stakeholder Group Codes', 'Y', 252, null);
insert into menu_items values (255, 'stakeholder.html', 'Stakeholder Maintenance', 'Y', 252, null);
insert into menu_items values (256, 'stakeholder.mail', 'Stakeholder Email Utility', 'Y', 252, null);
set define on

insert into maint_yes_no_flags values ('STAKEHOLDER_CODES.REQUIRE_AFFILIATION');
insert into maint_yes_no_flags values ('STAKEHOLDER_CODES.REQUIRE_CONFERENCE');
insert into maint_yes_no_flags values ('STAKEHOLDER_CODES.REQUIRE_SERIES');
insert into maint_yes_no_flags values ('STAKEHOLDER_CODES.REQUIRE_DATES');

CREATE GLOBAL TEMPORARY TABLE stakeholder_codes_TEMP (
stakeholder_code	varchar2(4) not null)
ON COMMIT DELETE ROWS;
