--	Copyright (C) 2004 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>

-- ****************************************************************************************

spool Create_Triggers.log

create or replace trigger persons_b4_ins
	before insert
	on persons for each row

begin

   if :new.person_id is null then
      select person_id_seq.nextval into :new.person_id from dual;
   end if;

   :new.fname		:= trim (:new.fname);
   :new.mname		:= trim (:new.mname);
   :new.pname		:= trim (:new.pname);
   :new.initials	:= trim (:new.initials);

-- Do not change the sequece of the next two lines

   :new.lname		:= trim (:new.lname);
   :new.soundex_lname	:= soundex (:new.lname);

   :new.sort_lname	:= common.sort_text (:new.lname);
   :new.sort_fname	:= common.sort_text (:new.fname);
   :new.sort_mname	:= common.sort_text (:new.mname);

   :new.user_id		:= trim (upper (:new.user_id));
   :new.update_date	:= common.get_UTC_time;

end;
/
show error

-- ************************************************************

create or replace trigger persons_b4_upd
	before update
	on persons for each row

begin

   :new.update_date	:= common.get_UTC_time;

end;
/
show error

-- ************************************************************

create or replace trigger persons_b4_upd_fname
	before update of fname
	on persons for each row

begin

   :new.fname		:= trim (:new.fname);
   :new.sort_fname	:= common.sort_text (:new.fname);

end;
/
show error

-- ************************************************************

create or replace trigger persons_b4_upd_mname
	before update of mname
	on persons for each row

begin

   :new.mname		:= trim (:new.mname);
   :new.sort_mname	:= common.sort_text (:new.mname);

end;
/
show error

-- ************************************************************

create or replace trigger persons_b4_upd_lname
	before update of lname
	on persons for each row

begin

-- Do not change the sequece of these lines

   :new.lname		:= trim (:new.lname);
   :new.soundex_lname	:= soundex (:new.lname);
   :new.sort_lname	:= common.sort_text (:new.lname);

end;
/
show error

-- ************************************************************

create or replace trigger persons_b4_upd_pname
	before update of pname
	on persons for each row

begin

   :new.pname		:= trim (:new.pname);

end;
/
show error

-- ************************************************************

create or replace trigger persons_b4_upd_user_id
	before update of user_id
	on persons for each row

begin

   :new.user_id		:= trim (upper (:new.user_id));

end;
/
show error

-- ************************************************************
-- ************************************************************

create or replace trigger profiles_b4_ins
	before insert
	on profiles for each row

begin

   if :new.profile_id is null then
      select profile_id_seq.nextval into :new.profile_id from dual;
   end if;

   :new.email_address	:= lower (trim (:new.email_address));
   :new.update_date	:= common.get_UTC_time;

end;
/
show error

-- ************************************************************

create or replace trigger profiles_b4_upd
	before update
	on profiles for each row

begin

   :new.email_address	:= lower (trim (:new.email_address));
   :new.update_date	:= common.get_UTC_time;

end;
/
show error

-- ************************************************************
-- ************************************************************

create or replace trigger addresses_b4_ins_upd
	before insert or update
	on addresses for each row

begin

   if :new.address_id is null then
      select address_id_seq.nextval into :new.address_id from dual;
   end if;

   :new.address1	:= common.trim_crlf(:new.address1);
   :new.address2	:= common.trim_crlf(:new.address2);
   :new.address3	:= common.trim_crlf(:new.address3);
   :new.po_box		:= common.trim_crlf(:new.po_box);
   :new.town		:= common.trim_crlf(:new.town)	;
   :new.postal_code	:= common.trim_crlf(:new.postal_code);
   :new.zip_code	:= common.trim_crlf(:new.zip_code);
   :new.update_date	:= common.get_UTC_time;

   if :new.address_id <> 0 then
      :new.search_text	:=	:new.address1		|| ' ' ||
				:new.address2		|| ' ' ||
				:new.address3		|| ' ' ||
				:new.po_box		|| ' ' ||
				:new.town		|| ' ' ||
				:new.postal_code	|| ' ' ||
				:new.zip_code		|| ' ' ||
				:new.country_code	|| ' ' ||
				common.decode_value ('country_descr', 'country_codes',
					'country_code', :new.country_code);
   end if;

end;
/
show error

-- ************************************************************
-- ************************************************************

create or replace trigger affiliations_b4_ins_upd
	before insert or update
	on affiliations for each row

begin

   if :new.affiliation_id is null then
      select affiliation_id_seq.nextval into :new.affiliation_id from dual;
   end if;

   :new.affil_abbrev	:= common.trim_crlf (:new.affil_abbrev);
   :new.name1		:= common.trim_crlf (:new.name1);
   :new.name2		:= common.trim_crlf (:new.name2);
   :new.department	:= common.trim_crlf(:new.department);
   :new.update_date	:= common.get_UTC_time;

   if :new.affiliation_id <> 0 then
      :new.search_text :=
			:new.affil_abbrev	|| ' ' ||
			:new.name1		|| ' ' ||
			:new.name2		|| ' ' ||
			:new.department;
   end if;

end;
/
show error

-- ************************************************************
-- ************************************************************

create or replace trigger country_codes_b4_ins_upd
	before insert or update
	on country_codes for each row

begin

   if :new.country_abbrev is null then
      :new.country_abbrev := :new.country_code;
   end if;

   :new.update_date	:= common.get_UTC_time;

end;
/
show error

-- ************************************************************
-- ************************************************************

create or replace trigger logs_b4_insert
	before insert
	on logs for each row

declare

cursor	driver (ID$ integer) is
select	common.get_paper_id (abstract_id) paper_id, title
from	abstracts
where	abstract_id = ID$;

rec	driver%ROWTYPE;

buffer		mail_blats.message%TYPE := null;
pid		integer;
admin_email	boolean := false;
PC_email	boolean := false;

begin

   if :new.log_id is null then
      select log_id_seq.nextval into :new.log_id from dual;
   end if;

   if :new.log_date is null then
      :new.log_date := common.get_UTC_time;
   end if;

   if :new.person_id is null then
      :new.person_id	:= nvl (common.gbl_priv_pid, common.gbl_pid);
   end if;

-- ********** Editor Status Code

   if common.decode_value ('send_email_notice', 'status_codes', 'status_code', :new.status_code) = 'Y' then

      open driver (:new.abstract_id); -- Get the abstract data
      fetch driver into rec;
      close driver;

      pid := common.get_author_profile_id (:new.abstract_id, common.primary_author$);
      pid := common.decode_value ('person_id', 'profiles', 'profile_id', pid);

      buffer := common.get_user_name (pid) || common.eol2$ || 'Your contribution ' ||
		rec.paper_id || ', "' || rec.title || '," ' ||
		common.decode_value ('email_msg', 'status_codes', 'status_code', :new.status_code);

      if common.decode_value ('user_validate', 'status_codes', 'status_code', :new.status_code) = 'Y' then
         buffer := buffer || common.eol2$ ||
			'You may Accept or Reject the Editor''s changes using the following URL: ' ||
			common.server || common.vdir$ || 'profile.login?URL=jacow.user_status%3Fabs_id%3D' ||
			:new.abstract_id;
      end if;

      send_mail (common.get_user_email (pid),
		common.conf_title$ || ' ' || rec.paper_id || ' Status Change',
		buffer, common.editor_email$);
   end if;

-- ********** Referee Status Code

   if common.decode_value ('send_email_notice', 'referee_codes', 'referee_code', :new.referee_code) = 'Y' then
      open driver (:new.abstract_id); -- Get the abstract data
      fetch driver into rec;
      close driver;

      pid := common.get_author_profile_id (:new.abstract_id, common.primary_author$);
      pid := common.decode_value ('person_id', 'profiles', 'profile_id', pid);
      send_mail (common.get_user_email (pid),
		common.conf_title$ || ' ' || rec.paper_id || ' Referee Status Change',
		common.get_user_name (pid) || common.eol2$ || 'Your contribution ' ||
		rec.paper_id || ', "' || rec.title || '," ' ||
		common.decode_value ('email_msg', 'referee_codes', 'referee_code', :new.referee_code),
		common.editor_email$);

      if common.referee_declined (:new.referee_code) then
         if common.referee_notify_decline$ = 'Y' then
            admin_email := true;
         end if;

         if common.referee_PC_notify_decline$ = 'Y' then
            PC_email := true;
         end if;
      end if;

      if common.referee_revisions (:new.referee_code) then
         if common.referee_notify_revisions$ = 'Y' then
            admin_email := true;
         end if;

         if common.referee_PC_notify_revision$ = 'Y' then
            PC_email := true;
         end if;
      end if;

      if admin_email then
         send_mail (common.editor_email$,
		common.conf_title$ || ' ' || rec.paper_id || ' Referee Status Change',
		rec.paper_id || ' "' || rec.title || '" ' || ' referee status was set to ' ||
		common.decode_value ('referee_descr', 'referee_codes', 'referee_code', :new.referee_code),
		common.editor_email$);
      end if;

      if PC_email and common.chairs_email$ is not null then
         send_mail (common.chairs_email$,
		common.conf_title$ || ' ' || rec.paper_id || ' Referee Status Change',
		rec.paper_id || ' "' || rec.title || '" ' || ' referee status was set to ' ||
		common.decode_value ('referee_descr', 'referee_codes', 'referee_code', :new.referee_code),
		common.editor_email$);
      end if;

   end if;

end;
/
show error

-- ************************************************************
-- ************************************************************

create or replace trigger abstracts_search_text
	before insert
	on abstracts for each row

begin

   :new.title		:= common.trim_crlf (:new.title);
   :new.description	:= common.trim_crlf (:new.description);
   :new.agency		:= common.trim_crlf (:new.agency);
   :new.footnote	:= trim (:new.footnote);
   :new.search_text	:= :new.title || ' ' || :new.description;

   jacow.validate_eclass_code (:new.classification_code, :new.sub_class_code);

   if :new.classification_code <> :old.classification_code OR
      nvl (:new.sub_class_code, '~~~') <> nvl (:old.sub_class_code, '~~~') then
      :new.eclass_code	:= jacow.get_eclass_code (:new.classification_code, :new.sub_class_code);
   end if;

end;
/
show error

-- ************************************************************
-- ************************************************************

create or replace trigger abstracts_after_insert
	after insert
	on abstracts for each row

begin

   insert into logs (abstract_id, log_code)
   values (:new.abstract_id, common.abstract_created$);

   if common.has_privs_to_page ('search.abs_qa', false) then
      insert into logs (abstract_id, log_code)
      values (:new.abstract_id, common.abstract_QA$);
   end if;

end;
/
show error

-- ************************************************************
-- ************************************************************

create or replace trigger abstracts_b4_update
	before update
	on abstracts for each row

begin

   jacow.validate_eclass_code (:new.classification_code, :new.sub_class_code);

   :new.eclass_code	:= jacow.get_eclass_code (:new.classification_code, :new.sub_class_code);
   :new.title		:= common.trim_crlf (:new.title);
   :new.description	:= common.trim_crlf (:new.description);
   :new.agency		:= common.trim_crlf (:new.agency);
   :new.footnote	:= trim (:new.footnote);

   if :new.title <> :old.title or
      :new.description <> :new.description then
      :new.search_text	:= :new.title || ' ' || :new.description;
   end if;

-- ********** 

   if nvl (common.gbl_priv_pid, common.gbl_pid) <> common.admin_pid$ then
      jacow.abstract_email (:new.abstract_id, common.get_paper_id (:new.abstract_id),
			:new.title, :new.description, :new.classification_code, :new.presentation_code,
			:old.title, :old.description, :old.classification_code, :old.presentation_code);

      if :new.title			<> :old.title			OR
	 nvl (:new.description,'~')	<> nvl (:old.description,'~')	OR
	 nvl (:new.agency,'~')		<> nvl (:old.agency,'~')	OR
	 nvl (:new.footnote,'~')	<> nvl (:old.footnote,'~')	OR
	:new.presentation_code		<> :old.presentation_code	OR
        :new.classification_code	<> :old.classification_code	OR
	 nvl (:new.sub_class_code,'~')	<> nvl (:old.sub_class_code,'~') then

         insert into logs (abstract_id, log_code)
         values (:new.abstract_id, common.abstract_updated$);

         if common.has_privs_to_page ('search.abs_qa', false) then
            insert into logs (abstract_id, log_code)
            values (:new.abstract_id, common.abstract_QA$);
         end if;
      end if;

   end if;

-- **********

   if common.referee_conf$ = 'Y' AND
		trim (common.referee_notice$) is not null AND
		common.publishable_status_code (:new.status_code) AND
		nvl(:old.status_code,'~') <> :new.status_code then

      if NOT JACoW.email_referee (:new.abstract_id, common.get_paper_id (:new.abstract_id), :new.title) then
         if common.referee_auto_assign$ = 'Y' then
            JACoW.assign_referee (:new.abstract_id, :new.classification_code);

            if JACoW.email_referee (:new.abstract_id, common.get_paper_id (:new.abstract_id), :new.title) then
               null;
            end if;
         end if;
      end if;

   end if;

-- ********** Log changes

   if nvl (common.gbl_priv_pid, common.gbl_pid) <> common.admin_pid$ then
      if :new.person_id <> :old.person_id then
         insert into logs (abstract_id, log_code)
         values (:new.abstract_id, common.abs_owner_change$);
      end if;
   end if;

end;
/
show error

-- ************************************************************
-- ************************************************************

create or replace trigger abstracts_b4_delete
	before delete
	on abstracts for each row

begin

   update control set withdrawn_count = withdrawn_count + 1;
      
end;
/
show error

-- ************************************************************
-- ************************************************************

create or replace trigger mail_blats_new
	before insert on mail_blats for each row

begin

   if :new.create_date is null then
      :new.create_date	:= common.get_UTC_time;
   end if;

   :new.recipient	:= common.trim_crlf (:new.recipient);
   :new.cc		:= common.trim_crlf (:new.cc);
   :new.bcc 		:= common.trim_crlf (:new.bcc);

end;
/
sho error

-- ************************************************************
-- ************************************************************

create or replace trigger files_b4_ins
	before insert
	on files for each row

begin

   if :new.file_id is null then
      select file_id_seq.nextval
      into   :new.file_id
      from   dual;
   end if;

   :new.uploaded_date	:= common.get_UTC_time;

end;
/
show error

-- ************************************************************
-- ************************************************************

create or replace trigger files_before_del
	before delete
	on files for each row

begin

   insert into logs (abstract_id, log_code, comments)
   values (:old.abstract_id, common.file_removed$, :old.file_name);

end;
/
show error

-- ************************************************************
-- ************************************************************

create or replace trigger files_after_ins
         after insert
         on files for each row

begin

   insert into logs (abstract_id, log_code, comments)
   values (:new.abstract_id, common.file_uploaded$, :new.file_name);

end;
/	 
show error	 

-- ************************************************************
-- ************************************************************

create or replace trigger control_pwd
	before insert or update of jacow_pwd
	on control for each row

begin

   :new.jacow_pwd := common.encrypt (:new.jacow_pwd);

end;
/
show error

-- ************************************************************
-- ************************************************************

create or replace trigger authors_insert
	before insert or update
	on authors for each row

begin

   select	author_type
   into		:new.author_type
   from		author_codes
   where	author_code = :new.author_code;

end;
/
show error

-- ************************************************************
-- ************************************************************

create or replace trigger presentation_codes_insert
	before insert
	on presentation_codes for each row

begin

   :new.editor_only			:= nvl (:new.editor_only, 'Y');
   :new.presentation_type_code		:= nvl (:new.presentation_type_code, 'O');
   :new.max_pages			:= 5;

end;
/
show error

-- **********

create or replace trigger present_codes_after_insert
	after insert on presentation_codes
	for each row

begin

   insert into presentation_file_codes
	(presentation_code, file_type_code, editor_only, always_available, weight)
   select :new.presentation_code, file_type_code, 'N', 'N', 0
   from   file_type_codes;

end;
/
show error

-- **********

create or replace trigger present_codes_B4_delete
	before delete on presentation_codes
	for each row

begin

   delete from presentation_file_codes
   where presentation_code = :old.presentation_code;

end;
/
show error

-- ************************************************************
-- ************************************************************

create or replace trigger keyword_codes_b4_insert
	before insert on keyword_codes
	for each row

begin

   if :new.conference_code is null then
      select conference_code into :new.conference_code from control;
   end if;

end;
/
show error

-- ************************************************************
-- ************************************************************

create or replace trigger poplist_items_b4_insert
	before insert on poplist_items
	for each row

begin

   if :new.poplist_item_id is null then
      select poplist_item_id_seq.nextval into :new.poplist_item_id from dual;
   end if;

end;
/
show error

-- ************************************************************
-- ************************************************************

create or replace trigger pages_b4_insert
	before insert on pages
	for each row

begin

   if :new.page_id is null then
      select page_id_seq.nextval into :new.page_id from dual;
   end if;

   :new.available		:= nvl (:new.available, 'N');
   :new.show_on_profile		:= nvl (:new.show_on_profile, 'N');
   :new.show_on_abstract	:= nvl (:new.show_on_abstract, 'N');
   :new.admin_privs		:= nvl (:new.admin_privs, 'Y');
   :new.login_req		:= nvl (:new.login_req, 'Y');

end;
/
show error

-- ************************************************************
-- ************************************************************

create or replace trigger daily_events_b4_insert
	before insert on daily_events
	for each row

begin

   if :new.daily_event_id is null then
      select daily_event_id_seq.nextval
      into :new.daily_event_id
      from dual;
   end if;

end;
/
show error

-- ************************************************************
-- ************************************************************

create or replace trigger registrations_b4_insert
	before insert on registrations
	for each row

begin

   if :new.timestamp is null then
      :new.timestamp := common.get_UTC_time;
   end if;

   if :new.registration_id is null then
      select registration_id_seq.nextval into :new.registration_id from dual;
   end if;

end;
/
show error

-- ************************************************************
-- ************************************************************

create or replace trigger fee_matrix_b4_insert
	before insert on fee_matrix
	for each row

begin

   if :new.fee_matrix_id is null then
      select fee_matrix_id_seq.nextval into :new.fee_matrix_id from dual;
   end if;

end;
/
show error

-- ************************************************************
-- ************************************************************

create or replace trigger meta_data_encrypt
	before update of encrypt on meta_data_codes
	for each row

begin

   if nvl (:new.encrypt, 'N') <> nvl (:old.encrypt, 'N') then
      if nvl (:new.encrypt, 'N') = 'Y' then
         update registrations
         set	value		= common.encrypt (value)
         where	meta_data_code	= :new.meta_data_code
         and	value		is not null;
      else
         update registrations
         set	value = common.decrypt (value)
         where	meta_data_code	= :new.meta_data_code
         and	value		is not null;
      end if;
   end if;

end;
/
show error

-- **********

create or replace trigger meta_data_late_fee
	before insert or update of fee, late_fee on meta_data_codes
	for each row

begin

   if nvl (:new.late_fee, 0) = 0 then
      if nvl (:new.fee, 0) <> 0 then
         :new.late_fee := :new.fee;
      end if;
   end if;

end;
/
show error

-- **********

create or replace trigger meta_data_4b_insert
	before insert on meta_data_codes
	for each row

begin

   if :new.reg_cat_code is null then
      select reg_cat_code into :new.reg_cat_code from reg_cat_codes where rownum < 2;
   end if;

   :new.SHORT_DESCR		:= nvl (:new.SHORT_DESCR, substr (common.trim_crlf (:new.meta_data_descr), 1, 20) );
   :new.DATA_LENGTH		:= nvl (:new.DATA_LENGTH,	'1');
   :new.VIEW_LENGTH		:= nvl (:new.VIEW_LENGTH,	'1');
   :new.freeze_updates		:= nvl (:new.freeze_updates,	'N');
   :new.REQUIRED		:= nvl (:new.REQUIRED,		'N');
   :new.FIXED_LENGTH		:= nvl (:new.FIXED_LENGTH,	'N');
   :new.ENCRYPT			:= nvl (:new.ENCRYPT,		'N');
   :new.FREEZE_UPDATES		:= nvl (:new.FREEZE_UPDATES,	'N');
   :new.SINGLE_SELECTION	:= nvl (:new.SINGLE_SELECTION,	'N');

end;
/
show error

-- ************************************************************
-- ************************************************************

create or replace trigger sub_class_codes_b4_insert
	before insert on sub_class_codes
	for each row

begin

   :new.sub_class_code := upper (:new.sub_class_code);

end;
/
show error

-- ************************************************************
-- ************************************************************

create or replace trigger credit_cards_b4_insert_trigger
	before insert
	on credit_cards
	for each row

begin

   if :new.credit_card_id is null then
      select credit_card_id_seq.nextval into :new.credit_card_id from dual;
   end if;

   :new.card_number	:= common.encrypt (:new.card_number);
   :new.cvv2		:= common.encrypt (:new.cvv2);

end;
/
show error

create or replace trigger credit_cards_b4_upd_num
	before update of card_number
	on credit_cards
	for each row

begin

   :new.card_number := common.encrypt (:new.card_number);

end;
/
show error

create or replace trigger credit_cards_b4_upd_cvv2
	before update of cvv2
	on credit_cards
	for each row

begin

   :new.cvv2 := common.encrypt (:new.cvv2);

end;
/
show error

-- ************************************************************
-- ************************************************************

create or replace trigger registrants_trigger
	before insert or update
	on registrants
	for each row

begin

   if inserting then
      :new.create_date := common.get_UTC_time;
   end if;

   :new.last_update := common.get_UTC_time;

end;
/
show error

-- ************************************************************
-- ************************************************************

create or replace trigger abs_presentation_trigger
	before insert
	on abstract_presentations
	for each row

begin

   :new.timestamp := common.get_UTC_time;

end;
/
show error

-- ************************************************************
-- ************************************************************

create or replace trigger abs_classifications_trigger
	before insert
	on abstract_classifications
	for each row

begin

   :new.timestamp := common.get_UTC_time;

end;
/
show error

-- ************************************************************
-- ************************************************************

create or replace trigger payments_b4_insert
	before insert on payments
	for each row

begin

   :new.timestamp := common.get_UTC_time;
   :new.person_id := nvl (:new.person_id, common.gbl_pid);

   if :new.payment_id is null then
      select payment_id_seq.nextval
      into   :new.payment_id
      from   dual;
   end if;

end;
/
show error

-- ************************************************************
-- ************************************************************

create or replace trigger checkbox_items_b4_insert
	before insert
	on checkbox_items
	for each row

begin

   if :new.checkbox_item_id is null then
      select checkbox_item_id_seq.nextval into :new.checkbox_item_id from dual;
   end if;

   :new.timestamp := common.get_UTC_time;

end;
/
show error

-- ************************************************************
-- ************************************************************

create or replace trigger persons_B4_delete
	before delete on persons
	for each row

begin

   insert into deleted_persons values (:old.person_id, common.get_UTC_time);

end;
/
show error

-- ************************************************************
-- ************************************************************

create or replace trigger profiles_B4_delete
	before delete on profiles
	for each row

begin

   insert into deleted_profiles values (:old.profile_id, common.get_UTC_time);

end;
/
show error

-- ************************************************************
-- ************************************************************

create or replace trigger file_types_after_insert
	after insert on file_type_codes
	for each row

begin

   insert into presentation_file_codes
	(presentation_code, file_type_code, editor_only, always_available, weight)
   select presentation_code, :new.file_type_code, 'N', 'N', 0
   from   presentation_codes;

end;
/
show error

-- **********

create or replace trigger file_types_B4_delete
	before delete on file_type_codes
	for each row

begin

   delete from presentation_file_codes
   where file_type_code = :old.file_type_code;

end;
/
show error

-- ************************************************************
-- ************************************************************

create or replace trigger presentation_file_codes_UPD
	before update of weight on presentation_file_codes
	for each row

begin

   update abstracts a
   set    assignable = greatest (nvl (assignable, 0) -
				 nvl(:old.weight, 0) +
				 nvl(:new.weight, 0),
			       0)
   where  exists
	(
		select	1
		from	files			f
		where	f.abstract_id		= a.abstract_id
		and	f.file_type_code	= :new.file_type_code
	)
   and	a.presentation_code = :new.presentation_code;

end;
/
show error

-- ************************************************************
-- ************************************************************

create or replace trigger class_sessions_B4_insert
	before insert on class_sessions
	for each row

declare	icount integer := 0;

begin

   if :new.classification_code is not null then
      icount := icount + 1;
   end if;

   if :new.sub_class_code is not null then
      icount := icount + 1;
   end if;

   if :new.eclass_code is not null then
      icount := icount + 1;
   end if;

   if icount <> 1 then
      raise_application_error(-20000, 'Foreign Key Arch violated.');
   end if;

   if :new.class_session_id is null then
      select class_session_id_seq.nextval
      into :new.class_session_id
      from dual;
   end if;

end;
/
show error

-- ************************************************************
-- ************************************************************

create or replace trigger status_codes_trigger
	before insert on status_codes
	for each row

begin

   :new.status_code := upper (:new.status_code);

end;
/
show error

-- ************************************************************
-- ************************************************************

create or replace trigger referee_codes_trigger
	before insert on referee_codes
	for each row

begin

   :new.referee_code := lower (:new.referee_code);

end;
/
show error

-- ************************************************************
-- ************************************************************

create or replace trigger payment_vendor_vars_B4
	before insert or update
	on payment_vendor_vars
	for each row

begin

   if inserting AND :new.payment_vendor_var_ID is null then
      select payment_vendor_var_ID_SEQ.nextval
      into :new.payment_vendor_var_ID from dual;
   end if;

   :new.var_text := common.trim_crlf (:new.var_text);

end;
/
show error

-- ************************************************************
-- ************************************************************

create or replace trigger no_login_urls_B4
	before insert or update
	on no_login_urls
	for each row

begin

   :new.URL := lower (:new.URL);

end;
/
show error

-- ************************************************************
-- ************************************************************

CREATE OR REPLACE TRIGGER "HOTEL_CAT_TRIGGER"
	before insert on "HOTEL_CATEGORIES"
	for each row
begin

   if inserting then
      if :NEW.HOTEL_CAT_ID is null then
         select HOTEL_CAT_SEQ.nextval into :NEW."HOTEL_CAT_ID" from dual;
      end if;
   end if;
end;
/
show error

-- ************************************************************
-- ************************************************************

CREATE OR REPLACE TRIGGER "HOTEL_PAY_TG"
	before insert on "HOTEL_PAYMENTS"
	for each row

begin
   if inserting then
      if :NEW.METHOD_ID is null then
         select HOTEL_PAY_SQ.nextval into :NEW."METHOD_ID" from dual;
      end if;
   end if;
end;
/
show error

-- ************************************************************
-- ************************************************************

CREATE OR REPLACE TRIGGER "HOTEL_ROOMTYPES"
	before insert on "HOTEL_ROOMTYPE_FEE"
	for each row

begin
   if inserting then
      if :NEW."HRF_ID" is null then
         select HOTEL_ROOMTYPE_SEQ.nextval into :NEW."HRF_ID" from dual;
      end if;
   end if;
end;
/
show error

-- ************************************************************
-- ************************************************************

CREATE OR REPLACE TRIGGER "HOTEL_ID_TRIG"
	before insert on "HOTELS"
	for each row

begin

   if inserting then
      if :NEW."HOTEL_ID" is null then
         select HOTEL_SEQ_ID.nextval into :NEW."HOTEL_ID" from dual;
      end if;
   end if;
end;
/
show error

-- ************************************************************
-- ************************************************************

CREATE OR REPLACE TRIGGER "REGISTRANTS_CHOICES_TR"
	before insert on "REGISTRANTS_CHOICES"
	for each row

begin

   if inserting then
      if :NEW."CHOICE_ID" is null then
         select REGISTRANTS_CHOICE_SQ.nextval into :NEW."CHOICE_ID" from dual;
      end if;
   end if;
end;
/
show error

-- ************************************************************
-- ************************************************************

CREATE OR REPLACE TRIGGER "ROOM_DEPOSIT_ID"
	before insert on "ROOM_DEPOSIT"
	for each row

begin

   if inserting then
      if :NEW."DEPOSIT_ID" is null then
         select ROOM_DEPOSIT_SQ.nextval into :NEW."DEPOSIT_ID" from dual;
      end if;
   end if;
end;
/
show error

-- ************************************************************
-- ************************************************************

CREATE OR REPLACE TRIGGER "ROOM_TYPES_TRIGGER"
	before insert on "ROOM_TYPES"
	for each row

begin
   if inserting then
      if :NEW."ROOM_TYPE_ID" is null then
         select ROOM_TYPES_SQ.nextval into :NEW."ROOM_TYPE_ID" from dual;
      end if;
   end if;
end;
/
show error

-- ************************************************************
-- ************************************************************

CREATE OR REPLACE TRIGGER "ROOM_HOTELS_ID"
	before insert on "ROOMS_HOTEL"
	for each row

begin

   if inserting then
      if :NEW."ROOM_ID" is null then
         select ROOM_HOTELS_SEQ.nextval into :NEW."ROOM_ID" from dual;
      end if;
   end if;
end;
/
show error

-- ************************************************************
-- ************************************************************

CREATE OR REPLACE TRIGGER TEMP_REGISTRANTS_CHOICES_TR
	before insert on TEMP_REGISTRANTS_CHOICES
	for each row

begin

   if :NEW.CHOICE_ID is null then
      select TEMP_REGISTRANTS_CHOICES_SQ.nextval
      into :NEW.CHOICE_ID
      from dual;
   end if;

end;
/
show error

-- ************************************************************
-- ************************************************************

CREATE OR REPLACE TRIGGER MISC_PAGE_TG
	before insert on MISC_DATA_PAGE
	for each row

begin

   if :NEW.MISC_ID is null then
      select MISC_PAGE_SQ.nextval
      into :NEW.MISC_ID
      from dual;
   end if;

end;
/
show error

-- ************************************************************
-- ************************************************************

CREATE OR REPLACE TRIGGER TEMPLATE_TG
	before insert on EXPORT_TEMPLATES
	for each row

begin

   if :NEW.TEMPLATE_ID is null then
      select EXPORT_TEMPLATES_SQ.nextval into :NEW.TEMPLATE_ID from dual;
   end if;

end;
/
show error

-- ************************************************************
-- ************************************************************

create or replace trigger regist_bypass_vars_b4_insert
	before insert on regist_bypass_vars
	for each row

begin

   if :new.bypass_id is null then
      select bypass_id_seq.nextval into :new.bypass_id from dual;
   end if;

end;
/
show error

-- ************************************************************
-- ************************************************************

create or replace trigger applications_B4_insert
	before insert on applications
	for each row

begin


   if :new.application_id is null then
      select application_id_seq.nextval
      into :new.application_id
      from dual;
   end if;

   :new.application_name := lower (:new.application_name);

end;
/
show error

-- ************************************************************
-- ************************************************************

create or replace trigger page_data_B4_insert
	before insert on page_data
	for each row

begin

   if :new.page_data_id is null then
      select page_data_id.nextval
      into :new.page_data_id
      from dual;
   end if;

end;
/
show error

-- ************************************************************
-- ************************************************************

create or replace trigger transparency_code_b4_update
	before update
	on transparency_codes
	for each row

begin

   :new.transparency_code := upper (:new.transparency_code);

end;
/
show error

-- ************************************************************
-- ************************************************************

create or replace trigger maint_yes_no_flags_b4
	before insert or update
	on maint_yes_no_flags
	for each row

begin

   :new.table_column_name := upper (:new.table_column_name);

end;
/
show error

-- ************************************************************
-- ************************************************************

-- ** Sync Tables -- Not used in conference instances
/*
create or replace trigger sync_data_B4_insert
	before insert on sync_data
	for each row

begin

   :new.timestamp	:= common.get_UTC_time;
   :new.table_name	:= upper (:new.table_name);
   :new.action		:= upper (:new.action);

end;
/
show error

-- **********

create or replace trigger sync_persons
	after insert or update or delete on persons
	for each row

declare

new_id	integer;
IUD	sync_data.action%TYPE;

begin

   if inserting then
      IUD := 'INSERT';
   elsif updating then
      IUD := 'UPDATE';
   elsif deleting then
       IUD := 'DELETE';
   end if;

   select sync_id_seq.nextval into new_id from dual;
   insert into sync_data (sync_id, table_name, action) values (new_id, 'PERSONS', IUD);

   insert into sync_persons values (
	new_id, nvl(:new.person_id, :old.person_id), :new.user_id, :new.password, :new.fname,
	:new.mname, :new.lname, :new.pname, :new.initials, :new.title, :new.sort_fname,
	:new.sort_mname, :new.sort_lname, :new.soundex_lname, :new.country_code, :new.date_format_code,
	:new.authorization_given, :new.update_date, :new.invitee, :new.tz_name, :new.requires_invitation,
	:new.admin_comments, :new.gender);

end sync_persons;
/
show error

-- **********

create or replace trigger sync_profiles
	after insert or update or delete on profiles
	for each row

declare

new_id	integer;
IUD	sync_data.action%TYPE;

begin

   if inserting then
      IUD := 'INSERT';
   elsif updating then
      IUD := 'UPDATE';
   elsif deleting then
       IUD := 'DELETE';
   end if;

   select sync_id_seq.nextval into new_id from dual;
   insert into sync_data (sync_id, table_name, action) values (new_id, 'PROFILES', IUD);

   insert into sync_profiles values (
	new_id, nvl (:new.profile_id, :old.profile_id), :new.person_id, :new.address_id,
	:new.email_address, :new.mail_stop, :new.po_box, :new.phone_number, :new.fax_number,
	:new.sort_order, :new.current_address, :new.update_date, :new.deleted);

end sync_profiles;
/
show error

-- **********

create or replace trigger sync_addresses
	after insert or update or delete on addresses
	for each row

declare

new_id	integer;
IUD	sync_data.action%TYPE;

begin

   if inserting then
      IUD := 'INSERT';
   elsif updating then
      IUD := 'UPDATE';
   elsif deleting then
       IUD := 'DELETE';
   end if;

   select sync_id_seq.nextval into new_id from dual;
   insert into sync_data (sync_id, table_name, action) values (new_id, 'ADDRESSES', IUD);

   insert into sync_addresses values (
	new_id, nvl (:new.address_id, :old.address_id), :new.address1, :new.address2,
	:new.address3, :new.po_box, :new.town, :new.postal_code, :new.zip_code,
	:new.country_code, :new.search_text, :new.affiliation_id, :new.update_date);

end sync_addresses;
/
show error

-- **********

create or replace trigger sync_affiliations
	after insert or update or delete on affiliations
	for each row

declare

new_id	integer;
IUD	sync_data.action%TYPE;

begin

   if inserting then
      IUD := 'INSERT';
   elsif updating then
      IUD := 'UPDATE';
   elsif deleting then
       IUD := 'DELETE';
   end if;

   select sync_id_seq.nextval into new_id from dual;
   insert into sync_data (sync_id, table_name, action) values (new_id, 'AFFILIATIONS', IUD);

   insert into sync_affiliations values (
	new_id, nvl (:new.affiliation_id, :old.affiliation_id), :new.affil_abbrev, :new.name1,
	:new.name2, :new.department, :new.web_site, :new.search_text, :new.affil_type_code,
	:new.update_date, :new.admin_comments);

end sync_affiliations;
/
show error

-- **********

create or replace trigger sync_merge_persons
	before insert on sync_merge_persons
	for each row

declare

new_id	integer;

begin

   select sync_id_seq.nextval into new_id from dual;
   :new.sync_id := new_id;
   insert into sync_data (sync_id, table_name, action) values (new_id, 'PERSONS', 'MERGE');

end sync_merge_persons;
/
show error

-- **********

create or replace trigger sync_person_memberships
	after insert or update or delete on person_memberships
	for each row

declare

new_id	integer;
IUD	sync_data.action%TYPE;

begin

   if inserting then
      IUD := 'INSERT';
   elsif updating then
      IUD := 'UPDATE';
   elsif deleting then
       IUD := 'DELETE';
   end if;

   select sync_id_seq.nextval into new_id from dual;
   insert into sync_data (sync_id, table_name, action) values (new_id, 'PERSON_MEMBERSHIPS', IUD);

   insert into sync_person_memberships values (new_id,
		nvl (:new.person_id, :old.person_id),
		nvl (:new.membership_code, :old.membership_code),
		:new.membership_number);

end sync_person_memberships;
/
show error

-- **********

create or replace trigger sync_mailing_lists
	after insert or update or delete on mailing_lists
	for each row

declare

new_id	integer;
IUD	sync_data.action%TYPE;

begin

   if inserting then
      IUD := 'INSERT';
   elsif updating then
      IUD := 'UPDATE';
   elsif deleting then
       IUD := 'DELETE';
   end if;

   select sync_id_seq.nextval into new_id from dual;
   insert into sync_data (sync_id, table_name, action) values (new_id, 'MAILING_LISTS', IUD);

   insert into sync_mailing_lists values (new_id,
	nvl (:new.person_id, :old.person_id),
	nvl (:new.conference_code, :old.conference_code) );

end sync_mailing_lists;
/
show error
*/

-- ************************************************************
-- ************************************************************

create or replace trigger abstract_sessions_b4_insupd
	before insert or update
	on abstract_sessions for each row

begin

   :new.paper_id := paper_ids.fmt_paper_seq (
			:new.session_id,
			:new.session_seq,
			:new.presentation_code);

end;
/
show error

-- ************************************************************
-- ************************************************************

create or replace trigger abstract_sessions_b4_insert
	before insert
	on abstract_sessions for each row

begin

   :new.abstract_session_id := abstract_session_seq.nextval;

   if nvl (:new.session_id, '~~~') <> nvl (:old.session_id, '~~~') OR
      nvl (:new.session_seq, 0)    <> nvl (:old.session_seq, 0) then
      :new.paper_id := paper_ids.fmt_paper_seq (:new.session_id, :new.session_seq,
						:new.presentation_code);
   end if;

end;
/
show error

-- ************************************************************
-- ************************************************************

create or replace trigger eclass_codes_b4_ins
	before insert
	on eclass_codes for each row

begin

   if :new.eclass_code is null then
      select eclass_code_seq.nextval into :new.eclass_code from dual;
   end if;

end;
/
show error

-- ************************************************************
-- ************************************************************

spool off