--	Copyright (C) 2004 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>

-- ****************************************************************************************

spool Create_Views.log

create or replace view affiliation_codes as
select	affiliation_id					affiliation_code,
	name1 || decode (department, null,null,
			' ' || department)	||
		 decode (name1, null,null,
			' ' || name2)	||
		 decode (affil_abbrev, null,null,
			' (' || affil_abbrev || ')')	affiliation_descr
from	affiliations;

create	or replace view session_codes as
select	session_id				session_code,
	session_id || ' - ' || title		session_descr,
	presentation_option_code,
	sort_order, session_date, beg_time
from	sessions;

create	or replace view am_pm_codes as
select	'AM' am_pm_code,
	'AM' am_pm_descr
from	dual
UNION
select	'PM' am_pm_code,
	'PM' am_pm_descr
from	dual;

-- ****************************************************************************************

-- Note:  The fields display, is_owner, is_primary and is_speaker make the coding
--	  in jacow.abs_authors easier

create or replace view abstract_authors as
select	/*+ INDEX (p) INDEX(abs) */
	decode (common.PC_mode$, 'Y','Proposer', 'Owner') ctype,
	1	ctype_order,
	decode (common.pc_mode$, 'Y','Proposer', 'Owner') author_code,
	decode (common.pc_mode$, 'Y','Proposer', 'Owner') author_type,
	p.person_id, common.get_user_primary_profile_id (p.person_id) profile_id,
	abs.abstract_id,
	common.get_user_name (p.person_id) full_name, soundex_lname,
	p.sort_lname, p.sort_fname, p.sort_mname, p.lname, p.fname, p.mname, p.initials iname,
	null sort_order, null on_leave,
	'Y' display, 'Y' is_owner, null is_primary, null is_speaker
from	persons		p,
	abstracts	abs
where	p.person_id	= abs.person_id
UNION
select	/*+ INDEX (p) INDEX(prof) INDEX(abs) INDEX(auth) */
	decode (common.pc_mode$, 'Y',ac.pc_mode_descr,
		decode (common.get_presentation_type(abs.abstract_id),
			'O', ac.author_descr,
			replace (ac.author_descr, 'Speaker', 'Presenter') ) ) ctype,
	ac.sort_order ctype_order,
	ac.author_code, ac.author_type,
	prof.person_id, prof.profile_id, abs.abstract_id,
	common.get_user_name (p.person_id) full_name, soundex_lname,
	p.sort_lname, p.sort_fname, p.sort_mname, p.lname, p.fname, p.mname, p.initials iname,
	auth.sort_order, auth.on_leave,
	'Y' display, null is_owner,
	decode (ac.author_code, 'PRI', 'Y', null) is_primary,
	decode (ac.author_code, 'PRE', 'Y', null) is_speaker
from	persons			p, 
	profiles		prof,
	author_codes		ac,
	authors			auth, 
	abstracts		abs
where	auth.abstract_id	= abs.abstract_id
and	ac.author_code		= auth.author_code
and	prof.profile_id		= auth.profile_id
and	p.person_id		= prof.person_id;

-- ****************************************************************************************

-- View to make it easier to use the common.show_popup_list procedure
create or replace view page_codes as
select	page_id		page_code,
	page_name	page_descr,
	available
from	pages;

-- ****************************************************************************************

create or replace view pfile_type_codes as
select	f.file_type_code, f.file_type_descr,
	pf.presentation_code, pf.editor_only, pf.always_available,
	f.upload_2nd_codes
from	file_type_codes		f,
	presentation_file_codes	pf
where	pf.file_type_code	= f.file_type_code;

-- ****************************************************************************************

create or replace view profile_attributes as
select	/*+ INDEX (PRO) */
	profile_id, pa.*
from	profiles		pro,
	person_attributes	pa
where	pa.person_id		= pro.person_id;

-- ****************************************************************************************

create or replace view application_codes as
select	application_id		application_code,
	application_name	application_descr
from	applications;

-- ****************************************************************************************

create or replace view gender_codes as
select	'M' gender_code, 'Male' gender_descr from dual
UNION
select	'F' gender_code, 'Female' gender_descr from dual;

spool off
