set define off

create or replace package body dependencies as

rpt_name$	constant varchar2(20)	:= 'Dependencies';
space$		constant varchar2(6)	:= '&nbsp;';
space2$		constant varchar2(12)	:= '&nbsp;&nbsp;';
doctype$	constant varchar2(110)	:= '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN" "http://www.w3.org/TR/REC-html40/loose.dtd">';

type	rec_type is record (
		URL		varchar2(100),
		line_count	integer);
type	recs_type is table of rec_type index by varchar2(20);
recs	recs_type;
otype	varchar2(20);

-- **************************************************************************
-- **************************************************************************

function get_ref_column(cons_name varchar2) return varchar2 is

cursor	driver is
select	column_name
from	user_cons_columns
where	constraint_name = cons_name;

driver_rec driver%ROWTYPE;

begin


  open driver;
  fetch driver into driver_rec;
  close driver;

  return (driver_rec.column_name);

end get_ref_column;

-- **************************************************************************

function col_indexed (this_table_name varchar2, this_col_name varchar2) return boolean is

cursor	driver is
select	1
from	user_ind_columns
where	table_name	= this_table_name
and	column_name	= this_col_name;

dummy		integer;
rec_found	boolean;

begin

   open driver;
   fetch driver into dummy;
   rec_found := driver%FOUND;
   close driver;

   return (rec_found);

end col_indexed;

-- **************************************************************************

procedure cons1 (cons_name varchar2) is

cursor	driver is
select	constraint_name,
	decode(constraint_type, 'C','Check Constraint on a Table',
				'P','Primay Key',
				'U','Unique Key',
				'R','Referential Integrity',
				'V','With Check Option on a View',
				constraint_type) constraint_type,
	table_name,
	status
from	user_constraints
where	r_constraint_name	= cons_name
order by
	table_name;

driver_rec	driver%ROWTYPE;
col_name	varchar2(100);
indexed		boolean;

begin

  common.public_header (rpt_name$);
  htp.header(3,cons_name);
  htp.tableOpen('BORDER');

  htp.tableRowOpen('LEFT','TOP');
  htp.tableHeader('Constraint Name');
  htp.tableHeader('Constraint Type');
  htp.tableHeader('Table');
  htp.tableHeader('Column');
  htp.tableHeader('Status');
  htp.tableRowClose;

  open driver;
  loop
     fetch driver into driver_rec;
     exit when driver%NOTFOUND;

     htp.tableRowOpen('LEFT','TOP');
     htp.tableData(driver_rec.constraint_name);
     htp.tableData(driver_rec.constraint_type);
     htp.tableData(driver_rec.table_name);

     col_name := get_ref_column(driver_rec.constraint_name);
     if col_indexed (driver_rec.table_name, col_name) then
        htp.tableData(col_name);
     else
        htp.tableData(col_name || htf.br || 'Not Indexed');
     end if;

     htp.tableData(driver_rec.status);
     htp.tableRowClose;
  end loop;
  close driver;

  htp.tableClose;
  common.footer;

end cons1;

-- **************************************************************************

procedure cons2 (cons_name varchar2) is

cursor	driver is
select	c.r_constraint_name,
	cc.table_name,
	cc.column_name
from	user_cons_columns	cc,
	user_constraints	c
where	c.constraint_name	= cons_name
and	cc.constraint_name	= c.r_constraint_name;

driver_rec  driver%ROWTYPE;

begin


  common.public_header (rpt_name$);
  htp.header(3,cons_name);
  htp.tableOpen('BORDER');

  htp.tableRowOpen('LEFT','BOTTOM');
  htp.tableHeader('References Constraint');
  htp.tableHeader('Table');
  htp.tableHeader('Column');
  htp.tableRowClose;

  open driver;
  fetch driver into driver_rec;

  htp.tableRowOpen('LEFT','BOTTOM');
  htp.tableData(driver_rec.r_constraint_name);
  htp.tableData(driver_rec.table_name);
  htp.tableData(driver_rec.column_name);
  htp.tableRowClose;

  htp.tableClose;
  close driver;
  common.footer;

end cons2;

-- **************************************************************************

function format_url(cons_name varchar2) return varchar2 is

cursor	c1 is
select	count(*)
from	user_constraints
where	r_constraint_name = cons_name;

icount  integer;

begin


   select count(*)
   into   icount
   from   user_constraints
   where  r_constraint_name = cons_name;

   if icount = 0 then

      select count(*) into icount
      from  user_cons_columns  cc,
            user_constraints   c
      where c.constraint_name     = cons_name
      and   cc.constraint_name    = c.r_constraint_name;

      if icount = 0 then
         return(cons_name);
      else
         return(htf.anchor ('dependencies.cons2?cons_name=' || cons_name, cons_name) );
      end if;
   else
      return(htf.anchor ('dependencies.cons1?cons_name=' || cons_name, cons_name));      
   end if;

end format_url;

-- **************************************************************************

procedure show_cons_columns (this_table_name varchar2,
                            this_column_name varchar2) is

cursor driver is
select constraint_name
from   user_cons_columns
where  table_name  = this_table_name
and    column_name = this_column_name
order by constraint_name;

driver_rec  driver%ROWTYPE;
temp        varchar2(2000);

begin


  temp := '';

  open driver;
  loop
     fetch driver into driver_rec;
     exit when driver%NOTFOUND;

     temp := temp || format_url (driver_rec.constraint_name) || '<BR>';
  end loop;
  close driver;

  htp.tableData(temp);

end show_cons_columns;

-- **************************************************************************

procedure show_col_comments (this_table_name varchar2, this_column_name varchar2) is

cursor	driver is
select	comments
from	user_col_comments
where	table_name	= this_table_name
and	column_name	= this_column_name;

driver_rec	driver%ROWTYPE;

begin

   open driver;
   fetch driver into driver_Rec;
   close driver;

   htp.tableData (replace(driver_rec.comments, chr(10), htf.br));

end show_col_comments;

-- **************************************************************************

procedure show_columns (obj_name varchar2, obj_type varchar2, comments_only boolean default false) is

cursor driver is
select column_name,
       data_type,
       data_length || decode(data_scale,null,null,0,null,'.' || data_scale) data_length,
       decode(nullable,'N','NOT NULL',null) nullable
from   user_tab_columns
where  table_name = obj_name
order by
	column_id;

driver_rec driver%ROWTYPE;

begin

   htp.tableOpen('BORDER');
   htp.tableRowOpen;

   htp.tableHeader('Column','left');

   if NOT comments_only then
      htp.tableHeader('Type','CENTER');
      htp.tableHeader('Length','CENTER');
      htp.tableHeader('Null', 'CENTER');
      htp.tableHeader('Constraint');
   end if;

   htp.tableHeader('Comments');
   htp.tableRowClose;

   open driver;
   loop
      fetch driver into driver_rec;
      exit when driver%NOTFOUND;

      htp.tableRowOpen('LEFT','TOP');
      htp.tableData(driver_rec.column_name);

      if NOT comments_only then
         htp.tableData(driver_rec.data_type);
         htp.tableData(driver_rec.data_length,'CENTER');
         htp.tableData(driver_rec.nullable, cnowrap=>'NOWRAP');
      end if;

      if obj_type = 'TABLE' then
         if NOT comments_only then
            show_cons_columns (obj_name, driver_rec.column_name);
         end if;
         show_col_comments (obj_name, driver_rec.column_name);
      else
         htp.tableData ('');
      end if;

      htp.tableRowClose;
   end loop;
   close driver;

   htp.tableClose;

end show_columns;

-- **************************************************************************

procedure show_indx_columns (this_index_name varchar2) is

cursor	driver is
select	column_name
from	user_ind_columns
where	index_name	= this_index_name
order by
	column_position;

driver_rec	driver%ROWTYPE;
br		varchar2(4);

begin

   open driver;
   loop
      fetch driver into driver_rec;
      exit when driver%NOTFOUND;

      htp.p (br);
      htp.p (driver_rec.column_name);
      br := '<BR>';
   end loop;
   close driver;

end show_indx_columns;

-- **************************************************************************

procedure show_indexes (obj_name varchar2) is

cursor	driver is
select	index_name,
	decode (uniqueness, 'UNIQUE','Yes', 'No') uniqueness
from	user_indexes
where	table_name	= obj_name
order by
	uniqueness, index_name;

driver_rec driver%ROWTYPE;

begin

   htp.tableOpen ('BORDER');
   htp.tableRowOpen ('LEFT', 'BOTTOM');
   htp.tableHeader ('Index Name');
   htp.tableHeader ('Unique');
   htp.tableHeader ('Columns');
   htp.tableRowClose;

   open driver;
   loop
      fetch driver into driver_rec;
      exit when driver%NOTFOUND;

      htp.tableRowOpen ('LEFT', 'TOP');
      htp.tableData (driver_rec.index_name);
      htp.tableData (driver_rec.uniqueness, 'CENTER');
      htp.p ('<TD>');
      show_indx_columns (driver_rec.index_name);
      htp.p ('</TD>');
      htp.tableRowClose;

   end loop;
   close driver;

   htp.tableClose;

end show_indexes;

-- **************************************************************************

function show_table_comment (obj_name varchar2) return varchar2 is

cursor	driver is
select	comments
from	user_tab_comments
where	table_name	= obj_name;

driver_rec	driver%ROWTYPE;

begin

   open driver;
   fetch driver into driver_rec;
   close driver;

   return replace (driver_rec.comments, chr(10), htf.br);

end show_table_comment;

-- **************************************************************************

procedure show_table (obj_name varchar2, obj_type varchar2) is

begin

   common.public_header (rpt_name$);
   htp.header (3, obj_type || ':' || space2$ || obj_name);

   if show_table_comment (obj_name) is not null then
      htp.header (4, show_table_comment (obj_name));
      htp.para;
   end if;

   show_columns(obj_name, obj_type);

   if obj_type = 'TABLE' then
      htp.para;
      show_indexes (obj_name);
   end if;

   htp.para;
   show_depend2 (obj_name);
   common.footer;

end show_table;

-- **************************************************************************

procedure show_tables is

cursor driver is
select object_name,
       object_type
from   user_objects
where  object_type in ('TABLE','VIEW')
order  by object_type, object_name;

begin


   common.public_header (rpt_name$);

   htp.header(3,'Select an object.');
   htp.tableOpen ('BORDER');

   for driver_rec in driver LOOP
      htp.tableRowOpen ('LEFT', 'TOP');
      htp.tableData (driver_rec.object_type);
      htp.tableData (htf.anchor('dependencies.show_table?obj_name=' || driver_rec.object_name ||
                '&obj_type=' || driver_rec.object_type, driver_rec.object_name) );
      htp.tableData (show_table_comment (driver_rec.object_name));
      htp.tableRowClose;
   end loop;

   htp.tableClose;
   common.footer;

end show_tables;

-- **************************************************************************

procedure gen_create_index (this_table varchar2, this_column varchar2) is

cursor	driver is
select	replace (tablespace_name, 'TABLE', 'INDEX')	tablespace
from	user_tables
where	table_name	= this_table;

driver_rec	driver%ROWTYPE;

begin

   open driver;
   fetch driver into driver_rec;
   close driver;

   htp.br;
   htp.p ('create index ' || this_table || '_' || this_column || '_IDX');
   htp.p (' on ' || this_table || ' (' || this_column || ')');
   htp.br;
   htp.p ('tablespace ' || driver_rec.tablespace || ';');

end gen_create_index;

-- **************************************************************************

procedure foreign_keys (primary_key varchar2) is

cursor	driver is
select	table_name,
	column_name
from	user_cons_columns
where	constraint_name	in
(	select	constraint_name
	from	user_constraints
	where	r_constraint_name	= primary_key)
order by
	table_name, column_name;

driver_rec	driver%ROWTYPE;

begin

   open driver;
   loop
      fetch driver into driver_rec;
      exit when driver%NOTFOUND;

      if col_indexed (driver_rec.table_name, driver_rec.column_name) then
         htp.listItem (driver_rec.table_name || '.' || driver_rec.column_name);
      else
         htp.fontOpen ('RED');
         htp.listItem (driver_rec.table_name || '.' || driver_rec.column_name);
         htp.fontClose;
         gen_create_index (driver_rec.table_name, driver_rec.column_name);
      end if;
   end loop;
   close driver;

end foreign_keys;

-- **************************************************************************

function primary_column_name (primary_key_name varchar2) return varchar2 is

cursor	driver is
select	column_name
from	user_ind_columns
where	index_name = primary_key_name;

driver_rec	driver%ROWTYPE;

begin

   open driver;
   fetch driver into driver_rec;
   close driver;
   return (driver_rec.column_name);

end primary_column_name;

-- **************************************************************************

procedure primary_keys is

cursor	driver is
select	constraint_name,
	table_name,
	status
from	user_constraints
where	constraint_type	= 'P'
order by
	table_name;

driver_rec	driver%ROWTYPE;

begin

   common.public_header ('Table Definitions - Primary/Foreign Keys');
   open driver;
   loop
      fetch driver into driver_rec;
      exit when driver%NOTFOUND;

      if driver_rec.status <> 'ENABLED' then
         htp.fontOpen ('RED');
         htp.p (driver_rec.table_name || '.' || primary_column_name(driver_rec.constraint_name));
         htp.fontClose;
      else
         htp.p (driver_rec.table_name || '.' || primary_column_name(driver_rec.constraint_name));
      end if;

      htp.olistOpen;
      foreign_keys (driver_rec.constraint_name);
      htp.olistClose;

      htp.para;
   end loop;
   close driver;
   common.footer;

end primary_keys;

-- **************************************************************************

procedure keys is

cursor	driver is
select	username
from	user_users
order by
	username;

driver_rec  driver%ROWTYPE;

begin

   dependencies.primary_keys;
   
end keys;

-- **************************************************************************

procedure show_source (this_name varchar2) is

cursor	driver is
select	*
from	user_source
where	name	= this_name
order by
	type,
	line;

tab	constant varchar2(1) := chr(9);

begin


   if common.header (rpt_name$) then
      htp.header(3, 'Source Code for: ' || this_name);

      htp.p ('<TT>');
      for driver_rec in driver LOOP
         driver_rec.text	:= replace (driver_rec.text, ' ', space$);
         driver_rec.text	:= replace (driver_rec.text, tab, space2$ || space2$);
         driver_rec.text	:= replace (driver_rec.text, chr(10), htf.br);
         htp.p (driver_rec.text);
      end loop;
      htp.p ('</TT>');

      common.footer;
   end if;

end show_source;

-- **************************************************************************

procedure show_depend1 (this_name varchar2) is

cursor	driver is
select	*
from	user_dependencies
where	name		= this_name
and	referenced_type	NOT IN ('NON-EXISTENT', 'SYNONYM')
and	NOT
	(referenced_owner	= 'SYS' and
	 referenced_name	= 'STANDARD'
	)
order by
	referenced_owner, type, name;

begin

   htp.header(3, this_name || ' References the following objects');
   htp.tableOpen ('BORDER', cattributes=>'CELLPADDING="5"');

   htp.tableRowOpen ('LEFT', 'BOTTOM');
   htp.tableHeader ('Access Type');
   htp.tableHeader ('Owner');
   htp.tableHeader ('Object');
   htp.tableRowClose;

   for driver_rec in driver LOOP
      if driver_rec.referenced_link_name is not null then
         driver_rec.referenced_link_name := ' (DB Link)';
      end if;

      htp.tableRowOpen ('LEFT', 'TOP');
      htp.tableData (driver_rec.referenced_type);
      htp.tableData (driver_rec.referenced_owner || driver_rec.referenced_link_name);
      htp.tableData (driver_rec.referenced_name);
      htp.tableRowClose;
   end loop;

   htp.tableClose;

end show_depend1;

-- **************************************************************************

procedure show_depend2 (this_name varchar2) is

cursor	driver is
select	*
from	user_dependencies
where	REFERENCED_name		= this_name
order by
	type, name;

begin

   htp.header(3, this_name || ' is Referenced by the following objects');
   htp.tableOpen ('BORDER', cattributes=>'CELLPADDING="5"');

   htp.tableRowOpen ('LEFT', 'BOTTOM');
   htp.tableHeader ('Access Type');
   htp.tableHeader ('Object');
   htp.tableRowClose;

   for driver_rec in driver LOOP
      htp.tableRowOpen ('LEFT', 'TOP');
      htp.tableData (driver_rec.type);
      htp.tableData (driver_rec.name);
      htp.tableRowClose;
   end loop;

   htp.tableClose;

end show_depend2;

-- **************************************************************************

procedure show_dependencies (this_name varchar2, this_type varchar2) is

begin

   common.public_header (rpt_name$);

   if this_type in ('PACKAGE', 'TRIGGER', 'FUNCTION', 'PROCEDURE') then
     htp.anchor('dependencies.show_source'	||
		'?this_name='||this_name,
		'View Source Code');
   end if;


   show_depend1 (this_name);
   htp.para;
   show_depend2 (this_name);
   common.footer;

end show_dependencies;

-- **************************************************************************

procedure show_objects (this_type varchar2) is

cursor	driver is
select	*
from	user_objects
where	object_type	= this_type
order by
	object_type, object_name;

begin

   common.public_header (rpt_name$);
   htp.header(3, 'Select a ' || this_type);
   htp.tableOpen;

   for driver_rec in driver LOOP
      common.show_data_row (driver_rec.object_type, 
			htf.anchor('dependencies.show_dependencies'		||
					'?this_name='||driver_rec.object_name	||
					'&this_type='||this_type, 
				driver_rec.object_name) );
   end loop;

   htp.tableClose;
   common.footer;

end show_objects;

-- **************************************************************************

procedure load_object_types is

cursor	driver is
select	distinct DECODE (object_type, 'VIEW', 'TABLE', object_type) object_type
from	user_objects
where	object_type	not in ('INDEX', 'PACKAGE BODY', 'LOB');

begin

   htp.header(3, 'Select an object type');

   for driver_rec in driver LOOP
      if driver_rec.object_type in ('TABLE', 'VIEW') then
         recs(driver_rec.object_type).URL := htf.anchor('dependencies.show_tables', driver_rec.object_type);
      else
         recs(driver_rec.object_type).URL := htf.anchor('dependencies.show_objects'	||
			'?this_type='||driver_rec.object_type, 
			driver_rec.object_type);
      end if;

      recs(driver_rec.object_type).line_count := 0;
   end loop;

   htp.para;

end load_object_types;

-- **************************************************************************

procedure load_object_counts is

cursor	driver is
select	count(1) icount,
	decode (type, 'PACKAGE BODY', 'PACKAGE', type) name
from	user_source
where	text <> chr(10)
group by type
UNION
select	count(1), 'SEQUENCE'
from	user_sequences
UNION
select	count(1), 'SYNONYM'
from	user_synonyms
UNION
select	count(1), 'TABLE'
from	user_tab_columns
UNION
select	2 * count(1), 'TABLE'
from	user_tables;

begin

   for driver_rec in driver LOOP
      recs(driver_rec.name).line_count := recs(driver_rec.name).line_count + driver_rec.icount;
   end loop;

end load_object_counts;

-- **************************************************************************

procedure load_view_counts is

cursor	driver is
select	text
from	user_views;

line_count	integer := 0;

begin

   for rec in driver LOOP
      line_count := line_count + 2 + length(rec.text) - length (replace (rec.text, chr(10), null));
   end loop;

   recs('TABLE').line_count := recs('TABLE').line_count + line_count;

end load_view_counts;

-- **************************************************************************

procedure html is

cursor	driver is
select	username
from	user_users
order by
	username;

total	integer := 0;

begin

   common.public_header (rpt_name$);
   load_object_types;
   load_object_counts;
   load_view_counts;

   htp.tableOpen ('BORDER');
   htp.tableRowOpen ('RIGHT', 'TOP');
   htp.tableData (common.space$);
   htp.tableHeader ('Lines of Code');
   htp.tableRowClose;

   otype := recs.FIRST;
   LOOP
      exit when otype is null;

      htp.tableRowOpen ('LEFT', 'TOP');
      htp.tableData (recs(otype).URL);
      htp.tableData (to_char (recs(otype).line_count, '999,990'), 'RIGHT');
      htp.tableRowClose;

      total := total + nvl(recs(otype).line_count, 0);
      otype := recs.NEXT(otype);
   end loop;

   htp.tableRowOpen ('RIGHT', 'TOP');
   htp.tableHeader ('Total Lines of Code');
   htp.tableHeader (to_char (total, '999,990'));
   htp.tableRowClose;
   htp.tableClose;

   common.footer;

end html;

-- **************************************************************************

end dependencies;
/
show error

set define on

--	Copyright (C) 2004 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>
