set define off

Create or Replace package body email as

rpt_title$	constant varchar2(50) := 'Email Application';
rpt_title2$	constant varchar2(50) := 'Email History';

primary_author$	constant varchar2(2) := 'PA';
account_owner$	constant varchar2(2) := 'AO';
speaker$	constant varchar2(3) := 'SPK';
administrator$	constant varchar2(5) := 'ADMIN';

oral_accept$	constant varchar2(1) := 'O';
poster_accept$	constant varchar2(1) := 'P';
generic$	constant varchar2(1) := 'G';
escQuote$	constant varchar2(3) := '%22';
QQ$		constant varchar2(1) := '"';
Q$		constant varchar2(1) := chr (39);


Include$	constant varchar2(1) := 'I';
Exclude$	constant varchar2(1) := 'X';

send_it$	constant varchar2(15):= 'Send the email';
view_it$	constant varchar2(15):= 'View an Example';

sql_stmt	varchar2(10000);

type	abstract_rec_def is record (
		person_id		persons.person_id%TYPE,
		primary_profile_id	profiles.profile_id%TYPE,
		spk_profile_id		profiles.profile_id%TYPE,
		paper_id		abstract_sessions.paper_id%TYPE,
		abstract_id		abstracts.abstract_id%TYPE,
		abstract_title		abstracts.title%TYPE,
		presentation_type	presentation_type_codes.presentation_type_code%TYPE,
		presentation_type_descr	presentation_type_codes.presentation_type_descr%TYPE,
		presentation_code	presentation_codes.presentation_code%TYPE,
		presentation_descr	presentation_codes.presentation_descr%TYPE,
		session_id		sessions.session_id%TYPE,
		session_title		sessions.title%TYPE,
		session_location_code	sessions.location_code%TYPE,
		session_beg_time	sessions.beg_time%TYPE,
		session_end_time	sessions.end_time%TYPE,
		session_date		varchar2(10),
		start_time		abstract_sessions.start_time%TYPE,
		duration		abstract_sessions.duration%TYPE );

abstract_rec	abstract_rec_def;

type		experts_type is varray(7) of varchar2(30);
expert_list	experts_type := experts_type (
				'Propose New Classification',
				'Propose New Presentation Type',
				'Approve New Classification',
				'Approve New Presentation Type',
				'Referee',
				'Referee Approver',
				'Session Coordinator');

-- ********************************************************************************************
-- ********************************************************************************************

function get_names (send_to common.ident_arr, name_part varchar2 default null) return varchar2 is

name	varchar2(4000) := null;
PID	integer;
plist	common.int_arr;

begin

   for i in 2..send_to.count LOOP
      if send_to(i) = primary_author$ then
	 PID	:= common.decode_value ('person_id', 'profiles', 'profile_id', abstract_rec.primary_profile_id);

      elsif send_to(i) = account_owner$ then
	 PID := abstract_rec.person_id;

      elsif send_to(i) = speaker$ then
	 PID	:= common.decode_value ('person_id', 'profiles', 'profile_id', abstract_rec.spk_profile_id);

      end if;

      if NOT plist.EXISTS(PID) then
         if name_part is null then
            name := name || ', ' || common.get_user_name (PID);
         else
            name := name || ', ' || common.get_user_name_part (PID, name_part);
         end if;
      end if;

      plist(PID) := pid;
   end loop;

   return substr (name, 3);

end get_names;

-- ********************************************************************************************
-- ********************************************************************************************

procedure init_roles_variables is

begin

   email.variables(1)	:= '[[Chair''s Email]]';
   email.variables(2)	:= '[[Editor''s Email]]';
   email.variables(3)	:= '[[Login URL]]';
   email.variables(4)	:= '[[Search URL]]';
   email.variables(5)	:= '[[Recipient''s Name]]';
   email.variables(6)	:= '[[Recipient''s First Name]]';
   email.variables(7)	:= '[[Recipient''s Last Name]]';

end init_roles_variables;

-- ********************************************************************************************

function replace_roles_variables (pid integer, text varchar2) return varchar2 is

buffer	long := text;

begin

   buffer   := replace (buffer, variables(1), common.chairs_email$);
   buffer   := replace (buffer, variables(2), common.editor_email$);
   buffer   := replace (buffer, variables(3), common.server || common.vdir$ || 'profile.html');
   buffer   := replace (buffer, variables(4), common.server || common.vdir$ || 'search.html');
   buffer   := replace (buffer, variables(5), common.get_user_name (pid) );
   buffer   := replace (buffer, variables(6), common.get_user_name_part (pid, common.fname$) );
   buffer   := replace (buffer, variables(7), common.get_user_name_part (pid, common.lname$) );

   return buffer;

end replace_roles_variables;

-- ********************************************************************************************
-- ********************************************************************************************

procedure init_svariables is

begin

   email.variables(1)	:= '[[Chair''s Email]]';
   email.variables(2)	:= '[[Editor''s Email]]';
   email.variables(3)	:= '[[Login URL]]';
   email.variables(4)	:= '[[Publications URL]]';
   email.variables(5)	:= '[[Search URL]]';
   email.variables(6)	:= '[[Session Date]]';
   email.variables(7)	:= '[[Session End Time]]';
   email.variables(8)	:= '[[Session End Time - 30]]';
   email.variables(9)	:= '[[Session ID]]';
   email.variables(10)	:= '[[Session Location]]';
   email.variables(11)	:= '[[Session Start Time]]';
   email.variables(12)	:= '[[Session Start Time - 30]]';
   email.variables(13)	:= '[[Session Title]]';
   email.variables(14)	:= '[[Recipient''s Name]]';
   email.variables(15)	:= '[[Recipient''s First Name]]';
   email.variables(16)	:= '[[Recipient''s Last Name]]';

end init_svariables;

-- ********************************************************************************************

function replace_svariables (driver_rec sessions%ROWTYPE, text varchar2) return varchar2 is

buffer		long := text;
beg_time	integer;
end_time	integer;
pid		integer;

begin

   end_time := to_char (to_date (to_char (sysdate, 'mmddyy') ||
		trim (to_char (abstract_rec.session_end_time, '0000')), 'mmddyyhh24mi') - (1/48), 'hh24mi');
   beg_time := to_char (to_date (to_char (sysdate, 'mmddyy') ||
		trim (to_char (beg_time, '0000')), 'mmddyyhh24mi') - (1/48), 'hh24mi');
   pid      := common.decode_value ('person_id', 'profiles', 'profile_id', driver_rec.chair_profile_id);

   buffer   := replace (buffer, variables(1),	common.chairs_email$);
   buffer   := replace (buffer, variables(2),	common.editor_email$);
   buffer   := replace (buffer, variables(3),	common.server || common.vdir$ || 'profile.html');
   buffer   := replace (buffer, variables(4),	common.pubs_home_page$);
   buffer   := replace (buffer, variables(5),	common.server || common.vdir$ || 'search.html');
   buffer   := replace (buffer, variables(6),	to_char (driver_rec.session_date, common.date_input_fmt_str$));
   buffer   := replace (buffer, variables(7),	common.format_time (driver_rec.end_time) );
   buffer   := replace (buffer, variables(8),	common.format_time (end_time) );
   buffer   := replace (buffer, variables(9),	driver_rec.session_id);
   buffer   := replace (buffer, variables(10),	common.decode_value ('location_descr', 'location_codes',
						'location_code', driver_rec.location_code));
   buffer   := replace (buffer, variables(11),	common.format_time (driver_rec.beg_time) );
   buffer   := replace (buffer, variables(12),	common.format_time (beg_time) );
   buffer   := replace (buffer, variables(13),	driver_rec.title);
   buffer   := replace (buffer, variables(14),	common.get_user_name (pid));
   buffer   := replace (buffer, variables(15),	common.get_user_name_part (pid, common.fname$) );
   buffer   := replace (buffer, variables(16),	common.get_user_name_part (pid, common.lname$) );


   return buffer;

end replace_svariables;

-- ********************************************************************************************
-- ********************************************************************************************

procedure init_variables is

begin

   email.variables(1)	:= '[[Chair''s Email]]';
   email.variables(2)	:= '[[Copyright URL]]';
   email.variables(3)	:= '[[Editor''s Email]]';
   email.variables(4)	:= '[[Login URL]]';
   email.variables(5)	:= '[[Paper ID]]';
   email.variables(6)	:= '[[Paper Title]]';
   email.variables(7)	:= '[[Presentation Type]]';
   email.variables(8)	:= '[[Publications URL]]';
   email.variables(9)	:= '[[Search URL]]';
   email.variables(10)	:= '[[Session Date]]';
   email.variables(11)	:= '[[Session End Time]]';
   email.variables(12)	:= '[[Session End Time - 30]]';
   email.variables(13)	:= '[[Session ID]]';
   email.variables(14)	:= '[[Session Location]]';
   email.variables(15)	:= '[[Session Start Time]]';
   email.variables(16)	:= '[[Session Start Time - 30]]';
   email.variables(17)	:= '[[Session Title]]';
   email.variables(18)	:= '[[Recipient''s Name]]';
   email.variables(19)	:= '[[Recipient''s First Name]]';
   email.variables(20)	:= '[[Recipient''s Last Name]]';
   email.variables(21)	:= '[[Presentation Start]]';
   email.variables(22)	:= '[[Presentation Duration]]';
   email.variables(23)	:= '[[Contribution ID]]';

end init_variables;

-- ********************************************************************************************

function replace_variables (message long, send_to common.ident_arr) return long is

buffer		long := message;
beg_time	integer;
end_time	integer;

begin

   end_time := to_char (to_date (to_char (sysdate, 'mmddyy') || trim (to_char (abstract_rec.session_end_time, '0000')), 'mmddyyhh24mi') - (1/48), 'hh24mi');
   beg_time := to_char (to_date (to_char (sysdate, 'mmddyy') || trim (to_char (beg_time, '0000')), 'mmddyyhh24mi') - (1/48), 'hh24mi');

   buffer   := replace (buffer, variables(1),	common.chairs_email$);
   buffer   := replace (buffer, variables(2),	common.copyright_url (abstract_rec.abstract_id) );
   buffer   := replace (buffer, variables(3),	common.editor_email$);
   buffer   := replace (buffer, variables(4),	common.server || common.vdir$ || 'profile.html');
   buffer   := replace (buffer, variables(5),	abstract_rec.paper_id);
   buffer   := replace (buffer, variables(6),	abstract_rec.abstract_title);
   buffer   := replace (buffer, variables(7),	abstract_rec.presentation_descr);
   buffer   := replace (buffer, variables(8),	common.pubs_home_page$);
   buffer   := replace (buffer, variables(9),	common.server || common.vdir$ || 'search.html');
   buffer   := replace (buffer, variables(10),	abstract_rec.session_date);
   buffer   := replace (buffer, variables(11),	common.format_time (abstract_rec.session_end_time) );
   buffer   := replace (buffer, variables(12),	common.format_time (end_time) );
   buffer   := replace (buffer, variables(13),	abstract_rec.session_id);
   buffer   := replace (buffer, variables(14),	common.decode_value ('location_descr', 'location_codes',
						'location_code', abstract_rec.session_location_code));
   buffer   := replace (buffer, variables(15),	common.format_time (abstract_rec.session_beg_time) );
   buffer   := replace (buffer, variables(16),	common.format_time (beg_time) );
   buffer   := replace (buffer, variables(17),	abstract_rec.session_title);
   buffer   := replace (buffer, variables(18),	get_names (send_to) );
   buffer   := replace (buffer, variables(19),	get_names (send_to, common.fname$) );
   buffer   := replace (buffer, variables(20),	get_names (send_to, common.lname$) );
   buffer   := replace (buffer, variables(21),	common.format_time (abstract_rec.start_time) );
   buffer   := replace (buffer, variables(22),	abstract_rec.duration );
   buffer   := replace (buffer, variables(23),	abstract_rec.abstract_id );

   return buffer;

end replace_variables;

-- ********************************************************************************************
-- ********************************************************************************************

procedure one (ID integer) is

cursor	driver is
select	*
from	mail_sent_blats
where	mail_id = ID;

cursor	c2 (email varchar2) is
select	common.get_user_name (person_id)	name,
	common.get_user_affil_list (person_id)	affil
from	profiles
where	email_address = email;

driver_rec	driver%ROWTYPE;
c2_rec		c2%ROWTYPE;
buffer		long;

begin

   if common.header ('Email History') then
      open driver;
      fetch driver into driver_rec;
      close driver;

      open c2 (driver_rec.from_addr);
      fetch c2 into c2_rec;
      close c2;

      if driver_rec.person_id is not null then
         buffer := htf.bold (common.get_user_name (driver_rec.person_id)) || htf.br ||
		   common.get_user_affil_list (driver_rec.person_id) || htf.br;
      end if;

      htp.tableOpen;
      common.show_data_row ('From',	c2_rec.name || htf.br || c2_rec.affil ||
					htf.br || driver_rec.from_addr);
      common.show_data_row ('To',		buffer || driver_rec.recipient);
      common.show_data_row ('Subject',	driver_rec.subject);
      common.show_data_row ('Message',	replace (driver_rec.message, chr(10), htf.br));
      common.show_data_row ('CC',		driver_rec.cc);
      common.show_data_row ('Sent',	common.local_time (driver_rec.create_date));
      htp.tableClose;

      common.footer;
   end if;

end one;

-- ********************************************************************************************
-- ********************************************************************************************

procedure hist (ID integer) is

cursor	driver is
select	*
from	mail_sent_blats
where	mail_id	in
(	select	mail_id
	from	email_logs
	where	email_id = ID);

cursor	C2 is
select	message
from	emails
where	email_id = ID;

buffer	long;
ltime	varchar2(100);

begin

   if common.header ('Email History') then
      open C2;
      fetch C2 into buffer;
      close C2;

      htp.tableOpen;

      htp.tableRowOpen ('LEFT', 'TOP');
      htp.tableHeader ('Subject:');
      htp.tableHeader ('Message');
      htp.tableRowClose;

      htp.tableRowOpen ('LEFT', 'TOP');
      htp.tableData (common.decode_value ('subject', 'emails', 'email_id', ID));
      htp.tableData (replace (buffer, chr(10), htf.br));
      htp.tableRowClose;

      htp.tableClose;
      htp.para;

      htp.tableOpen ('BORDER');
      htp.tableRowOpen ('LEFT', 'BOTTOM');
      htp.tableHeader ('Sent');
      htp.tableHeader ('Recipient');
      htp.tableRowClose;

      for driver_rec in driver LOOP
         buffer := null;
         if driver_rec.person_id is not null then
	    buffer := htf.bold (common.get_user_name (driver_rec.person_id)) || htf.br ||
		   common.get_user_affil_list (driver_rec.person_id) || htf.br;
         end if;

         ltime := common.local_time (driver_rec.create_date);

         htp.tableRowOpen ('LEFT', 'TOP');
         htp.tableData (htf.anchor ('email.one?ID=' || driver_rec.mail_id, ltime));
         htp.tableData (buffer || replace (driver_rec.recipient, ',', ', ') ); -- allow column to wrap
         htp.tableRowClose;
      end loop;

      htp.tableClose;
      htp.para;
      common.footer;
   end if;

end hist;

-- ********************************************************************************************
-- ********************************************************************************************

procedure history is

cursor	driver is
select	*
from	emails
order by
	timestamp desc;

begin

   if  common.header ('Email History') then
      htp.tableOpen ('Border');
      htp.tableRowOpen ('LEFT', 'BOTTOM');
      htp.tableHeader ('Date');
      htp.tableHeader ('Subject');
      htp.tableHeader ('Message');
      htp.tableRowClose;

      for driver_rec in driver LOOP
         htp.tableRowOpen ('LEFT', 'TOP');
         htp.tableData (htf.anchor ('email.hist?ID=' || driver_rec.email_id,
				common.local_time (driver_rec.timestamp)));
         htp.tableData (driver_rec.subject);
         htp.tableData (replace (driver_rec.message, chr(10), htf.br));
         htp.tableRowClose;
      end loop;

      htp.tableClose;
      common.footer;
   end if;

end history;

-- ********************************************************************************************
-- ********************************************************************************************

procedure build_sql_stmt (log_codes common.ident_arr, file_types common.ident_arr, this_classification common.ident_arr,
			this_status common.ident_arr, this_referee common.ident_arr,
			this_presentation_type common.ident_arr, this_presentation common.ident_arr,
			this_presentation_option common.ident_arr, no_files common.ident_arr,
			police common.ident_arr, qaNoPolice common.ident_arr, noregist common.ident_arr,
			affil varchar2, cid common.ident_arr, pid common.ident_arr, attrib common.ident_arr) is

action		varchar2(1);
code		varchar2(10);
attrib2		common.ident_arr;
attrib_in_list	varchar2(4000);

begin

   sql_stmt	:=
	'select		/*+ INDEX(a) */ a.person_id, '						|| common.eol$ ||
			'common.get_author_profile_id (a.abstract_id, '''			||
				common.primary_author$ || '''), '				|| common.eol$ ||
			'common.get_author_profile_id (a.abstract_id, '''			||
				common.speaker$ || '''), '					|| common.eol$ ||
			'sa.paper_id, a.abstract_id, a.title abstract_title, '			|| common.eol$ ||
			'pt.presentation_type_code, pt.presentation_type_descr, '		|| common.eol$ ||
			'pc.presentation_code, pc.presentation_descr, '				|| common.eol$ ||
			's.session_id, s.title, s.location_code, s.beg_time, s.end_time, '	|| common.eol$ ||
			'to_char(s.session_date, ''' || common.date_input_fmt_str$ || '''), '	|| common.eol$ ||
			'sa.start_time, sa.duration '						|| common.eol$ ||
		'from	 presentation_type_codes	pt, '					|| common.eol$ ||
			'presentation_codes		pc, '					|| common.eol$ ||
			'sessions			s, '					|| common.eol$ ||
			'abstract_sessions		sa, '					|| common.eol$ ||
			'abstracts			a '					|| common.eol$ ||
		'where	s.session_id(+)			= sa.session_id '			|| common.eol$ ||
		'and	sa.abstract_id(+)		= a.abstract_id '			|| common.eol$ ||
		'and	pc.presentation_code		= a.presentation_code '			|| common.eol$ ||
		'and	pt.presentation_type_code	= pc.presentation_type_code '		|| common.eol$;

   if no_files.count = 2 then
      sql_stmt	:= sql_stmt || 'and nvl (a.assignable, 0) = 0 and a.editor_id is null ';
   end if;

   if police.count = 2 then
      sql_stmt	:= sql_stmt || 'and a.abstract_id in (select abstract_id '	||
			'from	poster_logs pl '				||
			'where	manned		= ''Y'' '			||
			'and	posted		= ''Y'' '			||
			'and	satisfactory	= ''Y'' '			||
			'and	0		= '				||
			'(	select	count(1) '				||
			'	from	files	f '				||
			'	where	f.abstract_id	= pl.abstract_id) '	||
			'and	a.abstract_id		= pl.abstract_id ) ';
   end if;

   if qaNoPolice.count = 2 then
      sql_stmt	:= sql_stmt ||
			'and a.abstract_id in ('			||
				'select abstract_id '			||
				'from	poster_logs '			||
				'where	manned		= ''N'' '	||
				'or	posted		= ''N'' '	||
				'or	satisfactory	= ''N'' ) '	||
			'and	a.abstract_id in ('			||
				'select	abstract_id '			||
				'from	logs l '			||
				'where	log_code in ('''		||
					common.final_QA_passed$		||
					''', '''			||
					common.final_QA_failed$		||
					''') '				||
				'and	log_date = ('			||
				'select	max (log_date) '		||
				'from	logs '				||
				'where	abstract_id = a.abstract_id '	||
				'and	log_code = '''		||
					common.final_QA_passed$		||
					''') ) ';
   end if;

   if noregist.COUNT = 2 then
      sql_stmt	:= sql_stmt || 'and regist_maint.person_list (a.abstract_id) is null ';
   end if;

   for i in 2..log_codes.count LOOP
      action	:= substr (log_codes(i), 1, 1);
      sql_stmt	:= sql_stmt || 'and ';

      if action = include$ then
	 sql_stmt := sql_stmt || 'EXISTS ';
      elsif action = exclude$ then
	 sql_stmt := sql_stmt || 'NOT EXISTS ';
      end if;

      code	:= common.quote$ || substr (log_codes(i), 2) || common.quote$;
      sql_stmt	:= sql_stmt ||'(select 1 from logs where logs.abstract_id = a.abstract_id and log_code = ' || code || ') ' || common.eol$;
   end loop;

-- **********

   for i in 2..file_types.count LOOP
      action	:= substr (file_types(i), 1, 1);
      sql_stmt	:= sql_stmt || 'and ';

      if action = include$ then
	 sql_stmt := sql_stmt || 'EXISTS ';
      elsif action = exclude$ then
	 sql_stmt := sql_stmt || 'NOT EXISTS ';
      end if;

      code	:= common.quote$ || substr (file_types(i), 2) || common.quote$;
      sql_stmt	:= sql_stmt ||'(select 1 from files where files.abstract_id = a.abstract_id and file_type_code = ' || code || ') ' || common.eol$;
   end loop;

-- **********

   sql_stmt := sql_stmt || common.build_in_list (' and ', this_classification,	'a.classification_code');
   sql_stmt := sql_stmt || common.build_in_list (' and ', this_status,		'a.status_code');
   sql_stmt := sql_stmt || common.build_in_list (' and ', this_referee,		'a.referee_code');
   sql_stmt := sql_stmt || common.build_in_list (' and ', this_presentation,	'a.presentation_code');

   if trim (affil) is not null then
      sql_stmt := sql_stmt || 'and common.get_author_profile_id (a.abstract_id, '''	||
			common.speaker$ || ''') in '					||
		'(select profile_id from profiles  where address_id in '		||
		'(select address_id from addresses where affiliation_id in '		||
		'(select affiliation_id from affiliations '				||
			'where contains (search_text, ' || Q$ || affil || q$	|| ', 1) > 0 )))';

   end if;

   if this_presentation_option.count > 0 then
      sql_stmt := sql_stmt || common.build_in_list (' and ', this_presentation_option, 'presentation_option_code');
   end if;

   for i in 1..this_presentation_type.count LOOP
      if this_presentation_type(i) is not null then
	 sql_stmt := sql_stmt || 'and a.presentation_code in ' ||
			'(select presentation_code from presentation_codes ' ||
			'where presentation_type_code = ' || common.quote$ ||
				this_presentation_type(i) || common.quote$ || ') ';
      end if;
   end loop;

   if trim (cid(1) || cid(2)) is not null then
      sql_stmt := sql_stmt || 'and a.abstract_id between ' || cid(1) || ' and ' || nvl (cid(2), cid(1)) || ' ' || common.eol$;
   end if;

   if trim (pid(1) || pid(2)) is not null then
      sql_stmt := sql_stmt || 'and paper_id between ''' || upper (pid(1)) ||
				''' and ''' || upper (nvl (pid(2), pid(1))) || ''' ' || common.eol$;
   end if;

-- **********

   if attrib.count > 1 then
      for i in 2..attrib.COUNT LOOP
	 attrib2(i-1) := attrib(i);
      end loop;

      attrib_in_list := ' and ' || common.build_IN_list (attrib2, 'attribute_code');

      sql_stmt := sql_stmt || ' and ' 							||
			'a.abstract_id in ('						||
				'select abstract_id '					||
				'from abs_attributes '					||
				'where 1 = 1 ' || attrib_in_list 			||
			'UNION '							||
				'select abstract_id '					||
				'from authors '						||
				'where profile_id in '					||
					'(select profile_id'				||
					' from profile_attributes'			||
					' where 1 = 1 ' || attrib_in_list || ') '	||
				'and	author_type in (''' || common.speaker$ || ''','	||
					   '''' || common.primary_author$  || ''') '	||
			'UNION '							||
				'select /*+ INDEX (auth) */ abstract_id '		||
				'from	authors auth '					||
				'where	profile_id in ' 				||
					'(select profile_id'				||
					' from profile_attributes'			||
					' where 1 = 1 ' || attrib_in_list || ') '	||
			') ';
   end if;

end build_sql_stmt;

-- ********************************************************************************************

function record_email_in_QA (subject varchar2) return varchar2 is

cursor	driver is
select	nvl (to_number (max (log_code)), 0) +1
from	log_codes
where	substr (log_code, 1, 1)	between '0' and '9'
and	substr (log_code, 2, 1)	between '0' and '9';

max_code	integer;
new_code	varchar2(4);
descr		log_codes.log_descr%TYPE;

begin

   open driver;
   fetch driver into max_code;
   close driver;

   new_code	:= trim (to_char (max_code, '00'));
   descr	:= 'Email ' || to_char(sysdate, 'mm/dd/yy hh24:mi') || ' Subject: ' || subject;

   insert into log_codes (log_code, log_descr, do_not_delete)
   values (new_code, descr, 'N');

   return new_code;

end record_email_in_QA;

-- ********************************************************************************************

function get_recipients (send_to common.ident_arr) return varchar2 is

recipient	mail_blats.recipient%TYPE := null;
email_addr	mail_blats.recipient%TYPE;
person_id	integer;

begin

   for i in 2..send_to.count LOOP
      if send_to(i) = primary_author$ then
	 person_id := common.decode_value ('person_id', 'profiles', 'profile_id', abstract_rec.primary_profile_id);
      elsif send_to(i) = account_owner$ then
	 person_id := abstract_rec.person_id;
      elsif send_to(i) = speaker$ then
	 person_id := common.decode_value ('person_id', 'profiles', 'profile_id', abstract_rec.spk_profile_id);
      end if;

      email_addr := common.get_user_email (person_id);

      if recipient is null then
	 recipient := email_addr;
      elsif instr (recipient, email_addr) = 0 then
	 recipient := recipient || ',' || email_addr;
      end if;
   end loop;

   return recipient;

end get_recipients;

-- ********************************************************************************************

procedure log_recipients (send_to common.ident_arr, email_id integer, mail_id integer) is

already_logged	exception;
pragma		exception_init (already_logged, -1);

person_id	integer;

begin

   for i in 2..send_to.count LOOP
      if send_to(i) = primary_author$ then
	 person_id := common.decode_value ('person_id', 'profiles', 'profile_id', abstract_rec.primary_profile_id);

      elsif send_to(i) = account_owner$ then
	 person_id := abstract_rec.person_id;

      elsif send_to(i) = speaker$ then
	 person_id := common.decode_value ('person_id', 'profiles', 'profile_id', abstract_rec.spk_profile_id);
      end if;

      insert into email_logs (person_id, email_id, mail_id) values (person_id, email_id, mail_id);
   end loop;

exception when already_logged then null; -- exit clean; don't crash.

end log_recipients;

-- ********************************************************************************************

function log_email (log_codes common.ident_arr, this_classification common.ident_arr,
		this_status common.ident_arr, this_referee common.ident_arr, this_presentation common.ident_arr,
		this_presentation_type common.ident_arr,  this_presentation_option common.ident_arr,
		no_files common.ident_arr, police common.ident_arr, qaNoPolice common.ident_arr,
		noregist common.ident_arr, cid common.ident_arr, pid common.ident_arr,
		send_to common.ident_arr, subject varchar2,
		message varchar2) return integer is

email_id	integer;
msg_text	mail_blats.message%TYPE := trim (message) || common.eol$;
sep		varchar2(2) := ' ';

begin

   msg_text := msg_text || common.eol$ || htf.bold ('Recipient(s):');
   for i in 2..send_to.count LOOP
      if send_to(i) = primary_author$ then
	 msg_text := msg_text || sep || 'Primary Author';

      elsif send_to(i) = account_owner$ then
	 msg_text := msg_text || sep || 'Account Owner';

      elsif send_to(i) = speaker$ then
	 msg_text := msg_text || sep || 'Speaker';

      elsif send_to(i) = administrator$ then
	 msg_text := msg_text || sep || 'Administrator';

      end if;

      sep := ', ';
   end loop;

   sep		:= ' ';
   msg_text	:= msg_text || common.eol$ || htf.bold ('Includes:');
   for i in 2..log_codes.count LOOP
      if substr (log_codes(i), 1, 1) = 'I' then
	 msg_text := msg_text || common.decode_value ('log_descr', 'log_codes',
							'log_code', substr (log_codes(i), 2));
	 sep := ', ';
      end if;
   end loop;

   sep		:= ' ';
   msg_text	:= msg_text || common.eol$ || htf.bold ('Excludes:');
   for i in 2..log_codes.count LOOP
      if substr (log_codes(i), 1, 1) = 'E' then
	 msg_text := msg_text || common.decode_value ('log_descr', 'log_codes',
							'log_code', substr (log_codes(i), 2));
	 sep := ', ';
      end if;
   end loop;

   sep		:= ' ';
   msg_text	:= msg_text || common.eol$ || htf.bold ('Classifications:');
   for i in 2..this_classification.count LOOP
      msg_text := msg_text || common.decode_value ('classification_descr', 'classification_codes',
							'classification_code', this_classification(i) );
      sep := ', ';
   end loop;

   sep		:= ' ';
   msg_text	:= msg_text || common.eol$ || htf.bold ('Editor Status:');
   for i in 2..this_status.count LOOP
      msg_text := msg_text || common.decode_value ('status_descr', 'status_codes',
							'status_code', this_status(i) );
      sep := ', ';
   end loop;

   sep		:= ' ';
   msg_text	:= msg_text || common.eol$ || htf.bold ('Referee Status:');
   for i in 2..this_referee.count LOOP
      msg_text := msg_text || common.decode_value ('referee_descr', 'referee_codes',
							'referee_code', this_referee(i) );
      sep := ', ';
   end loop;

   sep		:= ' ';
   msg_text	:= msg_text || common.eol$ || htf.bold ('Presentation Type:');
   for i in 2..this_presentation_type.count LOOP
      msg_text := msg_text || common.decode_value ('presentation_type_descr', 'presentation_type_codes',
							'presentation_type_code', this_presentation_type(i) );
      sep := ', ';
   end loop;

   sep		:= ' ';
   msg_text	:= msg_text || common.eol$ || htf.bold ('Presentation:');
   for i in 2..this_presentation.count LOOP
      msg_text := msg_text || common.decode_value ('presentation_descr', 'presentation_codes',
							'presentation_code', this_presentation(i) );
      sep := ', ';
   end loop;

   sep		:= ' ';
   msg_text	:= msg_text || common.eol$ || htf.bold ('Presentation Option:');
   for i in 2..this_presentation_option.count LOOP
      msg_text := msg_text || common.decode_value ('presentation_option_descr', 'presentation_option_codes',
							'presentation_option_code', this_presentation_option(i) );
      sep := ', ';
   end loop;

   if no_files.count = 2 then
      msg_text	:= msg_text || common.eol$ || htf.bold ('No File Uploaded');
   end if;

   if police.count = 2 then
      msg_text	:= msg_text || common.eol$ || htf.bold ('Police OK; No Files');
   end if;

   if qaNoPolice.count = 2 then
      msg_text	:= msg_text || common.eol$ || htf.bold ('QA Passed; Police Failed');
   end if;

   if noregist.COUNT = 2 then
      msg_text	:= msg_text || common.eol$ || htf.bold ('No one Registered');
   end if;

   if trim (cid(1)) || trim (cid(2)) is not null then
      msg_text	:= msg_text || common.eol$ || htf.bold ('ID Range') || ' From ' ||
		nvl (trim (cid(1)), trim (cid(2)) ) || ' To ' || nvl (trim (cid(2)), trim (cid(1)));
   end if;

   if trim (pid(1)) || trim (pid(2)) is not null then
      msg_text	:= msg_text || common.eol$ || htf.bold ('Paper ID Range') || ' From ' ||
		nvl (trim (pid(1)), trim (pid(2)) ) || ' To ' || nvl (trim (pid(2)), trim (pid(1)));
   end if;

-- **********

   select email_id_seq.nextval into email_id from dual;

   insert into emails (email_id, subject, message, timestamp)
	values (email_id, trim (subject), msg_text, sysdate);

   return email_id;

end log_email;

-- ********************************************************************************************

function recipient_count (recipients varchar2) return integer is

begin

   if recipients is null then
      return 0;
   end if;

   for i in 1..length(recipients) LOOP
      if instr (recipients, ',', 1, i) = 0 then
	 return i;
      end if;
   end loop;

   return 1;

end recipient_count;

-- ********************************************************************************************

procedure step3 (log_codes common.ident_arr, file_types common.ident_arr, this_classification common.ident_arr,
		this_status common.ident_arr, this_referee common.ident_arr,
		this_presentation common.ident_arr, this_presentation_type common.ident_arr,
		this_presentation_option common.ident_arr, no_files common.ident_arr,
		police common.ident_arr, qaNoPolice common.ident_arr, noregist common.ident_arr,
		affil varchar2, cid common.ident_arr, pid common.ident_arr, send_to common.ident_arr,
		subject varchar2, message varchar2, submit varchar2, attrib common.ident_arr) is

type		abstract_cursor_type is REF CURSOR;
abstract_cursor	abstract_cursor_type;

buffer		long;
icount		integer	:= 0;
rcount		integer := 0;
rec_found	boolean;

new_subj	mail_blats.subject%TYPE;
recipient	mail_blats.recipient%TYPE;
admin_email	mail_blats.recipient%TYPE := null;
user_email$	mail_blats.recipient%TYPE;

mail_id		integer;
next_email_id	integer;

begin

   if common.header (rpt_title$) then
      init_variables;
      user_email$ := common.get_user_email (nvl (common.gbl_priv_pid, common.gbl_pid));
      build_sql_stmt (log_codes, file_types, this_classification, this_status, this_referee, this_presentation_type,
		this_presentation, this_presentation_option, no_files, police, qaNoPolice, noregist,
		affil, cid, pid, attrib);

      if submit = view_it$ then
         open abstract_cursor for sql_stmt;
         fetch abstract_cursor into abstract_rec;
         rec_found := abstract_cursor%FOUND;
         close abstract_cursor;

         if rec_found then
            new_subj := replace (subject, escQuote$, QQ$);
            new_subj := replace_variables (new_subj, send_to);

            buffer   := replace (message, escQuote$, QQ$);
            buffer   := replace_variables (buffer, send_to);
            buffer   := replace (buffer, common.eol$, htf.br);

            htp.p ('Subject: ' || new_subj);
            htp.br;
            htp.p ('Recipient: ' || get_names (send_to));
            htp.para;
            htp.p (buffer);
            htp.hr;
            htp.header (3, 'Use the browser''s back button to submit the email.');
            htp.para;
         else
            htp.header (3, 'No abstracts meet your search criteria.');
         end if;

      else

         for i in 1..send_to.count LOOP
            if send_to(i) = administrator$ then
               admin_email := common.editor_email$;
            end if;
         end loop;

--    Log the email summary for the administrator
         next_email_id := log_email (log_codes, this_classification, this_status, this_referee, this_presentation,
			this_presentation_type,  this_presentation_option, no_files, police,
			qaNoPolice, noregist, cid, pid, send_to, subject, message);

         open abstract_cursor for sql_stmt;
         LOOP
            fetch abstract_cursor into abstract_rec;
            exit when abstract_cursor%NOTFOUND;

            recipient := get_recipients (send_to);

            if recipient is not null then
               new_subj	:= replace (subject, escQuote$, QQ$);
               new_subj	:= replace_variables (new_subj, send_to);

               buffer	:= replace (message, escQuote$, QQ$);
               buffer	:= replace_variables (buffer, send_to);
               mail_id	:= send_mail2 (recipient, new_subj, buffer, user_email$, admin_email,
				pid=>abstract_rec.person_id, abs_id=>abstract_rec.abstract_id);

               icount := icount + 1;
               rcount := rcount + recipient_count (recipient);
               log_recipients (send_to, next_email_id, mail_id);
            end if;
         end loop;
         close abstract_cursor;

         if icount = 0 then
            delete from emails where email_id = next_email_id;
         end if;

         htp.header (3, icount || ' Email(s) sent to ' || rcount || ' recipient(s).');
         htp.formOpen ('email.html');
         htp.formSubmit (null, 'Send Another Email');
         htp.formClose;
      end if;

      common.footer;
   end if;

end step3;

-- ********************************************************************************************
-- ********************************************************************************************

function format_message (message varchar2, show_header boolean default true) return varchar2 is

buffer	long := message;

begin

   if show_header then
      htp.header (3, 'The email re-formatter recognized the following variable(s), '	||
			'which are displayed in red.');
      htp.hr;
      htp.para;
   end if;

   for i in 1..variables.count LOOP
      buffer := replace (buffer, variables(i),
					htf.fontOpen ('RED')	||
					htf.bold (variables(i))	||
					htf.fontClose);
   end loop;

   buffer := replace (buffer, common.eol$, htf.br);

   return buffer;

end format_message;

-- ********************************************************************************************

function valid_input (log_codes common.ident_arr, file_types common.ident_arr, subject varchar2,
			message varchar2, cid common.ident_arr, pid common.ident_arr, send_to common.ident_arr)
		return boolean is

/* Each element of log_codes/file_types is in the following format:
   Byte 1: 	Flag (include, exclude)
   Byes 2 & 3:	The log_code from log_codeS.
		There should be only one log_code
*/

valid	boolean := true;

begin

   for i in 2..log_codes.count LOOP
      for j in i+1..log_codes.count LOOP

	 if substr (log_codes(i), 2) = substr (log_codes(j), 2) then
	    htp.header (3, 'Duplicate selection for: ' ||
		common.decode_value ('log_descr', 'log_codes', 'log_code', substr (log_codes(j), 2) ) );
	    valid := false;
	 end if;

      end loop;
   end loop;

   for i in 2..file_types.count LOOP
      for j in i+1..file_types.count LOOP

	 if substr (file_types(i), 2) = substr (file_types(j), 2) then
	    htp.header (3, 'Duplicate selection for: ' ||
		common.decode_value ('file_type_descr', 'file_type_codes', 'file_type_code', substr (file_types(j), 2) ) );
	    valid := false;
	 end if;

      end loop;
   end loop;

-- **********

   if NOT valid then
      htp.header (3, 'Check at most one box per horizontal line.');
   end if;

   if subject is null then
      htp.header (3, 'Please supply a subject for your email.');
      valid := false;
   end if;

   if replace (replace (message, common.eol$, null), chr(13), null) is null then
      htp.header (3, 'Please supply a message for your email.');
      valid := false;
   end if;

   if trim (cid(1) || cid(2)) is not null then
      if NOT common.validate_number (cid(1)) or NOT common.validate_number (cid(2)) then
	 htp.header (3, 'Contribution IDs must be numeric.');
	 valid := false;

      elsif trim (cid(2)) is not null then
	 if trim (cid(1)) is null then
	    htp.header (3, 'You must specify a starting Contribution ID.');
	    valid := false;

	 elsif cid(2) < cid(1) then
	    htp.header (3, 'Starting Contribution ID must be less than Ending Contribution ID.');
	    valid := false;
	 end if;

      end if;
   end if;

   if trim (pid(1) || pid(2)) is not null then
      if trim (pid(2)) is not null then
	 if trim (pid(1)) is null then
	    htp.header (3, 'You must specify a starting Paper ID.');
	    valid := false;
	 elsif pid(2) < pid(1) then
	    htp.header (3, 'Starting Paper ID must be less than Ending Paper ID.');
	    valid := false;
	 end if;
      end if;
   end if;

   if send_to.count < 2 then
      htp.header (3, 'You must supply at least one recipient.');
      valid := false;
   end if;

   return valid;

end valid_input;

-- ********************************************************************************************

procedure step2 (log_codes common.ident_arr, file_types common.ident_arr, this_classification common.ident_arr,
		this_status common.ident_arr, this_referee common.ident_arr,
		this_presentation common.ident_arr, this_presentation_type common.ident_arr,
		this_presentation_option common.ident_arr, no_files common.ident_arr,
		police common.ident_arr, qaNoPolice common.ident_arr, noregist common.ident_arr,
		affil varchar2, send_to common.ident_arr, subject varchar2, message varchar2,
		cid common.ident_arr, pid common.ident_arr, attrib common.ident_arr) is

ipos	integer;
type	cursor_type is REF CURSOR;
driver	cursor_type;

begin

   if common.header (rpt_title$) then
      if valid_input (log_codes, file_types, subject, message, cid, pid, send_to) then

	 init_variables;
	 htp.p ( format_message ('SUBJECT: ' || subject));
	 htp.hr;
	 htp.p ( format_message (message));

	 htp.formOpen ('email.step3', 'POST');
         common.hidden_array ('log_codes',			log_codes);
         common.hidden_array ('file_types',			file_types);
	 common.hidden_array ('this_classification',		this_classification);
	 common.hidden_array ('this_status',			this_status);
	 common.hidden_array ('this_referee',			this_referee);
	 common.hidden_array ('this_presentation',		this_presentation);
	 common.hidden_array ('this_presentation_type',		this_presentation_type);
	 common.hidden_array ('this_presentation_option',	this_presentation_option);
	 common.hidden_array ('no_files',			no_files);
	 common.hidden_array ('police',				police);
	 common.hidden_array ('qaNoPolice',			qaNoPolice);
	 common.hidden_array ('noregist',			noregist);
	 htp.formHidden ('affil',				affil);
	 common.hidden_array ('cid',				cid);
	 common.hidden_array ('pid',				pid);
	 common.hidden_array ('send_to',			send_to);
	 common.hidden_array ('attrib',				attrib);

	 htp.formHidden ('subject',		replace (subject, QQ$, escQuote$));
	 htp.formHidden ('message',		replace (message, QQ$, escQuote$));

	 build_sql_stmt (log_codes, file_types, this_classification, this_status, this_referee, this_presentation_type,
			this_presentation, this_presentation_option, no_files, police, qaNoPolice,
			noregist, affil, cid, pid, attrib);
	 ipos := instr (sql_stmt, 'from');
	 sql_stmt := 'select count(1) ' || substr (sql_stmt, ipos);

	 open driver for sql_stmt;
	 fetch driver into ipos;
	 close driver;

	 htp.hr;
	 htp.bold ('Note: Clicking "' || send_it$ || '" will send ' || ipos || ' email(s).',
			cattributes=>common.red$);
	 htp.para;
	 htp.formSubmit ('submit', send_it$);
	 htp.formSubmit ('submit', view_it$);
	 htp.formClose;

      end if;

      common.footer;
   end if;

end step2;

-- ********************************************************************************************
-- ********************************************************************************************

procedure show_variables (show_warning boolean default true) is

temp		varchar2(50);

begin

   htp.hr;
   htp.header (3, 'Use the EXACT text below as substitutes values.');

   for i in 1..variables.count LOOP
      if variables(i) is not null then
         temp := replace (variables(i), '<', '&lt;');
         temp := replace (temp, '>', '&gt;');

         htp.p (temp);
         htp.br;
      end if;
   end loop;

   if show_warning then
      htp.fontOpen ('RED');
      htp.bold ('Warning: ');
      htp.p ('Do not use the SESSION substitution variables until after the session assigment process.');
      htp.fontClose;
   end if;

end show_variables;

-- ********************************************************************************************
-- ********************************************************************************************

procedure show_qa_options is

cursor	driver is
select	'log_codes' html_var_name, log_code, log_descr
from	log_codes
where	log_descr not like 'Email % Subject: %'
UNION
select	'file_types' html_var_name, file_type_code, file_type_descr
from	file_type_codes
order by 1 desc, 3;

begin

   htp.tableRowOpen ('CENTER', 'TOP');
   htp.tableData ('Check at most one box per horizontal line. Leave blank to ignore.', ccolspan=>'4');
   htp.tableRowClose;

   htp.tableRowOpen ('CENTER', 'BOTTOM');
   htp.tableHeader ('Include');
   htp.tableHeader ('Exclude');
   htp.tableRowClose;

   for rec in driver LOOP
      htp.tableRowOpen ('CENTER', 'TOP');
      htp.tableData (htf.formCheckBox (rec.html_var_name, include$ || rec.log_code));
      htp.tableData (htf.formCheckBox (rec.html_var_name, exclude$ || rec.log_code));
      htp.tableData (rec.log_descr, 'LEFT');
      htp.tableRowClose;
   end loop;

end show_qa_options;

-- ********************************************************************************************

procedure show_options2 is

colspan	integer := 2;

begin

   htp.tableOpen;

   htp.tableRowOpen ('LEFT', 'TOP');

      htp.p ('<TD ROWSPAN="3">');
         htp.bold ('Classification');
         htp.br;
         common.show_popup_list ('classification', multiple=>true);
      htp.p ('</TD>');

      htp.p ('<TD>');
         htp.bold ('Editor Status');
         htp.br;
         common.show_popup_list ('status', multiple=>true);
      htp.p ('</TD>');

      if common.referee_conf$ = 'Y' then
         colspan := 3;
         htp.p ('<TD>');
            htp.bold ('Referee Status');
            htp.br;
            common.show_popup_list ('referee', multiple=>true);
         htp.p ('</TD>');
      end if;

      htp.p ('<TD>');
	 htp.bold ('Presentation Type');
         htp.br;
         common.show_popup_list ('presentation_type',	multiple=>true);
      htp.p ('</TD>');
   htp.tableRowClose;

-- **********

   htp.tableRowOpen ('LEFT', 'TOP');
      htp.p ('<TD COLSPAN="' || colspan || '">');
         htp.bold ('Contribution Presentation Type');
         htp.br;
         common.show_popup_list ('presentation', multiple=>true);
      htp.p ('</TD>');
   htp.tableRowClose;

   htp.tableRowOpen ('LEFT', 'TOP');
      htp.p ('<TD COLSPAN="' || colspan || '">');
         htp.bold ('Presentation Option');
         htp.br;
         common.show_popup_list ('presentation_option', multiple=>true);
      htp.p ('</TD>');
   htp.tableRowClose;

-- **********

   htp.tableRowOpen ('LEFT', 'TOP');
      htp.tableData (htf.bold ('Affiliation'), 'RIGHT', cnowrap=>'NOWRAP');
      htp.tableData (htf.formText ('affil', 50, 400), ccolspan=>'3');
   htp.tableRowClose;

-- **********

   common.show_data_row ('No Files Uploaded',		htf.formHidden ('no_files', 'dummy')	||
							htf.formCheckBox ('no_files', 'YES'));

   common.show_data_row ('No One Registered',		htf.formHidden ('noregist', 'dummy')	||
							htf.formCheckBox ('noregist', 'YES'));

   common.show_data_row ('Police Ok; No Files',		htf.formHidden ('police', 'dummy')	||
							htf.formCheckBox ('police', 'YES'));

   common.show_data_row ('QA Passed; Police Failed',	htf.formHidden ('qaNoPolice', 'dummy')	||
							htf.formCheckBox ('qaNoPolice', 'YES'));

-- **********

   htp.tableRowOpen ('LEFT', 'TOP');
      htp.tableData (htf.bold ('Contribution ID Range'), 'RIGHT', cnowrap=>'NOWRAP');
      htp.tableData (htf.formText ('cid', 5, 5) || ' ' || htf.formText ('cid', 5, 5), ccolspan=>colspan );
   htp.tableRowClose;

   htp.tableRowOpen ('LEFT', 'TOP');
      htp.tableData (htf.bold ('Paper ID Range'), 'RIGHT', cnowrap=>'NOWRAP');
      htp.tableData (htf.formText ('pid', 10, 15) || ' ' || htf.formText ('pid', 10, 15), ccolspan=>colspan);
   htp.tableRowClose;

   htp.tableClose;

end show_options2;

-- ********************************************************************************************

procedure show_options is

begin

   htp.header (3, 'Select abstracts to include or exclude.');

   htp.tableOpen;

   htp.tableRowOpen ('LEFT', 'TOP');

   htp.p ('<TD>');
      htp.tableOpen;
         show_qa_options;
         htp.para;
      htp.tableClose;

      htp.bold ('Select Attributes');
      htp.para;
      htp.formHidden ('attrib', 'dummy');
      htp.tableOpen;
      jacow.abs_attrib_fields;
      htp.tableClose;
   htp.p ('</TD>');

   htp.p ('<TD>');
      show_options2;
   htp.p ('</TD>');

   htp.tableRowClose;
   htp.tableClose;
   htp.hr;

end show_options;

-- ********************************************************************************************

procedure show_recipients is

begin
      htp.formHidden ('send_to', 'dummy');
      htp.bold ('Recipient(s)');
      htp.br;

      htp.formCheckBox ('send_to', primary_author$, 'CHECKED');
      htp.p (' Submitting (Primary) Author');
      htp.br;

      htp.formCheckBox ('send_to', account_owner$);
      htp.p (' Account Owner');
      htp.br;

      htp.formCheckBox ('send_to', speaker$);
      htp.p (' Speaker');
      htp.br;

      htp.formCheckBox ('send_to', administrator$);
      htp.p (' Copy Administrator');
      htp.para;

end show_recipients;

-- ********************************************************************************************

procedure show_subj_body (show_subs_text boolean default true) is

subs_text	varchar2(90) := common.space2$ || 'Use the substitution parameters below.';

begin

   if NOT show_subs_text then
      subs_text := null;
   end if;

   htp.hr;

   htp.bold ('Subject: ');
   htp.formText ('subject', 100, 1000);

   htp.header (3, 'Enter the email message below.' || subs_text);
   htp.formTextArea ('message', 15, 75);

   htp.para;

end show_subj_body;

-- ********************************************************************************************
-- ********************************************************************************************

procedure one_email (abs_id integer) is

cursor	driver is
select	common.get_paper_id (abstract_id) paper_id, title
from	abstracts
where	abstract_id = abs_id;

driver_rec	driver%ROWTYPE;

begin

   if common.header (rpt_title$) then
      open driver;
      fetch driver into driver_rec;
      close driver;

      htp.header (3, nvl (driver_rec.paper_id, abs_id) || common.space2$ ||
				driver_rec.title);

      htp.formOpen ('email.step2', 'POST');

      htp.formHidden ('log_codes',			'');
      htp.formHidden ('file_types',			'');
      htp.formHidden ('this_classification',		'');
      htp.formHidden ('this_status',			'');
      htp.formHidden ('this_referee',			'');
      htp.formHidden ('this_presentation',		'');
      htp.formHidden ('this_presentation_type',		'');
      htp.formHidden ('this_presentation_option',	'');
      htp.formHidden ('no_files',			'');
      htp.formHidden ('police',				'');
      htp.formHidden ('qaNoPolice',			'');
      htp.formHidden ('noregist',			'');
      htp.formHidden ('cid',				abs_id);
      htp.formHidden ('cid',				abs_id);
      htp.formHidden ('pid',				'');
      htp.formHidden ('pid',				'');
      htp.formHidden ('affil',				'');
      htp.formHidden ('attrib',				'');

      show_recipients;
      show_subj_body;

      htp.formSubmit;
      htp.formClose;

      init_variables;
      show_variables;
      common.footer;
   end if;

end one_email;

-- ********************************************************************************************
-- ********************************************************************************************

procedure read (mid integer) is

cursor	C1 is
select	*
from	mail_blats
where	mail_id	= mid;

cursor	C2 is
select	*
from	mail_sent_blats
where	mail_id	= mid;

cursor	C3 (aid$ integer) is
select	1
from	abstract_authors
where	abstract_id	= aid$
and	author_code	in (common.primary_author$, common.speaker$, 'Owner')
and	person_id	= common.gbl_pid;

dummy		integer;
rec_found	boolean;
driver_rec	C1%ROWTYPE;
continue	boolean;

begin

   common.get_gbl_pid;

   open C1;
   fetch C1 into driver_rec;
   rec_found := C1%FOUND;
   close C1;

   if NOT rec_found then
      open C2;
      fetch C2 into driver_rec;
      close C2;
   end if;

   open C3 (driver_rec.abstract_id);
   fetch C3 into dummy;
   continue := C3%FOUND;
   close C3;

   continue := continue OR common.has_privs_to_page (show_err_msg=>false);

   if continue then
      common.public_header (rpt_title$);
      htp.tableOpen;

      common.show_data_row ('Date:',	common.local_time (driver_rec.create_date) );
      common.show_data_row ('From:',	driver_rec.from_addr);
      common.show_data_row ('To:',	driver_rec.recipient);
      common.show_data_row ('Subject:',	driver_rec.subject);
      common.show_data_row ('Message:',	replace (driver_rec.message, chr(10), htf.br));

      htp.tableClose;
      common.footer;
   else
      common.access_denied;
   end if;

end read;

-- ********************************************************************************************
-- ********************************************************************************************

procedure schair3 (sid common.ident_arr, subject varchar2, message varchar2) is

cursor	driver (ID varchar2) is
select	*
from	sessions
where	session_id = ID;

driver_rec	driver%ROWTYPE;
buffer		long;
icount		integer	:= 0;

new_subject	mail_blats.subject%TYPE;
admin_email	mail_blats.recipient%TYPE := common.editor_email$;
user_email$	mail_blats.recipient%TYPE;
recipient	mail_blats.recipient%TYPE;
pid		integer;

mail_id		integer;
next_email_id	integer;

begin

   if common.header (rpt_title$) then
      init_svariables;

      user_email$ := common.get_user_email (nvl (common.gbl_priv_pid, common.gbl_pid));
      if common.cc_all_email$ = 'N' then
         admin_email := null;
      end if;

-- Log the email summary for the administrator
      select email_id_seq.nextval into next_email_id from dual;
      insert into emails (email_id, subject, message, timestamp)
	values (next_email_id, trim(subject), trim(message), sysdate);

      for i in 2..sid.count LOOP
         open driver (sid(i));
         fetch driver into driver_rec;
         close driver;

         pid	:= common.decode_value ('person_id', 'profiles', 'profile_id', driver_rec.chair_profile_id);
         recipient := common.get_user_email (pid);

         if recipient is not null then
            new_subject	:= replace_svariables (driver_rec, subject);
            buffer	:= replace_svariables (driver_rec, message);
            mail_id	:= send_mail2 (recipient, new_subject, buffer, user_email$,
							admin_email, pid=>pid);
            icount	:= icount + 1;

            insert into email_logs (person_id, email_id, mail_id)
            values (pid, next_email_id, mail_id);
         end if;
      end loop;

      if icount = 0 then
         delete from emails where email_id = next_email_id;
      end if;

      htp.header (3, icount || ' Email(s) sent');
      htp.formOpen ('email.schairs');
      htp.formSubmit (null, 'Send Another Email');
      htp.formClose;

      common.footer;
   end if;

end schair3;

-- ********************************************************************************************
-- ********************************************************************************************

procedure schair2 (sid common.ident_arr, subject varchar2, message varchar2) is

dummy	common.ident_arr;
send_to	common.ident_arr;

begin

   if common.header (rpt_title$) then
      dummy(1)		:= null;
      dummy(2)		:= null;
      send_to(1)	:= ' ';
      send_to(2)	:= ' ';

      if valid_input (dummy, dummy, subject, message, dummy, dummy, send_to) then

	 if sid.count > 1 then
	    init_svariables;
	    htp.p ( format_message ('SUBJECT: ' || subject));
	    htp.hr;
	    htp.p ( format_message (message));

	    htp.formOpen ('email.schair3', 'POST');
	    common.hidden_array ('sid',	sid);
	    htp.formHidden ('subject',	replace (subject, QQ$, escQuote$));
	    htp.formHidden ('message',	replace (message, QQ$, escQuote$));

	    htp.hr;
	    htp.bold ('Note: Clicking "' || send_it$ || '" will send ' ||
			to_char (sid.count-1) || ' email(s).', cattributes=>common.red$);
	    htp.para;
	    htp.formSubmit (null, send_it$);
--	    htp.formSubmit ('submit', view_it$);
	    htp.formClose;

	 else
	    htp.header (3, 'You must select at least one recipient.', cattributes=>common.red$);

	 end if;
      end if;

      common.footer;
   end if;

end schair2;

-- ********************************************************************************************
-- ********************************************************************************************

procedure show_chairs is

cursor	driver is
select	*
from	sessions
order by
	sort_order nulls last, session_date, beg_time, end_time;

begin

   htp.header (3, 'Select the Chairs');
   htp.formHidden ('sid', '-1');

   htp.tableOpen ('BORDER');
   htp.tableRowOpen ('LEFT', 'BOTTOM');
   htp.tableData (common.space$);
   htp.tableHeader ('Session');
   htp.tableHeader ('Title');
   htp.tableHeader ('Date');
   htp.tableHeader ('Chair');
   htp.tableRowClose;

   for driver_rec in driver LOOP
      driver_rec.session_date := driver_rec.session_date + (driver_rec.beg_time/1440);

      htp.tableRowOpen ('LEFT', 'TOP');

      if driver_rec.chair_profile_id is null then
	 htp.tableData (common.space$);
      else
	 htp.tableData (htf.formCheckBox ('sid', driver_rec.session_id, 'CHECKED'));
      end if;

      htp.tableData (driver_rec.session_id);
      htp.tableData (driver_rec.title);
      htp.tableData (to_char (driver_rec.session_date, common.date_fmt_str$));

      if driver_rec.chair_profile_id is null then
	 htp.tableData (common.space$);
      else
	 htp.tableData (common.get_user_name_affiliation (driver_rec.chair_profile_id, 'Y'));
      end if;
      htp.tableRowClose;
   end loop;
   htp.tableClose;
   htp.para;

end show_chairs;

-- ********************************************************************************************

procedure schairs is

begin

   if common.header (rpt_title$) then
      htp.formOpen ('email.schair2', 'POST');

      show_chairs;
      show_subj_body;

      htp.formSubmit;
      htp.formClose;

      init_svariables;
      show_variables;
      common.footer;
   end if;

end schairs;

-- ********************************************************************************************
-- ********************************************************************************************

procedure send_role_emails (pids common.int_arr, subj varchar2, msg varchar2, email_id integer) is

mail_id		integer;
mail_rec	mail_blats%ROWTYPE;

begin

   mail_rec.from_addr := common.get_user_email (nvl (common.gbl_priv_pid, common.gbl_pid));

   if common.cc_all_email$ = 'N' then
      mail_rec.cc := null;
   end if;

   for i in 1..pids.COUNT LOOP
      mail_rec.recipient := common.get_user_email (pids(i));
      if mail_rec.recipient is not null then
	 mail_rec.subject	:= replace_roles_variables (pids(i), subj);
	 mail_rec.message	:= replace_roles_variables (pids(i), msg);

	 mail_id := send_mail2 (mail_rec.recipient, mail_rec.subject,
				mail_rec.message, mail_rec.from_addr,
				mail_rec.cc, pid=>pids(i) );
	 insert into email_logs (person_id, email_id, mail_id)
	 values (pids(i), email_id, mail_id);
      end if;
   end loop;

end send_role_emails;

-- ********************************************************************************************

function roles_email (email_id integer, this_function common.ident_arr,
			subject varchar2, message varchar2) return common.int_arr is

sql_stmt	long := null;
pids		common.int_arr;

type		cursor_type is REF CURSOR;
driver		cursor_type;

begin

   if this_function.COUNT > 1 then
      sql_stmt := 'select distinct person_id ' ||
		'from user_functions where 1 = 1 and ' ||
		common.build_in_list (this_function, 'function_code') ||
		' order by person_id';

      open driver for sql_stmt;
      fetch driver BULK COLLECT into pids;
      close driver;
   end if;

   return pids;

end roles_email;

-- ********************************************************************************************

function send_expert_email (driver_rec experts%ROWTYPE, expert common.ident_arr) return boolean is

begin

   if driver_rec.propose_reclass = 'Y' then
      for i in 2..expert.COUNT LOOP
	 if expert(i) = 'Propose New Classification' then
	    return true;
	 end if;
      end loop;
   end if;

   if driver_rec.approve_reclass = 'Y' then
      for i in 2..expert.COUNT LOOP
	 if expert(i) = 'Approve New Classification' then
	    return true;
	 end if;
      end loop;
   end if;

   if driver_rec.propose_present = 'Y' then
      for i in 2..expert.COUNT LOOP
	 if expert(i) = 'Propose New Presentation Type' then
	    return true;
	 end if;
      end loop;
   end if;

   if driver_rec.approve_present = 'Y' then
      for i in 2..expert.COUNT LOOP
	 if expert(i) = 'Approve New Presentation Type' then
	    return true;
	 end if;
      end loop;
   end if;

   if driver_rec.referee = 'Y' then
      for i in 2..expert.COUNT LOOP
	 if expert(i) = 'Referee' then
	    return true;
	 end if;
      end loop;
   end if;

   if driver_rec.referee_approver = 'Y' then
      for i in 2..expert.COUNT LOOP
	 if expert(i) = 'Referee Approver' then
	    return true;
	 end if;
      end loop;
   end if;

   if driver_rec.coordinator = 'Y' then
      for i in 2..expert.COUNT LOOP
	 if expert(i) = 'Session Coordinator' then
	    return true;
	 end if;
      end loop;
   end if;

   return false;

end send_expert_email;

-- ********************************************************************************************

function experts_email (pids IN OUT common.int_arr, next_id integer, expert common.ident_arr,
			subject varchar2, message varchar2) return common.int_arr is

cursor	driver is
select	*
from	experts	e
where	NOT exists
(
	select	1
	from	email_logs	l
	where	l.person_id	= e.person_id
	and	l.email_id	= next_id
)
order by person_id;

new_pids	common.int_arr := pids;

begin

   for driver_rec in driver LOOP
      if send_expert_email (driver_rec, expert) then
	 new_pids(new_pids.COUNT+1) := driver_rec.person_id;
      end if;
   end loop;

   return new_pids;

end experts_email;

-- ********************************************************************************************

function invitees_email (pids IN OUT common.int_arr) return common.int_arr is

cursor	driver is
select	person_id
from	persons
where	invitee = 'Y';

new_pids	common.int_arr;
add_pid		boolean;

begin

   open driver;
   fetch driver BULK COLLECT into new_pids;
   close driver;

   for i in 1..new_pids.COUNT LOOP
      add_pid := true;
      for j in 1..pids.COUNT LOOP
	 exit when new_pids(i) > pids(j);  -- pids list is sorted
	 exit when NOT add_pid;

	 if new_pids(i) = pids(j) then
	    add_pid := false;
	 end if;
      end loop;

      if add_pid then
	 pids(pids.COUNT+1) := new_pids(i);
      end if;
   end loop;

   return pids;

end invitees_email;

-- ********************************************************************************************

procedure roles3 (expert common.ident_arr, this_function common.ident_arr,
			invitees common.ident_arr, subj varchar2, msg varchar2) is

next_email_id	integer;
pids		common.int_arr;
new_msg		long := replace (msg, '%22', '"');
new_subj	long := replace (subj, '%22', '"');

begin

   if common.header (rpt_title$) then
      init_roles_variables;

-- Log the email summary for the administrator
      select email_id_seq.nextval into next_email_id from dual;
      insert into emails (email_id, subject, message, timestamp)
      values (next_email_id, new_subj, new_msg, sysdate);

      pids := roles_email (next_email_id, this_function, new_subj, new_msg);
      pids := experts_email (pids, next_email_id, expert, new_subj, new_msg);
      if invitees.COUNT > 1 then
	 pids := invitees_email (pids);
      end if;

      if pids.COUNT = 0 then
	 delete from emails where email_id = next_email_id;
      else
	 send_role_emails (pids, new_subj, new_msg, next_email_id);
      end if;

      htp.header (3, pids.COUNT || ' Email(s) sent');
      htp.formOpen ('email.roles');
      htp.formSubmit (null, 'Send Another Email');
      htp.formClose;

      common.footer;
   end if;

end roles3;

-- ********************************************************************************************

procedure roles2 (expert common.ident_arr, this_function common.ident_arr,
			invitees common.ident_arr, subject varchar2, message varchar2) is

dummy		common.ident_arr;
send_to		common.ident_arr;
pids		common.int_arr;

subj		mail_blats.subject%TYPE;
msg		mail_blats.message%TYPE;

begin

   if common.header (rpt_title$) then
      dummy(1)		:= null;
      dummy(2)		:= null;
      send_to(1)	:= ' ';
      send_to(2)	:= ' ';

      if valid_input (dummy, dummy, subject, message, dummy, dummy, send_to) then

	 if this_function.COUNT + expert.COUNT + invitees.COUNT > 1 then
	    init_roles_variables;
	    pids := roles_email (null, this_function, subject, message);
	    pids := experts_email (pids, null, expert, subject, message);
	    if invitees.COUNT > 1 then
	       pids := invitees_email (pids);
	    end if;

	    htp.header (3, 'Your selection will generate ' || pids.COUNT || ' email(s).');
	    if pids.COUNT > 0 then
	       subj	:= replace_roles_variables (pids(1), subject);
	       MSG	:= replace_roles_variables (pids(1), message);
	       msg	:= replace (msg, chr(10), htf.br);

	       htp.tableOpen;
	       common.show_data_row ('Subject',	subj);
	       common.show_data_row ('Message',	msg);
	       htp.tableClose;
	       htp.para;

	       htp.formOpen ('email.roles3');
	       common.hidden_array ('expert',	 	expert);
	       common.hidden_array ('this_function',	this_function);
	       common.hidden_array ('invitees',		invitees);

	       htp.formHidden ('subj',	replace (subject, '"', '%22') );
	       htp.formHidden ('msg',	replace (message, '"', '%22') );
	       htp.formSubmit (null, 'Send Email');
	       htp.formClose;
	    end if;

	 else
	    htp.header (3, 'You must select at least one recipient.', cattributes=>common.red$);

	 end if;
      end if;

      common.footer;
   end if;

end roles2;

-- ********************************************************************************************
-- ********************************************************************************************

procedure roles is

cursor	driver is
select	function_code, function_descr
from	function_codes
where	function_code in (select distinct function_code from user_functions)
order by function_descr;

begin

   if common.header (rpt_title$) then
      htp.formOpen ('email.roles2', 'POST');
      htp.formHidden ('expert', 'dummy');
      htp.formHidden ('this_function', 'dummy');
      htp.formHidden ('invitees', 'dummy');

      htp.header (3, 'Select Recipients');

      htp.tableOpen;
      htp.tableRowOpen ('LEFT', 'TOP');
      htp.p ('<TD>');
	 htp.bold ('Functional Roles');
	 htp.br;

	 for driver_rec in driver LOOP
	    htp.formCheckBox ('this_function', driver_rec.function_code);
	    htp.p (driver_rec.function_descr);
	    htp.br;
	 end loop;

      htp.p ('</TD>');

      htp.p ('<TD NOWRAP>');
	 htp.bold ('Experts');
	 htp.br;
	 for i in 1..expert_list.COUNT LOOP
	    htp.formCheckbox ('expert', expert_list(i));
	    htp.p (expert_list(i));
	    htp.br;
	 end loop;

	 if common.invitation_only$ = 'Y' then
	    htp.para;
	    htp.formCheckbox ('invitees', 'invitees');
	    htp.bold ('Invitees');
	 end if;
      htp.p ('</TD>');

      htp.tableRowClose;
      htp.tableClose;

      htp.para;
      show_subj_body (false);

      htp.formSubmit;
      htp.formClose;

      init_roles_variables;
      show_variables;
      common.footer;
   end if;

end roles;

-- ********************************************************************************************
-- ********************************************************************************************

procedure html is

begin

   if common.header (rpt_title$) then
      htp.formOpen ('email.step2', 'POST');
      htp.formHidden ('log_codes', 'dummy');
      htp.formHidden ('file_types', 'dummy');

      if common.referee_conf$ = 'N' then
	 htp.formHidden ('this_referee', '');
      end if;

      show_options;
      show_recipients;
      show_subj_body;

      htp.formSubmit;
      htp.formClose;

      init_variables;
      show_variables;
      common.footer;
   end if;

end html;

-- ********************************************************************************************

end email;

/
show error

set define on

--	Copyright (C) 2004-2008 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>
