
Create or Replace package body email_queue is

cursor	driver is
select	rowid, mail_blats.*
from	mail_blats
order by create_date;

driver_rec	driver%ROWTYPE;

rec_found	boolean;
process_name$	constant varchar2(50) := common.get_conf_code || ' Email Queue';
max_len$	constant integer := 998;

-- ************************************************************************************

procedure wait is

now		timestamp := systimestamp;
stoptime$	constant timestamp := now + interval '0.25' second;
failsafe	integer := 0;

begin

   LOOP
      now := systimestamp;
      exit when now >= stoptime$;

      failsafe := failsafe + 1;
      exit when failsafe > 500000;
   end loop;

end wait;

-- ************************************************************************************

function fmt_address (mail_conn in out utl_smtp.connection, buffer varchar2, is_recipient boolean default true) return varchar2 is

cursor	driver (addr$ varchar2) is
select	common.get_user_name (person_id) NAME
from	profiles
where	email_address = addr$;

driver_rec	driver%ROWTYPE;
recipient	mail_blats.recipient%TYPE;
temp		mail_blats.recipient%TYPE := buffer;
ipos		integer;
sepchar		varchar2(1) := null;


begin

   if buffer is null then
      return null;
   end if;

   ipos := instr (temp, ',');
   if ipos = 0 then
      recipient	:= temp;
      temp	:= null;
  else
      recipient	:= substr (temp, 1, ipos-1);
      temp	:= substr (temp, ipos+1);
      sepchar	:= ',';
   end if;

   open driver (recipient);
   fetch driver into driver_rec;
   close driver;

   if is_recipient then
      utl_smtp.rcpt (mail_conn, '<' || recipient || '>');
   end if;
	
   if driver_rec.name is null or driver_rec.name='' then
   	return trim ('<' || recipient || '>' || sepchar || fmt_address (mail_conn, temp, is_recipient) );
   else
   	return trim ('"' || driver_rec.name || '" <' || recipient || '>' || sepchar || fmt_address (mail_conn, temp, is_recipient) );
   end if;

end fmt_address;

-- ************************************************************************************

function next_string (message in out long) return varchar2 is

ipos	integer;
buffer	varchar2(1000);

begin

   if length (message) <= max_len$ then
      buffer	:= message;
      message	:= null;
      return buffer;
   end if;

   ipos := instr (message, utl_tcp.CRLF);

   if ipos between 1 and max_len$ then
      ipos    := ipos + length (utl_tcp.CRLF);
      buffer  := substr (message, 1, ipos-1);
      message := substr (message, ipos);
      return buffer;
   end if;

-- Find the next word break

   for i in reverse 1..max_len$ LOOP
      if substr(message, i, 1) in (common.sp32$, common.tab$) then
	 buffer  := substr (message, 1, i) || utl_tcp.CRLF;
	 message := substr (message, i+1);
	 return buffer;
      end if;
   end loop;

-- No word break in the entire 998 byte charater string

   buffer  := substr (message, 1, max_len$) || utl_tcp.CRLF;
   message := substr (message, max_len$+1);
   return buffer;

end next_string;

-- ************************************************************************************

procedure send_1_email is

mail_conn	utl_smtp.connection;
mail_reply	utl_smtp.reply;
buffer		long := null;
msg		long;
ilen		integer;
ipos		integer;

code		mail_blats_errors.error_code%TYPE;
emessage	mail_blats_errors.error_message%TYPE;
from_addr$	constant mail_blats.recipient%TYPE := nvl (common.from_email$, driver_rec.from_addr);
reply_to$	constant mail_blats.recipient%TYPE := common.editor_email$;

begin

      mail_conn := utl_smtp.open_connection (common.smtp_host$, 25);
      utl_smtp.helo (mail_conn, common.smtp_host$);
      utl_smtp.mail (mail_conn, nvl (from_addr$, reply_to$));

      buffer := 'Date: '|| to_char (common.get_UTC_time, 'dd MON yy hh24:mi:ss') || ' UT'|| utl_tcp.CRLF;
      buffer := buffer	|| 'From: '	|| fmt_address (mail_conn, from_addr$, false)	 || utl_tcp.CRLF;

      if reply_to$ is not null then
         buffer := buffer	|| 'Reply-To: '	|| fmt_address (mail_conn, reply_to$, false)	 || utl_tcp.CRLF;
      end if;

      buffer := buffer	|| 'To: '	|| fmt_address (mail_conn, driver_rec.recipient) || utl_tcp.CRLF;
      buffer := buffer	|| 'Subject: '	|| driver_rec.subject || utl_tcp.CRLF;

      if driver_rec.cc is not null then
	 buffer := buffer || 'CC: ' || fmt_address (mail_conn, driver_rec.cc) || utl_tcp.CRLF;
      end if;

      if driver_rec.bcc is not null then
	 buffer := buffer || 'BCC: ' || fmt_address (mail_conn, driver_rec.bcc) || utl_tcp.CRLF;
      end if;

      utl_smtp.open_data (mail_conn);
      utl_smtp.write_data (mail_conn, 'MIME-version: 1.0' || utl_tcp.CRLF);
      utl_smtp.write_data (mail_conn, 'Content-Type: text/plain; charset=iso-8859-15' || utl_tcp.CRLF);
      utl_smtp.write_data (mail_conn, 'Content-Transfer-Encoding: 8bit' || utl_tcp.CRLF);
      utl_smtp.write_data (mail_conn, buffer || utl_tcp.CRLF);

      msg := driver_rec.message;
      LOOP
	 exit when msg is null;
	 buffer := next_string (msg);
	 utl_smtp.write_data (mail_conn, buffer);
      end loop;

      utl_smtp.close_data (mail_conn);
      utl_smtp.quit (mail_conn);

      insert into mail_sent_blats
		(mail_id, from_addr, recipient, subject, message, cc, bcc, create_date,
		 person_id, abstract_id)
      values
		(driver_rec.mail_id, driver_rec.from_addr, driver_rec.recipient,
		 driver_rec.subject, driver_rec.message, driver_rec.cc, driver_rec.bcc,
		 driver_rec.create_date, driver_rec.person_id, driver_rec.abstract_id);

exception
   when others then
      code	:= sqlcode;
      emessage	:= sqlerrm;

      insert into mail_blats_errors
	(mail_id, from_addr, recipient, subject,
	message, cc, bcc, create_date, person_id, abstract_id, error_code, error_message)
      values
	(driver_rec.mail_id, driver_rec.from_addr, driver_rec.recipient,
	 driver_rec.subject, driver_rec.message, driver_rec.cc, driver_rec.bcc,
	 driver_rec.create_date, driver_rec.person_id, driver_rec.abstract_id, code, emessage);

end send_1_email;

-- ************************************************************************************

procedure send is

/* Under normal circumstances using commit within a procedure is not advised,
   however certain precautions are necessary to prevent sending the same messages
   multiple times. */

begin

   if NOT common.process_is_running (process_name$) then
      dbms_application_info.set_client_info (process_name$);

      LOOP
         open driver;
         fetch driver into driver_rec;
         rec_found := driver%FOUND;
         close driver;

         if NOT rec_found then
            exit; -- LOOP
         end if;

         delete from mail_blats where rowid = driver_rec.rowid;

         send_1_email;
         UTL_TCP.CLOSE_ALL_CONNECTIONS;
         wait;

         commit;
      end loop;
   end if;

end send;

-- ************************************************************************************

procedure reset is

begin

   common.set_process_name ('');

end reset;

-- ************************************************************************************

procedure reload_errors (ecode varchar2 default '%') is

begin

   insert into mail_blats
   select mail_id, from_addr, recipient, subject, message, CC, BCC, flag, create_date, person_id, abstract_id
   from   mail_blats_errors
   where  error_code = ecode;

   delete from mail_blats_errors where error_code = ecode;

end reload_errors;

-- ****************************************************************************************

end email_queue;

/
show error

set define on

--	Copyright (C) 2004-2008 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>
