set define off

create or replace package body expert as

rpt_title$		constant varchar2(35)	:= 'Classification Expert List';
propose_reclass$	constant integer	:= 1;
approve_reclass$	constant integer	:= 2;
propose_present$	constant integer	:= 3;
approve_present$	constant integer	:= 4;
referee$		constant integer	:= 5;
referee_approver$	constant integer	:= 6;
coordinator$		constant integer	:= 7;
email$			constant integer	:= 8;

-- **********************************************************************************
-- **********************************************************************************

procedure assign (aid integer, pid integer, code varchar2) is

cursor	driver is
select	common.get_paper_id (abstract_id) paper_id, title
from	abstracts
where	abstract_id = aid;

driver_rec	driver%ROWTYPE;

begin

   insert into referee_abstracts
   select pid, aid
   from   dual
   where  not exists
	(	select	1
		from	referee_abstracts
		where	person_id	= pid
		and	abstract_id	= aid);

   if SQL%rowcount = 1 then
      if common.referee_assigned$ is not null then
         open driver;
         fetch driver into driver_rec;
         close driver;

         send_mail (common.get_user_email (pid),
			common.conf_title$ || ' Referee Assignment',
			'Dear ' || common.get_user_name (pid) || common.eol2$ ||
			common.referee_assigned$ || common.eol2$ ||
			driver_rec.paper_id || ' ' || driver_rec.title || common.eol$ ||
			common.server || common.vdir$ || 'referee.html');
      end if;
   end if;

   common.redirect_URL ('expert.list_papers?pid=' || pid || '&code=' || code);

end assign;

-- **********************************************************************************
-- **********************************************************************************

procedure remove (aid integer, pid integer, code varchar2) is

begin

   delete from referee_abstracts where person_id = pid and abstract_id = aid;
   common.redirect_URL ('expert.list_papers?pid=' || pid || '&code=' || code);

end remove;

-- **********************************************************************************
-- **********************************************************************************

function get_referee_list (aid integer, code varchar2) return varchar2 is

cursor	driver is
select	person_id,
	common.get_user_name (person_id)	name,
	common.get_user_affil_abbrev_list(person_id)	affil
from	referee_abstracts
where	abstract_id = aid;

buffer	long := null;

begin

   for driver_rec in driver LOOP
      buffer := buffer || htf.br ||
		htf.anchor ('expert.remove'			||
				'?aid='  || aid			||
				'&pid='  || driver_rec.person_id ||
				'&code=' || code,
			replace (driver_rec.name, ' ', common.space$) ) || common.space$ ||
			replace (driver_rec.affil, ' ', common.space$);
   end loop;

   return substr (buffer, length(htf.br)+1);

end get_referee_list;

-- **********************************************************************************

procedure list_papers (pid integer, code varchar2) is

cursor	driver is
select	common.decode_value ('presentation_descr', 'presentation_codes',
				'presentation_code', presentation_code)		presentation_descr,
	common.decode_value ('classification_descr', 'classification_codes',
				'classification_code', classification_code)	classification_descr,
	common.get_paper_id (abstract_id) paper_id, abstract_id, title,
	common.get_user_name_affiliation (common.get_author_profile_id (abstract_id, common.primary_author$), 'Y') name
from	abstracts
where	classification_code	in
(
	select	classification_code
	from	experts
	where	person_id	= pid -- nvl (common.gbl_priv_pid, common.gbl_pid)
	and	referee		= 'Y')
order by 2, 1, 3;

begin

   if common.header (rpt_title$) then
      htp.header (3, 'Click on a Paper ID to assign the paper to ' ||
	common.get_user_name (pid) || ', ' || common.get_user_affil_abbrev_list (pid)	||
	'.  Click on a referee name to remove the referee.');

      htp.anchor ('expert.code?code=' || code, 'Return');
      htp.tableOpen ('BORDER');

      htp.tableRowOpen ('LEFT', 'BOTTOM');
      htp.tableHeader ('Paper ID');
      htp.tableHeader ('Title');
      htp.tableHeader ('Author');
      htp.tableHeader ('Referee');
      htp.tableHeader ('Presenation');
      htp.tableHeader ('Classification');
      htp.tableRowClose;

      for driver_rec in driver LOOP
         htp.tableRowOpen ('LEFT', 'TOP');
         htp.tableData (htf.anchor ('expert.assign'			||
				'?pid='  || pid 			||
				'&aid='  || driver_rec.abstract_id	||
				'&code=' || code,
			driver_rec.paper_id));
         htp.tableData (driver_rec.title);
         htp.tableData (driver_rec.name);
         htp.tableData (nvl (get_referee_list (driver_rec.abstract_id, code), common.space$));
         htp.tableData (driver_rec.presentation_descr);
         htp.tableData (driver_rec.classification_descr);
         htp.tableRowClose;
      end loop;

      htp.tableClose;
      htp.anchor ('expert.code?code=' || code, 'Return');
      common.footer;
   end if;

end list_papers;

-- **********************************************************************************
-- **********************************************************************************

procedure upd (code varchar2, pid integer, flag varchar2) is

begin

   common.get_gbl_pid;
   if common.has_privs_to_page then
      update experts
      set	propose_reclass	= decode (flag, propose_reclass$,
						decode (propose_reclass, 'Y','N', 'Y'),
							propose_reclass),
		approve_reclass	= decode (flag, approve_reclass$,
						decode (approve_reclass, 'Y','N', 'Y'),
							approve_reclass),
		propose_present	= decode (flag, propose_present$,
						decode (propose_present, 'Y','N', 'Y'),
							propose_present),
		approve_present	= decode (flag, approve_present$,
						decode (approve_present, 'Y','N', 'Y'),
							approve_present),
		referee		= decode (flag, referee$,
						decode (referee, 'Y','N', 'Y'),
							referee),
		referee_approver= decode (flag, referee_approver$,
						decode (referee_approver, 'Y','N', 'Y'),
							referee_approver),
		coordinator	= decode (flag, coordinator$,
						decode (coordinator, 'Y','N', 'Y'),
							coordinator)
      where person_id		= pid
      and   classification_code	= code;

      common.redirect_URL (common.server || common.vdir$ || 'expert.code?code=' || code);
   end if;

end upd;

-- **********************************************************************************

procedure del (code varchar2, pid integer) is

begin

   common.get_gbl_pid;
   if common.has_privs_to_page then
      delete from experts
      where person_id		= pid
      and   classification_code	= code;

      common.redirect_URL (common.server || common.vdir$ || 'expert.code?code=' || code);
   end if;

end del;

-- **********************************************************************************

procedure add (code varchar2, uid varchar2, flag common.ident_arr) is

pid$	constant integer := common.get_user_PID (uid);

type	yes_no_type is varray(8) of varchar2(1);
opts	yes_no_type := yes_no_type ('N', 'N', 'N', 'N', 'N', 'N', 'N', 'N');

begin

   if common.header (rpt_title$, test_only=>true) then
      if pid$ is null then
         common.public_header (rpt_title$);
         htp.header (3, 'No Such User ID', cattributes=>common.red$);
         common.footer;

      else
         for i in 3..flag.count LOOP
            opts (flag(i)) := 'Y';
         end loop;

         insert into experts (person_id, classification_code,
				propose_reclass, approve_reclass,
				propose_present, approve_present,
				referee, referee_approver,
				coordinator)
         select	pid$, code, opts(1), opts(2), opts(3), opts(4), opts(5), opts(6), opts(7)
         from	dual
         where	not exists
	   (	select	1
		from	experts
		where	person_id		= pid$
		and	classification_code	= code);

       if opts(email$) = 'Y' then
          if common.referee_instr$ is not null then
             send_mail (common.get_user_email (pid$), common.conf_title$ || ' Program Committee',
		common.referee_instr$, common.editor_email$, pid=>pid$);
          end if;
       end if;

       common.redirect_URL (common.server || common.vdir$ || 'expert.code?code=' || code);
      end if;
   end if;

end add;

-- **********************************************************************************

procedure show_check_box (flag integer) is

type	prompt_type is varray(8) of varchar2(60);
buffer	prompt_type := prompt_type (
			'May propose abstracts for re-classification',
			'May approve abstract re-classification proposals',
			'May propose abstracts for new presentation type',
			'May approve abstract presentation type proposed changes',
			'Referee',
			'Approve abstacts declided by a referee',
			'Session (Main Classification) Coordinator',
			'Send email message');

begin

   htp.br;
   htp.formCheckBox ('flag', flag);
   htp.p (buffer(flag));

end show_check_box;

-- **********************************************************************************

procedure show_assigned_papers (pid integer) is

begin

null;

end show_assigned_papers;

-- **********************************************************************************

procedure code (code varchar2) is

cursor	driver is
select	e.*,
	common.get_user_name (p.person_id)		name,
	common.get_user_affil_abbrev_list (p.person_id)	affil
from	persons			p,
	experts			e
where	e.classification_code	= code
and	p.person_id		= e.person_id
order by
	p.sort_lname, p.sort_fname, p.sort_mname;

class_descr	constant classification_codes.classification_descr%TYPE :=
			common.decode_value ('classification_descr', 'classification_codes',
					'classification_code', code);
URL		varchar2(200);

begin

   if common.header (rpt_title$) then
      htp.p (profile.js$);

      htp.formOpen ('maint.tab', 'POST');
      htp.formHidden ('tab_name',	'classification_codes');
      htp.formHidden ('order_by',	'classification_descr');
      htp.formSubmit (null, 'Return To The Maintenance Table');
      htp.formClose;

      htp.anchor ('javascript:newWindow(''profile.find_account'')', 'Search for an Account');
      htp.formOpen ('expert.add', 'POST', cattributes=>'NAME="SPMS"');
      htp.formHidden ('code', code);
      htp.formHidden ('flag', 'dummy');  -- no idea why the second dummy value is needed.
      htp.formHidden ('flag', 'dummy');  -- not like this in other routines.

      htp.p ('Enter the User ID of the Expert: ');
      htp.formText ('uid', 30, 30);

      show_check_box (propose_reclass$);
      show_check_box (approve_reclass$);
      show_check_box (propose_present$);
      show_check_box (approve_present$);
      show_check_box (referee$);
      show_check_box (referee_approver$);
      show_check_box (coordinator$);

      htp.br;
      htp.formSubmit (null, 'Add New Expert');
      show_check_box (email$);
      htp.anchor ('javascript:newWindow(''params.get_value?num=81'')', 'View/Edit Email Message');
      htp.formClose;
      common.form_focus ('SPMS', 'uid');

      htp.header (3, 'Authorized Experts - ' || class_descr);
      htp.tableOpen ('BORDER');

      htp.tableRowOpen ('LEFT', 'BOTTOM');
      htp.tableHeader (common.space$,	crowspan=>'2');
      htp.tableHeader ('Name',		crowspan=>'2');
      htp.tableHeader ('Coordinator',	crowspan=>'2');

      htp.tableHeader ('Reclassify',	'CENTER', ccolspan=>'2');
      htp.tableHeader ('Presentation',	'CENTER', ccolspan=>'2');
      htp.tableHeader ('Referee',	'CENTER', ccolspan=>'2');
      htp.tableHeader ('Papers',	crowspan=>'2');
      htp.tableRowClose;

      htp.tableRowOpen ('LEFT', 'BOTTOM');
      htp.tableHeader ('Propose');
      htp.tableHeader ('Approve');
      htp.tableHeader ('Propose');
      htp.tableHeader ('Approve');

      htp.tableHeader ('Standard');
      htp.tableHeader ('Approver');
      htp.tableRowClose;

      for driver_rec in driver LOOP
         htp.tableRowOpen ('CENTER', 'TOP');
         htp.tableData (htf.anchor ('expert.del?code=' || code ||
				'&pid=' || driver_rec.person_id, 'Remove'), 'LEFT' );
         htp.tableData (driver_rec.name || ', ' || driver_rec.affil, 'LEFT');

         URL := 'expert.upd?code=' || code || '&pid=' || driver_rec.person_id || '&flag=';

         htp.tableData (htf.anchor (URL || coordinator$,	common.yes_no (driver_rec.coordinator) ) );
         htp.tableData (htf.anchor (URL || propose_reclass$,	common.yes_no (driver_rec.propose_reclass) ) );
         htp.tableData (htf.anchor (URL || approve_reclass$,	common.yes_no (driver_rec.approve_reclass) ) );
         htp.tableData (htf.anchor (URL || propose_present$,	common.yes_no (driver_rec.propose_present) ) );
         htp.tableData (htf.anchor (URL || approve_present$,	common.yes_no (driver_rec.approve_present) ) );
         htp.tableData (htf.anchor (URL || referee$,		common.yes_no (driver_rec.referee) ) );
         htp.tableData (htf.anchor (URL || referee_approver$,	common.yes_no (driver_rec.referee_approver) ) );

         htp.p ('<TD>');
         htp.anchor ('expert.list_papers?pid=' || driver_rec.person_id || '&code=' || code, 'Assign Paper');
         htp.p (common.space2$);
         show_assigned_papers (driver_rec.person_id);
         htp.p ('</TD>');

         htp.tableRowClose;
      end loop;

      htp.tableClose;
   end if;
   common.footer;

end code;

-- **********************************************************************************

end expert;
/
show error

set define on
