create or replace package body invited_extract as

-- ****************************************************************************************

procedure header is

begin

   htp.prn (common.crlf$);
   reports.write ('Abstract ID');
   reports.write ('Paper ID');
   reports.write ('First');
   reports.write ('Last');
   reports.write ('Affiliation');
   reports.write ('Speaker Email');
   reports.write ('Mail Stop');
   reports.write ('Address 1');
   reports.write ('Address 2');
   reports.write ('Address 3');
   reports.write ('PO Box');
   reports.write ('Town');
   reports.write ('Postal Code');
   reports.write ('Zip Code');
   reports.write ('Country Code');
   reports.write ('Type');
   reports.write ('Title');
   reports.write ('Abstract');
   reports.write ('Main Classification');
   reports.write ('Sub Classification');
   reports.write ('Date');
   reports.write ('Time');
   reports.write ('Duration');
   reports.write ('Priority');
   reports.write ('Coordinator');
   reports.write ('Coordinator Email', true);

end header;

-- ****************************************************************************************

procedure address (addr_id integer) is

cursor	driver is
select	*
from	addresses
where	address_id = addr_id;

driver_rec	driver%ROWTYPE;
region_code	region_codes.region_code%TYPE;

begin

   open driver;
   fetch driver into driver_rec;
   close driver;

   reports.write (driver_rec.address1);
   reports.write (driver_rec.address2);
   reports.write (driver_rec.address3);
   reports.write (driver_rec.po_box);
   reports.write (driver_rec.town);
   reports.write (driver_rec.postal_code);
   reports.write (driver_rec.zip_code);
   reports.write (driver_rec.country_code);

end address;

-- ****************************************************************************************

procedure speaker (spk_id integer) is

cursor	driver is
select	pro.email_address, pro.mail_stop, pro.address_id,
	p.lname, p.fname
from	persons		p,
	profiles	pro
where	pro.profile_id	= spk_id
and	p.person_id	= pro.person_id;

driver_rec	driver%ROWTYPE;

begin

   open driver;
   fetch driver into driver_rec;
   close driver;

   reports.write (driver_rec.fname);
   reports.write (driver_rec.lname);
   reports.write (common.get_profile_affil (spk_id));
   reports.write (driver_rec.email_address);
   reports.write (driver_rec.mail_stop);
   address (driver_rec.address_id);

end speaker;

-- ****************************************************************************************

procedure coordinator (code varchar2) is

cursor	driver is
select	common.get_user_email (person_id)	email,
	common.get_user_name (person_id)	name
from	experts
where	classification_code	= code
and	coordinator		= 'Y';

begin

   for driver_rec in driver LOOP
      reports.write (driver_rec.name);
      reports.write (driver_rec.email);
   end LOOP;

end coordinator;

-- ****************************************************************************************

procedure xls is

cursor	driver is
select	a.abstract_id, sa.paper_id, sa.presentation_code,
	sa.start_time, sa.duration, a.classification_code, a.sub_class_code,
	a.title, a.description, sa.session_id,
	common.get_author_profile_id (a.abstract_id, common.speaker$) spk_profile_id
from	abstracts		a,
	abstract_sessions	sa
where	sa.presentation_code	in
	(	select	presentation_code
		from	presentation_codes
		where	presentation_type_code = common.oral$
	)
and	a.abstract_id		= sa.abstract_id
UNION
select	abstract_id, null, presentation_code, null, null,
	classification_code, sub_class_code,
	title, description, null,
	common.get_author_profile_id (a.abstract_id, common.speaker$) spk_profile_id
from	abstracts		a
where	presentation_code	in
	(	select	presentation_code
		from	presentation_codes
		where	presentation_type_code = common.oral$
	)
and	a.abstract_id	NOT IN	(	select	abstract_id
					from	abstract_sessions);

class_descr	classification_codes.classification_descr%TYPE;
sub_class_descr	sub_class_codes.sub_class_descr%TYPE;
begin

   common.get_gbl_pid;
   if common.has_privs_to_page then
      owa_util.mime_header('application/vnd.ms-excel', TRUE);
      header;

      for driver_rec in driver LOOP
         class_descr := common.decode_value ('classification_descr', 'classification_codes',
				'classification_code', driver_rec.classification_code);

         sub_class_descr := common.decode_value ('sub_class_descr', 'sub_class_codes',
				'sub_class_code', driver_rec.sub_class_code);

         reports.write (driver_rec.abstract_id);
         reports.write (driver_rec.paper_id);
         speaker (driver_rec.spk_profile_id);
         reports.write (common.decode_value ('presentation_descr', 'presentation_codes',
				   'presentation_code', driver_rec.presentation_code) );
         reports.write (driver_rec.title);
         reports.write (driver_rec.description);
         reports.write (class_descr);
         reports.write (sub_class_descr);
         reports.write (common.decode_value ('session_date',	'sessions',
				   'session_id',	driver_rec.session_id) );
         reports.write (driver_rec.start_time);
         reports.write (driver_rec.duration);
--         reports.write (common.decode_value ('priority_descr',	'priority_codes',
--				   'priority_code',	driver_rec.priority_code) );
         coordinator (driver_rec.classification_code);
         htp.prn (common.crlf$);
      end loop;

   end if;

end xls;

-- ****************************************************************************************

end invited_extract;
/
show error

--	Copyright (C) 2004 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>

-- ****************************************************************************************
