--	Copyright (C) 2004 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>

set define off

create or replace package JACoW as

   procedure html;
   procedure login;
   procedure js;

   procedure abstract_email (abstract_id integer, paper_id varchar2,
				new_title varchar2, new_description varchar2, new_classification_code varchar2,
				new_presentation_code varchar2, old_title varchar2, old_description varchar2,
				old_classification_code varchar2, old_presentation_code varchar2);
   procedure new_abstract;
   procedure withdraw_abstract (abs_id integer, refresh_opener varchar2 default 'N');
   procedure print_abstract (abs_id integer);
   procedure view_abstract (abs_id integer);
   procedure view_abstract2 (abs_id integer, show_files_flag varchar2 default 'Y');
   procedure edit_abstract (abs_id integer, verify varchar2 default 'N');
   procedure edit_abstract_editor (abs_id integer);
   procedure edit_abstract2 (abs_id integer default null, verify varchar2 default 'N');

   procedure set_editor (abs_id integer);
   procedure upd_editor (abs_id integer, pid integer);

   procedure save_title (abs_id integer, this_title varchar2);
   procedure save_abstract (abs_id integer, this_title varchar2, this_presentation varchar2,
		this_classification varchar2, this_sub_class varchar2, this_description varchar2,
		this_agency varchar2, this_footnote varchar2, primary_profile integer,
		verify varchar2 default 'N');

   procedure save_abs_editor (abs_id integer, this_presentation varchar2,
		this_title varchar2, this_classification varchar2, 
		this_sub_class varchar2, this_description varchar2, this_agency varchar2,
		this_footnote varchar2, primary_profile integer,
		attrib common.ident_arr, verify varchar2 default 'N');

   procedure abstract_saved (verify varchar2 default 'N');
   procedure abstract_saved_editor (verify varchar2 default 'N');
   procedure delete_abstract (this_id integer, refresh_opener varchar2 default 'N');
   procedure journal_abstract (this_id integer);
   procedure abs_authors (abs_id integer, verify varchar2 default 'N');

   procedure add_author (abs_id integer, pid integer);
   procedure repo_add_author  (pid varchar2, prof_id integer, abs_id integer);

   procedure select_author_profile (abs_id integer, this_pid integer, this_profile_id common.ident_arr);

   procedure save_author (abs_id integer, this_lname varchar2, this_fname varchar2,
				this_mname varchar2, this_country varchar2);

   procedure save_profile (pid varchar2, URL varchar2, abs_id integer, author_id integer,
			orig_addr_id integer, this_addr_id integer, email varchar2);

   procedure add_profile (abs_id integer, this_profile_id integer);
   procedure repo_add_profile (auth_id integer, abs_id integer, prof_id integer);

   procedure upd_authors (abs_id integer, prim_auth common.ident_arr,
				speaker common.ident_arr, owner common.ident_arr,
				rem_auth common.ident_arr, this_prof_id common.ident_arr,
				leave common.ident_arr, this_sort common.ident_arr);

   procedure select_profile2 (abs_id integer, pid integer, this_profile_id integer);
   procedure copyright (this_id integer);

   procedure file_info (abs_id integer, paper_id varchar2 default null);
   procedure upload_file (abs_id integer, paper_id varchar2, validated varchar2,
			this_file_type varchar2, this_platform varchar2, file_name varchar2,
			root_dir varchar2, return_URL varchar2);

   procedure upload_file_succeeded (pid varchar2, abs_id integer, this_file_type varchar2,
			this_platform varchar2, file_name varchar2, comments varchar2,
			paper_id varchar2);
   procedure upload_file_succeeded (abs_id integer, this_file_type varchar2,
			this_platform varchar2, file_name varchar2, comments varchar2,
			paper_id varchar2);

   procedure upload_file_failed (abs_id integer, paper_id varchar2, err_msg varchar2);

   procedure remove_file (fid integer, pid varchar2);
   procedure remove_file2 (fid integer, pid varchar2);

   procedure change_owner (abs_id integer);
   procedure change_owner2 (abs_id integer, uid varchar2);
   procedure change_owner3 (abs_id integer, new_id integer);

   procedure abs_log (abs_id integer);
   procedure validate_eclass_code (class varchar2, subclass varchar2);
   procedure show_sub_class_lookup (label varchar2, ccode varchar2, scode varchar2);
   procedure abs_attrib_fields;

   procedure user_status (abs_id integer);
   procedure user_status2 (abs_id integer);
   procedure user_status3 (abs_id integer, action varchar2, text varchar2);
   procedure assign_referee (abs_id integer, ccode varchar2);
   procedure additional_sessions (aid integer);
   procedure additional_sessions_prompts (aid integer);
   procedure additional_sessions_add (aid integer);
   procedure additional_sessions_save (aid integer, sid varchar2, seq varchar2, pcode varchar2, pflag varchar2, stime varchar2, dur varchar2, sv_URL varchar2);
   procedure additional_sessions_del (asid integer, aid integer);
   procedure additional_sessions_pri (asid integer, aid integer);
   procedure additional_sessions_edt (aid integer, asid common.ident_arr, pcode common.ident_arr, stime common.ident_arr, dur common.ident_arr, sv_URL common.ident_arr);

   procedure verify_for_upload (abs_id integer, paper_id varchar2);

   function get_eclass_code (ccode varchar2, scode varchar2) return varchar2;
   function js$ return varchar2;
   function set_label (true_txt varchar2, false_txt varchar2) return varchar2;
   function email_referee (abs_id integer, paper_id varchar2, title varchar2) return boolean;
   function additional_sessions_validate (sid varchar2, seq varchar2, pcode varchar2, stime varchar2, dur varchar2, validate_session boolean) return long;

end JACoW;
/
show error

set define on
