set define off

create or replace package body keywords as

-- *****************************************************************************************

procedure clear is

begin

   delete from abstract_keywords;

end clear;

-- *****************************************************************************************

procedure add (pid varchar2, kword varchar2) is

cursor	driver (aid integer, kcode varchar2) is
select	count(1)
from	abstract_keywords
where	abstract_id	= aid
and	keyword_code	like kcode;

icount	integer;
abs_id	constant integer := common.decode_value ('abstract_id', 'abstracts', 'paper_id', upper (pid) );
kcode	number	 := common.decode_value ('keyword_code', 'keyword_codes', 'keyword_descr', kword);

begin

   if abs_id is null then
      raise_application_error (-20006, 'No such paper ID.');
   end if;

   if kcode is null then
      select keyword_code_seq.nextval into kcode from dual;

      insert into keyword_codes
      select kcode, kword, null, conference_code
      from   control;
   end if;

-- **********

   open driver (abs_id, '%');
   fetch driver into icount;
   close driver;

   if icount >= common.max_keywords$ then
      raise_application_error (-20008, 'Maximum keyword count exceeded.');
   end if;

-- **********

   open driver (abs_id, kcode);
   fetch driver into icount;
   close driver;

   if icount > 0 then
      raise_application_error (-20009, 'Keyword already attached to paper.');
   end if;

   insert into abstract_keywords (abstract_id, keyword_code)
   values (abs_id, kcode);

end add;

-- *****************************************************************************************

procedure set_regexp (kword varchar2, regexp varchar2) is

begin

   update keyword_codes
   set	  regular_expression	= regexp
   where  keyword_descr		= kword;

   if SQL%rowcount <> 1 then
      raise_application_error (-20010, 'No such keyword.');
   end if;

end set_regexp;

-- *****************************************************************************************

end keywords;
/

show error

set define on

--	Copyright (C) 2004 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>
