-- SPMS - Scientific Programme Management System
-- see at the bottom of this file for licensing
-- and copyright information

set define off

Create or Replace package body maint as

rpt_title$		constant varchar2(20)	:= 'Table Maintenance';
primary$		constant varchar2(1)	:= 'P';
foreign$		constant varchar2(1)	:= 'R';
date$			constant varchar2(4)	:= 'DATE';
number$			constant varchar2(6)	:= 'NUMBER';
input_row_cnt$		constant integer	:= 3;
max_text_len$		constant integer	:= 40;

icount			integer		:= 1;
sql_stmt		varchar2(5000)	:= null;
col_name		common.ident_arr;
data_type		common.ident_arr;
data_length		common.int_arr;
nullable		common.ident_arr;
primary_key		common.bool_arr;
foreign_key		common.bool_arr;
yes_no			common.bool_arr;
admin$			constant boolean := common.user_is_jacow_admin;

Q$			constant varchar2(1) := chr(39);
Q2$			constant varchar2(2) := Q$ || Q$;
escQ$			constant varchar2(2) := '\' || chr(39);

type	cursor_type is REF CURSOR;
type	rec_type is record (
	var1	varchar2(4000),
	var2	varchar2(4000),
	var3	varchar2(4000),
	var4	varchar2(4000),
	var5	varchar2(4000),
	var6	varchar2(4000),
	var7	varchar2(4000),
	var8	varchar2(4000),
	var9	varchar2(4000),
	var10	varchar2(4000),
	var11	varchar2(4000),
	var12	varchar2(4000),
	var13	varchar2(4000),
	var14	varchar2(4000),
	var15	varchar2(4000),
	var16	varchar2(4000),
	var17	varchar2(4000) );
max_columns$	constant integer	:= 17;

driver_rec rec_type;

type	menu_rec_type is record (
	ilevel		integer,
	item_id		menu_items.item_id%TYPE,
	parent_item_id	menu_items.parent_item_id%TYPE,
	link_name	menu_items.link_name%TYPE,
	URL		menu_items.URL%TYPE,
	has_privs	char(1) );

type		menu_type is table of menu_rec_type index by binary_integer;
menu_recs	menu_type;

-- **********************************************************************************
-- **********************************************************************************

procedure header (rpt_title varchar2, body_open boolean default true) is

begin

   common.public_header (rpt_title, body_open=>body_open, JS_src=>common.dtree_URL$ || 'dtree.js');
   htp.p ('<link rel=stylesheet type="text/css" href="' || common.dtree_URL$ || 'dtree.css">');

end header;

-- **********************************************************************************
-- **********************************************************************************

function all_null (value common.ident_arr, beg_offset integer, cnt integer) return boolean is

end_offset$	constant integer := beg_offset + cnt - 1;

begin

   for i in beg_offset..end_offset$ LOOP
      if trim (value(i)) is not null then
	 return false;
      end if;
   end loop;

   return true;

end all_null;

-- **********************************************************************************

function fmt_col_name (col_name varchar2) return varchar2 is

new_name	user_tab_columns.column_name%TYPE := col_name;

begin

   new_name := replace (new_name, '_', ' ');
-- new_name := replace (new_name, ' CODE', null);
   new_name := replace (new_name, ' ID', null);
   new_name := initcap (new_name);

   return new_name;

end fmt_col_name;

-- **********************************************************************************
-- **********************************************************************************

function valid_value (rowcnt integer, col_name varchar2, data_type varchar2,
			nullable varchar2, value varchar2) return varchar2 is

buffer	constant varchar2(100) := 'Row: ' || rowcnt || ' Column:' || common.space2$ || fmt_col_name (col_name) || ' ';

begin

   if trim (value) is null AND nullable = 'N' then
      return buffer || 'must have a value.' || htf.br;

   elsif data_type = date$ then
      if NOT common.validate_date (value, common.date_input_fmt_str$) then
	 return buffer || 'contains an invalid date (' || value || ').	Proper date format is: ' || common.date_input_fmt_str$ || htf.br;
      end if;

   elsif data_type = number$ then
      if NOT common.validate_number (value) then
	 return buffer || 'contains an invalid number. (' || value || ')' || htf.br;
      end if;

   end if;

   return null;

end valid_value;

-- **********************************************************************************

function valid_data (col_name common.ident_arr, data_type common.ident_arr,
			nullable common.ident_arr, value common.ident_arr) return boolean is

-- last value being updated; rest are new records
updcnt$		constant integer := value.count - (input_row_cnt$ * col_name.count);

i		integer := 1;
msg		long 	:= null;
rowcnt		integer	:= 0;
continue	boolean;

begin

   LOOP
      exit when i > value.count;
      rowcnt	:= rowcnt + 1;
      continue	:= false;

      if i < updcnt$ then
	 continue := true;
      elsif all_null (value, i, col_name.count) then
	 i := i + col_name.count;  -- skip to the next row
      else
	 continue := true;
      end if;

      if continue then
	 for j in 1..col_name.count LOOP
	    msg := msg || valid_value (rowcnt, col_name(j), data_type(j), nullable(j), value(i));
	    i := i + 1;
	 end loop;
      end if;
   end loop;

   if msg is null then
      return true;
   end if;

   common.public_header (rpt_title$);
   htp.header (3, 'Data Validation Failed');
   htp.p (msg);
   htp.para;
   common.footer;

   return false;

end valid_data;

-- **********************************************************************************

function fmt_value (value varchar2, data_type varchar2) return varchar2 is

begin

   if trim (value) is null then
      return 'null';
   elsif data_type = date$ then
      return 'to_date (' || Q$ || value || Q$ || ', ' || Q$ || common.date_input_fmt_str$ || Q$ || ')';
   end if;

   return Q$ || replace (value, Q$, Q2$) || Q$;

end fmt_value;

-- **********************************************************************************

procedure save (tab_name varchar2, order_by varchar2, pkey_pos varchar2, action varchar2 default null,
		col_name common.ident_arr,
		data_type common.ident_arr, nullable common.ident_arr, value common.ident_arr) is

updcnt$		constant integer	:= value.count - (input_row_cnt$ * col_name.count);
upd_stmt$	constant varchar2(80)	:= 'update ' || tab_name || ' set ';
inp_stmt$	constant varchar2(80)	:= 'insert into ' || tab_name || ' values (';

sql_stmt	varchar2(5000);
sep		varchar2(2);
pkey_value	varchar2(4000);
i		integer := 1;
continue	boolean;

begin

   if valid_data (col_name, data_type, nullable, value) then

      LOOP
	 exit when i > value.count;
	 sql_stmt	:= null;
	 sep		:= null;
	 continue	:= false;

	 if i < updcnt$ then
	    continue := true;
	 elsif NOT all_null (value, i, col_name.count) then
	    continue := true;
	 end if;

	 if continue then
	    for j in 1..col_name.count LOOP

	       if i > updcnt$ AND tab_name NOT in ('PROGRAM_SORT_FIELDS') then
		  sql_stmt := sql_stmt || sep || fmt_value (value(i), data_type(j));
		  sep	:= ', ';

	       else
		  if j = pkey_pos then
		     pkey_value := value(i);
		  end if;

		  if j <> pkey_pos then
		     sql_stmt := sql_stmt || sep || col_name(j) || ' = ' || fmt_value (value(i), data_type(j));
		     sep := ', ';
		  end if;
	       end if;

	       i := i + 1;
	    end loop;

	 else
	    i := i + col_name.count;
	 end if;

-- **********

	 if continue then
	    if i > updcnt$ + col_name.count AND tab_name not in ('PROGRAM_SORT_FIELDS') then -- new record
	       sql_stmt	:= inp_stmt$ || sql_stmt || ')';
	    else
	       sql_stmt	:= upd_stmt$ || sql_stmt || ' where ' || col_name(pkey_pos) || ' = ' || Q$ || pkey_value || Q$;
	    end if;

	    execute immediate sql_stmt;
	 end if;

      end loop;

      common.redirect_URL ('maint.tab?tab_name=' || tab_name || '&order_by=' || order_by || '&action=' || action);
   end if;

exception
   when others then
      htp.header (3, 'An Error Occurred on the Following Statement');
      htp.p (sql_stmt);
      htp.br;

end save;

-- **********************************************************************************
-- **********************************************************************************

procedure del (tab_name varchar2, col_name varchar2, value varchar2) is

integrity_constraint	exception;
pragma			exception_init (integrity_constraint, -2292);

begin

   if tab_name = 'CLASSIFICATION_CODES' then
      delete from class_sessions
      where  classification_code = value;
   elsif tab_name = 'SUB_CLASS_CODES' then
      delete from class_sessions
      where  sub_class_code = value;
   end if;

   execute immediate	'delete from '	|| tab_name ||
			' where '	|| col_name || '=' || Q$ || value || Q$;

   common.redirect_URL ('maint.tab?tab_name=' || tab_name);

exception when integrity_constraint then
   common.public_header (rpt_title$);
   htp.header (3, 'Unable to Delete');
   htp.p ('This record is in use on one or more records in the database.');
   htp.br;
   htp.formOpen ('maint.tab', 'GET');
   htp.formHidden ('tab_name', tab_name);
   htp.formSubmit (null, 'Continue');
   htp.formClose;
   common.footer;

end del;

-- **********************************************************************************
-- **********************************************************************************

procedure show_popup_field (tab_name varchar2, offset integer, buffer varchar2) is

begin

   htp.p ('<TD>');

   if tab_name = 'REGIST_BYPASS_VARS' and col_name(offset) = 'PAGE_ID' then
      common.show_popup_list ('PAGE', buffer, form_var_name=>'value');

   elsif col_name(offset) like '%CODE' then
      common.show_popup_list (replace (col_name(offset), '_CODE', null), buffer, form_var_name=>'value');

   elsif col_name(offset) = 'SESSION_ID' then
      common.show_popup_table ('session_id', 'title', 'sessions', null, buffer, var_name=>'value');

   elsif col_name(offset) = 'USER_VALIDATE_STATUS' then
      common.show_popup_table ('status_code', 'status_descr', 'status_codes', null, buffer, var_name=>'value');

   else
      htp.p (buffer);

   end if;

   htp.p ('</TD>');

end show_popup_field;

-- **********************************************************************************

procedure show_data (tab_name varchar2, buffer varchar2, offset integer) is

view_len	integer;
max_len		integer;
buffer2		constant varchar2(4000) := replace (buffer, '"', '&quot;');

begin

   if col_name(offset) is not null then

      if primary_key(offset) then

	 if buffer is null then
	    if tab_name not in ('PROGRAM_SORT_FIELDS') then
	       htp.tableData ('New',cattributes=>'class=boldtext');
	       htp.tableData (htf.formText ('value', least (max_text_len$, data_length(offset)), data_length(offset), buffer2));
	    end if;
	 else
	    if tab_name not in ('PROGRAM_SORT_FIELDS') then
	       htp.tableData (htf.anchor ('maint.del'		||
			'?tab_name='	|| tab_name		||
			'&col_name='	|| col_name(offset)	||
			'&value='	|| buffer2, 'Delete') );
	    end if;
	    htp.tableData (buffer2 || htf.formHidden ('value', buffer2),cattributes=>'class=boldtext');
	 end if;

      elsif foreign_key(offset) then
	 show_popup_field (tab_name, offset, buffer);

      elsif yes_no(offset) then
	 htp.tableData (common.popup_yes_no ('value', buffer2));

      else
	 max_len := data_length(offset);

	 if data_type (offset) = number$ then
	    view_len	:= 5;
	 elsif data_type (offset) = date$ then
	    view_len	:= 15;
	    max_len	:= 15;
	 else
	    view_len	:= least (max_text_len$, data_length(offset));
	 end if;

	 if max_len >= 1000 then
	    htp.tableData (htf.formTextAreaOpen ('value', 5, 40) || buffer2 || htf.formTextAreaClose);
	 else
	    htp.tableData (htf.formText ('value', view_len, max_len, buffer2));
	 end if;

      end if;

   end if;

end show_data;

-- **********************************************************************************

procedure show_session_chair (SID varchar2, prof_id integer) is

pid	integer		:= null;
affil	varchar2(4000)	:= null;

begin

   if prof_id is NOT null then
      pid	:= common.decode_value ('person_id', 'profiles', 'profile_id', prof_id);
      affil	:= common.space2$ || htf.italic (common.get_profile_affil (prof_id) );
   end if;

   htp.tableData (htf.formHidden ('value', prof_id) ||
			htf.anchor ('session_maint.add_chair?sid=' || SID,
			nvl (common.get_user_name (pid), 'Add Chair'))	|| affil, cnowrap=>'NOWRAP');

end show_session_chair;

-- **********************************************************************************

procedure build_form (tab_name varchar2, order_by varchar2, action varchar2 default null) is

driver		cursor_type;
pkey_pos	integer := null;
style		varchar2(20);
istart		integer := 1;

begin

   htp.formOpen ('maint.save', 'POST');
   htp.formHidden ('tab_name', tab_name);
   htp.formHidden ('order_by', order_by);
   htp.formHidden ('action', action);

   for i in 1..col_name.count LOOP
      if col_name(i) is not null then
	 htp.formHidden ('col_name',	col_name(i));
	 htp.formHidden ('data_type',	data_type(i));
	 htp.formHidden ('nullable',	nullable(i));

	 if primary_key(i) then
	    pkey_pos := i;
	 end if;
      end if;
   end loop;

   htp.formHidden ('pkey_pos', pkey_pos);

-- **********

   htp.tableOpen ('BORDER');
   htp.tableRowOpen ('LEFT', 'BOTTOM');

   if tab_name not in ('PROGRAM_SORT_FIELDS') then
      htp.tableHeader (common.space$);
   end if;

   for i in 1..col_name.count LOOP
      if col_name(i) is not null then
	 style := null;
	 if nullable(i) = 'N' then
	    style := 'STYLE="COLOR:RED"';
	 end if;
	 htp.tableHeader (fmt_col_name (col_name(i)), cattributes=>style);
      end if;
   end loop;

   if tab_name = 'CLASSIFICATION_CODES' then
      htp.tableData (common.space$, ccolspan=>'2');
   end if;
   if tab_name = 'SUB_CLASS_CODES' then
      htp.tableData (common.space$);
   end if;

   htp.tableRowClose;

-- **********

   open driver for sql_stmt;
   LOOP
      fetch driver into driver_rec;
      exit when driver%NOTFOUND;

      htp.tableRowOpen ('LEFT', 'TOP');
      show_data (tab_name, driver_rec.var1,  1);
      show_data (tab_name, driver_rec.var2,  2);
      show_data (tab_name, driver_rec.var3,  3);
      show_data (tab_name, driver_rec.var4,  4);
      show_data (tab_name, driver_rec.var5,  5);
      show_data (tab_name, driver_rec.var6,  6);
      show_data (tab_name, driver_rec.var7,  7);
      show_data (tab_name, driver_rec.var8,  8);
      show_data (tab_name, driver_rec.var9,  9);

      if tab_name = 'SESSIONS' then
	 show_session_chair (driver_rec.var1, driver_rec.var10);
      else
	 show_data (tab_name, driver_rec.var10, 10);
      end if;

      show_data (tab_name, driver_rec.var11,  11);
      show_data (tab_name, driver_rec.var12,  12);
      show_data (tab_name, driver_rec.var13,  13);
      show_data (tab_name, driver_rec.var14,  14);
      show_data (tab_name, driver_rec.var15,  15);
      show_data (tab_name, driver_rec.var16,  16);
      show_data (tab_name, driver_rec.var17,  17);

      if tab_name = 'CLASSIFICATION_CODES' then
	 htp.tableData (htf.anchor ('expert.code?code=' || driver_rec.var1, 'Experts'));
	 htp.tableData (class_maint.session_list (class_maint.ccode$, driver_rec.var1) );
      end if;

      if tab_name = 'SUB_CLASS_CODES' then
	 htp.tableData (class_maint.session_list (class_maint.scode$, driver_rec.var1) );
      end if;

      htp.tableRowClose;
   end loop;
   close driver;

-- Empty rows for new records
   if tab_name in ('PROGRAM_SORT_FIELDS') then
      istart := input_row_cnt$ + 1;
   end if;

   for i in istart..input_row_cnt$ LOOP
      htp.tableRowOpen ('LEFT', 'TOP');
      for i in 1..9 LOOP
         show_data (tab_name, null,  i);
      end loop;

      if tab_name = 'SESSIONS' then
	 htp.tableData ('Set after saving.' || htf.formHidden ('value', null));
      else
	 show_data (tab_name, null, 10);
      end if;

      for i in 11..max_columns$ LOOP
         show_data (tab_name, null,  i);
      end loop;

      if tab_name = 'CLASSIFICATION_CODES' then
	 htp.tableData (common.space$, ccolspan=>'2');
      end if;
      if tab_name = 'SUB_CLASS_CODES' then
	 htp.tableData (common.space$);
      end if;

      htp.tableRowClose;
   end loop;

   htp.tableClose;
   htp.para;
   htp.formSubmit (null, 'Save');
   htp.formClose;

end build_form;

-- **********************************************************************************

function is_RI_key (tab_name varchar2, col_name varchar2, RI_type varchar2) return boolean is

cursor	driver is
select	1
from	user_cons_columns	cc,
	user_constraints	c
where	c.table_name		= tab_name
and	c.constraint_name	= cc.constraint_name
and	c.constraint_type	= RI_type
and	cc.table_name		= c.table_name
and	cc.column_name		= col_name;

dummy		integer;
rec_found	boolean;

begin

   open driver;
   fetch driver into dummy;
   rec_found := driver%FOUND;
   close driver;

   return rec_found;

end is_RI_key;

-- **********************************************************************************

function is_yes_no_column (tab_col_name varchar2) return boolean is

cursor	driver is
select	1
from	maint_yes_no_flags
where	table_column_name = tab_col_name;

dummy		integer;
rec_found	boolean;

begin

   open driver;
   fetch driver into dummy;
   rec_found := driver%FOUND;
   close driver;

   return rec_found;

end is_yes_no_column;

-- **********************************************************************************

procedure build_sql_stmt (tab_name varchar2, order_by varchar2 default null) is

cursor	driver is
select	*
from	user_tab_columns
where	table_name	= tab_name
order by column_id;

sep	varchar2(2)	:= null;
i	integer		:= 0;

begin

   sql_stmt := 'select ';

   for driver_rec in driver LOOP
      sql_stmt	:= sql_stmt || sep;

      if driver_rec.data_type = date$ then
	 sql_stmt := sql_stmt || 'to_char (' || driver_rec.column_name ||
				', ' || Q$ || common.date_input_fmt_str$ || Q$ || ')';
      else
	 sql_stmt := sql_stmt || driver_rec.column_name;
      end if;

      sep	:= ', ';
      i		:= i + 1;

      col_name(i)	:= driver_rec.column_name;
      data_type(i)	:= driver_rec.data_type;
      data_length(i)	:= driver_rec.data_length;
      nullable(i)	:= driver_rec.nullable;
      primary_key(i)	:= is_RI_key (tab_name, driver_rec.column_name, primary$);
      foreign_key(i)	:= is_RI_key (tab_name, driver_rec.column_name, foreign$);
      yes_no(i)		:= false;

      if is_yes_no_column (tab_name || '.' || driver_rec.column_name) then
	 yes_no(i)	:= true;
      end if;
   end loop;

   LOOP
      i := i + 1;
      exit when i > max_columns$;
      sql_stmt		:= sql_stmt || ', null';
      col_name(i)	:= null;
      data_type(i)	:= null;
      data_length(i)	:= null;
      primary_key(i)	:= false;
      foreign_key(i)	:= false;
   end loop;

   sql_stmt := sql_stmt || ' from ' || tab_name;

   if tab_name = 'ROLE_CODES' then
      sql_stmt := sql_stmt || ' where spms_flag = ''N''';
   elsif tab_name = 'PAYMENT_VENDOR_VARS' then
      sql_stmt := sql_stmt || ' where payment_vendor_code = ' || Q$ || common.cc_vendor$ || Q$;
   end if;

   if order_by is not null then
      sql_stmt := sql_stmt || ' order by ' || order_by;
   end if;

end build_sql_stmt;

-- **********************************************************************************

procedure tab (tab_name varchar2, order_by varchar2 default null, action varchar2 default null) is

URL		varchar2(200);
continue	boolean	:= true;

begin

   if common.header (rpt_title$) then
      if upper(tab_name) = 'PAYMENT_VENDOR_VARS' then
	 if common.cc_vendor$ is null then
	    htp.bold ('Note:', cattributes=>common.red$);
	    htp.p ('You must first select the Credit Card Vendor from the');
	    htp.anchor ('params.show_cats?cat=REG', 'System Parameters page.');
	    htp.p ('You may then return to this page to add vendor variables.');
	    continue := false;
	 end if;
      end if;

      if continue then
	 build_sql_stmt (upper (tab_name), order_by);
	 if (lower(tab_name) = 'reg_cat_codes'	and action = 'wizard') then
	    htp.div(calign=>'left');
	    htp.p(htf.anchor ('reg_form_maint.registration_opening_text',htf.IMG(common.prevImg$,cattributes=>'border=0 alt=Back title=Back')));
	    htp.p('&nbsp;');
	    htp.p(htf.anchor ('reg_form_maint.list_fields?cat_code=',htf.IMG(common.nextImg$,cattributes=>'border=0 alt=Next title=Next')));
	    htp.p('&nbsp;');
	    htp.p(htf.anchor('repository.html',htf.IMG(common.cancelImg$,cattributes=>'border=0 alt="Cancel" title="Cancel"'), cattributes=>'onclick=''return confirmation()'''));
	    htp.p('</div>');
	 end if;
	 htp.bold (initcap (replace (tab_name, '_', ' ')));
	 htp.bold (':' || common.space2$);
	 htp.bold (dependencies.show_table_comment (upper (tab_name)));
	 htp.para;
	 dependencies.show_columns (upper (tab_name), 'TABLE', true);
	 build_form (upper (tab_name), order_by, action);
      end if;

      common.footer;
   end if;

end tab;

-- **********************************************************************************
-- **********************************************************************************

function is_regist_item (ID integer) return boolean is

cursor	driver is
select	1
from	menu_items
where	item_id	= ID
connect by prior item_ID = parent_item_id
start with item_ID = 94;

rec_found	boolean;
dummy		integer;

begin

   open driver;
   fetch driver into dummy;
   rec_found := driver%FOUND;
   close driver;

   return rec_found;

end is_regist_item;

-- **********************************************************************************
-- **********************************************************************************

function has_privs_any_level (idx integer, ID integer) return varchar2 is

begin

   if menu_recs(idx).has_privs = 'Y' then
      return 'Y';
   end if;

   for i in 1..menu_recs.count LOOP
      if menu_recs(i).parent_item_id = ID then
         if has_privs_any_level (i, menu_recs(i).item_id) = 'Y' then
            return 'Y';
         end if;
      end if;
   end loop;

   return 'N';

end has_privs_any_level;

-- **********************************************************************************

procedure build (ID integer default 0, lvl integer default 0) is

begin

   for i in 1..menu_recs.count LOOP
      if menu_recs(i).parent_item_id = ID then
         if has_privs_any_level (i, menu_recs(i).item_id) = 'Y' then
            htp.p ('d.add(' || icount || ',' || lvl || ',' || Q$ ||
			replace (menu_recs(i).link_name, Q$, escQ$) || Q$ ||
			',' || Q$ || menu_recs(i).URL || Q$ || ');');
            icount := icount + 1;
	    build (menu_recs(i).item_id, icount-1);
         end if;
      end if;
   end loop;

end build;

-- **********************************************************************************

procedure menu (ID integer default 0) is

cursor	driver is
select	level, item_id, parent_item_id, link_name, URL,
	decode (URL, null,'N', common.has_privs_to_URL(URL)) has_privs
from	menu_items

where	nvl (repo_only, 'N')	like decode (common.repo_db$, null,'N', '%')
connect by prior item_id = parent_item_id
start with parent_item_id = 0
order siblings by sortorder, link_name;

link_name$	menu_items.link_name%TYPE;

begin

   open driver;
   fetch driver BULK COLLECT into menu_recs;
   close driver;
/*
for i in 1..menu_recs.count LOOP
   for j in 1..menu_recs(i).ilevel loop
      htp.p ('&mdash;');
   end loop;

   htp.p (menu_recs(i).has_privs		|| ' ' ||
		menu_recs(i).item_id		|| ' ' ||
		menu_recs(i).parent_item_id	|| ' ' ||
		has_privs_any_level(i, menu_recs(i).item_id)	|| ' ' ||
		menu_recs(i).link_name		|| ' ' ||
		menu_recs(i).url);
   htp.br;
end loop;
htp.hr;
*/
   link_name$ := common.decode_value ('link_name', 'menu_items', 'item_id', id);
   link_name$ := replace (link_name$, Q$, '\' || Q$);

   htp.p ('<DIV CLASS="dtree">');
   htp.anchor ('javascript: d.openAll();', 'Open All');
   htp.p (common.space2$);
   htp.anchor ('javascript: d.closeAll();', 'Close All');
   htp.para;
   htp.p ('<script type="text/javascript">');
   htp.p ('d = new dTree(' || Q$ || 'd' || Q$ || ');');
   htp.p ('d.add(0,-1,' || Q$ || link_name$ || Q$ || ');');

   build (ID);

   htp.p ('document.write(d);');
   htp.p ('</script>');
   htp.p ('</DIV>');

end menu;

-- **********************************************************************************
-- **********************************************************************************

end maint;

/
show error

set define on

--	Copyright (C) 2004-2008 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>
