set define off

create or replace package body paper_ids as

rpt_title$	constant varchar2(50) := 'Assign Paper IDs';

type	session_rec_type is record (
		session_id	sessions.session_id%TYPE,
		capacity	integer,
		total		integer);
type	session_type is table of session_rec_type index by binary_integer;

duplicate_session	exception;
pragma			exception_init (duplicate_session, -1);

type	dup_insert_type is record (
		abstract_id	abstracts.abstract_id%TYPE,
		session_id	sessions.session_id%TYPE);
type	dup_inserts_type is table of dup_insert_type index by binary_integer;
dup_inserts	dup_inserts_type;

-- ***********************************************************************************************
-- ***********************************************************************************************

procedure show_sort_order (caption varchar2) is

cursor	driver is
select	field_name
from	program_sort_fields
where	sort_order is not null
order by sort_order;

begin

   htp.bold ('Note:');
   htp.p (common.space$);
   htp.p (caption);
   htp.p ('You may change the sort order on the');
   htp.anchor ('maint.tab?tab_name=program_sort_fields&order_by=sort_order%20nulls%20last',
			'Program Sort Order Screen.');
   htp.olistOpen;

   for driver_rec in driver LOOP
      htp.listItem (driver_rec.field_name);
   end loop;

   htp.olistClose;

end show_sort_order;

-- ************************************************************************************
-- ************************************************************************************

procedure bulk_assign (abs_id common.ident_arr, sid varchar2, pcode varchar2, pflag varchar2, stime varchar2, dur varchar2) is

cursor	driver is
select	ID abs_id
from	ID_list
order by gen_sort_order (ID, 0);

msg		long;
start_date	date := trunc(sysdate);
start_time	integer;

begin

   msg := jacow.additional_sessions_validate (sid, NULL, pcode, stime, dur, true);

   if msg is null then
      delete from abstract_sessions where session_id = sid;

      for i in 1..abs_id.COUNT loop
         insert into ID_list values (abs_id(i)); -- temporary table
      end loop;

      start_date := common.add_time (start_date, stime);

      for rec in driver loop
         start_time := to_char (start_date, 'hh24mi');
         common.add_abs_session (rec.abs_id, sid, pcode, pflag, start_time, dur);
         start_date := common.add_time (start_date, dur);
      end loop;

      common.redirect_URL (common.server || common.vdir$ || 'paper_ids.bulk_search');

   elsif common.header (rpt_title$) then
      htp.p (msg);
      htp.p (common.click_back$);
      common.footer;

   end if;

end bulk_assign;

-- ***********************************************************************************************
-- ***********************************************************************************************

procedure results (name_array owa.vc_arr, value_array owa.vc_arr) is

rec	search.search_type;

begin

   if common.header (rpt_title$) then
      rec := search.search_params_to_rec (name_array, value_array);

-- Suppress standard search output.
      search.show_links		:= false;
      search.show_admin_data	:= false;
      search.results2 (rec, 'paper_ids.bulk_search');

      htp.formOpen ('paper_ids.bulk_assign', 'POST');
      htp.header (3, 'Bulk Session Assignment');
      for i in 1..search.abs_recs.COUNT loop
         htp.formHidden ('abs_id', search.abs_recs(i).abstract_id);
      end loop;

      htp.tableOpen;
      jacow.additional_sessions_prompts (0);
      common.show_data_row (null, htf.formSubmit (null, 'Assign'));
      htp.tableClose;
      htp.formClose;

      show_sort_order ('These abstracts will be sorted as follows.');

      common.footer;
   end if;

end results;

-- ***********************************************************************************************

procedure bulk_search is

begin

   if common.header (rpt_title$) then
      search.html_form ('!paper_ids.results');
      common.footer;
   end if;

end bulk_search;

-- ***********************************************************************************************
-- ***********************************************************************************************

function get_next_session_seq (sid varchar2) return integer is

cursor	driver is
select	max (session_seq)
from	abstract_sessions
where	session_id = sid;

seq	integer;

begin

   open driver;
   fetch driver into seq;
   close driver;

   return nvl(seq, 0) + 1;

end get_next_session_seq;

-- ***********************************************************************************************
-- ***********************************************************************************************

procedure close_gaps (this_session varchar2) is

cursor	driver is
select	rowid, session_seq
from	abstract_sessions
where	session_id	= this_session
order by session_seq;

type	rec_type is table of driver%ROWTYPE index by binary_integer;
recs	rec_type;

begin

   if common.header (rpt_title$, test_only=>true) then
      open driver;
      fetch driver BULK COLLECT into recs;
      close driver;

      for i in 1..recs.COUNT loop
         if i <> recs(i).session_seq then
            update abstract_sessions
            set    session_seq	= i
            where  rowid	= recs(i).rowid;
         end if;
      end loop;
   end if;

   common.redirect_URL ('paper_IDs.validate');

end close_gaps;

-- ***********************************************************************************************
-- ***********************************************************************************************

function fmt_paper_seq (sid varchar2, seq integer, present_code varchar2) return varchar2 is

cursor	driver is
select	presentation_type_code, include_in_paper_id
from	presentation_codes
where	presentation_code	= present_code;

driver_rec	driver%ROWTYPE;

fmt_string	varchar2(20);

begin

   if sid is null Or seq is null then
      return null;
   end if;

   open driver;
   fetch driver into driver_rec;
   close driver;

   if driver_rec.presentation_type_code = common.poster$ then
      fmt_string := rpad ('0', common.paper_id_poster_len$, '0');
   else
      fmt_string := rpad ('0', common.paper_id_oral_len$, '0');
   end if;

   if driver_rec.include_in_paper_id = 'Y' then
      return sid || present_code || trim (to_char (seq, fmt_string));
   end if;

   return sid || trim (to_char (seq, fmt_string));

end fmt_paper_seq;

-- ************************************************************************************

function files_uploaded return boolean is

cursor	driver is
select	1
from	files	f
where	exists
(	select	1
	from	abstracts		abs
	where	abs.abstract_id		= f.abstract_id
	and	presentation_code	in
	(	select	presentation_code
		from	presentation_codes
		where	presentation_type_code = 'P'
)	);

dummy		integer;
rec_found	boolean;

begin

   open driver;
   fetch driver into dummy;
   rec_found := driver%FOUND;
   close driver;

   return rec_found;

end files_uploaded;

-- ***********************************************************************************************
-- ***********************************************************************************************

function get_room_count return integer is

cursor	driver is
select	count(1)
from	location_codes
where	location_code	in
(
	select	location_code
	from	sessions
	where	presentation_option_code	in
	(	select	presentation_option_code
		from	presentation_option_codes
		where	presentation_type_code	= common.poster$
)	);

icount	integer;

begin

   open driver;
   fetch driver into icount;
   close driver;

   return icount;

end get_room_count;

-- ***********************************************************************************************

function gen_sort_order (abs_id integer, sorder number) return varchar2 is

cursor	driver is
select	*
from	program_sort_fields
where	sort_order >= sorder
order by sort_order;

driver_rec	driver%ROWTYPE;
rec_found	boolean;

country_descr	country_codes.country_descr%TYPE;
affiliation	affiliations.name1%TYPE;
town		addresses.town%TYPE;
lname		persons.lname%TYPE;
fname		persons.fname%TYPE;
main_class	classification_codes.classification_descr%TYPE;
sub_class	sub_class_codes.sub_class_descr%TYPE;

profile_id	integer;
person_id	integer;
address_id	integer;
affil_id	integer;
country_code	country_codes.country_code%TYPE;
class_code	classification_codes.classification_code%TYPE;
sclass_code	sub_class_codes.sub_class_code%TYPE;
eclass_code	eclass_codes.eclass_code%TYPE;

buffer		varchar2(500);

begin

   open driver;
   fetch driver into driver_rec;
   rec_found := driver%FOUND;
   close driver;

   if NOT rec_found then
      return null;
   end if;

   if driver_rec.field_name = 'Main Classification' then
      eclass_code := common.decode_value ('eclass_code', 'abstracts', 'abstract_id', abs_id);
      class_code  := NVL (common.decode_value ('classification_code', 'abstracts', 'abstract_id', abs_id),
			common.decode_value ('classification_code', 'eclass_codes', 'eclass_code', eclass_code) );
      main_class  := common.decode_value ('classification_descr', 'classification_codes', 'classification_code', class_code);
      buffer	  := main_class;

   elsif driver_rec.field_name = 'Sub Classification' then
      eclass_code := common.decode_value ('eclass_code', 'abstracts', 'abstract_id', abs_id);
      sclass_code := NVL (common.decode_value ('sub_class_code', 'abstracts', 'abstract_id', abs_id),
			common.decode_value ('sub_class_code', 'eclass_codes', 'eclass_code', eclass_code) );

      sub_class	  := common.decode_value ('sub_class_descr', 'sub_class_codes', 'sub_class_code', sclass_code);
      buffer	  := sub_class;

   else

      profile_id := common.get_speaker_profile_id (abs_id);
      person_id	 := common.decode_value ('person_id', 'profiles', 'profile_id', profile_id);

      if driver_rec.field_name = 'Last Name of Presenter' then
         lname	:= common.decode_value ('lname', 'persons', 'person_id', person_id);
         buffer	:= lname || to_char (person_id, '000000000');

      elsif driver_rec.field_name = 'First Name' then
         person_id	:= common.decode_value ('person_id', 'profiles', 'profile_id', profile_id);
         fname		:= common.decode_value ('fname', 'persons', 'person_id', person_id);
         buffer		:= fname;

      else

         address_id := common.decode_value ('address_id', 'profiles', 'profile_id', profile_id);

         if driver_rec.field_name = 'Country of Presenter' then
            country_code	:= common.decode_value ('country_code', 'addresses', 'address_id', address_id);
            country_descr	:= common.decode_value ('country_descr', 'country_codes', 'country_code', country_code);
            buffer		:= country_descr;

         elsif driver_rec.field_name = 'Town of Presenter' then
            town	:= common.decode_value ('town', 'addresses', 'address_id', address_id);
            buffer	:= town;

         elsif driver_rec.field_name = 'Affiliation of Presenter' then
            affil_id	:= common.decode_value ('affiliation_id', 'addresses', 'address_id', address_id);
            affiliation	:= common.decode_value ('name1', 'affiliations', 'affiliation_id', affil_id);
            buffer	:= affiliation;
         end if;

      end if;
   end if;

   return rpad (buffer, 500) || gen_sort_order (abs_id, driver_rec.sort_order+1);

end gen_sort_order;

-- ***********************************************************************************************
-- ***********************************************************************************************

function valid_sequences (this_session varchar2) return boolean is

cursor	driver is
select	*
from	abstract_sessions
where	session_id	= this_session
and	session_seq	is not null
order by
	session_seq;

icount	integer := 0;

begin

   for driver_rec in driver LOOP
      icount := icount + 1;
      if icount <> driver_rec.session_seq then
         return false;
      end if;
   end loop;

   return true;

end valid_sequences;

-- ***********************************************************************************************

procedure move_paper_id (this_session varchar2, beg_seq integer, end_seq integer, direction integer) is

begin

   update abstract_sessions
   set	  session_seq	= session_seq + direction
   where  session_id	= this_session
   and    session_seq	between beg_seq and end_seq;

end move_paper_id;

-- ***********************************************************************************************
-- ***********************************************************************************************

procedure insert_abstract_session (abs_id integer, this_session varchar2, this_seq integer default null) is

i	integer;
seq	integer;

begin

   seq := nvl (this_seq, get_next_session_seq (this_session));

   update	abstract_sessions
   set		primary_flag	= 'N'
   where	abstract_id	= abs_id
   and		primary_flag	= 'Y';

   insert into abstract_sessions
		(abstract_id, presentation_code, session_id, session_seq, primary_flag)
   select	abstract_id, presentation_code, this_session, seq, 'Y'
   from		abstracts
   where 	abstract_id = abs_id;

exception when duplicate_session then
   i := dup_inserts.COUNT + 1;
   dup_inserts(i).abstract_id	:= abs_id;
   dup_inserts(i).session_id	:= this_session;

end insert_abstract_session;

-- ***********************************************************************************************

procedure set_paper_id (this_abstract integer, this_session varchar2, this_seq integer) is

cursor	driver is
select	rowid, session_id, session_seq
from	abstract_sessions
where	abstract_id	= this_abstract
and	session_id	= this_session;

driver_rec	driver%ROWTYPE;
element_num	integer;
switch_session	boolean	:= false;
moving_up	boolean := false;

maxseq		integer;
code		constant sessions.presentation_option_code%TYPE := common.decode_value ('presentation_option_code',
				'sessions', 'session_id', this_session);

begin

   open driver;
   fetch driver into driver_rec;
   close driver;

   delete from abstract_sessions where rowid = driver_rec.rowid;

   if this_session <> driver_rec.session_id then
      switch_session := true;
   end if;

   if this_seq > driver_rec.session_seq then
      moving_up := true;
   end if;

   if switch_session then
      maxseq := power (10, common.get_paper_seq_len(code, 'presentation_option_codes') ) - 1;
      move_paper_id (this_session, this_seq, maxseq, +1); -- make a hole in the new session
      move_paper_id (driver_rec.session_id, driver_rec.session_seq, maxseq, -1); -- close the gap in the old session

   elsif moving_up then
      move_paper_id (this_session, driver_rec.session_seq, this_seq, -1);

   else -- moving_down
      move_paper_id (this_session, this_seq, driver_rec.session_seq, +1);

   end if;

   insert_abstract_session (this_abstract, this_session, this_seq);

   if NOT valid_sequences (this_session) then
      htp.header (3, 'Warning: ' || this_session || ' is out of sequence.');
   end if;

   if switch_session then
      if NOT valid_sequences (driver_rec.session_id) then
         htp.header (3, 'Warning: ' || driver_rec.session_id || ' is out of sequence.');
      end if;
   end if;

end set_paper_id;

-- ************************************************************************************

procedure show_line (abs_id integer, paper_id varchar2, attrib varchar2 default null) is

begin

   htp.tableRowOpen ('LEFT', 'TOP');
   htp.tableData (abs_id, cattributes=>attrib);
   htp.tableData (paper_id, cattributes=>attrib);
   htp.tableRowClose;

end show_line;

-- ************************************************************************************

procedure show_paper_ids (this_id varchar2) is

cursor	driver is
select	abstract_id, session_seq, paper_id
from	abstract_sessions
where	session_id = this_id
order by
	session_seq;

type		rec_type is table of driver%ROWTYPE index by binary_integer;
recs		rec_type;
last_seq	integer := 0;

begin

   open driver;
   fetch driver BULK COLLECT into recs;
   close driver;

   if recs.COUNT > 0 then
      if recs(1).session_seq <> 1 then
         show_line (recs(1).abstract_id, recs(1).paper_id, common.red$);
      end if;
   end if;

   for i in 2..recs.COUNT LOOP
      if recs(i).session_seq <> recs(i-1).session_seq + 1 then
         if last_seq <> recs(i-1).session_seq then
            show_line (recs(i-1).abstract_id, recs(i-1).paper_id);
         end if;

         htp.tableRowOpen ('LEFT', 'TOP');
         show_line (recs(i).abstract_id, recs(i).paper_id, common.red$);
         last_seq := recs(i).session_seq;
      end if;
   end loop;

end show_paper_ids;

-- ************************************************************************************

procedure validate is

cursor	driver is
select	s.session_id, lc.capacity
from	location_codes		lc,
	sessions		s
where	lc.location_code	= s.location_code
order by
	s.sort_order, s.session_date, s.session_id;

icount		integer;
files_uploaded$	constant boolean := files_uploaded;


begin

   if common.header (rpt_title$) then
      htp.tableOpen;
      for driver_rec in driver LOOP

         htp.tableRowOpen ('LEFT', 'TOP');
         htp.tableHeader (driver_rec.session_id);

         if valid_sequences (driver_rec.session_id) then
            htp.tableHeader ('OK');
            select count(1) into icount from abstract_sessions where session_id = driver_rec.session_id;
            htp.tableData (icount || ' Abstracts');

            if icount > driver_rec.capacity then
               htp.tableData ('Room Capacity Exceeded ' || driver_rec.capacity, cattributes=>common.red$);
            end if;

            htp.tableRowClose;
         else
            htp.tableHeader ('Out of Sequence');
            if NOT files_uploaded and common.poster_id_assign_enabled$ = 'Y' then
               htp.tableData (htf.anchor ('paper_IDs.close_gaps?this_session=' ||
				driver_rec.session_id, 'Resequence') );
            end if;
            htp.tableRowClose;

            show_paper_ids (driver_rec.session_id);
         end if;

      end loop;
      htp.tableClose;

      common.footer;
   end if;

end validate;

-- ************************************************************************************
-- ************************************************************************************

procedure move (aid varchar2, sid varchar2, seq varchar2) is

cursor	driver (abs_id integer) is
select	*
from	abstracts
where	abstract_id = abs_id;

driver_rec	driver%ROWTYPE;
continue	boolean	:= true;
buffer		long	:= null;

ID	integer;

begin

   if common.header (rpt_title$, test_only=>true) then
      if common.validate_number (aid) then
         ID := aid;
      else
         ID := common.decode_value ('abstract_id', 'abstract_sessions', 'paper_id', aid);
      end if;

      open driver (ID);
      fetch driver into driver_rec;
      continue := driver%FOUND;
      close driver;

      if NOT continue then
         buffer := htf.header (3, 'Abstract ID ' || aid || ' does not exist.');
      end if;

      if NOT common.validate_number (seq) then
         buffer := buffer || htf.header (3, 'Sequence is not a number. ' || seq);
         continue := false;
      end if;

      if continue then
         set_paper_id (ID, upper(sid), seq);
         common.redirect_url ('paper_ids.html');
      else
         common.public_header (rpt_title$);
         htp.p (buffer);
         common.footer;
      end if;

   end if;

end move;

-- ************************************************************************************
-- ************************************************************************************

procedure check_room_capacity is

cursor	driver is
select	s.session_date, s.beg_time, s.end_time,
	s.session_id, l.capacity, l.location_descr, l.location_code,
	max (a.session_seq) max_seq
from	abstract_sessions		a,
	location_codes			l,
	sessions			s
where	s.location_code			= l.location_code
and	s.presentation_option_code	in
(
	select	presentation_option_code
	from	presentation_option_codes
	where	presentation_type_code	= common.poster$
)
and	a.session_id			= s.session_id
group by
	s.session_date, s.beg_time, s.end_time,
	s.session_id, l.capacity, l.location_descr, l.location_code
having	max (a.session_seq) > l.capacity
order by
	s.session_date, s.beg_time, s.end_time, s.session_id;

capacity_exceeded	boolean := false;

begin

   for driver_rec in driver LOOP
      if driver_rec.max_seq > driver_rec.capacity then
         if NOT capacity_exceeded then -- first time through
            common.public_header (rpt_title$);
         end if;

         htp.header (3, 'Room Capacity Exceeded', cattributes=>common.red$);
         htp.tableOpen;
         driver_rec.session_date := common.add_time (driver_rec.session_date, driver_rec.beg_time);
         common.show_data_row ('Date',		to_char (driver_rec.session_date, 'Day dd-Mon-yy hh24:mi'));
         common.show_data_row ('Location',	driver_rec.location_descr);
         common.show_data_row ('Capacity',	driver_rec.capacity);
         common.show_data_row ('Posters',	driver_rec.max_seq);
         htp.tableClose;
         htp.br;

         capacity_exceeded := true;
      end if;
   end loop;

   if capacity_exceeded OR dup_inserts.COUNT > 0 then
      if dup_inserts.COUNT > 0 then
         htp.header (3, 'The following absracts have issues.');
         htp.p ('These abstracts were added with alternate program codes,');
         htp.p ('however their classification puts them into the same session as the alternate.');
         htp.p ('Please fix this issue and rerun the poster numbering routine.');
         htp.br;

         htp.tableOpen;
         htp.tableRowOpen;
         htp.tableHeader ('Abstract');
         htp.tableHeader ('Session');
         htp.tableRowClose;

         for i in 1..dup_inserts.COUNT LOOP
            htp.tableRowOpen;
            htp.tableData (dup_inserts(i).abstract_id);
            htp.tableData (dup_inserts(i).session_id);
            htp.tableRowClose;
         end loop;
         htp.tableClose;
      end if;

      common.footer;
   else
      common.redirect_url ('reports.disjointed_authors');
   end if;

end check_room_capacity;

-- ************************************************************************************

function get_abs_session_data (ecode varchar2, ccode varchar2, scode varchar2) return session_type is

cursor	driver is
select	1 field_order, cs.sort_order, cs.session_id, lc.capacity,
	s.location_code, s.session_date, s.beg_time
from	location_codes		lc,
	sessions		s,
	class_sessions		cs
where	cs.eclass_code		= ecode
and	s.session_id		= cs.session_id
and	lc.location_code	= s.location_code
UNION
select	2, cs.sort_order, cs.session_id, lc.capacity,
	s.location_code, s.session_date, s.beg_time
from	location_codes		lc,
	sessions		s,
	class_sessions		cs
where	cs.sub_class_code 	= scode
and	s.session_id		= cs.session_id
and	lc.location_code	= s.location_code
UNION
select	3, cs.sort_order, cs.session_id, lc.capacity,
	s.location_code, s.session_date, s.beg_time
from	location_codes		lc,
	sessions		s,
	class_sessions		cs
where	cs.classification_code	= ccode
and	s.session_id		= cs.session_id
and	lc.location_code	= s.location_code
order by 1, 2;  -- sort order is default but important to maintain here.

cursor	C2 (lcode varchar2, sdate date, sbeg integer) is
select	count(1)
from	abstract_sessions
where	session_id	in
	(	select	session_id
		from	sessions
		where	location_code	= lcode
		and	session_date	= sdate
		and	beg_time	= sbeg)
and	primary_flag	= 'Y';

location_count	integer;
session_data	session_type;
i		integer;
total		integer;

begin

   session_data.delete;

   for rec in driver LOOP
      open C2 (rec.location_code, rec.session_date, rec.beg_time);
      fetch C2 into total;
      close C2;

      i := session_data.COUNT+1;
      session_data(i).session_id	:= rec.session_id;
      session_data(i).capacity		:= rec.capacity;
      session_data(i).total		:= nvl (total, 0);
   end loop;

   return session_data;

end get_abs_session_data;

-- ************************************************************************************

procedure assign_poster is

cursor	driver is
select	abstract_id, eclass_code,
	classification_code class_code, sub_class_code
from	abstracts
where	presentation_code	in
	(	select	presentation_code
		from	presentation_codes
		where	presentation_type_code = common.poster$)
order by gen_sort_order (abstract_id, 0);

type		rec_type is table of driver%ROWTYPE;
recs		rec_type;
sessions	session_type;
session_set	boolean;

begin

   if common.header (rpt_title$, test_only=>true) then

	-- Clear all poster session IDs
      delete from abstract_sessions
      where	abstract_id	in
		(	select	abstract_id
			from	abstracts
			where	presentation_code	in
			(	select	presentation_code
				from	presentation_codes
				where	presentation_type_code = common.poster$
		)	)
      and	primary_flag	= 'Y';

      open driver;
      fetch driver BULK COLLECT into recs;
      close driver;

      for i in 1..recs.COUNT LOOP
         sessions	:= get_abs_session_data (recs(i).eclass_code, recs(i).class_code,
						recs(i). sub_class_code);
         session_set	:= false;

         for j in 1..sessions.COUNT LOOP
            if (sessions(j).total < sessions(j).capacity) or (j = sessions.COUNT) then
               insert_abstract_session (recs(i).abstract_id, sessions(j).session_id);

               session_set := true;
            end if;

            exit when session_set;
         end loop;
      end loop;

      check_room_capacity;
   end if;

end assign_poster;

-- ************************************************************************************
-- ************************************************************************************

procedure poster is

begin

   if common.header (rpt_title$) then
      if common.poster_id_assign_enabled$ = 'N' then
         htp.header (3, 'Program code assignment is disabled.', cattributes=>common.red$);

      elsif files_uploaded then
         htp.header (3, 'Poster files uploaded. Poster program code assignment is unavailable.', cattributes=>common.red$);

      else
         htp.header (3, 'Are you sure you wish to reset the poster paper IDs?');
         htp.bold ('You will need to resequence disjointed papers.');

         htp.formOpen ('paper_ids.assign_poster');
         htp.formSubmit (null, 'Yes');
         htp.formClose;

         show_sort_order ('Abstracts within each Session will sort in the following order.');
      end if;

      common.footer;
   end if;

end poster;

-- ************************************************************************************

procedure html is

cursor	driver is
select	s.session_id, s.title, s.session_date, s.beg_time, l.location_descr
from	location_codes			l,
	sessions			s,
	presentation_option_codes	p
where	p.presentation_type_code	= common.poster$
and	s.presentation_option_code	= p.presentation_option_code
and	l.location_code			= s.location_code
and	l.capacity			= 0;

continue	boolean := true;

begin

   if common.header (rpt_title$) then
      if common.poster_id_assign_enabled$ = 'N' then
         htp.header (3, 'Program code assignment is disabled.', cattributes=>common.red$);
         continue := false;

      elsif common.files_uploaded then
         htp.header (3, 'One or more files have been uploaded.', cattributes=>common.red$);
         htp.p ('You cannot change Program Codes once files have been uploaded.');
         htp.p ('If you uploaded files as a test you may request the SPMS administrator delete the files from the database.');
         htp.p ('Please remember to delete them from the remote file server as well.');
         continue := false;

      else
         htp.formOpen ('paper_ids.poster');
         htp.formSubmit (null, 'Reset All Poster Paper IDs');
         htp.formClose;
      end if;

      if continue then
         htp.formOpen ('paper_ids.move', 'POST');
         htp.tableOpen;

         common.show_data_row ('Move Abstract/Paper ID',	htf.formText ('aid', 10, 10) );

         common.show_popup_table_row ('session_id',
					'session_id || '' - '' || title',
				   'sessions', 'To Session',
				   var_name=>'sid',
				   order_by=>'order by sort_order nulls last, session_date, beg_time, end_time');

         common.show_data_row ('Sequence',			htf.formText ('seq', 10, 10) );
         common.show_data_row (null,			htf.formSubmit);
         htp.tableClose;
         htp.formClose;
      end if;

      htp.para;
      htp.anchor ('reports.disjointed_authors', 'Disjointed Authors');
      htp.p (common.space2$);
      htp.anchor ('paper_ids.validate', 'Validate Paper ID Sequences');
      htp.para;
      common.footer;
   end if;

end html;

-- ************************************************************************************

end paper_ids;
/
show error

set define on

--	Copyright (C) 2004 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>
