set define off

create or replace package body params as

rpt_title$	constant char(17)		:= 'System Parameters';
rpt_title2$	constant char(38)		:= 'Regional Default Initialization Values';

-- ************************************************************************************

procedure show_init_header is

cursor	driver is
select	distinct region
from	initial_parameters;

begin

   htp.tableOpen;
   htp.tableRowOpen ('LEFT', 'TOP');

   for rec in driver LOOP
      htp.tableData (common.space2$);
      htp.tableData (htf.anchor ('params.initial?code=' || rec.region, rec.region));
      htp.tableData (common.space2$);
   end loop;

   htp.tableRowClose;
   htp.tableClose;
   htp.para;

end show_init_header;

-- ************************************************************************************

procedure initial_save_params (code varchar2, ID common.ident_arr, value common.ident_arr) is

begin

   common.get_gbl_pid;
   if common.has_privs_to_page then

      for i in 1..ID.COUNT loop
         update	initial_parameters
         set	value	= initial_save_params.value(i)
         where	ID	= initial_save_params.ID(i)
         and	region	= code;
      end loop;

      initial (code);
   end if;

end initial_save_params;

-- ************************************************************************************

procedure initial_save_status_codes (code varchar2, status_code common.ident_arr, URL common.ident_arr) is

begin

   common.get_gbl_pid;
   if common.has_privs_to_page then

      for i in 1..status_code.COUNT loop
         update	initial_status_codes
         set	image_URL	= initial_save_status_codes.URL(i)
         where	status_code	= initial_save_status_codes.status_code(i)
         and	region		= code;
      end loop;

      initial (code);
   end if;

end initial_save_status_codes;

-- ************************************************************************************

procedure initial_save_referee_codes (code varchar2, referee_code common.ident_arr, URL common.ident_arr) is

begin

   common.get_gbl_pid;
   if common.has_privs_to_page then

      for i in 1..referee_code.COUNT loop
         update	initial_referee_codes
         set	image_URL	= initial_save_referee_codes.URL(i)
         where	referee_code	= initial_save_referee_codes.referee_code(i)
         and	region		= code;
      end loop;

      initial (code);
   end if;

end initial_save_referee_codes;

-- ************************************************************************************

function show_image (URL varchar2) return varchar2 is

begin

   if lower (URL) like 'http%://%' then
      return htf.img (URL);
   end if;

   return null;

end show_image;

-- ************************************************************************************

procedure initial_params (code varchar2) is

cursor	driver is
select	i.id, p.category_code, p.description, i.value
from	parameters		p,
	initial_parameters	i
where	i.region		= code
and	i.id			= p.id
order by p.category_code, p.description;

begin

      htp.header (3, code || '''s Initial Parameters');
      htp.formOpen ('params.initial_save_params', 'POST');
      htp.formHidden ('code', code);

      htp.tableOpen ('BORDER');
      htp.tableRowOpen ('LEFT', 'BOTTOM');
      htp.tableRowClose;

      for rec in driver LOOP
         htp.tableRowOpen ('LEFT', 'TOP');
         htp.tableData (rec.category_code || htf.formHidden ('ID', rec.ID));
         htp.tableData (rec.description || show_image (rec.value) );
         htp.tableData (htf.formText ('value', 100, 2000, rec.value));
         htp.tableRowClose;
      end loop;

      htp.tableClose;
      htp.para;
      htp.formSubmit;
      htp.formClose;

end initial_params;

-- ************************************************************************************

procedure initial_status_codes (code varchar2) is

cursor	driver is
select	sc.status_code, sc.status_descr, isc.image_URL
from	status_codes		sc,
	initial_status_codes	isc
where	isc.region		= code
and	sc.status_code		= isc.status_code
order by sc.sort_order nulls last, sc.status_descr;

begin

      htp.header (3, code || '''s Initial Editor Status Code URLs');
      htp.formOpen ('params.initial_save_status_codes', 'POST');
      htp.formHidden ('code', code);

      htp.tableOpen ('BORDER');
      htp.tableRowOpen ('LEFT', 'BOTTOM');
      htp.tableRowClose;

      for rec in driver LOOP
         htp.tableRowOpen ('LEFT', 'TOP');
         htp.tableData (rec.status_descr || htf.formHidden ('status_code', rec.status_code) ||
			show_image (rec.image_URL) );
         htp.tableData (htf.formText ('URL', 100, 2000, rec.image_URL));
         htp.tableRowClose;
      end loop;

      htp.tableClose;
      htp.para;
      htp.formSubmit;
      htp.formClose;

end initial_status_codes;

-- ************************************************************************************

procedure initial_referee_codes (code varchar2) is

cursor	driver is
select	sc.referee_code, sc.referee_descr, isc.image_URL
from	referee_codes		sc,
	initial_referee_codes	isc
where	isc.region		= code
and	sc.referee_code		= isc.referee_code
order by sc.sort_order nulls last, sc.referee_descr;

begin

      htp.header (3, code || '''s Initial Referee Status Code URLs');
      htp.formOpen ('params.initial_save_referee_codes', 'POST');
      htp.formHidden ('code', code);

      htp.tableOpen ('BORDER');
      htp.tableRowOpen ('LEFT', 'BOTTOM');
      htp.tableRowClose;

      for rec in driver LOOP
         htp.tableRowOpen ('LEFT', 'TOP');
         htp.tableData (rec.referee_descr || htf.formHidden ('referee_code', rec.referee_code) ||
			show_image (rec.image_URL) );
         htp.tableData (htf.formText ('URL', 100, 2000, rec.image_URL));
         htp.tableRowClose;
      end loop;

      htp.tableClose;
      htp.para;
      htp.formSubmit;
      htp.formClose;

end initial_referee_codes;

-- ************************************************************************************

procedure initial (code varchar2) is

begin

   if common.header (rpt_title2$) then
      show_init_header;
      initial_params (code);
      initial_status_codes (code);
      initial_referee_codes (code);
   end if;

end initial;

-- ************************************************************************************

procedure initial is

begin

   if common.header (rpt_title2$) then
      show_init_header;
   end if;

end initial;

-- ************************************************************************************
-- ************************************************************************************

function file_weights_assigned return boolean is

cursor	driver is
select	presentation_code, sum (weight) weight
from	presentation_file_codes
group by presentation_code;

all_assigned	boolean	:= true;
max_weight	number	:= 0;

begin

   if nvl (common.file_weight_threshold$, 0) = 0 then
      return false;
   end if;

   for rec in driver LOOP
      if rec.weight <= 0 then
         all_assigned := false;
      end if;

      max_weight := greatest (max_weight, rec.weight);
   end loop;

   if max_weight < common.file_weight_threshold$ then
      return false;
   end if;

   return all_assigned;

end file_weights_assigned;

-- ************************************************************************************

procedure reseq (cat varchar2 default null) is

begin

-- The number of digits changed.  Must be done via redirect to load changed values.
   update abstract_sessions set session_seq = session_seq where session_seq is not null;
   common.redirect_url ('params.show_cats?cat=' || cat);

end reseq;

-- ************************************************************************************

procedure set_value (num integer, new_value varchar2) is

msg	varchar2(4000)	:= null;
dtype	constant parameters.dtype%TYPE := common.decode_value ('dtype', 'parameters', 'ID', num);

begin

   common.get_gbl_pid;
   if common.has_privs_to_page then

      if dtype = dtype_date$ then

         if NOT common.validate_date (new_value) then
            msg := htf.header (3, 'Invalid Date: ' || new_value, cattributes=>common.red$);
         end if;

      elsif dtype = dtype_number$ then

         if NOT common.validate_number (new_value) then
            msg := htf.header (3, 'Invalid Number: ' || new_value, cattributes=>common.red$);
         end if;

      end if;

      if msg is null then
         update parameters
         set	value	= new_value
         where	ID	= num;

         if num in (49, 50) then -- reset the paper id sequence length
            common.redirect_url ('params.reseq?cat=' ||
			common.decode_value ('category_code', 'parameters', 'id', num) );
         else
            common.redirect_url ('params.show_cats?cat=' ||
			common.decode_value ('category_code', 'parameters', 'id', num) );
         end if;

      else
         common.public_header (rpt_title$);
         htp.p (msg);
         htp.p (common.click_back$);
         htp.para;
         common.footer;
      end if;

   end if;

end set_value;

-- ************************************************************************************

procedure get_value (num integer) is

cursor	driver is
select	description, value, dtype, long_text
from	parameters
where	ID = num;

driver_rec	driver%RoWTYPE;
rec_found	boolean;
selected	varchar2(8) := 'SELECTED';

begin

   if common.header (rpt_title$, test_only=>true) then
      open driver;
      fetch driver into driver_rec;
      rec_found := driver%FOUND;
      close driver;

      if rec_found then

         if driver_rec.value in ('Y', 'N') then

            if num = 100 AND common.accept_abstracts$ = 'Y' then
               common.public_header (rpt_title$);
               htp.header (3, 'Error - Unable to enter Invited Oral mode', cattributes=>common.red$);
               htp.bold ('You can not allow input for Invited Orals when you are accepting abstracts from users.');
               common.footer;

            elsif num = 17 AND common.PC_mode$ = 'Y' then
               common.public_header (rpt_title$);
               htp.header (3, 'Error - Unable to Accept Abstracts', cattributes=>common.red$);
               htp.bold ('You can not accept abstracts from users while in Programm Committee Mode.');
               common.footer;

            elsif num = 18 AND driver_rec.value = 'N' AND common.author_upload_script$ is null then
               common.public_header (rpt_title$);
               htp.header (3, 'Error - Unable to Accept File Uploads', cattributes=>common.red$);
               htp.bold ('You cannot accept files until you define the "Author File Upload Script" URL.');
               htp.para;
               common.footer;

            elsif num = 18 AND driver_rec.value = 'N' AND not file_weights_assigned then
               common.public_header (rpt_title$);
               htp.header (3, 'Error - Unable to Accept File Uploads', cattributes=>common.red$);
               htp.bold ('You cannot accept files until you properly assign file weights and a  file weight threshold.');
               htp.para;
               htp.anchor ('present_files.html', 'File weights');
               htp.p ('are assigned to file types required from authors.');
               htp.p ('Required files, such as source files and postscript files, are generally assigned a weight');
               htp.p ('of 2 and 1 respectively.  Assigning a ');
               htp.anchor ('params.get_value?num=95', 'file weight threshold');
               htp.p ('of 1 will allow and editor to process');
               htp.p ('a contribution where only the postscript file exists.  However, contributions where');
               htp.p ('the source is uploaded or the source and postscript are uploaded will be assigned before');
               htp.p ('those contribution with only the postscript.  There should be at least one file type');
               htp.p ('with a non-zero weight for each presentation type.');
               htp.para;
               htp.bold ('The threashold value must not exceed the maximum, total weight for any presentation, nor can the threshold be zero.');
               htp.para;
               common.footer;

            elsif num = 18 AND driver_rec.value = 'N' AND common.hide_program_codes$ = 'Y' then
               common.public_header (rpt_title$);
               htp.header (3, 'Error - Unable to Accept File Uploads', cattributes=>common.red$);
               htp.bold ('You cannot accept files while program codes are hidden.');
               htp.para;
               common.footer;

            elsif num = 138 AND driver_rec.value = 'N' AND common.accept_file_uploads$ = 'Y' then
               common.public_header (rpt_title$);
               htp.header (3, 'Error - Unable to Hide Program Codes', cattributes=>common.red$);
               htp.bold ('You cannot hide program codes while accepting file uploads from users.');
               htp.para;
               common.footer;

            else
               update parameters set value = decode (value, 'Y','N','Y') where id = num;

		-- exiting reclassify and/or presentation changes, freeze updates by users to those fields.
               if driver_rec.value = 'N' then
                  if num = 24 then
                     update parameters set value = 'Y' where id = 98;
                  elsif num = 65 then
                     update parameters set value = 'Y' where id = 75;
                  end if;
               end if;

               common.redirect_url ('params.show_cats?cat=' ||
			common.decode_value ('category_code', 'parameters', 'id', num) );
            end if;

         else
            common.public_header (rpt_title$);
            htp.formOpen ('params.set_value', 'POST', cattributes=>'NAME="PARAMS"');
            htp.formHidden ('num', num);
            htp.bold (driver_rec.description);

            if num = 105 then
               common.show_popup_table ('payment_vendor_code', 'payment_vendor_descr',
				'payment_vendor_codes', this_value=>driver_rec.value,
				var_name=>'new_value');

            elsif driver_rec.dtype = dtype_date$ then
               htp.formText ('new_value', 15, 10, driver_rec.value);
               htp.p ('Note: Enter date in MM/DD/YYYY format (e.g. ' ||
			to_char (sysdate, 'MM/DD/YYYY') || ').');

            elsif driver_rec.long_text = 'Y' then
               htp.formTextAreaOpen ('new_value', 5, 50);
               htp.p (htf.escape_sc (driver_rec.value));
               htp.formTextAreaClose;
               htp.bold ('Limit 4000 characters');

            elsif driver_rec.dtype = dtype_number$ then
               htp.formText ('new_value', 10, 100, driver_rec.value);
               htp.bold ('Enter a numeric value.');

            else
               htp.formText ('new_value', 50, 4000, driver_rec.value);

            end if;

            htp.para;
            htp.p (common.decode_value ('comments', 'parameters', 'id', num));

            htp.para;
            htp.formSubmit (null, 'Save');
            htp.formClose;
            common.form_focus ('PARAMS', 'new_value');
            common.footer;
         end if;

      else
         common.public_header (rpt_title$);
         htp.header (3, 'No such parameter ' || num);
         common.footer;
      end if;

   else
      common.public_header (rpt_title$);
      htp.header (3, 'Only the Conference Administrator may modify System Parameters.');
      common.footer;
   end if;

end get_value;

-- ************************************************************************************

procedure show_cat (cat varchar2) is

cursor	driver is
select	ID, description, value, comments, long_text,
	decode (value, 'Y','Yes', 'N','No', value) value2
from	parameters
where	ID		<> 0
and	category_code	like cat
order by
	sort_order nulls last,
	description;

shade	varchar2(30) := null;

begin

   htp.tableOpen (cattributes=>'CELLPADDING="5" CELLSPACING="0"');

   for driver_rec in driver LOOP

      if driver_rec.id = 105 then
         driver_rec.value2 := common.decode_value ('payment_vendor_descr', 'payment_vendor_codes',
				'payment_vendor_code', driver_rec.value);
      elsif driver_rec.long_text = 'Y' then
         driver_rec.value2 := substr (driver_rec.value2, 1, 25) || '...';
      end if;

      if shade is null then
         shade := 'STYLE="{background: #DDDDDD}"';
      else
         shade := null;
      end if;

      htp.tableRowOpen ('LEFT', 'TOP', cattributes=>shade);
      htp.tableHeader (driver_rec.description, 'RIGHT', cnowrap=>'NOWRAP');

      htp.tableData (htf.anchor ('params.get_value?num=' || driver_rec.ID,
				nvl (driver_rec.value2, '*** NO VALUE ***') ));

      htp.tableData (driver_rec.comments);
      htp.tableRowClose;

   end loop;

   htp.tableClose;

end show_cat;

-- ************************************************************************************

procedure show_cats (cat varchar2 default null) is

cursor	driver is
select	*
from	param_categories
order by
	sort_order;

begin

   if common.header (rpt_title$) then
      htp.tableOpen (cattributes=>'WIDTH="100%"');

      htp.tableRowOpen ('CENTER', 'TOP');
      htp.p ('<TD>');

      for driver_rec in driver LOOP
         if driver_rec.category_code = cat then
            htp.p (driver_rec.description);
         else
            htp.anchor ('params.show_cats?cat=' || driver_rec.category_code, driver_rec.description);
         end if;
         htp.p (common.space2$);
      end loop;

      htp.p ('</TD>');
      htp.tableRowClose;

-- **********

      common.show_data_sep (1);

      htp.tableRowOpen ('CENTER', 'TOP');
      htp.p ('<TD>');
      show_cat (cat);
      htp.p ('</TD>');
      htp.tableRowClose;

-- **********

      htp.tableClose;
      htp.para;
      common.footer;
   end if;

end show_cats;

-- ************************************************************************************

procedure html is

begin

   show_cats;

end html;

-- ************************************************************************************

end params;
/
show error

set define on

--	Copyright (C) 2004 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>
