set define off

create or replace package body poster_police as

rpt_title$	constant varchar2(30) := 'Poster Police';

-- *********************************************************************************************
-- *********************************************************************************************

procedure column_headers is

begin

   htp.tableRowOpen ('LEFT', 'BOTTOM');
   htp.tableHeader ('ID');
   htp.tableHeader ('Presenter');
   htp.tableHeader ('Title');
   htp.tableHeader ('Manned');
   htp.tableHeader ('Posted');
   htp.tableHeader ('Satisfactory');
   htp.tableHeader ('Picture');
   htp.tableHeader ('Comments');
   htp.tableRowClose;

end column_headers;

-- *********************************************************************************************

function yn_txt (code varchar2, txt varchar2) return varchar2 is

begin

   if txt = 'Picture Taken' then
      if code = 'Y' then
         return ', ' || txt || '.';
      end if;
      return '.';
   end if;

   if code = 'N' then
      return 'Poster Not ' || txt;
   end if;

   return 'Poster ' || txt;

end yn_txt;

-- *********************************************************************************************

function status_txt (PP varchar2, PM varchar2, PS varchar2, PT varchar2) return varchar2 is

begin

   return	yn_txt (PP, 'Posted')		|| ', ' ||
		yn_txt (PM, 'Manned')		|| ', ' ||
		yn_txt (PS, 'Satisfactory')	||
		yn_txt (PT, 'Picture Taken');

end status_txt;

-- *********************************************************************************************

procedure save (aid common.ident_arr, PM common.ident_arr, PT common.ident_arr,
		PP common.ident_arr, PS common.ident_arr, comment common.ident_arr) is

cursor	driver (a integer) is
select	1
from	poster_logs
where	abstract_id	= A;

dummy		integer;
txt		varchar2(300);
rec_found	boolean;
add_rec		boolean;

begin

   common.get_gbl_pid;

   for i in 1..aid.count LOOP
      open driver (aid(i));
      fetch driver into dummy;
      rec_found := driver%FOUND;
      close driver;

      add_rec := false;
      if rec_found then
         update	poster_logs
         set	manned			= PM(i),
		posted			= PP(i),
		satisfactory		= PS(i),
		picture			= PT(i),
		comments		= trim (comment(i))
         where	abstract_id		= aid(i)
         and  (	nvl(manned,'~')		<> nvl(PM(i),'~')
         or	nvl(posted,'~')		<> nvl(PP(i),'~')
         or	nvl(satisfactory,'~')	<> nvl(PS(i),'~')
         or	nvl(picture,'~')	<> nvl(PT(i),'~')
         or	nvl(comments,'~')	<> nvl(trim(comment(i)),'~') );

         if SQL%ROWCOUNT > 0 then -- poster police status changed
            add_rec := true;
         end if;
      else
         add_rec := true;
      end if;

      if add_rec then
         insert into poster_logs (abstract_id, manned, posted, satisfactory, picture, comments)
         values (aid(i), PM(i), PP(i), PS(i), PT(i), trim(comment(i)) );

         txt := status_txt (PP(i), PM(i), PS(i), PT(i));
         insert into logs (abstract_id, log_code, comments, editor_notes)
         values (aid(i), common.poster_police$, trim(txt), trim(comment(i)) );
      end if;

   end loop;

   common.redirect_URL ('poster_police.html');

end save;

-- *********************************************************************************************
-- *********************************************************************************************

procedure edit_log_fields (aid integer) is

cursor	driver is
select	*
from	poster_logs
where	abstract_id = aid;

driver_rec	driver%ROWTYPE;

begin

   open driver;
   fetch driver into driver_rec;
   close driver;

   htp.tableData (common.popup_yes_no ('PM', nvl (driver_rec.manned, 'Y'), false), 'CENTER');
   htp.tableData (common.popup_yes_no ('PP', nvl (driver_rec.posted, 'Y'), false), 'CENTER');
   htp.tableData (common.popup_yes_no ('PS', nvl (driver_rec.satisfactory, 'Y'), false), 'CENTER');
   htp.tableData (common.popup_yes_no ('PT', nvl (driver_rec.picture, 'N'), false), 'CENTER');

   htp.tableData (htf.formText ('comment', 40, 4000, driver_rec.comments) );

end edit_log_fields;

-- *********************************************************************************************

procedure edit (sid varchar2) is

cursor	driver is
select	a.*, sa.paper_id,
	common.get_author_profile_id (a.abstract_id, common.speaker$) spk_profile_id
from	abstracts		a,
	abstract_sessions	sa
where	sa.session_id		= sid
and	a.abstract_id		= sa.abstract_id
order by
	sa.session_seq;

begin

   if common.header (rpt_title$) then
      htp.formOpen ('poster_police.save', 'POST');
      htp.tableOpen ('BORDER', cattributes=>'CELLPADDING="5"');
      column_headers;

      for driver_rec in driver LOOP
         htp.tableRowOpen ('LEFT', 'TOP');
         htp.tableData (htf.formHidden ('aid', driver_rec.abstract_id) || driver_rec.paper_id);
         htp.tableData (common.get_user_name_affiliation (driver_rec.spk_profile_id, 'Y'));
         htp.tableData (driver_rec.title);

         edit_log_fields (driver_rec.abstract_id);
         htp.tableRowClose;
      end loop;

      htp.tableClose;
      htp.br;
      htp.formSubmit;
      htp.formClose;
      common.footer;
   end if;


end edit;

-- *********************************************************************************************
-- *********************************************************************************************

procedure show_log_fields (aid integer) is

cursor	driver is
select	*
from	poster_logs
where	abstract_id = aid;

driver_rec	driver%ROWTYPE;
rec_found	boolean;

begin

   open driver;
   fetch driver into driver_rec;
   rec_found := driver%FOUND;
   close driver;

   if rec_found then
      htp.tableData (common.yes_no (driver_rec.manned), 'CENTER');
      htp.tableData (common.yes_no (driver_rec.posted), 'CENTER');
      htp.tableData (common.yes_no (driver_rec.satisfactory), 'CENTER');
      htp.tableData (common.yes_no (driver_rec.picture), 'CENTER');
      htp.tableData (driver_rec.comments);

   else
      for i in 1..4 LOOP
         htp.tableData (common.space$);
      end loop;
      htp.tableData (rpad (common.space2$, 280, common.space2$) );

   end if;

end show_log_fields;

-- *********************************************************************************************

procedure print (sid varchar2) is

cursor	driver is
select	a.*, sa.paper_id,
	common.get_author_profile_id (a.abstract_id, common.speaker$) spk_profile_id
from	abstracts		a,
	abstract_sessions	sa
where	sa.session_id		= sid
and	a.abstract_id		= sa.abstract_id
order by
	sa.session_seq;

begin

   if common.header (rpt_title$) then
      htp.tableOpen ('BORDER', cattributes=>'CELLPADDING="5"');
      column_headers;

      for driver_rec in driver LOOP
         htp.tableRowOpen ('LEFT', 'TOP');
         htp.tableData (driver_rec.paper_id);
         htp.tableData (common.get_user_name_affiliation (driver_rec.spk_profile_id, 'Y'));
         htp.tableData (driver_rec.title);
         show_log_fields (driver_rec.abstract_id);
         htp.tableRowClose;
      end loop;

      htp.tableClose;
      htp.p ('<SCRIPT LANGUAGE=''JavaScript''>window.print()</SCRIPT>');
      htp.p ('<SCRIPT LANGUAGE=''JavaScript''>window.setTimeout("window.close()",1000);</SCRIPT>');
      common.footer;
   end if;

end print;

-- *********************************************************************************************
-- *********************************************************************************************

procedure html is

cursor	driver is
select	*
from	sessions
where	presentation_option_code in
(	select	presentation_option_code
	from	presentation_option_codes
	where	presentation_type_code = 'P')
order by
	sort_order, session_date, beg_time, session_id;

begin

   if common.header (rpt_title$) then
      htp.tableOpen ('BORDER', cattributes=>'CELLPADDING="5"');

      for driver_rec in driver LOOP
         htp.tableRowOpen ('LEFT', 'TOP');
         htp.tableData (htf.anchor ('poster_police.edit?sid=' || driver_rec.session_id, 'Edit'));
         htp.tableData (htf.anchor2 ('poster_police.print?sid=' || driver_rec.session_id, 'Print',
				ctarget=>'_new'));
         htp.tableData (driver_rec.session_id);
         htp.tableData (driver_rec.title);
         htp.tableRowClose;
      end loop;

      htp.tableClose;
      common.footer;
   end if;

end html;

end poster_police;
/
show error

-- *********************************************************************************************

--	Copyright (C) 2004 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>

set define on
