set define off

create or replace package body present_files as

rpt_title$	constant varchar2(25) := 'Presentation File Codes';

type	ptype is table of presentation_codes%ROWTYPE	index by binary_integer;
type	ftype is table of file_type_codes%ROWTYPE	index by binary_integer;

prec	ptype;
frec	ftype;

-- ******************************************************************************************
-- ******************************************************************************************

procedure JS is

begin

   htp.p ('function back() {');
   htp.p ('   window.opener.location.href="' || common.server ||
			common.vdir$ || 'present_files.html";');
   htp.p ('   window.setTimeout("window.close()",1000);');
   htp.p ('}');

end JS;

-- ***********************************************************************
-- ***********************************************************************

procedure weight2 (pcode varchar2, fcode varchar2, w varchar2) is

begin

   common.blank_header (rpt_title$);
   htp.print ('<script language="JavaScript" src="' ||
		common.server || common.vdir$ || 'present_files.js"></script>');

   if common.validate_number (w) then
      update	presentation_file_codes
      set	weight			= w
      where	presentation_code	= pcode
      and	file_type_code		= fcode;

      htp.p ('<SCRIPT language="JavaScript">back();</SCRIPT>');

   else
      htp.header (3, 'Invalid Number: ' || w, cattributes=>common.red$);
      htp.p (common.click_back$);

   end if;

   common.blank_footer;

end weight2;

-- ***********************************************************************
-- ***********************************************************************

procedure weight (pcode varchar2, fcode varchar2, w varchar2) is

begin

   common.blank_header (rpt_title$);
   htp.bold (common.decode_value ('presentation_descr', 'presentation_codes',
				  'presentation_code', pcode) );
   htp.br;
   htp.bold (common.decode_value ('file_type_descr', 'file_type_codes',
				  'file_type_code', fcode) );
   htp.para;
   htp.p ('Enter the weight of this file type and presentation type: ');
   htp.formOpen ('present_files.weight2', 'POST', cattributes=>'NAME="SPMS"');
   htp.formHidden ('pcode', pcode);
   htp.formHidden ('fcode', fcode);
   htp.formText ('w', 1, 1, w);
   htp.formSubmit (null, 'Save');
   htp.formClose;
   common.form_focus ('SPMS', 'w');
   common.blank_footer;

end weight;

-- ***********************************************************************
-- ***********************************************************************

procedure yn (var varchar2, pcode varchar2, fcode varchar2) is

begin

   common.get_gbl_pid;
   if common.user_is_jacow_admin then

      if var = 'editor' then
         update	presentation_file_codes
         set	editor_only		= decode (editor_only, 'Y','N', 'Y')
         where	presentation_code	= pcode
         and	file_type_code		= fcode;
      end if;

      if var = 'always' then
         update	presentation_file_codes
         set	always_available	= decode (always_available, 'Y','N', 'Y')
         where	presentation_code	= pcode
         and	file_type_code		= fcode;
      end if;

   end if;

   common.redirect_URL ('present_files.html');

end yn;

-- ***********************************************************************
-- ***********************************************************************

procedure load_data is

cursor	C1 is
select	*
from	presentation_codes
order by presentation_descr;

cursor	C2 is
select	*
from	file_type_codes
order by file_type_descr;

begin

   open C1;
   fetch C1 BULK COLLECT into prec;
   close C1;

   open C2;
   fetch C2 BULK COLLECT into frec;
   close C2;

end load_data;

-- ***********************************************************************

procedure html is

cursor	driver (pcode varchar2, fcode varchar2) is
select	*
from	presentation_file_codes
where	presentation_code	= pcode
and	file_type_code		= fcode;

driver_rec	driver%ROWTYPE;

begin

   if common.header (rpt_title$) then
      common.js$;
      load_data;
      htp.tableOpen ('BORDER');

      htp.tableRowOpen ('LEFT', 'BOTTOM');
      htp.tableHeader ('Presentation', 'LEFT', crowspan=>'2', cattributes=>'VALIGN="MIDDLE"');
      for i in 1..frec.COUNT LOOP
         htp.tableHeader (frec(i).file_type_descr, 'CENTER', ccolspan=>'3');
      end loop;
      htp.tableRowClose;

-- **********

      htp.tableRowOpen ('CENTER', 'BOTTOM');
      for i in 1..frec.COUNT LOOP
         htp.tableHeader ('Weight');
         htp.tableHeader ('Editor Only');
         htp.tableHeader ('Always Available');
      end loop;
      htp.tableRowClose;

-- **********

      for i in 1..prec.COUNT LOOP
         htp.tableRowOpen ('CENTER', 'TOP');
         htp.tableHeader (prec(i).presentation_descr, 'LEFT', cnowrap=>'NOWRAP');

         for j in 1..frec.COUNT LOOP
            open driver (prec(i).presentation_code, frec(j).file_type_code);
            fetch driver into driver_rec;
            close driver;

            htp.tableData (htf.anchor ('javascript:popUp(''present_files.weight'	||
					'?pcode=' || prec(i).presentation_code	||
					'&fcode=' || frec(j).file_type_code	||
					'&w='	  || driver_rec.weight || ''')',
				nvl (driver_rec.weight, 0) ) );
            htp.tableData (htf.anchor ('present_files.yn'	||
					'?var=editor'		||
					'&pcode=' || prec(i).presentation_code	||
					'&fcode=' || frec(j).file_type_code,
				common.yes_no (driver_rec.editor_only)) );
            htp.tableData (htf.anchor ('present_files.yn'	||
					'?var=always'		||
					'&pcode=' || prec(i).presentation_code	||
					'&fcode=' || frec(j).file_type_code,
				common.yes_no (driver_rec.always_available)) );
         end loop;

         htp.tableRowClose;
      end loop;

      htp.tableClose;
      htp.para;
      common.footer;
   end if;

end html;

-- ***********************************************************************
-- ***********************************************************************

end present_files;
/
show error

set define on

--	Copyright (C) 2004 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>
