set define off

create or replace package body prog_cmtte as

rpt_title$	constant varchar2(50)	:= 'Program Committee';
js$		constant varchar2(500)	:= '<script language="JavaScript" src="' ||
						common.server ||
						common.vdir$  || 'prog_cmtte.js"></script>';
Q$		constant varchar2(1)	:= chr(39);

type		sort_varray is varray(6) of varchar2(50);
sort_opts	sort_varray := sort_varray ('Contribution ID', 'Author', 'Speaker/Presenter',
					'Main/Sub-Classification', 'Speaker/Presenter''s Affiliation',
					'Type of Presentation ');

type	rec_type is RECORD (
	presentation_descr	presentation_codes.presentation_descr%TYPE,
	classification_descr	classification_codes.classification_descr%TYPE,
	paper_id		abstract_sessions.paper_id%TYPE,
	abstract_id		abstracts.abstract_id%TYPE,
	title			abstracts.title%TYPE,
	presentation_code	presentation_codes.presentation_code%TYPE,
	classification_code	classification_codes.classification_code%TYPE,
	sub_class_code		sub_class_codes.sub_class_code%TYPE,
	description		abstracts.description%TYPE,
        primary_profile_id	profiles.profile_id%TYPE,
	spk_profile_id		profiles.profile_id%TYPE);

type	link_rec_type is record
	(	link_id		varchar2(10),
		link_text	varchar2(2000),
		msg		varchar2(2000) );

type	link_coll_type is table of link_rec_type index by binary_integer;

all$		constant varchar2(90)	:= 'Display all abstracts in the selected classification.';
changeClass$	constant varchar2(90)	:= 'Display only the abstracts proposed for a change in classification.';
changePresent$	constant varchar2(90)	:= 'Display only the abstracts proposed for a change in presentation type.';
notSelected$	constant varchar2(90)	:= 'Display the abstracts not proposed for a change.';
reject_class$	constant integer	:= 1;
reject_present$	constant integer	:= 2;

-- ****************************************************************************************
-- ****************************************************************************************

procedure JS is

begin

htp.p ('
function closePopWin() {
   opener.focus();
   window.close();
}');

end JS;

-- ****************************************************************************************

function is_coordinator (class_code varchar2, pid integer default common.gbl_pid) return boolean is

cursor	driver is
select	1
from	experts
where	person_id		= pid
and	classification_code	like class_code
and	coordinator		= 'Y';

dummy		integer;
rec_found	boolean;

begin

   if common.PC_mode$ = 'N' then
      return false;
   end if;

   if common.has_privs_to_page (common.jacow_admin$, false) then
      return true;
   end if;

   open driver;
   fetch driver into dummy;
   rec_found := driver%FOUND;
   close driver;

   return rec_found;

end is_coordinator;

-- ****************************************************************************************

function is_coordinator (aid integer) return boolean is

begin

   return is_coordinator (
		common.decode_value (
			'classification_code', 'abstracts',
			'abstract_id', aid));

end is_coordinator;

-- ****************************************************************************************
-- ****************************************************************************************

procedure save_abs (abs_id integer, this_title varchar2, this_description varchar2) is

begin

   common.get_gbl_pid;
   common.public_header (rpt_title$);

   if is_coordinator (abs_id) then
      if trim(this_title) || trim(this_description) is null then
         htp.header (3, 'You must supply a title and abstract.', cattributes=>common.red$);
         htp.bold (common.click_back$);
      else
         update	abstracts
         set	title		= trim (this_title),
		description	= trim (this_description)
         where	abstract_id	= abs_id;

         htp.header (3, 'Update Complete');
         htp.anchor ('#', 'Close Window', cattributes=>'onClick="window.close();"');
      end if;
   end if;

   common.footer;

end save_abs;

-- ****************************************************************************************

procedure edit_abs (abs_id integer) is

cursor	driver is
select	a.title, a.description, sa.paper_id
from	abstract_sessions	sa,
	abstracts		a
where	a.abstract_id		= abs_id
and	sa.abstract_id		= a.abstract_id
and	sa.primary_flag		is not null;

driver_rec	driver%ROWTYPE;

begin

   common.get_gbl_pid;
   common.public_header (rpt_title$);
   if is_coordinator (abs_id) then
--      htp.p (js$);
--      htp.p (profile.js$);
      htp.p (editor.js$);
      htp.para;

      open driver;
      fetch driver into driver_rec;
      close driver;

      htp.formOpen ('prog_cmtte.save_abs', 'POST', cattributes=>'NAME="SPMS"');
      htp.formHidden ('abs_id', abs_id);

      htp.tableOpen;

      common.show_data_row ('Paper ID',	driver_rec.paper_id);
      common.show_data_row ('Title',	htf.formText ('this_title', 75, 200, htf.escape_sc (driver_rec.title)) ||
		common.space2$ ||
		'<input type="button" name="Title Case" value="TitleCase" '	||
		'onClick="document.SPMS.this_title.value=document.SPMS.this_title.value.toLowerCase();'	||
			 'document.SPMS.this_title.value=document.SPMS.this_title.value.toTitleCase();"' );

      common.show_data_row ('Abstract*', 'Abstracts should not exceed ' || common.max_abstract_len$ || ' characters.');
      common.show_data_row (null, htf.formTextAreaOpen ('this_description', 10, 75, cattributes=>'WRAP=VIRTUAL')  ||
				replace (driver_rec.description, '"', '&quot;') || htf.formTextAreaClose);

      common.show_data_row (null,	htf.formSubmit (null, 'Save') );

      htp.tableClose;
      htp.formClose;

      htp.hr;
      common.show_abstract_log (abs_id, false);
   end if;

   common.footer;

end edit_abs;

-- ****************************************************************************************
-- ****************************************************************************************

procedure set_priority (aid integer, code varchar2) is

pid		integer;

begin

   common.public_header (rpt_title$);
   common.get_gbl_pid;
   pid	:= nvl (common.gbl_priv_pid, common.gbl_pid);

   if common.has_privs_to_page (common.set_proposal_page$, false) or is_coordinator (aid) then
      delete from abstract_proposals
      where	abstract_id	= aid
      and	person_id	= pid;

      if code is null then
         htp.header (3, 'Your proposal preference was cleared.');
      else
         insert into abstract_proposals
		(abstract_id, priority_code, proposal_status_date, person_id) 
         values (aid, code, sysdate, pid);

         htp.header (3,
		common.decode_value ('priority_descr', 'priority_codes', 'priority_code', code) ||
		' set for this proposal.');
      end if;

      htp.bold ('You will see your priority preference after you refresh the search results window.');
      htp.para;
      htp.p ('<input type="button" value="Close Window" onClick="window.close()">');
   end if;
   common.blank_footer;

end set_priority;

-- ****************************************************************************************

procedure set_proposal (aid integer, flag integer, code varchar2) is

-- flag = proposal status (accept or null)
-- code = priority code (can be null)

pid		integer;

begin

   common.public_header (rpt_title$);
   common.get_gbl_pid;
   pid	:= nvl (common.gbl_priv_pid, common.gbl_pid);

   if common.has_privs_to_page ('prog_cmtte.set_proposal', false) or is_coordinator (aid) then
      delete from abstract_proposals
      where	abstract_id	= aid
      and	person_id	= pid;

      if flag is null then
         htp.header (3, 'The priority was cleared.');
      else
         insert into abstract_proposals
		(abstract_id, priority_code, proposal_status_code, proposal_status_date, person_id) 
         values (aid, code, flag, sysdate, pid);

         htp.header (3,
		common.decode_value ('priority_descr', 'priority_codes', 'priority_code', code) ||
		' ' ||
		common.decode_value ('proposal_status_descr', 'proposal_status_codes', 'proposal_status_code', flag) ||
		' for this proposal.');
         htp.bold ('Note: Now that this abstract is accepted, individual preferences can no longer be set.');
      end if;

      htp.bold ('You will see the update after you refresh the search results window.');

      htp.para;
      htp.p ('<input type="button" value="Close Window" onClick="window.close()">');
   end if;
   common.blank_footer;

end set_proposal;

-- ****************************************************************************************
-- ****************************************************************************************

function is_expert (expert_role integer, pid integer default common.gbl_pid,
			ccode varchar2 default '%') return boolean is

cursor	driver is
select	1
from	experts
where	person_id		= pid
and	classification_code	like ccode
and	propose_reclass		like decode (expert_role, propose_reclass$,'Y', '%')
and	approve_reclass		like decode (expert_role, approve_reclass$,'Y', '%')
and	propose_present		like decode (expert_role, propose_present$,'Y', '%')
and	approve_present		like decode (expert_role, approve_present$,'Y', '%');

dummy		integer;
rec_found	boolean;

begin

-- Administrator always has access

   if common.user_is_jacow_admin then
      return true;
   end if;

-- Check system parameters first.

   if common.PC_mode$ = 'N' then
      if expert_role in (propose_reclass$, approve_reclass$) then
         if common.PC_reclassify_enabled$ = 'N' then
            return false;
         end if;
      elsif expert_role in (propose_present$, approve_present$) then
         if common.PC_present_enabled$ = 'N' then
            return false;
         end if;
      end if;
   end if;

-- Check system access

   if expert_role = propose_reclass$ then
      if common.has_privs_to_page ('prog_cmtte.class', false) then
         return true;
      end if;

   elsif expert_role = approve_reclass$ then
      if common.has_privs_to_page ('prog_cmtte.accept_cls', false) then
         return true;
      end if;

   elsif expert_role = propose_present$ then
      if common.has_privs_to_page ('prog_cmtte.present', false) then
         return true;
      end if;

   elsif expert_role = approve_present$ then
      if common.has_privs_to_page ('prog_cmtte.accept_present', false) then
         return true;
      end if;

   end if;

-- Check user access

   open driver;
   fetch driver into dummy;
   rec_found := driver%FOUND;
   close driver;

   return rec_found;

end is_expert;

-- ****************************************************************************************
-- ****************************************************************************************

procedure show_re_classifiation_link is

begin

   if common.reclass_instr_URL$ is not null then
      htp.anchor2 (common.reclass_instr_URL$, 'Instructions', ctarget=>'Instructions');
   end if;

end show_re_classifiation_link;

-- ****************************************************************************************
-- ****************************************************************************************

function withdrawing (abs_id integer, task integer) return boolean is

cursor	driver is
select	1
from	abstract_classifications
where	task		= reject_class$
and	abstract_id	= abs_id
and	person_id	= common.gbl_pid
UNION
select	1
from	abstract_presentations
where	task		= reject_present$
and	abstract_id	= abs_id
and	person_id	= common.gbl_pid;

rec_found	boolean;
dummy		integer;

begin

   open driver;
   fetch driver into dummy;
   rec_found := driver%FOUND;
   close driver;

   return rec_found;

end withdrawing;

-- ****************************************************************************************

function short_header (abs_id integer default null, task integer default null) return boolean is

cursor	driver is
select	abstract_id, title
from	abstracts
where	abstract_id = abs_id;

driver_rec	driver%ROWTYPE;

begin

   common.get_gbl_pid;

   if is_expert (approve_reclass$) OR is_expert (approve_present$) OR
	withdrawing (abs_id, task) then
      open driver;
      fetch driver into driver_rec;
      close driver;

      htp.htmlOpen;
      htp.headOpen;
      htp.title (rpt_title$);
      htp.headClose;
      htp.bodyOpen;

      if abs_id is not null then
         open driver;
         fetch driver into driver_rec;
         close driver;
         htp.bold (abs_id || common.space2$ || driver_rec.title);
         htp.para;
      end if;

      return true;
   end if;

   return false;

end short_header;

-- ****************************************************************************************
-- ****************************************************************************************

procedure update_complete is

begin

   htp.header (3, 'Update Complete');
   htp.p ('<input type=button value="Close" onClick="javascript:window.close();">');
   htp.para;
   htp.bold ('Note: You will not see your changes until you refresh the main screen.');
   htp.bodyClose;
   htp.htmlClose;

end update_complete;

-- ****************************************************************************************
-- ****************************************************************************************

procedure accept_present (abs_id integer, code varchar2) is

begin

   if short_header (abs_id) then
      htp.formOpen ('prog_cmtte.accept_present2', 'POST');
      htp.formHidden ('abs_id', abs_id);
      htp.formHidden ('code', code);
      htp.formSubmit (null, 'Change to ' ||
		common.decode_value ('presentation_descr', 'presentation_codes', 'presentation_code', code) ||
		'?');
      htp.formClose;
      htp.bodyClose;
      htp.htmlClose;
   end if;

end accept_present;

-- ****************************************************************************************

procedure accept_present2 (abs_id integer, code varchar2) is

begin

   update abstracts
   set	  presentation_code	= code
   where  abstract_id		= abs_id;

   reject_present2 (abs_id, code);  -- delete the existing requests

end accept_present2;

-- ****************************************************************************************
-- ****************************************************************************************

procedure reject_present (abs_id integer, code varchar2) is

descr$	constant presentation_codes.presentation_descr%TYPE :=
			common.decode_value ('presentation_descr', 'presentation_codes',
						'presentation_code', code);
buffer	varchar2(10)	:= 'Reject';

begin

   if short_header (abs_id, reject_present$) then

      if withdrawing (abs_id, reject_present$) then
         buffer := 'Withdraw';
      end if;

      htp.formOpen ('prog_cmtte.reject_present2', 'POST');
      htp.formHidden ('abs_id', abs_id);
      htp.formHidden ('code', code);
      htp.formSubmit (null, buffer || ' Proposal ' || descr$ || '?');
      htp.formClose;
      htp.bodyClose;
      htp.htmlClose;
   end if;

end reject_present;

-- ****************************************************************************************

procedure reject_present2 (abs_id integer, code varchar2) is

dummy	boolean;

begin

   if short_header (abs_id, reject_present$) then
      delete from abstract_presentations
      where  abstract_id		= abs_id
      and    presentation_code	= code;

      dummy := short_header (null, reject_present$);
      update_complete;
   end if;

end reject_present2;

-- ****************************************************************************************
-- ****************************************************************************************

procedure accept_cls (abs_id integer, ccode varchar2, scode varchar2) is

begin

   if short_header (abs_id) then
      htp.formOpen ('prog_cmtte.accept_cls2', 'POST');
      htp.formHidden ('abs_id', abs_id);
      htp.formHidden ('ccode', ccode);
      htp.formHidden ('scode', scode);
      htp.formSubmit (null, 'Reclassify to ' ||
	common.decode_value ('classification_descr', 'classification_codes',
				'classification_code', ccode) );
      htp.formClose;
      htp.bodyClose;
      htp.htmlClose;
   end if;

end accept_cls;

-- ****************************************************************************************

procedure accept_cls2 (abs_id integer, ccode varchar2, scode varchar2) is

begin

   update abstracts
   set	  classification_code	= ccode,
	  sub_class_code	= scode
   where  abstract_id		= abs_id;

   reject_cls2 (abs_id, '%', '%');  -- delete the existing requests

end accept_cls2;

-- ****************************************************************************************
-- ****************************************************************************************

procedure reject_cls (abs_id integer, ccode varchar2, scode varchar2) is

buffer	long;

begin

   if short_header (abs_id, reject_class$) then
      buffer := common.decode_value ('classification_descr', 'classification_codes', 'classification_code', ccode);
      if scode is not null then
         buffer := buffer || '/' ||
		common.decode_value ('sub_class_descr', 'sub_class_codes', 'sub_class_code', scode);
      end if;

      htp.formOpen ('prog_cmtte.reject_cls2', 'POST');
      htp.formHidden ('abs_id', abs_id);
      htp.formHidden ('ccode',	ccode);
      htp.formHidden ('scode',	scode);
      htp.formSubmit (null, 'Reject ' || buffer);
      htp.formClose;
      htp.bodyClose;
      htp.htmlClose;
   end if;

end reject_cls;

-- ****************************************************************************************

procedure reject_cls2 (abs_id integer, ccode varchar2, scode varchar2) is

dummy	boolean;

begin

   if short_header (abs_id, reject_class$) then
      delete from abstract_classifications
      where  abstract_id			= abs_id
      and	  classification_code		like ccode
      and	  nvl (sub_class_code, '~')	like nvl (scode, '~');

      dummy := short_header (null, reject_class$);
      update_complete;
   end if;

end reject_cls2;

-- ****************************************************************************************
-- ****************************************************************************************

function rc_OK return boolean is

begin

   common.get_gbl_pid;
   if common.PC_present_enabled$ = 'Y' OR common.PC_reclassify_enabled$ = 'Y' then

      if common.has_privs_to_page ('prog_cmtte.html', false)	    OR
		prog_cmtte.is_expert (prog_cmtte.PROPOSE_RECLASS$) OR
		prog_cmtte.is_expert (prog_cmtte.PROPOSE_PRESENT$) OR
		prog_cmtte.is_expert (prog_cmtte.APPROVE_RECLASS$) OR
		prog_cmtte.is_expert (prog_cmtte.APPROVE_PRESENT$) then
         common.public_header (rpt_title$);
         return true;
      else
         common.public_header (rpt_title$);
         htp.header (3, 'You do not have access to Abstract Re-classification or Change Presentation Type Functionallity');
         htp.bold ('Please contact the conference administrator.');
         htp.para;
         common.footer;
         return false;
      end if;

   else
      common.public_header (rpt_title$);
      htp.header (3, 'Abstract Re-classifications and Change Presentation Types Are Disabled');
      htp.bold ('Please contact the conference administrator.');
      htp.para;
      common.footer;
   end if;

   return false;

end rc_OK;

-- ****************************************************************************************
-- ****************************************************************************************

function list_present_codes (abs_id integer, code varchar2) return varchar2 is

cursor	driver is
select	common.get_user_affil_abbrev_list (person_id)	affiliations,
	common.get_user_name (person_id)		name,
	person_id
from	persons
where	person_id	in
(	select	person_id
	from	abstract_presentations	ap
	where	abstract_id		= abs_id
	and	presentation_code	= code
)
order by
	decode (person_id, nvl (common.gbl_priv_pid, common.gbl_pid), 0, 9),
	sort_lname, sort_fname, sort_mname;

buffer	long := null;
URL	varchar2(300);

begin

   for driver_rec in driver LOOP
      buffer := buffer || '; ';
      if driver_rec.person_id = nvl (common.gbl_priv_pid, common.gbl_pid) then
         if common.has_privs_to_page ('prog_cmtte.present_withd', false) then
            buffer := buffer || '<A HREF="javascript:popUp(''prog_cmtte.present_withd?aid=' ||
				abs_id || '&this_present=' || code || Q$ || ')">Withdraw My Proposal</A>' || common.space2$;
         end if;
      end if;
      buffer := buffer || htf.bold (driver_rec.name) || ' (' || driver_rec.affiliations || ')';
   end loop;

   if buffer is not null then
      buffer := substr (buffer, 3);
   end if;

   return buffer;

end list_present_codes;

-- ****************************************************************************************

function user_made_abs_propoal(abs_id integer, rec presentation_codes%ROWTYPE) return boolean is

cursor	driver is
select	1
from	abstract_presentations
where	abstract_id		= abs_id
and	person_id		= common.gbl_pid
and	presentation_code	= rec.presentation_code;

dummy		integer;
rec_found	boolean;

begin

   open driver;
   fetch driver into dummy;
   rec_found := driver%FOUND;
   close driver;

   return rec_found;

end user_made_abs_propoal;

-- ****************************************************************************************

procedure show_other_proposed_present (abs_id integer) is

cursor	driver is
select	*
from	presentation_codes
where	presentation_code	in
	(	select	presentation_code
		from	abstract_presentations
		where	abstract_id = abs_id);

prompt		varchar2(40) := 'Proposed Presentation';
buffer		varchar2(200)	:= null;
URL		varchar2(300);
ccode		abstracts.classification_code%TYPE;
an_expert	boolean;
user_proposed	boolean;

begin

   ccode := common.decode_value ('classification_code', 'abstracts', 'abstract_id', abs_id);

   for rec in driver LOOP

      an_expert		:= is_expert (APPROVE_PRESENT$, ccode=>ccode);
      user_proposed	:= user_made_abs_propoal(abs_id, rec);

      if an_expert then
         if common.freeze_abs_present$ = 'Y' then
            URL		:= common.quote$ || 'prog_cmtte.accept_present'		||
					'?abs_id='	|| abs_id	||
					'&code='	|| rec.presentation_code || common.quote$;
            URL		:= 'javascript:popUp(' || URL || ')';
            buffer	:= htf.anchor (URL, 'Accept') || common.space2$;
         end if;
      end if;

      if an_expert or user_proposed then
         URL	:= common.quote$ || 'prog_cmtte.reject_present'	||
					'?abs_id='	|| abs_id	||
					'&code='	|| rec.presentation_code || common.quote$;
         URL	:= 'javascript:popUp(' || URL || ')';

         if user_proposed then
            buffer := buffer || htf.anchor (URL, 'Withdraw') || common.space2$;
         else
            buffer := buffer || htf.anchor (URL, 'Reject') || common.space2$;
         end if;
      end if;

      common.show_data_row (prompt, buffer || rec.presentation_descr || ' ' ||
				list_present_codes (abs_id, rec.presentation_code));
      prompt := null;
      buffer := null;
      URL    := null;
   end loop;

end show_other_proposed_present;

-- ****************************************************************************************

function list_class_codes (abs_id integer, ccode varchar2, scode varchar2) return varchar2 is

cursor	driver is
select	common.get_user_affil_abbrev_list (person_id)	affiliations,
	common.get_user_name (person_id)		name,
	person_id
from	persons
where	person_id	in
(	select	person_id
	from	abstract_classifications
	where	abstract_id			= abs_id
	and	classification_code		= ccode
	and	nvl (sub_class_code, '~')	= nvl (scode, '~') )
order by
	decode (person_id, nvl (common.gbl_priv_pid, common.gbl_pid), 0, 9),
	sort_lname, sort_fname, sort_mname;
buffer	long := null;

begin

   for driver_rec in driver LOOP
      buffer := buffer || '; ';
      if driver_rec.person_id = nvl (common.gbl_priv_pid, common.gbl_pid) then
         buffer := buffer || common.space$ ||
			'<A HREF="javascript:popUp(''prog_cmtte.class_withd'	 ||
				'?aid=' || abs_id ||
				'&this_class=' || ccode || Q$ || ')">Withdraw My Proposal</A>' || common.space2$;
      end if;
      buffer := buffer || htf.bold (driver_rec.name) || ' (' || driver_rec.affiliations || ')';
   end loop;

   return substr (buffer, 3);

end list_class_codes;

-- ****************************************************************************************

procedure show_other_proposed_cls (abs_id integer) is

cursor	driver is
select	distinct
	common.decode_value ('classification_descr', 'classification_codes', 'classification_code', classification_code) cdescr,
	common.decode_value ('sub_class_descr', 'sub_class_codes', 'sub_class_code', sub_class_code) sdescr,
	classification_code, sub_class_code, decode (sub_class_code, null,null, '/') sep_chr
from	abstract_classifications
where	abstract_id	= abs_id;

prompt	varchar2(40)	:= 'Proposed Classification';
buffer	varchar2(2000)	:= null;
URL	varchar2(3000);

begin

   for driver_rec in driver LOOP

      if is_expert (APPROVE_RECLASS$, ccode=>driver_rec.classification_code) then
         if common.freeze_abs_reclass$ = 'Y' then
            URL		:= Q$ || 'prog_cmtte.accept_cls'			  ||
				'?abs_id='	|| abs_id			  ||
				'&ccode='	|| driver_rec.classification_code ||
				'&scode='	|| driver_rec.sub_class_code	  || Q$;
            URL		:= 'javascript:popUp(' || URL || ')';
            buffer	:= htf.anchor (URL, 'Accept') || common.space2$;
         end if;

         URL	:= Q$ || 'prog_cmtte.reject_cls'				||
			'?abs_id='	|| abs_id				||
			'&ccode='	|| driver_rec.classification_code	||
			'&scode='	|| driver_rec.sub_class_code		|| Q$;
         URL	:= 'javascript:popUp(' || URL || ')';
         buffer	:= buffer || htf.anchor (URL, 'Reject') || common.space2$;
      end if;

      common.show_data_row (prompt, buffer		||
				driver_rec.cdescr	||
				driver_rec.sep_chr	||
				driver_rec.sdescr	|| ' ' ||
		list_class_codes (abs_id, driver_rec.classification_code, driver_rec.sub_class_code));
      prompt := null;
      buffer := null;
   end loop;

end show_other_proposed_cls;

-- ****************************************************************************************

function get_proposed_class (abs_id integer) return varchar2 is

cursor	driver is
select	classification_code
from	abstract_classifications
where	abstract_id	= abs_id
and	person_id	= nvl (common.gbl_priv_pid, common.gbl_pid);

code		abstract_classifications.classification_code%TYPE := null;

begin

   open driver;
   fetch driver into code;
   close driver;

   return code;

end get_proposed_class;

-- ****************************************************************************************

function presentation_proposed (abs_id integer) return boolean is

cursor	driver is
select	1
from	abstract_presentations
where	abstract_id	= abs_id
and	person_id	= nvl (common.gbl_priv_pid, common.gbl_pid);

dummy		integer;
rec_found	boolean;

begin

   open driver;
   fetch driver into dummy;
   rec_found := driver%FOUND;
   close driver;

   return rec_found;

end presentation_proposed;

-- ****************************************************************************************

procedure class (aid integer, ptype varchar2) is

class_code	classification_codes.classification_code%TYPE;
sub_class_code	sub_class_codes.sub_class_code%TYPE;

buffer	varchar2(50) := null;

begin

   common.get_gbl_pid;
   htp.htmlOpen;
   htp.headOpen;
   htp.title (rpt_title$);
   htp.p (js$);
   htp.p (jacow.js$);
   htp.bodyOpen;
   htp.header (3, 'Propose a New Classification');

   htp.formOpen ('prog_cmtte.class2', 'POST', cattributes=>'NAME="ABSTRACT"');
   htp.formHidden ('aid', aid);

   class_code	:= common.decode_value ('classification_code', 'abstracts', 'abstract_id', aid);

   if common.require_sub_class$ = 'Y' then
      buffer := 'onChange="load_sub_class(this.value);"';
   end if;
   common.show_lookup_table  ('classification', htf.bold('Classification'), onChange=>buffer);
   htp.br;

   if common.require_sub_class$ = 'Y' then
      sub_class_code := common.decode_value ('sub_class_code', 'abstracts', 'abstract_id', aid);
      jacow.show_sub_class_lookup (htf.bold('Sub Classification'), class_code, sub_class_code);
   else
      htp.formHidden ('this_sub_class', ''); 
   end if;

   htp.br;
   htp.formSubmit (null, 'Save');
   htp.formClose;

   if common.require_sub_class$ = 'N' then
      htp.formHidden ('THIS_SUB_CLASS', '');
   end if;

   htp.bodyClose;
   htp.htmlClose;

end class;

-- ****************************************************************************************
-- ****************************************************************************************

procedure change_msg (action varchar2, this_classification varchar2, this_presentation varchar2) is

begin

   htp.header (3, 'Data Saved');
   htp.p ('Your name was');
   htp.p (action);
   htp.p ('the list of individuals who propose that this');
   htp.p ('contribution be');

   if this_classification is not null then
      htp.p ('reclassified to');
      htp.p (common.decode_value ('classification_descr', 'classification_codes',
				'classification_code', this_classification) );
   elsif this_presentation is not null then
      htp.p ('changed to');
      htp.bold (common.decode_value ('presentation_descr', 'presentation_codes',
					'presentation_code', this_presentation) );
   end if;

   htp.p ('.');
   htp.para;
   htp.bold ('Note: You will not see your proposals until you refresh the main screen.');

   htp.para;
   htp.anchor ('javascript:closePopWin();', 'Close Window');

   htp.bodyClose;
   htp.htmlClose;

end change_msg;

-- ****************************************************************************************

procedure class_withd (aid integer, this_Class varchar2) is

begin

   common.get_gbl_pid;
   delete from abstract_classifications
   where  person_id	= nvl (common.gbl_priv_pid, common.gbl_pid)
   and    abstract_id	= aid;

   htp.htmlOpen;
   htp.headOpen;
   htp.title (rpt_title$);
   htp.p (js$);
   htp.bodyOpen;

   change_msg ('removed from', this_class, null);

end class_withd;

-- ****************************************************************************************

procedure class2 (aid integer, this_classification varchar2 default null, this_sub_class varchar2 default null) is

continue	boolean		:= true;

begin

   common.get_gbl_pid;
   htp.htmlOpen;
   htp.headOpen;
   htp.title (rpt_title$);
   htp.p (js$);
   htp.bodyOpen;

   if this_classification is null then
      continue := false;
      htp.header (3, 'You must select a classification');
   end if;

   if common.require_sub_class$ = 'Y' then
      if this_sub_class is null then
         continue := false;
         htp.header (3, 'You must select a sub classification');
      end if;
   end if;

-- **********

   if continue then
      delete from abstract_classifications
      where  person_id		= nvl (common.gbl_priv_pid, common.gbl_pid)
      and    abstract_id	= aid;


      insert into abstract_classifications (abstract_id, person_id, classification_code, sub_class_code)
      values (aid, nvl (common.gbl_priv_pid, common.gbl_pid), this_classification, this_sub_class);

      change_msg ('added to', this_classification, null);
   else
      htp.bold (common.click_back$);
      htp.bodyClose;
      htp.htmlClose;
   end if;

end class2;

-- ****************************************************************************************
-- ****************************************************************************************

procedure present (aid integer) is

where_claws	varchar2(200);
present_code	presentation_codes.presentation_code%TYPE;

begin

   common.get_gbl_pid;
   htp.htmlOpen;
   htp.headOpen;
   htp.title (rpt_title$);
   htp.p (js$);
   htp.bodyOpen;
   htp.header (3, 'Propose a New Presentation Type');

   htp.formOpen ('prog_cmtte.present2', 'POST');
   htp.formHidden ('aid', aid);

   present_code	:= common.decode_value ('presentation_code', 'abstracts', 'abstract_id', aid);
   where_claws	:=	'where presentation_code   <> ' || Q$ || present_code || Q$ ||
			' and propose_oral = ' || Q$ || 'Y' || Q$;

   common.show_lookup_table  ('presentation', 'Presentation', where_claws=>where_claws);
   htp.br;
   htp.formSubmit (null, 'Save');
   htp.formClose;

   htp.bodyClose;
   htp.htmlClose;

end present;

-- ****************************************************************************************
-- ****************************************************************************************

procedure present_withd (aid integer, this_present varchar2) is

begin

   common.get_gbl_pid;
   htp.htmlOpen;
   htp.headOpen;
   htp.title (rpt_title$);
   htp.p (js$);
   htp.bodyOpen;

   delete from abstract_presentations
   where  person_id	= nvl (common.gbl_priv_pid, common.gbl_pid)
   and    abstract_id	= aid;

   change_msg ('removed from', null, this_present);

end present_withd;

-- ****************************************************************************************

procedure present2 (aid integer, this_presentation varchar2 default null) is

buffer	varchar2(20) := 'removed from';

begin

   common.get_gbl_pid;
   htp.htmlOpen;
   htp.headOpen;
   htp.title (rpt_title$);
   htp.p (js$);
   htp.bodyOpen;

   delete from abstract_presentations
   where  person_id	= nvl (common.gbl_priv_pid, common.gbl_pid)
   and    abstract_id	= aid;

   if this_presentation is not null then
      insert into abstract_presentations (abstract_id, person_id, presentation_code)
      values (aid, nvl (common.gbl_priv_pid, common.gbl_pid), this_presentation);
      buffer := 'added to';
   end if;

   change_msg ('added to', null, this_presentation);

end present2;

-- ****************************************************************************************
-- ****************************************************************************************

procedure show_abstract (driver_rec rec_type, proposing boolean) is

presentation_type	presentation_type_codes.presentation_type_code%TYPE;
buffer			varchar2(1000) := null;
present_privs$		constant boolean := is_expert(propose_present$, ccode=>driver_rec.classification_code);
reclass_privs$		constant boolean := is_expert (propose_reclass$, ccode=>driver_rec.classification_code);
edit_URL		varchar2(600)	 := null;

begin

   common.show_data_row (nvl (driver_rec.paper_id, to_char(driver_rec.abstract_id) ), driver_rec.title);

   if present_privs$ or reclass_privs$ then

      if common.PC_edit$ = 'Y' then
         if common.has_privs_to_page ('jacow.edit_abstract', false) then
            edit_URL := htf.anchor2 ('jacow.edit_abstract?abs_id=' || driver_rec.abstract_id, 'Edit', ctarget=>'SPMS');
            edit_URL := edit_URL || common.space2$;
         end if;

         if common.has_privs_to_page ('jacow.abs_authors', false) then
            edit_URL := edit_URL || htf.anchor2 ('jacow.abs_authors?abs_id=' || driver_rec.abstract_id, 'Authors', ctarget=>'SPMS');
            edit_URL := edit_URL;
         end if;
      end if;

      if edit_URL is not null then
         common.show_data_row (null, edit_URL);
      end if;

   end if;

-- **********

   if proposing AND present_privs$ AND common.PC_present_enabled$ = 'Y' AND
		NOT presentation_proposed (driver_rec.abstract_id) then
      buffer := common.space2$ || '<A HREF="javascript:popUp(''prog_cmtte.present'	||
		'?aid=' || driver_rec.abstract_id || Q$ ||  ')">Propose New Presentation Type</A>';
   end if;

   common.show_data_row ('Presentation', common.decode_value ('presentation_descr',
		'presentation_codes', 'presentation_code', driver_rec.presentation_code) || buffer);
   show_other_proposed_present (driver_rec.abstract_id);

-- **********

   buffer := null;
   if proposing AND reclass_privs$ AND common.PC_reclassify_enabled$ = 'Y' AND
		get_proposed_class (driver_rec.abstract_id) is null then
      presentation_type := common.decode_value ('presentation_type_code', 'presentation_codes',
						'presentation_code', driver_rec.presentation_code);
      buffer := common.space2$ || '<A HREF="javascript:popUp(''prog_cmtte.class?'	||
				'aid='	  || driver_rec.abstract_id ||
				'&ptype=' || presentation_type || ''')">Propose New Classification</A>';
   end if;

   common.show_data_row ('Classification', common.decode_value ('classification_descr',
		'classification_codes', 'classification_code', 	driver_rec.classification_code) || buffer);

   if common.require_sub_class$ = 'Y' then
      common.show_data_row ('Sub-Classification', common.decode_value ('sub_class_descr',
		'sub_class_codes', 'sub_class_code', 	driver_rec.sub_class_code));
   end if;

   show_other_proposed_cls (driver_rec.abstract_id);

-- **********

   common.show_data_row ('Speaker', common.get_user_name_affiliation (driver_rec.spk_profile_id, 'Y'));
   common.show_data_row ('Abstract', driver_rec.description);
   common.show_data_row ('Author', common.get_user_name_affiliation (driver_rec.primary_profile_id, 'Y'));
   common.show_data_row ('Co-Authors', common.get_author_list2 (driver_rec.abstract_id, false));

end show_abstract;

-- ****************************************************************************************
-- ****************************************************************************************

procedure reclass_list is

cursor	driver is
select	common.decode_value ('presentation_descr', 'presentation_codes',
				'presentation_code', presentation_code)	presentation_descr,
	common.decode_value ('classification_descr', 'classification_codes',
				'classification_code', classification_code)	classification_descr,
	common.get_paper_id (abstract_id) paper_id,
	abstract_id, title, presentation_code, classification_code, sub_class_code,
	description,
	common.get_author_profile_id (abstract_id, common.primary_author$) primary_profile_id,
	common.get_author_profile_id (abstract_id, common.speaker$) spk_profile_id
from	abstracts
where	abstract_id	in
(	select	abstract_id
	from	abstract_classifications
	where	classification_code	in
	(
		select	classification_code
		from	experts
		where	person_id	= nvl (common.gbl_priv_pid, common.gbl_pid)
		and	approve_reclass	= 'Y'
)	);

begin

   if common.header (rpt_title$) then
      show_re_classifiation_link;

      if common.freeze_abs_reclass$ = 'N' then
         htp.header (3, 'Note: You may not ACCEPT proposals until the conference administrator freezes user updates to classification and presenation type.');
      end if;

      htp.p (common.space2$);
      htp.anchor ('prog_cmtte.html', 'New Search');
      htp.para;

      htp.tableOpen;
      for driver_rec in driver LOOP
         show_abstract (driver_rec, false);
         common.show_data_sep (2);
      end loop;
      htp.tableClose;

      common.footer;
   end if;

end reclass_list;

-- ****************************************************************************************
-- ****************************************************************************************

function is_proposed_present (abs_id integer) return boolean is

cursor	driver is
select	1
from	abstract_presentations
where	abstract_id = abs_id;

icount	integer;

begin

   open driver;
   fetch driver into icount;
   close driver;

   if icount > 0 then
      return true;
   end if;

   return false;

end is_proposed_present;

-- ****************************************************************************************

function is_proposed_class (abs_id integer) return boolean is

cursor	driver is
select	1
from	abstract_classifications
where	abstract_id = abs_id;

icount	integer;

begin

   open driver;
   fetch driver into icount;
   close driver;

   if icount > 0 then
      return true;
   end if;

   return false;

end is_proposed_class;

-- ****************************************************************************************

function OK_to_show (opt varchar2, abs_id integer) return boolean is

begin


   if opt = all$ then
      return true;
   elsif opt = changeClass$ then
      return is_proposed_class (abs_id);
   elsif opt = changePresent$ then
      return is_proposed_present (abs_id);
   else
      return NOT (is_proposed_class(abs_id) OR is_proposed_present(abs_id) );
   end if;

end OK_to_show;

-- ****************************************************************************************

procedure list (this_classification varchar2, this_sub_class varchar2,
		sortby varchar2, opt varchar2) is

cursor	driver is
select	common.decode_value ('presentation_descr', 'presentation_codes',
				'presentation_code', presentation_code)	presentation_descr,
	common.decode_value ('classification_descr', 'classification_codes',
				'classification_code', classification_code)	classification_descr,
	common.get_paper_id (abstract_id) paper_id,
	abstract_id, title, presentation_code, classification_code, sub_class_code,
	description,
	common.get_author_profile_id (abstract_id, common.primary_author$) primary_profile_id,
	common.get_author_profile_id (abstract_id, common.speaker$) spk_profile_id
from	abstracts
where	abstract_id	in
(	select	abstract_id
	from	abstracts
	where	classification_code	 like nvl (this_classification, '%')
	and	nvl (sub_class_code,'~') like nvl (this_sub_class, '%')
	and	presentation_code	 in (select presentation_code from presentation_codes where reclassify = 'Y')
	UNION
	select	abstract_id
	from	abstract_classifications
	where	classification_code	= this_classification
	or	sub_class_code		= this_sub_class
)
order by
	decode (sortby,
	1, to_char (abstract_id, '000000'),
	2, common.decode_value ('sort_lname', 'persons', 'person_id', common.get_author_profile_id (abstract_id, common.primary_author$)),
	3, common.decode_value ('sort_lname', 'persons', 'person_id', common.get_author_profile_id (abstract_id, common.speaker$)),
	4, common.decode_value ('classification_descr', 'classification_codes', 'classification_code', classification_code),
	5, common.get_profile_affil (common.get_author_profile_id (abstract_id, common.speaker$), 'Y'),
	6, common.decode_value ('presentation_descr', 'presentation_codes', 'presentation_code', presentation_code) ),
	decode (sortby,
	4, common.decode_value ('sub_class_descr', 'sub_class_codes', 'sub_class_code', sub_class_code) ),
	to_char (abstract_id, '000000');

icount	integer := 0;

begin

   if rc_OK then
      common.js$;
      show_re_classifiation_link;
      htp.p (common.space2$);
      htp.anchor ('prog_cmtte.html', 'New Search');
      htp.para;

      htp.tableOpen;
      for driver_rec in driver LOOP
         if OK_to_show (opt, driver_rec.abstract_id) then
            common.show_data_sep (2);
            show_abstract (driver_rec, true);
            icount := icount + 1;
         end if;

      end loop;
      htp.tableClose;

      htp.header (3, 'Records Found: ' || icount);

   else
      htp.header (3, 'You are not authorized to access this page.', cattributes=>common.red$);
   end if;

   common.footer;

end list;

-- ****************************************************************************************
-- ****************************************************************************************

procedure show_link (noun varchar2, URL varchar2, icount integer) is

verb	varchar2(3) := 'are';
plural	varchar2(1) := 's';

begin

   if common.has_privs_to_page (URL, false) then

      if icount = 0 then
         htp.p ('There are no abstract ' || noun || 's pending approval.');
         htp.br;
      else

         if icount = 1 then
            verb	:= 'is';
            plural	:= null;
         end if;

         htp.p ('There ' || verb);
         htp.anchor (URL, icount || ' abstract ' || noun || plural);
         htp.p ('pending approval.');
         htp.br;
      end if;

   end if;

end show_link;

-- ****************************************************************************************

procedure show_pending_present_link is


cursor	driver is
select	count(1)
from	abstract_presentations
where	abstract_id	in
(
	select	abstract_id
	from	abstracts
	where	classification_code	in
	(
		select	classification_code
		from	experts
		where	person_id	= nvl (common.gbl_priv_pid, common.gbl_pid)
		and	approve_reclass	= 'Y'
)	)
OR	exists
(	select	1
	from	user_functions		uf,
		function_privileges	fp,
		application_privs	ap,
		applications		apps
	where	apps.application_name	in (expert.present_admin_page$, common.repository_URL$)
	and	ap.application_id	= apps.application_id
	and	fp.privilege_code	= ap.privilege_code
	and	uf.function_code	= fp.function_code
	and	uf.person_id		= nvl (common.gbl_priv_pid, common.gbl_pid)
);

icount	integer;

begin

   open driver;
   fetch driver into icount;
   close driver;

   show_link ('presentation change', 'prog_cmtte.present_list', icount);

end show_pending_present_link;

-- ****************************************************************************************
/*
function gen_expert_cls return varchar2 is

buffer	long := 'where classification_code in ';

begin

   if common.has_privs_to_page (expert.relass_page$, false) then
       return null;
   end if;

   if common.has_privs_to_page (expert.present_page$, false) then
       return null;
   end if;

   return buffer || '(select classification_code from experts where person_id = ' || common.gbl_pid ||
			' and (propose_reclass = ''Y'' OR propose_present = ''Y''))';

end gen_expert_cls;
*/
-- ****************************************************************************************

procedure html is

cursor driver is
select	presentation_descr
from	presentation_codes
where	reclassify = 'Y'
order by 1;

descr	common.ident_arr;

begin

   if rc_ok then

      htp.p (jacow.js$);
      htp.formOpen ('prog_cmtte.list', 'POST', cattributes=>'NAME="ABSTRACT"');
      htp.tableOpen;
      common.show_lookup_table ('CLASSIFICATION', 'Classification',
					onChange=>'onChange="load_sub_class(this.value);"');
      jacow.show_sub_class_lookup ('Sub Classification', null, null);

      htp.tableRowOpen ('LEFT', 'TOP');
      htp.tableHeader ('Sort Order', 'RIGHT');
      htp.p ('<TD>');
         htp.formSelectOpen ('sortby');
         htp.formSelectOption (sort_opts(1), 'SELECTED', cattributes=>'VALUE="1"');

         for i in 2..sort_opts.COUNT LOOP
            htp.formSelectOption (sort_opts(i), cattributes=>'VALUE="' || i || '"');
         end loop;
         htp.formSelectClose;
      htp.p ('</TD>');
      htp.tableRowClose;

      htp.tableRowOpen ('LEFT', 'TOP');
      htp.tableData (common.space$);
      htp.p ('<TD>');
         htp.formRadio ('opt', all$, 'CHECKED');
         htp.p (all$);
         htp.br;

         if common.PC_reclassify_enabled$ = 'Y' then
            htp.formRadio ('opt', changeClass$);
            htp.p (changeClass$);
            htp.br;
         end if;

         if common.PC_present_enabled$ = 'Y' then
            htp.formRadio ('opt', changePresent$);
            htp.p (changePresent$);
            htp.br;
         end if;

         htp.formRadio ('opt', notSelected$);
         htp.p (notSelected$);
         htp.br;
      htp.p ('</TD>');
      htp.tableRowClose;

      common.show_data_row (null, htf.formSubmit);
      htp.tableClose;
      htp.formClose;
      common.form_focus ('ABSTRACT', 'this_CLASSIFICATION');

      htp.para;

      open driver;
      fetch driver BULK COLLECT into descr;
      close driver;

      if descr.COUNT = 0 then
         htp.header (3, 'There are no presentation types set for reclassification at this time',
			cattributes=>common.red$);
      else
         htp.bold ('Note:');
         htp.p ('The following presentation types are available for reclassification.');
         htp.br;

         for i in 1..descr.COUNT loop
            htp.p (descr(i));
            htp.br;
         end loop;
      end if;

      common.footer;

   end if;

end html;

-- ****************************************************************************************

end prog_cmtte;
/
show error

set define on

--	Copyright (C) 2004 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>

-- ****************************************************************************************
