set define off

create or replace package body quick_regist as

-- **********************************************************************

procedure print_msg (msg varchar2, print boolean) is

begin

   if print then
      htp.p (msg);
   end if;

end print_msg;

-- **********************************************************************

procedure save_one (this_page integer, pid integer, print boolean default false) is

emessage	mail_blats_errors.error_message%TYPE;

begin

   update	registrants
   set		attended		= 'yes'
   where	page_id			= this_page
   and		person_id		= pid;

   if SQL%ROWCOUNT = 0 then
      insert into registrants (page_id, person_id, attended)
      select	this_page, pid, 'yes'
      from	dual
      where	not exists
		(	select	1
			from	registrants
			where	page_id		= this_page
			and	person_id	= pid);

      print_msg (pid || ',OKNEW', print);
   else
      htp.p (pid || ',OKUPD');
   end if;

exception
   when others then
      emessage	:= sqlerrm;
      print_msg (pid || ',ERR ' || emessage, print);

end save_one;

-- **********************************************************************

procedure sync (this_page integer, pass_phrase varchar2, ID common.ident_arr) is

begin

   if pass_phrase = common.quick_regist_passphrase$ then
      for i in 1..ID.COUNT loop
         save_one (this_page, ID(i), true);
      end loop;
   end if;

end sync;

-- **********************************************************************

procedure save (this_page integer, uid varchar2) is

ids	long;
ipos	integer;
ID	varchar2(30);
fail_safe integer := 0;

begin

   if common.header (rpt_title$) then
      htp.header (3, common.decode_value ('page_name', 'pages', 'page_id', this_page));
      htp.bold ('Recording registrants in SPMS');
      htp.para;

      ids := trim (uid);
      ids := replace (ids, chr(13) || chr(10), chr(10) );

      LOOP
         exit when ids is null;

         ipos	:= instr (ids, chr(10));
         if ipos = 0 then
            ID	:= ids;
            ids := null;
         else
            ID	:= trim (substr (ids, 1, ipos-1));
            ids := trim (substr (ids, ipos+1));
         end if;

         if NOT common.validate_number(ID) then
            ID := common.decode_value ('person_id', 'persons', 'user_id', ID);
         end if;

         if ID is not null then
            save_one (this_page, ID);

            htp.p (common.get_user_name (ID) || ',');
            htp.p (common.get_user_affil_abbrev_list (ID));
            htp.br;
         end if;

         fail_safe := fail_safe + 1;
         exit when fail_safe > 5000;
      end loop;

      htp.para;
      htp.anchor ('quick_regist.html', 'Record more registrants in SPMS');
      htp.para;
   end if;
   common.footer;

end save;

-- **********************************************************************

procedure html is

begin

   if common.header (rpt_title$) then
      htp.p (profile.js$);
      htp.header (3, 'Use this form to record persons registered in an external system');
      htp.formOpen ('quick_regist.save', 'POST', cattributes=>'NAME="SPMS"');

      htp.bold ('Select a registration page');
      htp.p (common.space2$);
      common.show_popup_list ('page');
      htp.para;

      htp.bold ('Enter registrant IDs; one per line');
      htp.anchor ('javascript:newWindow(''profile.find_account'')', 'Search for an Account');
      htp.br;

      htp.formTextArea ('uid', 40, 30);
      htp.para;

      htp.formSubmit;
      htp.formClose;
   end if;
   common.footer;

end html;

-- **********************************************************************

end quick_regist;
/
show error

set define on


--	Copyright (C) 2004 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>

set define on
