set define off

create or replace package body rpt_activity as

rpt_title$		constant varchar2 (30)	:= 'Activity Report';

-- ********************************************************************************************
-- ********************************************************************************************

procedure failed_QA (pid integer, total_edits integer) is

cursor	driver is
select	/*+ INDEX(L1) */
	abstract_id, log_date
from	logs		L1
where	person_id	= pid
and	log_code	= common.edit_complete$;

cursor	C2 (abs_id integer, ldate date) is
select	/*+ INDEX(l) */
	log_date
from	logs		l
where	abstract_id	= abs_id
and	log_date	> ldate
and	log_code	= common.edit_complete$;

cursor	C3 (abs_id integer, ldate date, ndate date) is
select	/*+ INDEX(L1) */
	1
from	logs		L1
where	abstract_id	= abs_id
and	log_code	= common.final_QA_failed$
and	log_date	between ldate and ndate;

dummy		integer;
next_date	date;
icount		integer := 0;
rate		number;

begin

   for driver_rec in driver LOOP

      next_date := null;
      open C2 (driver_rec.abstract_id, driver_rec.log_date);
      fetch C2 into next_date;
      close C2;

      next_date := nvl (next_date, sysdate+1);

      open C3 (driver_rec.abstract_id, driver_rec.log_date, next_date);
      fetch C3 into dummy;
      if C3%FOUND then
         icount := icount + 1;
      end if;
      close C3;
   end loop;

   if total_edits = 0 then
      htp.tableData (common.space$);
   elsif icount = 0 then
      htp.tableData (icount);
   else
      rate := round (icount*100/total_edits,1);
      htp.tableData (icount || ' (' || to_char (rate, '990.0') || '%)', cnowrap=>'NOWRAP' );
   end if;

end failed_QA;

-- ********************************************************************************************

function editor_status_count (pid integer, code varchar2) return integer is

cursor	driver is
select	/*+ INDEX(L1) */
	count(1)
from	logs		L1
where	person_id	= pid
and	status_code	= code
and	log_code	= common.edit_complete$
and	log_date	=
(
	select	/* INDEX(L2) */
	min (log_date)
	from	logs		L2
	where	L2.person_id	= pid
	and	L2.abstract_id	= L1.abstract_id
	and	L2.log_code	= L1.log_code
);

icount	integer;

begin

   open driver;
   fetch driver into icount;
   close driver;

   return icount;

end editor_status_count;

-- ********************************************************************************************

procedure html is

cursor	names is
select	/*+ INDEX(p) */
	distinct p.sort_lname, p.person_id, common.get_user_name (p.person_id) full_name
from	persons			p,
	user_functions		uf,
	function_privileges	fp,
	privilege_codes		pc
where	pc.activity_log		= 'Y'
and	fp.privilege_code	= pc.privilege_code
and	uf.function_code	= fp.function_code
and	p.person_id		= uf.person_id
and	exists
(	select	1
	from	logs		l
	where	l.person_id	= p.person_id
	and	l.log_code	in
	(	select	log_code
		from	log_codes
		where	activity_log = 'Y'
	)
)
order by 1;

cursor	log is
select	/*+ INDEX(lc) */
	log_code, log_descr
from	log_codes	lc
where	activity_log	= 'Y'
order by decode (log_code, common.edit_complete$, 'ZZZ', log_descr);

cursor	statuses is
select	status_code, status_descr
from	status_codes
where	status_code	not in
(	select	status_code
	from	log_codes
	where	log_code = common.paper_assigned$)
order by sort_order nulls last, status_descr;

cursor	counts (pid integer, code varchar2) is
select	/*+ INDEX(l) */
	count (distinct abstract_id)
from	logs		l
where	person_id	= pid
and	log_code	= code;

cursor	c_assigned is
select	count(1)
from	logs
where	log_code = common.paper_assigned$;

type	name_type	is table of names%ROWTYPE	index by binary_integer;
type	log_type	is table of log%ROWTYPE		index by binary_integer;
type	status_type	is table of statuses%ROWTYPE	index by binary_integer;

name_recs	name_type;
log_recs	log_type;
status_recs	status_type;

icount		integer;
last_icount	integer;
status_count	integer;
assigned	integer;

begin

   if common.header ('Editor Activity Log') then
      open log;
      fetch log BULK COLLECT into log_recs;
      close log;

      open statuses;
      fetch statuses BULK COLLECT into status_recs;
      close statuses;

      open names;
      fetch names BULK COLLECT into name_recs;
      close names;

      open c_assigned;
      fetch c_assigned into assigned;
      close c_assigned;

-- **********

      htp.tableOpen ('BORDER', cattributes=>'CELLPADDING="3" ' || common.table_hover$);

      htp.tableRowOpen ('CENTER', 'BOTTOM');
      htp.tableData (common.space$, crowspan=>'2');
      for i in 1..log_recs.COUNT LOOP
         htp.tableHeader (log_recs(i).log_descr, crowspan=>'2');
      end loop;

      htp.tableHeader ('Editors'' QA Failures', crowspan=>'2');
      htp.tableHeader ('Initial Dot Status', ccolspan=>status_recs.COUNT);
      htp.tableRowClose;

      htp.tableRowOpen ('CENTER', 'BOTTOM');
      for i in 1..status_recs.COUNT LOOP
         htp.tableHeader (status_recs(i).status_descr);
      end loop;
      htp.tableRowClose;

-- **********

      for i in 1..name_recs.COUNT LOOP
         htp.tableRowOpen ('CENTER', 'MIDDLE');
         htp.tableHeader (name_recs(i).full_name, 'LEFT');

         for j in 1..log_recs.COUNT LOOP
            open counts (name_recs(i).person_id, log_recs(j).log_code);
            fetch counts into icount;
            close counts;

            if log_recs(j).log_code = common.paper_assigned$ then
               if assigned <> 0 then
                  htp.tableData (icount || common.space2$ || '(' ||
				to_char (round (icount*100/assigned,2) || '%') || ')' );
               else
                  htp.tableData (icount);
               end if;
            else
               htp.tableData (icount);
            end if;
            last_icount := icount;
         end loop;

         failed_QA (name_recs(i).person_id, last_icount);

         for j in 1..status_recs.COUNT LOOP
            status_count := editor_status_count (name_recs(i).person_id, status_recs(j).status_code );
            if last_icount <> 0 then
               htp.tableData (round (status_count*100/last_icount, 1) || '%');
            else
               htp.tableData (common.space$);
            end if;
         end loop;

         htp.tableRowClose;
      end loop;

      htp.tableClose;
      common.footer;
   end if;

end html;

-- ********************************************************************************************
-- ********************************************************************************************

end rpt_activity;
/
show error

set define on

--	Copyright (C) 2004 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>
