set define off

create or replace package body rpt_chart as

-- ****************************************************************************************************

procedure chart (recs recs_type, chart_title varchar2) is

url	varchar2(4000) := 'http://chart.apis.google.com/chart?cht=lc&chs=1000x300&chxt=y,x';
url2	varchar2(4000) := null;
url3	varchar2(4000) := null;
sep	varchar2(1);
maxY	integer := 0;
totalY	integer := 0;

begin

   url2 := url2 || '&chtt=' || chart_title;  -- Title
   url3 := url3 || '&chtt=' || chart_title || ' Running Total';  -- Title

   url2 := url2 || '&chd=t:';  -- data points
   url3 := url3 || '&chd=t:';  -- data points
   sep := null;
   for i in 1..recs.COUNT LOOP
      totalY	:= totalY + recs(i).icount;
      url2	:= url2 || sep || recs(i).icount;
      url3	:= url3 || sep || totalY;

      sep	:= ',';
      maxY	:= greatest (maxY, recs(i).icount);
   end loop;

-- **********

   url2 := url2 || '&chds=0,' || maxY;  -- Data Scaling
   url3 := url3 || '&chds=0,' || totalY;  -- Data Scaling

-- **********

   url2 := url2 || '&chxl=0:||';  -- y-axis labels
   url3 := url3 || '&chxl=0:||';  -- y-axis labels
   sep := '|';

   for i in 1..maxY LOOP
      if mod (i, trunc(maxY/10) ) = 0 then
         url2 := url2 || i || sep;
      end if;
   end loop;

   for i in 1..totalY LOOP
      if mod (i, trunc(totalY/10) ) = 0 then
         url3 := url3 || i || sep;
      end if;
   end loop;

-- **********

   url2 := url2 || '1:';  -- x-axis labels
   url3 := url3 || '1:';  -- x-axis labels
   sep := '|';
   for i in 1..recs.COUNT LOOP
      url2 := url2 || sep;
      url3 := url3 || sep;
      if i = 1 or mod (i, trunc(recs.COUNT/10)) = 0 then
         url2 := url2 || sep || to_char (recs(i).log_date, 'dd-Mon-yy');
         url3 := url3 || sep || to_char (recs(i).log_date, 'dd-Mon-yy');
      end if;
   end loop;

   htp.img (url || url2);
   htp.para;
   htp.img (url || url3);

end chart;

-- ****************************************************************************************************
-- ****************************************************************************************************

procedure log_chart2 (code varchar2, descr varchar2) is

cursor	driver is
select	trunc (common.local_time_stamp(log_date)) log_date, count(1) icount
from	logs
where	log_code = code
group by trunc (common.local_time_stamp (log_date))
order by 1;

begin

   open driver;
   fetch driver BULK COLLECT into recs;
   close driver;

   if recs.COUNT > 0 then
      chart (recs, descr);
   end if;

end log_chart2;

-- ****************************************************************************************************

procedure log_chart is

cursor	driver is
select	log_code,
	replace (log_descr, ' ', '%20') log_descr
from 	log_codes
where	chart_flag = 'Y'
order by 2;

begin

   for rec in driver LOOP
      htp.hr;
      log_chart2 (rec.log_code, rec.log_descr);
   end loop;

end log_chart;

-- ****************************************************************************************************
-- ****************************************************************************************************

procedure regist_chart2 (ID integer, name varchar2) is

cursor	driver is
select	trunc (common.local_time_stamp(create_date)), count(1)
from	registrants
where	page_id	= ID
group by trunc (common.local_time_stamp(create_date))
order by 1;

begin

   open driver;
   fetch driver BULK COLLECT into recs;
   close driver;

   if recs.COUNT > 0 then
      chart (recs, name);
   end if;

end regist_chart2;

-- ****************************************************************************************************

procedure regist_chart is

cursor	driver is
select	page_id, replace (page_name, ' ', '%20') page_name
from	pages
order by 2;

begin

   for rec in driver LOOP
      htp.hr;
      regist_chart2 (rec.page_id, rec.page_name);
   end loop;

end regist_chart;

-- ****************************************************************************************************
-- ****************************************************************************************************

procedure html is

cursor	driver is
select	log_code,
	replace (log_descr, ' ', '%20') log_descr
from 	log_codes
where	chart_flag = 'Y'
order by 2;

begin

   common.public_header ('Activity Charts');
   log_chart;
   regist_chart;
   common.footer;

end html;

-- ****************************************************************************************************

end rpt_chart;
/
show error

set define on