set define off

create or replace package body rpt_class as

-- Raw data from each query

type		present_arr	is table of integer index by varchar2(10);

type		class_rec is record (
			name	varchar2(500),
			total	integer,
			counts	present_arr);
type		class_data is table of class_rec index by binary_integer;
classes		class_data;

type		present_rec is record (
			code	varchar2(10),
			descr	varchar2(500),
			total	integer);
type		present_data is table of present_rec index by binary_integer;

presents	present_data;  -- master list of presentation_codes;
raw		present_data;

gbl_abs_count	integer;
proposed$	constant varchar2(2)	:= '~%';

-- ********************************************************************
-- ********************************************************************

procedure sort_data is

temp		class_rec;
switched	boolean;

begin

   LOOP
      switched := false;

      for i in 2..classes.COUNT LOOP

--	 Primary Sort on total
         if classes(i-1).total < classes(i).total then
            temp	:= classes(i-1);
            classes(i-1):= classes(i);
            classes(i)	:= temp;
            switched	:= true;
         end if;

--	 Secondary Sort on Name
         if classes(i-1).total = classes(i).total then
            if nlssort (classes(i-1).name, 'NLS_SORT = Latin') >
		nlssort (classes(i).name, 'NLS_SORT = Latin') then
               temp		:= classes(i-1);
               classes(i-1)	:= classes(i);
               classes(i)	:= temp;
               switched		:= true;
            end if;
         end if;

      end loop;

      exit when NOT switched;
   end loop;

end sort_data;

-- ********************************************************************

procedure total_data is

begin

   for i in 1..presents.COUNT LOOP
      for j in 1..classes.COUNT LOOP
         presents(i).total := presents(i).total + classes(j).counts(presents(i).code);
      end loop;
   end loop;

end total_data;

-- ********************************************************************

procedure show_count (icount integer, total integer) is

percent		number;

begin

   htp.tableData (icount);
   if total = 0 then
      htp.tableData (common.space$);
   else
      percent := icount * 100 / total;
      htp.tableData (to_char (percent, '990.00') || '%');
   end if;

end show_count;

-- ********************************************************************

procedure show_totals is

total	integer := 0;

begin

   for i in 1..presents.COUNT LOOP
      if presents(i).code not like proposed$ then
         total := total + presents(i).total;
      end if;
   end loop;

   for i in 1..presents.COUNT LOOP
      show_count (presents(i).total, total);
   end loop;

   show_count (total, 0);

end show_totals;

-- ********************************************************************

procedure show_report (rpt_name varchar2) is

begin

   if common.header ('Contribution Counts by ' || rpt_name) then
      sort_data;
      total_data;
      gbl_abs_count := reports.get_abs_count;

      htp.tableOpen ('BORDER', cattributes=>'CELLPADDING="3"');
      htp.tableRowOpen ('CENTER', 'BOTTOM');
      htp.tableHeader ('#');
      htp.tableHeader (rpt_name, 'LEFT');

      for i in 1..presents.COUNT LOOP
         htp.tableHeader (presents(i).descr, ccolspan=>'2');
      end loop;

      htp.tableHeader ('Total', ccolspan=>'2');
      htp.tableRowClose;

      for i in 1..classes.COUNT LOOP
         htp.tableRowOpen ('CENTER', 'TOP');
         htp.tableData (i);
         htp.tableData (classes(i).name, 'LEFT');

         for j in 1..presents.COUNT LOOP
            show_count (classes(i).counts(presents(j).code), presents(j).total);
         end loop;

         show_count (classes(i).total, gbl_abs_count);
      end loop;

      htp.tableRowOpen ('CENTER', 'BOTTOM');
      htp.tableHeader ('Total Contributions', 'RIGHT', ccolspan=>'2');
      show_totals;
      htp.tableRowClose;

      htp.tableClose;
      common.footer;
   end if;

end show_report;

-- ********************************************************************
-- ********************************************************************

procedure load_presentation_codes is

cursor	driver is
select	presentation_code, presentation_descr, 0
from	presentation_codes
order by sort_order nulls last, presentation_descr;

cursor	reclassify is
select	distinct
	proposed$||pc.presentation_code,
	'Proposed' || htf.br || pc.presentation_descr, 0
from	presentation_codes	pc,
	abstract_presentations	ap
where	pc.presentation_code	= ap.presentation_code
order by 2;

rec2	present_data;

begin

   open driver;
   fetch driver BULK COLLECT into presents;
   close driver;

   if common.PC_present_enabled$ = 'Y' then
      open reclassify;
      fetch reclassify BULK COLLECT into rec2;
      close reclassify;

      for i in 1..rec2.COUNT loop
         presents(presents.COUNT+1) := rec2(i);
      end loop;
   end if;

end load_presentation_codes;

-- ********************************************************************

procedure move_raw_to_classes is

previous_descr	varchar2(500) := '~';
idx		integer;

begin

   for i in 1..raw.COUNT LOOP
      if previous_descr <> raw(i).descr then  -- initialize
         idx := classes.COUNT + 1;
         classes(idx).name	:= raw(i).descr;
         classes(idx).total	:= 0;

         for i in 1..presents.COUNT LOOP
            classes(idx).counts(presents(i).code) := 0;
         end loop;
      end if;

      if raw(i).code not like proposed$ then
         classes(idx).total := classes(idx).total + raw(i).total;
      end if;

      classes(idx).counts(raw(i).code) := raw(i).total;
      previous_descr := raw(i).descr;
   end loop;

end move_raw_to_classes;

-- ********************************************************************
-- ********************************************************************
-- ********************************************************************

procedure by_affiliation is

cursor	driver is
select	abs.presentation_code,
	affil.name1 ||
	decode (affil.affil_abbrev, null,' ', ' (' || affil.affil_abbrev || ') ' ),
	count(1)
from	affiliations		affil,
	addresses		addr,
	profiles		pro,
	authors			auth,
	abstracts		abs
where	auth.abstract_id	= abs.abstract_id
and	auth.author_code	= common.speaker$
and	pro.profile_id		= auth.profile_id
and	addr.address_id		= pro.address_id
and	affil.affiliation_id	= addr.affiliation_id
group by abs.presentation_code,	affil.name1,
	decode (affil.affil_abbrev, null,' ', ' (' || affil.affil_abbrev || ') ' )
UNION
select	proposed$||abs.presentation_code,
	affil.name1 ||
	decode (affil.affil_abbrev, null,' ', ' (' || affil.affil_abbrev || ') ' ),
	count(distinct abs.abstract_id)
from	affiliations		affil,
	addresses		addr,
	profiles		pro,
	authors			auth,
	abstract_presentations	abs
where	'Y'			= common.PC_present_enabled$
and	auth.abstract_id	= abs.abstract_id
and	auth.author_code	= common.speaker$
and	pro.profile_id		= auth.profile_id
and	addr.address_id		= pro.address_id
and	affil.affiliation_id	= addr.affiliation_id
group by abs.presentation_code,	affil.name1,
	decode (affil.affil_abbrev, null,' ', ' (' || affil.affil_abbrev || ') ' )
order by 2, 1;

begin

   open driver;
   fetch driver BULK COLLECT into raw;
   close driver;

   load_presentation_codes;
   move_raw_to_classes;
   show_report ('Affiliation');

end by_affiliation;

-- ********************************************************************
-- ********************************************************************

procedure by_country is

cursor	driver is
select	abs.presentation_code, cc.country_descr, count(1)
from	country_codes		cc,
	addresses		addr,
	profiles		pro,
	authors			auth,
	abstracts		abs
where	auth.abstract_id	= abs.abstract_id
and	auth.author_code	= common.speaker$
and	pro.profile_id		= auth.profile_id
and	addr.address_id		= pro.address_id
and	cc.country_code		= addr.country_code
group by abs.presentation_code, cc.country_descr
UNION
select	proposed$||abs.presentation_code, cc.country_descr, count(distinct abs.abstract_id)
from	country_codes		cc,
	addresses		addr,
	profiles		pro,
	authors			auth,
	abstract_presentations	abs
where	'Y'			= common.PC_present_enabled$
and	auth.abstract_id	= abs.abstract_id
and	auth.author_code	= common.speaker$
and	pro.profile_id		= auth.profile_id
and	addr.address_id		= pro.address_id
and	cc.country_code		= addr.country_code
group by abs.presentation_code, cc.country_descr
order by 2, 1;

begin

   open driver;
   fetch driver BULK COLLECT into raw;
   close driver;

   load_presentation_codes;
   move_raw_to_classes;
   show_report ('Country');

end by_country;

-- ********************************************************************
-- ********************************************************************

procedure by_region is

cursor	driver is
select	abs.presentation_code,
	nvl (rc.region_descr, '?') region_descr,
	count(1) icount
from	region_codes		rc,
	country_codes		cc,
	addresses		addr,
	profiles		pro,
	authors			auth,
	abstracts		abs
where	auth.abstract_id	= abs.abstract_id
and	auth.author_code	= common.speaker$
and	pro.profile_id		= auth.profile_id
and	addr.address_id		= pro.address_id
and	cc.country_code		= addr.country_code
and	rc.region_code(+)	= cc.region_code
group by	abs.presentation_code, rc.region_descr
UNION
select	proposed$||abs.presentation_code,
	nvl (rc.region_descr, '?') region_descr,
	count(distinct abs.abstract_id)
from	region_codes		rc,
	country_codes		cc,
	addresses		addr,
	profiles		pro,
	authors			auth,
	abstract_presentations	abs
where	'Y'			= common.PC_present_enabled$
and	auth.abstract_id	= abs.abstract_id
and	auth.author_code	= common.speaker$
and	pro.profile_id		= auth.profile_id
and	addr.address_id		= pro.address_id
and	cc.country_code		= addr.country_code
and	rc.region_code(+)	= cc.region_code
group by abs.presentation_code, rc.region_descr
order by 2, 1;

begin

   open driver;
   fetch driver BULK COLLECT into raw;
   close driver;

   load_presentation_codes;
   move_raw_to_classes;
   show_report ('Region');

end by_region;

-- ********************************************************************

procedure by_sregion is

cursor	driver is
select	abs.presentation_code,
	nvl (src.super_region_descr, '?') sregion_descr,
	count(1) icount
from	super_region_codes	src,
	region_codes		rc,
	country_codes		cc,
	addresses		addr,
	profiles		pro,
	authors			auth,
	abstracts		abs
where	auth.abstract_id	= abs.abstract_id
and	auth.author_code	= common.speaker$
and	pro.profile_id		= auth.profile_id
and	addr.address_id		= pro.address_id
and	cc.country_code		= addr.country_code
and	rc.region_code(+)	= cc.region_code
and	src.super_region_code(+)= rc.super_region_code
group by	abs.presentation_code, src.super_region_descr
UNION
select	proposed$||abs.presentation_code,
	nvl (src.super_region_descr, '?') sregion_descr,
	count(distinct abs.abstract_id)
from	super_region_codes	src,
	region_codes		rc,
	country_codes		cc,
	addresses		addr,
	profiles		pro,
	authors			auth,
	abstract_presentations	abs
where	'Y'			= common.PC_present_enabled$
and	auth.abstract_id	= abs.abstract_id
and	auth.author_code	= common.speaker$
and	pro.profile_id		= auth.profile_id
and	addr.address_id		= pro.address_id
and	cc.country_code		= addr.country_code
and	rc.region_code(+)	= cc.region_code
and	src.super_region_code(+)= rc.super_region_code
group by abs.presentation_code, src.super_region_descr
order by 2, 1;

begin

   open driver;
   fetch driver BULK COLLECT into raw;
   close driver;

   load_presentation_codes;
   move_raw_to_classes;
   show_report ('Super Region');

end by_sregion;

-- ********************************************************************
-- ********************************************************************

procedure by_session is

cursor	driver is
select	cc.classification_descr, sc.sub_class_descr,
	s.session_id, s.session_date, s.beg_time,
	count(abs.abstract_id) icount
from	abstracts		abs,
	sub_class_codes		sc,
	classification_codes	cc,
	class_sessions		cs,
	sessions		s
where	cs.session_id		= s.session_id
and	cs.sort_order		=
	(	select	min (sort_order)
		from	class_sessions			cs2
		where	cs2.classification_code		= cs.classification_code
		and	nvl (cs2.sub_class_code,'~')	= nvl (cs.sub_class_code,'~')
	)
and	cc.classification_code	= cs.classification_code
and	sc.sub_class_code(+)	= cs.sub_class_code
and	abs.classification_code	= cs.classification_code
and	abs.presentation_code	in
	(	select	presentation_code
		from	presentation_codes
		where	presentation_type_code = common.poster$
	)
and	abs.sub_class_code(+)	= cs.sub_class_code
group by cc.classification_descr, sc.sub_class_descr,
	s.session_id, s.session_date, s.beg_time
UNION
select	cc.classification_descr, sc.sub_class_descr,
	s.session_id, s.session_date, s.beg_time,
	count(abs.abstract_id) icount
from	abstracts		abs,
	sub_class_codes		sc,
	classification_codes	cc,
	eclass_codes		ec,
	class_sessions		cs,
	sessions		s
where	cs.session_id		= s.session_id
and	cs.sort_order		=
	(	select	min (sort_order)
		from	class_sessions	cs2
		where	cs2.eclass_code	= cs.eclass_code
	)
and	ec.eclass_code		= cs.eclass_code
and	abs.eclass_code		= cs.eclass_code
and	abs.presentation_code	in
	(	select	presentation_code
		from	presentation_codes
		where	presentation_type_code = common.poster$
	)
and	cc.classification_code	= ec.classification_code
and	sc.sub_class_code(+)	= ec.sub_class_code
group by cc.classification_descr, sc.sub_class_descr,
	s.session_id, s.session_date, s.beg_time
order by 5, 4, 3, 1, 2;

type		rec_type is table of driver%ROWTYPE;
recs		rec_type;
total		integer := 0;
gtotal		integer := 0;
prev_session	sessions.session_id%TYPE := null;

begin

   if common.header ('Contribution Counts by Session Assignment' ) then
      open driver;
      fetch driver BULK COLLECT into recs;
      close driver;

      htp.tableOpen ('BORDER', cattributes=>'CELLPADDING="3"');
      htp.tableRowOpen ('LEFT', 'BOTTOM');
      htp.tableHeader ('Session', ccolspan=>'2');
      htp.tableHeader ('Classification');
      htp.tableHeader ('Sub-Classification');
      htp.tableHeader ('Count');
      htp.tableRowClose;

      for i in 1..recs.COUNT LOOP
         if prev_session <> recs(i).session_id then
            htp.tableRowOpen ('LEFT', 'BOTTOM');
            htp.tableHeader (prev_session || ' Total', 'RIGHT', ccolspan=>'4');
            htp.tableHeader (total, 'CENTER');
            htp.tableRowClose;
            total := 0;
         end if;

         recs(i).session_date := common.add_time (recs(i).session_date, recs(i).beg_time);
         htp.tableRowOpen ('LEFT', 'TOP');
         htp.tableData (recs(i).session_id);
         htp.tableData (to_char (recs(i).session_date, common.date_fmt_str$), cnowrap=>'nowrap' );
         htp.tableData (recs(i).classification_descr);
         htp.tableData (recs(i).sub_class_descr);
         htp.tableData (recs(i).icount, 'CENTER');

         prev_session	:= recs(i).session_id;
         total		:= total + recs(i).icount;
         gtotal		:= gtotal + recs(i).icount;
      end loop;

      htp.tableRowOpen ('LEFT', 'BOTTOM');
      htp.tableHeader (prev_session || ' Total', 'RIGHT', ccolspan=>'4');
      htp.tableHeader (total, 'CENTER');
      htp.tableRowClose;

      htp.tableRowOpen ('LEFT', 'BOTTOM');
      htp.tableHeader ('Grand Total', 'RIGHT', ccolspan=>'4');
      htp.tableHeader (gtotal, 'CENTER');
      htp.tableRowClose;

      htp.tableClose;
      common.footer;
   end if;

end by_session;

-- ********************************************************************

end rpt_class;
/
show error

set define on

--	Copyright (C) 2004 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>
