set define off

Create or Replace package body rpt_master_status as

rpt_title$	constant varchar2(20)	:= 'Master Status Report';

-- ***********************************************************************************

procedure show_log_codes (id integer) is

cursor	driver is
select	error_descr
from	error_codes	ec,
	log_errors	le
where	le.log_id	= id
and	ec.error_code	= le.error_code
order by 1;

begin

   for rec in driver LOOP
      htp.p (rec.error_descr);
      htp.br;
   end loop;

end show_log_codes;

-- ***********************************************************************************

procedure show_error_log (abs_id integer, ecode varchar2, rcode varchar2) is

cursor	driver is
select	/*+ FIRST_ROW */
	log_id, comments
from	logs
where	abstract_id	= abs_id
and  (	status_code	= ecode
     or referee_code	= rcode)
order by log_date desc;

rec	driver%ROWTYPE;

begin

   open driver;
   fetch driver into rec;
   close driver;

   htp.p ('<TD ALIGN="LEFT">');

   if rec.log_id is not null then
      show_log_codes (rec.log_id);
   end if;

   if rec.comments is not null then
      htp.p (rec.comments);
   end if;

   htp.p ('</TD>');

end show_error_log;

-- ***********************************************************************************

function copyright_received (abs_id integer) return varchar2 is

cursor	driver is
select	/*+ INDEX(l) */	1
from	logs			l
where	abstract_id		= abs_id
and	log_code		= common.copyright_received$;

dummy		integer;
rec_found	boolean;

begin

   open driver;
   fetch driver into dummy;
   rec_found := driver%FOUND;
   close driver;

   if rec_found then
      return 'Yes';
   end if;

   return 'No';

end copyright_received;

-- ***********************************************************************************

procedure rpt (this_status varchar2, this_referee varchar2) is

cursor	driver is
select	a.abstract_id, sa.paper_id, a.editor_id, a.status_code, a.referee_code
from	abstract_sessions		sa,
	abstracts			a
where	nvl (a.status_code, '~')	like nvl (this_status, '~')
and	nvl (a.referee_code, '~')	like nvl (this_referee, '~')
and	sa.abstract_id			= a.abstract_id
and	sa.primary_flag			is not null
order by sa.session_id, sa.session_seq;

icount	integer := 0;

begin


if common.header (rpt_title$) then
   htp.tableOpen ('BORDER');

   htp.tableRowOpen ('LEFT', 'BOTTOM');
   htp.tableHeader ('Paper ID',	crowspan=>'2');
   htp.tableHeader ('Files',	crowspan=>'2');
   htp.tableHeader ('Final QA',	crowspan=>'2');

   if common.copyright_required$ = 'Y' then
      htp.tableHeader ('Copyright Received', crowspan=>'2');
   end if;

   if common.referee_conf$ = 'Y' then
      htp.tableHeader ('Referee', 'CENTER', ccolspan=>'3');
   end if;

   htp.tableHeader ('Editor', 'CENTER', ccolspan=>'3');
   htp.tableRowClose;

   htp.tableRowOpen ('LEFT', 'BOTTOM');

   if common.referee_conf$ = 'Y' then
      htp.tableHeader ('Status');
      htp.tableHeader ('Name');
      htp.tableHeader ('Log/Notes');
   end if;

   htp.tableHeader ('Status');
   htp.tableHeader ('Name');
   htp.tableHeader ('Log/Notes');
   htp.tableRowClose;

   for driver_rec in driver LOOP
      htp.tableRowOpen ('CENTER', 'TOP');
      htp.tableData (htf.anchor ('search.abs_id?aid=' || driver_rec.abstract_id,
			driver_rec.paper_id), 'LEFT');

      if editor.new_files (driver_rec.abstract_id) = 'YES' then
	 htp.tableData ('New');
      elsif editor.files_uploaded (driver_rec.abstract_id) then
	 htp.tableData ('Yes');
      else
	 htp.tableData ('No');
      end if;

      htp.tableData (common.final_QA (driver_rec.abstract_id));

      if common.copyright_required$ = 'Y' then
         htp.tableData (copyright_received (driver_rec.abstract_id));
      end if;

      if common.referee_conf$ = 'Y' then
	 if driver_rec.referee_code is null then
	    htp.tableData (common.space$);
	 else
	    htp.tableData (common.get_dot_image (driver_rec.referee_code, common.referee_dot$));
	 end if;

	 htp.p ('<TD ALIGN="LEFT">');
	 referee.show_referees (driver_rec.abstract_id, true);
	 htp.p ('</TD>');

         show_error_log (driver_rec.abstract_id, null, driver_rec.referee_code);
      end if;

      if driver_rec.status_code is null then
	 htp.tableData (common.space$);
      else
	 htp.tableData (common.get_dot_image (driver_rec.status_code, common.editor_dot$));
      end if;

      htp.tableData (nvl (common.get_user_name (driver_rec.editor_id), common.space$), 'LEFT');
      show_error_log (driver_rec.abstract_id, driver_rec.status_code, null);

      htp.tableRowClose;
      icount := icount + 1;
   end loop;

   htp.tableClose;

   htp.para;
   htp.bold ('Found:');
   htp.bold (icount);
   common.footer;
end if;

end rpt;

-- ***********************************************************************************

procedure show_popup_list (the_list varchar2) is

cursor	driver is
select	sort_order, status_code, status_descr, 'status' list_name
from	status_codes
UNION
select	sort_order, referee_code, referee_descr, 'referee'
from	referee_codes
order by 1 nulls last, 3;

begin

   htp.formSelectOpen ('this_' || the_list);
   htp.formSelectOption ('*** All ***', 'selected', cattributes=>'VALUE="%"');
   htp.formSelectOption ('*** No Status ***', null, cattributes=>'VALUE=""');

   for rec in driver LOOP
      if rec.list_name = the_list then
         htp.formSelectOption (rec.status_descr, null,
		cattributes=>'VALUE="' || rec.status_code || '"');
      end if;
   end loop;

   htp.formSelectClose;

end show_popup_list;

-- ***********************************************************************************

procedure html is

begin

   if common.header (rpt_title$) then
      htp.formOpen ('rpt_master_status.rpt', 'POST', cattributes=>'name="RPT"');
      htp.header (3, 'Select an editor status');
      show_popup_list ('status');

      if common.referee_conf$ = 'Y' then
         htp.header (3, 'Select a referee status');
         show_popup_list ('referee');
      else
         htp.formHidden ('this_referee', '');
      end if;

      htp.formSubmit;
      htp.formClose;
      common.form_focus ('RPT', 'this_status');
      htp.para;
      common.footer;
   end if;

end html;

-- ***********************************************************************************************

end rpt_master_status;

/
show error

set define on

--	Copyright (C) 2004-2008 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>
