create or replace package body RPT_papers_by_platform as

type	extension_type	is record (
		assigned	integer := 0,
		unassigned	integer := 0);
type	extensions_type	is table of extension_type	index by varchar2(1000);

type	platform_type	is record (
		platform	varchar2(1000),
		extension	extensions_type);
type	platforms_type	is table of platform_type	index by varchar2(1000);

recs	platforms_type;

-- ****************************************************************************************

function get_platforms (abs_id integer) return varchar2 is

cursor	driver is
select	distinct platform_descr
from	presentation_file_codes	pfc,
	platform_codes		pc,
	files			f
where	f.abstract_id		= abs_id
and	exists
	(
		select	1
		from	source_codes	sc
		where	sc.source_code	<> 'Any'
		and	regexp_like (f.file_name, sc.reg_exp, 'i')
	)
and	pc.platform_code	= f.platform_code
and	pfc.file_type_code	= f.file_type_code
and	pfc.weight		> 0
and	pfc.editor_only		= 'N'
order by 1;

buffer	varchar2(1000) := null;	

begin

   for rec in driver LOOP
      buffer := buffer || ' / ' || rec.platform_descr;
   end loop;

   return substr (buffer, 4);

end get_platforms;

-- ****************************************************************************************

function get_file_extensions (abs_id integer) return varchar2 is

cursor	driver is
select	distinct upper (substr (f.file_name, instr(f.file_name, '.', -1) + 1)) extension
from	presentation_file_codes	pfc,
	files			f
where	f.abstract_id		= abs_id
and	exists
	(
		select	1
		from	source_codes	sc
		where	sc.source_code	<> 'Any'
		and	regexp_like (f.file_name, sc.reg_exp, 'i')
	)
and	pfc.file_type_code	= f.file_type_code
and	pfc.weight		> 0
and	pfc.editor_only		= 'N'
order by 1;

buffer	varchar2(1000) := null;	

begin

   for rec in driver LOOP
      buffer := buffer || ' / ' || rec.extension;
   end loop;

   return substr (buffer, 4);

end get_file_extensions;

-- ****************************************************************************************

procedure load_data is

cursor	driver is
select	abstract_id, assignable, editor_id
from	abstracts
where	assignable > 0;

file_platform	varchar2(1000);
file_ext	varchar2(1000);
new_element	boolean;

test	extensions_type;

begin

   for rec in driver LOOP
      file_platform	:= get_platforms (rec.abstract_id);
      file_ext		:= get_file_extensions (rec.abstract_id);

      if file_platform is not null and file_ext is not null then
         new_element := false;

         if NOT recs.EXISTS(file_platform) then
            new_element := true;
         elsif NOT recs(file_platform).extension.EXISTS(file_ext) then
            new_element := true;
         end if;

         if new_element then
            recs(file_platform).extension(file_ext).unassigned := 0;
            recs(file_platform).extension(file_ext).assigned := 0;
         end if;

         if rec.editor_id is null then
		recs(file_platform).extension(file_ext).unassigned :=
		recs(file_platform).extension(file_ext).unassigned + 1;
         else
		recs(file_platform).extension(file_ext).assigned :=
		recs(file_platform).extension(file_ext).assigned + 1;
         end if;

      end if;

   end loop;

end load_data;

-- ****************************************************************************************

procedure display_data (show_all boolean default true) is

file_platform	varchar2(1000);
file_ext	varchar2(1000);
fail_safe	integer := 0;
total1		integer := 0;
total2		integer := 0;

begin

   if show_all then
      htp.header (3, 'Papers By Platform/File Type');
   else
      htp.header (3, 'Unassigned Papers By Platform/File Type');
   end if;

   htp.tableOpen ('BORDER');
   htp.tableRowOpen ('RIGHT', 'BOTTOM');
   htp.tableHeader ('Platform', 'LEFT');
   htp.tableHeader ('File Types', 'LEFT');
   htp.tableHeader ('Assigned');
   htp.tableHeader ('Unassigned');
   htp.tableRowClose;

   file_platform := recs.FIRST;
   LOOP
      exit when file_platform is null;
      fail_safe := fail_safe + 1;
      exit when fail_safe > 10000;

      file_ext := recs(file_platform).extension.FIRST;
      LOOP
         exit when file_ext is null;
         fail_safe := fail_safe + 1;
         exit when fail_safe > 10000;

         if recs(file_platform).extension(file_ext).unassigned > 0 OR show_all then
            total1 := total1 + recs(file_platform).extension(file_ext).assigned;
            total2 := total2 + recs(file_platform).extension(file_ext).unassigned;

            htp.tableRowOpen ('RIGHT', 'TOP');
            htp.tableData (file_platform, 'LEFT');
            htp.tableData (file_ext, 'LEFT');
            htp.tableData (recs(file_platform).extension(file_ext).assigned);
            htp.tableData (recs(file_platform).extension(file_ext).unassigned);
            htp.tableRowClose;
         end if;

         file_ext := recs(file_platform).extension.next(file_ext);
      end loop;

      file_platform := recs.NEXT(file_platform);
   end loop;

   htp.tableRowOpen ('RIGHT', 'BOTTOM');
   htp.tableHeader ('Totals', ccolspan=>'2');
   htp.tableHeader (total1);
   htp.tableHeader (total2);
   htp.tableRowClose;

   htp.tableClose;

   if NOT show_all then
      htp.anchor2 ('rpt_papers_by_platform.html', 'View All Papers by Platform/File Type',
			ctarget=>'_blank');
   end if;

end display_data;

-- ****************************************************************************************

procedure html is

cursor	driver is
select	abstract_id, assignable, editor_id
from	abstracts;

file_platform	varchar2(1000);
file_ext	varchar2(1000);

test	extensions_type;

begin

   load_data;
   display_data;

end html;

-- ****************************************************************************************

end RPT_papers_by_platform;
/
show error

--	Copyright (C) 2004 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>

