set define off

create or replace package body rpt_proposals as

rpt_title$	constant char(48)	:= 'Proposals for Presentation/Classification Change';

type	name_type is record (
		fname	persons.fname%TYPE,
		lname	persons.fname%TYPE);

type	affil_type is record (
		abbrev	affiliations.affil_abbrev%TYPE,
		name	affiliations.name1%TYPE);

type	person_type is record (
		person	name_type,
		affil	affil_type);

type	persons_type is table of person_type index by binary_integer;

type	proposal_type is record (
		proposed	varchar2(80),
		proposers	persons_type);

type	proposals_type is table of proposal_type index by binary_integer;

type	rec_type is record (
		abstract_id	integer,
		abs_title	abstracts.title%TYPE,
		original	varchar2(80),
		main_class	classification_codes.classification_descr%TYPE,
		owner		person_type,
		pauthor		person_type,
		proposal	proposals_type);

type	recs_type is table of rec_type index by binary_integer;
recs	recs_type;

-- *******************************************************************************
-- *******************************************************************************

procedure output_present_changes is

begin

   owa_util.mime_header('application/vnd.ms-excel', TRUE);
--   htp.p('Content-Disposition: filename="' ||
--	replace (common.conf_title$, ' ', '_') || '_Proposals.xls"');
   owa_util.http_header_close;

   reports.write ('ID');
   reports.write ('Abstract Title');
   reports.write ('Classification');
   reports.write ('Original');
   reports.write ('Owner Lname');
   reports.write ('Owner Fname');
   reports.write ('Owner Affil');
   reports.write ('Owner Affil Name');
   reports.write ('Primary Author Lname');
   reports.write ('Primary Author Fname');
   reports.write ('Primary Author Affil');
   reports.write ('Primary Author Affil Name');
   reports.write ('Proposed');
   reports.write ('Proposed Count');
   reports.write ('Proposer Lname');
   reports.write ('Proposer Fname');
   reports.write ('Proposer Affil');
   reports.write ('Proposer Affil Name', true);

   for i in 1..recs.COUNT LOOP
      reports.write (recs(i).abstract_id);
      reports.write (recs(i).abs_title);
      reports.write (recs(i).main_class);
      reports.write (recs(i).original);
      reports.write (recs(i).owner.person.lname);
      reports.write (recs(i).owner.person.fname);
      reports.write (recs(i).owner.affil.abbrev);
      reports.write (recs(i).owner.affil.name);
      reports.write (recs(i).pauthor.person.lname);
      reports.write (recs(i).pauthor.person.fname);
      reports.write (recs(i).pauthor.affil.abbrev);
      reports.write (recs(i).pauthor.affil.name);

      for j in 1..recs(i).proposal.COUNT loop
         reports.write (recs(i).proposal(j).proposed);
         reports.write (recs(i).proposal(j).proposers.COUNT);

         for k in 1..recs(i).proposal(j).proposers.COUNT loop
            reports.write (recs(i).proposal(j).proposers(k).person.lname);
            reports.write (recs(i).proposal(j).proposers(k).person.fname);
            reports.write (recs(i).proposal(j).proposers(k).affil.abbrev);
            reports.write (recs(i).proposal(j).proposers(k).affil.name);
         end loop;
      end loop;

      reports.write ('', true);
   end loop;


end output_present_changes;

-- *******************************************************************************
-- *******************************************************************************

function get_affil_info (affil_id integer) return affil_type is

cursor	driver is
select	affil_abbrev, name1
from	affiliations
where	affiliation_id = affil_id;

rec	affil_type;

begin

   open driver;
   fetch driver into rec;
   close driver;

   return rec;

end get_affil_info;

-- *******************************************************************************

function get_primary (abs_id integer) return person_type is

cursor	driver is
select	aa.lname, aa.fname, aa.profile_id, pro.address_id, addr.affiliation_id
from	addresses		addr,
	profiles		pro,
	abstract_authors	aa
where	aa.abstract_id		= abs_id
and	aa.is_primary		= 'Y'
and	pro.profile_id		= aa.profile_id
and	addr.address_id		= pro.address_id;

lrec	driver%ROWTYPE;
rec	person_type;

begin

   open driver;
   fetch driver into lrec;
   close driver;

   rec.person.fname	:= lrec.fname;
   rec.person.lname	:= lrec.lname;
   rec.affil		:= get_affil_info (lrec.affiliation_id);

   return rec;

end get_primary;

-- *******************************************************************************

function get_proposers (abs_id integer, code varchar2) return persons_type is

cursor	driver is
select	p.lname, p.fname, common.get_user_primary_affil_id (ap.person_id) affil_id
from	abstract_presentations	ap,
	persons			p
where	ap.abstract_id		= abs_id
and	ap.presentation_code	= code
and	p.person_id		= ap.person_id
order by p.lname, p.fname;

type	lrecs_type is table of driver%ROWTYPE index by binary_integer;
lrecs	lrecs_type;

recs	persons_type;

begin

   open driver;
   fetch driver BULK COLLECT into lrecs;
   close driver;

   for i in 1..lrecs.COUNT loop
      recs(i).person.lname	:= lrecs(i).lname;
      recs(i).person.fname	:= lrecs(i).fname;
      recs(i).affil		:= get_affil_info (lrecs(i).affil_id);
   end loop;

   return recs;

end get_proposers;

-- *******************************************************************************

function get_propsals (abs_id integer) return proposals_type is

cursor	driver is
select	distinct pc.sort_order, pc.presentation_descr, ap.presentation_code
from	presentation_codes	pc,
	abstract_presentations	ap
where	ap.abstract_id		= abs_id
and	pc.presentation_code	= ap.presentation_code
order by pc.sort_order nulls last, pc.presentation_descr;

type	lrec_type is table of driver%ROWTYPE;
lrecs	lrec_type;

recs	proposals_type;

begin

   open driver;
   fetch driver BULK COLLECT into lrecs;
   close driver;

   for i in 1..lrecs.COUNT loop
      recs(i).proposed	:= lrecs(i).presentation_descr;
      recs(i).proposers	:= get_proposers (abs_id, lrecs(i).presentation_code);
   end loop;

   return recs;

end get_propsals;

-- *******************************************************************************

procedure load_present_changes is

cursor	driver is
select	abs.abstract_id, abs.title, pc.presentation_descr,
	abs.person_id, p.fname, p.lname, cc.classification_descr,
	common.get_user_primary_affil_id (abs.person_id) affil_id
from	classification_codes	cc,
	presentation_codes	pc,
	persons			p,
	abstracts		abs
where	abs.abstract_id		in
	(	select	distinct abstract_id
		from	abstract_presentations)
and	pc.presentation_code	= abs.presentation_code
and	p.person_id		= abs.person_id
and	cc.classification_code	= abs.classification_code
order by abs.abstract_id;

type	lrec_type is table of driver%ROWTYPE index by binary_integer;
lrecs	lrec_type;

begin

   open driver;
   fetch driver BULK COLLECT into lrecs;
   close driver;

   for i in 1..lrecs.COUNT LOOP
      recs(i).abstract_id		:= lrecs(i).abstract_id;
      recs(i).abs_title			:= lrecs(i).title;
      recs(i).original			:= lrecs(i).presentation_descr;
      recs(i).main_class		:= lrecs(i).classification_descr;
      recs(i).owner.person.fname	:= lrecs(i).fname;
      recs(i).owner.person.lname	:= lrecs(i).lname;
      recs(i).owner.affil		:= get_affil_info (lrecs(i).affil_id);
      recs(i).pauthor			:= get_primary (lrecs(i).abstract_id);
      recs(i).proposal			:= get_propsals (lrecs(i).abstract_id);
   end loop;

end load_present_changes;

-- *******************************************************************************

procedure xls is

begin

   load_present_changes;
   output_present_changes;

end xls;

-- *******************************************************************************

procedure html is

begin

   xls;

end html;

-- *******************************************************************************

end rpt_proposals;
/
show error

-- *******************************************************************************

set define on

--	Copyright (C) 2004 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>
