set define off

create or replace PACKAGE body rpt_repat as

rpt_title$	constant char(23)	:= 'Repatriation Reports - ';

type	lrec_type is record (
		row_label		varchar2(1000),
		conference_title	conf_conferences.conference_title%TYPE,
		presentation_descr	conf_presentations.presentation_descr%TYPE,
		icount			integer);
type	lrecs_type is table of lrec_type index by binary_integer;
lrecs	lrecs_type;

-- ********

type	stat_type is record (
		icount	integer);
type	stats_type is table of stat_type index by conf_presentations.presentation_descr%TYPE;

type	conf_type is record (
		stats	stats_type,
		total	integer := 0);
type	confs_type is table of conf_type index by conf_conferences.conference_title%TYPE;

type	rec_type is record (
		confs	confs_type);
type	recs_type is table of rec_type index by varchar2(1000);

recs	recs_type;

rHash		varchar2(1000);
cHash		conf_conferences.conference_title%TYPE;
pHash		conf_presentations.presentation_descr%TYPE;

primary_author$	constant conf_functions.function_code%TYPE	:= 'a~PRI';

-- ***********************************************************************************
-- ***********************************************************************************

procedure select_series (form_action varchar2) is

cursor	driver is
select	distinct conference_code
from	conf_conferences
order by 1;

begin

   htp.header (3, 'Select a Conference Series');
   htp.formOpen (form_action, 'POST');

   for rec in driver LOOP
      htp.formCheckBox ('conf_code', rec.conference_code);
      htp.p (common.space2$);
      htp.p (rec.conference_code);
      htp.br;
   end loop;

   htp.formSubmit;
   htp.formClose;

end select_series;

-- ***********************************************************************************

procedure select_conferences (conf_codes common.ident_arr, form_action varchar2) is

cursor	driver (code$ varchar2) is
select	*
from	conf_conferences
where	conference_code = code$
and	repatriated	is not null
order by start_date;

begin

   htp.header (3, 'Select Conferences');
   htp.formOpen (form_action, 'POST');

   htp.tableOpen ('BORDER');

   htp.tableRowOpen ('LEFT', 'BOTTOM');
   htp.tableHeader (common.space$);
   htp.tableHeader ('Conference');
   htp.tableHeader ('Location');
   htp.tableHeader ('Repatriated');
   htp.tableRowClose;

   for i in 1..conf_codes.COUNT loop
      for rec in driver (conf_codes(i)) LOOP
         htp.tableRowOpen ('LEFT', 'BOTTOM');
         htp.tableData (htf.formCheckBox ('conf_IDs', rec.conference_ID, 'CHECKED'));
         htp.tableData (rec.conference_title);
         htp.tableData (rec.location);
         htp.tableData (rec.repatriated);
         htp.tableRowClose;
      end loop;
   end loop;

   htp.tableClose;
   htp.formSubmit;
   htp.formClose;

end select_conferences;

-- ***********************************************************************************
-- ***********************************************************************************

function get_conf_title (ID integer) return varchar2 is

cursor	driver is
select	conference_title
from	conf_conferences
where	conference_id = ID;

rec	driver%ROWTYPE;

begin

   open driver;
   fetch driver into rec;
   close driver;

   return rec.conference_title;

end get_conf_title;

-- ***********************************************************************************

procedure display_conf_titles is

cursor	driver is
select	conference_title
from	conf_conferences
where	conference_id in (select ID from rpt_IDs)
order by 1;

begin

   for rec in driver loop
      htp.tableHeader (rec.conference_title);

      rHash := recs.FIRST;
      LOOP -- create any missing data cells
         exit when rHash is null;
         if not recs(rHash).confs.EXISTS(rec.conference_title) then
            recs(rHash).confs(rec.conference_title) := null;
         end if;
         rHash := recs.NEXT(rHash);
      end loop;
   end loop;

end display_conf_titles;

-- ***********************************************************************************

procedure display_totals (rec conf_type) is

begin

   htp.tableOpen (cattributes=>'WIDTH="100%" CELLPADDING="5"');

   pHash := rec.stats.FIRST;
   LOOP
      exit when pHash is null;

      htp.tableRowOpen ('RIGHT', 'TOP');
      htp.tableData (pHash, 'LEFT');
      htp.tableData (rec.stats(pHash).icount, cattributes=>'WIDTH="20%"');
      htp.tableData (round (rec.stats(pHash).icount / rec.total * 100, 2) || '%', cattributes=>'WIDTH="30%"');
      htp.tableRowClose;

      if pHash <> rec.stats.LAST then
         common.show_data_sep (3, htf.hr);
      end if;

      pHash := rec.stats.NEXT(pHash);
   end loop;

   htp.tableClose;

end display_totals;

-- ***********************************************************************************

procedure display_report (label varchar2) is

background$	constant char(32)	:= 'STYLE="background-color:#EEEEEE"';
style		varchar2(32)		:= null;

begin

   htp.tableOpen ('BORDER');
   htp.tableRowOpen ('LEFT', 'BOTTOM');
   htp.tableHeader (label);
   display_conf_titles;
   htp.tableRowClose;

   rHash := recs.FIRST;
   LOOP
      exit when rHash is null;
      htp.tableRowOpen ('RIGHT', 'TOP', cattributes=>style);
      htp.tableData (htf.bold(rHash), 'LEFT');

      cHash := recs(rHash).confs.FIRST;
      LOOP
         exit when cHash is null;

         htp.p ('<TD>');
         display_totals (recs(rHash).confs(cHash));
         htp.p ('</TD>');

         chash := recs(rHash).confs.NEXT(cHash);
      end loop;

      htp.tableRowClose;
      rHash := recs.NEXT(rHash);

      if style is null then
         style := background$;
      else
         style := null;
      end if;
   end loop;

   htp.tableClose;
   delete from rpt_IDs;

end display_report;

-- ***********************************************************************************

procedure load_conf_IDs (conf_IDs common.ident_arr) is

begin

   for i in 1..conf_IDs.COUNT loop
      insert into rpt_IDs values (conf_IDs(i));
   end loop;

end load_conf_IDs;

-- ***********************************************************************************

procedure move_ldata_to_recs is

begin

   for i in 1..lrecs.COUNT LOOP
      rHash := lrecs(i).row_label;
      cHash := lrecs(i).conference_title;
      pHash := lrecs(i).presentation_descr;

      recs(rHash).confs(cHash).stats(pHash).icount := lrecs(i).icount;
      recs(rHash).confs(cHash).total := recs(rHash).confs(cHash).total + lrecs(i).icount;
   end loop;

end move_ldata_to_recs;

-- ***********************************************************************************
-- ***********************************************************************************

procedure by_region3 (conf_IDs common.ident_arr default mt_arr) is

cursor	driver is
select	/*+ INDEX(pro) INDEX(addr) INDEX(pre) */
	rc.region_descr, c.conference_title,
	pre.presentation_descr, count(1) icount
from	region_codes		rc,
	country_codes		cc,
	addresses		addr,
	profiles		pro,
	conf_presentations	pre,
	conf_abstracts		abs,
	conf_roles		r,
	conf_functions		f,
	conf_conferences	c
where	c.conference_id		in (select ID from rpt_IDs)
and	f.conference_id		= c.conference_id
and	f.function_code		= primary_author$
and	r.function_id		= f.function_id
and	abs.abstract_id		= r.abstract_id
and	pre.presentation_id	= abs.presentation_id
and	pro.profile_id		= r.profile_id
and	addr.address_id		= pro.address_id
and	cc.country_code		= addr.country_code
and	rc.region_code		= cc.region_code
group by c.conference_title, rc.region_descr, pre.presentation_descr;

begin

   if common.header (rpt_title$) then
      load_conf_IDs (conf_IDs);

      open driver;
      fetch driver BULK COLLECT into lrecs;
      close driver;
      move_ldata_to_recs;

      display_report ('Region');
      common.footer;
   end if;

end by_region3;

-- ***********************************************************************************

procedure by_region2 (conf_code common.ident_arr default mt_arr) is

begin

   if common.header (rpt_title$) then
      select_conferences (conf_code, 'rpt_repat.by_region3');
      common.footer;
   end if;

end by_region2;

-- ***********************************************************************************

procedure by_region is

begin

   if common.header (rpt_title$) then
      select_series ('rpt_repat.by_region2');
      common.footer;
   end if;

end by_region;

-- ***********************************************************************************
-- ***********************************************************************************

procedure by_country3 (conf_IDs common.ident_arr default mt_arr) is

cursor	driver is
select	/*+ INDEX(pro) INDEX(addr) INDEX(pre) */
	cc.country_descr, c.conference_title,
	pre.presentation_descr, count(1) icount
from	country_codes		cc,
	addresses		addr,
	profiles		pro,
	conf_presentations	pre,
	conf_abstracts		abs,
	conf_roles		r,
	conf_functions		f,
	conf_conferences	c
where	c.conference_id		in (select ID from rpt_IDs)
and	f.conference_id		= c.conference_id
and	f.function_code		= primary_author$
and	r.function_id		= f.function_id
and	abs.abstract_id		= r.abstract_id
and	pre.presentation_id	= abs.presentation_id
and	pro.profile_id		= r.profile_id
and	addr.address_id		= pro.address_id
and	cc.country_code		= addr.country_code
group by c.conference_title, cc.country_descr, pre.presentation_descr;

begin

   if common.header (rpt_title$) then
      load_conf_IDs (conf_IDs);

      open driver;
      fetch driver BULK COLLECT into lrecs;
      close driver;
      move_ldata_to_recs;

      display_report ('Country');
      common.footer;
   end if;

end by_country3;

-- ***********************************************************************************

procedure by_country2 (conf_code common.ident_arr default mt_arr) is

begin

   if common.header (rpt_title$) then
      select_conferences (conf_code, 'rpt_repat.by_country3');
      common.footer;
   end if;

end by_country2;

-- ***********************************************************************************

procedure by_country is

begin

   if common.header (rpt_title$) then
      select_series ('rpt_repat.by_country2');
      common.footer;
   end if;

end by_country;

-- ***********************************************************************************
-- ***********************************************************************************

procedure by_affil3 (conf_IDs common.ident_arr default mt_arr) is

cursor	driver is
select	/*+ INDEX(pro) INDEX(addr) INDEX(pre) */
	affil.name1 ||
	decode (affil.affil_abbrev, null,' ', ' (' || affil.affil_abbrev || ') ' ),
	c.conference_title, pre.presentation_descr, count(1) icount
from	affiliations		affil,
	addresses		addr,
	profiles		pro,
	conf_presentations	pre,
	conf_abstracts		abs,
	conf_roles		r,
	conf_functions		f,
	conf_conferences	c
where	c.conference_id		in (select ID from rpt_IDs)
and	f.conference_id		= c.conference_id
and	f.function_code		= primary_author$
and	r.function_id		= f.function_id
and	abs.abstract_id		= r.abstract_id
and	pre.presentation_id	= abs.presentation_id
and	pro.profile_id		= r.profile_id
and	addr.address_id		= pro.address_id
and	affil.affiliation_id	= addr.affiliation_id
group by c.conference_title, affil.name1, affil.affil_abbrev, pre.presentation_descr;

begin

   if common.header (rpt_title$) then
      load_conf_IDs (conf_IDs);

      open driver;
      fetch driver BULK COLLECT into lrecs;
      close driver;
      move_ldata_to_recs;

      display_report ('Affiliation');
      common.footer;
   end if;

end by_affil3;

-- ***********************************************************************************

procedure by_affil2 (conf_code common.ident_arr default mt_arr) is

begin

   if common.header (rpt_title$) then
      select_conferences (conf_code, 'rpt_repat.by_affil3');
      common.footer;
   end if;

end by_affil2;

-- ***********************************************************************************

procedure by_affil is

begin

   if common.header (rpt_title$) then
      select_series ('rpt_repat.by_affil2');
      common.footer;
   end if;

end by_affil;

-- ***********************************************************************************

end rpt_repat;
/
show error

set define on

--	Copyright (C) 2004 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>