set define off

create or replace package body rpt_repo_counts as

total_affils	integer;
total_profiles	integer;

--Profiles in the Central Repository - how many are there?
--Profiles with accounts - how many are there?
--Profiles with affiliation type "industry" - how many are there?

-- ********************************************************************

procedure show_hdr (txt varchar2) is

begin

   htp.tableRowOpen ('LEFT', 'TOP');
   htp.tableHeader (txt, ccolspan=>'3');
   htp.tableRowClose;

end show_hdr;

-- ********************************************************************

procedure show_row (txt varchar2, numb1 integer, numb2 integer default 0) is

begin

   htp.tableRowOpen ('CENTER', 'TOP');
   htp.tableData (txt, 'LEFT');
   htp.tableData (to_char (numb1, '999,999,990'));

   if numb2 <> 0 then
      htp.tableData (to_char (numb1*100/numb2, '990.99') || '%');
   else
      htp.tableData (common.space$);
   end if;

   htp.tableRowClose;

end show_row;

-- ********************************************************************

procedure affil_counts is

cursor	driver is
select	count(1)			total,
	count (distinct affil_abbrev)	abbrevs,
	count (distinct NAME1)		names
from	affiliations;

driver_rec	driver%ROWTYPE;

begin

   open driver;
   fetch driver into driver_rec;
   close driver;

   total_affils := driver_rec.total;

   show_row ('Total Affiliations',	driver_rec.total);
   show_row ('Distinct Abbreviations',	driver_rec.abbrevs, total_affils);
   show_row ('Distinct Names',		driver_rec.names, total_affils);

end affil_counts;

-- ********************************************************************

procedure affil_type_counts is

cursor	driver is
select	affil_type_code, count(1) total
from	affiliations
group by affil_type_code
order by 2;

descr	affil_type_codes.affil_type_descr%TYPE;

begin

   for driver_rec in driver LOOP
      descr := common.decode_value ('affil_type_descr', 'affil_type_codes',
					'affil_type_code', driver_rec.affil_type_code);
      show_row (nvl (descr, '** No Type **'), driver_rec.total, total_affils);
   end loop;

end affil_type_counts;

-- ********************************************************************

procedure person_counts is

cursor	driver is
select	count(1)		total,
	count(distinct user_id)	accounts
from	persons;

driver_rec	driver%ROWTYPE;

begin

   open driver;
   fetch driver into driver_rec;
   close driver;

   show_row ('Total Person',		driver_rec.total);
   show_row ('Persons w/Accounts',	driver_rec.accounts, driver_rec.total);

end person_counts;

-- ********************************************************************

procedure profile_counts is

cursor	driver is
select	count(profile_id)	total,
	count(user_id)		profiles
from	persons			p,
	profiles		pro
where	p.person_id		= pro.person_id;

driver_rec	driver%ROWTYPE;

begin

   open driver;
   fetch driver into driver_rec;
   close driver;

   total_profiles := driver_rec.total;

   show_row ('Total Profiles',		total_profiles);
   show_row ('Profiles w/Accounts',	driver_rec.profiles, total_profiles);

end profile_counts;

-- ********************************************************************

procedure profile_type_counts is

cursor	driver is
select	affil.affil_type_code, count(profile_id) total
from	affiliations		affil,
	addresses		addr,
	profiles		pro
where	addr.address_id		= pro.address_id
and	affil.affiliation_id	= addr.affiliation_id
group by affil.affil_type_code
order by 2;

descr	affil_type_codes.affil_type_descr%TYPE;

begin

   for driver_rec in driver LOOP
      descr := common.decode_value ('affil_type_descr', 'affil_type_codes',
					'affil_type_code', driver_rec.affil_type_code);
      show_row (nvl (descr, '** No Type **'), driver_rec.total, total_profiles);
   end loop;

end profile_type_counts;

-- ********************************************************************

procedure html is

begin

   if common.header ('Repository Counts') then
      htp.tableOpen ('BORDER');

      show_hdr ('Affiliations');
      affil_counts;

      show_hdr ('Affiliation Types');
      affil_type_counts;

-- **********

      show_hdr ('Persons');
      person_counts;

-- **********

      show_hdr ('Profiles');
      profile_counts;

      show_hdr ('Profiles by Affliation Type');
      profile_type_counts;

      htp.tableClose;
      common.footer;
   end if;

end html;

-- ********************************************************************

end rpt_repo_counts;
/
show error

set define on

--	Copyright (C) 2004 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>
