set define off

create or replace package body rpt_session_export as

sepstr	varchar2(2);
buffer	varchar2(400);

-- ***********************************************************************************

procedure column_headers is

begin

   htp.prn (common.crlf$);
   reports.write ('First');
   reports.write ('Last');
   reports.write ('Email');
   reports.write ('Affiliation');
   reports.write ('Name1');
   reports.write ('Name2');
   reports.write ('Department');
   reports.write ('Address1');
   reports.write ('Address2');
   reports.write ('Address3');
   reports.write ('PO Box');
   reports.write ('Mail Stop');
   reports.write ('Town');
   reports.write ('Postal Code');
   reports.write ('Zip');
   reports.write ('Country');
   reports.write ('Date');
   reports.write ('Start');
   reports.write ('End');
   reports.write ('Type');
   reports.write ('Speakers');
   reports.write ('Field');
   reports.write ('Coordinator');
   reports.write ('Coordinator e-mail', true);

end column_headers;

-- ***********************************************************************************

procedure session_chair_info (prof_id integer) is

cursor	driver is
select	p.lname, p.fname,
	prof.email_address, prof.mail_stop,
	addr.*,
	affil.affil_abbrev, affil.name1, affil.name2, affil.department
from	affiliations		affil,
	addresses		addr,
	persons			p,
	profiles		prof
where	prof.profile_id		= prof_id
and	p.person_id		= prof.person_id
and	addr.address_id		= prof.address_id
and	affil.affiliation_id	= addr.affiliation_id;

rec	driver%ROWTYPE;

begin

   open driver;
   fetch driver into rec;
   close driver;

   reports.write (rec.fname);
   reports.write (rec.lname);
   reports.write (rec.email_address);
   reports.write (rec.affil_abbrev);
   reports.write (rec.name1);
   reports.write (rec.name2);
   reports.write (rec.department);
   reports.write (rec.address1);
   reports.write (rec.address2);
   reports.write (rec.address3);
   reports.write (rec.po_box);
   reports.write (rec.mail_stop);
   reports.write (rec.town);
   reports.write (rec.postal_code);
   reports.write (rec.zip_code);
   reports.write (rec.country_code);

end session_chair_info;

-- ***********************************************************************************

procedure session_speakers (sid varchar2) is

cursor	driver is
select	common.get_user_name_affiliation (auth.profile_id, 'Y')	speaker_info
from	authors			auth,
	abstract_sessions	abs
where	abs.session_id		= sid
and	auth.abstract_id	= abs.abstract_id
and	auth.author_code	= common.speaker$;

begin

   sepstr := null;
   buffer := null;

   for rec in driver LOOP
      buffer := buffer || sepstr || rec.speaker_info;
      sepstr := '; ';
   end loop;

   reports.write (buffer);

end session_speakers;

-- ***********************************************************************************

procedure session_classifications (sid varchar2) is

cursor	driver is
select	classification_descr
from	classification_codes
where	classification_code	in
	(	select	distinct classification_code
		from	abstract_sessions	sa,
			abstracts		a
		where	sa.session_id		= sid
		and	a.abstract_id		= sa.abstract_id
	)
order by 1;

begin

   sepstr := null;
   buffer := null;

   for rec in driver LOOP
      buffer := buffer || sepstr || rec.classification_descr;
      sepstr := '; ';
   end loop;

   reports.write (buffer);

end session_classifications;

-- ***********************************************************************************

procedure session_coordinators (sid varchar2) is

cursor	driver is
select	common.get_user_name (person_id)	coordinator_name
from	experts
where	classification_code	in
	(	select	distinct a.classification_code
		from	abstracts		a,
			abstract_sessions	sa
		where	sa.session_id		= sid
		and	a.abstract_id		= sa.abstract_id
	)
and	coordinator	= 'Y'
order by 1;

begin

   sepstr := null;
   buffer := null;

   for rec in driver LOOP
      buffer := buffer || sepstr || rec.coordinator_name;
      sepstr := '; ';
   end loop;

   reports.write (buffer);

end session_coordinators;

-- ***********************************************************************************

procedure session_coordinator_emails (sid varchar2) is

cursor	driver is
select	common.get_user_email (person_id)	coordinator_email
from	experts
where	classification_code	in
	(	select	distinct a.classification_code
		from	abstracts		a,
			abstract_sessions	sa
		where	sa.session_id		= sid
		and	a.abstract_id		= sa.abstract_id
	)
and	coordinator	= 'Y'
order by 1;

begin

   sepstr := null;
   buffer := null;

   for rec in driver LOOP
      buffer := buffer || sepstr || rec.coordinator_email;
      sepstr := '; ';
   end loop;

   reports.write (buffer);

end session_coordinator_emails;

-- ***********************************************************************************

procedure xls is

cursor	driver is
select	s.*, c.presentation_option_descr
from	sessions			s,
	presentation_option_codes	c
where	c.presentation_type_code	= common.oral$
and	s.presentation_option_code	= c.presentation_option_code
order by s.sort_order nulls last, s.session_date, s.beg_time, s.end_time;

begin

   common.get_gbl_pid;
   if common.has_privs_to_page then
      owa_util.mime_header('application/vnd.ms-excel', TRUE);
      column_headers;

      for rec in driver LOOP
         session_chair_info (rec.chair_profile_id);
         reports.write (rec.session_date);
         reports.write (rec.beg_time);
         reports.write (rec.end_time);
         reports.write (rec.presentation_option_descr);
         session_speakers (rec.session_id);
         session_classifications (rec.session_id);
         session_coordinators (rec.session_id);
         session_coordinator_emails (rec.session_id);
         reports.write ('', true);
      end loop;
   end if;

end xls;

-- ***********************************************************************************

end rpt_session_export;
/
show errors

set define on

--	Copyright (C) 2004 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>
