set define off

create or replace package body rpt_user_dot_reassign as

rpt_title$		constant char(35)	:= 'Paper "Dots" Reassigned by Authors';

-- ********************************************************************************************
-- ********************************************************************************************

procedure html is

cursor	driver is
select	sa.paper_id, common.local_time_stamp (l.log_date) log_date, l.comments,
	common.get_user_name (a.editor_id) editor_name
from	sessions		s,
	abstract_sessions	sa,
	abstracts		a,
	logs			l
where	l.person_id		in
	(	select	aa.person_id
		from	abstract_authors	aa
		where	aa.abstract_id		= a.abstract_id)
and	l.log_code	= 'DR'
and	l.log_date	=
	(	select	max (log_date)
		from	logs	l2
		where	l2.abstract_id	= a.abstract_id
		and	l2.log_code	= l.log_code)
and	a.abstract_id		= l.abstract_id
and	sa.abstract_id		= a.abstract_id
and	s.session_id		= sa.session_id
order by s.session_date, sa.paper_id;

type	rec_type is table of driver%ROWTYPE index by binary_integer;
recs	rec_type;

begin

   if common.header (rpt_title$) then
      open driver;
      fetch driver BULK COLLECT into recs;
      close driver;

-- **********

      htp.tableOpen ('BORDER', cattributes=>'CELLPADDING="3"');

      htp.tableRowOpen ('LEFT', 'BOTTOM');
      htp.tableHeader ('Paper ID', cnowrap=>'NOWRAP');
      htp.tableHeader ('Editor');
      htp.tableHeader ('Timestamp');
      htp.tableHeader ('Author Comments');
      htp.tableRowClose;
      htp.tableRowClose;

-- **********

      for i in 1..recs.COUNT LOOP
         htp.tableRowOpen ('LEFT', 'MIDDLE');
         htp.tableData (recs(i).paper_id);
         htp.tableData (recs(i).editor_name, cnowrap=>'NOWRAP');
         htp.tableData (to_char (recs(i).log_date, 'dd-Mon hh24:mi:ss'), cnowrap=>'NOWRAP');
         htp.tableData (nvl (recs(i).comments, common.space$));
         htp.tableRowClose;
      end loop;

      htp.tableClose;
      common.footer;
   end if;

end html;

-- ********************************************************************************************
-- ********************************************************************************************

end rpt_user_dot_reassign;
/
show error

set define on

--	Copyright (C) 2004 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>
