set define off

create or replace package body rpt_withdrawn as

rpt_title$	constant char(19)	:= 'Withdrawn Abstracts';

-- ***************************************************************************************

procedure dump_authors (abs_id integer, output_type varchar2) is

cursor	driver is
select	ac.author_descr, pro.person_id,
	common.get_user_name (pro.person_id) full_name,
	affil.name1,
	decode (addr.town, null,null, ' ' || addr.town) town,
	common.decode_value ('country_descr', 'country_codes', 'country_code', addr.country_code) country
from	affiliations		affil,
	addresses		addr,
	profiles		pro,
	author_codes		ac,
	jn_authors		ja
where	ja.abstract_id		= abs_id
and	ac.author_code		= ja.author_code
and	ac.author_code		<> common.co_author$
and	pro.profile_id		= ja.profile_id
and	addr.address_id		= pro.address_id
and	affil.affiliation_id	= addr.affiliation_id
order by ac.sort_order;

type		rec_type is table of driver%ROWTYPE index by binary_integer;
recs		rec_type;
continue	boolean;

begin

   open driver;
   fetch driver BULK COLLECT into recs;
   close driver;

   if output_type = common.out_html$ and recs.COUNT > 0 then
      htp.tableRowOpen;
      htp.tableData (common.space$, ccolspan=>'5');
      htp.p ('<TD ALIGN="LEFT">');
         htp.tableOpen;
   end if;

   for i in 1..recs.COUNT loop
      continue := true;

      if i = 2 then
         if recs(i).person_id = recs(i-1).person_id then
            continue := false;
         end if;
      end if;

      if continue then
         if output_type = common.out_html$ then
            if recs(i).country is not null then
               recs(i).country := ', ' || recs(i).country;
            end if;

            htp.tableRowOpen;
            htp.tableHeader (recs(i).author_descr, 'RIGHT');
            htp.tableData (recs(i).full_name || common.mdash$ ||
			recs(i).name1 || recs(i).town || recs(i).country, 'LEFT');
            htp.tableRowClose;
         else
            common.fmtOUT (recs(i).author_descr,	output_type);
            common.fmtOUT (recs(i).full_name,	output_type);
            common.fmtOUT (recs(i).name1,		output_type);
            common.fmtOUT (recs(i).town,		output_type);
            common.fmtOUT (recs(i).country,	output_type);
         end if;
      end if;
   end loop;

   if output_type = common.out_html$ and recs.COUNT > 0 then
         htp.tableClose;
      htp.p ('</TD>');
      htp.tableRowClose;
   end if;

end dump_authors;

-- ***************************************************************************************

procedure rpt (output_type varchar2) is

cursor	driver is
select	abstract_id, title,
	to_char (withdraw_date, common.date_fmt_str$) withdrawn,
	common.get_user_name (withdrawn_by) withdrawn_by,
	to_char (
	(	select	min(log_date)
		from	jn_logs	l
		where	l.abstract_id	= abs.abstract_id
		and	l.log_code	= common.abstract_created$
	), common.date_fmt_str$ || ' hh24:mi:ss') create_date
from	jn_abstracts	abs
order by withdraw_date;

type		rec_type is table of driver%ROWTYPE index by binary_integer;
recs		rec_type;

begin

   if common.header (rpt_title$, test_only=>true) then
      open driver;
      fetch driver BULK COLLECT into recs;
      close driver;

      if output_type = common.out_html$ then
         common.public_header (rpt_title$);
         htp.tableOpen ('BORDER');
         htp.tableRowOpen ('LEFT', 'TOP');
      else
         owa_util.mime_header('text/csv',false, common.charset$);
         htp.p('Content-Disposition: filename="' || 'Withdrawn_Abstracts.csv"');
         owa_util.http_header_close;
      end if;

      common.fmtOUT ('Withdrawn',	output_type, true);
      common.fmtOUT ('Withdrawn By',	output_type, true);
      common.fmtOUT ('Created',		output_type, true);
      common.fmtOUT ('Abstract ID',	output_type, true);
      common.fmtOUT ('Paper ID',	output_type, true);
      common.fmtOUT ('Title',		output_type, true);

      if output_type = common.out_html$ then
         htp.tableRowClose;
      else
         common.fmtOUT ('Authors',	output_type, true);
         htp.prn (common.crlf$);
      end if;

      for i in 1..recs.COUNT LOOP
         if output_type = common.out_html$ then
            htp.tableRowOpen ('LEFT', 'TOP');
         end if;

         common.fmtOUT (recs(i).withdrawn,	output_type);
         common.fmtOUT (recs(i).withdrawn_by,	output_type);
         common.fmtOUT (recs(i).create_date,	output_type);
         common.fmtOUT (recs(i).abstract_id,	output_type);
         common.fmtOUT (recs(i).title,		output_type);

         if output_type = common.out_html$ then
            htp.tableRowClose;
            dump_authors (recs(i).abstract_id, output_type);
         else
            dump_authors (recs(i).abstract_id, output_type);
            htp.prn (common.crlf$);
         end if;
      end loop;

      if output_type = common.out_html$ then
         htp.tableClose;
         common.footer;
      end if;

   end if;

end rpt;

-- ***************************************************************************************

procedure html is

begin

   if common.header (rpt_title$) then
      htp.formOpen ('rpt_withdrawn.rpt', 'POST');
      htp.header (4, 'Please select an output type');

      htp.formRadio ('output_type', common.out_html$, 'SELECTED');
      htp.p (common.out_html$);
      htp.p (common.space2$);
      htp.p (common.space2$);

      htp.formRadio ('output_type', common.out_csv$);
      htp.p (common.out_csv$);
      htp.p (common.space2$);
      htp.p (common.space2$);

      htp.formSubmit;
      htp.formClose;
      common.footer;
   end if;

end html;

-- ***************************************************************************************

end rpt_withdrawn;
/
show error

set define on

--	Copyright (C) 2004 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>