set define off

create or replace package body rpt_ioral as

rpt_title1$	constant char(13)	:= 'Invited Orals';
rpt_title2$	constant char(14)	:= 'Proposed Orals';
rpt_title	varchar2(14)		:= rpt_title1$;

priority_codes	common.ident_arr;

-- ***************************************************************************************

procedure priority_headers (output_type varchar2) is

cursor	driver is
select	priority_code, priority_descr
from	priority_codes
order by priority_code;

begin

   for rec in driver LOOP
      common.fmtOUT (rec.priority_descr, output_type, true);
      priority_codes(priority_codes.COUNT+1) := rec.priority_code;
   end loop;

end priority_headers;

-- ***************************************************************************************

procedure dump_accepted (abs_id integer, output_type varchar2) is

cursor	driver (code varchar2) is
select	common.get_user_name(person_id) || ', ' ||
	common.get_user_affil_abbrev_list(person_id)	name
from	abstract_proposals
where	abstract_id		= abs_id
and	proposal_status_code	= prog_cmtte.proposal_accepted$
order by 1;

buffer	long		:= null;
sepstr	varchar2(2);

begin

   for i in 1..priority_codes.COUNT loop
      sepstr := null;
      for rec in driver(priority_codes(i)) LOOP
         buffer := buffer || sepstr || rec.name;
         sepstr := '; ';
      end loop;

      common.fmtOUT (buffer, output_type);
      buffer := null;
   end loop;

end dump_accepted;

-- ***************************************************************************************

procedure dump_priorities (abs_id integer, output_type varchar2) is

cursor	driver (code varchar2) is
select	common.get_user_name(person_id) || ', ' ||
	common.get_user_affil_abbrev_list(person_id)	name
from	abstract_proposals
where	abstract_id	= abs_id
and	priority_code	= code
order by 1;

buffer	long		:= null;
sepstr	varchar2(2);

begin

   for i in 1..priority_codes.COUNT loop
      sepstr := null;
      for rec in driver(priority_codes(i)) LOOP
         buffer := buffer || sepstr || rec.name;
         sepstr := '; ';
      end loop;

      common.fmtOUT (buffer, output_type);
      buffer := null;
   end loop;

end dump_priorities;

-- ***************************************************************************************

procedure dump_alternates (abs_id integer, output_type varchar2, max_coauthors integer) is

cursor	driver is
select	common.get_user_name_affiliation(profile_id)	alternate
from	authors
where	abstract_id	= abs_id
and	author_code	= common.co_author$
order by sort_order nulls last;

type	rec_type is table of driver%ROWTYPE index by binary_integer;
recs	rec_type;

begin

   open driver;
   fetch driver BULK COLLECT into recs;
   close driver;

   for i in 1..recs.COUNT LOOP
      common.fmtOUT (recs(i).alternate, output_type);
   end loop;

   if output_type = common.out_HTML$ then
      if recs.COUNT < max_coauthors then
         htp.tableData (common.space$, ccolSpan=>(max_coauthors-recs.COUNT));
      end if;
   end if;

end dump_alternates;

-- ***************************************************************************************

procedure rpt (output_type varchar2) is

cursor	driver is
select	a.abstract_id, sa.paper_id, a.title, a.description,
	pc.presentation_descr, cc.classification_descr,
	common.get_user_name(a.person_id)			|| ', ' ||
	common.get_user_affil_abbrev_list(a.person_id)	proposer,
	common.get_user_name(prof.person_id)			speaker,
	common.get_profile_affil (auth.profile_id, 'Y')		speaker_affil,
	common.decode_value ('sub_class_descr', 'sub_class_codes', 'sub_class_code', a.sub_class_code) sub_class,
	(	select	count(1)
		from	authors	auth2
		where	auth2.abstract_id	= a.abstract_id
		and	author_code		= common.co_author$
	) coauthor_count
from	profiles			prof,
	authors				auth,
	sessions			s,
	classification_codes		cc,
	abstracts			a,
	abstract_sessions		sa,
	presentation_codes		pc
where	pc.presentation_type_code	= common.oral$
and	a.presentation_code		= pc.presentation_code
and	cc.classification_code		= a.classification_code
and	sa.abstract_id(+)		= a.abstract_id
and	s.session_id(+)			= sa.session_id
and	auth.abstract_id(+)		= a.abstract_id
and	auth.author_code(+)		= common.speaker$
and	prof.profile_id(+)		= auth.profile_id
order by s.sort_order nulls last,
	 s.session_date, s.beg_time, s.session_id,
	 sa.session_seq;

type		rec_type is table of driver%ROWTYPE index by binary_integer;
recs		rec_type;
max_coauthors	integer := 0;
buffer		varchar2(1000);

begin

   if common.PC_mode$ = 'Y' then
      rpt_title := rpt_title2$;
   end if;

   if common.header (rpt_title, test_only=>TRUE) then

      open driver;
      fetch driver BULK COLLECT into recs;
      close driver;

      for i in 1..recs.COUNT LOOP
         max_coauthors := greatest (max_coauthors, recs(i).coauthor_count);
      end loop;

      if output_type = common.out_html$ then
         common.public_header (rpt_title);
         htp.tableOpen ('BORDER');
         htp.tableRowOpen ('LEFT', 'TOP');
      else
         owa_util.mime_header('text/csv',false, common.charset$);
         htp.p('Content-Disposition: filename="' || 'Invited_Orals.csv"');
         owa_util.http_header_close;
      end if;

      if common.PC_mode$ = 'Y' then
         common.fmtOUT ('ID', output_type, true);
         common.fmtOUT ('Title', output_type, true);
         common.fmtOUT ('Proposer', output_type, true);
         common.fmtOUT ('Preferred Speaker', output_type, true);
         common.fmtOUT ('Main Classification', output_type, true);
         common.fmtOUT ('Sub Classification', output_type, true);
         common.fmtOUT ('Abstract', output_type, true);
         common.fmtOUT ('Accepted', output_type, true);
         priority_headers (output_type);
         common.fmtOUT ('Alternate Speakers...', output_type, true, max_coauthors);
      else
         common.fmtOUT ('Paper ID', output_type, true);
         common.fmtOUT ('Type', output_type, true);
         common.fmtOUT ('Title', output_type, true);
         common.fmtOUT ('Speaker', output_type, true);
         common.fmtOUT ('Affiliation', output_type, true);
      end if;

      if output_type = common.out_html$ then
         htp.tableRowClose;
      else
         htp.prn (common.crlf$);
      end if;

      for i in 1..recs.COUNT LOOP
         if output_type = common.out_html$ then
            htp.tableRowOpen ('LEFT', 'TOP');
         end if;

         if common.PC_mode$ = 'Y' then
            buffer := recs(i).speaker || ', ' || recs(i).speaker_affil;
            if recs(i).speaker is null then
               buffer := null;
            end if;

            common.fmtOUT (recs(i).abstract_id, output_type);
            common.fmtOUT (recs(i).title, output_type);
            common.fmtOUT (recs(i).proposer, output_type);
            common.fmtOUT (buffer, output_type);
            common.fmtOUT (recs(i).classification_descr, output_type);
            common.fmtOUT (recs(i).sub_class, output_type);
            common.fmtOUT (recs(i).description, output_type);
            dump_accepted  (recs(i).abstract_id, output_type);
            dump_priorities (recs(i).abstract_id, output_type);
            dump_alternates (recs(i).abstract_id, output_type, max_coauthors);
         else
            common.fmtOUT (recs(i).paper_id, output_type);
            common.fmtOUT (recs(i).presentation_descr, output_type);
            common.fmtOUT (recs(i).title, output_type);
            common.fmtOUT (recs(i).speaker, output_type);
            common.fmtOUT (recs(i).speaker_affil, output_type);
         end if;

         if output_type = common.out_html$ then
            htp.tableRowClose;
         else
            htp.prn (common.crlf$);
         end if;
      end loop;

      if output_type = common.out_html$ then
         htp.tableClose;
         common.footer;
      end if;

   end if;

end rpt;

-- ***************************************************************************************

procedure html is

begin

   if common.PC_mode$ = 'Y' then
      rpt_title := rpt_title2$;
   end if;

   if common.header (rpt_title) then
      htp.formOpen ('rpt_iOral.rpt', 'POST');
      htp.header (4, 'Please select an output type');

      htp.formRadio ('output_type', common.out_html$, 'SELECTED');
      htp.p (common.out_html$);
      htp.p (common.space2$);
      htp.p (common.space2$);

      htp.formRadio ('output_type', common.out_csv$);
      htp.p (common.out_csv$);
      htp.p (common.space2$);
      htp.p (common.space2$);

      htp.formSubmit;
      htp.formClose;
      common.footer;
   end if;

end html;

-- ***************************************************************************************

end rpt_ioral;
/
show error

set define on

--	Copyright (C) 2004 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>