set define off

create or replace package body reassign_red as

rpt_title$	constant varchar2(30)	:= 'Reassign "Red" Dot Papers';

-- ********************************************************************************
-- ********************************************************************************

procedure assign (abs_ids common.ident_arr, edt_id integer) is

msg		long := null;
continue	boolean := true;

begin

   if common.header (rpt_title$, test_only=>true) then
      if edt_id is null then
         msg		:= htf.header (3, 'You must select an edior.', cattributes=>common.red$);
         continue	:= false;
      end if;

      if abs_ids.count = 1 then
         msg		:= msg || htf.header (3, 'You select at least one paper.', cattributes=>common.red$);
         continue	:= false;
      end if;

      if continue then
         common.gbl_priv_pid := edt_id;  -- set for the assign_paper routine only; not set as cookie

         for i in 2..abs_ids.COUNT LOOP
            editor.assign_paper (abs_ids(i), common.paper_assigned$);	
         end loop;

         common.redirect_URL ('reassign_red.html');
      else
         common.public_header (rpt_title$);
         htp.p (msg);
         common.footer;
      end if;
   end if;

end assign;

-- ********************************************************************************
-- ********************************************************************************

procedure list_editors is

cursor	driver is
select	distinct p.sort_lname, p.sort_fname, p.person_id,
	common.get_user_name (p.person_id) name
from	persons			p,
	user_functions		uf,
	function_privileges	fp,
	application_privs	ap,
	applications		apps
where	common.editor_page$	like apps.application_name
and	ap.application_id	= apps.application_id
and	fp.privilege_code	= ap.privilege_code
and	uf.function_code	= fp.function_code
and	p.person_id		= uf.person_id
order by
	p.sort_lname, p.sort_fname;

type	rec_type is table of driver%ROWTYPE index by binary_integer;
recs	rec_type;

begin

   open driver;
   fetch driver BULK COLLECT into recs;
   close driver;

   htp.formSelectOpen ('edt_id');
   htp.formSelectOption ('*** Select an Editor ***', 'SELECTED', cattributes=>'VALUE=""');

   for i in 1..recs.COUNT LOOP
      htp.formSelectOption (recs(i).name, cattributes=>'VALUE="' || recs(i).person_id || '"');
   end loop;

   htp.formSelectCLose;

end list_editors;

-- ********************************************************************************

procedure html is

cursor	driver is
select	editor_id, abstract_id, a.status_code,
	common.get_user_name (editor_id) editor,
	common.get_paper_id (abstract_id) paper_id
from	persons		p,
	abstracts	a,
	status_codes	sc
where	sc.ok_to_upload	= 'Y'
and	a.status_code	= sc.status_code
and	p.person_id	= a.editor_id
order by
	p.sort_lname, p.sort_fname;

type	rec_type is table of driver%ROWTYPE index by binary_integer;
recs	rec_type;

begin

   if common.header (rpt_title$) then
      open driver;
      fetch driver BULK COLLECT into recs;
      close driver;

      if recs.COUNT = 0 then
         htp.header (3, 'There are no "Red" dot papers.');
      else
         htp.formOpen ('reassign_red.assign', 'POST');
         htp.formHidden ('abs_ids', 'dummy');

         htp.tableOpen;
         htp.tableRowOpen ('LEFT', 'BOTTOM');
         htp.tableHeader ('Reassign');
         htp.tableHeader (common.space$);
         htp.tableHeader ('Paper ID');
         htp.tableHeader ('Editor');
         htp.tableHeader ('New Files');
         htp.tableRowClose;

         for i in 1..recs.COUNT LOOP
            htp.tableRowOpen ('LEFT', 'TOP');
            htp.tableData (htf.formCheckbox ('abs_ids', recs(i).abstract_id), 'CENTER');
            htp.tableData (common.get_dot_image (recs(i).status_code, common.editor_dot$));
            htp.tableData (recs(i).paper_id);
            htp.tableData (recs(i).editor);

            if editor.new_files (recs(i).abstract_id) = 'YES' then
               htp.tableData ('YES');
            else
               htp.tableData (common.space$);
            end if;

            htp.tableRowClose;
         end loop;

         htp.tableClose;

         htp.p ('Reassign all papers checked above to');
         list_editors;

         htp.para;
         htp.formSubmit;
         htp.formClose;
      end if;

      common.footer;
   end if;

end html;

-- ********************************************************************************

end reassign_red;
/
show error

set define on

--	Copyright (C) 2004 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>

