set define off

create or replace package body referee as

rpt_title$	constant varchar2(20) := 'Referee Module';
referee$	constant log_codes.log_code%TYPE := 'REF';

-- ****************************************************************************************
-- ****************************************************************************************

function papers_assigned (pid integer) return integer is

cursor	driver is
select	count(1)
from	referee_abstracts
where	person_id = pid;

icount	integer;

begin

   open driver;
   fetch driver into icount;
   close driver;

   return nvl (icount, 0);

end papers_assigned;

-- ****************************************************************************************
-- ****************************************************************************************

function assigned (abs_id integer) return varchar2 is

cursor	driver is
select	1
from	referee_abstracts
where	abstract_id = abs_id;

rec_found	boolean;
dummy		integer;

begin

   open driver;
   fetch driver into dummy;
   rec_found := driver%FOUND;
   close driver;

   if rec_found then
      return 'Y';
   end if;

   return 'N';

end assigned;

-- ****************************************************************************************
-- ****************************************************************************************

procedure dots_rpt (ts date) is

cursor	driver is
select	common.get_paper_id (a.abstract_id) paper_id,
	l.log_date, l.log_code, l.referee_code, a.abstract_id
from	abstracts	a,
	logs		l
where	log_date	>= ts
and	l.referee_code in
(	select	referee_code
	from	referee_codes)
and	a.abstract_id	= l.abstract_id
and	log_date	=
(	select	max (log_date)
	from	logs	l2
	where	l2.abstract_id	= l.abstract_id
	and	l2.log_code	= l.log_code)
order by
	log_date desc;

privs	boolean;

begin

   htp.p (profile.js$);
   htp.anchor ('referee.all_dots', 'All Dots');
   htp.p (common.space2$);
   htp.anchor ('referee.dots', 'Today''s Dots');
   htp.header (3, 'Dots Assigned Since ' ||
			to_char (ts, common.date_fmt_str$));
   privs := common.has_privs_to_page (common.editor_page$, false) or common.has_privs_to_page (common.sort_file_page$, false);
   htp.tableOpen;

   for driver_rec in driver LOOP
      htp.tableRowOpen ('LEFT', 'TOP');
      htp.tableHeader (driver_rec.paper_id);
      htp.tableData (common.get_dot_image (driver_rec.referee_code, common.referee_dot$));

      if privs then
         htp.tableData (htf.anchor ('javascript:newWindow (''abstract_maint.edit?id=' ||
		driver_rec.abstract_id || ''')', 'Sort/File/Edit') );
      end if;

      htp.tableData (to_char (driver_rec.log_date, common.date_fmt_str$));
      htp.tableRowClose;
   end loop;

   htp.tableClose;

end dots_rpt;

-- ****************************************************************************************

procedure dots is

begin

   if common.header (rpt_title$) then
      dots_rpt (trunc (sysdate));
      common.footer;
   end if;

end dots;

-- ****************************************************************************************

procedure dot_totals is

cursor	driver is
select	nvl (sort_order, 99999), rc.referee_descr, count(1) icount
from	referee_codes	rc,
	abstracts	a
where	a.referee_code	= rc.referee_code
group by
	sort_order, rc.referee_descr
UNION
select	0, 'No Status', count(1)
from	abstracts
where	referee_code	is null;

begin

   htp.tableOpen;
   for driver_rec in driver LOOP
      common.show_data_row (to_char(driver_rec.icount), driver_rec.referee_descr);
   end loop;
   htp.tableClose;

end dot_totals;

-- ****************************************************************************************

procedure all_dots is

cursor	driver is
select	min (log_date)
from	logs
where	referee_code in
(	select	referee_code
	from	referee_codes);

log_date	date;

begin

   if common.header (rpt_title$) then
      open driver;
      fetch driver into log_date;
      close driver;

      dots_rpt (log_date);
      htp.para;
      dot_totals;
      common.footer;
   end if;

end all_dots;

-- ****************************************************************************************
-- ****************************************************************************************

procedure list_missing is

cursor	driver is
select	a.abstract_id, a.title,
	common.get_paper_id (a.abstract_id) paper_id,
	common.get_author_profile_id (a.abstract_id, common.speaker$) spk_profile_id
from	sessions		s,
	abstract_sessions	sa,
	abstracts		a
where	not exists
(	select	1
	from	referee_abstracts	r
	where	r.abstract_id		= a.abstract_id)
and	sa.abstract_id		= a.abstract_id
and	s.session_id		= sa.session_id
order by
	s.sort_order nulls last,
	s.session_date, s.beg_time;

icount	integer := 0;

begin

   if common.header (rpt_title$) then
      for driver_rec in driver LOOP
         htp.bold (driver_rec.paper_id);
         htp.p ('"' || driver_rec.title || '"');
         htp.p ('-');
         htp.p (common.get_user_name_affiliation (driver_rec.spk_profile_id));
         htp.para;

         icount := icount + 1;
      end loop;

      if icount = 0 then
         htp.header (3, 'There are no papers unassigned to a referee.');
      end if;

      common.footer;
   end if;

end list_missing;

-- ****************************************************************************************
-- ****************************************************************************************

procedure show_abstract_log (aid integer) is

begin

   if common.header (rpt_title$) then
      common.show_abstract_log (aid, true);
      common.footer;
   end if;

end show_abstract_log;

-- ****************************************************************************************
-- ****************************************************************************************

procedure show_referees (aid integer, text_only boolean default false) is

cursor	driver is
select	distinct
	p.sort_lname, p.sort_fname, p.sort_mname, p.person_id,
	common.get_user_name (p.person_id)	name,
	common.get_user_affil_list (p.person_id, ', ')	affil
from	persons			p,
	referee_abstracts	x
where	x.abstract_id		= aid
and	p.person_id		= x.person_id
order by
	p.sort_lname, p.sort_fname, p.sort_mname;

begin

   for driver_rec in driver LOOP
      if not text_only then
         htp.p (common.space2$);
         htp.p (common.space2$);
      end if;

      if text_only then
         htp.p (driver_rec.name);
      else
         htp.bold (driver_rec.name);
         htp.p (driver_rec.affil);
         htp.br;
      end if;
   end loop;

end show_referees;

-- ****************************************************************************************

procedure list_by_paper is

cursor	driver is
select	a.abstract_id, a.title, a.referee_code,
	common.get_paper_id (a.abstract_id) paper_id,
	common.get_author_profile_id (a.abstract_id, common.speaker$) spk_profile_id
from	abstracts		a,
	abstract_sessions	sa,
	sessions		s
where	sa.session_id		= s.session_id
and	a.abstract_id		= sa.abstract_id
order by
	s.sort_order nulls last,
	s.session_date, s.beg_time;

begin

   if common.header (rpt_title$) then
      htp.p (profile.js$);

      for driver_rec in driver LOOP
         if driver_rec.referee_code is not null then
            htp.p (common.get_dot_image (driver_rec.referee_code, common.referee_dot$));
         end if;

         htp.bold (driver_rec.paper_id);

         htp.tableData (htf.anchor ('javascript:newWindow (''abstract_maint.edit?id=' ||
		driver_rec.abstract_id || ''')', 'Sort/File/Edit') );

         htp.p ('"' || driver_rec.title || '"');
         htp.p ('-');
         htp.p (common.get_user_name_affiliation (driver_rec.spk_profile_id));
         htp.br;
         show_referees (driver_rec.abstract_id);
         htp.br;
      end loop;
      common.footer;
   end if;

end list_by_paper;

-- ****************************************************************************************
-- ****************************************************************************************

procedure show_papers (pid integer, privs boolean) is

cursor	driver is
select	sa.paper_id, a.title, sa.presentation_code, a.classification_code,
	a.referee_code, a.abstract_id,
	common.get_author_profile_id (a.abstract_id, common.speaker$) spk_profile_id
from	sessions		s,
	abstract_sessions	sa,
	abstracts		a,
	referee_abstracts	x
where	x.person_id		= pid
and	a.abstract_id		= x.abstract_id
and	sa.abstract_id		= a.abstract_id
and	s.session_id		= sa.session_id
order by
	s.sort_order nulls last,
	s.session_date, s.beg_time, sa.session_seq;

class_descr	classification_codes.classification_descr%TYPE;
present_descr	presentation_codes.presentation_descr%TYPE;

begin

   for driver_rec in driver LOOP
      htp.tableRowOpen ('LEFT', 'TOP');

      if driver_rec.referee_code is null then
         htp.tableData (common.space$);
      else
         htp.tableData (common.get_dot_image (driver_rec.referee_code, common.referee_dot$));
      end if;

      htp.tableHeader (driver_rec.paper_id);

      if privs then
         htp.tableData (htf.anchor ('javascript:newWindow (''abstract_maint.edit?id=' ||
		driver_rec.abstract_id || ''')', 'Sort/File/Edit') );
      end if;

      htp.tableData ('"' || driver_rec.title || '"' || '-' ||
			common.get_user_name_affiliation (driver_rec.spk_profile_id));
      htp.tableRowClose;
   end loop;

end show_papers;

-- **********************************************************************************

procedure list_by_person is

cursor	driver is
select	distinct
	p.sort_lname, p.sort_fname, p.sort_mname, p.person_id,
	common.get_user_name (p.person_id)	name,
	common.get_user_affil_list (p.person_id, ', ')	affil
from	persons		p,
	experts		x
where	x.referee	= 'Y'
and	p.person_id	= x.person_id
order by
	p.sort_lname, p.sort_fname, p.sort_mname;

privs	boolean;
cols	integer := 3;

begin

   if common.header (rpt_title$) then
      htp.p (profile.js$);
      privs := common.has_privs_to_page (common.editor_page$, false) or common.has_privs_to_page (common.sort_file_page$, false);
      if privs then
         cols := 5;
      end if;
      htp.tableOpen;

      for driver_rec in driver LOOP
         htp.tableRowOpen ('LEFT', 'TOP');
         htp.tableData (htf.bold (driver_rec.name) || common.space2$ || driver_rec.affil, ccolspan=>cols);
         htp.tableRowClose;

         show_papers (driver_rec.person_id, privs);
         common.show_data_sep (cols);
      end loop;

      htp.tableRowClose;
      common.footer;
   end if;

end list_by_person;

-- **********************************************************************************
-- **********************************************************************************

function js$ return varchar2 is

begin

   return '<SCRIPT TYPE="text/javascript" language="JavaScript" src="' || common.server ||
		common.vdir$ || 'profile.js"></SCRIPT>';

end js$;

-- *********************************************************************************************

procedure js is

begin

   htp.print ('');
   htp.prn (
'function newWindow (URL) {

   w = parseInt(screen.width*.75);
   h = parseInt(screen.height*.75);
   newWin = window.open (URL, "newWin' || common.time$ || '", "menubar=0,height=" + h + ",width=" + w + ",scrollbars=1,toolbar=1,status=1,resizable=1")
   newWin.focus()

} // end newWindow
');

end js;

-- ****************************************************************************************
-- ****************************************************************************************

function is_a_referee return boolean is

cursor	driver is
select	1
from	experts
where	person_id	= common.gbl_pid
and	referee		= 'Y';

dummy		integer;
rec_found	boolean;

begin

   common.get_gbl_pid;
   if common.referee_conf$ = 'N' then
      return false;
   end if;

   if common.user_is_jacow_admin then
      return true;
   end if;

   open driver;
   fetch driver into dummy;
   rec_found := driver%FOUND;
   close driver;

   return rec_found;

end is_a_referee;

-- ****************************************************************************************
-- ****************************************************************************************

procedure save (abs_id integer, code varchar2, this_comment varchar2, this_note varchar2) is

continue	boolean := true;
buffer		long := this_comment;

begin

   if common.header (rpt_title$, test_only=>true) then

      if NOT common.publishable_referee_code (code) then
         buffer := replace (buffer, chr( 9), null);
         buffer := replace (buffer, chr(10), null);
         buffer := replace (buffer, chr(13), null);
         buffer := replace (buffer, chr(32), null);

         if buffer is null then
            common.public_header (rpt_title$);
            htp.header (3, 'You must enter a comment to the author.', cattributes=>common.red$);
            htp.p ('You are setting the referee status to a non-publishable status.');
            htp.p ('Doing so requires an explanation to the author.');
            htp.p (common.click_back$);
            continue := false;
            common.footer;
         end if;
      end if;

      if continue then
         update abstracts set referee_code = code where abstract_id = abs_id;
         insert into logs (abstract_id, log_code, referee_code, comments, editor_notes)
         values (abs_id, common.referee$, code, trim (this_comment), trim (this_note) );

         common.redirect_URL ('referee.view_abs?abs_id=' || abs_id);
      end if;
   end if;

end save;

-- ****************************************************************************************
-- ****************************************************************************************

procedure edit (abs_id integer) is

cursor	driver is
select	title, referee_code,
	common.get_paper_id (abstract_id) paper_id
from	abstracts
where	abstract_id = abs_id;

driver_rec	driver%ROWTYPE;

begin

   if is_a_referee then
      common.public_header (rpt_title$);
      open driver;
      fetch driver into driver_rec;
      close driver;

      htp.header (3, driver_rec.paper_id || ' - ' || driver_rec.title);

      htp.formOpen ('referee.save', 'POST', cattributes=>'NAME="SPMS"');
      htp.formHidden ('abs_id', abs_id);

      htp.bold ('Enter the paper''s status: ');
      common.status_code_popup (driver_rec.referee_code, '%', 'Y');
      htp.para;

      htp.tableOpen;
      htp.tableRowOpen ('LEFT', 'BOTTOM');
      htp.tableHeader ('Comments to Author');
      htp.tableHeader ('Referee''s Notes (Private)');
      htp.tableRowClose;

      htp.tableRowOpen ('LEFT', 'BOTTOM');
      htp.tableHeader (htf.formTextArea ('this_comment', 5, 40) );
      htp.tableHeader (htf.formTextArea ('this_note',    5, 40) );
      htp.tableRowClose;
      htp.tableClose;
      htp.para;

      htp.formSubmit (null, 'Save');
      htp.formClose;
      common.form_focus ('SPMS', 'code');
   end if;
   common.footer;

end edit;

-- ****************************************************************************************
-- ****************************************************************************************

procedure view_abs (abs_id integer) is

begin

   if is_a_referee then
      common.public_header (rpt_title$);
      search.abstract_id (abs_id);
      htp.header (3, 'Abstract Activity Log');
      common.show_abstract_log (abs_id, true);
      common.footer;
   end if;

end view_abs;

-- ****************************************************************************************
-- ****************************************************************************************

function get_dot_status (abs_id integer) return varchar2 is

cursor	driver is
select	referee_code
from	logs
where	abstract_id	= abs_id
and	person_id	= nvl (common.gbl_priv_pid, common.gbl_pid)
and	log_code	= referee$
order by
	log_date desc;

referee_code	referee_codes.referee_code%TYPE;

begin

   open driver;
   fetch driver into referee_code;
   close driver;

   if referee_code is null then
      return null;
   end if;

   return referee_code;

end get_dot_status;

-- ****************************************************************************************

procedure html is

cursor	driver is
select	a.*, common.get_paper_id (a.abstract_id) paper_id,
	common.get_author_profile_id (a.abstract_id, common.speaker$) spk_profile_id,
	common.get_author_profile_id (a.abstract_id, common.primary_author$) primary_profile_id
from	sessions		s,
	abstract_sessions	sa,
	abstracts		a,
	referee_abstracts	r
where	r.person_id		= common.gbl_pid
and	a.abstract_id		= r.abstract_id
and	sa.abstract_id		= a.abstract_id
and	s.session_id		= sa.session_id
order by
	decode (get_dot_status (a.abstract_id), null,0, 1),
	s.sort_order nulls last,
	s.session_date, s.beg_time;

present_descr	presentation_type_codes.presentation_type_descr%TYPE;
author_pid	integer;
speaker_pid	integer;
buffer		long;

begin

   if is_a_referee then
      common.public_header (rpt_title$);
      htp.p (js$);
      htp.tableOpen ('BORDER', cattributes=>'CELLPADDING="3"');
      htp.tableRowOpen ('LEFT', 'BOTTOM');
      htp.tableHeader ('Paper ID');
      htp.tableHeader ('Referee Status');
      htp.tableHeader ('Type');
      htp.tableHeader ('Title');
      htp.tableHeader ('Author');
      htp.tableRowClose;

      for driver_rec in driver LOOP
         present_descr	:= common.decode_value ('presentation_type_descr',
				'presentation_type_codes', 'presentation_type_code',
				common.get_presentation_type (driver_rec.presentation_code) );
         author_pid	:= common.decode_value ('person_id', 'profiles', 'profile_id',
				driver_rec.primary_profile_id);
         speaker_pid	:= common.decode_value ('person_id', 'profiles', 'profile_id',
				driver_rec.spk_profile_id);

         buffer := htf.bold ('Author: ') ||
			common.get_user_name_affiliation (driver_rec.primary_profile_id);
         if author_pid <> speaker_pid then
            buffer := buffer || htf.br || htf.bold ('Speaker: ') ||
			common.get_user_name_affiliation (driver_rec.spk_profile_id);
         end if;

         htp.tableRowOpen ('LEFT', 'TOP');
         if common.pdf_uploaded (driver_rec.abstract_id) then
            htp.tableData (htf.anchor ('javascript:newWindow (''referee.view_abs?abs_id=' ||
				driver_rec.abstract_id || ''')', driver_rec.paper_id) );
         else
            htp.tableData (driver_rec.paper_id);
         end if;

         htp.tableData (nvl (common.get_dot_image (
				get_dot_status (driver_rec.abstract_id), common.editor_dot$),
			common.space$), 'CENTER');
         htp.tableData (present_descr);
         htp.tableData (driver_rec.title);
         htp.tableData (buffer);
         htp.tableRowClose;
      end loop;

      htp.tableClose;

   else
      htp.header (3, 'You are not authorized to use the Referee Module.');
   end if;

   common.footer;

end html;

-- ****************************************************************************************

end referee;
/

show error

set define on
