set define off

Create or Replace package body reg_email as

rpt_title$	constant varchar2(50) := 'Registration Email';

send_it$	constant varchar2(15)	:= 'Send the email';
view_it$	constant varchar2(15)	:= 'View an Example';
dummy$		constant varchar2(2)	:= '~~';

bal_ignore$	constant varchar2(15)	:= 'Ignore Balance';
bal_due$	constant varchar2(15)	:= 'Balance Due';
bal_zero$	constant varchar2(15)	:= 'Paid in Full';

type		list_type is varray(8) of varchar2(50);
sql_stmt	varchar2(10000);

driver_rec	persons%ROWTYPE;

-- ********************************************************************************************

procedure init_variables is

begin

   email.variables(1)	:= '<<Balance Due>>';
   email.variables(2)	:= '<<Conference Name>>';
   email.variables(3)	:= '<<Editor''s Email>>';
   email.variables(4)	:= '<<Login URL>>';
   email.variables(5)	:= '<<Name, First>>';
   email.variables(6)	:= '<<Name, Last>>';
   email.variables(7)	:= '<<Name, Middle>>';
   email.variables(8)	:= '<<Name, Initials>>';

end init_variables;

-- ********************************************************************************************
-- ********************************************************************************************

procedure build_sql_stmt (this_page integer, ptype common.ident_arr, balue varchar2) is

begin

   sql_stmt := 'select * from persons' || common.eol$ ||
		'where common.get_user_email(person_id) is not null '	||
		'and person_id in ' ||
		'(select person_id from registrants ' ||
		'where page_id = ' || this_page || ') ';

   if ptype.count > 1 then
      sql_stmt := sql_stmt || ' and person_id in '		||
			'(select r.person_id '			||
			'from	registrants r, payments p '	||
			'where	p.person_id(+) = r.person_id '	||
			'and ' || common.build_IN_list (ptype, 'nvl (p.payment_code, ''?'')') || ') ';
   end if;

   if balue = bal_due$ then
      sql_stmt := sql_stmt || ' and  regist_maint.balancedue (common.ENCRYPT(persons.person_id),'  || this_page ||' )>0';

   elsif balue = bal_zero$ then
      sql_stmt := sql_stmt || ' and  regist_maint.balancedue (common.ENCRYPT(persons.person_id),'  || this_page ||' )<=0';

   end if;

end build_sql_stmt;

-- ********************************************************************************************

function replace_variables (this_page integer, message long) return long is

buffer	long	:= message;
bal_due	number	:= regist_maint.balancedue (common.ENCRYPT(driver_rec.person_id), this_page);
bal_str	varchar2(25);

begin

   bal_str := common.currency_symbol$ || trim (to_char (bal_due, '999,990.00'));

   email.variables(1)	:= '<<Balance Due>>';
   email.variables(2)	:= '<<Conference Name>>';
   email.variables(3)	:= '<<Editor''s Email>>';
   email.variables(4)	:= '<<Login URL>>';
   email.variables(5)	:= '<<Name, First>>';
   email.variables(6)	:= '<<Name, Last>>';
   email.variables(7)	:= '<<Name, Middle>>';
   email.variables(8)	:= '<<Name, Initials>>';

   buffer := replace (buffer, email.variables(1),	bal_str);
   buffer := replace (buffer, email.variables(2),	common.conf_title$);
   buffer := replace (buffer, email.variables(3),	common.editor_email$);
   buffer := replace (buffer, email.variables(4),	common.server || common.vdir$ || 'profile.html');
   buffer := replace (buffer, email.variables(5),	driver_rec.fname);
   buffer := replace (buffer, email.variables(6),	driver_rec.lname);
   buffer := replace (buffer, email.variables(7),	driver_rec.mname);
   buffer := replace (buffer, email.variables(8),	driver_rec.initials);

   return buffer;

end replace_variables;

-- ********************************************************************************************

function log_email (this_page integer, ptype common.ident_arr, baldue varchar2,
			subject varchar2, message varchar2) return integer is

email_id	integer;
msg_text	mail_blats.message%TYPE := trim (message) || common.eol$;
sep		varchar2(100);
buffer		payment_codes.payment_descr%TYPE;

begin

   msg_text := msg_text || common.eol$ || htf.bold ('Registration Page: ') ||
		common.decode_value ('page_name', 'pages', 'page_id', this_page);

   sep := common.eol2$ || htf.bold ('Payment Type(s): ');
   for i in 2..ptype.COUNT LOOP
      if ptype(i) = '?' then
	 buffer := 'Unspecified';
      else
	 buffer := common.decode_value ('payment_descr', 'payment_codes',
					'payment_code', ptype(i));
      end if;

      msg_text := msg_text || sep || buffer;
      sep := ', ';
   end loop;

   if baldue = bal_due$ then
      msg_text := msg_text || common.eol2$ || htf.bold ('With Balance Due');
   elsif baldue = bal_zero$ then
      msg_text := msg_text || common.eol2$ || htf.bold ('Paid In Full');
   end if;

-- **********

   select email_id_seq.nextval into email_id from dual;

   insert into emails (email_id, subject, message, timestamp)
	values (email_id, trim (subject), msg_text, sysdate);

   return email_id;

end log_email;

-- ********************************************************************************************

procedure step3 (this_page integer, ptype common.ident_arr, baldue varchar2,
		subject varchar2, message varchar2, submit varchar2) is

type		cursor_type is REF CURSOR;
driver		cursor_type;

buffer		long;
new_subject	mail_blats.subject%TYPE;
recipient	mail_blats.recipient%TYPE;
icount		integer	:= 0;
rec_found	boolean;
mail_id		integer;
next_email_id	integer;

now$		constant date := common.get_UTC_time;
user_email$	mail_blats.recipient%TYPE;

begin

   if common.header (rpt_title$) then
      init_variables;
      user_email$ := common.get_user_email (nvl (common.gbl_priv_pid, common.gbl_pid));
      build_sql_stmt (this_page, ptype, baldue);

      if submit = view_it$ then
         open driver for sql_stmt;
         fetch driver into driver_rec;
         rec_found := driver%FOUND;
         close driver;

         new_subject := replace_variables (this_page, subject);

         if rec_found then
            buffer := replace_variables (this_page, message);
            buffer := replace (buffer, common.eol$, htf.br);
            buffer := replace (buffer, dummy$, '"');

            htp.p ('Subject: ' || new_subject);
            htp.para;
            htp.p (buffer);
            htp.hr;
            htp.header (3, 'Use the browser''s back button to submit the email.');
            htp.para;
         else
            htp.header (3, 'No profiles meet your search criteria.');
         end if;

      else
         next_email_id := log_email (this_page, ptype, baldue, subject, message);

         open driver for sql_stmt;
         LOOP
            fetch driver into driver_rec;
            exit when driver%NOTFOUND;

            new_subject	:= replace_variables (this_page, subject);
            buffer	:= replace_variables (this_page, message);
            recipient	:= common.get_user_email (driver_rec.person_id);
            mail_id	:= send_mail2 (recipient, new_subject, buffer, user_email$,
					pid=>driver_rec.person_id);
            icount	:= icount + 1;

            insert into email_logs (person_id, email_id, mail_id)
		values (driver_rec.person_id, next_email_id, mail_id);
         end loop;
         close driver;

         if icount = 0 then
            delete from emails where email_id = next_email_id;
         end if;

         htp.header (3, icount || ' Email(s) sent');
         htp.formOpen ('reg_email.html');
         htp.formSubmit (null, 'Send Another Email');
         htp.formClose;

      end if;

      common.footer;
   end if;

end step3;

-- ********************************************************************************************
-- ********************************************************************************************

function valid_input (subject varchar2, message varchar2) return boolean is

valid	boolean := true;

begin

   if subject is null then
      htp.header (3, 'Please supply a subject for your email.');
      valid := false;
   end if;

   if replace (replace (message, common.eol$, null), chr(13), null) is null then
      htp.header (3, 'Please supply a message for your email.');
      valid := false;
   end if;

   return valid;

end valid_input;

-- ********************************************************************************************

procedure step2 (this_page integer, ptype common.ident_arr, baldue varchar2,
		subject varchar2, message varchar2) is

begin

   if common.header (rpt_title$) then

      if valid_input (subject, message) then
	 htp.p ( email.format_message ('SUBJECT: ' || subject));
	 htp.hr;
	 htp.p ( email.format_message (message));

	 htp.formOpen ('reg_email.step3', 'POST');
	 htp.formHidden ('this_page',	this_page);
	 common.hidden_array ('ptype',	ptype);
	 htp.formHidden ('baldue',	baldue);
	 htp.formHidden ('subject',	replace (subject, '"', dummy$));
	 htp.formHidden ('message',	replace (message, '"', dummy$));

	 htp.para;
	 htp.formSubmit ('submit', send_it$);
	 htp.formSubmit ('submit', view_it$);
	 htp.formClose;
      end if;

      common.footer;
   end if;

end step2;

-- ********************************************************************************************
-- ********************************************************************************************

procedure show_option (prompt varchar2, code varchar2) is

begin

   htp.p ('<TD>');
   htp.p (common.font2 (prompt));
   htp.br;
   common.show_popup_list (code, multiple=>true);
   htp.p ('</TD>');

end show_option;

-- ********************************************************************************************

procedure show_radio (varname varchar2, varvalue varchar2, prompt varchar2,
			selected varchar2 default null) is

begin

   htp.formRadio (varname, varvalue, selected);
   htp.p (prompt);
   htp.br;

end show_radio;

-- ********************************************************************************************

procedure show_payment_types is

cursor	driver is
select	1 sorder, payment_descr, payment_code
from	payment_codes
UNION
select	0, 'Unspecified', '?'
from	dual;

begin

   htp.p ('<TD NOWRAP>');
   htp.formHidden ('ptype', 'dummy');
   htp.p (common.font2 ('Payment Type'));
   htp.hr;

   for driver_rec in driver LOOP
      htp.formCheckBox ('ptype', driver_rec.payment_code);
      htp.p (driver_rec.payment_descr);
      htp.br;
   end loop;

   htp.p ('</TD>');

end show_payment_types;

-- ********************************************************************************************

procedure show_options is

begin

   htp.header (3, 'Select Registrants to Include.');

   htp.tableOpen (cattributes=>'CELLPADDING="10"');
   htp.tableRowOpen ('LEFT', 'TOP');

   htp.p ('<TD>');
   htp.p (common.font2 ('Registration Page'));
   htp.hr;
   common.show_popup_list ('page', show_null=>false);
   htp.p ('</TD>');

   show_payment_types;

   htp.p ('<TD NOWRAP>');
      htp.p (common.font2 ('Balance Type'));
      htp.hr;
      show_radio ('baldue', bal_ignore$, bal_ignore$, 'C');
      show_radio ('baldue', bal_due$, bal_due$);
      show_radio ('baldue', bal_zero$, bal_zero$);
   htp.p ('</TD>');

   htp.tableRowClose;
   htp.tableClose;
   htp.hr;

end show_options;

-- ********************************************************************************************
-- ********************************************************************************************

procedure html is

begin

   if common.header (rpt_title$) then
      init_variables;
      htp.formOpen ('reg_email.step2', 'POST');
      show_options;

      htp.bold ('Subject: ');
      htp.formText ('subject', 50, 50); -- Max length 50

      htp.header (3, 'Enter the email message below.  Use the substitution parameters below.');
      htp.formTextArea ('message', 15, 75);

      htp.para;
      htp.formSubmit;
      htp.formClose;
      htp.hr;

      email.show_variables;
      common.footer;
   end if;

end html;

-- ********************************************************************************************

end reg_email;

/
show error

set define on

--	Copyright (C) 2004-2008 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>
