-- SPMS - Scientific Programme Management System
-- see at the bottom of this file for licensing
-- and copyright information

set define off

Create or Replace package body reg_form_maint as

rpt_title$	constant varchar2(30)	:= 'Registration ';

disp_txt$	constant display_codes.display_code%TYPE := 'TXT';
disp_YN$	constant display_codes.display_code%TYPE := 'YN';
disp_popup$	constant display_codes.display_code%TYPE := 'POP';
disp_multi$	constant display_codes.display_code%TYPE := 'MULT';
disp_checkbox$	constant display_codes.display_code%TYPE := 'CHK';
disp_mcb$	constant display_codes.display_code%TYPE := 'MCB';

dataTnum$	constant data_type_codes.data_type_code%TYPE := 'NUM';
dataTstr$	constant data_type_codes.data_type_code%TYPE := 'STR';
dataTdte$	constant data_type_codes.data_type_code%TYPE := 'DTE';
dataTfee$	constant data_type_codes.data_type_code%TYPE := 'FEE';
dataTfee2$	constant data_type_codes.data_type_code%TYPE := 'FEE2';
dataTsps$	constant data_type_codes.data_type_code%TYPE := 'SPS';

not_authorized$	constant varchar2(50) := 'You are not authorized to access this page.';
ignore_admin$	boolean default true;
validate_IP$	boolean default true;


-- *******************************************************************************************
-- *******************************************************************************************

procedure link_backs (cat_code varchar2 default null) is

begin

   htp.anchor ('reg_form_maint.html', 'Define Fields');
   htp.p (common.space2$);
   htp.anchor ('reg_form_maint.new?cat_code=' || cat_code || '&page_code=', 'New Data Field');
   htp.p (common.space2$);
   htp.anchor ('reg_form_maint.list_pages', 'Page Layout');
   htp.p (common.space2$);
   htp.anchor ('reg_form_maint.fee_matrix', 'Fee Matrix');
   htp.p (common.space2$);
   htp.anchor2 ('maint.tab?tab_name=reg_cat_codes&order_by=reg_cat_descr', 'Edit Field Groups',
		ctarget=>'_new');
   htp.para;

end link_backs;

-- *******************************************************************************************
-- *******************************************************************************************

procedure fee_matrix_save1 (row_id integer, col_id integer, fee1 integer, fee2 integer) is

cursor	driver is
select	fee_matrix_id
from	fee_matrix
where (	poplist_item_id1	= row_id
and	poplist_item_id2	= col_id)
OR (	poplist_item_id2	= row_id
and	poplist_item_id1	= col_id);

driver_rec	driver%ROWTYPE;
rec_found	boolean;

begin

   open driver;
   fetch driver into driver_rec;
   rec_found := driver%FOUND;
   close driver;

   if rec_found then

      if fee1 is null AND fee2 is null then
	 delete from fee_matrix where fee_matrix_id = driver_rec.fee_matrix_id;
      else
	 update fee_matrix
	 set	fee		= fee1,
		late_fee	= fee2
	 where	fee_matrix_id	= driver_rec.fee_matrix_id;
      end if;

   elsif NOT (fee1 is null AND fee2 is null) then
      insert into fee_matrix (poplist_item_id1, poplist_item_id2, fee, late_fee)
      values (row_id, col_id, fee1, fee2);

   end if;

end fee_matrix_save1;

-- *******************************************************************************************

function valid_fees (fees common.ident_arr) return varchar2 is

msg	long := null;

begin

   for i in 1..fees.count LOOP
      if NOT common.validate_number (trim(fees(i))) then
	 msg := msg || 'Fees must be numeric: ' || fees(i) || htf.br;
      elsif trim (fees(i)) < 0 then
	 msg := msg || 'Fees can not be negative: ' || fees(i) || htf.br;
      end if;
   end loop;

   return msg;

end valid_fees;

-- *******************************************************************************************

procedure fee_matrix_save (row_ids common.ident_arr, col_ids common.ident_arr,
				fee1 common.ident_arr, fee2 common.ident_arr) is

icount	integer := 1;
msg	long := null;

begin

   if common.header (rpt_title$, test_only=>true) then
      msg := valid_fees (fee1) || valid_fees (fee2);
      if msg is null then

	 for i in 1..row_ids.count LOOP
	    for j in 1..col_ids.count LOOP
	       fee_matrix_save1 (row_ids(i), col_ids(j), trim (fee1(icount)), trim (fee2(icount)));
	       icount := icount + 1;
	    end loop;
	 end loop;

	 fee_matrix;

      else
	 common.public_header (rpt_title$);
	 htp.header (3, 'One or more errors found', cattributes=>common.red$);
	 htp.p (msg);
	 htp.p (common.click_back$);
	 htp.para;
	 common.footer;
      end if;
   end if;

end fee_matrix_save;

-- *******************************************************************************************
-- *******************************************************************************************

procedure get_matrix_fees (ID1 integer, ID2 integer, fee1 in out number, fee2 in out number) is

cursor	driver is
select	fee, late_fee
from	fee_matrix
where (	poplist_item_id1 = ID1
and	poplist_item_id2 = ID2)
OR (	poplist_item_id2 = ID1
and	poplist_item_id1 = ID2);

begin

   open driver;
   fetch driver into fee1, fee2;
   close driver;

end get_matrix_fees;

-- *******************************************************************************************

procedure fee_matrix_edit (this_meta_data common.ident_arr) is

cursor	driver (code varchar2) is
select	poplist_item_id, item_value
from	poplist_items
where	meta_data_code	= code
order by sort_order nulls last, item_value;

row_ids		common.int_arr;
row_descr	common.ident_arr;
col_ids		common.int_arr;
col_descr	common.ident_arr;

fee1		number;
fee2		number;

begin

   if common.header (rpt_title$) then
      link_backs;
      htp.p (common.space2$);
      htp.bold ('Note:', cattributes=>common.red$);
      htp.p ('Leave cells empty (not zero) for combinations that are not avaiable.');

      if this_meta_data(1) is null or this_meta_data(2) is null then
	 htp.header (3, 'You must select two fees to build a fee matrix', cattributes=>common.red$);
	 htp.p (common.click_back$);

      elsif this_meta_data(1) = this_meta_data(2) then
	 htp.header (3, 'You must select two <I>different</I> fees to build a fee matrix', cattributes=>common.red$);
	 htp.p (common.click_back$);

      else
	 open driver (this_meta_data(1));
	 fetch driver BULK COLLECT into row_ids, row_descr;
	 close driver;

	 open driver (this_meta_data(2));
	 fetch driver BULK COLLECT into col_ids, col_descr;
	 close driver;

-- ******

	 htp.formOpen ('reg_form_maint.fee_matrix_save', 'POST');
	 for i in 1..row_ids.count LOOP
	    htp.formHidden ('row_ids', row_ids(i));
	 end loop;

	 for i in 1..col_ids.count LOOP
	    htp.formHidden ('col_ids', col_ids(i));
	 end loop;

-- ******

	 htp.tableOpen ('BORDER');

	 htp.tableRowOpen ('LEFT', 'BOTTOM');
	 htp.tableData (common.space$, crowspan=>'2');
	 for i in 1..col_ids.count LOOP
	    htp.tableHeader (col_descr(i), 'CENTER', ccolspan=>'2');
	 end loop;
	 htp.tableRowClose;

-- ******

	 htp.tableRowOpen ('LEFT', 'BOTTOM');
	 for i in 1..col_ids.count LOOP
	    htp.tableHeader ('Fee');
	    htp.tableHeader ('Late');
	 end loop;
	 htp.tableRowClose;

-- ******

	 for i in 1..row_ids.count LOOP
	    htp.tableRowOpen ('LEFT', 'TOP');
	    htp.tableHeader (row_descr(i));

	    for j in 1..col_ids.count LOOP
	       get_matrix_fees (row_ids(i), col_ids(j), fee1, fee2);
	       htp.tableData (htf.formText ('fee1', 5, 5, fee1));
	       htp.tableData (htf.formText ('fee2', 5, 5, fee2));
	    end loop;

	    htp.tableRowClose;
	 end loop;

	 htp.tableClose;
	 htp.para;
	 htp.formSubmit (null, 'Save');
	 htp.formClose;

      end if;

      common.footer;
   end if;

end fee_matrix_edit;

-- *******************************************************************************************
-- *******************************************************************************************

procedure fee_matrix is

cursor	driver is
select	count(1)
from	meta_data_codes
where	data_type_code	= dataTfee$
and	display_code	= disp_popup$;

icount		integer;
where_claws	varchar2(100);
Q$		constant varchar2(1) := common.quote$;

begin

   if common.header (rpt_title$) then
      open driver;
      fetch driver into icount;
      close driver;

      link_backs;

      htp.header (3, 'Fee matrix allows you to define fees based on the selection of two fee popup fields.');
      htp.para;

      if icount < 2 then
	 htp.header (3, 'You must have two popup fees to use this feature', cattributes=>common.red$);

      else
	 where_claws := 'where data_type_code = ' || Q$ || dataTfee$ || Q$ ||
			'and display_code = '  || Q$ || disp_popup$ || Q$;

	 htp.formOpen ('reg_form_maint.fee_matrix_edit', 'POST');

	 htp.bold ('Popup List 1');
	 common.show_popup_list ('meta_data', where_claws=>where_claws);
	 htp.br;

	 htp.bold ('Popup List 2');
	 common.show_popup_list ('meta_data', where_claws=>where_claws);
	 htp.para;

	 htp.formSubmit;
	 htp.formClose;
      end if;

      common.footer;
   end if;

end fee_matrix;

-- *******************************************************************************************
-- *******************************************************************************************

procedure del (meta_code varchar2) is

begin

   delete from meta_data_codes where meta_data_code = meta_code;
   common.redirect_URL ('reg_form_maint.html');

end del;

-- *******************************************************************************************
-- *******************************************************************************************

function popup_list_in_use (item_id integer) return boolean is

cursor	driver is
select	1
from	registrations
where	poplist_item_id = item_id
UNION
select	1
from	fee_matrix
where	poplist_item_id1 = item_id
or	poplist_item_id2 = item_id;

rec_found	boolean;
dummy		integer;

begin

   open driver;
   fetch driver into dummy;
   rec_found := driver%FOUND;
   close driver;

   return rec_found;

end popup_list_in_use;

-- *******************************************************************************************

function valid_popup_data (fee1 varchar2, fee2 varchar2, srt_ord varchar2,
				msg in out varchar2) return boolean is

valid_data	boolean := true;

begin

   if NOT common.validate_number (fee1) then
      msg	 := msg || htf.header (3, 'Fee must be numeric: ' || fee1);
      valid_data := false;
   end if;

   if NOT common.validate_number (fee2) then
      msg	 := msg || htf.header (3, 'Late fee must be numeric: ' || fee2);
      valid_data := false;
   end if;

   if NOT common.validate_number (srt_ord) then
      msg	 := msg || htf.header (3, 'Sort order must be numeric: ' || srt_ord);
      valid_data := false;
   end if;

   return valid_data;

end valid_popup_data;

-- *******************************************************************************************

procedure popup_save (meta_code integer, item_id common.ident_arr, lvalue common.ident_arr,
			fee1 common.ident_arr, fee2 common.ident_arr, srt_ord common.ident_arr) is

msg	long	:= null;

del_cnt	integer	:= 0;
ins_cnt	integer := 0;
upd_cnt	integer := 0;

begin

   for i in 1..item_id.count LOOP
      if item_id(i) is null then

	 if lvalue(i) is not null then
	    if valid_popup_data (fee1(i), fee2(i), srt_ord(i), msg) then
	       insert into poplist_items (meta_data_code, item_value, fee, late_fee, sort_order)
	       values (meta_code, trim (lvalue(i)), fee1(i), fee2(i), srt_ord(i) );
	    end if;
	 end if;

      elsif lvalue(i) is null then

	 if popup_list_in_use (item_id(i)) then
	    msg := msg || htf.header (3, 'Item in use. Unable to delete ' ||
		common.decode_value ('item_value', 'poplist_items', 'poplist_item_id', item_id(i) ) );

	 else
	    delete from poplist_items where poplist_item_id = item_id(i);
	    del_cnt := del_cnt + 1;

	 end if;

      else
	 if valid_popup_data (fee1(i), fee2(i), srt_ord(i), msg) then
	    update	poplist_items
	    set		item_value		 = trim (lvalue(i)),
			fee			 = fee1(i),
			late_fee		 = fee2(i),
			sort_order		 = srt_ord(i)
	    where	poplist_item_id 	 = item_id(i)
	    and	(	item_value		<> trim (lvalue(i))
		or	nvl(fee,-1)		<> nvl(fee1(i),-1)
		or	nvl(late_fee,-1)	<> nvl(fee2(i),-1)
		or	nvl(sort_order,-1)	<> nvl(srt_ord(i),-1)
		);

	    upd_cnt := upd_cnt + SQL%rowcount;
	 end if;

      end if;

   end loop;

   if msg is null then
      popup (meta_code);

   else
      common.public_header (rpt_title$, body_open=>false);
      htp.bodyOpen;
      htp.p (msg);

      htp.p (ins_cnt || ' Items Inserted');
      htp.br;
      htp.p (upd_cnt || ' Items Updated');
      htp.br;
      htp.p (del_cnt || ' Items Deleted');
      htp.br;

      htp.bodyClose;
      htp.htmlClose;
   end if;

end popup_save;

-- *******************************************************************************************
-- *******************************************************************************************

function get_display_type (meta_code integer) return varchar2 is

cursor	driver is
select	display_code
from	meta_data_codes
where	meta_data_code = meta_code;

driver_rec	driver%ROWTYPE;

begin

   open driver;
   fetch driver into driver_rec;
   close driver;

   return driver_rec.display_code;

end get_display_type;

-- *******************************************************************************************

procedure display_popup_item (driver_rec poplist_items%ROWTYPE,
			      data_type$ data_type_codes.data_type_code%TYPE) is

begin

   htp.tableRowOpen ('LEFT', 'TOP');
   htp.tableData (htf.formhidden ('item_id', driver_rec.poplist_item_id)	||
		  htf.formText ('lvalue', 75, 4000, driver_rec.item_value) );

   if data_type$ = dataTfee$ then
      htp.tableData (htf.formText ('fee1', 8, 8, to_char(driver_rec.fee, common.currency_fmt$)));
      htp.tableData (htf.formText ('fee2', 8, 8, to_char(driver_rec.late_fee, common.currency_fmt$)));
   else
      htp.formHidden ('fee1', '');
      htp.formHidden ('fee2', '');
   end if;

   htp.tableData (htf.formText ('srt_ord', 3, 3, driver_rec.sort_order));
   htp.tableRowClose;

end display_popup_item;

-- *******************************************************************************************

procedure popup (meta_code integer) is

cursor	driver is
select	*
from	poplist_items
where	meta_data_code	= meta_code
order by
	sort_order nulls last,
	item_value;

data_type$		constant data_type_codes.data_type_code%TYPE := common.decode_value (
				'data_type_code', 'meta_data_codes', 'meta_data_code', meta_code);
meta_data_descr$	constant meta_data_codes.meta_data_descr%TYPE := common.decode_value (
				'meta_data_descr', 'meta_data_codes', 'meta_data_code', meta_code);

begin

   if common.header (rpt_title$, js_src=>regist_maint.JS$) then
      if get_display_type (meta_code) in (disp_popup$, disp_checkbox$) then
	 htp.header (3, 'Popup Items For ' || meta_data_descr$);

	 htp.anchor ('javascript:closePopWin();', 'Close Window');
	 htp.p (common.space2$);
	 htp.bold ('Note: You must save your changes here and on the main window before closing this window.', cattributes=>common.red$);

	 htp.formOpen ('reg_form_maint.popup_save', 'POST', cattributes=>'NAME="SPMS"');
	 htp.formHidden ('meta_code', meta_code);

	 htp.tableOpen;
	 htp.tableRowOpen ('LEFT', 'BOTTOM');
	 htp.tableHeader ('Item Text');
	 if data_type$ = dataTfee$ then
	    htp.tableHeader ('Fee');
	    htp.tableHeader ('Late Fee');
	 end if;
	 htp.tableHeader ('Sort');
	 htp.tableRowClose;

	 for driver_rec in driver LOOP
	    display_popup_item (driver_rec, data_type$);
	 end loop;

	 for i in 1..5 LOOP
	    display_popup_item (null, data_type$);
	 end loop;

	 htp.tableClose;
	 htp.br;
	 htp.formSubmit (null, 'Save');
	 htp.formClose;
	 common.form_focus ('SPMS', 'lvalue[0]');

      else
	 htp.header (3, 'The display type for this item is not a Popup List.',
			cattributes=>common.red$);
      end if;

      htp.bodyClose;
      htp.htmlClose;
   end if;

end popup;

-- *******************************************************************************************
-- *******************************************************************************************

procedure reseq (pid integer) is

cursor	driver is
select	rowid
from	page_data
where	page_id = pid
order by
	sort_order;

iseq	integer := 10;

begin

   for driver_rec in driver LOOP
      update	page_data
      set	sort_order	= iseq
      where	rowid		= driver_rec.rowid;

      iseq := iseq + 10;
   end loop;

end reseq;

-- *******************************************************************************************
-- *******************************************************************************************

procedure pdel (pid integer) is

begin

   delete from pages where page_id = pid;
   common.redirect_URL ('reg_form_maint.list_pages');

end pdel;

-- *******************************************************************************************
-- *******************************************************************************************

procedure psave1 (name varchar2, pid integer, pdid integer, meta varchar2, txt varchar2, sort varchar2) is

this_pid	integer := pid;

begin

   if meta||trim(txt) is null then
      delete from page_data where page_data_id = pdid;

   else
      update	page_data
      set	meta_data_code		= meta,
		text			= trim (txt),
		sort_order		= sort
      where	page_data_id		= pdid;

      if SQL%rowcount = 0 then

	 insert into page_data (page_data_id, page_id, meta_data_code, text, sort_order)
	 values (page_data_id.nextval, this_pid, meta, trim(txt), sort);
      end if;

   end if;

end psave1;

-- *******************************************************************************************

function psave_valid (meta varchar2, txt varchar2, sort varchar2) return varchar2 is

msg	varchar2(2000) := null;

begin

   if meta||trim(txt)||trim(sort) is null then
      return null;
   end if;

   if sort is null then
      msg := htf.header (3, 'You must Specify a Sort Order');
   elsif NOT common.validate_number (sort) then
      msg := htf.header (3, 'Sort Order Must Be A Number');
   end if;

   return msg;

end psave_valid;

-- *******************************************************************************************

function page_name_in_use (text varchar2) return boolean is

cursor	driver is
select	1
from	pages
where	page_name = text;

rec_found	boolean;
dummy		integer;

begin

   open driver;
   fetch driver into dummy;
   rec_found := driver%FOUND;
   close driver;

   return rec_found;

end page_name_in_use;

-- *******************************************************************************************

procedure psave (name varchar2, pid integer, pdid common.ident_arr,
		this_meta_data common.ident_arr, txt common.ident_arr,
		sort common.ident_arr) is

this_pid	integer := pid;
msg		varchar2(2000) := null;

begin

   if pid is null then
      if trim (name) is null then
	 msg := htf.header (3, 'You must supply a name for your page.');

      elsif page_name_in_use (trim (name)) then
	 msg := htf.header (3, 'A page with this name already exists.');

      else
	 insert into pages (page_name) values (trim (name));
      end if;
   end if;

   for i in 1..pdid.count LOOP
      exit when msg is not null;
      msg := psave_valid (this_meta_data(i), txt(i), sort(i));

      if msg is null then
	 psave1 (name, this_pid, pdid(i), this_meta_data(i), txt(i), sort(i));
      end if;
   end loop;

   if msg is null then
      reseq (this_pid);
      common.redirect_URL ('reg_form_maint.pedit1?PID=' || this_pid);
   else

      common.public_header (rpt_title$, body_open=>false);
      htp.p (msg);
      htp.p (common.click_back$);
      htp.bodyClose;
      htp.htmlClose;
   end if;

end psave;

-- *******************************************************************************************
-- *******************************************************************************************

procedure pedit2 (pid integer) is

begin

   if common.header (null, body_open=>false, js_src=>regist_maint.JS$) then
      htp.anchor ('javascript:closePopWin();', 'Close Window');
      htp.para;
      registration.build_data_entry_form (pid, false,null);
      htp.bodyClose;
      htp.htmlClose;
   end if;

end pedit2;

-- *******************************************************************************************
-- *******************************************************************************************

procedure pedit1_show (driver_rec page_data%ROWTYPE, ord integer) is


begin

   htp.tableRowOpen ('LEFT', 'TOP');

   htp.p ('<TD>');
   htp.formHidden ('pdid', driver_rec.page_data_id);
   common.show_popup_list ('meta_data', driver_rec.meta_data_code);
   htp.p ('</TD>');


   if  ord is not null then
	htp.tableData (htf.formText ('sort', 5, 5, ord));
   else
	htp.tableData (htf.formText ('sort', 5, 5, driver_rec.sort_order));
   end if;
   htp.tableData (	htf.formTextAreaOpen ('txt', 2, 50)	||
			driver_rec.text				||
			htf.formTextAreaClose);
   htp.tableRowClose;

end pedit1_show;

-- *******************************************************************************************

procedure pedit1 (pid integer,cat_code varchar2 default null) is

counter integer :=10;

cursor	driver is
select	*
from	page_data
where	page_id = pid
order by
	sort_order;

begin

   if common.header (rpt_title$, js_src=>regist_maint.JS$) then
      htp.bold ('Page Designer - ' ||
		common.decode_value ('page_name', 'pages', 'page_id', pid) || ' Page',
		cattributes=>common.red$);
      htp.p (common.space2$);
      htp.anchor ('javascript:popUp(''reg_form_maint.pedit2?pid=' ||
					pid || ''')', 'View Sample Data Entry Screen');
      htp.p (common.space2$);
      link_backs;

      htp.formOpen ('reg_form_maint.psave', 'POST');
      htp.formHidden ('pid', pid);
      htp.tableOpen;
      htp.tableRowOpen();
      htp.p('<td colspan=3>');

      if pid is null then
	 htp.bold ('Page Name: ');
	 htp.formText ('name', 40, 80);
	 htp.para;
      else
	 htp.formHidden ('name', 'dummy');
      end if;

      htp.p('</td>');
      htp.tableRowClose();

      htp.tableRowOpen ('LEFT', 'BOTTOM');
      htp.tableHeader ('Data Field');
      htp.tableHeader ('Order');
      htp.tableHeader ('Text');
      htp.tableRowClose;

      for driver_rec in driver LOOP
	 pedit1_show (driver_rec,null);
	 counter := counter + 10;
      end loop;

      for i in 1..3 LOOP
	 pedit1_show (null, counter);
	 counter := counter + 10;
      end loop;

      htp.tableClose;
      htp.para;
      htp.formSubmit;
      htp.formClose;
      htp.para;
      common.footer;
   end if;

end pedit1;

-- *******************************************************************************************

procedure pedit (pid integer, cat_code varchar2 default null) is

begin

   pedit1 (pid,cat_code);

end pedit;

-- *******************************************************************************************
-- *******************************************************************************************

procedure pflag (pid integer, flag varchar2) is

begin

   if flag = 'A' then
      update	pages
      set	available	= decode (available, 'Y','N', 'Y')
      where	page_id		= pid;

   elsif flag = 'P' then
      update	pages
      set	show_on_profile	= decode (show_on_profile, 'Y','N', 'Y')
      where	page_id		= pid;

   elsif flag = 'C' then
      update	pages
      set	show_on_abstract	= decode (show_on_abstract, 'Y','N', 'Y')
      where	page_id		= pid;

   elsif flag = 'V' then
      update	pages
      set	admin_privs	= decode (admin_privs, 'Y','N', 'Y')
      where	page_id		= pid;

   elsif flag = 'L' then
      update	pages
      set	login_req	= decode (login_req, 'Y','N', 'Y')
      where	page_id		= pid;

   end if;

   common.redirect_URL ('reg_form_maint.list_pages');

end pflag;

-- *******************************************************************************************
-- *******************************************************************************************

procedure preturn_save (ID integer, URL varchar2) is

begin

   update pages
   set	  return_URL	= URL
   where  page_id	= ID;

   common.redirect_URL ('reg_form_maint.list_pages');

end preturn_save;

-- *******************************************************************************************
-- *******************************************************************************************

procedure preturn (ID integer) is

cursor	driver is
select	*
from	pages
where	page_id = ID;

driver_rec	driver%ROWTYPE;

begin

   if common.header (rpt_title$) then
      open driver;
      fetch driver into driver_rec;
      close driver;

      htp.header (3, 'Enter the return URL');
      htp.p ('Enter the URL to which the user will return upon successfully completing this page.');
      htp.para;
      htp.formOpen ('reg_form_maint.preturn_save', 'POST', cattributes=>'NAME="SPMS"');
      htp.formHidden ('ID', ID);
      htp.formText ('URL', 50, 200, driver_rec.return_URL);
      htp.formSubmit (null, 'Save');
      htp.formClose;
      common.form_focus ('SPMS', 'URL');
      htp.para;
      common.footer;
   end if;

end preturn;

-- *******************************************************************************************
-- *******************************************************************************************

procedure pattr_save (pid integer, pname varchar2, email varchar2, maxr varchar2, txt varchar2,
			eURL varchar2, bURL varchar2, hide_inv varchar2,
			incl_rpt varchar2, incl_stats varchar2) is

cursor	driver is
select	1
from	pages
where	page_name	= trim (pname)
and	page_id		<> pid;

dummy		integer;
rec_found	boolean;
buffer		long := null;

begin

   if common.header (rpt_title$, test_only=>true) then
      if trim (pname) is null then
	 buffer := buffer || htf.header (3, 'You must supply a page name.');
      else
	 open driver;
	 fetch driver into dummy;
	 rec_found := driver%FOUND;
	 close driver;

	 if rec_found then
	    buffer := buffer || htf.header (3, 'Page name must be unique.');
	 end if;
      end if;

      if NOT (maxr is null or common.validate_number (maxr) ) then
	 buffer := buffer || htf.header (3, 'Maximum registrats must be numeric.');
      end if;

      if eURL is not null then
	 if instr (lower (eURL), '?external=' || pid) = 0 then
	    buffer := buffer || htf.header (3, 'You must include the query string '	||
			'?external=' || pid || ' in the External URL');
	 end if;
      end if;

      if trim (eURL) is not null and trim (bURL) is not null then
	 buffer := buffer || htf.header (3, 'External CC Vendor URL and External Registration URL are mutually exclusive.');
      end if;

      if buffer is null then
	 update	pages
	 set	max_registrants		= maxr,
		page_name		= trim (pname),
		admin_email		= trim (email),
		post_info		= trim (txt),
		hide_invoice		= hide_inv,
		external_URL		= trim (external_URL),
		include_participant_rpt	= incl_rpt,
		include_statistics	= incl_stats,
		bypass_URL		= trim (bURL)
	 where	page_id			= pid;

	 common.redirect_url (common.vdir$ || 'reg_form_maint.list_pages');
      else
	 common.public_header (rpt_title$);
	 htp.header (3, buffer);
	 htp.bold (common.click_back$);
	 common.footer;
      end if;

   end if;

end pattr_save;

-- *******************************************************************************************
-- *******************************************************************************************

procedure pattrib (PID integer) is

cursor	driver is
select	*
from	pages
where	page_id = PID;

driver_rec	driver%ROWTYPE;
URL		varchar2(200);


begin

   if common.header (rpt_title$) then
      open driver;
      fetch driver into driver_rec;
      close driver;

      link_backs;
      htp.p (common.space2$);
      htp.formOpen ('reg_form_maint.pattr_save', 'POST', cattributes=>'NAME="SPMS"');
      htp.formHidden ('pid', PID);

      htp.tableOpen (cattributes=>'CELLPADDING="3"');

      common.show_data_row ('Page Name',	htf.formText ('pname', 40, 80, driver_rec.page_name)	||
						common.space2$ || 'Page name must be unique.');

      common.show_data_row ('Admin''s Email',	htf.formText ('email', 35, 100, driver_rec.admin_email)	||
						common.space2$ || 'Leave blank to send registration '	||
						'email to the conference administrator.');

      common.show_data_row ('Max Registrants',	htf.formText ('maxr', 5, 5, driver_rec.max_registrants)  ||
						common.space2$ || 'Leave blank for unlimited registrants.');

      common.show_data_row ('Completion Text',	htf.formText ('txt', 60, 4000, htf.escape_sc(driver_rec.post_info)) ||
						common.space2$ || 'Text displayed upon completing this form.');

      common.show_data_row ('Return URL',	htf.anchor ('reg_form_maint.preturn?ID=' || driver_rec.page_id,
						nvl (driver_rec.return_URL, 'None: Will Close Window') ) );

      URL := htf.anchor ('maint.tab?tab_name=payment_vendor_vars&order_by=2,6,4,3', 'Payment Vendor Vairables');
      common.show_data_row ('External CC Vendor URL',
						htf.formText ('eURL', 60, 200, driver_rec.external_URL)	||
						common.space2$ || 'Pass SPMS registration data to an '	||
						'external credit card (CC) processing vendor.'		||
						common.space2$ || htf.bold ('Note:')			||
						'This requires then name/value pairs named in CC '	||
						URL || '.  Also, you ' || htf.bold ('must')		||
						' include the name/value pair of ?external=' || PID	||
						' in the External URL text above ' || htf.bold ('and')	||
						' it must be passed back to the !cc_vendor.IPN '	||
						'routine by the external registration system.');

      URL := htf.anchor ('maint.tab?tab_name=regist_bypass_vars&order_by=page_id,var_name',
			'Bypass Name/Value Pairs');
      common.show_data_row ('External Registration URL',
						htf.formText ('bURL', 60, 200, driver_rec.bypass_URL)	||
						common.space2$ || 'Bypass SPMS registration completely'	||
						' for an external registration system.'			||
						common.space2$ || htf.bold ('Note: ') || 'You must '	||
						'define the data transmitted to the external '		||
						'registration system via the ' || URL || ' page.');

      common.show_data_row ('Hide Receipt',	common.radio_yes_no ('hide_inv',
						nvl (driver_rec.hide_invoice, 'N'), false) || htf.br ||
						htf.bold ('Warning: ') || 'Do not hide invoices for pages with fees.');

      common.show_data_row ('Include Registrants on List Of Participants Report',
						common.radio_yes_no ('incl_rpt',
						nvl (driver_rec.include_participant_rpt, 'Y'), false) );

      common.show_data_row ('Include Registrants on Statistical Reports',
						common.radio_yes_no ('incl_stats',
						nvl (driver_rec.include_statistics, 'Y'), false) );

      URL := registration.get_reg_page_URL (driver_rec.page_id, 'page');
      URL := replace (URL, '_debug', null);
      common.show_data_row ('Data Entry URL', htf.anchor (URL, URL) );

      URL := common.server;
      if driver_rec.admin_privs = 'Y' then
	 URL := common.sserver;
      end if;

      /*
      URL := registration.get_reg_page_URL (driver_rec.page_id, 'rpt');
      URL := replace (URL, '_debug', null);
      common.show_data_row ('Report URL', htf.anchor (URL, URL) );
      */

      common.show_data_row (null, htf.formSubmit);
      htp.tableClose;
      htp.para;
      htp.formClose;
      common.form_focus ('SPMS', 'pname');
      common.footer;
   end if;

end pattrib;

-- *******************************************************************************************
-- *******************************************************************************************

function page_in_use (pid integer) return boolean is

cursor	driver is
select	1
from	page_data
where	page_id = pid;

rec_found	boolean;
dummy		integer;

begin

   open driver;
   fetch driver into dummy;
   rec_found := driver%FOUND;
   close driver;

   return rec_found;

end page_in_use;

-- *******************************************************************************************

procedure list_pages(action varchar2 default null) is

cursor	driver is
select	*
from	pages
order by
	page_name;

begin

   if common.header (rpt_title$) then
      link_backs;
      htp.p (common.space2$);
     -- htp.anchor ('reg_form_maint.pedit?PID=', 'Create a New Page');
      htp.anchor ('reg_form_maint.pedit?PID=',htf.IMG(common.newImg$,cattributes=>'border=0 alt="New page" title="New Page"'));
      htp.p(common.space$);
      if(action = 'wizard') then
	htp.anchor('reg_form_maint.list_fields?cat_code=',htf.IMG(common.prevImg$,cattributes=>'border=0 alt="Back" title="Back"'));
	htp.p('&nbsp;');
	htp.p(htf.anchor('jacow.html',htf.IMG(common.cancelImg$,cattributes=>'border=0 alt="Cancel" title="Cancel"'), cattributes=>'onclick=''return confirmation()'''));
      end if;
      htp.para;

      htp.tableOpen ('BORDER', cattributes=>'CELLPADDING="3"');
      htp.tableCaption ('Click on the page name to add fields to the layout.  Click edit to update addtional page properties.');

      htp.tableRowOpen ('CENTER', 'BOTTOM');
      htp.tableHeader (common.space$, crowspan=>'2');
      htp.tableHeader ('Page Name',			'LEFT', crowspan=>'2');
      htp.tableHeader ('Delete',			crowspan=>'2');
      htp.tableHeader ('Add available Payments', crowspan=>'2');
      htp.tableHeader ('Available<BR>To Users',		crowspan=>'2');
      htp.tableHeader ('Login<BR>Required',		crowspan=>'2');
      htp.tableHeader ('Admin Priv<BR>Required to<BR>View Data',crowspan=>'2');
      htp.tableHeader ('Show This Page As A Link On',	'CENTER', ccolspan=>'2');
      htp.tableRowClose;

      htp.tableRowOpen ('LEFT', 'BOTTOM');
      htp.tableHeader ('Profile Page');
      htp.tableHeader ('Contribution Page');
      htp.tableRowClose;

      for driver_rec in driver LOOP
	 htp.tableRowOpen ('LEFT', 'TOP');

	 htp.tableData (htf.anchor ('reg_form_maint.pattrib?PID=' || driver_rec.page_id,
					htf.IMG(common.editImg$,cattributes=>'border=0 alt=Edit title=Edit')) );

	 htp.tableData (htf.anchor ('reg_form_maint.pedit?PID=' || driver_rec.page_id,
					driver_rec.page_name) );

	 if page_in_use (driver_rec.page_id) then
	    htp.tableData (htf.IMG(common.usedImg$,cattributes=>'border=0 alt=Used title=Used'));
	 else
	    htp.tableData (htf.anchor ('reg_form_maint.pdel?PID=' || driver_rec.page_id, htf.IMG(common.deleteImg$,cattributes=>'border=0 alt=Delete title=Delete')) );
	 end if;
	-- htp.tableData (htf.anchor ('reg_form_maint.add_available_payments?PID=' || driver_rec.page_id || '&action=' || action, 'Add'));
	 htp.p('<td>');
	    show_selected_payments(driver_rec.page_id);
	    htp.p(htf.anchor ('reg_form_maint.add_available_payments?PID=' || driver_rec.page_id || '&action=' || action, 'Associate/Modify'));
	 htp.p('</td>');
	 htp.tableData (htf.anchor ('reg_form_maint.pflag?pid=' || driver_rec.page_id || '&flag=A',
					common.yes_no (driver_rec.available) ), 'CENTER' );
	 htp.tableData (htf.anchor ('reg_form_maint.pflag?pid=' || driver_rec.page_id || '&flag=L',
					common.yes_no (driver_rec.login_req) ), 'CENTER' );
	 htp.tableData (htf.anchor ('reg_form_maint.pflag?pid=' || driver_rec.page_id || '&flag=V',
					common.yes_no (driver_rec.admin_privs) ), 'CENTER' );
	 htp.tableData (htf.anchor ('reg_form_maint.pflag?pid=' || driver_rec.page_id || '&flag=P',
					common.yes_no (driver_rec.show_on_profile) ), 'CENTER' );
	 htp.tableData (htf.anchor ('reg_form_maint.pflag?pid=' || driver_rec.page_id || '&flag=C',
					common.yes_no (driver_rec.show_on_abstract) ), 'CENTER' );

	 htp.tableRowClose;
      end loop;

      htp.tableClose;
      htp.para;
      common.footer;
   end if;

end list_pages;

-- *******************************************************************************************
-- *******************************************************************************************

procedure save1 (code number, promp varchar2, short_prompt varchar2, this_data_type varchar2,
		this_display varchar2, fee1 varchar2, fee2 varchar2, cutoff varchar2, mlen varchar2,
		vlen varchar2, crypt varchar2, freeze varchar2, single varchar2,
		req varchar2, this_reg_cat varchar2, flen varchar2, maxchk varchar2, maxsps varchar2 default null) is

cursor	driver is
select	meta_data_seq.nextval
from	dual;

new_req		meta_data_codes.required%TYPE;
next_code	meta_data_codes.meta_data_code%TYPE := code;
conditional	boolean := false;

begin

   if req is null then
      new_req		:= 'N';
   elsif common.validate_number (req) then
      new_req		:= 'N';
      conditional	:= true;
   else
      new_req		:= req;
   end if;

   if code is null then
      open driver;
      fetch driver into next_code;
      close driver;

      insert into meta_data_codes
	(meta_data_code, meta_data_descr, data_type_code, display_code, data_length,
		 view_length, required, fixed_length, fee, late_fee, cutoff_date, short_descr, encrypt,
		 freeze_updates, single_selection, reg_cat_code, checkbox_max_items, dropdown_max_items)
	values (next_code, trim(promp), this_data_type,
		this_display, mlen, vlen, new_req, flen, fee1, fee2, to_date (cutoff, common.date_input_fmt_str$),
		trim(short_prompt), crypt, freeze, single, this_reg_cat, maxchk, maxsps);
   else
      update	meta_data_codes
      set	meta_data_descr		= trim(promp),
		short_descr		= trim(short_prompt),
		data_type_code		= this_data_type,
		display_code		= this_display,
		data_length		= mlen,
		view_length		= vlen,
		required		= new_req,
		fixed_length		= flen,
		fee			= fee1,
		late_fee		= fee2,
		cutoff_date		= to_date (cutoff, common.date_input_fmt_str$),
		encrypt			= crypt,
		freeze_updates		= freeze,
		single_selection	= single,
		reg_cat_code		= this_reg_cat,
		checkbox_max_items	= maxchk,
		dropdown_max_items	= maxsps
      where	meta_data_code		= code;

      if new_req in ('Y', 'N') then
	 delete from conditional_requirements
	 where meta_data_code = next_code;
      end if;

   end if;

   if conditional then
      delete from conditional_requirements
      where meta_data_code = next_code;

      insert into conditional_requirements (yesno_meta_code, meta_data_code)
      values (req, code);
   end if;

end save1;

-- *******************************************************************************************

function valid_input (promp varchar2, short_prompt varchar2, fee1 varchar2, fee2 varchar2,
			cutoff varchar2, mlen varchar2, vlen varchar2,
			this_reg_cat varchar2, maxchk varchar2) return varchar2 is

msg	varchar2(2000) := null;

begin

   if trim (promp) is null then
      msg := htf.header (3, 'Prompt is required', cattributes=>common.red$);
   end if;

   if trim (short_prompt) is null then
      msg := msg || htf.header (3, 'Short description is required', cattributes=>common.red$);
   end if;

   if NOT common.validate_number (fee1) then
      msg := msg || htf.header (3, 'Fee must be a number', cattributes=>common.red$);
   end if;

   if NOT common.validate_number (fee2) then
      msg := msg || htf.header (3, 'Late fee must be a number', cattributes=>common.red$);
   end if;

   if NOT common.validate_date (cutoff, common.date_input_fmt_str$) then
      msg := msg || htf.header (3, 'Invalid late fee cutoff date', cattributes=>common.red$);
   end if;

   if NOT common.validate_number (maxchk) then
      msg := msg || htf.header (3, 'Max checkboxes per person must be a number.  Leave blank for unlimited', cattributes=>common.red$);
   end if;

   if this_reg_cat is null then
      msg := msg || htf.header (3, 'You must select a registration group.', cattributes=>common.red$);
   end if;

   if trim (mlen) is null then
      msg := msg || htf.header (3, 'Max Len is required', cattributes=>common.red$);
   elsif NOT common.validate_number (mlen) then
      msg := msg || htf.header (3, 'Max Len must be a number', cattributes=>common.red$);
   end if;

   if trim (vlen) is null then
      msg := msg || htf.header (3, 'View Len is required', cattributes=>common.red$);
   elsif NOT common.validate_number (vlen) then
      msg := msg || htf.header (3, 'View Len must be a number', cattributes=>common.red$);
   end if;

   return msg;

end valid_input;

-- *******************************************************************************************
-- *******************************************************************************************

procedure upd (code varchar2, promp varchar2, short_prompt varchar2, this_data_type varchar2,
		fee1 varchar2, fee2 varchar2, cutoff varchar2, this_display varchar2,
		mlen varchar2, vlen varchar2, req varchar2, maxchk varchar2,
		this_reg_cat varchar2, flen varchar2, crypt common.ident_arr,
		freeze common.ident_arr, single common.ident_arr, maxsps varchar2 default null) is

msg	varchar2(2000) := null;

-- Ignore the fees if the data type is not Fee/Mult
f1	varchar2(10) := null;
f2	varchar2(10) := null;
cut	varchar2(10) := null;

begin

   if this_data_type in (dataTfee$, dataTfee2$) then
      f1  := fee1;
      f2  := fee2;
      cut := cutoff;
   end if;

   msg := valid_input (promp, short_prompt, f1, f2, cut, mlen, vlen, this_reg_cat, maxchk);
   if msg is NOT null then
      common.public_header (rpt_title$);
      htp.p (msg);
      common.footer;

   elsif common.header (rpt_title$, test_only=>true) then
      save1 (code, promp, short_prompt, this_data_type, this_display,
		f1, f2, cut, mlen, vlen, 'N', 'N', 'N', req, this_reg_cat, flen, maxchk, maxsps);

      if crypt.COUNT = 2 then  -- Now set the checked ones to yes
	 update	meta_data_codes
	 set	encrypt		= 'Y'
	 where	meta_data_code	= crypt(2);
      end if;

      if freeze.COUNT = 2 then	-- Now set the checked ones to yes
	 update	meta_data_codes
	 set	freeze_updates	= 'Y'
	 where	meta_data_code	= freeze(2);
      end if;

      if single.COUNT = 2 then -- Now set the checked ones to yes
	 update	meta_data_codes
	 set	single_selection= 'Y'
	 where	meta_data_code	= single(2);
      end if;

      common.redirect_URL ('reg_form_maint.fields?cat_code=' || this_reg_cat);
   end if;

end upd;

-- *******************************************************************************************
-- *******************************************************************************************

procedure save (promp varchar2, short_prompt varchar2, this_data_type varchar2, this_display varchar2,
		fee1 varchar2, fee2 varchar2, cutoff varchar2, mlen varchar2, vlen varchar2,
		crypt common.ident_arr, freeze common.ident_arr, single common.ident_arr,
		req varchar2, this_reg_cat varchar2, flen varchar2, page_code varchar2) is

msg	varchar2(2000);
crypt2	varchar2(1) := 'N';
freeze2	varchar2(1) := 'N';
single2	varchar2(1) := 'N';

begin

   if common.header (rpt_title$, test_only=>true) then
      msg := valid_input (promp, short_prompt, fee1, fee2, cutoff, mlen, vlen, this_reg_cat, null);
      if msg is null then

	 if crypt.count = 2 then
	    crypt2 := 'Y';
	 end if;

	 if freeze.count = 2 then
	    freeze2 := 'Y';
	 end if;

	 if single.count = 2 then
	    single2 := 'Y';
	 end if;

	 save1 (null, promp, short_prompt, this_data_type, this_display, null, null, null,
			mlen, vlen, crypt2, freeze2, single2, req, this_reg_cat, flen, null);

	 if page_code = 'list_fields' then
	    common.redirect_URL ('reg_form_maint.list_fields?cat_code=' || this_reg_cat);
	 else
	    common.redirect_URL ('reg_form_maint.fields?cat_code=' || this_reg_cat);
	end if;

      else
	 common.public_header (rpt_title$);
	 htp.p (msg);
	 htp.p (common.click_back$);
	 common.footer;
      end if;
   end if;

end save;

-- *******************************************************************************************
-- *******************************************************************************************

procedure show_required_popup (meta_code integer, var_value varchar2) is

cursor	driver is
select	1			sort_order,
	'Yes'			descr,
	'Y'			code
from	dual
UNION
select	2			sort_order,
	'No'			descr,
	'N'			code
from	dual
UNION
select	9			sort_order,
	meta_data_descr		descr,
	to_char(meta_data_code)	code
from	meta_data_codes
where	meta_data_code		<> meta_code
and	display_code		= disp_YN$;

selected	varchar2(8)	:= null;

begin

   if var_value is null then
      selected := 'SELECTED';
   end if;

   htp.tableRowOpen ('LEFT', 'TOP');
   htp.tableData ('Required', 'RIGHT',cattributes => 'class=boldtext');

   htp.p ('<TD>');
   htp.formSelectOpen ('req');
   htp.formSelectOption ('', selected);

   for driver_rec in driver LOOP
      selected := null;
      if driver_rec.code = var_value then
	 selected := 'SELECTED';
      end if;
      htp.formSelectOption (driver_rec.descr, selected, cattributes=>'VALUE="' || driver_rec.code || '"');
   end loop;

   htp.formSelectClose;
   htp.p ('</TD>');

   htp.tableRowClose;

end show_required_popup;

-- *******************************************************************************************
-- *******************************************************************************************

procedure check_box (prompt varchar2, varname varchar2, value varchar2, meta_code varchar2, msg varchar2) is

checked	varchar2(7) := null;

begin

   if nvl (value, 'N') = 'Y' then
      checked := 'CHECKED';
   end if;

   common.show_data_row (prompt, htf.formCheckBox (varname, meta_code, checked) || common.space2$ || msg);

end check_box;

-- *******************************************************************************************

procedure new (cat_code varchar2, page_code varchar2) is

begin

   if common.header (rpt_title$) then
      htp.p(htf.anchor('javascript:history.go(-1);',htf.IMG(common.prevImg$,cattributes=>'border=0 alt="Back" title="Back"'))
	    || '&nbsp;' || htf.anchor('jacow.html',htf.IMG(common.cancelImg$,cattributes=>'border=0 alt="Cancel" title="Cancel"'), cattributes=>'onclick=''return confirmation()'''));

      htp.formOpen ('reg_form_maint.save?page_code=' || page_code, 'POST', cattributes=>'NAME="SPMS"');
      htp.formHidden ('crypt',	'dummy');
      htp.formHidden ('freeze', 'dummy');
      htp.formHidden ('single', 'dummy');
     -- htp.formHidden('cat_code',cat_code);
     --htp.formHidden('page_code',page_code);
      htp.tableOpen;

      common.show_data_row ('Prompt',		htf.formText ('promp', 75, 4000) );
      common.show_data_row ('Short Description',htf.formText ('short_prompt', 20, 20));
      common.show_lookup_table ('reg_cat',	'Group', cat_code);
      common.show_lookup_table ('data_type',	'Data Type', 'STR', show_null=>false);
      common.show_lookup_table ('display',	'Display', 'TXT', show_null=>false);
      common.show_data_row ('Fee',		htf.formText ('fee1', 10, 10) || 'Valid for Data Type Fee/Mult Only');
      common.show_data_row ('Late Fee',		htf.formText ('fee2', 10, 10) || 'Valid for Data Type Fee/Mult Only' );
      common.show_data_row ('Late Fee Cutoff Date',	htf.formText ('cutoff', 10, 10) ||
								'Format: ' || common.date_input_fmt_str$);
      common.show_data_row ('Max Len',		htf.formText ('mlen', 10, 10) );
      common.show_data_row ('View Len',		htf.formText ('vlen', 10, 10) );

      check_box ('Encrypt',	'crypt',	null, null, 'Encrypt the users'' data within the database.  Recommended for credit card data.');
      check_box ('Freeze',	'freeze',	null, null, 'Do not allow the user to change their registration once processing is complete.');
      check_box ('Single',	'single',	null, null, 'Restrict selection to the first registrant to select an item.');

      htp.tableRowOpen ('LEFT', 'TOP');
      show_required_popup (0, null);
      htp.tableRowClose;

      common.show_data_row ('Fixed Len',	common.popup_yes_no ('flen',  'N') );
      common.show_data_row (htf.formReset,	htf.formSubmit);

      htp.tableClose;
      htp.formClose;
      common.form_focus ('SPMS', 'promp');
      common.footer;
   end if;

end new;

-- *******************************************************************************************
-- *******************************************************************************************

function has_popup_list_items (meta_code integer) return boolean is

cursor	driver is
select	1
from	poplist_items
where	meta_data_code = meta_code;

rec_found	boolean;
dummy		integer;

begin

   open driver;
   fetch driver into dummy;
   rec_found := driver%FOUND;
   close driver;

   return rec_found;

end has_popup_list_items;

-- *******************************************************************************************

function meta_data_in_use (meta_code integer) return boolean is

cursor	driver is
select	1
from	registrations
where	meta_data_code = meta_code
UNION
select	1
from	page_data
where	meta_data_code = meta_code;

rec_found	boolean;
dummy		integer;

begin

   open driver;
   fetch driver into dummy;
   rec_found := driver%FOUND;
   close driver;

   return rec_found;

end meta_data_in_use;

-- *******************************************************************************************

procedure save_warning is

begin

   htp.formSubmit (null, 'Save');
   htp.p (common.space2$);
   htp.bold ('Warning:', cattributes=>common.red$);
   htp.bold ('Save your changes before editing the Popup Items List.');
   htp.para;

end save_warning;

-- *******************************************************************************************

function get_conditional_req (meta_code integer, req varchar2) return varchar2 is

cursor	driver is
select	yesno_meta_code
from	conditional_requirements
where	meta_data_code = meta_code;

driver_rec	driver%ROWTYPE;
rec_found	boolean;

begin

   open driver;
   fetch driver into driver_rec;
   rec_found := driver%FOUND;
   close driver;

   if rec_found then
      return driver_rec.yesno_meta_code;
   end if;

   return req;

end get_conditional_req;

-- *******************************************************************************************

procedure edit (meta_code integer) is

cursor	driver is
select	*
from	meta_data_codes
where	meta_data_code = meta_code;

driver_rec	driver%ROWTYPE;

begin

   if common.header (rpt_title$, js_src=>regist_maint.JS$) then
      open driver;
      fetch driver into driver_rec;
      close driver;

      htp.formOpen ('reg_form_maint.upd', 'POST', cattributes=>'NAME="SPMS"');
      htp.formHidden ('code', meta_code);
      htp.formHidden ('crypt', 'dummy');
      htp.formHidden ('freeze', 'dummy');
      htp.formHidden ('single', 'dummy');

      if NOT (driver_rec.display_code = disp_txt$ AND driver_rec.data_type_code = dataTstr$) then
	 htp.formHidden ('mlen', driver_rec.data_length);
	 htp.formHidden ('vlen', driver_rec.view_length);
	 htp.formHidden ('flen', driver_rec.fixed_length);
      end if;

      if driver_rec.data_type_code not in (dataTfee$, dataTfee2$) then
	  htp.formHidden ('fee1', '');
	  htp.formHidden ('fee2', '');
	  htp.formHidden ('cutoff', '');
      end if;

      if driver_rec.display_code <> disp_checkbox$ then
	  htp.formHidden ('maxchk', '');
      end if;

      htp.tableOpen;

      common.show_data_row ('Prompt',	htf.formText ('promp', 75, 4000, htf.escape_sc (driver_rec.meta_data_descr)));
      common.show_data_row ('Short Description',htf.formText ('short_prompt', 20, 20, driver_rec.short_descr) ||
						common.space2$ || 'Used as labels in exports.');
      common.show_lookup_table ('data_type',	'Data Type',	driver_rec.data_type_code);
      common.show_lookup_table ('display',	'Display Type',	driver_rec.display_code);

      if driver_rec.data_type_code in (dataTfee$, dataTfee2$) then
	 common.show_data_row ('Fee',			htf.formText ('fee1', 10, 10, to_char(driver_rec.fee, common.currency_fmt$)));
	 common.show_data_row ('Late Fee',		htf.formText ('fee2', 10, 10, driver_rec.late_fee));
	 common.show_data_row ('Late Fee Cutoff Date',	htf.formText ('cutoff', 10, 10,
							to_char (driver_rec.cutoff_date, common.date_input_fmt_str$)) ||
								'Format: ' || common.date_input_fmt_str$);
      end if;

      if driver_rec.display_code = disp_popup$ then
	 common.show_data_row ('Popup Items',	htf.anchor ('javascript:popUp(''reg_form_maint.popup?meta_code=' ||
							driver_rec.meta_data_code || ''')', 'Edit' ));
      elsif driver_rec.display_code = disp_checkbox$ then
	 common.show_data_row ('Checkbox Items',	htf.anchor ('javascript:popUp(''reg_form_maint.popup?meta_code=' ||
							driver_rec.meta_data_code || ''')', 'Edit' ));
       elsif driver_rec.display_code = disp_mcb$ then
	common.show_data_row ('How many participants sponsorship available?', htf.formText ('maxsps', 3, 5, driver_rec.dropdown_max_items));
      end if;

      if driver_rec.display_code = disp_checkbox$ then
	 common.show_data_row ('Max Checkboxes Per Person', htf.formText ('maxchk', 3, 5, driver_rec.checkbox_max_items));
      end if;

      if driver_rec.display_code = disp_txt$ AND driver_rec.data_type_code = dataTstr$ then
	 common.show_data_row ('Max. Length', htf.formText ('mlen', 5, 10, driver_rec.data_length));
	 common.show_data_row ('View Length', htf.formText ('vlen', 5, 10, driver_rec.view_length));
	 common.show_data_row ('Fixed Length',common.popup_yes_no ('flen', driver_rec.fixed_length));
      end if;

      show_required_popup (driver_rec.meta_data_code, get_conditional_req (driver_rec.meta_data_code, driver_rec.required));
      common.show_lookup_table ('reg_cat',	'Group',	driver_rec.reg_cat_code);

      check_box ('Encrypt',	'crypt',	driver_rec.encrypt,		driver_rec.meta_data_code, 'Encrypt the users'' data within the database.  Recommended for credit card data.');
      check_box ('Freeze',	'freeze',	driver_rec.freeze_updates,	driver_rec.meta_data_code, 'Do not allow the user to change their registration once processing is complete.');
      check_box ('Single',	'single',	driver_rec.single_selection,	driver_rec.meta_data_code, 'Restrict selection to the first registrant to select an item.');

      if meta_data_in_use (driver_rec.meta_data_code) then
	 common.show_data_row ('Delete', 'In use on a registration page');

      elsif has_popup_list_items (driver_rec.meta_data_code) then
	 common.show_data_row ('Delete', 'Contains popoup/checkbox items');

      else
	 common.show_data_row ('Delete', htf.anchor ('reg_form_maint.del?meta_code=' || driver_rec.meta_data_code, 'Delete'));
      end if;

      htp.tableRowClose;
      htp.tableClose;
      htp.para;
      htp.formSubmit;
      htp.formClose;
      common.form_focus ('SPMS', 'promp');
      common.footer;
   end if;

end edit;

-- *******************************************************************************************

procedure fields (cat_code varchar2) is

cursor	driver is
select	*
from	meta_data_codes
where	reg_cat_code = cat_code
order by
	meta_data_descr;

begin

   if common.header (rpt_title$, js_src=>regist_maint.JS$) then
      link_backs (cat_code);

      htp.header (3, 'Field Group:' || common.space2$ ||
	common.decode_value ('reg_cat_descr', 'reg_cat_codes', 'reg_cat_code', cat_code) );

      htp.tableOpen ('BORDER');
      htp.tableRowOpen ('LEFT', 'BOTTOM');
      htp.tableHeader (common.space$);
      htp.tableHeader ('Prompt');
      htp.tableHeader ('Type');
      htp.tableHeader ('Display');
      htp.tableHeader ('Fee/Late');
      htp.tableHeader ('Required');
      htp.tableHeader ('Encrypt');
      htp.tableHeader ('Freeze');
      htp.tableHeader ('Single');
      htp.tableHeader ('Late Fee Cutoff');
      htp.tableHeader ('Delete');
      htp.tableRowClose;

      for driver_rec in driver LOOP
	 htp.tableRowOpen ('LEFT', 'TOP');
	 --htp.tableData (htf.anchor ('reg_form_maint.edit?meta_code=' || driver_rec.meta_data_code, 'Edit') );

	 htp.tableData (htf.anchor ('reg_form_maint.edit?meta_code=' || driver_rec.meta_data_code, htf.IMG(common.editImg$,cattributes=>'border=0 alt=Edit title=Edit')) );
	 htp.tableData (htf.escape_sc (driver_rec.meta_data_descr));
	 htp.tableData (common.decode_value ('data_type_descr', 'data_type_codes', 'data_type_code', driver_rec.data_type_code));
	 htp.tableData (common.decode_value ('display_descr', 'display_codes', 'display_code', driver_rec.display_code));

	 if driver_rec.data_type_code in (dataTfee$, dataTfee2$) then
	    htp.tableData (to_char(driver_rec.fee, common.currency_fmt$) || '/' || to_char(driver_rec.late_fee, common.currency_fmt$), 'CENTER');
	 else
	    htp.tableData (common.space$);
	 end if;

	 htp.tableData (common.yes_no (driver_rec.required), 'CENTER');
	 htp.tableData (common.yes_no (driver_rec.encrypt), 'CENTER');
	 htp.tableData (common.yes_no (driver_rec.freeze_updates), 'CENTER');
	 htp.tableData (common.yes_no (driver_rec.single_selection), 'CENTER');
	 htp.tableData (to_char (driver_rec.cutoff_date, common.date_input_fmt_str$), 'CENTER');

	 if meta_data_in_use (driver_rec.meta_data_code) then
	    htp.tableData (htf.IMG(common.usedImg$,cattributes=>'border=0 alt=Used title=Used'));
	 elsif has_popup_list_items (driver_rec.meta_data_code) then
	    htp.tableData ('Contains popoup/checkbox items');
	 else
	    htp.tableData (htf.anchor ('reg_form_maint.del?meta_code=' || driver_rec.meta_data_code, htf.IMG(common.deleteImg$,cattributes=>'border=0 alt=Delete title=Delete')));
	 end if;

	 htp.tableRowClose;
      end loop;

      htp.tableClose;
      htp.para;
      htp.formClose;
      common.footer;
   end if;

end fields;

-- ****************************************************************************************
-- *******************************************************************************************

procedure html is

cursor	driver is
select	*
from	reg_cat_codes
order by
	reg_cat_descr;

begin

   if common.header (rpt_title$, js_src=>regist_maint.JS$) then
      link_backs;

      htp.header (3, 'Select a Registration Field Group');
      htp.tableOpen;
      htp.tableRowOpen ('CENTER', 'TOP');

      for driver_rec in driver LOOP
	 htp.tableData (htf.anchor ('reg_form_maint.fields?cat_code=' ||
					driver_rec.reg_cat_code,
						driver_rec.reg_cat_descr) );
	 htp.tableData (common.space2$);
      end loop;

      htp.tableRowClose;
      htp.tableClose;
      common.footer;
   end if;

end html;

-- ****************************************************************************************

procedure list_fields(cat_code varchar2) is

cursor driver is
select md.*, dt.data_type_descr, rg.reg_cat_descr
from meta_data_codes  md
inner join data_type_codes  dt
on dt.data_type_code = md.data_type_code
join reg_cat_codes rg
on md.reg_cat_code = rg.reg_cat_code
where md.reg_cat_code like '%'|| cat_code ||'%'
order by meta_data_descr;

begin
    common.get_gbl_pid;
    if common.has_privs_to_page (common.reg_readonly_page$, false) or common.has_privs_to_page (common.reg_admin_page$, false) then
	    maint.header('Registration page field list');
	    regist_maint.jsFunctions;
	    htp.tableOpen();
	    htp.tableRowOpen();
	    htp.p('<td width=25%>');
	    maint.menu(0);
	    htp.p('</td>');
	    htp.p('<td>');
	    htp.tableOpen(cattributes=>'border=1px');
	    htp.tableRowOpen();
	    htp.p('<td colspan="4" align="left">');
	    show_lookup_groups(cat_code);
	    htp.p('</td>');
	    htp.tableRowClose();
	    htp.tableRowOpen();
	    htp.tableData(htf.anchor ('reg_form_maint.new?cat_code=&page_code=list_fields',htf.IMG(common.newImg$,cattributes=>'border=0 alt=New title=New')) || '&nbsp;' || htf.anchor ('maint.tab?tab_name=reg_cat_codes&order_by=reg_cat_descr&action=wizard',htf.IMG(common.prevImg$,cattributes=>'border=0 alt=Previous title=Previous')) || htf.anchor ('reg_form_maint.list_pages?action=wizard',htf.IMG(common.nextImg$,cattributes=>'border=0 alt=Next title=Next')) || '&nbsp;' || htf.anchor('jacow.html',htf.IMG(common.cancelImg$,cattributes=>'border=0 alt="Cancel" title="Cancel"'), cattributes=>'onclick=''return confirmation()'''));
	    htp.tableHeader('Description');
	    htp.tableHeader('Data type');
	    htp.tableHeader('Group');
	    htp.tableRowClose();

	    for driver_rec in driver loop
		htp.tableRowOpen();
		htp.tableData(htf.anchor ('reg_form_maint.edit?meta_code=' || driver_rec.meta_data_code, htf.IMG(common.editImg$,cattributes=>'border=0 alt=Edit title=Edit')));
		htp.tableData(driver_rec.meta_data_descr);
		htp.tableData(driver_rec.data_type_descr);
		htp.tableData(driver_rec.reg_cat_descr);
		htp.tableRowClose();
	    end loop;
	    htp.tableRowOpen();
	    htp.tableData(htf.anchor ('reg_form_maint.new?cat_code=&page_code=list_fields',htf.IMG(common.newImg$,cattributes=>'border=0 alt=New title=New')) || '&nbsp;' || htf.anchor ('maint.tab?tab_name=reg_cat_codes&order_by=reg_cat_descr&action=wizard',htf.IMG(common.prevImg$,cattributes=>'border=0 alt=Previous title=Previous')) || htf.anchor ('reg_form_maint.list_pages?action=wizard',htf.IMG(common.nextImg$,cattributes=>'border=0 alt=Next title=Next')) || '&nbsp;' || htf.anchor('jacow.html',htf.IMG(common.cancelImg$,cattributes=>'border=0 alt="Cancel" title="Cancel"'), cattributes=>'onclick=''return confirmation()'''), cattributes=>'colspan="4"');
	   -- htp.tableData(common.space$,cattributes=>'colspan = 2');
	    htp.tableRowClose();
	    htp.tableClose();
	    htp.p('</td>');
	    htp.tableRowClose();
	    htp.tableClose();
	    common.footer;
    else
	 htp.header (3, not_authorized$);
    end if;

end list_fields;

-- ***************************************************************************************
-- ***************************************************************************************

procedure registration_opening_text is
begin
   common.get_gbl_pid;
   if common.has_privs_to_page (common.reg_readonly_page$, false) or common.has_privs_to_page (common.reg_admin_page$, false) then
		maint.header('Registration page field list');
		regist_maint.jsFunctions;
		htp.tableOpen();
		htp.tableRowOpen();
		htp.p('<td width=25%>');
		maint.menu(0);
		htp.p('</td>');
		htp.p('<td>');
		htp.tableOpen();
		htp.tableRowOpen();
		htp.p('<td align=justify>');
		htp.p('Welcome to the delegate registration form wizard. On the next page you will
				find a list of already defined groups you can use; if you want to create your
				personal registration field group just click on the add icon');
		htp.p('</td>');
		htp.tableRowClose();
		htp.tableRowOpen();
		htp.p('<td align=left>');
		htp.p(htf.anchor ('maint.tab?tab_name=reg_cat_codes&order_by=reg_cat_descr&action=wizard',htf.IMG(common.nextImg$,cattributes=>'border=0 alt=Next title=Next')));
		htp.p('&nbsp;');
		htp.p(htf.anchor('jacow.html',htf.IMG(common.cancelImg$,cattributes=>'border=0 alt="Cancel" title="Cancel"'), cattributes=>'onclick=''return confirmation()'''));
		htp.p('</td>');
		htp.tableRowClose();
		htp.tableClose();
		htp.p('</td>');
		htp.tableRowClose();
		htp.tableClose();
		common.footer;
   else
	 	htp.header (3, not_authorized$);
   end if;
end registration_opening_text;
-- ***************************************************************************************
-- ***************************************************************************************

procedure show_lookup_groups(cat_code varchar2) is

cursor driver is
select * from reg_cat_codes
order by reg_cat_descr;

selected varchar2(100) :='';

begin
 htp.tableOpen(cattributes=>'class="filter"');
 htp.formOpen(curl=>'reg_form_maint.list_fields',cmethod=>'post');
 htp.tableRowOpen();
 htp.tableData('Filter by group:');
 htp.p('<td align="left">');
 htp.formSelectOpen(cname=>'cat_code');
 htp.formSelectOption ('', cattributes=>'VALUE=""');
 htp.p('</option>');
 for driver_rec in driver loop
    if(cat_code is not null and driver_rec.reg_cat_code = cat_code) then
      htp.formSelectOption(driver_rec.reg_cat_descr, cattributes=>'VALUE="' || driver_rec.reg_cat_code || '" selected');
    else
      htp.formSelectOption(driver_rec.reg_cat_descr, cattributes=>'VALUE="' || driver_rec.reg_cat_code || '" ');
    end if;
    htp.p('</option>');
 end loop;
 htp.formSelectClose();
 htp.p('</td>');
 htp.tableData(htf.formSubmit(),cattributes=>'align="left"');
 htp.tableRowClose();
 htp.formClose();
 htp.tableClose();
end show_lookup_groups;

-- ***************************************************************************************
-- ***************************************************************************************

procedure add_available_payments(pid number,action varchar2) is

cursor code is
select * from payment_codes;

cursor added is
select * from payment_codes_page
where page_id = pid;

checked$ varchar2(1) :=null;
begin

  if common.header ('Add payment methods') then
      htp.p (common.space2$);
    if(action = 'wizard') then
	htp.anchor('reg_form_maint.list_pages?action='|| action,htf.IMG(common.prevImg$,cattributes=>'border=0 alt="Back" title="Back"'));
	htp.p('&nbsp;');
	htp.p(htf.anchor('jacow.html',htf.IMG(common.cancelImg$,cattributes=>'border=0 alt="Cancel" title="Cancel"'), cattributes=>'onclick=''return confirmation()'''));
    else
       htp.anchor('#',htf.IMG(common.prevImg$,cattributes=>'border=0 alt="Back" title="Back"'), cattributes=>'onclick="history.back(-1);return false"');
    end if;
    htp.tableOpen();
    htp.tableRowOpen();
    htp.tableData('&nbsp;', cattributes=>'width="25%"');
    htp.p('<td>');
     htp.formOpen(curl=>'reg_form_maint.add_selected_payments', cmethod=>'post');
     htp.formHidden(cname=>'action',cvalue=>action);
     htp.formHidden(cname=>'pid',cvalue=>pid);
       htp.tableOpen(cattributes=>'border="1px"');
       htp.tableRowOpen();
       htp.tableHeader('Payment method');
       htp.tableHeader('Available online');
       htp.tableHeader('Available to conf.');
       htp.tableHeader();
       htp.tableRowClose();
       for code_rec in code loop
	 htp.tableRowOpen();
	 htp.tableData(code_rec.payment_descr);
	 for added_rec in added loop
	    if(code_rec.payment_code = added_rec.payment_code) then
		checked$ :=1;
	    end if;
	 end loop;
	 htp.tableData(code_rec.online_option);
	 htp.tableData(code_rec.conf_option);
	 htp.tableData(htf.formCheckbox('selected',code_rec.payment_code,checked$));
      	 checked$:=null;
	 htp.tableRowClose();
       end loop;
       htp.tableRowOpen();
       htp.tableData(htf.formSubmit(cvalue=>'Add'),cattributes=>'align="left" colspan="4"');
       htp.tableRowClose();
       htp.tableClose();
     htp.formClose();
    htp.p('</td>');
    htp.tableData('&nbsp;', cattributes=>'width="25%"');
    htp.tableRowClose();
    htp.tableClose();
  end if;
end add_available_payments;

procedure add_selected_payments(selected owa_util.ident_arr default empty_arr, action varchar2, pid number) is
begin
  common.get_gbl_pid;
  if common.has_privs_to_page (common.reg_readonly_page$, false) or common.has_privs_to_page (common.reg_admin_page$, false) then
    --if(selected is not empty) then
 if(selected is not null ) then
     delete from payment_codes_page where page_id = pid;
     for i in 1..selected.count loop
	insert into payment_codes_page (page_id,payment_code) values (pid,selected(i));
     end loop;
      common.redirect_URL (common.sserver || common.vdir$ || 'reg_form_maint.list_pages?action=' || action);
    else
      common.redirect_URL (common.sserver || common.vdir$ || 'reg_form_maint.add_available_payments?pid='|| pid ||'&action=' || action);
    end if;
  end if;
end add_selected_payments;
-- ***************************************************************************************

procedure show_selected_payments(pid number) is

cursor p is
select payment_code from
payment_codes_page where
page_id = pid;

cursor pc (pcode varchar2) is
select payment_descr from
payment_codes where
payment_code = pcode;

begin

  for p_rec in p loop
    for pc_rec in pc (p_rec.payment_code) loop
      htp.p(pc_rec.payment_descr || '<br />');
    end loop;
  end loop;
end show_selected_payments;


/*

Procedure to test the new meta data code creation - javascript

*/
procedure test is


	cursor data_type is
		Select *
		from data_type_codes;

	cursor display_type(dt varchar2) is
		select dc.*
		from type_display_data tdd
		     inner join
		     display_codes dc on tdd.DISPLAY_CODE=dc.DISPLAY_CODE
		where tdd.DATA_TYPE_CODE=dt;

begin
	htp.htmlopen;
	HTP.HEADOPEN;
	htp.p('
		<script type="text/javascript">
			function show()
			{
				var type=document.getElementById("dt").selectedIndex;
				var val=document.getElementById("dt").options[type].value;
				var name="ds_" + val ;
				var sel_opt=document.getElementsByTagName("DIV");
				var length=sel_opt.length;

				for (i=0 ; i<=length; i++)
				{
					if (sel_opt[i].id==name)
					{
						sel_opt[i].style.display="block";
					}else
					{
						sel_opt[i].style.display="none";
					}
				}

			}

		</script>
		');
	HTP.HEADCLOSE;
	HTP.BODYOPEN;
	htp.formselectopen('dt','<b>Data Type </b>',cattributes=>'ID="dt" onchange="show()"');
	htp.formselectoption(common.space$,'SELECTED',cattributes=>'value="dummy"');
	for driver in data_type loop
		htp.formselectoption(driver.data_type_descr,
				     cattributes=>'value="'||driver.data_type_code||'"');
	end loop;
	htp.formselectclose;
	htp.br;
	for driver in data_type loop
		htp.p('<div ID="ds_'||driver.data_type_code||'">');
		htp.formselectopen('ds_'||driver.data_type_code,'<b>Display Type ' ||
				    driver.data_type_code || '</b>');
		htp.formselectoption(common.space$,'SELECTED',cattributes=>'value="dummy"');
		for driver1 in display_type(driver.data_type_code) loop
			htp.formselectoption(driver1.display_descr,
					     cattributes=>'value="'||driver1.display_code||'"');

		end loop;
		htp.formselectclose;
		htp.p('</div>');
	end loop;
	HTP.BODYCLOSE;
	htp.htmlclose;

end test;

end reg_form_maint;

/
show error

set define on

--	Copyright (C) 2004-2008 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>
