set define off

Create or Replace package body reg_rpt as

type num_arr is table of number;

rpt_title$	constant char(23)	:= 'Registration Statistics';

disp_txt$	constant display_codes.display_code%TYPE := 'TXT';
disp_YN$	constant display_codes.display_code%TYPE := 'YN';
disp_popup$	constant display_codes.display_code%TYPE := 'POP';
disp_multi$	constant display_codes.display_code%TYPE := 'MULT';
disp_checkbox$	constant display_codes.display_code%TYPE := 'CHK';
disp_mcb$	constant display_codes.display_code%TYPE := 'MCB';

dataTnum$	constant data_type_codes.data_type_code%TYPE := 'NUM';
dataTstr$	constant data_type_codes.data_type_code%TYPE := 'STR';
dataTdte$	constant data_type_codes.data_type_code%TYPE := 'DTE';
dataTfee$	constant data_type_codes.data_type_code%TYPE := 'FEE';
dataTfee2$	constant data_type_codes.data_type_code%TYPE := 'FEE2';
dataTsps$	constant data_type_codes.data_type_code%TYPE := 'SPS';

regist_ID_meta$	constant char(4) := 'meta';
regist_ID_popl$	constant char(4) := 'popl';
regist_ID_mtrx$	constant char(4) := 'mtrx';
regist_ID_paym$	constant char(4) := 'paym';
regist_ID_ccpm$	constant char(4) := 'ccpm';
regist_ID_chkb$	constant char(4) := 'chkb';

not_authorized$	constant varchar2(50) := 'You are not authorized to access this page.';

type		vtype is varray(3) of integer;
type		rec_type is record (
			dummy	integer,
			icounts	vtype := vtype (0,0,0));

type 		arr_type is	table of rec_type
				index by presentation_codes.presentation_code%TYPE;
reg_by_abs	arr_type;

type	lblType is varray(3) of varchar2(30);
lables	lblType := lblType ('Abstracts', 'Speaker' || htf.br || 'Registered',
				'Anyone' || htf.br || 'Registered');

abs$		constant integer := 1;
spk_reg$	constant integer := 2;
any_reg$	constant integer := 3;

total$		constant varchar2(5) := '~~';


-- ******************************************************************************
-- ******************************************************************************

function abstract_registered (aid integer) return varchar2 is

cursor	driver is
select	person_id
from	abstract_authors
where	abstract_id = aid;

begin

   for driver_rec in driver LOOP
      if regist_maint.is_registered (driver_rec.person_id) = 'Y' then
	 return 'Y';
      end if;
   end loop;

   return 'N';

end abstract_registered;


-- ******************************************************************************
-- ******************************************************************************

procedure show_registrant_counts is

	cursor	driver is
		select	page_id, page_name
		from	pages
		where	'Y' = nvl (include_statistics, 'Y')
		order by page_name;

	cursor	C2 (pg_id varchar2) is
		select	person_id, regist_maint.balancedue (common.ENCRYPT(person_id),pg_id) paid
		from	registrants
		where	page_id	= pg_id
		order by person_id;

	total		integer := 0;
	paid		integer := 0;
	pcount		integer;

	type		rec_type is table of C2%ROWTYPE index by binary_integer;
	recs		rec_type;

-- Dummy vars to count the distinct registrants and if they are paid in full
	distinct_regs	common.int_arr;
	distinct_paid	common.int_arr;

begin

   htp.header (3, 'Counts by Registration Page');
   htp.tableOpen ('BORDER');
   htp.tableRowOpen ('LEFT', 'BOTTOM');
   htp.tableHeader ('Page Name');
   htp.tableHeader ('Registrants');
   htp.tableHeader (common.space$ || 'Paid' || common.space$);
   htp.tableRowClose;

   for driver_rec in driver LOOP
      open C2 (driver_rec.page_id);
      fetch C2 BULK COLLECT into recs;
      close C2;

      htp.tableRowOpen ('CENTER', 'TOP');
      htp.tableData (driver_rec.page_name, 'LEFT');

      pcount := 0;
      for i in 1..recs.COUNT LOOP
		  distinct_regs(recs(i).person_id) := 0;
		  if recs(i).paid <= 0 then
		    pcount := pcount + 1;
		    distinct_paid(recs(i).person_id) := 0;
		  end if;
      end loop;

      htp.tableData (recs.COUNT);
      htp.tableData (pcount);
      htp.tableRowClose;

      total	:= total + recs.COUNT;
      paid	:= paid  + pcount;
   end loop;

   htp.tableRowOpen ('CENTER', 'TOP');
   htp.tableHeader ('Totals', 'LEFT');
   htp.tableHeader (total);
   htp.tableHeader (paid);
   htp.tableRowClose;

   htp.tableRowOpen ('CENTER', 'TOP');
   htp.tableHeader ('Distinct Registrants', 'LEFT');
   htp.tableHeader (distinct_regs.COUNT);
   htp.tableHeader (distinct_paid.COUNT);
   htp.tableRowClose;

   htp.tableClose;
   htp.para;

end show_registrant_counts;

-- ******************************************************************************
-- ******************************************************************************

procedure show_payment_counts is

cursor	received	is
	select pc.PAYMENT_CODE, pc.PAYMENT_DESCR descr, Sum(p.AMOUNT) total,
		   sum(p.AMOUNT*nvl(pc.PERCENTAGE_COST,0)/100+nvl(pc.OPERATION_COST,0)) COST
	from
		payments p
		inner join payment_codes pc on p.PAYMENT_CODE=pc.PAYMENT_CODE
	where pc.WAIVED='N'
	group by pc.PAYMENT_CODE, pc.PAYMENT_DESCR;

cursor	waived	is
	select pc.PAYMENT_CODE, pc.PAYMENT_DESCR descr, Sum(p.AMOUNT) total,
		   sum(p.AMOUNT*nvl(pc.PERCENTAGE_COST,0)/100+nvl(pc.OPERATION_COST,0)) COST
	from
		payments p
		inner join payment_codes pc on p.PAYMENT_CODE=pc.PAYMENT_CODE
	where pc.WAIVED='Y'
	group by pc.PAYMENT_CODE, pc.PAYMENT_DESCR;



	total	number;
	totalCost number :=0;

begin

		htp.header (3, 'Payment Counts');
		htp.tableOpen ('BORDER');
		htp.tableHeader ('Received');
		htp.tableHeader ('Amount', 'RIGHT');
		htp.tableheader ('Costs');
		total := 0;
		totalCost := 0;
		for received_rec in received LOOP
			htp.tableRowOpen ;
			htp.tableData (received_rec.descr);
			htp.tableData (to_char (received_rec.total, common.currency_fmt$), 'RIGHT');
			htp.tableData (to_char (received_rec.cost, common.currency_fmt$), 'RIGHT');
			htp.tableRowClose;
			total := total + received_rec.total;
			totalCost := totalCost + received_rec.cost;
		end loop;
		htp.tableRowOpen ('LEFT', 'TOP');
		htp.tableHeader ('Total');
		htp.tableHeader (to_char (total, common.currency_fmt$), 'RIGHT');
		htp.tableHeader (to_char (totalCost, common.currency_fmt$), 'RIGHT');
		htp.tableclose;
		htp.p('<br /><br />');
		htp.tableOpen ('BORDER');
		htp.tableHeader ('Waived');
		htp.tableHeader ('Amount', 'RIGHT');
		htp.tableheader ('Costs');
		total := 0;
		totalCost := 0;
		for waived_rec in waived LOOP
			htp.tableRowOpen ;
			htp.tableData (waived_rec.descr);
			htp.tableData (to_char (waived_rec.total, common.currency_fmt$), 'RIGHT');
			htp.tableData (to_char (waived_rec.cost, common.currency_fmt$), 'RIGHT');
			htp.tableRowClose;
			total := total + waived_rec.total;
			totalCost := totalCost + waived_rec.cost;
		end loop;
		htp.tableRowOpen ('LEFT', 'TOP');
		htp.tableHeader ('Total');
		htp.tableHeader (to_char (total, common.currency_fmt$), 'RIGHT');
		htp.tableHeader (to_char (totalCost, common.currency_fmt$), 'RIGHT');
		htp.tableclose;


end show_payment_counts;

-- ******************************************************************************
-- ******************************************************************************

procedure init_reg_by_abs is

cursor	driver is
select	presentation_code code
from	presentation_codes;

begin

   for driver_rec in driver LOOP
      reg_by_abs(driver_rec.code).dummy := 0;
   end loop;
   reg_by_abs(total$).dummy := 0;

end init_reg_by_abs;

-- ******************************************************************************

procedure calc_registrant_by_abstract is

cursor	driver is
select	abstract_id, presentation_code,
	    common.get_author_profile_id (abstract_id, common.speaker$) spk_profile_id
from	abstracts;

code	abstracts.presentation_code%TYPE;
pid	integer;

begin

   for driver_rec in driver LOOP
      code	:= driver_rec.presentation_code;
      pid	:= common.decode_value ('person_id', 'profiles',
			'profile_id', driver_rec.spk_profile_id);
      reg_by_abs(code).icounts(abs$) := reg_by_abs(code).icounts(abs$) + 1;

      if regist_maint.is_registered (pid) = 'Y' then
	 reg_by_abs(code).icounts(spk_reg$) := reg_by_abs(code).icounts(spk_reg$) + 1;
	 reg_by_abs(code).icounts(any_reg$) := reg_by_abs(code).icounts(any_reg$) + 1;
      elsif reg_rpt.abstract_registered (driver_rec.abstract_id) = 'Y' then
	 reg_by_abs(code).icounts(any_reg$) := reg_by_abs(code).icounts(any_reg$) + 1;
      end if;
   end loop;

-- get the totals

   code := reg_by_abs.FIRST;
   for i in 1..reg_by_abs.COUNT-1 LOOP
      for j in 1..reg_by_abs(code).icounts.COUNT LOOP
	 reg_by_abs(total$).icounts(j) := reg_by_abs(total$).icounts(j) +
					  reg_by_abs(code).icounts(j);
      end loop;
      code := reg_by_abs.NEXT(code);
   end loop;

end calc_registrant_by_abstract;

-- ******************************************************************************

procedure show_registrant_by_abstract is

cursor	driver is
	select	presentation_code code, presentation_descr descr
	from	presentation_codes
	order by sort_order;

begin

   htp.header (3, 'Registrants by Abstract');
   htp.tableOpen ('BORDER');

   htp.tableRowOpen ('CENTER', 'BOTTOM');
   htp.tableData (common.space$);

   for i in 1..lables.COUNT LOOP
      htp.tableHeader (lables(i));
   end loop;
   htp.tableRowClose;

-- *****

   for driver_rec in driver LOOP
      htp.tableRowOpen ('RIGHT', 'BOTTOM');
      htp.tableHeader (driver_rec.descr, 'LEFT');

      for j in 1..reg_by_abs(driver_rec.code).icounts.COUNT LOOP
	 	htp.tableData (reg_by_abs(driver_rec.code).icounts(j), 'CENTER');
      end loop;

      htp.tableRowClose;
   end loop;

   htp.tableRowOpen ('RIGHT', 'BOTTOM');
   htp.tableHeader ('Total', 'LEFT');

   for j in 1..reg_by_abs(total$).icounts.COUNT LOOP
      htp.tableHeader (reg_by_abs(total$).icounts(j), 'CENTER');
   end loop;

   htp.tableRowClose;

   htp.tableClose;

end show_registrant_by_abstract;

-- ******************************************************************************
-- ******************************************************************************

procedure summary is

begin

   if common.header (rpt_title$) then
      init_reg_by_abs;
      calc_registrant_by_abstract;

      htp.tableOpen (cattributes=>'WIDTH="95%"');

      htp.tableRowOpen ('LEFT', 'TOP');
      htp.p ('<TD ROWSPAN="2">');
	 show_registrant_counts;
	 show_registrant_by_abstract;
      htp.p ('</TD>');

      htp.p ('<TD>');
	 show_payment_counts;
      htp.p ('</TD>');

      htp.tableRowClose;
      htp.tableClose;
      common.footer;
   end if;

end summary;

-- ******************************************************************************
-- ******************************************************************************

procedure budget is

	cursor driver is
		Select q1.PAGE_ID, q1.META_DATA_CODE, q1.poplist_item_id, Sum(regist_maint.get_fee_amt(q1.REGISTRATION_ID,sysdate)) amount
		from
			pages p
			left outer join
				(Select q.REGISTRATION_ID, q.PERSON_ID, q.PAGE_ID, q.META_DATA_CODE, q.poplist_item_id
				 from
					registrants rg
					inner join
					(Select r.REGISTRATION_ID, r.PERSON_ID, r.PAGE_ID, r.META_DATA_CODE, r.poplist_item_id
					 from registrations r
						 inner join meta_data_codes mdc on r.META_DATA_CODE=mdc.META_DATA_CODE
					 where mdc.DATA_TYPE_CODE in ('FEE', 'FEE2')) q on q.person_id=rg.person_id and q.page_id=rg.page_id
				 where rg.cancel is null) q1 on p.page_id=q1.page_id
		where p.include_statistics='Y'
		group by q1.page_id, q1.meta_data_code, q1.poplist_item_id
		order by q1.page_id, q1.meta_data_code, q1.poplist_item_id;


	page_id	pages.PAGE_ID%TYPE default -1;
	mdc 	meta_data_codes.META_DATA_CODE%type default -1;
	poplist poplist_items.POPLIST_ITEM_ID%type default -1;

	total		number := 0;
	glb_total	number := 0;
	received	number := 0;
	waived		number := 0;


	descr 		meta_data_codes.META_DATA_DESCR%type;


begin

	common.get_gbl_pid;
	if common.has_privs_to_page (common.reg_readonly_page$, false) or common.has_privs_to_page (common.reg_admin_page$, false) then
		regist_maint.JSFUNCTIONS;
		maint.header ('Budget');
		htp.tableOpen();
		htp.tableRowOpen();
		htp.p('<td width=40%>');
		maint.menu (0);
		htp.p('</td>');
		htp.p('<td>');
		htp.tableOpen ('BORDER', cattributes=>'align = left class=budget');
		for driver_rec in driver loop
			if page_id<>driver_rec.page_id then
				if page_id<>-1 then
					htp.tablerowopen;
					htp.tableData ('Total Fees', cnowrap=>'NOWRAP', cattributes=>'class=boldtext');
					htp.tableData (htf.underline (common.currency_symbol$ || to_char (total, common.currency_fmt$))
									,cattributes=>'class=boldtext');
					htp.tablerowclose;
				end if;
				received:=received+regist_maint.get_fee_received (driver_rec.page_id,null);
				waived:=waived+regist_maint.get_fee_waived (driver_rec.page_id,null);
				htp.tableHeader(common.DECODE_VALUE('page_name','pages','page_id',driver_rec.page_id),cattributes=>'colspan="2"');
				page_id:=nvl(driver_rec.page_id,-1);
				glb_total:=glb_total+total;
				total:=0;
			end if;
			htp.tablerowopen;
			if driver_rec.poplist_item_id is null then
				descr:=common.DECODE_VALUE('meta_data_descr','meta_data_codes','meta_data_code',driver_rec.meta_data_code);
			else
				descr:=common.DECODE_VALUE('item_value','poplist_items','poplist_item_id',driver_rec.poplist_item_id);
			end if;
			htp.tableData (descr, cnowrap=>'NOWRAP', cattributes=>'class=boldtext');
			htp.tableData (to_char (driver_rec.amount, common.currency_fmt$));
			htp.tablerowclose;
			total:=total+driver_rec.amount;
		end loop;
		if page_id<>-1 then
			htp.tablerowopen;
			htp.tableData ('Total Fees', cnowrap=>'NOWRAP', cattributes=>'class=boldtext');
			htp.tableData (htf.underline (common.currency_symbol$ || to_char (total, common.currency_fmt$))
							,cattributes=>'class=boldtext');
			htp.tablerowclose;
			glb_total:=glb_total+total;
		end if;
		htp.tablerowopen;
		htp.tableData (common.space$, cattributes=>'colspan="2"');
		htp.tablerowclose;
		htp.tablerowopen;
		htp.tablerowopen;
		htp.tableData ('Gran Total Fees', cnowrap=>'NOWRAP', cattributes=>'class=boldtext');
		htp.tableData (htf.underline (common.currency_symbol$ || to_char (glb_total, common.currency_fmt$))
						,cattributes=>'class=boldtext');
		htp.tablerowclose;
		htp.tablerowopen;
		htp.tableData ('Fees Collected', cnowrap=>'NOWRAP', cattributes=>'class=boldtext');
		htp.tableData (htf.underline (common.currency_symbol$ || to_char (received, common.currency_fmt$))
						,cattributes=>'class=boldtext');
		htp.tablerowclose;
		htp.tablerowopen;
		htp.tableData ('Fees Waived', cnowrap=>'NOWRAP', cattributes=>'class=boldtext');
		htp.tableData (htf.underline (common.currency_symbol$ || to_char (waived, common.currency_fmt$))
						,cattributes=>'class=boldtext');
		htp.tablerowclose;
		htp.tablerowopen;
		htp.tableData ('Balance Due', cnowrap=>'NOWRAP', cattributes=>'class=boldtext');
		htp.tableData (htf.underline (common.currency_symbol$ || to_char (glb_total-received-waived, common.currency_fmt$))
						,cattributes=>'class=boldtext');
		htp.tablerowclose;
		htp.tableClose;
		htp.p('</td>');
		htp.tableRowClose();
		htp.tableClose();
		common.footer;
	end if;
end budget;

-- *******************************************************************************************
-- *******************************************************************************************

procedure counts_per_regpage(pagids NUM_ARR_TYPE) is

pageids	num_arr;

cursor	driver (pageIDs IN NUM_ARR_TYPE) is
select	src.super_region_descr, rc.region_descr, cc.country_descr, count(DISTINCT r.person_id) num
from	super_region_codes	src,
	region_codes		rc,
	country_codes		cc,
	addresses		addr,
	registrants		r
where	r.cancel		is null
and	r.page_id		in (select column_value from table(cast(pageIDs as NUM_ARR_TYPE)))
and	addr.address_id		= COMMON.GET_USER_PRIMARY_ADDR_ID(r.person_id)
and	cc.country_code		= addr.country_code
and	rc.region_code		= cc.region_code
and	src.super_region_code	= rc.super_region_code
group by src.super_region_descr, rc.region_descr, cc.country_descr;

type	country_type	is table of integer		index by country_codes.country_descr%TYPE;
type	region_type	is record (
					country	country_type,
					total	integer default 0);
type	regions_type	is table of region_type		index by region_codes.region_descr%TYPE;
type	sregion_type	is record (
					region	regions_type,
					total	integer default 0);
type	sregions_type	is table of sregion_type	index by super_region_codes.super_region_descr%TYPE;

counts	sregions_type;

type	rec_type is table of driver%ROWTYPE index by pls_integer;
recs	rec_type;

sregion	super_region_codes.super_region_descr%TYPE;
region	region_codes.region_descr%TYPE;
country	country_codes.country_descr%TYPE;

total	integer;
gtotal	integer := 0;

begin

   open driver(pagids);
   fetch driver BULK COLLECT into recs;
   close driver;

   for i in 1..recs.COUNT loop
      sregion	:= recs(i).super_region_descr;
      region	:= recs(i).region_descr;
      country	:= recs(i).country_descr;

      counts(sregion).region(region).country(country) := recs(i).num;
   end loop;

   htp.tableOpen;
   sregion := counts.FIRST;
   loop
      exit when sregion is null;

      region := counts(sregion).region.FIRST;
      loop
         exit when region is null;

         country := counts(sregion).region(region).country.FIRST;
         loop
            exit when country is null;
            total := counts(sregion).region(region).country(country);

            htp.tableRowOpen;
            htp.tableData(country);
            htp.tableData(common.space$);
            htp.tableData(total, 'RIGHT');
            htp.tableRowClose;

            counts(sregion).region(region).total	:= total + counts(sregion).region(region).total;
            counts(sregion).total			:= total + counts(sregion).total;

            country := counts(sregion).region(region).country.NEXT(country);
         end loop;

         common.show_data_sep (3);

         htp.tableRowOpen;
         htp.tableHeader(region, 'RIGHT');
         htp.tableData(common.space$);
         htp.tableHeader(counts(sregion).region(region).total, 'RIGHT');
         htp.tableRowClose;

         region := counts(sregion).region.NEXT(region);
      end loop;

      common.show_data_sep (3);

      htp.tableRowOpen;
      htp.tableHeader(htf.italic(sregion), 'RIGHT');
      htp.tableData(common.space$);
      htp.tableHeader(counts(sregion).total, 'RIGHT');
      htp.tableRowClose;

      gtotal := gtotal + counts(sregion).total;
      common.show_data_sep (3, common.space$);

      sregion := counts.NEXT(sregion);
   end loop;

   htp.tableRowOpen;
   htp.tableHeader('Grand Total', 'RIGHT');
   htp.tableData(common.space$);
   htp.tableHeader(gtotal, 'RIGHT');
   htp.tableRowClose;

   htp.tableClose;

end counts_per_regpage;

-- *******************************************************************************************
-- *******************************************************************************************

procedure counts is

cursor	page_driver is
select	page_id, page_name from pages
where	include_statistics ='Y'
order by page_name;

region		region_codes.REGION_DESCR%TYPE default 'dummy';
page_ids_sng	NUM_ARR_TYPE := NUM_ARR_TYPE(1);
page_ids_all	NUM_ARR_TYPE := NUM_ARR_TYPE();
i		integer	:= 1;
total		integer	:= 0;
gtotal		integer	:= 0;

begin

   if common.header (rpt_title$) then

  -- single registration pages first
      for driver_rec in page_driver LOOP
         htp.hr();
         htp.hr();
         htp.header(2,driver_rec.page_name);
         page_ids_sng(1) := driver_rec.page_id;
         counts_per_regpage(page_ids_sng);

    -- add this page to the collection of all pages for global stats
         page_ids_all.EXTEND(1);
         page_ids_all(i) := driver_rec.page_id;
         i := i+1;
      end loop;

  -- global stats, now

      htp.hr();
      htp.hr();

      htp.header(2,'Overall General Statistics');
      htp.header(3,'(Unique registrants in all registration pages with stats enabled)');

      counts_per_regpage(page_ids_all);
      common.footer;
   end if;

end counts;

-- *******************************************************************************************

procedure build_poplist is

cursor	driver is
select	p.poplist_item_id, m.meta_data_code,
	m.short_descr || ': ' || p.item_value NAME
from	meta_data_codes		m,
	poplist_items		p
where	p.poplist_item_id	in
(
	select	distinct poplist_item_id
	from	registrations
)
and	m.meta_data_code	= p.meta_data_code
order by
	m.short_descr, p.sort_order nulls last, p.item_value;

begin

   htp.formSelectOpen ('this_poplist_id', cattributes=>'MULTIPLE');
   htp.formSelectOption ('', 'SELECTED');
   for driver_rec in driver LOOP
	 htp.formSelectOption (driver_rec.name, cattributes=>'VALUE="' ||
				driver_rec.poplist_item_id || '"');
 --	end if;
   end loop;
   htp.formSelectClose;

end build_poplist;

procedure build_fee_matrix is

cursor	driver is
select	f.fee_matrix_id,
	m1.short_descr || ': ' || p1.item_value || ' / ' ||
	m2.short_descr || ': ' || p2.item_value NAME
from	meta_data_codes		m1,
	poplist_items		p1,
	meta_data_codes		m2,
	poplist_items		p2,
	fee_matrix		f
where	f.fee_matrix_id		in
(
	select	distinct fee_matrix_id
	from	registrations
)
and	p1.poplist_item_id	= f.poplist_item_id1
and	m1.meta_data_code	= p1.meta_data_code
and	p2.poplist_item_id	= f.poplist_item_id2
and	m2.meta_data_code	= p2.meta_data_code
order by
	m1.short_descr, p1.sort_order nulls last, p1.item_value,
	m2.short_descr, p2.sort_order nulls last, p2.item_value;

begin

   htp.formSelectOpen ('this_fee_matrix_id', cattributes=>'MULTIPLE');
   htp.formSelectOption ('', 'SELECTED');
   for driver_rec in driver LOOP
      htp.formSelectOption (driver_rec.name, cattributes=>'VALUE="' ||
				driver_rec.fee_matrix_id || '"');
   end loop;
   htp.formSelectClose;

end build_fee_matrix;

procedure stats2 is

begin

   if common.header (rpt_title$) then
      htp.formOpen ('reg_rpt.stats2_rpt', 'POST');
      htp.tableOpen;
      htp.tableRowOpen ('LEFT', 'TOP');

      htp.p ('<TD>');
	 htp.bold ('Pages');
	 htp.br;
	 common.show_popup_list ('page', multiple=>true);
      htp.p ('</TD>');

      htp.p ('<TD>');
	 htp.bold ('Prompts');
	 htp.br;
	 common.show_popup_list ('meta_data', multiple=>true,
				where_claws=>'where meta_data_code in ' ||
				'(select distinct meta_data_code from registrations)');
      htp.p ('</TD>');

      htp.p ('<TD>');
	 htp.bold ('Prompt Popup Items');
	 htp.br;
	 build_poplist;
      htp.p ('</TD>');

      htp.tableRowClose;
      htp.tableRowOpen ('LEFT', 'TOP');

      htp.p ('<TD COLSPAN="3">');
	 htp.bold ('Matrix Items');
	 htp.br;
	 build_fee_matrix;
      htp.p ('</TD>');

      htp.tableRowClose;
      htp.tableClose;

      htp.para;
      htp.formSubmit;
      htp.p (common.space2$);
      htp.formReset;
      htp.formClose;
      common.footer;
   end if;

end stats2;

-- *******************************************************************************************

function stats2_pages (this_page integer) return integer is

cursor	driver is
select	count(1)
from	registrants
where	page_id = this_page;

icount	integer;

begin

   open driver;
   fetch driver into icount;
   close driver;

   htp.tableRowOpen ('RIGHT', 'TOP');
   htp.tableHeader (common.decode_value ('page_name', 'pages', 'page_id', this_page), 'LEFT');
   htp.tableData (common.space2$);
   htp.tableData ('Count', 'LEFT');
   htp.tableData (common.space2$);
   htp.tableData (icount, 'LEFT');
   htp.tableRowClose;

   return icount;

end stats2_pages;

-- *******************************************************************************************

function get_pop_id_descr (ID integer) return varchar2 is

cursor	driver is
select	m.short_descr, p.item_value
from	meta_data_codes		m,
	poplist_items		p
where	p.poplist_item_id	= ID
and	m.meta_data_code	= p.meta_data_code;

driver_rec	driver%ROWTYPE;

begin

   open driver;
   fetch driver into driver_rec;
   close driver;

   return driver_rec.short_descr || ': ' || driver_rec.item_value;

end get_pop_id_descr;


procedure get_pop_fee2 (this_pop_id integer, fee in out number, late_fee in out number) is

cursor	driver is
select	nvl (fee, 0), nvl (late_fee, 0)
from	poplist_items
where	poplist_item_id = this_pop_id;


begin

   open driver;
   fetch driver into fee, late_fee;
   close driver;

end get_pop_fee2;

function get_fee_matrix_id_descr (ID integer, sep varchar2) return varchar2 is

cursor	driver is
select	*
from	fee_matrix
where	fee_matrix_id = ID;

driver_rec	driver%ROWTYPE;

begin

   open driver;
   fetch driver into driver_rec;
   close driver;

   return 	get_pop_id_descr (driver_rec.poplist_item_id1) || sep ||
		get_pop_id_descr (driver_rec.poplist_item_id2);

end get_fee_matrix_id_descr;

-- *******************************************************************************************

procedure get_matrix_fee2 (this_fee_matrix integer, fee in out number, late_fee in out number) is

cursor	driver is
select	nvl (fee, 0), nvl (late_fee, 0)
from	fee_matrix
where	fee_matrix_id = this_fee_matrix;

driver_rec	driver%ROWTYPE;

begin

   open driver;
   fetch driver into fee, late_fee;
   close driver;

end get_matrix_fee2;

-- *******************************************************************************************
-- *******************************************************************************************

procedure get_meta_fee2 (this_meta_data integer, fee in out number, late_fee in out number) is

cursor	driver is
select	nvl (fee, 0), nvl (late_fee, 0)
from	meta_data_codes
where	meta_data_code = this_meta_data;

driver_rec	driver%ROWTYPE;

begin

   open driver;
   fetch driver into fee, late_fee;
   close driver;

end get_meta_fee2;


-- *******************************************************************************************

procedure show_stat (descr varchar2, stat varchar2, icount integer, unit_price number, total number) is

begin

   htp.tableRowOpen ('RIGHT', 'TOP');

   if descr is NOT null then
      htp.tableHeader (descr, 'LEFT', crowspan=>'2');
   end if;

   htp.tableData (common.space2$);

   htp.tableData (stat, 'LEFT');
   htp.tableData (common.space2$);

   htp.tableData (icount, 'LEFT');
   htp.tableData ('@ ' || to_char (unit_price, common.currency_fmt$) || ' each');

   htp.tableData (common.space2$);
   htp.tableData (to_char (total, common.currency_fmt$));

   htp.tableRowClose;

end show_stat;

-- *******************************************************************************************

procedure stats2_get_reg (this_meta_data integer, this_poplist integer, this_fee_matrix integer) is

cursor	driver is
select	*
from	registrations
where	meta_data_code		= this_meta_data
or	poplist_item_id		= this_poplist
or	fee_matrix_id		= this_fee_matrix;

fee		number;
late_fee	number;
total_fees	number := 0;
total_late	number := 0;

early_count	integer := 0;
late_count	integer := 0;

descr		varchar2(4000);
pop_id1		integer;
pop_id2		integer;
icount		integer;
multi_fee	boolean := false;

begin

   if this_meta_data is not null then
      descr := common.decode_value ('short_descr', 'meta_data_codes', 'meta_data_code', this_meta_data);
      get_meta_fee2 (this_meta_data, fee, late_fee);
   elsif this_poplist is not null then
      descr := get_pop_id_descr (this_poplist);
      get_pop_fee2 (this_poplist, fee, late_fee);
   else
      descr := get_fee_matrix_id_descr (this_fee_matrix, htf.br);
      get_matrix_fee2 (this_fee_matrix, fee, late_fee);
   end if;

   if common.decode_value ('data_type_code', 'meta_data_codes', 'meta_data_code', this_meta_data) = dataTfee2$ then
      multi_fee := true;
   end if;

-- **********

   for driver_rec in driver LOOP
      icount := 1;
      if multi_fee then
	 icount := driver_rec.value;
      end if;

      if driver_rec.timestamp >
	     registration.get_cutoff_date (driver_rec.meta_data_code, regist_ID_meta$) + common.grace_period$ then
	 late_count := late_count	+ icount;
	 total_late := total_late	+ (late_fee * icount);
      else
	 early_count := early_count	+ icount;
	 total_fees := total_fees	+ (fee * icount);
      end if;
   end loop;

   show_stat (descr,	'Early', early_count,	fee,	  total_fees);
   show_stat (null,	'Late',	 late_count,	late_fee, total_late);
   common.show_data_sep (8);

end stats2_get_reg;

-- *******************************************************************************************


procedure stats2_get_num (this_meta_code integer) is

cursor	driver is
select	sum (nvl (value, 0)) total
from	registrations
where	meta_data_code	= this_meta_code;

total	number;

begin

   open driver;
   fetch driver into total;
   close driver;

   common.show_data_row (common.decode_value ('meta_data_descr', 'meta_data_codes',
			'meta_data_code', this_meta_code), total);

end stats2_get_num;

-- *******************************************************************************************

procedure stats2_get_str (this_meta_code integer) is

cursor	driver is
select	decode (m.display_code, disp_YN$,
		decode (r.value, 'Y', 'Yes', 'N', 'No', r.value),
		r.value)	descr,
	count(1) total
from	registrations		r,
	meta_data_codes		m
where	m.meta_data_code	= this_meta_code
and	r.meta_data_code	= m.meta_data_code
and	r.value			is not null
group by
	decode (m.display_code, disp_YN$,
		decode (r.value, 'Y', 'Yes', 'N', 'No', r.value),
		r.value);

descr	meta_data_codes.meta_data_descr%TYPE;

begin

   descr := common.decode_value ('meta_data_descr', 'meta_data_codes',
				'meta_data_code', this_meta_code) || ': ';
   for driver_rec in driver LOOP
      common.show_data_row (descr || driver_rec.descr, driver_rec.total);
   end loop;

end stats2_get_str;

-- *******************************************************************************************
-- *******************************************************************************************

procedure stats2_rpt (	this_page		common.ident_arr,
			this_meta_data		common.ident_arr,
			this_poplist_id		common.ident_arr,
			this_fee_matrix_id	common.ident_arr) is

icount		integer := 0;
data_type	meta_data_codes.data_type_code%TYPE;

begin

   if common.header ('Registration,statistics') then
      htp.tableOpen;

      for i in 1..this_page.count LOOP
	 if this_page(i) is not null then
	    icount := icount + stats2_pages (this_page(i));
	 end if;
      end loop;

      if icount > 0 then
	 htp.tableRowOpen ('RIGHT', 'TOP');
	 htp.tableHeader ('Total', 'LEFT', ccolspan=>'4');
	 htp.tableData (icount, 'LEFT');
	 htp.tableRowClose;
	 common.show_data_sep (8);
      end if;

      for i in 1..this_meta_data.count LOOP
	 if this_meta_data(i) is not null then
	    data_type := common.decode_value ('data_type_code', 'meta_data_codes',
						'meta_data_code', this_meta_data(i));
	    if data_type in (dataTfee$, dataTfee2$) then
	       stats2_get_reg (this_meta_data(i), null, null);
	    elsif data_type = dataTnum$ then
	       stats2_get_num (this_meta_data(i));
	    elsif data_type in (dataTstr$, dataTdte$) then
	       stats2_get_str (this_meta_data(i));
	    end if;

	 end if;
      end loop;

      for i in 1..this_poplist_id.count LOOP
	 if this_poplist_id(i) is not null then
	    stats2_get_reg (null, this_poplist_id(i), null);
	 end if;
      end loop;

      for i in 1..this_fee_matrix_id.count LOOP
	 if this_fee_matrix_id(i) is not null then
	    stats2_get_reg (null, null, this_fee_matrix_id(i));
	 end if;
      end loop;

      htp.tableClose;
   else
      htp.header (3, not_authorized$);
   end if;
   common.footer;

end stats2_rpt;


-- ******************************************************************************
-- ******************************************************************************

/* paying_summary
 * Creates a matrix counting how many (potential) payees are for every fee
 * item in every (statistics-enabled) page
 */

procedure paying_summary is

  type page_ids_type is table of pages.page_id%type;
  page_ids page_ids_type;
  type page_names_type is table of pages.page_name%type;
  page_names page_names_type;

	cursor	driver_page is
		select	page_id, page_name
		from	pages
		where	'Y' = nvl (include_statistics, 'Y')
		order by page_name
    ;

  cursor driver_fee is
    select r.meta_data_code, r.page_id, COUNT(*) TOT
    from registrations r
    join meta_data_codes mdc
      on (r.meta_data_code = mdc.meta_data_code)
    join pages p
      on (p.page_id = r.page_id)
    join registrants s
      on (r.page_id = s.page_id and r.person_id = s.person_id)
    where nvl(p.include_statistics, 'Y') = 'Y'
      and s.cancel is null
      and mdc.data_type_code = 'FEE'
    group by r.meta_data_code, r.page_id
		UNION
    select r.meta_data_code, r.page_id, SUM(r.value) TOT
    from registrations r
    join meta_data_codes mdc
      on (r.meta_data_code = mdc.meta_data_code)
    join pages p
      on (p.page_id = r.page_id)
    join registrants s
      on (r.page_id = s.page_id and r.person_id = s.person_id)
    where nvl(p.include_statistics, 'Y') = 'Y'
      and s.cancel is null
      and mdc.data_type_code = 'FEE2'
    group by r.meta_data_code, r.page_id
    ;

  cursor driver_mdc(code meta_data_codes.meta_data_code%TYPE) is
    select mdc.meta_data_descr, mdc.data_type_code
    from meta_data_codes mdc
    where mdc.meta_data_code = code
    ;


  type pages_type is table of pages.page_name%TYPE index by binary_integer;
  pages_arr pages_type;

  type fee_items_type is table of pages_type index by binary_integer;
  fee_items fee_items_type;

  i integer;
  j integer;
  ic integer;
  jc integer;

  mdcname meta_data_codes.meta_data_descr%TYPE;
  mdctype meta_data_codes.data_type_code%TYPE;

  aretherefee2s boolean default false;

begin

  if common.header ('Paying summary') then

    -- Some instructions
    htp.p('<p>The following matrix shows how many (paying) items in registraton
    forms have been selected.</p>
    <p>This will give a quick glimpse on the expected incomes and, more important<br />
    the major sources of the incomes (in choosable options, not associated fee).</p>
    <p>For multioption items, this table shows if one of the options has been selected.<br />
    For "fee-mult" (e.g., How many additional tickets...), the total sum is calculated.
    </p>
    ');

    -- create matrix in memory

    for driver_rec in driver_fee LOOP
      fee_items(driver_rec.meta_data_code)(driver_rec.page_id) := driver_rec.TOT;
    end loop;

    open driver_page;
    fetch driver_page bulk collect into page_ids, page_names;
    close driver_page;

    /*	-- <DEBUG>
    for i in 1..page_ids.count loop
      htp.p(page_ids(i) || ' - ' || page_names(i) || '<br />');

    end loop;
    -- </DEBUG>
     */

    htp.tableOpen ('BORDER');
    htp.tableRowOpen ('LEFT', 'BOTTOM');
    htp.tableHeader(htf.em('Item\Page'));
    for i in 1..page_names.count loop
      htp.tableHeader(page_names(i));
    end loop;

    htp.tableRowClose;

    ic := fee_items.first;
    -- let's scan the mdc's
    for i in 1..fee_items.count loop
      open driver_mdc(ic);
      fetch driver_mdc into mdcname, mdctype;
      close driver_mdc;
      htp.tableRowOpen ('LEFT', 'BOTTOM');
      if (mdctype = 'FEE2') then
	htp.tableHeader(mdcname || '*');
	aretherefee2s := true;
      else
	htp.tableHeader(mdcname);
      end if;
      -- now we go through the pages...
      for jc in 1..page_ids.count loop
	if fee_items(ic).exists(page_ids(jc)) then
	  htp.tableData(nvl(fee_items(ic)(page_ids(jc)),'0') );
	else
	  htp.tableData('0');
	end if;
      end loop;
      htp.tableRowClose;

      ic := fee_items.next(ic);
    end loop;


    htp.tableClose;

    if (aretherefee2s) then
      htp.p('<p><strong>*</strong> fee-mult type, sum is displayed.</p>');
    end if;

    common.footer;

  else
    htp.header (3, not_authorized$);
  end if;

end paying_summary;

-- *******************************************************************************
-- ******************************************************************************



end reg_rpt;

/
show error

set define on

--	Copyright (C) 2004-2008 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>
