set define off

create or replace package BODY regist_import as

rpt_title$	constant char(32)	:= 'Registration Form Import Utility';

type	pages_type	is table of pages%ROWTYPE			index by pls_integer;
type	page_data_type	is table of page_data%ROWTYPE			index by pls_integer;
type	meta_data_type	is table of meta_data_codes%ROWTYPE		index by pls_integer;
type	cond_req_type	is table of conditional_requirements%ROWTYPE	index by pls_integer;
type	poplist_type	is table of poplist_items%ROWTYPE		index by pls_integer;
type	fee_matrix_type	is table of fee_matrix%ROWTYPE			index by pls_integer;
type	reg_cat_type	is table of reg_cat_codes%ROWTYPE		index by pls_integer;

type	reg_data_type is record
	(	page		pages_type,
		page_data	page_data_type,
		meta_data	meta_data_type,
		cond_req	cond_req_type,
		poplist		poplist_type,
		fee_matrix	fee_matrix_type,
		reg_cat		reg_cat_type);

-- *************************************************************************************************************

function regist_data_exists return boolean is

cursor	driver is
select	count(1)
from	registrants;

icount	integer;

begin

   open driver;
   fetch driver into icount;
   close driver;

   if icount = 0 then
      return false;
   end if;

   return true;

end regist_data_exists;

-- *************************************************************************************************************

function regist_form_data_exists return boolean is

cursor	driver is
select	count(1)
from	pages;

icount	integer;

begin

   open driver;
   fetch driver into icount;
   close driver;

   if icount = 0 then
      return false;
   end if;

   return true;

end regist_form_data_exists;

-- *************************************************************************************************************

function people_registerd return boolean is

cursor	driver is
select	count(1)
from	registrations;

icount	integer;

begin

   open driver;
   fetch driver into icount;
   close driver;

   if icount = 0 then
      return false;
   end if;

   return true;

end people_registerd;

-- *************************************************************************************************************
-- *************************************************************************************************************

procedure wipe_form_data is

begin

   delete from page_data;
   delete from pages;
   delete from fee_matrix;
   delete from conditional_requirements;
   delete from poplist_items;
   delete from meta_data_codes;
   delete from reg_cat_codes;

end wipe_form_data;

-- *************************************************************************************************************

function load_form_data (conf_id number, pageIDs common.ident_arr) return reg_data_type is

recs		reg_data_type;

begin

   for i in 1..pageIDs.COUNT loop
      select	PAGE_ID, PAGE_NAME, AVAILABLE, SHOW_ON_PROFILE, SHOW_ON_ABSTRACT, RETURN_URL, ADMIN_PRIVS,
		LOGIN_REQ, MAX_REGISTRANTS, POST_INFO, ADMIN_EMAIL, HIDE_INVOICE, INCLUDE_PARTICIPANT_RPT,
		INCLUDE_STATISTICS, EXTERNAL_URL, BYPASS_URL
		into recs.page(i)
      from	XML_pages
      where	conference_id	= conf_id
      and	page_id		= pageIDs(i);

      insert into ID_list values (recs.page(i).page_id);
   end loop;

   select	PAGE_DATA_ID, PAGE_ID, META_DATA_CODE, SORT_ORDER, TEXT
		BULK COLLECT into recs.page_data
   from		XML_page_data
   where	conference_id	= conf_id
   and		page_id		in (select ID from ID_list);

-- **********

   select	META_DATA_CODE, META_DATA_DESCR, DATA_TYPE_CODE, DISPLAY_CODE, DATA_LENGTH, VIEW_LENGTH, 
		REQUIRED, FIXED_LENGTH, FEE, LATE_FEE, SHORT_DESCR, ENCRYPT, MISCELLANEOUS, FREEZE_UPDATES, 
		SINGLE_SELECTION, CUTOFF_DATE, REG_CAT_CODE, CHECKBOX_MAX_ITEMS, DROPDOWN_MAX_ITEMS
		BULK COLLECT into recs.meta_data
   from		XML_meta_data_codes
   where	conference_id	= conf_id
   and		meta_data_code	in
		(	select	meta_data_code
			from	XML_page_data
			where	conference_id	= conf_id
			and	page_id		in (select ID from ID_list)
		);

   delete from ID_list;

   for i in 1..recs.meta_data.COUNT loop
      insert into ID_list values (recs.meta_data(i).meta_data_code);
   end loop;

-- **********

   select	YESNO_META_CODE, META_DATA_CODE
		BULK COLLECT into recs.cond_req
   from		XML_conditional_requirements
   where	conference_id	= conf_id
   and	(	yesno_meta_code	in (select ID from ID_list)
	 OR	meta_data_code	in (select ID from ID_list)
        );

-- **********

   select	REG_CAT_CODE, REG_CAT_DESCR
		BULK COLLECT into recs.reg_cat
   from		XML_reg_cat_codes
   where	conference_id	= conf_id
   and		reg_cat_code	in
		(	select	reg_cat_code
			from	xml_meta_data_codes
			where	conference_id	= conf_id
			and	meta_data_code	in (select ID from ID_list)
		);

-- **********

   select	POPLIST_ITEM_ID, META_DATA_CODE, ITEM_VALUE, SORT_ORDER, FEE, LATE_FEE
		BULK COLLECT into recs.poplist
   from		XML_poplist_items
   where	conference_id	= conf_id
   and		meta_data_code	in (select ID from ID_list);

   delete from ID_list;

-- **********

   for i in 1..recs.poplist.COUNT loop
      insert into ID_list values (recs.poplist(i).poplist_item_id);
   end loop;

   select	FEE_MATRIX_ID, POPLIST_ITEM_ID1, POPLIST_ITEM_ID2, FEE, LATE_FEE
		BULK COLLECT into recs.fee_matrix
   from		XML_fee_matrix
   where	conference_id		= conf_id
   and	(	poplist_item_id1	in (select ID from ID_list)
	 OR	poplist_item_id2	in (select ID from ID_list)
	);

   return recs;

end load_form_data;

-- *************************************************************************************************************

procedure change_form_IDs (recs IN OUT reg_data_type) is

cursor	C_pages (ID$ integer) is
select	1
from	pages
where	page_id = ID$;

cursor	C_page_data (ID$ integer) is
select	1
from	page_data
where	page_data_id = ID$;

cursor	C_meta_data (ID$ integer) is
select	1
from	meta_data_codes
where	meta_data_code = ID$;

cursor	C_poplist (ID$ integer) is
select	1
from	poplist_items
where	poplist_item_id = ID$;

cursor	C_fee_matrix (ID$ integer) is
select	1
from	fee_matrix
where	fee_matrix_id = ID$;

cursor	C_reg_cat (code$ varchar2, descr$ varchar2) is
select	1
from	reg_cat_codes
where	reg_cat_code		= code$
and	upper(reg_cat_descr)	= upper(descr$);

cursor	C2_reg_cat is
select	reg_cat_code
from	reg_cat_codes
where	regexp_like(reg_cat_code, '^[0-9]$');

rec_found	boolean;
dummy		integer;
new_id		integer;
new_id_str	char(4);

begin

   for i in 1..recs.page.COUNT loop
      open C_pages(recs.page(i).page_id);
      fetch C_pages into dummy;
      rec_found := C_pages%FOUND;
      close C_pages;

      if rec_found then
         new_id := page_id_seq.nextval;

         for j in 1..recs.page_data.COUNT loop
            if recs.page_data(j).page_id = recs.page(i).page_id then
               recs.page_data(j).page_id := new_id;
            end if;
         end loop;

         recs.page(i).page_id := new_id;
      end if;

   end loop;

-- **********

   for i in 1..recs.page_data.COUNT loop
      open C_page_data(recs.page_data(i).page_data_id);
      fetch C_page_data into dummy;
      rec_found := C_page_data%FOUND;
      close C_page_data;

      if rec_found then
         recs.page_data(i).page_data_id := page_data_id.nextval;
      end if;

   end loop;

-- **********

   for i in 1..recs.meta_data.COUNT loop
      open C_meta_data (recs.meta_data(i).meta_data_code);
      fetch C_meta_data into dummy;
      rec_found := C_meta_data%FOUND;
      close C_meta_data;

      if rec_found then
         new_id := meta_data_seq.nextval;

         for j in 1..recs.page_data.COUNT loop
            if recs.page_data(j).meta_data_code = recs.meta_data(i).meta_data_code then
               recs.page_data(j).meta_data_code := new_id;
            end if;
         end loop;

         for j in 1..recs.cond_req.COUNT loop
            if recs.cond_req(j).yesno_meta_code = recs.meta_data(i).meta_data_code then
               recs.cond_req(j).yesno_meta_code := new_id;
            end if;

            if recs.cond_req(j).meta_data_code = recs.meta_data(i).meta_data_code then
               recs.cond_req(j).meta_data_code := new_id;
            end if;

            if recs.poplist(j).meta_data_code = recs.meta_data(i).meta_data_code then
               recs.poplist(j).meta_data_code := new_id;
            end if;
         end loop;

         recs.meta_data(i).meta_data_code := new_id;
      end if;
   end loop;

-- **********

   for i in 1..recs.poplist.COUNT loop
      open C_poplist(recs.poplist(i).poplist_item_id);
      fetch C_poplist into dummy;
      rec_found := C_poplist%FOUND;
      close C_poplist;

      if rec_found then
         new_id := poplist_item_id_seq.nextval;

         for j in 1..recs.fee_matrix.COUNT loop
            if recs.fee_matrix(i).poplist_item_id1 = recs.poplist(i).poplist_item_id then
               recs.fee_matrix(i).poplist_item_id1 := new_id;
            end if;

            if recs.fee_matrix(i).poplist_item_id2 = recs.poplist(i).poplist_item_id then
               recs.fee_matrix(i).poplist_item_id2 := new_id;
            end if;
         end loop;

         recs.poplist(i).poplist_item_id := new_id;
      end if;      
   end loop;

-- **********

   for i in 1..recs.fee_matrix.COUNT loop
      open C_fee_matrix(recs.fee_matrix(i).fee_matrix_id);
      fetch C_fee_matrix into dummy;
      rec_found := C_fee_matrix%FOUND;
      close C_fee_matrix;

      if rec_found then
         recs.fee_matrix(i).fee_matrix_id := fee_matrix_id_seq.nextval;
      end if;
   end loop;

-- **********

   for i in 1..recs.reg_cat.COUNT loop
      open C_reg_cat(recs.reg_cat(i).reg_cat_code, recs.reg_cat(i).reg_cat_descr);
      fetch C_reg_cat into dummy;
      rec_found := C_reg_cat%FOUND;
      close C_reg_cat;

      if rec_found then
         open C2_reg_cat;
         fetch C2_reg_cat into new_id;
         close C2_reg_cat;

         new_id		:= nvl(new_id,0) + 1;
         new_id_str	:= trim(to_char(new_id,'0000'));

         for j in 1..recs.meta_data.COUNT loop
            if recs.meta_data(i).reg_cat_code = recs.reg_cat(i).reg_cat_code then
               recs.meta_data(i).reg_cat_code := new_id_str;
            end if;
         end loop;

         recs.reg_cat(i).reg_cat_code := new_id_str;
      end if;
   end loop;

end change_form_IDs;

-- *************************************************************************************************************

procedure insert_form_data (recs reg_data_type) is

begin

   FORALL i in 1..recs.reg_cat.COUNT
      insert into reg_cat_codes values recs.reg_cat(i);

   FORALL i in 1..recs.meta_data.COUNT
      insert into meta_data_codes values recs.meta_data(i);

   FORALL i in 1..recs.poplist.COUNT
      insert into poplist_items values recs.poplist(i);

   FORALL i in 1..recs.cond_req.COUNT
      insert into conditional_requirements values recs.cond_req(i);

   FORALL i in 1..recs.fee_matrix.COUNT
      insert into fee_matrix values recs.fee_matrix(i);

   FORALL i in 1..recs.page.COUNT
      insert into pages values recs.page(i);

   FORALL i in 1..recs.page_data.COUNT
      insert into page_data values recs.page_data(i);

end insert_form_data;

-- *************************************************************************************************************

procedure copy (conf_id number, wipe varchar2 default 'N',
		pageIDs common.ident_arr default common.mt_ident_arr$) is

recs		reg_data_type;
continue	boolean := true;

begin

   htp.p ('');
   if common.header (rpt_title$) then

      if pageIDs.COUNT = 0 and wipe = 'N' then
         htp.header (3, 'You must wipe the existing data and/or select at least one Form to import',
				cattributes=>common.red$);
      else
         if wipe = 'Y' then -- wipe and reload all registration data
            wipe_form_data;
         end if;

         recs := load_form_data(conf_id, pageIDs);
         change_form_IDs(recs);
         insert_form_data(recs);

         htp.header (3, rpt_title$ || ' Completed Successfully');
         htp.p ('Return to');
         htp.anchor ('regist_import.html', rpt_title$);
         htp.para;
      end if;

   end if;

   common.footer;

end copy;

-- *************************************************************************************************************
-- *************************************************************************************************************

procedure show_conferece_info (conf_id number) is

cursor	driver is
select	*
from	repo_conf_conferences
where	conference_id = conf_id;

rec	driver%ROWTYPE;

begin

   htp.br;
   open driver;
   fetch driver into rec;
   close driver;

   htp.header (2, 'Copy Registration Form Data from ' || rec.conference_title);
   htp.italic (rec.location);
   htp.br;
   htp.italic (rec.start_date);
   htp.italic (common.mdash$);
   htp.italic (rec.end_date);
   htp.para;

end show_conferece_info;

-- *************************************************************************************************************

procedure show_pages(conf_id number) is

cursor	driver is
select	page_id, page_name
from	xml_pages
where	conference_id = conf_id;

cursor	C2 (text$ varchar2) is
select	1
from	pages
where	page_name = text$;

rec_found	boolean;
dummy		integer;

begin

   common.show_data_row (null, htf.bold('Import the following Registration Forms') );
   for rec in driver LOOP

      open C2(rec.page_name);
      fetch C2 into dummy;
      rec_found := C2%FOUND;
      close C2;

      if rec_found then
         common.show_data_row (rec.page_name, common.mdash$ || 'A page with this name already exists.');
      else
         common.show_data_row (rec.page_name, htf.formCheckBox ('pageIDs', rec.page_id) );
      end if;
   end loop;

end show_pages;

-- *************************************************************************************************************

procedure step2 (conf_id number) is

continue	boolean := true;
show_footnote	boolean := false;

begin

   htp.p ('');
   if common.header (rpt_title$) then
      if conf_id is null then
         htp.header (3, 'You must select a conference', cattributes=>common.red$);

      else
         show_conferece_info (conf_id);

         htp.formOpen ('regist_import.copy', 'POST');
         htp.formHidden ('conf_id', conf_id);

         htp.tableOpen;

         if regist_form_data_exists and NOT people_registerd then
            show_footnote := true;
            common.show_data_row ('Wipe Existing Data' || htf.sup('*', cattributes=>common.red$),
				htf.formCheckBox('wipe','Y') );
         end if;

         show_pages(conf_id);

         common.show_data_row (null, htf.formSubmit);
         htp.tableClose;
         htp.formClose;

         if show_footnote then
            htp.sup('*', cattributes=>common.red$);
            htp.p ('Checking this option will delete all Registration Form Data in your conference');
            htp.p ('prior to uploading the selected registration data.');
         end if;

      end if;

   end if;

   common.footer;

end step2;

-- *************************************************************************************************************
-- *************************************************************************************************************

procedure show_form is

cursor	driver is
select	c.conference_id, c.conference_title
from	repo_conf_conferences	c
where	conference_id		in
(	select	DISTINCT conference_id
	from	repo_XML_data)
order by to_char(c.start_date,'yyyy') desc, c.conference_title;

recs	common.popups_type;
buffer	varchar2(50);

begin

   open driver;
   fetch driver BULK COLLECT into recs;
   close driver;

   htp.header (3, 'Select the confernece you wish to use as the registration form source.');
   htp.formOpen ('regist_import.step2', 'POST');
   htp.tableOpen;

   common.show_popup_array_row ('Conference', 'conf_id', recs);
   common.show_data_row (buffer, htf.formSubmit);

   htp.tableClose;
   htp.formClose;
   htp.para;

end show_form;

-- *************************************************************************************************************

procedure HTML is

begin

   htp.p ('');
   if common.header (rpt_title$) then
      htp.p ('The');
      htp.p (rpt_title$);
      htp.p ('copies all registration data from another conference.');

      if regist_data_exists then
         htp.header (3, 'One or more pesons have already registered. You cannot use this utility.',
			cattributes=>common.red$);
         htp.p ('If the registration data is test data, the SPMS regional manager may be able to remove your test data.');
      else
         show_form;
      end if;

   end if;

   common.footer;

end HTML;

-- *************************************************************************************************************

end regist_import;
/
show error

set define on

--	Copyright (C) 2004 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>

