-- SPMS - Scientific Programme Management System
-- see at the bottom of this file for licensing
-- and copyright information

set define off

Create or Replace PACKAGE BODY regist_maint AS

  record_per_page$ CONSTANT INTEGER := 10 ;
  rpt_title$	CONSTANT VARCHAR2(30)	:= 'Registration ';
  rpt_title2$	CONSTANT VARCHAR2(30)	:= rpt_title$ || 'Payments';
  rpt_title3$	CONSTANT VARCHAR2(30)	:= rpt_title$ || 'Statistics';
  rpt_title4$	CONSTANT VARCHAR2(30)	:= rpt_title$ || 'Receipt';
  rpt_title5$	CONSTANT VARCHAR2(30)	:= rpt_title$ || 'List';
  rpt_title6$	CONSTANT VARCHAR2(30)	:= rpt_title$ || 'Payment Failed';
  rpt_title7$	CONSTANT VARCHAR2(40)	:= 'All Attendees - View/Edit/Cancel';
  select_one$	CONSTANT VARCHAR2(40)	:= 'Select at least one attendee';

  disp_txt$	CONSTANT display_codes.display_code%TYPE := 'TXT';
  disp_YN$	CONSTANT display_codes.display_code%TYPE := 'YN';
  disp_popup$	CONSTANT display_codes.display_code%TYPE := 'POP';
  disp_multi$	CONSTANT display_codes.display_code%TYPE := 'MULT';
  disp_checkbox$	CONSTANT display_codes.display_code%TYPE := 'CHK';
  disp_mcb$	CONSTANT display_codes.display_code%TYPE := 'MCB';

  dataTnum$	CONSTANT data_type_codes.data_type_code%TYPE := 'NUM';
  dataTstr$	CONSTANT data_type_codes.data_type_code%TYPE := 'STR';
  dataTdte$	CONSTANT data_type_codes.data_type_code%TYPE := 'DTE';
  dataTfee$	CONSTANT data_type_codes.data_type_code%TYPE := 'FEE';
  dataTfee2$	CONSTANT data_type_codes.data_type_code%TYPE := 'FEE2';
  dataTsps$	CONSTANT data_type_codes.data_type_code%TYPE := 'SPS';

  checkbox$	CONSTANT VARCHAR2(8)	:= 'checkbox';
  non_present$	CONSTANT VARCHAR2(2)	:= 'ZZ';
  waived$		CONSTANT payment_codes.payment_code%TYPE := 'WAVE';
  back2_pay_inv$	CONSTANT VARCHAR2(200)	:= htf.anchor ('jacow.html', 'Back to Payments and Receipts') || htf.para;

  js_close_win$	CONSTANT VARCHAR2(99)	:= '<input type="button" value="No" name="close" onClick="window.close()">';
  notices_1st$	CONSTANT INTEGER := 1;
  notices_all$	CONSTANT INTEGER := 2;

  regist_ID_meta$	CONSTANT VARCHAR2(4) := 'meta';
  regist_ID_popl$	CONSTANT VARCHAR2(4) := 'popl';
  regist_ID_mtrx$	CONSTANT VARCHAR2(4) := 'mtrx';
  regist_ID_paym$	CONSTANT VARCHAR2(4) := 'paym';
  regist_ID_ccpm$	CONSTANT VARCHAR2(4) := 'ccpm';
  regist_ID_chkb$	CONSTANT VARCHAR2(4) := 'chkb';

  HTML_output	BOOLEAN DEFAULT FALSE;
  print_only	BOOLEAN DEFAULT FALSE;
  ignore_admin$	BOOLEAN DEFAULT TRUE;
  validate_IP$	BOOLEAN DEFAULT TRUE;

  TYPE		pcode_type IS TABLE OF NUMBER INDEX BY presentation_codes.presentation_code%TYPE;
  TYPE		pmt_type IS TABLE OF payment_codes.instructions%TYPE INDEX BY VARCHAR2(4);
  pmt_instr	pmt_type;
  init_pcode	pcode_type;

  TYPE		page_type IS RECORD (
  page_id		INTEGER,
  page_name	pages.page_name%TYPE);
  TYPE		page_tbl IS TABLE OF page_type INDEX BY BINARY_INTEGER;
  page_rec	page_tbl;

  pcodes		common.ident_arr;


-- *******************************************************************************************
-- *******************************************************************************************

  FUNCTION JS$ RETURN VARCHAR2 IS

  BEGIN

    RETURN common.server || common.vdir$ || 'regist_maint.js';

  END JS$;

-- *******************************************************************************************
-- *******************************************************************************************

  FUNCTION person_list (abs_id INTEGER, sepchr VARCHAR2 DEFAULT ', ') RETURN VARCHAR2 IS

  CURSOR	driver IS
    SELECT	DISTINCT p.sort_lname, p.sort_fname, p.sort_mname, a.person_id
    FROM	persons			p,
    abstract_authors	a
    WHERE	a.abstract_id 		= abs_id
    AND	p.person_id		= a.person_id
    ORDER BY
    p.sort_lname, p.sort_fname, p.sort_mname;

  buffer	LONG := NULL;

  BEGIN

    FOR driver_rec IN driver LOOP
      IF is_registered (driver_rec.person_id) = 'Y' THEN
	buffer := buffer || sepchr || common.get_user_name (driver_rec.person_id);
      END IF;
    END LOOP;

    IF buffer IS NULL THEN
      RETURN buffer;
    END IF;

    RETURN substr (buffer, length(sepchr) + 1);

  END person_list;

-- *******************************************************************************************
-- *******************************************************************************************

  FUNCTION is_registered (pid INTEGER) RETURN VARCHAR2 IS

  CURSOR	driver IS
    SELECT	'Y'
    FROM	pages			p,
    registrants		r
    WHERE	r.person_id		= pid
    AND	p.page_id		= r.page_id
    AND	p.include_participant_rpt	= 'Y';

  dummy		VARCHAR2(1);
  rec_found	BOOLEAN;

  BEGIN

    OPEN driver;
    FETCH driver INTO dummy;
    rec_found := driver%FOUND;
    CLOSE driver;

    IF rec_found THEN
      RETURN dummy;
    END IF;

    RETURN 'N';

  END is_registered;

-- *******************************************************************************************
-- *******************************************************************************************

  FUNCTION get_fee_amt (item_id INTEGER, tmsdate DATE, considergraceperiod BOOLEAN DEFAULT TRUE) RETURN NUMBER IS

  CURSOR driver (rid NUMBER) IS
    SELECT *
    FROM
    (
    SELECT r.registration_id, r.PERSON_ID, r.META_DATA_CODE, r.VALUE,
    r.POPLIST_ITEM_ID, r.FEE_MATRIX_ID, r.TIMESTAMP, r.PAGE_ID,
    r.CUTOFF_DATE,  r.DATA_TYPE_CODE, r.DISPLAY_CODE, r.short_descr || ' : ' || pl.ITEM_VALUE short_descr, pl.FEE fee, pl.LATE_FEE late_fee
    FROM  (SELECT r1.registration_id, r1.PERSON_ID, r1.META_DATA_CODE, r1.VALUE, r1.POPLIST_ITEM_ID, r1.FEE_MATRIX_ID,
    r1.TIMESTAMP,  r1.PAGE_ID, mdc.CUTOFF_DATE, mdc.DATA_TYPE_CODE, mdc.DISPLAY_CODE, mdc.SHORT_DESCR
    FROM registrations r1
    inner join meta_data_codes mdc ON mdc.META_DATA_CODE = r1.META_DATA_CODE
    WHERE r1.registration_id = rid AND r1.POPLIST_ITEM_ID IS NOT NULL) r
    inner join poplist_items pl ON r.POPLIST_ITEM_ID = pl.POPLIST_ITEM_ID
    UNION
    SELECT r.registration_id, r.PERSON_ID, r.META_DATA_CODE, r.VALUE, r.POPLIST_ITEM_ID, r.FEE_MATRIX_ID,
    r.TIMESTAMP, r.PAGE_ID,
    mdc.CUTOFF_DATE,  mdc.DATA_TYPE_CODE, mdc.DISPLAY_CODE, mdc.SHORT_DESCR, mdc.FEE, mdc.LATE_FEE
    FROM registrations r
    inner join meta_data_codes mdc ON mdc.META_DATA_CODE = r.META_DATA_CODE
    WHERE r.registration_id = rid AND r.POPLIST_ITEM_ID IS  NULL
    )
    WHERE data_type_code IN ('FEE', 'FEE2')
    ORDER BY meta_data_code;

  driver_rec driver%ROWTYPE;
  found BOOLEAN;
  fee		INTEGER;
  descr VARCHAR2(2000);
  grace_period NUMBER := common.grace_period$;

  BEGIN

    -- consider grace_period or not
    IF (considergraceperiod) THEN
      grace_period := common.grace_period$;
    ELSE
      grace_period := 0;
    END IF;

    OPEN driver(item_id);
    FETCH driver INTO driver_rec;
    found := driver%FOUND;
    CLOSE driver;

    IF found THEN
      IF driver_rec.fee_matrix_id IS NOT NULL THEN
	fee := nvl (driver_rec.fee, fee);
      END IF;

      if tmsdate <= driver_rec.cutoff_date + common.grace_period$ + (common.tz_offset$ / 24) then
	fee := driver_rec.fee;
      else
	fee := nvl(driver_rec.late_fee, driver_rec.fee);
      END IF;
      IF driver_rec.display_code = disp_YN$ AND nvl (driver_rec.value, 'N') = 'N' THEN
	fee := 0;
      END IF;

      IF(fee IS NULL) THEN
	fee := 0;
      END IF;

      IF driver_rec.data_type_code = dataTfee2$ THEN
	fee	:= driver_rec.value * fee;
      END IF;
    ELSE
      fee := 0;
    END IF;

    RETURN nvl(fee, 0);

  END get_fee_amt;

-- *******************************************************************************************-- *******************************************************************************************

  FUNCTION get_reg_descr (item_id INTEGER) RETURN VARCHAR2 IS

  CURSOR driver (rid NUMBER) IS
    SELECT	r.short_descr || ' : ' || pl.ITEM_VALUE short_descr
    FROM  (SELECT r1.POPLIST_ITEM_ID, mdc.SHORT_DESCR
    FROM registrations r1
    inner join meta_data_codes mdc ON mdc.META_DATA_CODE = r1.META_DATA_CODE
    WHERE r1.registration_id = rid AND r1.POPLIST_ITEM_ID IS NOT NULL) r
    inner join poplist_items pl ON r.POPLIST_ITEM_ID = pl.POPLIST_ITEM_ID
    UNION
    SELECT mdc.SHORT_DESCR
    FROM registrations r
    inner join meta_data_codes mdc ON mdc.META_DATA_CODE = r.META_DATA_CODE
    WHERE r.registration_id = rid AND r.POPLIST_ITEM_ID IS  NULL ;

  descr VARCHAR2(100);
  found BOOLEAN;

  BEGIN

    OPEN driver(item_id);
    FETCH driver INTO descr;

    RETURN descr;

  END get_reg_descr;

-- *******************************************************************************************

  FUNCTION get_fee_total (this_page INTEGER DEFAULT NULL, crypt_pid VARCHAR2 DEFAULT NULL, print BOOLEAN DEFAULT TRUE, show_prompt OUT BOOLEAN) RETURN NUMBER IS

  fee NUMBER;

  CURSOR driver (page INTEGER, person INTEGER) IS
    SELECT q.*
    FROM
    (
    SELECT r.registration_id, r.PERSON_ID, r.META_DATA_CODE, r.VALUE,
    r.POPLIST_ITEM_ID, r.FEE_MATRIX_ID, r.TIMESTAMP, r.PAGE_ID,
    r.CUTOFF_DATE,  r.DATA_TYPE_CODE, r.DISPLAY_CODE, r.short_descr || ' : ' || pl.ITEM_VALUE short_descr, pl.FEE fee, pl.LATE_FEE late_fee
    FROM  (SELECT r1.registration_id, r1.PERSON_ID, r1.META_DATA_CODE, r1.VALUE, r1.POPLIST_ITEM_ID, r1.FEE_MATRIX_ID,
    r1.TIMESTAMP,  r1.PAGE_ID, mdc.CUTOFF_DATE, mdc.DATA_TYPE_CODE, mdc.DISPLAY_CODE, mdc.SHORT_DESCR
    FROM registrations r1
    inner join meta_data_codes mdc ON mdc.META_DATA_CODE = r1.META_DATA_CODE
    WHERE r1.person_id = person AND r1.page_id = page AND r1.POPLIST_ITEM_ID IS NOT NULL) r
    inner join poplist_items pl ON r.POPLIST_ITEM_ID = pl.POPLIST_ITEM_ID
    UNION
    SELECT r.registration_id, r.PERSON_ID, r.META_DATA_CODE, r.VALUE, r.POPLIST_ITEM_ID, r.FEE_MATRIX_ID,
    r.TIMESTAMP, r.PAGE_ID,
    mdc.CUTOFF_DATE,  mdc.DATA_TYPE_CODE, mdc.DISPLAY_CODE, mdc.SHORT_DESCR, mdc.FEE, mdc.LATE_FEE
    FROM registrations r
    inner join meta_data_codes mdc ON mdc.META_DATA_CODE = r.META_DATA_CODE
    WHERE r.person_id = person AND r.page_id = page AND r.POPLIST_ITEM_ID IS  NULL
    ) q
    inner join registrants rg ON q.person_id = rg.PERSON_ID AND q.page_id = rg.PAGE_ID
    WHERE rg.CANCEL IS NULL AND q.data_type_code IN ('FEE', 'FEE2')
    ORDER BY registration_id;

  CURSOR paid(reg_id NUMBER) IS
    SELECT p.PROCESSED
    FROM
    registrations_payments rp
    inner join
    payments p ON rp.PAYMENT_ID = p.PAYMENT_ID
    WHERE rp.REGISTRATION_ID = reg_id AND p.processed IS NOT NULL;


  descr VARCHAR(2000) DEFAULT NULL;
  msg VARCHAR(2000) DEFAULT NULL;
  found BOOLEAN := TRUE;
  regist_ID_type VARCHAR2(4);
  display_code VARCHAR2(4);
  display_fee INTEGER;
  total INTEGER := 0;
  processed 	payments.PROCESSED%TYPE DEFAULT NULL;
  tmsdate DATE;
  reg_id_prev NUMBER := -1;
  chk_exist BOOLEAN := FALSE;
  pid persons.PERSON_ID%TYPE;

  BEGIN

    common.GET_GBL_PID;

    IF crypt_pid IS NULL THEN
      pid := common.gbl_pid;
    ELSE
      pid := common.DECRYPT(crypt_pid);
    END IF;

    total := 0;
    tmsdate := SYSDATE;
    show_prompt := FALSE;
    FOR driver_rec IN driver(this_page, pid) LOOP

      descr := NULL;
      msg := NULL;

      IF common.decode_value ('encrypt', 'meta_data_codes',
	'meta_data_code', driver_rec.meta_data_code) = 'Y' THEN
	driver_rec.value := common.decrypt (driver_rec.value);
      END IF;

      regist_ID_type := regist_ID_meta$;

/*
      Check how matrix fee works
      */

      IF driver_rec.fee_matrix_id IS NOT NULL THEN

	fee := nvl (driver_rec.fee, fee);
	regist_ID_type := regist_ID_mtrx$;
      END IF;

      IF driver_rec.poplist_item_id IS NOT NULL THEN
	regist_ID_type := regist_ID_popl$;
      END IF;

      OPEN paid(driver_rec.registration_id);
      FETCH paid INTO processed;
      CLOSE paid;

      tmsdate := nvl(processed, tmsdate);

      IF driver_rec.cutoff_date IS NULL OR tmsdate <= driver_rec.cutoff_date + common.grace_period$ + (common.tz_offset$ / 24)	THEN
	fee := driver_rec.fee;
      ELSE
	fee := nvl(driver_rec.late_fee, driver_rec.fee);
      END IF;

      IF driver_rec.display_code = disp_YN$ AND nvl (driver_rec.value, 'N') = 'N' THEN
	fee := 0;
      END IF;

      IF(fee IS NULL) THEN
	fee := 0;
      END IF;

      IF driver_rec.data_type_code = dataTfee2$ THEN
	msg	:= driver_rec.value || ' @ ' || fee || ' each';
	fee	:= driver_rec.value * fee;
      END IF;

      IF print THEN
	registration.show_row (chk_exist, driver_rec.short_descr, fee, msg, FALSE, FALSE, driver_rec.registration_id);
	IF chk_exist = TRUE THEN
	  show_prompt := TRUE;
	END IF;
      END IF;

      total := total + fee;

    END LOOP;

    RETURN total;

  END get_fee_total;

  FUNCTION get_fee_total_query (this_page INTEGER DEFAULT NULL, crypt_pid VARCHAR2 DEFAULT NULL) RETURN NUMBER IS

  fee NUMBER;

  CURSOR driver (page INTEGER, person INTEGER) IS
    SELECT q.*
    FROM
    (
    SELECT r.registration_id, r.PERSON_ID, r.META_DATA_CODE, r.VALUE,
    r.POPLIST_ITEM_ID, r.FEE_MATRIX_ID, r.TIMESTAMP, r.PAGE_ID,
    r.CUTOFF_DATE,  r.DATA_TYPE_CODE, r.DISPLAY_CODE, r.short_descr || ' : ' || pl.ITEM_VALUE short_descr, pl.FEE fee, pl.LATE_FEE late_fee
    FROM  (SELECT r1.registration_id, r1.PERSON_ID, r1.META_DATA_CODE, r1.VALUE, r1.POPLIST_ITEM_ID, r1.FEE_MATRIX_ID,
    r1.TIMESTAMP,  r1.PAGE_ID, mdc.CUTOFF_DATE, mdc.DATA_TYPE_CODE, mdc.DISPLAY_CODE, mdc.SHORT_DESCR
    FROM registrations r1
    inner join meta_data_codes mdc ON mdc.META_DATA_CODE = r1.META_DATA_CODE
    WHERE r1.person_id = person AND r1.page_id = page AND r1.POPLIST_ITEM_ID IS NOT NULL) r
    inner join poplist_items pl ON r.POPLIST_ITEM_ID = pl.POPLIST_ITEM_ID
    UNION
    SELECT r.registration_id, r.PERSON_ID, r.META_DATA_CODE, r.VALUE, r.POPLIST_ITEM_ID, r.FEE_MATRIX_ID,
    r.TIMESTAMP, r.PAGE_ID,
    mdc.CUTOFF_DATE,  mdc.DATA_TYPE_CODE, mdc.DISPLAY_CODE, mdc.SHORT_DESCR, mdc.FEE, mdc.LATE_FEE
    FROM registrations r
    inner join meta_data_codes mdc ON mdc.META_DATA_CODE = r.META_DATA_CODE
    WHERE r.person_id = person AND r.page_id = page AND r.POPLIST_ITEM_ID IS  NULL
    ) q
    inner join registrants rg ON q.person_id = rg.PERSON_ID AND q.page_id = rg.PAGE_ID
    WHERE rg.CANCEL IS NULL AND q.data_type_code IN ('FEE', 'FEE2')
    ORDER BY registration_id;

  CURSOR paid(reg_id NUMBER) IS
    SELECT p.PROCESSED
    FROM
    registrations_payments rp
    inner join
    payments p ON rp.PAYMENT_ID = p.PAYMENT_ID
    WHERE rp.REGISTRATION_ID = reg_id AND p.processed IS NOT NULL;


  descr VARCHAR(2000) DEFAULT NULL;
  msg VARCHAR(2000) DEFAULT NULL;
  found BOOLEAN := TRUE;
  regist_ID_type VARCHAR2(4);
  display_code VARCHAR2(4);
  display_fee INTEGER;
  total INTEGER := 0;
  processed 	payments.PROCESSED%TYPE DEFAULT NULL;
  tmsdate DATE;
  reg_id_prev NUMBER := -1;
  chk_exist BOOLEAN := FALSE;
  pid persons.PERSON_ID%TYPE;

  BEGIN

    IF crypt_pid IS  NOT NULL THEN
      pid := common.DECRYPT(crypt_pid);

      total := 0;
      tmsdate := SYSDATE;
      FOR driver_rec IN driver(this_page, pid) LOOP

	descr := NULL;
	msg := NULL;

	IF common.decode_value ('encrypt', 'meta_data_codes',
	  'meta_data_code', driver_rec.meta_data_code) = 'Y' THEN
	  driver_rec.value := common.decrypt (driver_rec.value);
	END IF;

	regist_ID_type := regist_ID_meta$;

/*
	Check how matrix fee works
	*/

	IF driver_rec.fee_matrix_id IS NOT NULL THEN
	  fee := nvl (driver_rec.fee, fee);
	  regist_ID_type := regist_ID_mtrx$;
	END IF;

	IF driver_rec.poplist_item_id IS NOT NULL THEN
	  regist_ID_type := regist_ID_popl$;
	END IF;

	OPEN paid(driver_rec.registration_id);
	FETCH paid INTO processed;
	CLOSE paid;

	tmsdate := nvl(processed, tmsdate);

	IF driver_rec.cutoff_date IS NULL OR tmsdate <= driver_rec.cutoff_date + common.grace_period$ + (common.tz_offset$ / 24)  THEN
	  fee := driver_rec.fee;
	ELSE
	  fee := nvl(driver_rec.late_fee, driver_rec.fee);
	END IF;

	IF driver_rec.display_code = disp_YN$ AND nvl (driver_rec.value, 'N') = 'N' THEN
	  fee := 0;
	END IF;


	IF(fee IS NULL) THEN
	  fee := 0;
	END IF;

	IF driver_rec.data_type_code = dataTfee2$ THEN
	  fee	:= driver_rec.value * fee;
	END IF;

	total := total + fee;

      END LOOP;

      RETURN total;
    ELSE
      RETURN 0;
    END IF;

  END get_fee_total_query;


/* get_fee_received
  calculates the amount of money received for a page or for a page and for one registrant
  summing up all payments but those canceled or waived
  */
  FUNCTION get_fee_received (this_page INTEGER DEFAULT NULL, crypt_pid VARCHAR2 DEFAULT NULL) RETURN NUMBER IS

  received payments.AMOUNT%TYPE DEFAULT NULL;

  CURSOR sum_per_page_per_person (page NUMBER, person NUMBER) IS
    SELECT SUM(amount)	FROM
    payments
    WHERE payment_id IN (SELECT DISTINCT rp.payment_id
    FROM registrations r
    inner join registrations_payments rp ON r.REGISTRATION_ID = rp.REGISTRATION_ID
    inner join registrants rg ON r.PAGE_ID = rg.PAGE_ID AND r.PERSON_ID = rg.PERSON_ID
    WHERE r.person_id = person AND r.page_id = page AND rg.CANCEL IS NULL)
    AND common.DECODE_VALUE('waived', 'payment_codes', 'payment_code', payments.PAYMENT_CODE) = 'N';

  CURSOR sum_per_page (page NUMBER) IS
    SELECT SUM(amount)	FROM
    payments
    WHERE payment_id IN (SELECT DISTINCT rp.payment_id
    FROM registrations r
    inner join registrations_payments rp ON r.REGISTRATION_ID = rp.REGISTRATION_ID
    inner join registrants rg ON r.PAGE_ID = rg.PAGE_ID AND r.PERSON_ID = rg.PERSON_ID
    WHERE r.page_id = page AND rg.CANCEL IS NULL)
    AND common.DECODE_VALUE('waived', 'payment_codes', 'payment_code', payments.PAYMENT_CODE) = 'N';

  BEGIN

    IF crypt_pid IS NULL THEN
      OPEN sum_per_page(this_page);
      FETCH sum_per_page INTO received;
      CLOSE sum_per_page;
    ELSE
      OPEN sum_per_page_per_person(this_page, common.DECRYPT(crypt_pid));
      FETCH sum_per_page_per_person INTO received;
      CLOSE sum_per_page_per_person;
    END IF;

    received := nvl(received, 0);

    RETURN received;

  END get_fee_received;


  FUNCTION get_fee_waived (this_page INTEGER DEFAULT NULL, crypt_pid VARCHAR2 DEFAULT NULL) RETURN NUMBER IS

  waived payments.AMOUNT%TYPE DEFAULT NULL;

  CURSOR sum_per_page_per_person (page NUMBER, person NUMBER) IS
    SELECT SUM(amount) FROM
    payments
    WHERE payment_id IN (SELECT DISTINCT rp.payment_id
    FROM registrations r
    inner join registrations_payments rp ON r.REGISTRATION_ID = rp.REGISTRATION_ID
    inner join registrants rg ON r.PAGE_ID = rg.PAGE_ID AND r.PERSON_ID = rg.PERSON_ID
    WHERE r.person_id = person AND r.page_id = page AND rg.CANCEL IS NULL)
    AND common.DECODE_VALUE('waived', 'payment_codes', 'payment_code', payments.PAYMENT_CODE) = 'Y';

  CURSOR sum_per_page (page NUMBER) IS
    SELECT SUM(amount) FROM
    payments
    WHERE payment_id IN (SELECT DISTINCT rp.payment_id
    FROM registrations r
    inner join registrations_payments rp ON r.REGISTRATION_ID = rp.REGISTRATION_ID
    inner join registrants rg ON r.PAGE_ID = rg.PAGE_ID AND r.PERSON_ID = rg.PERSON_ID
    WHERE r.page_id = page AND rg.CANCEL IS NULL)
    AND common.DECODE_VALUE('waived', 'payment_codes', 'payment_code', payments.PAYMENT_CODE) = 'Y';


  BEGIN
    IF crypt_pid IS NULL THEN
      OPEN sum_per_page(this_page);
      FETCH sum_per_page INTO waived;
      CLOSE sum_per_page;
    ELSE
      OPEN sum_per_page_per_person(this_page, common.DECRYPT(crypt_pid));
      FETCH sum_per_page_per_person INTO waived;
      CLOSE sum_per_page_per_person;
    END IF;

    waived := nvl(waived, 0);

    RETURN waived;

  END get_fee_waived;

-- *******************************************************************************************
-- *******************************************************************************************

  PROCEDURE process_payments(action VARCHAR, this_page INTEGER DEFAULT NULL, crypt_pid VARCHAR2 DEFAULT NULL,
  reg_id common.ident_arr, pay_id common.ident_arr,
  amt common.ident_arr, payment_type common.ident_arr) IS


  CURSOR payment (person INTEGER, page INTEGER) IS
    SELECT *
    FROM
    (
    SELECT r.registration_id, r.META_DATA_CODE, r.VALUE,
    r.POPLIST_ITEM_ID, r.FEE_MATRIX_ID,
    r.CUTOFF_DATE,  r.DATA_TYPE_CODE, r.DISPLAY_CODE, r.short_descr || ' : ' || pl.ITEM_VALUE short_descr, pl.FEE fee, pl.LATE_FEE late_fee
    FROM  (SELECT r1.registration_id, r1.META_DATA_CODE, r1.VALUE, r1.POPLIST_ITEM_ID, r1.FEE_MATRIX_ID,
    mdc.CUTOFF_DATE, mdc.DATA_TYPE_CODE, mdc.DISPLAY_CODE, mdc.SHORT_DESCR
    FROM registrations r1
    inner join meta_data_codes mdc ON mdc.META_DATA_CODE = r1.META_DATA_CODE
    WHERE r1.person_id = person AND  r1.PAGE_ID = page AND r1.POPLIST_ITEM_ID IS NOT NULL) r
    inner join poplist_items pl ON r.POPLIST_ITEM_ID = pl.POPLIST_ITEM_ID
    UNION
    SELECT r.registration_id, r.META_DATA_CODE, r.VALUE, r.POPLIST_ITEM_ID, r.FEE_MATRIX_ID,
    mdc.CUTOFF_DATE,  mdc.DATA_TYPE_CODE, mdc.DISPLAY_CODE, mdc.SHORT_DESCR, mdc.FEE, mdc.LATE_FEE
    FROM registrations r
    inner join meta_data_codes mdc ON mdc.META_DATA_CODE = r.META_DATA_CODE
    WHERE r.person_id = person AND r.PAGE_ID = page AND r.POPLIST_ITEM_ID IS  NULL
    ) q
    left join
    (SELECT rp.REGISTRATION_ID reg_id, rp.amount paid , p.*
    FROM registrations_payments rp
    inner join payments p ON rp.PAYMENT_ID = p.PAYMENT_ID
    )rrp ON q.REGISTRATION_ID = rrp.reg_ID
    WHERE  q.data_type_code IN ('FEE', 'FEE2')
    ORDER BY rrp.PAYMENT_ID nulls last, rrp.TIMESTAMP, rrp.PROCESSED nulls last;

  CURSOR due(person INTEGER, page INTEGER) IS
    SELECT *
    FROM
    (
    SELECT r.registration_id, r.META_DATA_CODE, r.VALUE,
    r.POPLIST_ITEM_ID, r.FEE_MATRIX_ID,
    r.CUTOFF_DATE,  r.DATA_TYPE_CODE, r.DISPLAY_CODE, r.short_descr || ' : ' || pl.ITEM_VALUE short_descr, pl.FEE fee, pl.LATE_FEE late_fee
    FROM  (SELECT r1.registration_id, r1.META_DATA_CODE, r1.VALUE, r1.POPLIST_ITEM_ID, r1.FEE_MATRIX_ID,
    mdc.CUTOFF_DATE, mdc.DATA_TYPE_CODE, mdc.DISPLAY_CODE, mdc.SHORT_DESCR
    FROM registrations r1
    inner join meta_data_codes mdc ON mdc.META_DATA_CODE = r1.META_DATA_CODE
    WHERE r1.person_id = person AND  r1.PAGE_ID = page AND r1.POPLIST_ITEM_ID IS NOT NULL) r
    inner join poplist_items pl ON r.POPLIST_ITEM_ID = pl.POPLIST_ITEM_ID
    UNION
    SELECT r.registration_id, r.META_DATA_CODE, r.VALUE, r.POPLIST_ITEM_ID, r.FEE_MATRIX_ID,
    mdc.CUTOFF_DATE,  mdc.DATA_TYPE_CODE, mdc.DISPLAY_CODE, mdc.SHORT_DESCR, mdc.FEE, mdc.LATE_FEE
    FROM registrations r
    inner join meta_data_codes mdc ON mdc.META_DATA_CODE = r.META_DATA_CODE
    WHERE r.person_id = person AND r.PAGE_ID = page AND r.POPLIST_ITEM_ID IS  NULL
    ) q
    left join
    (SELECT rp.REGISTRATION_ID reg_id, rp.amount paid , p.*
    FROM registrations_payments rp
    inner join payments p ON rp.PAYMENT_ID = p.PAYMENT_ID
    )rrp ON q.REGISTRATION_ID = rrp.reg_ID
    WHERE q.data_type_code IN ('FEE', 'FEE2')
    ORDER BY q.registration_id, rrp.processed nulls last ;


  payment_rec payment%ROWTYPE;
  due_rec due%ROWTYPE;
  rid_prev INTEGER;
  pmt_id payments.PAYMENT_ID%TYPE;
  refund payments%ROWTYPE;
  found BOOLEAN;
  tms_date DATE;
  tms TIMESTAMP;
  TYPE not_paid_rec IS RECORD(
  reg_id registrations.REGISTRATION_ID%TYPE,
  amt	 payments.AMOUNT%TYPE
  );
  TYPE table_not_paid IS TABLE OF not_paid_rec INDEX BY BINARY_INTEGER;
  not_paid 	table_not_paid;
  fee	 payments.AMOUNT%TYPE;
  received    payments.AMOUNT%TYPE;
  i INTEGER;
  query VARCHAR2(500);
  show_button BOOLEAN ;
  pid NUMBER;
  redirect 	VARCHAR2(200);
  BEGIN

    IF common.header (rpt_title$, cache_timeout => 15 / 1440, test_only => TRUE) THEN
      pid := common.DECRYPT(crypt_pid);
      tms_date := SYSDATE;
      IF action = 'show' THEN
	common.public_header (rpt_title$, cache_timeout => 15 / 1440);
	htp.header (4, 'Record Payment for ' || common.get_user_name (pid) || '<br />' ||
	common.get_user_affil_list (pid, '<br />'));
	htp.tableopen;
	htp.tablerowopen;
	htp.p('<td colspan="3">');
	htp.p('<hr />');
	htp.p('</td>');
	htp.tablerowclose;
	htp.tablerowopen;
	htp.tableheader('Fee description');
	htp.tableheader('Due');
	htp.tableheader(' ');
	htp.tablerowclose;
	rid_prev := -1;
	i := 1;
	FOR due_rec IN due(pid, this_page) LOOP
	  IF rid_prev != due_rec.registration_id THEN
	    rid_prev := due_rec.registration_id;
	    fee := get_fee_amt(due_rec.registration_id, tms_date);
	    /*
	    IF due_rec.processed IS NOT NULL THEN
	      fee := get_fee_amt(due_rec.registration_id, due_rec.processed, FALSE);
	    ELSE
	      fee := get_fee_amt(due_rec.registration_id, tms_date, TRUE);
	    END IF;
	    */
	    htp.tablerowopen;
	    htp.tabledata('<B>' || get_reg_descr(due_rec.registration_id) || ':&nbsp&nbsp </b>', 'RIGHT');
	    htp.tableData (fee);
	    htp.tabledata(htf.anchor('regist_maint.add_payment?reg_id=' || due_rec.registration_id	||
	    '&action=add&page=' || this_page, 'Add extra payment/Refund'));
	    htp.tablerowclose;

	    SELECT SUM(rp.amount) INTO received FROM
	    registrations_payments rp
	    WHERE rp.REGISTRATION_ID = due_rec.registration_id;
	    received := nvl(received, 0);

	    IF (fee - received) != 0 THEN
	      not_paid(i).reg_id := due_rec.registration_id;
	      not_paid(i).amt := fee - received;
	      i := i + 1;
	    END IF;
	  END IF;
	END LOOP;

	htp.tablerowopen;
	htp.p('<td colspan="3" valign="middle" align="center">');
	htp.formOpen ('regist_maint.new_item', 'POST');
	htp.formHidden('this_page', this_page);
	htp.formHidden ('crypt_pid', crypt_pid);


	redirect := 'regist_maint.process_payments?reg_id=dummy&action=show&this_page=' || this_page
	|| '&crypt_pid=' || crypt_pid || '&pay_id=dummy&amt=dummy&payment_type=dummy';

	redirect := REPLACE(redirect, '&', '|');
	htp.formhidden('redirect', redirect);
	htp.formsubmit (NULL, 'Record Miscellaneous Fee');
	htp.formclose;
	htp.p('<hr />');
	htp.p('</td>');
	htp.tablerowclose;

	htp.tableclose;


	htp.p('<br />');
	htp.formopen('regist_maint.process_payments', 'POST');
	htp.tableopen;
	htp.formhidden('this_page', this_page);
	htp.formhidden('crypt_pid', crypt_pid);
	htp.tableheader('Date of payment promise');
	htp.tableheader('Description');
	htp.tableheader('Paid');
	htp.tableheader('Notes');
	htp.tableheader(common.space$);
	htp.tableheader(common.space$);
	htp.tableRowOpen;
	htp.tabledata(common.space$, cattributes => 'colspan="5"');
	htp.tableRowclose;
	pmt_id := -1;
	show_button := FALSE;
	FOR payment_rec IN payment(pid, this_page) LOOP
	  htp.tablerowopen;
	  IF payment_rec.payment_id IS NOT NULL THEN
	    IF pmt_id <> payment_rec.payment_id THEN
	      htp.tableData(	common.local_time (payment_rec.timestamp), 'LEFT');
	      pmt_id := payment_rec.payment_id;
	    ELSE
	      htp.tabledata(common.space$);
	    END IF;
	    htp.tabledata('<B>' || get_reg_descr(payment_rec.registration_id) || ':&nbsp&nbsp </b>', 'RIGHT');
	    IF payment_rec.processed IS NOT NULL THEN
	      htp.tabledata(to_char(payment_rec.paid, common.currency_fmt$), 'RIGHT');
	      htp.tabledata(nvl(payment_rec.comments, common.space$));
	      htp.tableData(' Processed on ' ||
	      common.local_time (payment_rec.processed) || ' ' ||
	      common.DECODE_VALUE('payment_descr', 'payment_codes', 'payment_code', payment_rec.payment_code), 'LEFT');
	      htp.tableData('<input type="button" value="R"
		     onclick="javascript:window.location.href = ''regist_maint.reassign_payment?reg_id=' || payment_rec.registration_id ||
	      '&action=show&pmt_id=' || payment_rec.payment_id || '''"' );
	    ELSE
	      htp.formhidden('reg_id', payment_rec.registration_id);
	      htp.formhidden('pay_id', payment_rec.payment_id);
	      htp.formhidden('payment_type', 'dummy');
	      htp.tableData (htf.formtext('amt', cvalue => payment_rec.paid));
	      htp.tabledata(nvl(payment_rec.comments, common.space$));
	      htp.tableData(	common.local_time (payment_rec.timestamp) || ' ' ||
	      common.DECODE_VALUE('payment_descr', 'payment_codes', 'payment_code', payment_rec.payment_code), 'LEFT');
	      htp.tabledata(htf.anchor('regist_maint.process_payments' ||
	      '?reg_id=' || payment_rec.registration_id ||
	      '&action=delete&this_page=' || this_page ||
	      '&crypt_pid=' || crypt_pid ||
	      '&pay_id=' || payment_rec.payment_id ||
	      '&amt=dummy&payment_type=dummy', 'Delete promise of payment'));
	      show_button := TRUE;
	    END IF;
	  END IF;
	  htp.tablerowclose;
	END LOOP;
	IF not_paid.count>0 THEN
	  show_button := TRUE;
	  htp.tableRowOpen;
	  htp.tabledata(common.space$, cattributes => 'colspan="3"');
	  htp.tabledata('<b>Method of payment</b>');
	  htp.tableRowclose;
	  FOR i IN 1..not_paid.count LOOP
	    htp.tabledata(common.space$);
	    htp.tabledata('<B>' || get_reg_descr(not_paid(i).reg_id) || ':&nbsp&nbsp </b>', 'RIGHT');
	    htp.formhidden('reg_id', not_paid(i).reg_id);
	    htp.formhidden('pay_id', 'dummy');
	    htp.tableData (htf.formtext('amt', cvalue => not_paid(i).amt));
	    query := 'Select pc.PAYMENT_CODE code, pc.PAYMENT_DESCR descr
		from
		  pages p
		    inner join
		  payment_codes_page pcp on p.PAGE_ID=pcp.PAGE_ID
		    inner join
		  payment_codes pc on pc.PAYMENT_CODE=pcp.PAYMENT_CODE
		where pcp.PAGE_ID=' || this_page ;
	    IF NOT common.has_privs_to_page (common.reg_admin_page$, FALSE) THEN
	      query := query || ' and pc.online_option = ''Y''';
	    END IF;
	    htp.p('<td align="LEFT">');
	    registration.show_popup_table_payments (query, 'payment_type', NULL, NULL, NULL, NULL, TRUE, FALSE);
	    htp.p('</td>');
	    htp.tablerowclose;
	  END LOOP;
	END IF;
	htp.tableRowOpen;
	htp.tabledata(common.space$);
	IF show_button THEN
	  Htp.tabledata('<br />' || htf.formsubmit('action', 'save'), 'CENTER');
	ELSE
	  htp.tabledata(common.space$);
	END IF;
	htp.tabledata('<br />' || htf.formreset('Cancel', cattributes => 'onClick="javascript:window.close();"'), 'CENTER');
	htp.tableRowclose;
	htp.tableRowOpen;
	htp.tabledata('<Br /><B>Some instructions</b><Br /><Br />' ||
	' - The value in the text boxes represent the actual paid that will be processed<br />' ||
	' - If you don''t choose any method of payment the the fee will not be processed<Br />' ||
	' - You can refund any amount by adding an extra negative payment<Br />', cattributes => 'colspan="3"' );
	htp.tableRowclose;
	htp.tableclose;
	htp.formclose;
	common.footer;
      ELSIF action = 'save' THEN
	SELECT systimestamp INTO tms FROM dual;
	FOR i IN 1..pay_id.count LOOP
	  IF pay_id(i) <> 'dummy' THEN
	    UPDATE registrations_payments SET amount = amt(i) WHERE registration_id = reg_id(i) AND payment_id = pay_id(i);
	    UPDATE payments SET amount = amount + amt(i), processed = tms WHERE payment_id = pay_id(i);
	  END IF;
	END LOOP;
	FOR i IN 1..pay_id.count LOOP
	  IF pay_id(i) = 'dummy' AND payment_type(i) != 'dummy' THEN
	    SELECT payment_id_seq.nextval INTO pmt_id FROM dual;
	    INSERT INTO payments (payment_id, person_id, amount, payment_code, processed)
	    VALUES (pmt_id, pid, amt(i), payment_type(i), tms);
	    INSERT INTO registrations_payments (registration_id, payment_id, amount)
	    VALUES (reg_id(i), pmt_id, amt(i));
	  END IF;
	END LOOP;
	common.redirect_URL (common.sserver || common.vdir$ || 'regist_maint.process_payments' ||
	'?reg_id=dummy&action=show&this_page=' || this_page ||
	'&crypt_pid=' || crypt_pid ||
	'&pay_id=dummy&amt=dummy&payment_type=dummy');
      ELSIF action = 'delete' THEN
	SELECT COUNT( * ) INTO i FROM registrations_payments WHERE payment_id = pay_id(1);
	DELETE FROM registrations_payments WHERE registration_id = reg_id(1) AND payment_id = pay_id(1);
	IF i = 1 THEN
	  DELETE FROM payments WHERE payment_id = pay_id(1);
	END IF;
	common.redirect_URL (common.sserver || common.vdir$ || 'regist_maint.process_payments' ||
	'?reg_id=dummy&action=show&this_page=' || this_page ||
	'&crypt_pid=' || crypt_pid ||
	'&pay_id=dummy&amt=dummy&payment_type=dummy');
      ELSE
	NULL;
      END IF;
    END IF;
  END process_payments;

  PROCEDURE add_payment (action VARCHAR2, amt payments.AMOUNT%TYPE DEFAULT NULL, reg_id registrations.REGISTRATION_ID%TYPE,
  page registrations.PAGE_ID%TYPE, note payments.COMMENTS%TYPE DEFAULT NULL , pmt_type VARCHAR2 DEFAULT NULL) IS

  no_payment_type EXCEPTION;
  pmt_id payments.PAYMENT_ID%TYPE;
  person registrations.PERSON_ID%TYPE;
  tms DATE;
  query VARCHAR2(500);

  BEGIN
    IF common.header ('Add extra payment / Refund', test_only => TRUE) THEN
      IF action = 'add' THEN
	common.public_header ('Add extra payment / Refund');
	htp.p('<br />');
	htp.formopen('', '');
	htp.formhidden('reg_id', reg_id);
	htp.formhidden('page', page);
	htp.tableopen;
	htp.tablerowopen;
	htp.tableheader('Description');
	Htp.tableheader('Extra Amount');
	Htp.tableheader('Method of Payment');
	htp.tableheader('Comment');
	htp.tablerowclose;
	htp.tablerowopen;
	htp.tabledata(common.space$);
	htp.tablerowclose;
	htp.tableRowOpen;
	htp.tabledata('<b>' || get_reg_descr(reg_id) || '</b>');
	htp.tabledata(htf.formtext('amt'));
	query := 'Select pc.PAYMENT_CODE code, pc.PAYMENT_DESCR descr
					from
						pages p
							inner join
						payment_codes_page pcp on p.PAGE_ID=pcp.PAGE_ID
							inner join
						payment_codes pc on pc.PAYMENT_CODE=pcp.PAYMENT_CODE
					where pcp.PAGE_ID=' || page ;
	IF NOT common.has_privs_to_page (common.reg_admin_page$, FALSE) THEN
	  query := query || ' and pc.online_option = ''Y''';
	END IF;
	htp.p('<td>');
	registration.show_popup_table_payments(query, 'pmt_type', NULL, NULL, NULL, NULL, TRUE, FALSE);
	htp.p('</td>');
	htp.tabledata(htf.formtext('note', '50'));
	htp.tablerowclose;
	htp.tableRowOpen;
	htp.tabledata(common.space$);
	Htp.tabledata('<br />' || htf.formsubmit('action', 'save'));
	htp.tabledata('<br />' || htf.formreset('Cancel', cattributes => 'onClick="javascript:history.back();"'));
	htp.tableRowclose;
	htp.tableclose;
	htp.formclose;
	common.footer;
      ELSIF action = 'save' THEN
	IF pmt_type IS NOT NULL AND pmt_type != 'dummy' THEN
	  tms := SYSDATE;
	  SELECT person_id INTO person FROM registrations WHERE registration_id = reg_id;
	  SELECT payment_id_seq.nextval INTO pmt_id FROM dual;
	  INSERT INTO payments (payment_id, person_id, amount, payment_code, processed, comments)
	  VALUES (pmt_id, person, amt, pmt_type, tms, note);
	  INSERT INTO registrations_payments (registration_id, payment_id, amount)
	  VALUES (reg_id, pmt_id, amt);
	  common.redirect_URL (common.sserver || common.vdir$ || 'regist_maint.process_payments' ||
	  '?reg_id=dummy&action=show&this_page=' || page ||
	  '&crypt_pid=' || common.ENCRYPT(person) ||
	  '&pay_id=dummy&amt=dummy&payment_type=dummy');
	ELSE
	  RAISE no_payment_type;
	END IF;
      END IF;
    END IF;
  EXCEPTION
    WHEN no_payment_type THEN
      common.public_header ('Add extra payment / Refund');
      htp.p('<br />');
      htp.p(htf.anchor ('#', htf.IMG(common.prevImg$, cattributes => 'border=0 alt="Back" title="Back"'), cattributes => 'onclick="history.back(-1);return false;"'));
      htp.p ('<br /> You must select a payment method<br /><br />');
      htp.bold ('Please click the back button to correct this error.');
      common.footer;
  END add_payment;


  FUNCTION need_input(meta meta_data_codes.META_DATA_CODE%TYPE) RETURN BOOLEAN	IS

  disp meta_data_codes.DISPLAY_CODE%TYPE;

  BEGIN

    disp := common.DECODE_VALUE('display_code', 'meta_data_codes', 'meta_data_code', meta);
    IF disp IN (disp_multi$, disp_YN$, disp_txt$) THEN
      RETURN TRUE;
    ELSE
      RETURN FALSE;
    END IF;

  END need_input;

  PROCEDURE input_request(action VARCHAR2, reg_id NUMBER, pmt_id NUMBER, amount_to_move NUMBER DEFAULT 0,
  meta meta_data_codes.META_DATA_CODE%TYPE, poplist poplist_items.POPLIST_ITEM_ID%TYPE ,
  page pages.PAGE_ID%TYPE, crypt_pid VARCHAR2, choice registrations.VALUE%TYPE DEFAULT NULL) IS

  disp meta_data_codes.DISPLAY_CODE%TYPE;
  descr 	VARCHAR2(50) DEFAULT NULL;

  CURSOR field (meta NUMBER, page NUMBER) IS
    SELECT pd.TEXT, mdc.META_DATA_DESCR, mdc.DISPLAY_CODE, mdc.DATA_TYPE_CODE,
    mdc.DATA_LENGTH, mdc.VIEW_LENGTH
    FROM page_data pd
    inner join
    meta_data_codes mdc ON pd.META_DATA_CODE = mdc.META_DATA_CODE
    WHERE pd.page_id = page AND pd.meta_data_code = meta;

  field_rec field%ROWTYPE;
  ilen CONSTANT INTEGER := length(common.date_input_fmt_str$);
  no_value EXCEPTION;
  new_reg_id registrations.REGISTRATION_ID%TYPE;
  amt  payments.AMOUNT%TYPE;
  person persons.PERSON_ID%TYPE := common.DECRYPT(crypt_pid);

  BEGIN

    IF action = 'show' THEN
      IF common.header ('Reassign payment', test_only => TRUE) THEN
	common.public_header ('Reassign payment');
	jsfunctions;
	htp.p('<br />');
	htp.tableopen;
	htp.formopen('regist_maint.input_request', 'POST');
	htp.tablerowopen;
	htp.tableheader('Description');
	Htp.tableheader('Amount to reassign');
	htp.tableheader('Fees');
	htp.tablerowclose;
	htp.tablerowopen;
	htp.tabledata('<b>' || get_reg_descr(reg_id) || '</b>');
	htp.tabledata(amount_to_move);
	htp.formhidden('reg_id', reg_id);
	htp.formhidden('pmt_id', pmt_id);
	htp.formhidden('amount_to_move', amount_to_move);
	htp.formhidden('meta', meta);
	htp.formhidden('poplist', poplist);
	htp.formhidden('page', page);
	htp.formhidden('crypt_pid', crypt_pid);
	descr := common.DECODE_VALUE('item_value', 'poplist_items', 'poplist_item_id', poplist);
	IF descr IS NOT NULL THEN
	  descr := common.DECODE_VALUE('short_descr', 'meta_data_codes', 'meta_data_code', meta) || ': ' || descr;
	ELSE
	  descr := common.DECODE_VALUE('short_descr', 'meta_data_codes', 'meta_data_code', meta);
	END IF;
	htp.tabledata(descr);
	htp.tableRowclose;
	htp.tablerowopen;
	htp.tabledata(common.space$, ccolspan => 3);
	htp.tableRowclose;
	htp.tablerowopen;
	OPEN field(meta, page);
	FETCH field INTO field_rec;
	CLOSE field;
	htp.tabledata(common.space$);
	htp.tabledata(field_rec.text);
	htp.tabledata(common.space$);
	htp.tableRowclose;
	htp.tablerowopen;
	htp.tabledata('<b>' || field_rec.meta_data_descr || '</b>');
	IF field_rec.display_code = disp_multi$ THEN
	  htp.p('<td>');
	  htp.formtextareaopen('choice', 3, 50);
	  htp.formtextareaclose;
	  htp.p('</td>');
	ELSIF field_rec.display_code = disp_YN$ THEN
	  htp.p('<td>');
	  htp.p(common.popup_yes_no('choice''choice', NULL, TRUE));
	  htp.p('</td>');
	ELSIF field_rec.display_code = disp_txt$ THEN
	  IF field_rec.data_type_code = dataTdte$ THEN
	    htp.tabledata(htf.formtext('choice', ilen, ilen));
	  ELSE
	    htp.tabledata(htf.formtext('choice', field_rec.view_length, field_rec.data_length));
	  END IF;
	END IF;
	htp.tabledata(common.space$);
	htp.tableRowclose;
	htp.tablerowopen;
	htp.tabledata(common.space$, ccolspan => 3);
	htp.tableRowclose;
	htp.tabledata(htf.formsubmit('action', 'save') || common.space$ || common.space$ ||
	htf.formreset('Cancel', 'onclick="document.location.href=''' ||
	'regist_maint.process_payments?reg_id=dummy&action=show' ||
	'&this_page=' || page ||
	'&crypt_pid=' || crypt_pid ||
	'&pay_id=dummy&amt=dummy&payment_type=dummy'';"'), 'CENTER', ccolspan => 3);
	htp.tableRowclose;
	htp.formclose;
	htp.tableclose;
	common.footer;
      END IF;
    ELSIF action = 'save' THEN
      IF choice IS NULL THEN
	RAISE no_value;
      END IF;
      IF poplist = 0 THEN
	INSERT INTO registrations (person_id, meta_data_code, TIMESTAMP, page_id, VALUE)
	VALUES (person, meta, SYSDATE, page, choice);
      ELSE
	INSERT INTO registrations (person_id, meta_data_code, poplist_item_id, TIMESTAMP, page_id, VALUE)
	VALUES (person, meta, poplist, SYSDATE, page, choice);
      END IF;
      SELECT MAX(registration_id) INTO new_reg_id FROM registrations WHERE person_id = person AND page_id = page;

      INSERT INTO registrations_payments (registration_id, payment_id, amount)
      VALUES (new_reg_id, pmt_id, amount_to_move);

      SELECT amount INTO amt FROM registrations_payments WHERE payment_id = pmt_id AND registration_id = reg_id;

      IF amt - amount_to_move = 0 THEN
	DELETE FROM registrations_payments
	WHERE registration_id = reg_id AND payment_id = pmt_id;
      ELSE
	UPDATE registrations_payments SET amount = amt - amount_to_move
	WHERE registration_id = reg_id AND payment_id = pmt_id;
      END IF;
      common.redirect_URL (common.sserver || common.vdir$ || 'regist_maint.process_payments' ||
      '?reg_id=dummy&action=show&this_page=' || page ||
      '&crypt_pid=' || crypt_pid ||
      '&pay_id=dummy&amt=dummy&payment_type=dummy');
    END IF;

  EXCEPTION
    WHEN no_value THEN
      common.public_header ('Reassign payment - Error');
      htp.p('<br />');
      htp.p(htf.anchor ('#', htf.IMG(common.prevImg$, cattributes => 'border=0 alt="Back" title="Back"'),
      cattributes => 'onclick="history.back(-1);return false;"'));
      htp.p('<B><h3>You must fill the data entry</h3></B>');
      common.footer;

  END input_request;


  PROCEDURE reassign_payment (action VARCHAR2, reg_id NUMBER, pmt_id NUMBER, amount_to_move NUMBER DEFAULT 0, fee_id VARCHAR2 DEFAULT NULL) IS

  amt  payments.AMOUNT%TYPE;
  page registrations.PAGE_ID%TYPE := common.DECODE_VALUE('page_id', 'registrations', 'registration_id', reg_id);
  person registrations.PERSON_ID%TYPE := common.DECODE_VALUE('person_id', 'registrations', 'registration_id', reg_id);
  crypt_pid VARCHAR2(32767) := common.ENCRYPT(person);
  meta meta_data_codes.META_DATA_CODE%TYPE := common.DECODE_VALUE('meta_data_code', 'registrations', 'registration_id', reg_id);
  poplist poplist_items.POPLIST_ITEM_ID%TYPE := nvl(common.DECODE_VALUE('poplist_item_id', 'registrations', 'registration_id', reg_id), 0);

  CURSOR fees(personid NUMBER, pageid NUMBER) IS
    SELECT pd.PAGE_ID, mdc.META_DATA_CODE, 0 poplist_item_id, mdc.DISPLAY_CODE, mdc.DATA_TYPE_CODE,
    mdc.short_descr descr
    FROM page_data pd
    inner join meta_data_codes mdc ON pd.META_DATA_CODE = mdc.META_DATA_CODE
    WHERE pd.page_id = pageid AND mdc.DATA_TYPE_CODE IN ('FEE', 'FEE2') AND mdc.DISPLAY_CODE NOT IN ('POP', 'CHK')
    AND NOT EXISTS (SELECT *
    FROM registrations r
    inner join meta_data_codes mdc1 ON r.META_DATA_CODE = mdc1.META_DATA_CODE
    WHERE mdc1.SINGLE_SELECTION = 'Y' AND r.META_DATA_CODE = mdc.META_DATA_CODE AND r.PAGE_ID = pageid
    AND r.person_id <> personid)
    UNION
    SELECT pd.PAGE_ID, mdc.META_DATA_CODE, pl.POPLIST_ITEM_ID, mdc.DISPLAY_CODE, mdc.DATA_TYPE_CODE,
    mdc.short_descr || ':' || pl.ITEM_VALUE descr
    FROM page_data pd
    inner join meta_data_codes mdc ON pd.META_DATA_CODE = mdc.META_DATA_CODE
    inner join poplist_items pl ON mdc.META_DATA_CODE = pl.META_DATA_CODE
    WHERE pd.page_id = pageid AND mdc.DATA_TYPE_CODE IN ('FEE', 'FEE2') AND mdc.DISPLAY_CODE IN ('POP', 'CHK')
    AND NOT EXISTS (SELECT *
    FROM registrations r
    inner join meta_data_codes mdc1 ON r.META_DATA_CODE = mdc1.META_DATA_CODE
    WHERE mdc1.SINGLE_SELECTION = 'Y' AND r.POPLIST_ITEM_ID = pl.poplist_item_id AND r.PAGE_ID = pageid
    AND r.person_id <> personid);
  too_much 	EXCEPTION;
  fee_empty   EXCEPTION;
  input_req 	EXCEPTION;

  CURSOR driver (meta NUMBER, page NUMBER, person NUMBER) IS
    SELECT registration_id
    FROM registrations
    WHERE meta_data_code = meta AND person_id = person AND page_id = page;

  CURSOR driver1 (meta NUMBER, poplist NUMBER, page NUMBER, person NUMBER) IS
    SELECT registration_id
    FROM registrations
    WHERE meta_data_code = meta AND poplist_item_id = poplist AND person_id = person AND page_id = page;

  CURSOR reg_pmt(reg NUMBER, pmt NUMBER) IS
    SELECT 1
    FROM registrations_payments
    WHERE registration_id = reg AND payment_id = pmt;

  dummy	NUMBER;
  found 	BOOLEAN;
  found_rp BOOLEAN;
  new_reg_id registrations.REGISTRATION_ID%TYPE;
  descr 	VARCHAR2(50) DEFAULT NULL;

  BEGIN
    IF common.header ('Reassign payment', test_only => TRUE) THEN
      SELECT amount INTO amt FROM registrations_payments WHERE payment_id = pmt_id AND registration_id = reg_id;
      IF action = 'show' THEN
	common.public_header ('Reassign payment');
	jsfunctions;
	htp.p('<br />');
	htp.tableopen;
	htp.formopen('regist_maint.REASSIGN_PAYMENT', 'POST', cattributes => 'name="reassign"');
	htp.tablerowopen;
	htp.tableheader('Description');
	Htp.tableheader('Amount to reassign');
	htp.tableheader('Fees');
	htp.tablerowclose;
	htp.tablerowopen;
	htp.tabledata('<b>' || get_reg_descr(reg_id) || '</b>');
	htp.tabledata(htf.formtext('amount_to_move', cvalue => amt));
	htp.p('<TD>');
	htp.FORMSELECTOPEN('fee_id');
	htp.formselectoption('', 'SELECTED', 'value="dummy"');
	FOR fees_rec IN fees(person, page) LOOP
	  IF (meta != fees_rec.meta_data_code AND poplist = 0) OR (poplist != fees_rec.poplist_item_id) THEN
	    htp.formselectoption(fees_rec.descr, NULL, 'value="' || fees_rec.meta_data_code || '/' || fees_rec.poplist_item_id || '"');
	  END IF;
	END LOOP;
	HTP.FORMSELECTCLOSE;
	htp.p('</TD>');
	htp.tableRowclose;
	htp.tablerowopen;
	htp.tabledata(common.space$, ccolspan => 3);
	htp.tableRowclose;
	htp.tablerowopen;
	htp.tabledata('-  If you want to reassign only a part of the paid amount you can <b>reduce</b> the amount value', ccolspan => 3);
	htp.tableRowclose;
	htp.tablerowopen;
	htp.tabledata(common.space$, ccolspan => 3);
	htp.tableRowclose;
	htp.formhidden('reg_id', reg_id);
	htp.formhidden('pmt_id', pmt_id);
	htp.tabledata(htf.formsubmit('action', 'save') || common.space$ || common.space$ ||
	htf.formreset('Cancel', 'onclick="document.location.href=''' ||
	'regist_maint.process_payments?reg_id=dummy&action=show' ||
	'&this_page=' || page ||
	'&crypt_pid=' || crypt_pid ||
	'&pay_id=dummy&amt=dummy&payment_type=dummy'';"'), 'CENTER', ccolspan => 3);
	htp.tableRowclose;
	htp.formclose;
	htp.tableclose;
	common.footer;
      ELSIF action = 'save' THEN
	IF amount_to_move>amt THEN
	  RAISE too_much;
	ELSIF fee_id = 'dummy' THEN
	  RAISE fee_empty;
	END IF;
	meta := to_number(common.stringtokenizer(fee_id, 1 , '/'));
	poplist := to_number(common.stringtokenizer(fee_id, 2 , '/'));
	IF poplist = 0 THEN
	  OPEN driver(meta, page, person);
	  FETCH driver INTO new_reg_id;
	  found := driver%FOUND;
	  CLOSE driver;
	ELSE
	  OPEN driver1(meta, poplist, page, person);
	  FETCH driver1 INTO new_reg_id;
	  found := driver1%FOUND;
	  CLOSE driver1;
	END IF;
	OPEN reg_pmt(new_reg_id, pmt_id);
	IF NOT found THEN
	  IF need_input(meta) = FALSE THEN
	    IF poplist = 0 THEN
	      INSERT INTO registrations (person_id, meta_data_code, TIMESTAMP, page_id)
	      VALUES (person, meta, SYSDATE, page);
	    ELSE
	      INSERT INTO registrations (person_id, meta_data_code, poplist_item_id, TIMESTAMP, page_id)
	      VALUES (person, meta, poplist, SYSDATE, page);
	    END IF;
	    SELECT MAX(registration_id) INTO new_reg_id FROM registrations WHERE person_id = person AND page_id = page;
	  ELSE
	    RAISE input_req;
	  END IF;
	ELSE
	  FETCH reg_pmt INTO dummy;
	  found_rp := reg_pmt%FOUND;
	  CLOSE reg_pmt;
	END IF;

	IF found_rp THEN
	  UPDATE registrations_payments SET amount = amount + amount_to_move
	  WHERE registration_id = new_reg_id AND payment_id = pmt_id;
	ELSE
	  INSERT INTO registrations_payments (registration_id, payment_id, amount)
	  VALUES (new_reg_id, pmt_id, amount_to_move);
	END IF;
	IF amt - amount_to_move = 0 THEN
	  DELETE FROM registrations_payments
	  WHERE registration_id = reg_id AND payment_id = pmt_id;
	ELSE
	  UPDATE registrations_payments SET amount = amt - amount_to_move
	  WHERE registration_id = reg_id AND payment_id = pmt_id;
	END IF;
	common.redirect_URL (common.sserver || common.vdir$ || 'regist_maint.process_payments' ||
	'?reg_id=dummy&action=show&this_page=' || page ||
	'&crypt_pid=' || crypt_pid ||
	'&pay_id=dummy&amt=dummy&payment_type=dummy');

      END IF;
    END IF;

  EXCEPTION
    WHEN too_much THEN
      common.public_header ('Reassign payment - Error');
      htp.p(htf.anchor ('#', htf.IMG(common.prevImg$, cattributes => 'border=0 alt="Back" title="Back"'),
      cattributes => 'onclick="javascript:window.location.href = ''regist_maint.reassign_payment?' ||
      'reg_id=' ||  reg_id ||
      '&action=show&pmt_id=' || pmt_id ||
      '''"' ));
      htp.p('<B><h3>The amount that you want to move is greater that the paid one</h3></B>');
      common.footer;
    WHEN fee_empty THEN
      common.public_header ('Reassign payment - Error');
      htp.p(htf.anchor ('#', htf.IMG(common.prevImg$, cattributes => 'border=0 alt="Back" title="Back"'),
      cattributes => 'onclick="javascript:window.location.href = ''regist_maint.reassign_payment?' ||
      'reg_id=' ||  reg_id ||
      '&action=show&pmt_id=' || pmt_id ||
      '''"' ));
      htp.p('<B><h3>You have to select a fee</h3></B>');
      common.footer;
    WHEN VALUE_ERROR THEN
      common.public_header ('Reassign payment - Error');
      htp.p(htf.anchor ('#', htf.IMG(common.prevImg$, cattributes => 'border=0 alt="Back" title="Back"'),
      cattributes => 'onclick="javascript:window.location.href = ''regist_maint.reassign_payment?' ||
      'reg_id=' ||  reg_id ||
      '&action=show&pmt_id=' || pmt_id ||
      '''"' ));
      htp.p('<B><h3>Generic error - try again</h3></B>');
      common.footer;
    WHEN input_req THEN
      input_request('show', reg_id, pmt_id, amount_to_move, meta, poplist, page, crypt_pid);


  END reassign_payment;


-- *******************************************************************************************
-- *******************************************************************************************

  PROCEDURE process_baldue IS

  CURSOR person IS
    SELECT r.*, p.LNAME lname, p.FNAME fname
    FROM registrants r
    inner join persons p ON r.PERSON_ID = p.PERSON_ID
    ORDER BY p.lname, p.FNAME, r.PAGE_ID;

  person_rec person%ROWTYPE;

  fee payments.AMOUNT%TYPE;
  received Payments.AMOUNT%TYPE;
  waived Payments.AMOUNT%TYPE;
  dummy 	BOOLEAN;
  crypt_pid VARCHAR2(32767);
  BEGIN

    IF NOT common.secure_login THEN
      common.redirect_URL (common.sserver || common.vdir$ || 'registration.process_baldue');
    ELSE
      common.get_gbl_pid;
      IF common.has_privs_to_page (common.reg_admin_page$) THEN
	maint.header (rpt_title2$ || ' Balance Due');
	jsFunctions;
	htp.tableOpen();
	htp.tableRowOpen();
	htp.p('<td width=25%>');
	maint.menu(0);
	htp.p('</td>');
	htp.p('<td>');
	htp.tableOpen ('BORDER');
	htp.tableRowOpen ;
	htp.tableHeader (common.space$);
	htp.tableHeader ('Due');
	htp.tableHeader ('Name');
	htp.tableHeader ('Registration form');
	htp.tableHeader ('Invoice');
	htp.tableRowClose;
	FOR person_rec IN person LOOP
	  crypt_pid := common.ENCRYPT(person_rec.person_id);
	  fee := get_fee_total(person_rec.page_id, crypt_pid, FALSE, dummy);
	  received := get_fee_received (person_rec.page_id, crypt_pid);
	  waived := get_fee_waived (person_rec.page_id, crypt_pid);
	  IF (fee - received - waived) != 0 THEN
	    htp.tableRowOpen ('LEFT', 'TOP');
	    htp.tableData (htf.anchor ('javascript:popUp(''regist_maint.process_payments'	||
	    '?reg_id=dummy&action=show&this_page=' || person_rec.page_id ||
	    '&crypt_pid=' || crypt_pid ||
	    '&pay_id=dummy&amt=dummy&payment_type=dummy' ||
	    ''')', 'Process'));

	    htp.tableData (to_char (fee - received - waived, common.currency_fmt$), 'RIGHT');

	    htp.tableData (htf.anchor2 ('regist_maint.show_profile?crypt_pid=' || crypt_pid,
	    person_rec.lname || ' ' || person_rec.fname, ctarget => 'SPMS'));

	    htp.tableData (htf.anchor2 ('regist_maint.show_registration_page?' ||
	    'ID=' || person_rec.page_id ||
	    '&crypt_pid=' || crypt_pid ,
	    common.DECODE_VALUE('page_name', 'pages', 'page_id', person_rec.page_id),
	    ctarget => 'SPMS'), cattributes => 'NOWRAP');


	    htp.tableData(htf.anchor ('registration.invoice2?this_page=' || person_rec.page_id ||
	    '&crypt_pid=' || crypt_pid,
	    htf.IMG(common.editImg$, cattributes => 'border=0 alt=Edit title=Edit'),
	    cattributes => 'target="_blank"'));

	    htp.tableRowClose;
	  END IF;
	END LOOP;
	htp.tableClose;
	htp.p('</td>');
	htp.tableRowClose;
	htp.tableClose;
	htp.para;
	common.footer;
      END IF;
    END IF;

  END process_baldue;

-- *******************************************************************************************
-- *******************************************************************************************

  PROCEDURE notices IS

  BEGIN

    common.get_gbl_pid;
    IF common.has_privs_to_page (common.reg_admin_page$) THEN
      maint.header(rpt_title5$);
      htp.tableOpen();
      htp.tableRowOpen();
      htp.p('<td width=25%>');
      maint.menu(0);
      htp.p('</td>');
      htp.p('<td>');

      htp.formOpen ('regist_maint.send_notices', 'POST');
      htp.header (3, 'Send registration email notices to unpaid balances.');

      htp.formRadio ('flag', notices_1st$, 'SELECTED');
      htp.p ('Send 1st email notice only.');
      htp.br;

      htp.formRadio ('flag', notices_all$);
      htp.p ('Send email notice to all unpaid balances.');
      htp.para;

      htp.formSubmit (NULL, 'Send Mail');
      htp.formClose;
      htp.para;
      htp.p('</td>');
      htp.tableRowclose();
      htp.tableclose();
      common.footer;
    END IF;

  END notices;

-- *******************************************************************************************
-- *******************************************************************************************

  FUNCTION load_email_msgs (pid INTEGER, delay IN OUT INTEGER) RETURN VARCHAR2 IS

  CURSOR	driver IS
    SELECT	email_msg, nvl (email_delay, 0) email_delay
    FROM	payment_codes
    WHERE	payment_code IN
    (	SELECT	payment_code
    FROM	payments
    WHERE	person_id	= pid
    AND	processed	IS NULL
    AND	reversed	IS NULL)
    AND	email_msg	IS NOT NULL;

  msg	LONG := NULL;

  BEGIN

    delay := 2; -- wait at least two days

    FOR driver_rec IN driver LOOP
      msg	:= msg || common.eol2$ || driver_rec.email_msg;
      delay	:= greatest (delay, driver_rec.email_delay);
    END LOOP;

    msg := substr (msg, 3);
    RETURN nvl (msg, common.no_payment_email_msg$);

  END load_email_msgs;

-- *******************************************************************************************

  FUNCTION send_notices2 (flag INTEGER) RETURN INTEGER IS

  CURSOR	driver IS
    SELECT	r.rowid, r.*, p.admin_email
    FROM	pages		p,
    registrants	r
    WHERE (
    (r.email_sent IS NULL AND flag = notices_1st$ ) OR flag = notices_all$
    )
    AND	regist_maint.balancedue (common.ENCRYPT(r.person_id), p.page_id ) > 0
    AND	p.page_id = r.page_id;

  now$		CONSTANT DATE := common.get_UTC_time;

  icount		INTEGER	:= 0;
  delay		INTEGER;
  msg		    mail_blats.message%TYPE;
  recipient	mail_blats.recipient%TYPE;

  BEGIN

    FOR driver_rec IN driver LOOP

      recipient := common.get_user_email (driver_rec.person_id);

      IF recipient IS NOT NULL AND driver_rec.last_update < now$ THEN
	icount			:= icount + 1;
	msg			:= load_email_msgs (driver_rec.person_id, delay);
	driver_rec.last_update := driver_rec.last_update + delay;

	IF msg IS NOT NULL AND driver_rec.last_update < now$ THEN
	  send_mail (recipient, subject$, msg, driver_rec.admin_email, pid => driver_rec.person_id);

	  UPDATE registrants
	  SET	   email_sent	= now$
	  WHERE  ROWID	= driver_rec.rowid;
	END IF;

      END IF;

    END LOOP;

    RETURN icount;

  END send_notices2;

-- *******************************************************************************************

  PROCEDURE send_notices (flag INTEGER) IS

  icount		INTEGER := 0;

  BEGIN

    IF common.header (rpt_title5$) THEN
      icount := send_notices2 (flag);
      htp.header (3, icount || ' Emails Sent');
      htp.p (back2_pay_inv$);
      common.footer;
    END IF;

  END send_notices;

-- *******************************************************************************************
-- *******************************************************************************************

  PROCEDURE auto_notices IS

  icount	INTEGER;

  BEGIN

    icount := regist_maint.send_notices2 (notices_1st$);

  END auto_notices;

-- *******************************************************************************************
-- *******************************************************************************************

  PROCEDURE show_profile (crypt_pid VARCHAR2) IS

  BEGIN

    common.get_gbl_pid;
    common.gbl_pid := common.DECRYPT(crypt_pid);
    profile.manage;

  END show_profile;

-- *******************************************************************************************
-- *******************************************************************************************

  FUNCTION get_status (ID INTEGER, flag INTEGER) RETURN BOOLEAN IS

  CURSOR	driver IS
    SELECT	*
    FROM	pages
    WHERE	page_id = ID;

  driver_rec	driver%ROWTYPE;

  BEGIN

    OPEN driver;
    FETCH driver INTO driver_rec;
    CLOSE driver;

    IF common.has_privs_to_page (common.reg_admin_page$, FALSE) THEN
      RETURN TRUE;
    END IF;

    IF flag = is_available$ THEN
      IF driver_rec.available = 'Y' THEN
	RETURN TRUE;
      END IF;

    ELSIF flag = login_required$ THEN
      IF driver_rec.login_req = 'Y' THEN
	RETURN TRUE;
      END IF;

    ELSIF flag = admin_required$ THEN
      IF driver_rec.admin_privs = 'Y' THEN
	RETURN TRUE;
      END IF;

    ELSIF flag = show_on_profile$ THEN
      IF driver_rec.show_on_profile = 'Y' THEN
	RETURN TRUE;
      END IF;

    ELSIF flag = show_on_contrib$ THEN
      IF driver_rec.show_on_abstract = 'Y' THEN
	RETURN TRUE;
      END IF;
    END IF;

    RETURN FALSE;

  END get_status;

-- *******************************************************************************************
-- *******************************************************************************************

  FUNCTION trimall (buffer VARCHAR2) RETURN VARCHAR2 IS

  ilen	CONSTANT INTEGER := length (buffer);

  BEGIN

    IF substr (buffer, 1, 1) IN (common.eol$, common.cr$, common.tab$, common.sp32$) THEN
      RETURN trimall (substr (buffer, 2) );
    END IF;

    IF substr (buffer, ilen, 1) IN (common.eol$, common.cr$, common.tab$, common.sp32$) THEN
      RETURN trimall (substr (buffer, 1, ilen - 1) );
    END IF;

    RETURN buffer;

  END trimall;

-- *******************************************************************************************
-- *******************************************************************************************

  PROCEDURE JS IS

  BEGIN

    htp.print ('
function popUp(URL) {
id = "_regist";
eval("page" + id + " = window.open(URL, ''" + id + "'', ''toolbar=0,scrollbars=1,statusbar=1,menubar=0,resizable=1,width=1000,height=400,left=50,top=50'');");
}

function closePopWin() {
   window.close();
//   opener.location.href=opener.location.href;
   opener.focus();
}'
    );

  END JS;


-- *******************************************************************************************
-- *******************************************************************************************

  PROCEDURE extract_headers(sep_char VARCHAR) IS

  BEGIN

    htp.prn (
    '"Unique ID"'	|| sep_char ||
    '"Regist Date"'	|| sep_char ||
    '"Lname"'		|| sep_char ||
    '"Fname"'		|| sep_char ||
    '"Mname"'		|| sep_char ||
    '"Initials"'	|| sep_char ||
    '"Email"'		|| sep_char ||
    '"Affiliation"'	|| sep_char ||
    '"Abbrev."'	|| sep_char ||
    '"Dept."'		|| sep_char ||
    '"Addr1"'		|| sep_char ||
    '"Addr2"'		|| sep_char ||
    '"Addr3"'		|| sep_char ||
    '"PO Box"'	|| sep_char ||
    '"Town"'		|| sep_char ||
    '"Postal Code"'	|| sep_char ||
    '"Zip Code"'	|| sep_char ||
    '"Country Code"'	|| sep_char ||
    '"Country"'	|| sep_char ||
    '"Fee"'		|| sep_char ||
    '"Paid"'		|| sep_char ||
    '"Waived"'	|| sep_char ||
    '"Due"');

  END extract_headers;


-- *******************************************************************************************

PROCEDURE extract_address (crypt_pid VARCHAR2, sep_char VARCHAR) IS

CURSOR	driver IS
SELECT	addr.*, affil.affil_abbrev, affil.name1, affil.name2, affil.department
FROM	affiliations		affil,
	addresses		addr,
	profiles		pro
WHERE	pro.person_id		= common.DECRYPT(crypt_pid)
AND	pro.deleted		IS NULL
AND	addr.address_id		= pro.address_id
AND	affil.affiliation_id	= addr.affiliation_id
ORDER BY pro.sort_order nulls last, common.get_affil_name(affil.affiliation_id);

driver_rec	driver%ROWTYPE;
country_abbrev	country_codes.country_abbrev%TYPE;
country_descr	country_codes.country_descr%TYPE;

BEGIN

    OPEN driver;
    FETCH driver INTO driver_rec;
    CLOSE driver;

    country_abbrev := common.decode_value ('country_abbrev', 'country_codes',
    'country_code', driver_rec.country_code);
    country_descr := common.decode_value ('country_descr', 'country_codes',
    'country_code', driver_rec.country_code);

    htp.prn ('"'  || driver_rec.name1	|| ' '	|| 	driver_rec.name2	|| '"' || sep_char ||
    '"'  || nvl (driver_rec.affil_abbrev, driver_rec.name1)	|| '"' || sep_char ||
    '"'  || driver_rec.department	|| '"' || sep_char ||
    '"'  || driver_rec.address1	|| '"' || sep_char ||
    '"'  || driver_rec.address2	|| '"' || sep_char ||
    '"'  || driver_rec.address3	|| '"' || sep_char ||
    '"'  || driver_rec.po_box	|| '"' || sep_char ||
    '"'  || driver_rec.town		|| '"' || sep_char ||
    '"'  || driver_rec.postal_code	|| '"' || sep_char ||
    '"'  || driver_rec.zip_code	|| '"' || sep_char ||
    '"'  || country_abbrev		|| '"' || sep_char ||
    '"'  || country_descr		|| '"');

END extract_address;

-- *******************************************************************************************

  PROCEDURE extract_regist_hdr (this_page INTEGER, char_sep VARCHAR) IS

  CURSOR	single_value_mdc IS
    SELECT	short_descr
    FROM
    page_data pd
    inner join	meta_data_codes mdc ON pd.META_DATA_CODE = mdc.META_DATA_CODE
    WHERE	pd.PAGE_ID = this_page
    AND mdc.display_code <> disp_mcb$
    AND mdc.display_code <> disp_checkbox$
    ORDER BY pd.SORT_ORDER;

  CURSOR multi_input_text IS
    SELECT	mdc.short_descr, mdc.DROPDOWN_MAX_ITEMS
    FROM
    page_data pd
    inner join	meta_data_codes mdc ON pd.META_DATA_CODE = mdc.META_DATA_CODE
    WHERE	pd.PAGE_ID = this_page
    AND mdc.display_code = disp_mcb$
    AND mdc.DROPDOWN_MAX_ITEMS IS NOT NULL
    ORDER BY pd.SORT_ORDER;

  CURSOR check_box_item IS
    SELECT	pl.ITEM_VALUE
    FROM
    page_data pd
    inner join	meta_data_codes mdc ON pd.META_DATA_CODE = mdc.META_DATA_CODE
    inner join poplist_items pl ON mdc.META_DATA_CODE = pl.META_DATA_CODE
    WHERE	pd.PAGE_ID = this_page
    AND mdc.display_code = disp_checkbox$
    ORDER BY pd.SORT_ORDER, pl.SORT_ORDER;

  BEGIN

    FOR driver_rec IN single_value_mdc LOOP
      htp.prn (char_sep);
      htp.prn ('"' || driver_rec.short_descr || '"');
    END LOOP;

    FOR driver_rec IN multi_input_text LOOP
      FOR i IN 1..driver_rec.DROPDOWN_MAX_ITEMS LOOP
	htp.prn (char_sep);
	htp.prn ('"' || driver_rec.short_descr || ' - ' || to_char(i) || '"');
      END LOOP;
    END LOOP;

    FOR driver_rec IN check_box_item LOOP
      htp.prn (char_sep);
      htp.prn ('"' || driver_rec.ITEM_VALUE || '"');
    END LOOP;

  END extract_regist_hdr;

-- *******************************************************************************************

  PROCEDURE extract_regist_responses (crypt_pid VARCHAR2, this_page INTEGER, char_sep VARCHAR) IS


  CURSOR	single_value_mdc IS
    SELECT q1.value, q.META_DATA_CODE, q.META_DATA_DESCR
    FROM
    (
    SELECT	pd.PAGE_ID, mdc.META_DATA_CODE, mdc.META_DATA_DESCR
    FROM
    page_data pd
    inner join	meta_data_codes mdc ON pd.META_DATA_CODE = mdc.META_DATA_CODE
    inner join registrants rg ON pd.PAGE_ID = rg.PAGE_ID
    WHERE	pd.PAGE_ID = this_page
    AND rg.PERSON_ID = common.DECRYPT(crypt_pid)
    AND mdc.display_code <> 'CHK'
    AND mdc.display_code <> 'MCB'
    ORDER BY pd.SORT_ORDER
    ) q
    left outer join
    (SELECT *
    FROM registrations r
    WHERE  r.PERSON_ID = common.DECRYPT(crypt_pid) AND r.PAGE_ID = this_page
    AND r.reversed IS NULL) q1
    ON q.page_id = q1.page_id AND q.META_DATA_CODE = q1.META_DATA_CODE;

  CURSOR multi_input_text IS
    SELECT q.META_DATA_CODE, q1.registration_id, common.get_user_name(q1.VALUE, 'Y') name
    FROM
    (
    SELECT	pd.SORT_ORDER, mdc.short_descr, mdc.DROPDOWN_MAX_ITEMS, pd.PAGE_ID, rg.PERSON_ID, mdc.META_DATA_CODE
    FROM
    page_data pd
    inner join	meta_data_codes mdc ON pd.META_DATA_CODE = mdc.META_DATA_CODE
    inner join registrants rg ON pd.PAGE_ID = rg.PAGE_ID
    WHERE	pd.PAGE_ID = this_page
    AND rg.PERSON_ID = common.DECRYPT(crypt_pid)
    AND mdc.display_code = 'MCB'
    AND mdc.DROPDOWN_MAX_ITEMS IS NOT NULL
    ORDER BY pd.SORT_ORDER
    )q
    left outer join
    (SELECT *
    FROM registrations r
    WHERE  r.PERSON_ID = common.DECRYPT(crypt_pid) AND r.PAGE_ID = this_page
    AND r.reversed IS NULL) q1
    ON q.PAGE_ID = q1.PAGE_ID AND q1.META_DATA_CODE = q.META_DATA_CODE
    ORDER BY q.sort_order, common.get_user_name(q1.VALUE, 'Y');

  CURSOR check_box_item IS
    SELECT	q.POPLIST_ITEM_ID, q.ITEM_VALUE, q1.registration_id
    FROM
    (
    SELECT pl.POPLIST_ITEM_ID, pl.META_DATA_CODE, pl.ITEM_VALUE, pl.SORT_ORDER, rg.PERSON_ID
    FROM
    page_data pd
    inner join	meta_data_codes mdc ON pd.META_DATA_CODE = mdc.META_DATA_CODE
    inner join poplist_items pl ON mdc.META_DATA_CODE = pl.META_DATA_CODE
    inner join registrants rg ON pd.PAGE_ID = rg.PAGE_ID
    WHERE	pd.PAGE_ID = this_page
    AND rg.PERSON_ID = common.DECRYPT(crypt_pid)
    AND mdc.display_code = 'CHK'
    ORDER BY pd.SORT_ORDER, pl.SORT_ORDER
    ) q
    left outer join
    (SELECT *
    FROM registrations r
    WHERE  r.PERSON_ID = common.DECRYPT(crypt_pid) AND r.PAGE_ID = this_page
    AND r.reversed IS NULL) q1
    ON q.POPLIST_ITEM_ID = q1.POPLIST_ITEM_ID AND q.META_DATA_CODE = q1.META_DATA_CODE;

  multi_drop_item_rec multi_input_text%ROWTYPE;

  mdc meta_data_codes.META_DATA_CODE%TYPE := -1;
  i 	meta_data_codes.DROPDOWN_MAX_ITEMS%TYPE;
  max_items meta_data_codes.DROPDOWN_MAX_ITEMS%TYPE;
  found BOOLEAN;

  BEGIN

    FOR driver_rec IN single_value_mdc LOOP
      htp.prn (char_sep);
      IF common.decode_value ('encrypt', 'meta_data_codes', 'meta_data_code', driver_rec.meta_data_code) = 'Y' THEN
	driver_rec.value := common.decrypt (driver_rec.value);
      END IF;
      driver_rec.value := REPLACE (driver_rec.value, common.eol$,	' ');
      driver_rec.value := REPLACE (driver_rec.value, common.cr$,	' ');
      driver_rec.value := REPLACE (driver_rec.value, common.tab$,	' ');
      driver_rec.value := REPLACE (driver_rec.value, common.dquote$,	common.dquote$ || common.dquote$);
      htp.prn ('"' || nvl(driver_rec.value, '') || '"');
    END LOOP;

    i := 1;
    OPEN multi_input_text;
    LOOP
      FETCH multi_input_text INTO multi_drop_item_rec;
      found := multi_input_text%found;
      EXIT WHEN found = FALSE;
      IF multi_drop_item_rec.meta_data_code != mdc THEN
	IF mdc != -1 THEN
	  max_items := common.DECODE_VALUE('DROPDOWN_MAX_ITEMS', 'meta_data_codes', 'meta_data_code', mdc);
	  max_items := nvl(max_items, 0);
	  FOR j IN i..max_items LOOP
	    htp.prn (char_sep);
	    htp.prn ('""');
	  END LOOP;
	END IF;
	i := 1;
	mdc := multi_drop_item_rec.meta_data_code;
      END IF;
      htp.prn (char_sep);
      IF multi_drop_item_rec.registration_id IS NOT NULL THEN
	htp.prn ('"' || multi_drop_item_rec.name || '"');
      ELSE
	htp.prn ('""');
      END IF;
      i := i + 1;
    END LOOP;
    max_items := nvl(common.DECODE_VALUE('DROPDOWN_MAX_ITEMS', 'meta_data_codes', 'meta_data_code', mdc), 0);

    FOR j IN i..max_items LOOP
      htp.prn (char_sep);
      htp.prn ('""');
    END LOOP;

    CLOSE multi_input_text;

    FOR driver_rec IN check_box_item LOOP
      htp.prn (char_sep);
      IF driver_rec.registration_id IS NOT NULL THEN
	htp.prn ('"Y"');
      ELSE
	htp.prn ('"N"');
      END IF;
    END LOOP;

  END extract_regist_responses;

-- *******************************************************************************************

  PROCEDURE xls (baldue common.ident_arr, this_page INTEGER) IS

  CURSOR	driver IS
    SELECT	p.*, r.create_date
    FROM	persons		p,
    registrants	r
    WHERE	r.page_id	= this_page
    AND	p.person_id	= r.person_id
    ORDER BY p.sort_lname, p.sort_fname, p.sort_mname;

  CURSOR driver1 IS
    SELECT page_name
    FROM pages
    WHERE page_id = this_page;

  char_sep$ 	CONSTANT VARCHAR(1) := ',';
  fee_due		NUMBER;
  received	NUMBER;
  waived	NUMBER;
  extract		BOOLEAN;
  dummy		BOOLEAN;
  reg_date	DATE;
  reg_date_str	VARCHAR2(50);
  filename 	pages.PAGE_NAME%TYPE;
  crypt_pid   VARCHAR2(32767);

  BEGIN

    common.get_gbl_pid;
    HTML_output := FALSE;

    IF common.header ('', test_only => TRUE) THEN

      OPEN driver1;
      FETCH driver1 INTO filename;
      CLOSE driver1;

      owa_util.mime_header('text/csv', FALSE, common.charset$);
      htp.p('Content-Disposition: filename="' || filename || '.csv"');
      owa_util.http_header_close;
      extract_headers(char_sep$);
      extract_regist_hdr (this_page, char_sep$);

      FOR driver_rec IN driver LOOP
	extract	:= FALSE;
	crypt_pid := common.ENCRYPT(driver_rec.person_id);
	fee_due	:= get_fee_total (this_page, crypt_pid, FALSE, dummy);
	received := get_fee_received (this_page, crypt_pid);
	waived := get_fee_waived (this_page, crypt_pid);

	IF baldue.count = 1 OR fee_due - received - waived <> 0 THEN
	  extract := TRUE;
	END IF;

	IF extract THEN
	  reg_date	:= common.local_time_stamp (driver_rec.create_date);
	  reg_date_str	:= to_char (reg_date, common.date_fmt_str$);

	  htp.prn (common.crlf$);
	  htp.prn ('"' || driver_rec.person_id	|| '"'	|| char_sep$);
	  htp.prn ('"' || reg_date_str		|| '"' || char_sep$);
	  htp.prn ('"' || driver_rec.lname		|| '"' || char_sep$);
	  htp.prn ('"' || driver_rec.fname		|| '"' || char_sep$);
	  htp.prn ('"' || driver_rec.mname		|| '"' || char_sep$);
	  htp.prn ('"' || driver_rec.initials	|| '"' || char_sep$);
	  htp.prn ('"' || common.get_user_email (driver_rec.person_id) || '"' || char_sep$);

	  extract_address (crypt_pid, char_sep$);

	  htp.prn ( char_sep$ || '"' || fee_due			|| '"' || char_sep$);
	  htp.prn ('"' || received			|| '"' || char_sep$);
	  htp.prn ('"' || waived			|| '"' || char_sep$);
	  htp.prn ('"' || to_char(fee_due - received - waived) || '"');

	  extract_regist_responses (crypt_pid, this_page, char_sep$);

	END IF;
      END LOOP;
    END IF;

  END xls;


-- *******************************************************************************************
-- *******************************************************************************************

  PROCEDURE extract IS

  BEGIN

    common.GET_GBL_PID;
    dbms_application_info.set_client_info (common.get_user_name (nvl (common.gbl_priv_pid, common.gbl_pid)));
    dbms_application_info.set_module (REPLACE (owa_util.get_cgi_env ('PATH_INFO'), '/', NULL), owa_util.get_cgi_env ('REMOTE_ADDR'));

    IF common.header (rpt_title$, test_only => TRUE) THEN
      maint.header(rpt_title$);
      jsfunctions;
      htp.tableOpen;
      htp.tableRowOpen;
      htp.p('<td width=40%>');
      maint.menu(0);
      htp.p('</td>');
      htp.p('<td>');
      htp.formOpen ('regist_maint.xls', 'POST');
      common.show_popup_list ('page', show_null => FALSE);
      htp.br;
      htp.formCheckBox ('baldue', 'Y');
      htp.formHidden ('baldue', 'dummy');
      htp.p (common.space2$);
      htp.p ('Export records only if a balance is due? ');
      htp.para;
      htp.formSubmit;
      htp.formClose;
      htp.p('</td>');
      htp.tableRowclose;
      htp.tableclose;
      common.footer;
    END IF;

  END extract;

-- *******************************************************************************************
-- *******************************************************************************************

  PROCEDURE payment_prompt (amt NUMBER, ADMIN BOOLEAN, pageId NUMBER DEFAULT NULL) IS

  num 	NUMBER;
  query  VARCHAR2(500);
  query_tot VARCHAR2(500);
  TYPE pmt_number IS REF CURSOR;
  driver pmt_number;

  BEGIN

    query := 'Select pc.PAYMENT_CODE code, pc.PAYMENT_DESCR descr
			from
				pages p
					inner join
				payment_codes_page pcp on p.PAGE_ID=pcp.PAGE_ID
					inner join
				payment_codes pc on pc.PAYMENT_CODE=pcp.PAYMENT_CODE
			where pcp.PAGE_ID=' || pageId ;

    IF NOT common.has_privs_to_page (common.reg_admin_page$, FALSE) THEN
      query := query || ' and pc.online_option = ''Y''';
    END IF;

    query_tot := 'Select count(*) from (' || query || ')';
    OPEN driver FOR query_tot;
    FETCH driver INTO num;
    CLOSE driver;

    IF num>0 THEN
      htp.tableRowOpen ('LEFT', 'TOP');
      htp.tableData (common.space$);
      htp.tabledata (htf.hr, ccolspan => '2');
      htp.tableData (common.space$, ccolspan => '3');
      htp.tableRowClose;
      htp.tableRowOpen ('LEFT', 'TOP');
      htp.tableHeader ('Payment Type', 'RIGHT');

      htp.p ('<TH ALIGN="LEFT">');
      registration.show_popup_table_payments (query, 'this_payment_code', NULL, NULL, NULL, NULL, TRUE, TRUE);
      htp.tableheader ('You can pay for all the items listed above or for only a subset.<br />
     Use the checkboxes (if any) on the right side of a fee still due to indicate what you are going to pay for.
     Multiple choices are allowed. <br .>
     Then, choose a method of payment with this popup menu and click ''Submit''', ccolspan => '4');
      htp.p ('</TH>');
      htp.tableRowClose;

	-- **********

      IF (amt > 0.0) OR (amt < 0.0 AND ADMIN) THEN
	htp.tableRowOpen ('LEFT', 'TOP');
	htp.tableheader ('Payment Comments', 'RIGHT');
	htp.tableHeader (htf.formText ('text', 50, 4000), 'LEFT', ccolspan =>'4');
	htp.tableHeader ('Any additional text you wish to send to the registration manager.', 'LEFT');
	htp.tableRowClose;
      END IF;

	-- **********

      htp.tableRowOpen ('LEFT', 'TOP');
      htp.tableData (common.space$);
      htp.tableData (htf.formSubmit, 'RIGHT');
      htp.tableData (common.pmt_prompt_notes$, ccolspan => '4');
      htp.tableRowClose;

    END IF;
    htp.tableClose;

  END payment_prompt;
-- *******************************************************************************************

  PROCEDURE show_address(crypt_pid VARCHAR2) IS

  CURSOR	driver IS
    SELECT	affiliation_id,
    common.get_affil_name (affiliation_id) affil_name,
    pro.*
    FROM	addresses	addr,
    profiles	pro
    WHERE	pro.person_id	= common.decrypt(crypt_pid)
    AND	pro.deleted	IS NULL
    AND	addr.address_id	= pro.address_id
    ORDER BY
    pro.current_address nulls last, pro.sort_order nulls last, pro.update_date DESC, 2;

  driver_rec	driver%ROWTYPE;

  BEGIN

    OPEN driver;
    LOOP
      FETCH driver INTO driver_rec;
      EXIT WHEN driver%NOTFOUND;
      IF(instr(lower(driver_rec.affil_name), lower('private')) = 0) THEN
	htp.bold(driver_rec.affil_name);
	htp.br;
	htp.p (common.get_address (driver_rec.address_id, htf.br));
	IF driver_rec.email_address IS NOT NULL THEN
	  htp.br;
	  htp.p (driver_rec.email_address);
	END IF;
	htp.br;
      END IF;
    END LOOP;
    CLOSE driver;

  END show_address;
-- *******************************************************************************************
-- *******************************************************************************************

  PROCEDURE show_invoice (this_page INTEGER, crypt_pid VARCHAR2, print_only BOOLEAN DEFAULT FALSE) IS

  person_id 	persons.PERSON_ID%TYPE := common.DECRYPT(crypt_pid);
  rec_found	BOOLEAN;
  fee			NUMBER := 0;
  received	NUMBER := 0;
  waived		NUMBER := 0;
  reg_number$	CONSTANT VARCHAR2(6) := to_char (person_id, 'FM000000');
  ADMIN		BOOLEAN;
  chk_exist 	BOOLEAN := FALSE;
  show_prompt	BOOLEAN;
  first 		BOOLEAN;
  redirect 	VARCHAR2(100);

  CURSOR pmt_instr IS
    SELECT pc.payment_descr, pc.instructions
    FROM payment_codes pc
    inner join
    payment_codes_page pcp ON pc.PAYMENT_CODE = pcp.PAYMENT_CODE
    WHERE pc.instructions IS NOT NULL AND pcp.PAGE_ID = this_page;

  BEGIN

    common.js$;
    htp.tableOpen (cattributes => 'WIDTH="95%"');
    common.show_data_sep (2);
    htp.tablerowopen;
    htp.tabledata(htf.img(common.logo_url$, cattributes => 'border="0"'), ccolspan => '2');
    htp.tablerowclose;
--common.show_data_sep (2);
    common.GET_GBL_PID;

    htp.tableRowOpen ('LEFT', 'TOP');

    htp.p ('<TD>');
    show_address(crypt_pid);
    htp.p ('</TD>');


    htp.tableData ('Your registration confirmation number is:'	|| common.space2$	||
    htf.bold (reg_number$)		|| htf.br		||
    htf.bold (common.get_user_name (person_id)) || htf.br || htf.br ||
    htf.bold (common.decode_value('page_name', 'pages', 'page_id', this_page)) , 'RIGHT', cnowrap => 'NOWRAP');
    htp.tableRowClose;

    common.show_data_sep (2);
    htp.tableclose;

    IF common.show_invoices$ = 'Y' THEN
      htp.formOpen ('registration.payment', 'POST', cattributes => 'NAME="SPMS"');
      htp.formHidden ('this_page', this_page);
      htp.formHidden ('crypt_pid', crypt_pid );
      htp.formHidden ('reg_id',	'dummy');
      htp.formHidden ('sel_reg_id',	'dummy');
      htp.formHidden ('amt', 'dummy');

      htp.tableOpen(cattributes => 'WIDTH="95%"');
      htp.tableRowOpen;
      htp.tableheader('Description', cattributes => 'align="center"');
      htp.tableheader('Due', cattributes => 'align="right"');
      htp.tableheader('Paid', cattributes => 'align="right"');
      htp.tableheader(common.space$, cattributes => 'align="center"');
      htp.tableheader('Notes', cattributes => 'align="left"');
      htp.tableheader('Processing information', cattributes => 'align="left"');
      htp.tablerowclose;
      htp.tableRowOpen;
      htp.tableData (common.space$, cattributes => 'colspan="6"');
      htp.tablerowclose;

      fee := get_fee_total(this_page, crypt_pid, TRUE, show_prompt);

      IF(fee IS NULL) THEN
	fee := 0;
      END IF;

      registration.show_row (chk_exist, 'Total Fees', fee, NULL, TRUE, TRUE);

      received := get_fee_received (this_page, crypt_pid);
      waived := get_fee_waived (this_page, crypt_pid);
      registration.show_row (chk_exist, 'Total Received', received, NULL, TRUE, FALSE);
      registration.show_row (chk_exist, 'Total waived', waived, NULL, TRUE, FALSE);

      registration.show_row (chk_exist, 'Balance Due', fee - received - waived, NULL, TRUE, FALSE);

    END IF;

    ADMIN := common.has_privs_to_page (common.reg_admin_page$, FALSE);

    IF print_only THEN
      htp.tableClose;
      htp.p (common.page_break$);
      htp.formClose;	-- close previous form first
      registration.end_registration (this_page);
    ELSIF ((fee - received > 0) OR (fee - received < 0 AND ADMIN)) AND show_prompt THEN
      payment_prompt (fee - received, ADMIN, this_page);
      htp.formClose;
      common.form_focus ('SPMS', 'this_payment_code');
      htp.para;
    ELSE
      htp.tableClose;
      htp.formClose;	-- close previous form first
      registration.end_registration (this_page);
    END IF;

    IF NOT print_only THEN

      htp.p('To cancel your registration ' ||
      htf.anchor ('regist_maint.cancel?action=check&this_page=' || this_page || '&crypt_pid=' ||
      crypt_pid || '&redirect=jacow.html', 'click here.'));
      htp.br;
      htp.br;
      htp.bold ('Please print this page for your records.');
      htp.para;

      first := TRUE;
      FOR pmt_instr_rec IN pmt_instr LOOP
	IF first THEN
	  first := FALSE;
	  htp.tableOpen;
	  htp.tableRowOpen;
	  htp.header (3, 'Payment Instructions');
	  htp.tableRowClose;
	  htp.tableRowOpen ('LEFT', 'TOP');
	END IF;
	htp.tableData (htf.bold (pmt_instr_rec.payment_descr ), ccolspan => '2' );
	htp.tableRowClose;
	htp.tableRowOpen;
	htp.tabledata(common.space$);
	htp.tabledata(REPLACE (pmt_instr_rec.instructions, chr(10), htf.br) );
	htp.tableRowClose;
	htp.tableRowOpen;
	htp.tabledata(common.space$, ccolspan => '2');
	htp.tableRowClose;
      END LOOP;
      IF NOT first THEN
	htp.tableClose;
      END IF;
    END IF;

  END show_invoice;

-- *******************************************************************************************
-- *******************************************************************************************

  PROCEDURE del_item (crypt_pid VARCHAR2, code INTEGER, this_page NUMBER DEFAULT NULL) IS

  BEGIN

    DELETE FROM registrations
    WHERE  person_id		= common.decrypt (crypt_pid)
    AND	  meta_data_code	= code;

    common.redirect_URL (common.sserver || common.vdir$ ||
    'registration.invoice2?crypt_pid=' || crypt_pid);

  END del_item;

-- *******************************************************************************************
-- *******************************************************************************************

  PROCEDURE new_item3 (crypt_pid VARCHAR2, descr VARCHAR2, amt VARCHAR2, this_page NUMBER DEFAULT NULL, redirect VARCHAR2 DEFAULT NULL) IS


  msg	VARCHAR2(4000) := NULL;
  ID	INTEGER;

  BEGIN

    IF common.header (rpt_title4$, test_only => TRUE) THEN
      IF trim (descr) IS NULL THEN
	msg := htf.header (3, 'You must supply a description', cattributes => common.red$);
      ELSIF trim (amt) IS NULL THEN
	msg := msg || htf.header (3, 'You must supply the fee amount', cattributes => common.red$);
      ELSIF NOT common.validate_number (amt) THEN
	msg := msg || htf.header (3, 'Fee must be a number', cattributes => common.red$);
      END IF;

      IF msg IS NULL THEN
	SELECT meta_data_seq.nextval INTO ID FROM dual;
	INSERT INTO meta_data_codes (meta_data_code, meta_data_descr, data_type_code,
	display_code, fee, MISCELLANEOUS)
	VALUES (ID, trim (descr), dataTfee$, disp_multi$, amt, 'Y');

	new_item2 (crypt_pid, ID, this_page, redirect);

      ELSE
	common.public_header (rpt_title4$);
	htp.p (msg);
	htp.p (common.click_back$);
	common.footer;

      END IF;

    END IF;

  END new_item3;

-- *******************************************************************************************
-- *******************************************************************************************

  PROCEDURE new_item2 (crypt_pid VARCHAR2, code VARCHAR2, this_page NUMBER DEFAULT NULL, redirect VARCHAR2 DEFAULT NULL) IS

  personId persons.person_id%TYPE;

  BEGIN

    IF common.header (rpt_title4$) THEN

      personId := common.decrypt (crypt_pid);

      IF code = '*' THEN
	htp.formOpen ('regist_maint.new_item3', 'POST', cattributes => 'NAME="SPMS"');
	htp.formHidden ('this_page', this_page);
	htp.formHidden ('crypt_pid', crypt_pid);
	htp.formHidden('redirect', redirect);
	htp.tableOpen;
	common.show_data_row ('Description',	htf.formText ('descr', 40, 4000));
	common.show_data_row ('Fee',		htf.formText ('amt', 10, 10));
	common.show_data_row (NULL, htf.formSubmit (NULL, 'Save'));
	htp.tableClose;
	htp.formClose;
	common.form_focus ('SPMS', 'descr');
	common.footer;

      ELSE
	INSERT INTO registrations (person_id, page_id, meta_data_code, fee, TIMESTAMP)
	VALUES (personId, this_page, code, common.DECODE_VALUE('fee', 'meta_data_codes', 'meta_data_code', code), SYSDATE);

	common.redirect_URL (common.sserver || common.vdir$ || REPLACE(redirect, '|', '&'));

      END IF;

    END IF;

  END new_item2;

-- *******************************************************************************************
-- *******************************************************************************************

  PROCEDURE new_item (crypt_pid VARCHAR2, this_page NUMBER DEFAULT NULL, redirect VARCHAR2 DEFAULT NULL) IS

  CURSOR	driver IS
    SELECT	meta_data_code, meta_data_descr, fee
    FROM	meta_data_codes
    WHERE	miscellaneous = 'Y'
    ORDER BY short_descr;

  TYPE		rec_type IS TABLE OF driver%ROWTYPE;
  driver_recs	rec_type;

  BEGIN

    IF common.header (rpt_title4$) THEN
      OPEN driver;
      FETCH driver BULK COLLECT INTO driver_recs;
      CLOSE driver;

      --if driver_recs.count = 0 then
	--new_item2 (crypt_pid, '*', this_page, redirect);
      htp.formOpen ('regist_maint.new_item2', 'POST');
      htp.formHidden ('this_page', this_page);
      htp.formHidden ('crypt_pid', crypt_pid);
      htp.formHidden ('redirect', redirect);
      htp.p ('Select a	Miscellaneous Fee ');
      htp.formSelectopen ('code');
      htp.formSelectOption ('');
      htp.formSelectOption ('*** Create a New Fee ***', cattributes => 'VALUE="*"');
      FOR i IN 1..driver_recs.COUNT
	LOOP
	htp.formSelectOption ( driver_recs(i).meta_data_descr || ' ' || TO_CHAR(driver_recs(i).fee, common.currency_fmt$), cattributes => 'VALUE="' || driver_recs(i).meta_data_code || '"');
      END LOOP;
      htp.formSelectClose;
      htp.formSubmit;
      htp.formclose;
      --end if;

      common.footer;
    END IF;

  END new_item;

-- *******************************************************************************************
-- *******************************************************************************************

  FUNCTION is_erasable (this_page pages.PAGE_ID%TYPE, pid registrations.PERSON_ID%TYPE) RETURN BOOLEAN IS

  CURSOR driver (page NUMBER, person NUMBER) IS
    SELECT 1 FROM
    payments
    WHERE payment_id IN (SELECT DISTINCT rp.payment_id
    FROM registrations r
    inner join registrations_payments rp ON r.REGISTRATION_ID = rp.REGISTRATION_ID
    WHERE r.person_id = person AND r.page_id = page)
    AND processed IS NOT NULL;

  found BOOLEAN;
  dummy NUMBER;

  BEGIN

    OPEN driver (this_page, pid);
    FETCH driver INTO dummy;
    found := driver%NOTFOUND;
    CLOSE driver;

    RETURN found;

  END is_erasable;

  PROCEDURE delete_registrations (this_page INTEGER, pid VARCHAR2) IS

  CURSOR payment(page NUMBER, person NUMBER) IS
    SELECT DISTINCT rp.payment_id
    FROM
    registrations r
    inner join
    registrations_payments rp ON r.REGISTRATION_ID = rp.REGISTRATION_ID
    WHERE r.PERSON_ID = person AND r.PAGE_ID = page;

  payment_rec payment%ROWTYPE;
  LIST VARCHAR2(3000);
  query VARCHAR2 (4000);

  BEGIN
    LIST := '';
    FOR payment_rec IN payment(this_page, pid) LOOP
      LIST := LIST || ',' || to_char(payment_rec.payment_id);
    END LOOP;
    LIST := substr(LIST, 2);

-- delete the many-to-many relation tuples first
    DELETE FROM registrations_payments
    WHERE registration_id IN (
    SELECT r.registration_id
    FROM registrations r
    WHERE r.person_id = pid AND r.page_id = this_page
    );

-- now delete payment records (IPNS first)
    if (list is not null) and (list != '') then
      query := 'delete from payment_vendor_IPNS where payment_id in (' || LIST || ')';
      EXECUTE IMMEDIATE query;
      query := 'delete from payments where payment_id in (' || LIST || ')';
      EXECUTE IMMEDIATE query;
    END IF;

-- now delete the registration page data
    DELETE FROM registrations WHERE person_id = pid AND page_id = this_page;
-- finally, the registrant data
    DELETE FROM registrants WHERE person_id = pid AND page_id = this_page;

  END delete_registrations;

-- *******************************************************************************************
-- *******************************************************************************************

/* cancel -- deletes one registration
  * Parameters:
  *   action == check : check if this registration is deletable, warn and or proceed
  *	       delete: deletes the registration
  *	       cancel: cancel the registration (..) TODO
  *   this_page == page ID
  *   crypt_pid == Person ID (encrypted)
  *   redirect	== where to redirect after the action has been performed
  */
  PROCEDURE CANCEL (action VARCHAR2, this_page INTEGER, crypt_pid VARCHAR2, redirect VARCHAR DEFAULT NULL) IS

  pid$	CONSTANT INTEGER := common.decrypt (crypt_pid);
  fee 	payments.AMOUNT%TYPE;
  received payments.AMOUNT%TYPE;
  email pages.ADMIN_EMAIL%TYPE;
  show_button BOOLEAN DEFAULT TRUE;
  tms TIMESTAMP;
  recipient VARCHAR2(1000);
  pagename VARCHAR2(1000) := common.decode_value('page_name', 'pages', 'page_id', this_page);
  confname VARCHAR2(1000) := common.conf_title$;
  subject VARCHAR2(100) := confname || '''s ' || pagename || ' canceled';
  msg VARCHAR2(1000) :=  'Dear <name>, your registration to ''' || confname || '''s ' || pagename || ''' has been cancelled.';


  BEGIN
    common.get_gbl_pid;
    IF action = 'check' THEN
      common.public_header (rpt_title5$);
      HTML_output := TRUE;
      show_invoice (this_page, crypt_pid, TRUE);
      htp.formopen ('regist_maint.cancel', 'POST');

      htp.bold ('Warning:');
      htp.p (common.space2$);
      htp.p ('Canceling a registration deletes ');
      htp.bold ('all');
      htp.p (' registration data for "<em>' || pagename || '</em>" including payments and promises of payment');
      htp.p ('To register again, re-entering all the data is needed.');
      htp.para;

      IF  common.has_privs_to_page(common.reg_admin_page$, FALSE) THEN	-- ADMIN
	IF is_erasable(this_page, pid$) THEN  -- no payments on this registration
	  htp.formhidden('action', 'delete');  -- OK can delete, really
	  htp.formhidden('redirect', redirect);
	  show_button := TRUE;
	ELSE  -- there are some payments: check if balance due is zero
	  received := get_fee_received(this_page, crypt_pid);
	  IF (received) = 0 THEN
					--htp.formhidden('action','cancel');  -- mark the registration as canceled, do not actually delete
	    htp.formhidden('action', 'delete');  -- OK can delete, really
	    show_button := TRUE;
	  ELSE
	    htp.bold ('Warning:');
	    htp.p (common.space2$);
	    htp.p ('It is not possible to proceed, since there are processed payments for <strong>' || common.currency_symbol$ || received || '</strong>. '
	    || 'To cancel this registration it is necessary to have all payments ');
	    htp.bold (' refunded ');
	    htp.p ('and all promises of payment ');
	    htp.bold(' deleted.');
	    htp.para;
	    show_button := FALSE;
	  END IF;
	END IF;
      ELSE  -- regular user
	IF pid$ = common.gbl_pid THEN
	  IF is_erasable(this_page, pid$) THEN -- no payments on this registration
	    htp.formhidden('action', 'delete');
	    show_button := TRUE;
	  ELSE
	    htp.bold ('Warning:');
	    htp.p (common.space2$);
	    htp.p ('It is not possible to proceed, since there are processed payments for <strong>' || common.currency_symbol$ || received || '</strong>.<br /><br />');
	    SELECT admin_email INTO email FROM pages WHERE page_id = this_page;
	    htp.p ('Contact ');
	    htp.anchor('mailto:' || email, email);
	    htp.br;
	    htp.br;
	    htp.para;
	    show_button := FALSE;
	  END IF;
	END IF;
      END IF;

      IF show_button THEN
	htp.header (3, 'Are you sure you wish to cancel this registration?');
	htp.formHidden ('crypt_pid', crypt_pid);
	htp.formHidden ('this_page', this_page);
	htp.formhidden('redirect', redirect);
	htp.formSubmit (NULL, 'Yes');
	htp.p (common.space2$);
	htp.p (js_close_win$);
      ELSE
	htp.p(htf.anchor ('#', htf.IMG(common.prevImg$, cattributes => 'border=0 alt="Back" title="Back"'), cattributes => 'onclick="history.back(-1);return false;"'));
      END IF;
      htp.para;
      common.footer;
      htp.formclose;

    ELSIF action = 'delete' THEN

      delete_registrations(this_page, pid$);
      SELECT admin_email INTO email FROM pages WHERE page_id = this_page;
      msg := REPLACE(msg, '<name>', common.get_user_name(pid$, 'N', 'N'));
      recipient := common.get_user_email (pid$);
      send_mail(recipient, subject, msg, email, email, NULL, pid$);
      common.redirect_URL (common.sserver || common.vdir$ || REPLACE(redirect, '|', '&'));

    ELSIF action = 'cancel' THEN  -- should not be used anymore, can be deleted #TODO

      tms := current_timestamp;
      UPDATE registrants SET CANCEL = tms WHERE person_id = pid$ AND page_id = this_page;
      SELECT admin_email INTO email FROM pages WHERE page_id = this_page;
      msg := REPLACE(msg, '<name>', common.get_user_name(pid$, 'N', 'N'));
      recipient := common.get_user_email (pid$);
      send_mail(recipient, subject, msg, email, email, NULL, pid$);
      common.redirect_URL (common.sserver || common.vdir$ || REPLACE(redirect , '|', '&'));
    END IF;

  END CANCEL;

-- *******************************************************************************************
-- *******************************************************************************************

  PROCEDURE invoices IS

  CURSOR	driver IS
    SELECT	*
    FROM registrants	r
    WHERE	r.cancel IS NULL;


  BEGIN

    IF NOT common.secure_login THEN
      common.redirect_URL (common.sserver || common.vdir$ || 'regist_maint.invoices');
    ELSIF common.header (rpt_title$, test_only => TRUE) THEN
      htp.htmlOpen;
      htp.headOpen;
      htp.p ('<link rel=stylesheet type="text/css" href="' || common.server || common.vdir$ || 'common.css">');
      htp.headClose;
      htp.bodyOpen;

      HTML_output	:= TRUE;
      print_only	:= TRUE;
      FOR driver_rec IN driver LOOP
	show_invoice (driver_rec.page_id, common.ENCRYPT(driver_rec.person_id), print_only);
      END LOOP;

      htp.bodyClose;
      htp.htmlClose;
    END IF;

  END invoices;

-- ***************************************************************************************
-- ***************************************************************************************

  PROCEDURE show_search_fields IS

/*cursor driver is
  select * from reg_cat_codes
  order by reg_cat_descr;*/

  CURSOR driver IS
    SELECT * FROM pages
    ORDER BY page_name;

  BEGIN

    IF common.header ('Registration search attendees', test_only => TRUE) THEN
      jsfunctions;
      maint.header ('Registration search attendees');
      htp.tableOpen();
      htp.tableRowOpen();
      htp.p('<td width=25%>');
      maint.menu(0);
      htp.p('</td>');
      htp.p('<td>');
      htp.tableOpen(cattributes => 'class="filter"');
      htp.p('<td colspan="2">');
      htp.p('You can search multiple attendees. Use % as a separator between two last names. If you want all the attendees just leave every field blank an click on the search button.');
      htp.p('</td>');
      htp.formOpen(curl => 'regist_maint.show_search_registrants', cmethod => 'post');
      htp.tableRowOpen();
      htp.tableData('Reg. type',  cattributes => 'align=''right''');
      htp.p('<td>');
      htp.formSelectOpen(cname => 'cat_code');
      htp.p('<option VALUE="">');
      htp.p('</option>');
      FOR driver_rec IN driver LOOP
	htp.formSelectOption(driver_rec.page_name, cattributes => 'VALUE="' || driver_rec.page_id || '"');
	htp.p('</option>');
      END LOOP;
      htp.formSelectClose();
      htp.p('</td>');
      htp.tableRowClose();
      htp.tableRowOpen();
      htp.tableData('Last name', cattributes => 'align="right"');
      htp.tableData(htf.formText(cname => 'lname', csize => '30'), cattributes => 'align="left"');
      htp.tableRowClose();
      htp.tableRowOpen();
      htp.tableData('Affiliation',  cattributes => 'align=''right''');
      htp.tableData(htf.formText(cname => 'affiliation', csize => '30'), cattributes => 'align="left"');
      htp.tableRowClose();
      htp.tableRowOpen();
      htp.tableData('Checked-in',  cattributes => 'align=''right''');
      htp.p('<td>');
      htp.formSelectOpen(cname => 'checked');
      htp.p('<option VALUE="">');
      htp.p('</option>');
      htp.formSelectOption('yes', cattributes => 'VALUE="yes"');
      htp.p('</option>');
      htp.formSelectOption('no', cattributes => 'VALUE="no"');
      htp.p('</option>');
      htp.formSelectClose();
      htp.p('</td>');
      htp.tableRowClose();
      htp.tableData('Balance Due', cattributes => 'align=''right''');
      htp.p('<td NOWRAP>');
      htp.formSelectOpen(cname => 'balancedue');
      htp.p('<option VALUE="">');
      htp.p('</option>');
      htp.formSelectOption('yes', cattributes => 'VALUE="yes"');
      htp.p('</option>');
      htp.formSelectOption('no', cattributes => 'VALUE="no"');
      htp.p('</option>');
      htp.formSelectClose();
      htp.p ('	(The searching can be slowed down using this parameter)');
      htp.p('</td>');
      htp.tableRowClose();
      htp.tableData('Sponsored',  cattributes => 'align=''right''');
      htp.p('<td>');
      htp.formSelectOpen(cname => 'sponsored');
      htp.p('<option VALUE="">');
      htp.p('</option>');
      htp.formSelectOption('yes', cattributes => 'VALUE="yes"');
      htp.p('</option>');
      htp.formSelectOption('no', cattributes => 'VALUE="no"');
      htp.p('</option>');
      htp.formSelectClose();
      htp.p('</td>');
      htp.tableRowOpen();
      htp.tableData('From:', cattributes => 'align=''right''');
      htp.tableData(htf.formText(cname => 'startDate', csize => '9'));
      htp.tableRowClose();
      htp.tableRowOpen();
      htp.tableData('To:', cattributes => 'align=''right''');
      htp.tableData(htf.formText(cname => 'endDate', csize => '9'));
      htp.tableRowClose();
      htp.tableRowOpen();
      htp.tableData(htf.formSubmit(cvalue => 'Search'), cattributes => 'align="left" colspan="2"');
      htp.tableRowClose();
      htp.tableRowOpen();
      htp.p('<td colspan="2">');
      FOR i IN 65..90 LOOP
	htp.p(htf.anchor('regist_maint.show_search_registrants?checked=&balancedue=&fname=&lname=' || chr(i) || '&affiliation=&cat_code=', chr(i) || '&nbsp;'));
      END LOOP;
      htp.p('</td>');
      htp.tableRowClose();
      htp.formClose();
      htp.tableClose();
      htp.p('</td>');
      htp.tableRowClose();
      htp.tableClose();
      common.footer;
    END IF;
  END show_search_fields;

-- ***************************************************************************************
-- ***************************************************************************************

  FUNCTION balancedue (crypt_pid VARCHAR2, pid registrants.PAGE_ID%TYPE) RETURN payments.AMOUNT%TYPE IS

  fee payments.AMOUNT%TYPE;
  received payments.AMOUNT%TYPE;
  waived 	payments.AMOUNT%TYPE;
  dummy 	BOOLEAN ;

  BEGIN

    fee := get_fee_total_query(pid, crypt_pid);
    IF(fee IS NULL) THEN
      fee := 0;
    END IF;

    received := get_fee_received (pid, crypt_pid);
    waived := get_fee_waived (pid, crypt_pid);

    RETURN fee - received - waived ;

  END balancedue;

-- ***************************************************************************************
-- ***************************************************************************************

  PROCEDURE show_search_registrants(fname VARCHAR2 DEFAULT NULL, lname VARCHAR2 DEFAULT NULL, affiliation VARCHAR2 DEFAULT NULL,
  cat_code VARCHAR2 DEFAULT NULL, checked VARCHAR2 DEFAULT NULL, balancedue VARCHAR2 DEFAULT NULL,
  sponsored VARCHAR2 DEFAULT NULL, stPage NUMBER DEFAULT 1, startDate DATE DEFAULT NULL, endDate DATE DEFAULT NULL ) IS


  CURSOR template IS
    SELECT *
    FROM export_templates;

  query VARCHAR2(32767);
  query_where VARCHAR2(32767);
  str_dummy VARCHAR2(100);
  last_update registrants.LAST_UPDATE%TYPE;
  email_sent registrants.EMAIL_SENT%TYPE;
  create_date registrants.CREATE_DATE%TYPE;
  meta_data_code registrants.META_DATA_CODE%TYPE;
  CANCEL registrants.CANCEL%TYPE;


  TYPE	cursor_type IS REF CURSOR;
  driver	cursor_type;
  query2 VARCHAR2(4000);
  driver2 cursor_type;
  name LONG := NULL;
  dummy VARCHAR2(32767);
  surname LONG := NULL;
  affil LONG := NULL;
  sortLname LONG := NULL;
  pageName LONG := NULL;
  attended VARCHAR2(3);
  notes VARCHAR2(4000);
  affid NUMBER;
  pageid NUMBER;
  addid NUMBER;
  pid NUMBER;
  sponsor_id NUMBER := NULL;
  affiliation_name LONG := NULL;
  meta_descr LONG := NULL;
  crypt_pid	VARCHAR2(100);
  counter INTEGER := 0;
  fee NUMBER := 0;
  received NUMBER := 0;
  waived NUMBER := 0;
  baldue NUMBER := 0;
  pending NUMBER := 0;
  token$ LONG;
  dummy_pid NUMBER := 0;
  dummy_regid NUMBER := 0;
  y NUMBER := 0;
  total_record NUMBER := 0;
  totalPages NUMBER := 0;
  page_number NUMBER := 0;
  remder NUMBER := 0;
  rowsNum NUMBER := 0;
  pmt NUMBER;
  redirect VARCHAR2(500);

  template_rec template%ROWTYPE;

  BEGIN

    query := 'Select  r.PERSON_ID, r.PAGE_ID, r.LAST_UPDATE, r.EMAIL_SENT, r.CREATE_DATE,
	   			  r.attended, r.SPONSORED_BY, r.META_DATA_CODE, r.NOTES, r.CANCEL,p.LNAME,p.FNAME
	   		 from registrants r
	   		      inner join persons p on r.PERSON_ID=p.PERSON_ID';

    query := query || ' where r.CANCEL is null';

    IF(cat_code IS NOT NULL) THEN
      query := query || ' and r.page_id = ' || cat_code;
    END IF;

    IF(checked IS NOT NULL ) THEN
      query := query || ' and lower(r.attended) =''' || checked || '''';
    END IF;

    IF(sponsored IS NOT NULL) THEN
      IF(sponsored = 'yes') THEN
	query := query || ' and r.sponsored_by is not null';
      ELSIF (sponsored = 'no') THEN
	query := query || ' and r.sponsored_by is null';
      END IF;
    END IF;

    IF(startDate IS NOT NULL) THEN
      query := query || ' and r.create_date >=''' || startDate || '''';
    END IF;

    IF(endDate IS NOT NULL) THEN
      query := query || ' and r.create_date <=''' || endDate || '''';
    END IF;


    IF(lname IS NOT NULL) THEN

      y := 1;
      str_dummy := NULL;
      token$ := common.stringTokenizer(lname, y , '%');
      WHILE token$ IS NOT NULL LOOP
	IF str_dummy IS NOT NULL THEN
	  str_dummy := str_dummy + ' or ';
	END IF;
	IF length(token$) = 1 THEN
	  str_dummy := str_dummy || ' lower(p.lname) like lower(''' || token$ || '%'')';
	ELSE
	  str_dummy := str_dummy || ' lower(p.lname) like lower(''%' || token$ || '%'')';
	END IF;
	y := y + 1;
	token$ := common.stringTokenizer(lname, y , '%');

      END LOOP;
      IF str_dummy IS NOT NULL THEN
	query := query || ' and (' || str_dummy || ')';
      END IF;
    END IF;

    IF(affiliation IS NOT NULL) THEN
      query := query ||
      ' and r.PERSON_ID in
								(Select p.PERSON_ID  from
								     registrants r
								     inner join profiles p on p.PERSON_ID=r.PERSON_ID
								     inner join addresses a on p.ADDRESS_ID=a.ADDRESS_ID
								     inner join affiliations aff on aff.AFFILIATION_ID=a.AFFILIATION_ID
								where lower(aff.NAME1) like lower(''%' || lower(affiliation) || '%'')
								or lower(aff.NAME2) like lower(''%' || lower(affiliation) || '%'')
								or lower(aff.SEARCH_TEXT) like lower(''%' || lower(affiliation) || '%''))';
    END IF;

    IF(balancedue IS NOT NULL) THEN
      IF(balancedue = 'yes') THEN
	query := query || ' and regist_maint.balancedue(common.ENCRYPT(r.person_id),r.page_id)<>0';
      ELSIF (balancedue = 'no') THEN
	query := query || ' and regist_maint.balancedue(common.ENCRYPT(r.person_id),r.page_id)=0';
      END IF;
    END IF;

    query := query ||  ' order by p.lname asc, p.fname';

		--htp.p(query);

    IF common.header ('Registration list attendees', test_only => TRUE) THEN
      maint.header('Registration list attendees');
      jsFunctions;
      htp.tableOpen();
      htp.tableRowOpen();
      htp.p('<td width=25%>');
      maint.menu(0);
      htp.p('</td>');
      htp.p('<td>');
      htp.tableOpen(cattributes => 'border = 1px');
      htp.formOpen(curl => 'regist_maint.show_selected_receipts', cmethod => 'post', cattributes => 'target="_blank"');
      htp.tableRowOpen();
      htp.tableHeader(common.space$);
      htp.tableHeader(common.space$);
      htp.tableHeader('Select');
      htp.tableHeader('Checked in');
      htp.tableHeader('Name');
      htp.tableHeader('Affiliation');
      htp.tableHeader('Registration page');
      htp.tableHeader('Sponsored by');
      htp.tableHeader('Sponsorship type');
      htp.tableHeader('Notes');
      htp.tableHeader('Edit receipt');
      htp.tableHeader('Edit registration');
      htp.tableHeader('');
      htp.tableRowClose();

      query2 := 'Select count(*) as counter from (' ||	query || ')';

      OPEN driver FOR query2;
      FETCH driver INTO total_record;
      CLOSE driver;
      remder := (total_record MOD record_per_page$);
      page_number := (total_record - remder) / record_per_page$;
      IF (remder>0) THEN
	page_number := page_number + 1;
      END IF;

      query := 'Select * from
   				(select rh.*,ROWNUM rnum
   		 		from (' || query || ') rh
   		 		where ROWNUM<=' || (stPage + record_per_page$ - 1) || ' ) where rnum >=' || stPage;

      OPEN driver FOR query;
      counter := 0;
      redirect := 'regist_maint.show_search_registrants?stPage=' || stpage || '&fname=' || fname ||
      '&lname=' || REPLACE(lname, '%', '%25') || '&affiliation=' || affiliation || '&cat_code=' || cat_code || '&checked=' || checked || '&balancedue=' || balancedue ||
      '&sponsored=' || sponsored || '&startDate=' || startDate || '&endDate=' || endDate;
      redirect := REPLACE(redirect, '&', '|');
      LOOP
	FETCH driver INTO pid, pageid, last_update, email_sent, create_date, attended, sponsor_id, meta_data_code, notes, CANCEL, surname, name, rowsNum;
	crypt_pid := common.ENCRYPT(pid);
	EXIT WHEN driver%NOTFOUND;
	counter := counter + 1;
	htp.tableRowOpen();
	htp.tableData(htf.anchor ('regist_maint.show_profile?crypt_pid=' || crypt_pid, htf.IMG(common.editImg$, cattributes => 'border=0 alt=Edit title=Edit'), cattributes => 'target="_blank"'));
	htp.tableData (htf.anchor ('javascript:popUp(''regist_maint.process_payments'	||
	'?reg_id=dummy&action=show&this_page=' || pageid ||
	'&crypt_pid=' || crypt_pid ||
	'&pay_id=dummy&amt=dummy&payment_type=dummy' ||
	''')', 'Process'));
	htp.p('<td>');
	htp.formCheckbox('selected', counter);
	htp.formHidden('crypt_pid', crypt_pid);
	htp.formHidden('pageid', pageid);
	htp.p('</td>');
	htp.p('<td>');
	htp.p(htf.anchor('regist_maint.check_in_registrants?checkedin=' || attended || '&crypt_pid=' || crypt_pid || '&pageid=' || pageid ||
	'&stPage=' || stPage || '&fname=' || fname ||
	'&lname=' || REPLACE(lname, '%', '%25') || '&affiliation=' || affiliation ||
	'&cat_code=' || cat_code || '&checked=' || checked || '&balancedue=' || balancedue ||
	'&sponsored=' || sponsored || '&startDate=' || startDate || '&endDate=' || endDate
	, attended, cattributes => 'onclick=''return checkInConfirm()'''));
	htp.p('</td>');
	htp.tableData(surname || common.space$ || name);
	htp.tabledata(' - ' || common.GET_USER_AFFIL_LIST(pid, '<br /> - '));
	htp.tableData(common.DECODE_VALUE('page_name', 'pages', 'page_id', pageid));
	IF(sponsor_id IS NOT NULL) THEN
	  affiliation_name := NULL;
	  query2 := 'select common.get_affil_name(ad.affiliation_id,null,null,null) from profiles pf
					       inner join addresses ad on ad.address_id = pf.address_id
					       inner join affiliations af on af.affiliation_id = ad.affiliation_id
					       where pf.person_id = ' || sponsor_id;

	  OPEN driver2 FOR query2;
	  FETCH driver2 INTO affiliation_name;
	  CLOSE driver2;
	  htp.tableData(affiliation_name);
	ELSE
	  htp.tabledata('&nbsp');
	END IF;
	meta_descr := nvl(common.DECODE_VALUE('short_descr', 'meta_data_codes', 'meta_data_code', meta_data_code), common.space$);
	htp.tableData(meta_descr);
	IF(trim(notes) IS NOT NULL OR trim(notes) != '') THEN
	  htp.tableData(htf.anchor('javascript:popUp(''regist_maint.add_registrant_delegates_notes?crypt_pid=' || crypt_pid || '&action=show&pgid=' || pageid || ''')', 'Show'));
	ELSE
	  htp.tableData(htf.anchor('javascript:popUp(''regist_maint.add_registrant_delegates_notes?crypt_pid=' || crypt_pid || '&action=add&pgid=' || pageid || ''')', 'Add'));
	END IF;
	htp.tableData(htf.anchor ('registration.invoice2?this_page=' || pageid || '&crypt_pid=' || crypt_pid, htf.IMG(common.editImg$, cattributes => 'border=0 alt=Edit title=Edit'), cattributes => 'target="_blank"'));
	htp.tableData(htf.anchor ('regist_maint.show_registration_page?ID=' || pageid || '&crypt_pid=' || crypt_pid, htf.IMG(common.editImg$, cattributes => 'border=0 alt=Edit title=Edit'), cattributes => 'target="_blank"'));
	htp.tableData(htf.anchor('regist_maint.cancel?action=check&this_page=' || pageid || '&crypt_pid=' || crypt_pid || '&redirect=' || redirect, htf.IMG(common.deleteImg$, cattributes => 'border=0 alt=Delete title=Delete'), cattributes => 'onclick=''return deleteConfirm()'''));
	htp.tableRowClose();

      END LOOP;
      CLOSE driver;
      htp.tableRowOpen();
      htp.p('<td colspan="13">');
      FOR i IN 1..page_number LOOP
	htp.p(htf.anchor ('regist_maint.show_search_registrants?stPage=' || ((i - 1) * record_per_page$ + 1) || '&fname=' || fname ||
	'&lname=' || REPLACE(lname, '%', '%25') || '&affiliation=' || affiliation || '&cat_code=' || cat_code || '&checked=' || checked || '&balancedue=' || balancedue ||
	'&sponsored=' || sponsored || '&startDate=' || startDate || '&endDate=' || endDate, i));
      END LOOP;
      htp.p('</td>');
      htp.tableRowClose();
      htp.tableRowOpen();
      htp.p('<td colspan="13" >');
      htp.p(htf.anchor('regist_maint.show_search_fields', htf.IMG(common.prevImg$, cattributes => 'border=0 alt=Previous title=Previous')));
      htp.p('&nbsp');
      IF(counter != 0) THEN
	htp.p(htf.formSubmit(cvalue => 'Print selected receipts') || '<br /><br />');
      END IF;
      htp.p('</td>');
      htp.tableRowOpen();
      htp.formClose();
      htp.formOpen(curl => 'regist_maint.export_search_registrants', cmethod => 'post');
      htp.tableRowOpen();
      htp.p('<td colspan="12">');
      htp.p('<br />Choose template');
      htp.formSelectOpen(cname => 'temp_id');
      htp.p('<option VALUE="-1">');
      htp.p('</option>');
      FOR template_rec IN template LOOP
	htp.formSelectOption(template_rec.template_name, cattributes => 'VALUE="' || template_rec.template_id || '"');
	htp.p('</option>');
      END LOOP;
      htp.formSelectClose();
      htp.formHidden(cname => 'fname', cvalue => fname);
      htp.formHidden(cname => 'lname', cvalue => lname);
      htp.formHidden(cname => 'affiliation', cvalue => affiliation);
      htp.formHidden(cname => 'cat_code', cvalue => cat_code);
      htp.formHidden(cname => 'checked', cvalue => checked);
      htp.formHidden(cname => 'balancedue', cvalue => balancedue);
      htp.formHidden(cname => 'sponsored', cvalue => sponsored);
      htp.formHidden(cname => 'startDate', cvalue => startDate);
      htp.formHidden(cname => 'endDate', cvalue => endDate);
      htp.p('&nbsp');
      htp.p(htf.formSubmit(cvalue => 'Export data') || '&nbsp&nbsp(The export is based on the filter selected before)<br /><br />');
      htp.p('</td>');
      htp.tableRowClose();
      htp.formClose();
      htp.tableClose();
      htp.p('</td>');
      htp.tableRowClose();
      htp.tableClose();
      common.footer;
    END IF;

  END show_search_registrants;


  PROCEDURE export_search_registrants(fname VARCHAR2 DEFAULT NULL, lname VARCHAR2 DEFAULT NULL, affiliation VARCHAR2 DEFAULT NULL,
  cat_code VARCHAR2 DEFAULT NULL, checked VARCHAR2 DEFAULT NULL, balancedue VARCHAR2 DEFAULT NULL,
  sponsored VARCHAR2 DEFAULT NULL, startDate DATE DEFAULT NULL, endDate DATE DEFAULT NULL,
  temp_id INTEGER DEFAULT - 1) IS



  CURSOR template IS
    SELECT * FROM export_templates
    WHERE template_id = temp_id;

  str_dummy VARCHAR2(100);
  last_update registrants.LAST_UPDATE%TYPE;
  email_sent registrants.EMAIL_SENT%TYPE;
  create_date registrants.CREATE_DATE%TYPE;
  meta_data_code registrants.META_DATA_CODE%TYPE;
  CANCEL registrants.CANCEL%TYPE;
  notes registrants.NOTES%TYPE;
  surname persons.LNAME%TYPE;
  name persons.FNAME%TYPE;
  query VARCHAR2(4000);
  dummy 	BOOLEAN ;

  TYPE	cursor_type IS REF CURSOR;
  driver	cursor_type;
  template_rec template%ROWTYPE;

  found BOOLEAN := FALSE;
  headers VARCHAR2(10000);
  free_row VARCHAR2(10000);
  fields VARCHAR2(20000);
  attended VARCHAR2(3);
  pageid NUMBER;
  sponsor_id NUMBER := NULL;
  pid NUMBER;
  fee NUMBER := 0;
  received NUMBER := 0;
  affiliation_name LONG := NULL;

  token$ LONG;
  y NUMBER := 0;


  BEGIN

    query := 'Select  r.PERSON_ID, r.PAGE_ID, r.LAST_UPDATE, r.EMAIL_SENT, r.CREATE_DATE,
   			  r.attended, r.SPONSORED_BY, r.META_DATA_CODE, r.NOTES, r.CANCEL, p.LNAME, p.FNAME
   		 from registrants r
   		      inner join persons p on r.PERSON_ID=p.PERSON_ID';

    query := query || ' where r.CANCEL is null';

    IF(cat_code IS NOT NULL) THEN
      query := query || ' and r.page_id = ' || cat_code;
    END IF;

    IF(checked IS NOT NULL) THEN
      query := query || ' and lower(r.attended) =''' || checked || '''';
    END IF;

    IF(sponsored IS NOT NULL) THEN
      IF(sponsored = 'yes') THEN
	query := query || ' and r.sponsored_by is not null';
      ELSIF (sponsored = 'no') THEN
	query := query || ' and r.sponsored_by is null';
      END IF;
    END IF;

    IF(startDate IS NOT NULL) THEN
      query := query || ' and r.create_date >=''' || startDate || '''';
    END IF;

    IF(endDate IS NOT NULL) THEN
      query := query || ' and r.create_date <=''' || endDate || '''';
    END IF;


    IF(lname IS NOT NULL) THEN

      y := 1;
      str_dummy := NULL;
      token$ := common.stringTokenizer(lname, y , '%');
      WHILE token$ IS NOT NULL LOOP
	IF str_dummy IS NOT NULL THEN
	  str_dummy := str_dummy + ' or ';
	END IF;
	IF length(token$) = 1 THEN
	  str_dummy := str_dummy || ' lower(p.lname) like lower(''' || token$ || '%'')';
	ELSE
	  str_dummy := str_dummy || ' lower(p.lname) like lower(''%' || token$ || '%'')';
	END IF;
	y := y + 1;
	token$ := common.stringTokenizer(lname, y , '%');

      END LOOP;
      IF str_dummy IS NOT NULL THEN
	query := query || ' and (' || str_dummy || ')';
      END IF;
    END IF;

    IF(affiliation IS NOT NULL) THEN
      query := query ||
      ' and r.PERSON_ID in
							(Select p.PERSON_ID  from
							     registrants r
							     inner join profiles p on p.PERSON_ID=r.PERSON_ID
							     inner join addresses a on p.ADDRESS_ID=a.ADDRESS_ID
							     inner join affiliations aff on aff.AFFILIATION_ID=a.AFFILIATION_ID
							where lower(aff.NAME1) like lower(''%' || lower(affiliation) || '%'')
							or lower(aff.NAME2) like lower(''%' || lower(affiliation) || '%'')
							or lower(aff.SEARCH_TEXT) like lower(''%' || lower(affiliation) || '%''))';
    END IF;

    IF(balancedue IS NOT NULL) THEN
      IF(balancedue = 'yes') THEN
	query := query || ' and regist_maint.balancedue(common.ENCRYPT(r.person_id),r.page_id)<>0';
      ELSIF (balancedue = 'no') THEN
	query := query || ' and regist_maint.balancedue(common.ENCRYPT(r.person_id),r.page_id)=0';
      END IF;
    END IF;

    query := query ||  ' order by p.lname asc, p.fname';

  -- htp.p(query);

    IF(temp_id != -1) THEN
      OPEN template;
      FETCH template INTO template_rec;
      found := template%FOUND;
      CLOSE template;
    END IF;
    owa_util.mime_header('text/csv', FALSE, common.charset$);
    htp.p('Content-Disposition: filename="attendees_list.csv"');
    owa_util.http_header_close;
    common.get_gbl_pid;
    headers := NULL;
    IF(found) THEN
      IF(template_rec.checked_in = 'Y') THEN
	headers := headers || 'Checked in,';
	free_row := free_row || ',';
      END IF;
      headers := headers || 'Last Name,First Name,';
      free_row := free_row || ',,';
      IF(template_rec.affiliation = 'Y') THEN
	headers := headers || 'Affiliation,';
	free_row := free_row || ',';
      END IF;
      IF(template_rec.reg_page = 'Y') THEN
	headers := headers || 'Registration page,';
	free_row := free_row || ',';
      END IF;
      IF(template_rec.reg_date = 'Y') THEN
	headers := headers || 'Registered,';
	free_row := free_row || ',';
      END IF;
      IF(template_rec.sponsored_by = 'Y') THEN
	headers := headers || 'Sponsored by,';
	free_row := free_row || ',';
      END IF;
      IF(template_rec.sponsor_type = 'Y') THEN
	headers := headers || 'Sponsorship type,';
	free_row := free_row || ',';
      END IF;
      IF(template_rec.fee = 'Y') THEN
	headers := headers || 'Fee,';
	free_row := free_row || ',';
      END IF;
      IF(template_rec.paid = 'Y') THEN
	headers := headers || 'Paid,';
	free_row := free_row || ',';
      END IF;
      IF(template_rec.due = 'Y') THEN
	headers := headers || 'Due,';
	free_row := free_row || ',';
      END IF;
      htp.p(substr(headers, 1, length(headers) - 1));
--	    htp.p(free_row);
    ELSE
      htp.p('Checked in,Last Name,First Name,Affiliation,Registration page,Registered,Sponsored by,Sponsorship type,Fee,Paid,Due');
--	    htp.p(',,,,,,,,,,,');
    END IF;

    OPEN driver FOR query;
    LOOP
      FETCH driver INTO pid, pageid, last_update, email_sent, create_date, attended, sponsor_id, meta_data_code, notes, CANCEL, surname, name;
      EXIT WHEN driver%NOTFOUND;
      IF(sponsor_id IS NOT NULL) THEN
	affiliation_name := common.GET_AFFIL_NAME(common.GET_USER_PRIMARY_AFFIL_ID(sponsor_id));
      ELSE
	affiliation_name := '';
      END IF;
      IF(found) THEN
	fields := '"';
	IF(template_rec.checked_in = 'Y') THEN
	  fields := fields || attended || '","';
	END IF;
	fields := fields || surname || '","' || name || '","';
	IF(template_rec.affiliation = 'Y') THEN
	  fields := fields || common.GET_AFFIL_NAME(common.GET_USER_PRIMARY_AFFIL_ID(pid)) || '","';
	END IF;
	IF(template_rec.reg_page = 'Y') THEN
	  fields := fields || common.DECODE_VALUE('page_name', 'pages', 'page_id', pageid) || '","';
	END IF;
	IF(template_rec.reg_date = 'Y') THEN
	  fields := fields || to_char(create_date, 'dd/mm/yyyy') || '","';
	END IF;
	IF(template_rec.sponsored_by = 'Y') THEN
	  fields := fields || affiliation_name || '","';
	END IF;
	IF(template_rec.sponsor_type = 'Y') THEN
	  fields := fields || common.DECODE_VALUE('meta_data_descr', 'meta_data_codes', 'meta_data_code', meta_data_code) || '","';
	END IF;
	fee := get_fee_total(pageid, pid, FALSE, dummy);
	IF(template_rec.fee = 'Y') THEN
	  fields := fields || to_char(fee) || '","';
	END IF;
	received := get_fee_received (pageid, pid);
	IF(template_rec.paid = 'Y') THEN
	  fields := fields || to_char(received) || '","';
	END IF;
	IF(template_rec.due = 'Y') THEN
	  fields := fields || to_char(fee - received) || '","';
	END IF;
	fields := substr(fields, 1, length(fields) - 1);
	htp.p(fields);
      ELSE
	fee := get_fee_total(pageid, pid, FALSE, dummy);
	received := get_fee_received (pageid, pid);
	htp.p('"' || attended || '","' || surname || '","' || name  || '","' ||
	common.GET_AFFIL_NAME(common.GET_USER_PRIMARY_AFFIL_ID(pid)) || '","' ||
	common.DECODE_VALUE('page_name', 'pages', 'page_id', pageid) || '","' ||
	to_char(create_date, 'dd/mm/yyyy') || '","' || affiliation_name || '","' ||
	common.DECODE_VALUE('meta_data_descr', 'meta_data_codes', 'meta_data_code', meta_data_code) || '","' ||
	to_char(fee) || '","' ||
	to_char(received) || '","' ||
	to_char(fee - received) || '"');
      END IF;
    END LOOP;
    CLOSE driver;

  END export_search_registrants;

-- ***************************************************************************************
-- ***************************************************************************************

  PROCEDURE show_selected_receipts(selected owa_util.ident_arr DEFAULT empty_arr , crypt_pid owa.vc_arr DEFAULT empty_arr_vc, pageid owa_util.ident_arr DEFAULT empty_arr) IS


  BEGIN

    IF NOT common.secure_login THEN
      common.redirect_URL (common.sserver || common.vdir$ || 'regist_maint.show_search_fields');
    ELSIF common.header ('', test_only => TRUE) THEN
      htp.htmlOpen;
      htp.headOpen;
      htp.p ('<link rel=stylesheet type="text/css" href="' || common.server || common.vdir$ || 'common.css">');
      htp.headClose;
      htp.bodyOpen;

      IF (selected.count > 0) THEN
	HTML_output	:= TRUE;
	print_only	:= TRUE;
	FOR i IN 1..selected.count LOOP
	  show_invoice (pageid(selected(i)), crypt_pid(selected(i)), print_only);
	END LOOP;
      ELSE
	htp.header (3, select_one$);
	htp.p(htf.anchor('#', 'Close window', cattributes => 'onclick="window.close();"'));
      END IF;

      htp.bodyClose;
      htp.htmlClose;
    END IF;

  END show_selected_receipts;

  PROCEDURE check_in_registrants(checkedin VARCHAR2, crypt_pid VARCHAR2 , pageid NUMBER, fname VARCHAR2 DEFAULT NULL, lname VARCHAR2 DEFAULT NULL, affiliation VARCHAR2 DEFAULT NULL,
  cat_code VARCHAR2 DEFAULT NULL, checked VARCHAR2 DEFAULT NULL, balancedue VARCHAR2 DEFAULT NULL,
  sponsored VARCHAR2 DEFAULT NULL, stPage NUMBER DEFAULT 1, startDate DATE DEFAULT NULL, endDate DATE DEFAULT NULL ) IS

  globalpid	INTEGER := common.gbl_pid;
  pid 		INTEGER := common.DECRYPT(crypt_pid);
  BEGIN


    IF NOT common.secure_login THEN
      common.redirect_URL (common.sserver || common.vdir$ || 'regist_maint.show_search_fields');
    ELSIF common.header ('', test_only => TRUE) THEN
      IF (checkedin = 'no') THEN
	UPDATE registrants r SET attended = 'yes' WHERE person_id =  pid AND page_id = pageid;
      ELSE
	UPDATE registrants r SET attended = 'no' WHERE person_id =  pid AND page_id = pageid;
      END IF;

      common.redirect_URL (common.sserver || common.vdir$ || 'regist_maint.show_search_registrants?fname=' || fname ||
      '&lname=' || lname ||
      '&affiliation=' || affiliation ||
      '&cat_code=' || cat_code ||
      '&checked=' || checked ||
      '&balancedue=' || balancedue ||
      '&sponsored=' || sponsored ||
      '&stpage=' ||  stPage ||
      '&startDate=' || startDate ||
      '&enddate=' || endDate);
    END IF;

  END check_in_registrants;


  PROCEDURE show_registration_page (ID INTEGER, crypt_pid VARCHAR2) IS


  pid NUMBER := common.DECRYPT(crypt_pid);

  CURSOR	driver IS
    SELECT	*
    FROM	pages
    WHERE	page_id = ID;

  CURSOR att IS
    SELECT attended
    FROM registrants
    WHERE person_id = pid;

  driver_rec	driver%ROWTYPE;
  att_rec 	att%ROWTYPE;
  rec_found	BOOLEAN;
  URL		VARCHAR2(200);
  CONTINUE	BOOLEAN := TRUE;
  registrants	INTEGER;

-- User data entry interface.

  BEGIN

    OPEN driver;
    FETCH driver INTO driver_rec;
    rec_found := driver%FOUND;
    CLOSE driver;

    common.get_gbl_pid;
    URL := registration.get_reg_page_URL (ID, 'page', TRUE);
   -- htp.p(URL);
    IF common.gbl_pid IS NULL THEN -- user not logged in
      CONTINUE	:= FALSE;
      common.write_cookie ('URL', URL);
      common.redirect_URL (URL);

    ELSIF driver_rec.admin_privs = 'Y' AND NOT common.secure_login THEN
      CONTINUE	:= FALSE;
      common.redirect_URL (URL);

    ELSIF driver_rec.max_registrants IS NOT NULL THEN
      registrants := registration.get_registrant_count (ID);
      IF registrants >= driver_rec.max_registrants THEN
	CONTINUE := FALSE;

	common.public_header (rpt_title$);
	htp.header (3, driver_rec.page_name);
	htp.header (3, 'Registration Closed', cattributes => common.red$);
	htp.bold ('Registration has reached maximum capacity.');
	htp.para;
	common.footer;
      END IF;

    END IF;

    IF CONTINUE THEN
      common.public_header (driver_rec.page_name, js_src => JS$, cache_timeout => 5);

      IF NOT rec_found THEN
	htp.header (3, 'Page #' || ID || ' does not exist.', cattributes => common.red$);

      ELSIF NOT get_status (ID, is_available$) THEN
	htp.header (3, 'This page is not available at this time.', cattributes => common.red$);

      ELSE
	htp.p('<b>Registration page of ' || common.get_user_name(pid) || '</b> <br />');
	IF common.has_privs_to_page (common.reg_readonly_page$, FALSE) OR common.has_privs_to_page (common.reg_admin_page$, FALSE) THEN
	  OPEN att;
	  FETCH att INTO att_rec;
	  CLOSE att;
	  htp.p(htf.anchor('regist_maint.check_in_registrants?checkedin=' || att_rec.attended || '&pageid=' || ID || '&crypt_pid=' || crypt_pid, '<br/>checked-in:' || att_rec.attended));
	END IF;
	htp.formOpen ('!registration.usave', 'POST', cattributes => 'name="saveForm"');
	htp.formHidden ('PID', ID);
	registration.build_data_entry_form (id, TRUE, crypt_pid);
	htp.formClose;

      END IF;

      common.footer;
    END IF;

  END show_registration_page;

-- ***************************************************************************************
-- ***************************************************************************************

  PROCEDURE webServiceTest IS
  soap_request VARCHAR2(30000);
  soap_respond VARCHAR2(30000);
  http_req utl_http.req;
  http_resp utl_http.resp;
  resp XMLType;
  xmlData VARCHAR2(30000);
  i INTEGER;
  v_message	   VARCHAR2(250);
  V_error_code	   NUMBER;
  V_error_msg	    VARCHAR2(300);
  BEGIN

    xmlData := '
	      <fo:root xmlns:fo="http://www.w3.org/1999/XSL/Format">

	      <fo:layout-master-set>
		<fo:simple-page-master master-name="voucher">
		  <fo:region_body />
		</fo:simple-page-master>
	      </fo:layout-master-set>

	      <fo:page-sequence master-reference="voucher">
		<fo:flow flow-name="xsl-region-body">
		  <fo:block>Hello W3Schools</fo:block>
		</fo:flow>
	      </fo:page-sequence>
	      </fo:root>

	     ';

    soap_request := '<?xml version = "1.0" encoding = "' || common.charset$ || '"?>
		<SOAP-ENV:Envelope
		 xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/"
		 xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
		 xmlns:xsd="http://www.w3.org/2001/XMLSchema">
		 <SOAP-ENV:Body>'
    || xmlData ||
    '</SOAP-ENV:Body>
		 </SOAP-ENV:Envelope>';
    htp.p(soap_request);
 /*
    http_req:= utl_http.begin_request
    ( 'http://apps.ais.pl/fojet/services/FOProcessor?wsdl'
    , 'POST'
    , 'HTTP/1.1'
    );
    utl_http.set_header(http_req, 'Content-Type', 'text/xml'); -- since we are dealing with plain text in XML documents
    utl_http.set_header(http_req, 'Content-Length', length(soap_request));
    utl_http.set_header(http_req, 'SOAPAction', ''); -- required to specify this is a SOAP communication
    utl_http.write_text(http_req, soap_request);
    http_resp:= utl_http.get_response(http_req);
    utl_http.read_text(http_resp, soap_respond);
    utl_http.end_response(http_resp);
    resp:= XMLType.createXML(soap_respond);
    /* resp:= resp.extract('/soap:Envelop/soap:Body/child::node()'
    , 'xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/"'
    );*/
    i := 0;
 /* loop
    htp.p(substr(soap_respond,1+ i*255,250));
    i:= i+1;
    if i*250> length(soap_respond)
    then
    exit;
    end if;
    end loop;
    htp.p(soap_respond);
    return;
    exception
    when others then
    v_error_code := SQLCODE;
    v_error_msg  := SQLERRM;
    v_message := 'ERROR: ' || v_error_code || ' ' || v_error_msg;
    htp.p (v_message);*/
  END webServiceTest;

-- ***************************************************************************************

  PROCEDURE jsFunctions IS
  BEGIN
    htp.p ('<SCRIPT TYPE="text/javascript">');
    htp.p('function popUp(URL) {
			id = "_regist";
			eval("page" + id + " = window.open(URL, ''" + id + "'', ''toolbar=0,scrollbars=1,statusbar=1,menubar=0,resizable=1,width=1110,height=760,left=50,top=50'');");
			}

			function closePopWin() {
			   window.close();
			//   opener.location.href=opener.location.href;
			   opener.focus();
			}'
    );


    htp.p ('function confirmation() {
			var answer = confirm("Are you sure you want to leave?");
			if(answer){
			    return true;
			} else{
			    return false;
			}
	       }');
    htp.p ('function roomConfirmation() {
			var answer = confirm("Are you sure you want to confirm this room?");
			if(answer){
			    return true;
			} else{
			    return false;
			}
	       }');
    htp.p ('function roomUnConfirmation() {
			var answer = confirm("Are you sure you want to unconfirm this room?");
			if(answer){
			    return true;
			} else{
			    return false;
			}
	       }');
    htp.p ('function deleteConfirm() {
			var answer = confirm("Are you sure you want to delete the selected item?");
			if(answer){
			    return true;
			} else{
			    return false;
			}
	       }');

    htp.p ('function hotelDeleteConfirm() {
			var answer = confirm("Are you sure you want to delete the selected item and all related data (rooms/fees/registrants choices)?");
			if(answer){
			    return true;
			} else{
			    return false;
			}
	       }');

    htp.p ('function checkInConfirm() {
			var answer = confirm("Are you sure you want to check-in/out the selected person?");
			if(answer){
			    return true;
			} else{
			    return false;
			}
	       }');

    htp.p ('function generalConfirm() {
			var answer = confirm("Are you sure?");
			if(answer){
			    return true;
			} else{
			    return false;
			}
	       }');

    htp.p('
	  var count=0;
	  function controlCheck(chk,max){
	      if (chk.checked){
		    count++;
		    if (count > max){
			alert("You cannot choose more than " + max + " attendes");
			chk.checked=false;
			count--;
		    }
	      }else{
	       count>0?count--:null;
	      }
	  }
      ');

    htp.p('
	 function addSelected(pos,metaCode){

	    var objradio = window.document.forms[''attendees''].elements[''selected[]''];
	    var names = window.document.forms[''attendees''].elements[''person[]''];
	    var parentNames = window.opener.document.forms[''saveForm''].elements[''persons_''+metaCode+''[''+pos+'']''];
	    var parentPid = window.opener.document.forms[''saveForm''].elements[''pid_''+metaCode+''[''+pos+'']''];
	    var y = -1;
	    var i = 0;
	    var length = objradio.length;

	    for( i=0;i<length;i++){
		  if(objradio[i].checked) {
		 y=i;
			 parentNames.value = names[i].value;
			 parentPid.value = objradio[i].value;
			 window.close();
		  }
	    }
	    if (y==-1) {
	    	alert(''Select a person or close the window to exit'');
	    }
	 }
      ');
    htp.p('
     function popup_pmt_reload(reg_id,pmt_id){
		var x=document.getElementById("payments");
		var y=0;

		txt="All options: ";
		url="regist_maint.reassign_payment?reg_id="+reg_id+"&action=show&pmt_id="+pmt_id;
		for (i=0;i<x.length;i++)
		  {
			  if (x.options[i].selected) {
				  y=i;
		      }
		  }
		url=url + "&pmt_id_selected=" + x.options[y].value;
		document.location.href=url;
     }

     ');
    htp.p ('</SCRIPT>');

  END jsFunctions;


  PROCEDURE add_registrant_delegates_notes(pgid NUMBER, crypt_pid VARCHAR2, action VARCHAR2, del_notes VARCHAR2 DEFAULT NULL) IS

  pid persons.PERSON_ID%TYPE := common.DECRYPT(crypt_pid);

  CURSOR driver IS
    SELECT notes FROM registrants
    WHERE person_id = pid AND
    page_id = pgid;

  dummy VARCHAR2(4000);
  BEGIN
    IF common.header ('', test_only => TRUE) THEN
      IF common.CSS_URL$ IS NOT NULL THEN
	htp.p ('<link rel=stylesheet type="text/css" href="' || common.CSS_URL$ || '">');
      ELSE
	htp.p ('<link rel=stylesheet type="text/css" href="' || common.sserver || common.vdir$ || 'common.css">');
      END IF;
      htp.tableOpen();
      htp.tableRowOpen();
      htp.p('<td width=25%>');
      htp.p(common.space$);
      htp.p('</td>');
      htp.p('<td>');
      IF (action = 'add') THEN
	htp.formOpen(curl => 'regist_maint.add_registrant_delegates_notes', cmethod => 'post');
	htp.formHidden(cname => 'crypt_pid', cvalue => crypt_pid);
	htp.formHidden(cname => 'pgid', cvalue => pgid);
	htp.formHidden(cname => 'action', cvalue => 'save');
	htp.tableOpen();
	htp.tableRowOpen();
	htp.tableData('Notes:', cattributes => 'class=boldtext');
	htp.p('<td>');
	htp.p('<textarea name="del_notes" cols="60" rows="10">');
	htp.p(del_notes);
	htp.p('</textarea>');
	htp.p('</td>');
	htp.tableRowClose();
	htp.tableRowOpen();
	htp.p('<td colspan="2" align="center">');
	htp.formSubmit('action', 'Save');
	htp.p(common.space$);
	htp.formReset('Cancel', 'onClick="javascript:window.close();"');
	htp.p('<td>');
	htp.tableRowClose();
	htp.tableClose();
	htp.formClose();
      ELSIF(action = 'show') THEN
	OPEN driver;
	FETCH driver INTO dummy;
	CLOSE driver;
	htp.formOpen(curl => 'regist_maint.add_registrant_delegates_notes', cmethod => 'post');
	htp.formHidden(cname => 'crypt_pid', cvalue => crypt_pid);
	htp.formHidden(cname => 'pgid', cvalue => pgid);
	htp.formHidden(cname => 'action', cvalue => 'save');
	htp.tableOpen();
	htp.tableRowOpen();
	htp.tableData('Notes:', cattributes => 'class=boldtext');
	htp.p('<td>');
	htp.p('<textarea name="del_notes" cols="60" rows="10">');
	htp.p(dummy);
	htp.p('</textarea>');
	htp.p('</td>');
	htp.tableRowClose();
	htp.tableRowOpen();
	htp.p('<td colspan="2" align="center">');
	htp.formSubmit('action', 'Save');
	htp.p(common.space$);
	htp.formReset('Cancel', 'onClick="javascript:window.close();"');
	htp.p('<td>');
	htp.tableRowClose();
	htp.tableClose();
	htp.formClose();
      ELSIF(action = 'save') THEN
	UPDATE registrants SET notes = del_notes WHERE person_id = pid AND page_id = pgid;
	htp.htmlOpen;
	htp.headOpen;
	htp.headClose;
	htp.BODYOPEN;
	htp.p('<SCRIPT TYPE="text/javascript">window.opener.location.reload() ; window.close();</SCRIPT>');
	htp.BODYCLOSE;
	htp.htmlClose;
      END IF;
    END IF;

  END add_registrant_delegates_notes;


  PROCEDURE edit_template(tid INTEGER DEFAULT - 1) IS

  CURSOR driver IS
    SELECT * FROM export_templates
    WHERE template_id = tid;

  driver_rec driver%ROWTYPE;
  check_arr owa_util.ident_arr;

  BEGIN

    common.get_gbl_pid;
    IF common.has_privs_to_page (common.reg_readonly_page$, FALSE) OR common.has_privs_to_page (common.reg_admin_page$, FALSE) THEN
      maint.header ('Edit export template');
      htp.p('<br/>');
      jsFunctions;
      htp.tableOpen();
      htp.tableRowOpen();
      htp.p('<td width=25%>');
      maint.menu(0);
      htp.p('</td>');
      htp.p('<td>');
      IF(tid = -1) THEN
	htp.formOpen(curl => 'regist_maint.add_template', cmethod => 'post');
	htp.tableOpen();
	htp.tableRowOpen();
	htp.formHidden(cname => 'chk_arr', cvalue => '-1');
	htp.tableData('Template name', cattributes => 'align="right"');
	htp.tableData(htf.formText(cname => 'chk_arr', cvalue => 'insert template name'));
	htp.tableRowClose();
	htp.tableRowOpen();
	htp.tableData('Last Name', cattributes => 'align="right"');
	htp.tableData(htf.formCheckbox(cname => 'chk_arr', cvalue => '3', cattributes => 'checked="true" disabled="true"'));
	htp.tableRowClose();
	htp.tableData('First Name', cattributes => 'align="right"');
	htp.tableData(htf.formCheckbox(cname => 'chk_arr', cvalue => '3', cattributes => 'checked="true" disabled="true"'));
	htp.tableRowClose();
	htp.tableData('Checked in', cattributes => 'align="right"');
	htp.tableData(htf.formCheckbox(cname => 'chk_arr', cvalue => '3'));
	htp.tableRowClose();
	htp.tableRowOpen();
	htp.tableData('Affiliation', cattributes => 'align="right"');
	htp.tableData(htf.formCheckbox(cname => 'chk_arr', cvalue => '4'));
	htp.tableRowClose();
	htp.tableRowOpen();
	htp.tableData('Registration type', cattributes => 'align="right"');
	htp.tableData(htf.formCheckbox(cname => 'chk_arr', cvalue => '5'));
	htp.tableRowClose();
	htp.tableRowOpen();
	htp.tableData('Registration date', cattributes => 'align="right"');
	htp.tableData(htf.formCheckbox(cname => 'chk_arr', cvalue => '6'));
	htp.tableRowClose();
	htp.tableRowOpen();
	htp.tableData('Sponsored by', cattributes => 'align="right"');
	htp.tableData(htf.formCheckbox(cname => 'chk_arr', cvalue => '7'));
	htp.tableRowClose();
	htp.tableRowOpen();
	htp.tableData('Sponsorship type', cattributes => 'align="right"');
	htp.tableData(htf.formCheckbox(cname => 'chk_arr', cvalue => '8'));
	htp.tableRowClose();
	htp.tableRowOpen();
	htp.tableData('Fee', cattributes => 'align="right"');
	htp.tableData(htf.formCheckbox(cname => 'chk_arr', cvalue => '9'));
	htp.tableRowClose();
	htp.tableRowOpen();
	htp.tableData('Paid', cattributes => 'align="right"');
	htp.tableData(htf.formCheckbox(cname => 'chk_arr', cvalue => '10'));
	htp.tableRowClose();
	htp.tableRowOpen();
	htp.tableData('Due', cattributes => 'align="right"');
	htp.tableData(htf.formCheckbox(cname => 'chk_arr', cvalue => '11'));
	htp.tableRowClose();
	htp.tableRowOpen();
	htp.tableData(htf.formSubmit, cattributes => 'colspan="2" align="center"');
	htp.tableRowClose();
	htp.tableClose();
	htp.formClose();
      ELSE
	OPEN driver;
	FETCH driver INTO driver_rec;
	CLOSE driver;
	check_arr := create_checked_arr(driver_rec);
	htp.formOpen(curl => 'regist_maint.add_template', cmethod => 'post');
	htp.tableOpen();
	htp.tableRowOpen();
	htp.formHidden(cname => 'chk_arr', cvalue => driver_rec.template_id);
	htp.tableData('Template name', cattributes => 'align="right"');
	htp.tableData(htf.formtext(cname => 'chk_arr', cvalue => driver_rec.template_name));
	htp.tableRowClose();
	htp.tableRowOpen();
	htp.tableData('Checked in', cattributes => 'align="right"');
	htp.tableData(htf.formCheckbox(cname => 'chk_arr', cvalue => '3', cattributes => check_arr(3)));
	htp.tableRowClose();
	htp.tableRowOpen();
	htp.tableData('Affiliation', cattributes => 'align="right"');
	htp.tableData(htf.formCheckbox(cname => 'chk_arr', cvalue => '4', cattributes => check_arr(4)));
	htp.tableRowClose();
	htp.tableRowOpen();
	htp.tableData('Registration type', cattributes => 'align="right"');
	htp.tableData(htf.formCheckbox(cname => 'chk_arr', cvalue => '5', cattributes => check_arr(5)));
	htp.tableRowClose();
	htp.tableRowOpen();
	htp.tableData('Registration date', cattributes => 'align="right"');
	htp.tableData(htf.formCheckbox(cname => 'chk_arr', cvalue => '6', cattributes => check_arr(6)));
	htp.tableRowClose();
	htp.tableRowOpen();
	htp.tableData('Sponsored by', cattributes => 'align="right"');
	htp.tableData(htf.formCheckbox(cname => 'chk_arr', cvalue => '7', cattributes => check_arr(7)));
	htp.tableRowClose();
	htp.tableRowOpen();
	htp.tableData('Sponsorship type', cattributes => 'align="right"');
	htp.tableData(htf.formCheckbox(cname => 'chk_arr', cvalue => '8', cattributes => check_arr(8)));
	htp.tableRowClose();
	htp.tableRowOpen();
	htp.tableData('Fee', cattributes => 'align="right"');
	htp.tableData(htf.formCheckbox(cname => 'chk_arr', cvalue => '9', cattributes => check_arr(9)));
	htp.tableRowClose();
	htp.tableRowOpen();
	htp.tableData('Paid', cattributes => 'align="right"');
	htp.tableData(htf.formCheckbox(cname => 'chk_arr', cvalue => '10', cattributes => check_arr(10)));
	htp.tableRowClose();
	htp.tableRowOpen();
	htp.tableData('Due', cattributes => 'align="right"');
	htp.tableData(htf.formCheckbox(cname => 'chk_arr', cvalue => '11', cattributes => check_arr(11)));
	htp.tableRowClose();
	htp.tableRowOpen();
	htp.tableData(htf.formSubmit, cattributes => 'colspan="2" align="center"');
	htp.tableRowClose();
	htp.tableClose();
	htp.formClose();
      END IF;
      htp.p('</td>');
      htp.tableData('&nbsp;', cattributes => 'width="5%"');
      htp.tableRowClose();
      htp.tableClose();
    END IF;
  END edit_template;

  PROCEDURE add_template(chk_arr IN owa_util.ident_arr DEFAULT	empty_arr, indx_arr IN owa_util.ident_arr DEFAULT empty_arr ) IS

  red_tid INTEGER := -1;
  query_update VARCHAR2(1000) := NULL;
  is_in BOOLEAN := FALSE;
  i INTEGER := 1;
  chk_arr_neg owa_util.ident_arr;

  BEGIN
    IF (chk_arr(1) = -1) THEN
      query_update := 'insert into export_templates (template_name,checked_in,affiliation,reg_page,reg_date,sponsored_by,
			  			 sponsor_type,fee,paid, due)';
      query_update := query_update || ' values(' ;
      query_update := query_update || '''' || REPLACE(chk_arr(2), '''', '''''') || ''',' ;
      FOR y IN 3..11 LOOP
	FOR i IN 1..chk_arr.count LOOP
	  IF(chk_arr(i) = to_char(y)) THEN
	    is_in := TRUE;
	  END IF;
	END LOOP;
	IF(is_in) THEN
	  query_update := query_update || ' ''Y'',' ;
	  is_in := FALSE;
	ELSE
	  query_update := query_update || ' ''N'',' ;
	END IF;
      END LOOP;
      query_update := substr(query_update, 1, length(query_update) - 1);
      query_update := query_update || ')';

      EXECUTE IMMEDIATE query_update;
		--select max(template_id) as id into red_tid from export_templates;
    ELSE
      red_tid := chk_arr(1);
      UPDATE export_templates SET
      checked_in = 'N',
      affiliation = 'N',
      reg_page = 'N',
      reg_date = 'N',
      sponsored_by = 'N',
      sponsor_type = 'N',
      fee = 'N',
      paid = 'N',
      due = 'N'
      WHERE template_id = chk_arr(1);
      query_update := 'update export_templates set template_name = ''' || REPLACE(chk_arr (2), common.quote$, common.quote$ || common.quote$) || '''';
      FOR i IN 3..chk_arr.count LOOP
	IF( chk_arr(i)	= '3') THEN
	  query_update := query_update || ', checked_in = ''Y''';
	END IF;
	IF( chk_arr(i)	= '4') THEN
	  query_update := query_update || ', affiliation = ''Y''';
	END IF;
	IF( chk_arr(i)	= '5') THEN
	  query_update := query_update || ', reg_page = ''Y''';
	END IF;
	IF( chk_arr(i)	= '6') THEN
	  query_update := query_update || ', reg_date = ''Y''';
	END IF;
	IF( chk_arr(i)	= '7') THEN
	  query_update := query_update || ', sponsored_by = ''Y''';
	END IF;
	IF( chk_arr(i)	= '8') THEN
	  query_update := query_update || ', sponsor_type = ''Y''';
	END IF;
	IF( chk_arr(i)	= '9') THEN
	  query_update := query_update || ', fee = ''Y''';
	END IF;
	IF( chk_arr(i)	= '10') THEN
	  query_update := query_update || ', paid = ''Y''';
	END IF;
	IF( chk_arr(i)	= '11') THEN
	  query_update := query_update || ', due = ''Y''';
	END IF;
      END LOOP;
      query_update := query_update || ' where template_id = ' || chk_arr(1);
      EXECUTE IMMEDIATE query_update;
    END IF;
    common.redirect_URL (common.sserver || common.vdir$ || 'regist_maint.show_export_templates');
  END add_template;

  FUNCTION create_checked_arr(driver_rec IN OUT export_templates%ROWTYPE) RETURN owa_util.ident_arr IS

  ret_arr owa_util.ident_arr;

  BEGIN
    IF(driver_rec.checked_in = 'Y') THEN
      ret_arr(3) := 'checked';
    ELSE
      ret_arr(3) := '';
    END IF;
    IF(driver_rec.affiliation = 'Y') THEN
      ret_arr(4) := 'checked';
    ELSE
      ret_arr(4) := '';
    END IF;
    IF(driver_rec.reg_page = 'Y') THEN
      ret_arr(5) := 'checked';
    ELSE
      ret_arr(5) := '';
    END IF;
    IF(driver_rec.reg_date = 'Y') THEN
      ret_arr(6) := 'checked';
    ELSE
      ret_arr(6) := '';
    END IF;
    IF(driver_rec.sponsored_by = 'Y') THEN
      ret_arr(7) := 'checked';
    ELSE
      ret_arr(7) := '';
    END IF;
    IF(driver_rec.sponsor_type = 'Y') THEN
      ret_arr(8) := 'checked';
    ELSE
      ret_arr(8) := '';
    END IF;
    IF(driver_rec.fee = 'Y') THEN
      ret_arr(9) := 'checked';
    ELSE
      ret_arr(9) := '';
    END IF;
    IF(driver_rec.paid = 'Y') THEN
      ret_arr(10) := 'checked';
    ELSE
      ret_arr(10) := '';
    END IF;
    IF(driver_rec.due = 'Y') THEN
      ret_arr(11) := 'checked';
    ELSE
      ret_arr(11) := '';
    END IF;
    RETURN ret_arr;
  END create_checked_arr;


  PROCEDURE show_export_templates IS

  CURSOR template IS
    SELECT	*
    FROM	export_templates
    ORDER BY template_id;

  template_rec template%ROWTYPE;

  BEGIN

    IF common.header ('Export templates', test_only => TRUE) THEN
      maint.header('Export templates');
      jsFunctions;
      htp.tableOpen();
      htp.tableRowOpen();
      htp.p('<td width=25%>');
      maint.menu(0);
      htp.p('</td>');
      htp.p('<td>');
      htp.tableOpen(cattributes => 'border = 1px');
      htp.tableRowOpen();
      htp.tableHeader(common.space$);
      htp.tableHeader('Template name');
      htp.tableHeader('Checked in');
      htp.tableHeader('Affiliation');
      htp.tableHeader('Registration page');
      htp.tableHeader('Registration date');
      htp.tableHeader('Sponsored by');
      htp.tableHeader('Sponsorship type');
      htp.tableHeader('Fee');
      htp.tableHeader('Paid');
      htp.tableHeader('Due');
      htp.tableHeader(common.space$);
      htp.tableRowClose();
      FOR template_rec IN template LOOP
	htp.tableRowOpen();
	htp.tableRowOpen();
	htp.tableData(htf.anchor ('regist_maint.edit_template?tid=' || template_rec.template_id, htf.IMG(common.editImg$, cattributes => 'border=0 alt=Edit title=Edit')));
	htp.tableData(template_rec.template_name);
	htp.tableData(template_rec.checked_in);
	htp.tableData(template_rec.affiliation);
	htp.tableData(template_rec.reg_page);
	htp.tableData(template_rec.reg_date);
	htp.tableData(template_rec.sponsored_by);
	htp.tableData(template_rec.sponsor_type);
	htp.tableData(template_rec.fee);
	htp.tableData(template_rec.paid);
	htp.tableData(template_rec.due);
	htp.tableData(htf.anchor ('regist_maint.remove_template?tid=' || template_rec.template_id , htf.IMG(common.deleteImg$, cattributes => 'border=0 alt="Remove Template" title="Remove Template"'), cattributes => 'onclick=''return deleteConfirm()'''));
	htp.tableRowClose();
      END LOOP;
      htp.tableRowOpen();
      htp.tableData(htf.anchor ('regist_maint.edit_template', htf.IMG(common.newImg$, cattributes => 'border=0 alt="New template" title="New template"')), cattributes => 'colspan="14"');
      htp.tableRowClose();
      htp.tableClose();
      htp.p('</td>');
      htp.tableRowClose();
      htp.tableClose();
      common.footer;
    END IF;

  END show_export_templates;

  PROCEDURE remove_template(tid INTEGER DEFAULT - 1) IS
  BEGIN
    IF(tid != -1) THEN
      DELETE FROM export_templates WHERE template_id = tid;
    END IF;
    common.redirect_URL (common.sserver || common.vdir$ || 'regist_maint.show_export_templates');
  END remove_template;


-- *******************************************************************************************
-- *******************************************************************************************


END regist_maint;

/
show error

set define on

--	Copyright (C) 2004-2008 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>
