set define off

create or replace PACKAGE body repat_conf as

-- ***********************************************************************************

procedure add_conference is

rec	repatriate.conf_rec_type;
conf_id	integer;

begin

   select conference_id into conf_id from control;

   select conference_code, conference_title, start_date, end_date, location
   into	  rec.conference_code, rec.conference_title, rec.start_date, rec.end_date, rec.location
   from	  repo_conf_conferences
   where  conference_id = conf_id;

   if conf_id is null then
      raise_application_error (-20000, 'There is no conference ID.');
   end if;

   repatriate.set_global_conference_id (conf_id);

end add_conference;

-- ***********************************************************************************

procedure add_classifications is

cursor	driver is
select	*
from	classification_codes;

recs	repatriate.classifications_type;

begin

   open driver;
   fetch driver BULK COLLECT into recs;
   close driver;

   repatriate.add_classifications (recs);

end add_classifications;

-- ***********************************************************************************

procedure add_code (recs in out repatriate.functions_type,
			code varchar2, descr varchar2) is

begin

   recs(recs.COUNT+1).function_code	:= code;
   recs(recs.COUNT).function_descr	:= descr;

end add_code;

-- ***********************************************************************************

procedure add_functions is

cursor	driver is
select	function_code, function_descr
from	function_codes
UNION
select	'a~' || author_code, author_descr
from	author_codes;

recs	repatriate.functions_type;

begin

   open driver;
   fetch driver BULK COLLECT into recs;
   close driver;

   add_code (recs, 'e~PR', 'Expert - Propose Reclassification');
   add_code (recs, 'e~AR', 'Expert - Approve Reclassification');
   add_code (recs, 'e~PP', 'Expert - Propose Presentation Type');
   add_code (recs, 'e~AP', 'Expert - Approve Presentation Type');
   add_code (recs, 'e~CO', 'Expert - Classification Coordinator');
   add_code (recs, 'r~REG', 'Registered Participant');

   repatriate.add_functions (recs);

end add_functions;

-- ***********************************************************************************

procedure add_keywords is

cursor	driver is
select	*
from	keyword_codes;

recs	repatriate.keywords_type;

begin

   open driver;
   fetch driver BULK COLLECT into recs;
   close driver;

   repatriate.add_keywords (recs);

end add_keywords;

-- ***********************************************************************************

procedure add_presentations is

cursor	driver is
select	presentation_code, presentation_descr, presentation_type_code
from	presentation_codes;

recs	repatriate.presentations_type;

begin

   open driver;
   fetch driver BULK COLLECT into recs;
   close driver;

   repatriate.add_presentations (recs);

end add_presentations;

-- ***********************************************************************************

procedure add_sub_classes is

cursor	driver is
select	*
from	sub_class_codes;

recs	repatriate.sub_classes_type;

begin

   open driver;
   fetch driver BULK COLLECT into recs;
   close driver;

   repatriate.add_sub_classes (recs);

end add_sub_classes;

-- ***********************************************************************************

procedure add_eclasses is

cursor	driver is
select	*
from	eclass_codes;

recs	repatriate.eclasses_type;

begin

   open driver;
   fetch driver BULK COLLECT into recs;
   close driver;

   repatriate.add_eclasses (recs);

end add_eclasses;

-- ***********************************************************************************

procedure add_abstracts is

cursor	driver is
select	sa.presentation_code, a.classification_code,
	a.sub_class_code, sa.paper_id, a.title
from	abstract_sessions	sa,
	abstracts		a
where	sa.primary_flag		= 'Y'
and	sa.abstract_id		= a.abstract_id
and	common.paper_OK_to_publish(a.abstract_id) = 'Yes';

recs	repatriate.abs_recs_type;

begin

   open driver;
   fetch driver BULK COLLECT into recs;
   close driver;

   repatriate.add_abstracts (recs);

end add_abstracts;

-- ***********************************************************************************

procedure add_roles is

cursor	driver is
select	function_code, person_id, null profile_id, null paper_id
from	user_functions
UNION
select	'a~'||author_code, person_id, profile_id,
	common.get_paper_ID(abstract_id) paper_id
from	abstract_authors	aa
where	common.paper_ok_to_publish(abstract_id) = 'Yes'
and	author_code in
	(	select	author_code
		from	author_codes)
UNION
select	'e~PR', person_id, null, null
from	experts
where	propose_reclass	= 'Y'
UNION
select	'e~AR', person_id, null, null
from	experts
where	approve_reclass	= 'Y'
UNION
select	'e~PP', person_id, null, null
from	experts
where	propose_present	= 'Y'
UNION
select	'e~AP', person_id, null, null
from	experts
where	approve_present	= 'Y'
UNION
select	'e~CO', person_id, null, null
from	experts
where	coordinator	= 'Y'
UNION
select	DISTINCT 'r~REG', person_id, null, null
from	registrants	r
where	exists 
	(	select	1
		from	affiliations		affil,
			addresses		addr,
			profiles		pro
		where	pro.person_id		= r.person_id
		and	addr.address_id		= pro.address_id
		and	affil.affiliation_id	= addr.affiliation_id
		and	affil.affil_type_code	<> 'IND'); -- exclude industry registrants

cursor	C2 (pid integer) is
select	*
from	profiles
where	profile_id = pid;

cursor	C3 (pid integer) is
select	*
from	persons
where	person_id = pid;

cursor	C4 (aid integer) is
select	*
from	addresses
where	address_id = aid;

cursor	C5 (aid integer) is
select	*
from	affiliations
where	affiliation_id = aid;

type	lrecs_type is table of driver%ROWTYPE index by binary_integer;
lrecs	lrecs_type;
recs	repatriate.roles_type;

begin

   open driver;
   fetch driver BULK COLLECT into lrecs;
   close driver;

   for i in 1..lrecs.COUNT loop
      recs(i).function_code	:= lrecs(i).function_code;
      recs(i).person_id		:= lrecs(i).person_id;
      recs(i).paper_id		:= lrecs(i).paper_id;

      open C2 (lrecs(i).profile_id);
      fetch C2 into recs(i).profile;
      close C2;

      open C3 (lrecs(i).person_id);
      fetch C3 into recs(i).person;
      close C3;

      open C4 (recs(i).profile.address_id);
      fetch C4 into recs(i).address;
      close C4;

      open C5 (recs(i).address.affiliation_id);
      fetch C5 into recs(i).affiliation;
      close C5;
   end loop;

   repatriate.add_roles (recs);

end add_roles;

-- ***********************************************************************************

procedure add_registrants is

cursor	driver is
select	distinct person_id
from	registrants	r
where	attended	= 'yes'
and	exists
	(	select	1
		from	affiliations		affil,
			addresses		addr,
			profiles		pro
		where	pro.person_id		= r.person_id
		and	addr.address_id		= pro.address_id
		and	affil.affiliation_id	= addr.affiliation_id
		and	affil_type_code		<> 'IND');

IDs	repatriate.registrants_type;

begin

   open driver;
   fetch driver BULK COLLECT into IDs;
   close driver;

   repatriate.add_registrants (IDs);

end add_registrants;

-- ***********************************************************************************

procedure data is

cursor	driver is
select	job
from	user_jobs;

begin

   add_conference;

   add_classifications;
   add_keywords;
   add_presentations;
   add_sub_classes;
   add_eclasses;

   add_abstracts;
   add_functions;
   add_roles;
   add_registrants;

   repatriate.set_repatriated_date;

   for rec in driver LOOP
      dbms_job.remove (rec.job);
   end loop;

end data;

-- ***********************************************************************************

end repat_conf;
/
show error

set define on

--	Copyright (C) 2004 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>