set define off

create or replace PACKAGE body repatriate as

fail_safe	integer;

-- ***********************************************************************************

procedure set_global_conference_id (conf_id integer) is

begin

   gbl_conf_id := conf_id;

end;

-- ***********************************************************************************

procedure set_repatriated_date is

begin

   update conf_conferences
   set	  repatriated	= common.get_UTC_time,
	  last_sync_id	= null
   where  conference_id	= gbl_conf_id;

end;

-- ***********************************************************************************

procedure delete_conference (conf_id integer) is

begin

   delete from conf_abs_keywords
   where abstract_id in
	(select abstract_id from conf_abstracts where conference_id = conf_id);

   delete from conf_roles
   where function_id in
	(select function_id from conf_functions where conference_id = conf_id);

   delete from conf_abstracts		where conference_id = conf_id;
   delete from conf_eclasses		where conference_id = conf_id;
   delete from conf_classifications	where conference_id = conf_id;
   delete from conf_functions		where conference_id = conf_id;
   delete from conf_keywords		where conference_id = conf_id;
   delete from conf_presentations	where conference_id = conf_id;
   delete from conf_sub_classes		where conference_id = conf_id;
   delete from conf_registrants		where conference_id = conf_id;

   update conf_conferences set repatriated = null where conference_id = conf_id;

end delete_conference;

-- ***********************************************************************************
-- ***********************************************************************************

procedure add_presentations (recs presentations_type) is

begin

   for i in 1..recs.COUNT loop
      insert into conf_presentations values
      (null, gbl_conf_id, recs(i).presentation_code,
	recs(i).presentation_descr, recs(i).presentation_type_code);
   end loop;

end add_presentations;

-- ***********************************************************************************

procedure add_classifications (recs classifications_type) is

begin

   for i in 1..recs.COUNT loop
      insert into conf_classifications values
      (null, gbl_conf_id, recs(i).classification_code, recs(i).classification_descr);
   end loop;

end add_classifications;

-- ***********************************************************************************

procedure add_sub_classes (recs sub_classes_type) is

begin

   for i in 1..recs.COUNT loop
      insert into conf_sub_classes values
      (null, gbl_conf_id, recs(i).sub_class_code, recs(i).sub_class_descr);
   end loop;

end add_sub_classes;

-- ***********************************************************************************

procedure add_eclasses (recs eclasses_type) is

cursor	C1 (code$ varchar2) is
select	classification_id
from	conf_classifications
where	conference_id	= gbl_conf_id
and	class_code = code$;

cursor	C2 (code$ varchar2) is
select	sub_class_id
from	conf_sub_classes
where	conference_id	= gbl_conf_id
and	sub_class_code = code$;

class_id	number;
sub_class_id	number;

begin

   for i in 1..recs.COUNT loop
      class_id		:= null;
      sub_class_id	:= null;

      open C1 (recs(i).classification_code);
      fetch C1 into class_id;
      close C1;

      open C2 (recs(i).sub_class_code);
      fetch C2 into sub_class_id;
      close C2;

      insert into conf_eclasses values
	(null, gbl_conf_id, class_id, sub_class_id);
   end loop;

end add_eclasses;

-- ***********************************************************************************

procedure add_keywords (recs keywords_type) is

begin

   for i in 1..recs.COUNT loop
      insert into conf_keywords values
	(null, gbl_conf_id, recs(i).keyword_code,
	recs(i).keyword_descr, recs(i).regular_expression);
   end loop;

end add_keywords;

-- ***********************************************************************************

procedure add_functions (recs functions_type) is

begin

   for i in 1..recs.COUNT loop
      insert into conf_functions values
	(null, gbl_conf_id, recs(i).function_code, recs(i).function_descr);
   end loop;

end add_functions;

-- ***********************************************************************************
-- ***********************************************************************************

procedure check_affiliation (IDs in out id_type, rec role_type) is

cursor	driver is
select	1
from	affiliations
where	affiliation_id = IDs.affiliation_id;

dummy		integer;
rec_found	boolean;

begin

   open driver;
   fetch driver into dummy;
   rec_found := driver%FOUND;
   close driver;

   if NOT rec_found then
      insert into affiliations values (rec.affiliation.affiliation_id,
	rec.affiliation.affil_abbrev, rec.affiliation.name1, rec.affiliation.name2,
	rec.affiliation.department, rec.affiliation.web_site, rec.affiliation.search_text,
	rec.affiliation.affil_type_code, rec.affiliation.update_date,
	rec.affiliation.admin_comments);
   end if;

end check_affiliation;

-- ***********************************************************************************

procedure check_address_merged (IDs in out id_type) is

cursor	driver is
select	merge_into_id
from	merged_addresses
where	old_addr_id = IDs.address_id;

new_id		integer;
rec_found	boolean;

begin

   fail_safe := fail_safe + 1;
   if fail_safe < 50 then
      open driver;
      fetch driver into new_id;
      rec_found := driver%FOUND;
      close driver;

      if rec_found then
         IDs.address_id := new_id;
         check_address_merged (IDs);
      end if;
   end if;

end check_address_merged;

-- ***********************************************************************************

procedure check_address_id (IDs in out id_type, rec role_type) is

cursor	driver is
select	1
from	addresses
where	address_id = IDs.address_id;

dummy		integer;
rec_found	boolean;

begin

   fail_safe := 0;
   check_address_merged (IDs);

   open driver;
   fetch driver into dummy;
   rec_found := driver%FOUND;
   close driver;

   if not rec_found then
      IDs.address_id		:= rec.address.address_id;
      IDs.affiliation_id	:= rec.affiliation.affiliation_id;
      check_affiliation (IDs, rec);

      insert into addresses values (rec.address.address_id, rec.address.address1,
	rec.address.address2, rec.address.address3, rec.address.po_box, rec.address.town,
	rec.address.postal_code, rec.address.zip_code, rec.address.country_code,
	rec.address.search_text, rec.address.affiliation_id, rec.address.update_date);
   end if;

end check_address_id;

-- ***********************************************************************************

procedure check_merged_persons (IDs in out id_type) is

-- Finds the next person_id assiged to the original ID.

cursor	driver is
select	new_person_id person_id, merged_repo merged_date
from	merge_logs
where	old_person_id	= IDs.person_id
and	merged_repo	> IDs.check_date;

rec		driver%ROWTYPE;
rec_found	boolean;

begin

   fail_safe := fail_safe + 1;
   if fail_safe < 50 then
      open driver;
      fetch driver into rec;
      rec_found := driver%FOUND;
      close driver;

      if rec_found then
         IDs.person_id	:= rec.person_id;
         IDs.check_date	:= rec.merged_date;
         check_merged_persons (IDs);
      end if;
   end if;

end check_merged_persons;

-- ***********************************************************************************

procedure check_person_id (IDs in out id_type, rec role_type) is

/* Checks the person_id sent by the conference to see if it still exists. If not, the
   ID was changed/merged so this routine finds the correct ID. */

cursor	driver is
select	1
from	persons
where	person_id = IDs.person_id;

cursor	C2 is
select	1
from	persons
where	user_id = rec.person.user_id;

rec_found	boolean;
dummy		integer;
the_user_id	persons.user_id%TYPE;

begin

   fail_safe		:= 0;
   IDs.check_date	:= to_date ('01/01/1000', 'mm/dd/yyyy');
   check_merged_persons (IDs);

   open driver;
   fetch driver into dummy;
   rec_found := driver%FOUND;
   close driver;

   if NOT rec_found then
      open C2;
      fetch C2 into dummy;
      rec_found := C2%FOUND;
      close C2;

      the_user_id := rec.person.user_id;
      if rec_found then
         the_user_id := null;
      end if;

      insert into persons values (rec.person.person_id, the_user_id,
	rec.person.password, rec.person.fname, rec.person.mname, rec.person.lname,
	rec.person.pname, rec.person.initials, rec.person.title, rec.person.sort_fname,
	rec.person.sort_mname, rec.person.sort_lname, rec.person.soundex_lname,
	rec.person.country_code, rec.person.date_format_code, rec.person.authorization_given,
	rec.person.update_date, rec.person.invitee, rec.person.tz_name,
	rec.person.requires_invitation, rec.person.admin_comments, rec.person.gender);

        IDs.person_id := rec.person.person_id;
   end if;

end check_person_id;

-- ***********************************************************************************

function profile_address_exists (IDs in out id_type) return boolean is

cursor	C2 is
select	profile_id
from	profiles
where	person_id	= IDs.person_id
and	address_id	= IDs.address_id;

dummy		integer;
rec_found	boolean;

begin

   open C2; -- is there an existing profile using the same address/affiliation?
   fetch C2 into dummy;
   rec_found := C2%FOUND;
   close C2;

   if rec_found then
      IDs.profile_ID := dummy; -- new profile_id, but same person and affiliation
   end if;

   return rec_found;

end profile_address_exists;

-- ***********************************************************************************

procedure check_profile_id (IDs in out id_type, rec role_type) is

cursor	driver is
select	1
from	profiles
where	profile_id = IDs.profile_id;

dummy		integer;
rec_found	boolean;

begin

   open driver;
   fetch driver into dummy;
   rec_found := driver%FOUND;
   close driver;

   if NOT rec_found then -- profile no longer in repository
      if NOT profile_address_exists (IDs) then
         check_address_id (IDs, rec);

         if NOT profile_address_exists (IDs) then -- address may have merged; check again
            insert into profiles values
		(IDs.profile_id, IDs.person_id, IDs.address_id,
		rec.profile.email_address, rec.profile.mail_stop, rec.profile.po_box,
		rec.profile.phone_number, rec.profile.fax_number, rec.profile.sort_order,
		rec.profile.current_address, rec.profile.update_date, rec.profile.deleted);
         end if;

      end if;
   end if;

end check_profile_id;

-- ***********************************************************************************

procedure validate_IDs (IDs in out ID_type, rec role_type) is

begin

   check_person_id (IDs, rec);
   if IDs.profile_id is NOT null then
      check_profile_id (IDs, rec);
   end if;

end validate_IDs;

-- ***********************************************************************************

procedure add_roles (recs roles_type) is

cursor	C1 (fcode varchar2) is
select	function_id
from	conf_functions
where	conference_id	= gbl_conf_id
and	function_code	= fcode;

cursor	C2 (pid$ varchar2) is
select	abstract_id
from	conf_abstracts
where	conference_id	= gbl_conf_id
and	paper_id	= pid$;

abs_id	integer;
fid	integer;
pid	integer;
prof_id	integer;

rec	role_type;
IDs	ID_type;

begin

   for i in 1..recs.COUNT loop
      IDs := null;

      IDs.person_id	:= recs(i).person_id;
      IDs.profile_id	:= recs(i).profile.profile_id;
      IDs.address_id	:= recs(i).profile.address_id;

      open C1 (recs(i).function_code);
      fetch C1 into IDs.function_id;
      close C1;

      open C2 (recs(i).paper_id);
      fetch C2 into IDs.abstract_id;
      close C2;

      if recs(i).function_code like 'a~%' then
         if IDs.abstract_id is null then
            raise_application_error (-20001, 'Function must contain an abstract.');
         end if;
      end if;

      validate_IDs (IDs, recs(i));

      insert into conf_roles
	select	IDs.function_id, IDs.person_id, IDs.profile_id, IDs.abstract_id
	from	dual
	where	NOT exists
		(	select	1
			from	conf_roles		r2
			where	r2.function_id		= IDs.function_id
			and	r2.person_id		= IDs.person_id
			and	nvl(r2.profile_id,0)	= nvl(IDs.profile_id,0)
			and	nvl(r2.abstract_id,0)	= nvl(IDs.abstract_id,0) );
   end loop;

end add_roles;

-- ***********************************************************************************

procedure add_abs_keywords (recs abs_keywords_type) is

cursor	C1 (code varchar2) is
select	keyword_id
from	conf_keywords
where	conference_id	= gbl_conf_id
and	keyword_code	= code;

cursor	C2 (pid varchar2) is
select	abstract_id
from	conf_abstracts
where	conference_id	= gbl_conf_id
and	paper_id	= pid;

kid	integer;
aid	integer;

begin

   for i in 1..recs.COUNT loop
      kid	:= null;
      aid	:= null;

      open C1 (recs(i).keyword_code);
      fetch C1 into kid;
      close C1;

      open C2 (recs(i).paper_id);
      fetch C2 into aid;
      close C2;

      insert into conf_abs_keywords values (aid, kid);
   end loop;

end add_abs_keywords;

-- ***********************************************************************************

procedure add_abstracts (recs abs_recs_type) is

cursor	C1 (code$ varchar2) is
select	presentation_id
from	conf_presentations
where	conference_id		= gbl_conf_id
and	presentation_code	= code$;

cursor	C2 (code$ varchar2) is
select	classification_id
from	conf_classifications
where	conference_id		= gbl_conf_id
and	class_code		= code$;

cursor	C3 (code$ varchar2) is
select	sub_class_id
from	conf_sub_classes
where	conference_id		= gbl_conf_id
and	sub_class_code		= code$;

cursor	C4 (cid$ integer, sid$ integer) is
select	eclass_id
from	conf_eclasses
where	conference_id		= gbl_conf_id
and	classification_id	= cid$
and	nvl(sub_class_id,0)	= nvl(sid$,0);

pid	integer;
cid	integer;
sid	integer;
eid	integer;

begin

   for i in 1..recs.COUNT loop
      pid := null;
      cid := null;
      sid := null;
      eid := null;

      open C1 (recs(i).presentation_code);
      fetch C1 into pid;
      close C1;

      open C2 (recs(i).classification_code);
      fetch C2 into cid;
      close C2;

      open C3 (recs(i).sub_class_code);
      fetch C3 into sid;
      close C3;

      open C4 (cid, sid);
      fetch C4 into eid;
      close C4;

      insert into conf_abstracts values
		(null, gbl_conf_id, recs(i).paper_id, recs(i).title, pid, cid, sid, eid);
   end loop;

end add_abstracts;

-- ***********************************************************************************

procedure add_registrants (IDs registrants_type) is

ID_rec		id_type;
checked_IDs	registrants_type;
j integer;

begin

   checked_IDs := IDs;
   for i in 1..checked_IDs.COUNT loop
      ID_rec.person_ID	:= checked_IDs(i);
      fail_safe		:= 0;
      ID_rec.check_date	:= to_date ('01/01/1000', 'mm/dd/yyyy');
      check_merged_persons (ID_rec);
      checked_IDs(i)	:= ID_rec.person_ID;
   end loop;

   FORALL i in 1..IDs.COUNT
      insert into conf_registrants (conference_id, person_id)
	values (gbl_conf_id, checked_IDs(i));

end add_registrants;

-- ***********************************************************************************
-- ***********************************************************************************

end repatriate;
/
show error

set define on

--	Copyright (C) 2004 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>