--	Copyright (C) 2004 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>

set define off

create or replace PACKAGE repatriate as

-- NOTE: This procedure exists only on the conference repository

gbl_conf_id	integer;

type	conf_rec_type is record (
		conference_code		conf_conferences.conference_code%TYPE,
		conference_title	conf_conferences.conference_title%TYPE,
		start_date		conf_conferences.start_date%TYPE,
		end_date		conf_conferences.end_date%TYPE,
		location		conf_conferences.location%TYPE);

type	abs_rec_type is record (
		presentation_code	abstracts.presentation_code%TYPE,
		classification_code	abstracts.classification_code%TYPE,
		sub_class_code		abstracts.sub_class_code%TYPE,
		paper_id		abstract_sessions.paper_id%TYPE,
		title			abstracts.title%TYPE);
type	abs_recs_type is table of abs_rec_type index by binary_integer;

type	presentation_type is record (
		presentation_code	presentation_codes.presentation_code%TYPE,
		presentation_descr	presentation_codes.presentation_descr%TYPE,
		presentation_type_code	presentation_codes.presentation_type_code%TYPE);

type	presentations_type	is table of presentation_type index by binary_integer;
type	classifications_type	is table of classification_codes%ROWTYPE index by binary_integer;
type	sub_classes_type	is table of sub_class_codes%ROWTYPE index by binary_integer;
type	eclasses_type		is table of eclass_codes%ROWTYPE index by binary_integer;
type	keywords_type		is table of keyword_codes%ROWTYPE index by binary_integer;
type	registrants_type	is table of integer index by binary_integer;

type	ID_type	is record (
		function_id	conf_functions.function_id%TYPE,
		abstract_id	conf_abstracts.abstract_id%TYPE,
		person_id	persons.person_id%TYPE,
		profile_id	profiles.profile_id%TYPE,
		address_id	addresses.address_id%TYPE,
		affiliation_id	affiliations.affiliation_id%TYPE,
		check_date	date);

type	function_type is record (
		function_code	conf_functions.function_code%TYPE,
		function_descr	function_codes.function_descr%TYPE);
type	functions_type		is table of function_type index by binary_integer;

type	role_type is record (
		function_code	conf_functions.function_code%TYPE,
		person_id	persons.person_id%TYPE,
		paper_id	abstract_sessions.paper_id%TYPE,
		profile		profiles%ROWTYPE,
		person		persons%ROWTYPE,
		address		addresses%ROWTYPE,
		affiliation	affiliations%ROWTYPE);
type	roles_type is table of role_type index by binary_integer;

type	abs_keyword_type is record (
		keyword_code	keyword_codes.keyword_code%TYPE,
		paper_id	abstract_sessions.paper_id%TYPE);
type	abs_keywords_type is table of abs_keyword_type index by binary_integer;

procedure add_presentations (recs presentations_type);
procedure add_classifications (recs classifications_type);
procedure add_sub_classes (recs sub_classes_type);
procedure add_eclasses (recs eclasses_type);
procedure add_keywords (recs keywords_type);
procedure add_functions (recs functions_type) ;
procedure add_roles (recs roles_type);
procedure add_abs_keywords (recs abs_keywords_type);
procedure add_abstracts (recs abs_recs_type);
procedure add_registrants (IDs registrants_type);
procedure delete_conference (conf_id integer);
procedure set_global_conference_id (conf_id integer);
procedure set_repatriated_date;

end repatriate;
/
show error

set define on