set define off

create or replace package body repatriate_search as

rpt_title$	constant varchar2(30)	:= 'Repatriate Search';
new_search$	constant varchar2(100)	:= htf.anchor ('repatriate_search.html',
							'New Search') || htf.para;
Q$		constant varchar2(1)	:= chr(39);
Q2$		constant varchar2(2)	:= Q$||Q$;

spk$		constant varchar2(30)	:= 'Speaker/Presenter';
auth$		constant varchar2(30)	:= 'Author';
affil$		constant varchar2(30)	:= 'Affiliation';
present$	constant varchar2(30)	:= 'Presentation';
main$		constant varchar2(30)	:= 'Main Classification';
conf$		constant varchar2(30)	:= 'Conference';

abbrev$		constant varchar2(1)	:= 'Y';
full$		constant varchar2(1)	:= 'N';

type		sort_type is varray(6) of varchar2(30);
sort_opts$	sort_type := sort_type (spk$, auth$, affil$, present$, main$, conf$);

sql_stmt	long;

-- *******************************************************************************
-- *******************************************************************************

function get_sort_name (abs_id integer, code varchar2) return varchar2 is

cursor	driver is
select	sort_lname || sort_fname || sort_mname	sname
from	persons		p,
	profiles	pro,
	conf_roles	r
where	r.abstract_id	= abs_id
and	r.function_code	= code
and	pro.profile_id	= r.profile_id
and	p.person_id	= pro.person_id;

driver_rec	driver%ROWTYPE;

begin

   open driver;
   fetch driver into driver_rec;
   close driver;

   if driver_rec.sname is null then
      if code = repatriate.speaker$ then
         return get_sort_name (abs_id, repatriate.presenter$);
      end if;
   end if;

   return driver_rec.sname;

end get_sort_name;

-- *******************************************************************************

function get_sort_affil (abs_id integer, code varchar2) return varchar2 is

cursor	driver is
select	common.get_profile_affil (profile_id, 'Y') affil_name
from	conf_roles
where	abstract_id	= abs_id
and	function_code	= code;

driver_rec	driver%ROWTYPE;

begin

   open driver;
   fetch driver into driver_rec;
   close driver;

   return driver_rec.affil_name;

end get_sort_affil;

-- *******************************************************************************

function get_conf_sdate (ID integer) return varchar2 is

cursor	driver is
select	to_char (start_date, 'yyyymmdd')
from	conf_conferences
where	conference_id = ID;

dte	varchar2(8);

begin

   open driver;
   fetch driver into dte;
   close driver;

   return dte;

end get_conf_sdate;

-- *******************************************************************************
-- *******************************************************************************

function fmt_conf (driver_rec conf_conferences%ROWTYPE,
			short_form boolean default false) return varchar2 is

begin

   if short_form then
      return driver_rec.conference_title;
   end if;

   return driver_rec.conference_title || ', ' ||
		driver_rec.location || ', ' ||
		to_char (driver_rec.start_date, common.date_input_fmt_str$) || ' - ' ||
		to_char (driver_rec.end_date, common.date_input_fmt_str$);

end fmt_conf;

-- *******************************************************************************

function fmt_conf (conf_id integer, short_form boolean default false) return varchar2 is

cursor	driver is
select	*
from	conf_conferences
where	conference_id = conf_id;

driver_rec	driver%ROWTYPE;

begin

   open driver;
   fetch driver into driver_rec;
   close driver;

   return fmt_conf (driver_rec, short_form);

end fmt_conf;

-- *******************************************************************************
-- *******************************************************************************

function get_name (abs_id integer, code varchar2, abbrev_only varchar2) return varchar2 is

cursor	driver is
select	profile_id
from	conf_roles
where	abstract_id	= abs_id
and	function_code	= code;

profile_id	integer := null;

begin

   open driver;
   fetch driver into profile_id;
   close driver;

   if profile_id is null then
      return null;
   end if;

   return common.get_user_name_affiliation (profile_id, abbrev_only);

end get_name;

-- *******************************************************************************

function get_names (abs_id integer, abbrev_only varchar2) return varchar2 is

author	long := get_name (abs_id, repatriate.author$, abbrev_only);
name	long := get_name (abs_id, repatriate.presenter$, abbrev_only);
label	varchar2(30) := 'Presenter: ';

begin

   if name is null then
      name	:= get_name (abs_id, repatriate.speaker$, abbrev_only);
      label	:= 'Speaker: ';
   end if;

   if name = author then
      label := 'Author/' || label;
      return label || name;
   end if;


   return 'Author: ' || name || htf.br || label || name;

end get_names;

-- *******************************************************************************
-- *******************************************************************************

function get_type (ID integer) return varchar2 is

cursor	driver is
select	decode (presentation_type, common.oral$, 'Oral/', 'Poster/') ||
	presentation_descr DESCR
from	conf_presentations
where	presentation_id = ID;

driver_rec	driver%ROWTYPE;

begin

   open driver;
   fetch driver into driver_rec;
   close driver;

   return driver_rec.descr;

end get_type;

-- *******************************************************************************

procedure show_coauthors (abs_id integer) is

cursor	driver is
select	common.get_user_name_affiliation (cr.profile_id, full$) name
from	persons		p,
	profiles	pro,
	conf_roles	cr
where	cr.abstract_id	= abs_id
and	cr.function_code	= repatriate.coauthor$
and	pro.profile_id	= cr.profile_id
and	p.person_id	= pro.person_id
order by
	p.sort_lname, p.sort_fname, p.sort_mname;

sep	varchar2(4) := null;

begin

   htp.tableRowOpen ('LEFT', 'TOP');
   htp.tableHeader ('Coauthors', 'RIGHT');
   htp.p ('<TD>');

   for driver_rec in driver LOOP
      htp.p (sep || driver_rec.name);
      sep := htf.br;
   end loop;

   htp.p ('</TD>');
   htp.tableRowClose;

end show_coauthors;

-- *******************************************************************************

procedure details (abs_id integer) is

cursor	driver is
select	*
from	conf_abstracts
where	abstract_id = abs_id;

driver_rec	driver%ROWTYPE;
name	long := get_name (abs_id, repatriate.presenter$, full$);
label	varchar2(30) := 'Presenter';

begin

   open driver;
   fetch driver into driver_rec;
   close driver;

   common.header (rpt_title$);
   htp.p (new_search$);
   htp.tableOpen;

   if name is null then
      name	:= get_name (abs_id, repatriate.speaker$, full$);
      label	:= 'Speaker';
   end if;

   common.show_data_row ('Paper ID',		driver_rec.paper_id);
   common.show_data_row ('Title',		driver_rec.title);
   common.show_data_row ('Type',		get_type (driver_rec.presentation_id) );
   common.show_data_row ('Conference',		fmt_conf (driver_rec.conference_id) );
   common.show_data_row ('Classification',	common.decode_value ('class_descr', 'conf_classifications',
							'classification_id', driver_rec.classification_id) );
   common.show_data_row ('Subclassification',	common.decode_value ('sub_class_descr', 'conf_sub_classes',
							'sub_class_id', driver_rec.sub_class_id) );
   common.show_data_row ('Author',		get_name (driver_rec.abstract_id, repatriate.author$, full$) );
   common.show_data_row (label,			name);
   show_coauthors (driver_rec.abstract_id);

   htp.tableClose;
   common.footer;

end details;

-- *******************************************************************************
-- *******************************************************************************

procedure fmt_sql_stmt (auth varchar2, affil varchar2, keyw varchar2, present varchar2,
			ptype varchar2, main varchar2, sub varchar2, role varchar2,
			conf varchar2) is

search_name$	varchar2(500)	:= replace (trim(auth),Q$,Q2$);
search_affil$	varchar2(500)	:= replace (trim(affil),Q$,Q2$);

begin

   sql_stmt := 'select /*+ INDEX (a) */ * from conf_abstracts a where 1 = 1 ';

   if search_name$ is not null then
      search_name$ := Q$ || search_name$ || Q$;
      sql_stmt := sql_stmt							||
			'and	abstract_id in '				||
			'(	select	/*+ INDEX (prof) INDEX (cr) */ '	||
			'	abstract_id '					||
			'	from	conf_roles	cr, '			||
			'		profiles	prof, '			||
			'		persons		p '			||
			'	where	p.sort_lname	= common.sort_text ('	|| search_name$ || ') ' ||
			'	and	prof.person_id	= p.person_id '		||
			'	and	cr.profile_id	= prof.profile_id '	||
			') ';
   end if;

   if search_affil$ is not null then
      search_affil$ := Q$ || common.fmt_search_string (search_affil$) || Q$;
      sql_stmt := sql_stmt					||
			'and abstract_id in ('			||
			'select	/*+ INDEX (cr) */ '		||
			'	abstract_id '		||
			'from	conf_roles 		cr, '	||
			'	profiles		prof, '	||
			'	addresses		addr, '	||
			'	affiliations		affil '	||
			'where	contains (affil.search_text, '	|| search_affil$ || ', 1) > 0 '||
			'and	addr.affiliation_id	= affil.affiliation_id '	||
			'and	prof.address_id		= addr.address_id '		||
			'and	cr.profile_id		= prof.profile_id '		||
			'and	cr.function_code		in (' || Q$ || repatriate.author$	|| Q$ || ', ' ||
								 Q$ || repatriate.presenter$	|| Q$ || ') ) ';
   end if;

   if keyw is not null then
      sql_stmt := sql_stmt					||
			'and abstract_id in ('			||
			'select	/*+ INDEX (cak) */ '		||
			'	cak.abstract_id '		||
			'from	conf_abs_keywords	cak, '	||
			'	conf_keywords		ck '	||
			'where	ck.keyword_descr	= '	|| Q$ || keyw || Q$ || ' ' ||
			'and	cak.keyword_id		= ck.keyword_id ) ';
   end if;

   if present is not null then
      sql_stmt := sql_stmt					||
			'and presentation_id in ('		||
			'select	presentation_id '		||
			'from	conf_presentations cp '		||
			'where	presentation_descr	= '	|| Q$ || present || Q$ || ' ' ||
			'and	cp.conference_id	= a.conference_id ) ';
   end if;

   if ptype is not null then
      sql_stmt := sql_stmt					||
			'and presentation_id in ('		||
			'select	presentation_id '		||
			'from	conf_presentations cp '		||
			'where	presentation_type	= '	|| Q$ || ptype || Q$ || ') ';
   end if;

   if main is not null then
      sql_stmt := sql_stmt					||
			'and classification_id in ('		||
			'select	classification_id '		||
			'from	conf_classifications cc '	||
			'where	cc.class_descr		= '	|| Q$ || present || Q$ || ' ' ||
			'and	cc.conference_id	= a.conference_id ) ';
   end if;

   if sub is not null then
      sql_stmt := sql_stmt				||
			'and sub_class_id in ('		||
			'select	sub_class_id '		||
			'from	conf_sub_classes cs '	||
			'where	sub_class_descr	= '	|| Q$ || sub || Q$ || ' ' ||
			'and	cs.conference_id	= a.conference_id ) ';
   end if;

   if role is not null then
      sql_stmt := sql_stmt			||
			'and abstract_id in ('	||
			'select	abstract_id '	||
			'from	conf_roles '	||
			'where	function_code	= ' || Q$ || role || Q$ || ') ';
   end if;

   if conf is not null then
      sql_stmt := sql_stmt || 'and conference_id = ' || conf || ' ';
   end if;

end fmt_sql_stmt;

-- *******************************************************************************

procedure fmt_sort_stmt (srt common.ident_arr) is

first_sort	boolean		:= true;
sep_chr		varchar2(1)	:= ' ';

begin

   for i in 1..srt.COUNT LOOP
      if srt(i) is not null then

         if first_sort then
            sql_stmt := sql_stmt || 'order by';
         else
            sep_chr := ',';
         end if;

         if srt(i) = auth$ then
            sql_stmt := sql_stmt || sep_chr ||
			'repatriate_search.get_sort_name (abstract_id, ' || Q$ || repatriate.author$ || Q$ || ')';
         elsif srt(i) = affil$ then
            sql_stmt := sql_stmt || sep_chr ||
			'repatriate_search.get_sort_affil (abstract_id, ' || Q$ || repatriate.author$ || Q$ || ')';
         elsif srt(i) = spk$ then
            sql_stmt := sql_stmt || sep_chr ||
			'repatriate_search.get_sort_name (abstract_id, ' || Q$ || repatriate.speaker$ || Q$ || ')';
         elsif srt(i) = present$ then
            sql_stmt := sql_stmt || sep_chr ||
			'common.decode_value (''presentation_descr'', ''conf_presentations'', ''presentation_id'', presentation_id)';
         elsif srt(i) = main$ then
            sql_stmt := sql_stmt || sep_chr ||
			'common.decode_value (''class_descr'', ''conf_classifications'', ''classification_id'', classification_id)';
         elsif srt(i) = 'Sub Classification' then
            sql_stmt := sql_stmt || sep_chr ||
			'common.decode_value (''sub_class_descr'', ''conf_sub_classes'', ''sub_class_id'', sub_class_id)';
         elsif srt(i) = conf$ then
            sql_stmt := sql_stmt || sep_chr ||
			'repatriate_search.get_conf_sdate (abstract_id)';
         end if;

         first_sort := false;
      end if;
   end loop;

end fmt_sort_stmt;

-- *******************************************************************************
-- *******************************************************************************

procedure results (auth varchar2, affil varchar2, keyw varchar2, present varchar2,
			ptype varchar2, main varchar2, sub varchar2, role varchar2,
			conf varchar2, srt common.ident_arr) is

type		cursor_type is REF CURSOR;
driver		cursor_type;
driver_rec	conf_abstracts%ROWTYPE;
class_descr	conf_classifications.class_descr%TYPE;
present_descr	conf_presentations.presentation_descr%TYPE;
icount		integer := 0;

begin

   fmt_sql_stmt (auth, affil, keyw, present, ptype, main, sub, role, conf);
   fmt_sort_stmt (srt);
   common.header (rpt_title$);
   htp.p (new_search$);

   htp.tableOpen ('BORDER');
   htp.tableRowOpen ('LEFT', 'BOTTOM');
   htp.tableHeader ('ID');
   htp.tableHeader ('Title');
   htp.tableHeader (auth$ || '/' || spk$);
   htp.tableHeader ('Type');
   htp.tableHeader (conf$);
   htp.tableHeader (main$);
   htp.tableRowClose;

   open driver for sql_stmt;
   LOOP
      fetch driver into driver_rec;
      exit when driver%NOTFOUND;

      class_descr	:= common.decode_value ('class_descr', 'conf_classifications', 'classification_id', driver_rec.classification_id);
      present_descr	:= common.decode_value ('presentation_descr', 'conf_presentations', 'presentation_id', driver_rec.presentation_id);

      htp.tableRowOpen ('LEFT', 'TOP');
      htp.tableData (htf.anchor ('repatriate_search.details?abs_id=' || driver_rec.abstract_id, driver_rec.paper_id) );
      htp.tableData (driver_rec.title);
      htp.tableData (get_names (driver_rec.abstract_id, abbrev$) );
      htp.tableData (present_descr);
      htp.tableData (fmt_conf (driver_rec.conference_id, true));
      htp.tableData (class_descr);
      htp.tableRowClose;

      icount := icount + 1;
   end loop;
   close driver;

   htp.tableClose;
   htp.para;

   htp.bold ('Records Found:');
   htp.bold (to_char (icount, 'FM999,999'));
   common.footer;

end results;

-- *******************************************************************************
-- *******************************************************************************

procedure roles (role varchar2, conf varchar2) is

cursor	driver is
select	p.sort_lname, p.sort_fname, p.sort_mname, cr.function_code, cr.conference_id,
	common.get_user_name (p.person_id) name
from	persons			p,
	conf_roles		cr
where	cr.function_code		like nvl (role, '%')
and	cr.conference_id	like nvl (conf, '%')
and	p.person_id		= cr.person_id
UNION
select	p.sort_lname, p.sort_fname, p.sort_mname, cr.function_code, cr.conference_id,
	common.get_user_name (p.person_id) name
from	persons			p,
	profiles		pro,
	conf_roles		cr
where	cr.function_code		like nvl (role, '%')
and	cr.conference_id	like nvl (conf, '%')
and	pro.profile_id		= cr.profile_id
and	p.person_id		= pro.person_id
order by
	sort_lname, sort_fname, sort_mname;

function_descr	function_codes.function_descr%TYPE;

begin

   common.header (rpt_title$);
   htp.p (new_search$);
   htp.tableOpen ('BORDER');
   htp.tableRowOpen ('LEFT', 'BOTTOM');
   htp.tableHeader ('Name');
   htp.tableHeader ('Role');
   htp.tableHeader (conf$);
   htp.tableRowClose;

   for driver_rec in driver LOOP
      function_descr := common.decode_value ('function_descr', 'function_codes', 'function_code', driver_rec.function_code);
      htp.tableRowOpen ('LEFT', 'TOP');
      htp.tableData (driver_rec.name);
      htp.tableData (function_descr);
      htp.tableData (fmt_conf (driver_rec.conference_id, true));
      htp.tableRowClose;
   end loop;

   htp.tableClose;
   common.footer;

end roles;

-- *******************************************************************************
-- *******************************************************************************

procedure keywords_popup is

cursor	driver is
select	distinct keyword_descr
from	conf_keywords
order by lower (keyword_descr);

begin

   htp.tableRowOpen ('LEFT', 'TOP');
   htp.tableHeader ('Keyword', 'RIGHT');

   htp.p ('<TD>');
   htp.formSelectOpen ('keyw');
   htp.formSelectOption (null, 'SELECTED');

   for driver_rec in driver LOOP
      htp.formSelectOption (driver_rec.keyword_descr);
   end loop;

   htp.formSelectClose;
   htp.p ('</TD>');
   htp.tableRowClose;

end keywords_popup;

-- *******************************************************************************
-- *******************************************************************************

procedure main_classifications_popup is

cursor	driver is
select	distinct class_descr
from	conf_classifications;

begin

   htp.tableRowOpen ('LEFT', 'TOP');
   htp.tableHeader (main$, 'RIGHT');

   htp.p ('<TD>');
   htp.formSelectOpen ('main');
   htp.formSelectOption (null, 'SELECTED');

   for driver_rec in driver LOOP
      htp.formSelectOption (driver_rec.class_descr);
   end loop;

   htp.formSelectClose;
   htp.p ('</TD>');
   htp.tableRowClose;

end main_classifications_popup;

-- *******************************************************************************

procedure presentations_popup is

cursor	driver is
select	distinct presentation_descr
from	conf_presentations;

begin

   htp.tableRowOpen ('LEFT', 'TOP');
   htp.tableHeader (present$, 'RIGHT');

   htp.p ('<TD>');
   htp.formSelectOpen ('present');
   htp.formSelectOption (null, 'SELECTED');

   for driver_rec in driver LOOP
      htp.formSelectOption (driver_rec.presentation_descr);
   end loop;

   htp.formSelectClose;
   htp.p ('</TD>');
   htp.tableRowClose;

-- **********

   htp.tableRowOpen ('LEFT', 'TOP');
   htp.tableHeader ('Orals/Posters', 'RIGHT');

   htp.p ('<TD>');
   htp.formSelectOpen ('ptype');
   htp.formSelectOption (null, 'SELECTED');
   htp.formSelectOption ('All Orals', cattributes=>'VALUE="' || common.oral$ || '"');
   htp.formSelectOption ('All Posters', cattributes=>'VALUE="' || common.poster$ || '"');
   htp.formSelectClose;
   htp.p ('</TD>');
   htp.tableRowClose;

end presentations_popup;

-- *******************************************************************************

procedure sub_classes_popup is

cursor	driver is
select	distinct sub_class_descr
from	conf_sub_classes;

begin

   htp.tableRowOpen ('LEFT', 'TOP');
   htp.tableHeader ('Sub Classification', 'RIGHT');

   htp.p ('<TD>');
   htp.formSelectOpen ('sub');
   htp.formSelectOption (null, 'SELECTED');

   for driver_rec in driver LOOP
      htp.formSelectOption (driver_rec.sub_class_descr);
   end loop;

   htp.formSelectClose;
   htp.p ('</TD>');
   htp.tableRowClose;

end sub_classes_popup;

-- *******************************************************************************

procedure roles_popup (abs_only varchar2) is

cursor	driver is
select	function_code, function_descr
from	function_codes
where	abstract_flag	like abs_only
order by function_descr;

begin

   htp.tableRowOpen ('LEFT', 'TOP');
   htp.tableHeader ('Role', 'RIGHT');

   htp.p ('<TD>');
   htp.formSelectOpen ('role');
   htp.formSelectOption (null, 'SELECTED');

   for driver_rec in driver LOOP
      htp.formSelectOption (driver_rec.function_descr,
		cattributes=>'VALUE="' || driver_rec.function_code || '"');
   end loop;

   htp.formSelectClose;
   htp.p ('</TD>');
   htp.tableRowClose;

end roles_popup;

-- *******************************************************************************

procedure conferences_popup is

cursor	driver is
select	*
from	conf_conferences
order by start_date;

begin

   htp.tableRowOpen ('LEFT', 'TOP');
   htp.tableHeader (conf$, 'RIGHT');

   htp.p ('<TD>');
   htp.formSelectOpen ('conf');
   htp.formSelectOption (null, 'SELECTED');

   for driver_rec in driver LOOP
      htp.formSelectOption (fmt_conf (driver_rec),
		cattributes=>'VALUE="' || driver_rec.conference_id || '"' );
   end loop;

   htp.formSelectClose;
   htp.p ('</TD>');
   htp.tableRowClose;

end conferences_popup;

-- *******************************************************************************

procedure sort_options is

type		sort_cnt is varray (3) of varchar2(3);
sort_pos$	sort_cnt := sort_cnt ('1st', '2nd', '3rd');

begin

   for i in 1..sort_pos$.count LOOP
      htp.tableRowOpen ('LEFT', 'TOP');
      htp.tableHeader (sort_pos$(i) || ' Sort', 'RIGHT');

      htp.p ('<TD>');
      htp.formSelectOpen ('srt');
      htp.formSelectOption (null, 'SELECTED');

      for j in 1..sort_opts$.count LOOP
         htp.formSelectOption (sort_opts$(j));
      end loop;

      htp.formSelectClose;
      htp.p ('</TD>');
      htp.tableRowClose;
   end loop;

end sort_options;

-- *******************************************************************************

procedure abstract_search_form is

begin

   htp.formOpen ('repatriate_search.results', 'POST', cattributes=>'NAME="SPMS"');
   htp.tableOpen;
   common.show_data_row (null, htf.bold ('Search By Contribution'));

   common.show_data_row (auth$,		htf.formText ('auth',  25, 100) );
   common.show_data_row (affil$,	htf.formText ('affil', 50, 200) );
   keywords_popup;
   presentations_popup;
   main_classifications_popup;
   sub_classes_popup;
   roles_popup ('Y');
   conferences_popup;

   common.show_data_row (htf.formReset, htf.formSubmit(null, 'Search') );

   sort_options;
   htp.tableClose;
   htp.formClose;
   common.form_focus ('SPMS', 'auth');

end abstract_search_form;

-- *******************************************************************************

procedure role_search_form is

begin

   htp.formOpen ('repatriate_search.roles', 'POST');
   htp.tableOpen;
   common.show_data_row (null, htf.bold ('Search By Role'));

   roles_popup ('%');
   conferences_popup;

   common.show_data_row (htf.formReset, htf.formSubmit(null, 'Search') );
   htp.tableClose;
   htp.formClose;

end role_search_form;

-- *******************************************************************************

procedure html is

begin

   common.header (rpt_title$);
   htp.tableOpen;
   htp.tableRowOpen ('LEFT', 'TOP');

   htp.p ('<TD>');
   abstract_search_form;
   htp.p ('</TD>');

   htp.p ('<TD ALIGN="RIGHT">');
   role_search_form;
   htp.p ('</TD>');

   htp.tableRowClose;
   htp.tableClose;
   common.footer;

end html;

-- *******************************************************************************

procedure html (pid varchar2) is

begin

   common.gbl_pid := common.decrypt (pid);
   common.write_cookie (common.cookie_name$, common.gbl_pid);
   html;

end html;

-- *******************************************************************************
-- *******************************************************************************

end repatriate_search;
/
show error

set define on

--	Copyright (C) 2004 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>
