set define off

create or replace package body repo_email as

rpt_title$	constant varchar2(50) := 'Email (Repository)';

oral_accept$	constant varchar2(1) := 'O';
poster_accept$	constant varchar2(1) := 'P';
generic$	constant varchar2(1) := 'G';
dummy$		constant varchar2(2) := '~~';

Include$	constant varchar2(1) := 'I';
Exclude$	constant varchar2(1) := 'X';

send_it$	constant varchar2(15):= 'Send the email';
view_it$	constant varchar2(15):= 'View an Example';
sql_stmt	varchar2(10000);

-- ********************************************************************************************
-- ********************************************************************************************

procedure init_variables is

begin

   email.variables(1)	:= '<<Chair''s Email>>';
   email.variables(2)	:= '<<Repository Name>>';
   email.variables(3)	:= '<<Editor''s Email>>';
   email.variables(4)	:= '<<Login URL>>';
   email.variables(5)	:= '<<Name, First>>';
   email.variables(6)	:= '<<Name, Last>>';
   email.variables(7)	:= '<<Name, Middle>>';
   email.variables(8)	:= '<<Name, Initials>>';

end init_variables;

-- ********************************************************************************************
-- ********************************************************************************************

function build_subq (pred varchar2, tab varchar2, field varchar2, list common.ident_arr,
			main_field varchar2 default 'person_id') return varchar2 is

begin

   if list.count = 0 then
      return null;
   end if;

   if list.count = 1 AND nvl (list(1), dummy$) = dummy$ then
      return null;
   end if;

   return common.eol$ || pred	|| ' '		|| main_field	|| ' in ' ||
		'(select '	|| main_field	||
		' from '	|| tab		||
		common.build_IN_list (' where ', list, field) || ') ';

end build_subq;

-- ********************************************************************************************

function build_subq2 (pred varchar2, tab varchar2, field varchar2,
			tab2 varchar2, field2 varchar2, list common.ident_arr,
			main_field varchar2 default 'person_id') return varchar2 is

begin

   if list.count = 0 then
      return null;
   end if;


   if list.count = 1 AND list(1) is null then
      return null;
   end if;

   return common.eol$ || pred	|| ' ' || main_field	|| ' in '	||
		'(select '	|| main_field		||
		' from '	|| tab			||
		build_subq ('where', tab2, field2, list, field) || ') ';

end build_subq2;

-- ********************************************************************************************

function build_subq3 ( tab  varchar2, field varchar2,
			tab2 varchar2, field2 varchar2,
			tab3 varchar2, field3 varchar2,
			list common.ident_arr) return varchar2 is

begin

   if list.count = 0 then
      return null;
   end if;


   if list.count = 1 AND list(1) is null then
      return null;
   end if;

   return common.eol$ || 'and person_id in '	||
			'(select person_id '	||
			'from '		|| tab	||
			build_subq2 ('where', tab2, field2, tab3, field3, list, field) || ') ';

end build_subq3;

-- ********************************************************************************************

procedure build_sql_stmt (this_conference common.ident_arr, this_membership common.ident_arr,
		this_affiliation common.ident_arr, this_country common.ident_arr,
		this_region common.ident_arr, this_affil_type common.ident_arr, attrib common.ident_arr) is

begin

   sql_stmt := 'select * from persons p ' 			|| 
		'where exists '					||
			'(select /*+ INDEX(pro) */ 1 '		||
			'from profiles pro '			||
			'where pro.person_id = p.person_id '	||
			'and pro.email_address is not null) ';

   sql_stmt := sql_stmt || common.build_IN_list (' and ', this_country, 'country_code');
   sql_stmt := sql_stmt || build_subq ('and', 'mailing_lists',	    'conference_code', this_conference);
   sql_stmt := sql_stmt || build_subq ('and', 'person_memberships', 'membership_code', this_membership);

   sql_stmt := sql_stmt || build_subq2 ('and', 'profiles', 'address_id', 'addresses', 'affiliation_id', this_affiliation);

   sql_stmt := sql_stmt || build_subq3 ('profiles', 'address_id', 'addresses', 'affiliation_id',
					'affiliations', 'affil_type_code', this_affil_type);

   sql_stmt := sql_stmt || build_subq3 ('profiles', 'address_id', 'addresses', 'country_code',
					'country_codes', 'region_code', this_region);
   sql_stmt := sql_stmt || build_subq ('and', 'person_attributes', 'attribute_code', attrib);


end build_sql_stmt;

-- ********************************************************************************************

function replace_variables (message long) return long is

buffer	long := message;

begin

   buffer := replace (buffer, email.variables(1),	common.chairs_email$);
   buffer := replace (buffer, email.variables(2),	common.conf_title$);
   buffer := replace (buffer, email.variables(3),	common.editor_email$);
   buffer := replace (buffer, email.variables(4),	common.server || common.vdir$ || 'profile.html');
   buffer := replace (buffer, email.variables(5),	driver_rec.fname);
   buffer := replace (buffer, email.variables(6),	driver_rec.lname);
   buffer := replace (buffer, email.variables(7),	driver_rec.mname);
   buffer := replace (buffer, email.variables(8),	driver_rec.initials);

   return buffer;

end replace_variables;

-- ********************************************************************************************

function log_email (this_conference common.ident_arr, this_membership common.ident_arr,
		this_affiliation common.ident_arr, this_country common.ident_arr,
		this_region common.ident_arr, this_affil_type common.ident_arr,
		subject varchar2, message varchar2) return integer is

email_id	integer;
msg_text	mail_blats.message%TYPE := trim (message) || common.eol$;
sep		varchar2(2) := ' ';

begin

   msg_text := msg_text || common.eol$ || htf.bold ('Mailing List(s):');
   for i in 2..this_conference.count LOOP
      msg_text := msg_text || sep || common.decode_value ('conference_descr', 'conference_codes',
							'conference_code', this_conference(i));
      sep := ', ';
   end loop;

   sep		:= ' ';
   msg_text	:= msg_text || common.eol$ || htf.bold ('Membership(s):');
   for i in 2..this_membership.count LOOP
      msg_text := msg_text || sep || common.decode_value ('membership_descr', 'membership_codes',
							'membership_code', this_membership(i));
      sep := ', ';
   end loop;

   sep		:= ' ';
   msg_text	:= msg_text || common.eol$ || htf.bold ('Affiliations:');
   for i in 2..this_affiliation.count LOOP
      msg_text := msg_text || sep || common.decode_value ('affiliation_descr', 'affiliation_codes',
							'affiliation_code', this_affiliation(i));
      sep := ', ';
   end loop;

   sep		:= ' ';
   msg_text	:= msg_text || common.eol$ || htf.bold ('Country(ies):');
   for i in 2..this_country.count LOOP
      msg_text := msg_text || sep || common.decode_value ('country_descr', 'country_codes',
							'country_code', this_country(i));
      sep := ', ';
   end loop;

   sep		:= ' ';
   msg_text	:= msg_text || common.eol$ || htf.bold ('Region(s):');
   for i in 2..this_region.count LOOP
      msg_text := msg_text || sep || common.decode_value ('region_descr', 'region_codes',
							'region_code', this_region(i));
      sep := ', ';
   end loop;

   sep		:= ' ';
   msg_text	:= msg_text || common.eol$ || htf.bold ('Affiliation Type(s):');
   for i in 2..this_affil_type.count LOOP
      msg_text := msg_text || sep || common.decode_value ('affil_type_descr', 'affil_type_codes',
							'affil_type_code', this_affil_type(i));
      sep := ', ';
   end loop;

-- **********

   select email_id_seq.nextval into email_id from dual;

   insert into emails (email_id, subject, message, timestamp)
	values (email_id, trim (subject), msg_text, sysdate);

   return email_id;

end log_email;

-- ********************************************************************************************

procedure step3 (this_conference common.ident_arr, this_membership common.ident_arr,
		this_affiliation common.ident_arr, this_country common.ident_arr,
		this_region common.ident_arr, this_affil_type common.ident_arr,
		attrib common.ident_arr, subject varchar2, message varchar2, submit varchar2) is

type		cursor_type is REF CURSOR;
driver		cursor_type;

buffer		long;
new_subject	mail_blats.subject%TYPE;
recipient	mail_blats.recipient%TYPE;
icount		integer	:= 0;
rec_found	boolean;
mail_id		integer;
next_email_id	integer;

now$		constant date := common.get_UTC_time;
user_email$	mail_blats.recipient%TYPE;

begin

   if common.header (rpt_title$) then
      init_variables;
      user_email$ := common.get_user_email (nvl (common.gbl_priv_pid, common.gbl_pid));
      build_sql_stmt (this_conference, this_membership, this_affiliation, this_country,
		this_region, this_affil_type, attrib);

      if submit = view_it$ then
         open driver for sql_stmt;
         fetch driver into driver_rec;
         rec_found := driver%FOUND;
         close driver;

         new_subject := replace_variables (subject);

         if rec_found then
            buffer := replace_variables (message);
            buffer := replace (buffer, common.eol$, htf.br);
            buffer := replace (buffer, dummy$, '"');

            htp.p ('Subject: ' || new_subject);
            htp.para;
            htp.p (buffer);
            htp.hr;
            htp.header (3, 'Use the browser''s back button to submit the email.');
            htp.para;
         else
            htp.header (3, 'No profiles meet your search criteria.');
         end if;

      else
         next_email_id := log_email (this_conference, this_membership, this_affiliation,
			this_country, this_region, this_affil_type, subject, message);

         open driver for sql_stmt;
         LOOP
            fetch driver into driver_rec;
            exit when driver%NOTFOUND;

            new_subject	:= replace_variables (subject);
            buffer	:= replace_variables (message);
            recipient	:= common.get_user_email (driver_rec.person_id);
            mail_id	:= send_mail2 (recipient, new_subject, buffer, user_email$,
					pid=>driver_rec.person_id);
            icount	:= icount + 1;

            insert into email_logs (person_id, email_id, mail_id)
		values (driver_rec.person_id, next_email_id, mail_id);
            end loop;
            close driver;

         if icount = 0 then
            delete from emails where email_id = next_email_id;
         end if;

         htp.header (3, icount || ' Email(s) sent');
         htp.formOpen ('repository.html');
         htp.formSubmit (null, 'Repository Home Page');
         htp.formClose;

         htp.formOpen ('repo_email.html');
         htp.formSubmit (null, 'Send Another Email');
         htp.formClose;
      end if;

      common.footer;
   end if;

end step3;

-- ********************************************************************************************
-- ********************************************************************************************

function valid_input (subject varchar2, message varchar2) return boolean is

valid	boolean := true;

begin

   if subject is null then
      htp.header (3, 'Please supply a subject for your email.');
      valid := false;
   end if;

   if replace (replace (message, common.eol$, null), chr(13), null) is null then
      htp.header (3, 'Please supply a message for your email.');
      valid := false;
   end if;

   return valid;

end valid_input;

-- ********************************************************************************************

procedure step2 (this_conference common.ident_arr, this_membership common.ident_arr,
		this_affiliation common.ident_arr, this_country common.ident_arr,
		this_region common.ident_arr, this_affil_type common.ident_arr,
		attrib common.ident_arr, subject varchar2, message varchar2) is

type		cursor_type is REF CURSOR;
driver		cursor_type;
icount		integer;

begin

   if common.header (rpt_title$) then
      if valid_input (subject, message) then
         htp.p ( email.format_message ('SUBJECT: ' || subject));
         htp.hr;
         htp.p ( email.format_message (message));

         htp.formOpen ('repo_email.step3', 'POST');
         common.hidden_array ('this_conference',	this_conference);
         common.hidden_array ('this_membership',	this_membership);
         common.hidden_array ('this_affiliation',	this_affiliation);
         common.hidden_array ('this_country',		this_country);
         common.hidden_array ('this_region',		this_region);
         common.hidden_array ('this_affil_type',	this_affil_type);
         common.hidden_array ('attrib',			attrib);

         htp.formHidden ('subject',		replace (subject, '"', dummy$));
         htp.formHidden ('message',		replace (message, '"', dummy$));

         htp.para;
         htp.formSubmit ('submit', send_it$);
         htp.formSubmit ('submit', view_it$);
         htp.formClose;

         build_sql_stmt (this_conference, this_membership, this_affiliation, this_country,
		this_region, this_affil_type, attrib);
         sql_stmt := replace (sql_stmt, 'select * from persons', 'select count(1) from persons');

         open driver for sql_stmt;
         fetch driver into icount;
         close driver;

         htp.para;
         htp.bold ('Your selection criteria will generate ' || to_char (icount, '999,999') || ' emails.');

      end if;

      common.footer;
   end if;

end step2;

-- ********************************************************************************************
-- ********************************************************************************************

procedure show_option (prompt varchar2, code varchar2, wclaws varchar2 default null) is

begin

   htp.p (common.font2 (prompt));
   htp.br;
   common.show_popup_list (code, where_claws=>wclaws, multiple=>true);
   htp.para;

end show_option;

-- ********************************************************************************************

procedure show_options is

begin

   htp.header (3, 'Select users to include.');

   htp.tableOpen;
   htp.tableRowOpen ('LEFT', 'TOP');
   htp.p ('<TD>');
      show_option ('Mailing List',	'conference', 'where discontinued is null');
   htp.p ('</TD>');
   htp.p ('<TD>');
      show_option ('Membership',	'membership');
   htp.p ('</TD>');
   htp.tableRowClose;
   htp.tableClose;

   htp.tableOpen;
   htp.tableRowOpen ('LEFT', 'TOP');
   htp.p ('<TD>');
      show_option ('Affiliations',	'affiliation');
   htp.p ('</TD>');
   htp.tableRowClose;
   htp.tableClose;

   htp.tableOpen;
   htp.tableRowOpen ('LEFT', 'TOP');

   htp.p ('<TD>');
      show_option ('Country',		'country');
   htp.p ('</TD>');

   htp.p ('<TD>');
      show_option ('Region',		'region');
   htp.p ('</TD>');

   htp.p ('<TD>');
      show_option ('Affiliation Type',	'affil_type');
   htp.p ('</TD>');

   htp.p ('<TD>');
      htp.formHidden ('attrib', dummy$);
      htp.tableOpen;
      htp.tableRowOpen ('LEFT');
      htp.tableHeader ('Select Attributes', ccolspan=>'2');
      htp.tableRowCLose;

      jacow.abs_attrib_fields;
      htp.tableClose;
   htp.p ('</TD>');

   htp.tableRowClose;
   htp.tableClose;

   htp.hr;

end show_options;

-- ********************************************************************************************
-- ********************************************************************************************

procedure html is

begin

   if common.header (rpt_title$) then
      init_variables;
      htp.anchor ('repository.html', 'Back To Repository Page');
      htp.para;
      htp.formOpen ('repo_email.step2', 'POST');

      show_options;

      htp.bold ('Subject: ');
      htp.formText ('subject', 100, 1000);

      htp.header (3, 'Enter the email message below.  Use the substitution parameters below.');
      htp.formTextArea ('message', 15, 75);

      htp.para;
      htp.formSubmit;
      htp.formClose;
      htp.hr;

      email.show_variables;
      common.footer;
   end if;

end html;

-- ********************************************************************************************

end repo_email;
/
show error

set define on

--	Copyright (C) 2004 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>
