set define off

Create or Replace package body reports as

rpt_title$	constant varchar2(20)	:= 'JACoW Reports';
rpt_title2$	constant varchar2(60)	:= 'Classifications &amp; Sub-classifications';

source_file$	constant file_type_codes.file_type_code%type := 'SRC';

gbl_abs_count	integer;
descr		common.ident_arr;
total		common.int_arr;
io_count	common.int_arr;
co_count	common.int_arr;
po_count	common.int_arr;

hex_red$	constant char(6)	:= 'FF0000';
hex_green$	constant char(6)	:= '00FF00';
hex_yellow$	constant char(6)	:= 'FFFF00';

io_total	integer := 0;
co_total	integer := 0;
po_total	integer := 0;

cursor	green_dots (this_code varchar2) is
select	a.abstract_id, a.title, common.get_paper_id (a.abstract_id) paper_id
from	sessions		s,
	abstract_sessions	sa,
	abstracts		a
where	a.status_code		in (select status_code from status_codes where publishable = 'Y')
and	not exists
(
	select	1
	from	logs		l
	where	l.abstract_id	= a.abstract_id
	and	l.log_code	= this_code
)
and	sa.abstract_id		= a.abstract_id
and	sa.primary_flag		= 'Y'
and	s.session_id		= sa.session_id
order by
	s.sort_order nulls last, s.session_date, s.beg_time, sa.session_id, sa.session_seq;

-- ***********************************************************************************
-- ***********************************************************************************

procedure write (buffer varchar2, eol boolean default false) is

begin

   htp.prn (buffer);
   if eol then
      htp.prn (common.crlf$);
   else
      htp.prn (common.tab$);
   end if;

end write;

-- ***********************************************************************************
-- ***********************************************************************************

procedure AIP is

cursor	driver is
select	/*+ INDEX(pro) INDEX(p) INDEX(auth) */
	abs.abstract_id, abs.title
from	persons			p,
	profiles		pro,
	authors			auth,
	abstracts		abs,
	presentation_codes	pc
where	abs.presentation_code	= pc.presentation_code
and	auth.abstract_id	= abs.abstract_id
and	auth.author_code	= common.speaker$
and	pro.profile_id		= auth.profile_id
and	p.person_id		= pro.person_id
order by
	pc.sort_order nulls last, p.sort_lname, p.sort_fname, p.sort_mname;

begin

   if common.header ('AIP Export') then
      for driver_rec in driver LOOP
         htp.p (driver_rec.title);
         htp.br;
         htp.p (common.get_author_list2 (driver_rec.abstract_id, initials_only=>'Y'));
         htp.br;
      end loop;
      common.footer;
   end if;

end AIP;

-- ***********************************************************************************
-- ***********************************************************************************

procedure registered is

cursor	driver is
select	a.abstract_id, a.title, common.get_paper_id (a.abstract_id) paper_id
from	abstracts		a,
	abstract_sessions	sa,
	sessions		s
where	sa.session_id		= s.session_id
and	sa.primary_flag		= 'Y'
and	a.abstract_id		= sa.abstract_id
order by
	s.sort_order nulls last, s.session_date, s.beg_time, s.session_id, sa.session_seq;

/* Taken from the editor routine.  Replace when merged with reports.sql */

cursor	C2 (abs_id integer) is
select	distinct
	p.sort_lname, p.sort_fname, p.sort_mname,
	common.get_user_name (aa.person_id) name
from	persons			p,
	abstract_authors	aa
where	aa.abstract_id	= abs_id
and	p.person_id	= aa.person_id
and	regist_maint.is_registered (aa.person_id)	= 'Y';


sepStr	varchar2(4);

/*
1, programme code
2, title of presentation
3, all co-authors
4, names of persons registered
*/

begin

   if common.header ('Registered Individuals by Contributuion') then
      htp.tableOpen ('BORDER');
      htp.tableRowOpen ('LEFT', 'BOTTOM');
      htp.tableHeader ('Paper ID');
      htp.tableHeader ('Title');
      htp.tableHeader ('Authors');
      htp.tableHeader ('Registered');
      htp.tableRowClose;

      for driver_rec in driver LOOP
         htp.tableRowOpen ('LEFT', 'TOP');
         htp.tableData (driver_rec.paper_id);
         htp.tableData (driver_rec.title);
         htp.tableData (common.get_author_list2 (driver_rec.abstract_id) );

         htp.p ('<TD NOWRAP>');
         sepStr := null;
         for C2_rec in C2 (driver_rec.abstract_id) LOOP
	    htp.p (sepStr || C2_rec.name);
            sepStr := htf.br;
         end loop;

         if sepStr is null then
            htp.p (common.space$);
         end if;
         htp.p ('</TD>');

         htp.tableRowClose;
      end loop;

      htp.tableClose;
      common.footer;
   end if;

end registered;

-- ***********************************************************************************
-- ***********************************************************************************

procedure QA_PDF is

cursor	driver is
select	l.abstract_id, l.person_id, l.log_date, a.paper_id
from	abstract_sessions	a,
	logs			l
where	l.log_code		= common.final_QA_passed$
and	trim(l.comments)	is not null
and	a.abstract_id		= l.abstract_id
and	a.primary_flag		= 'Y'
and	NOT EXISTS
(
	select	1
	from	logs	l2
	where	l2.abstract_id	= l.abstract_id
	and	l2.log_code	= common.file_uploaded$
	and	l2.log_date	between l.log_date - (5/1444) and common.get_UTC_time
)
order by l.person_id, a.session_id, a.session_seq;

begin

   if common.header (rpt_title$) then
      htp.tableOpen ('BORDER');

      htp.tableRowOpen ('LEFT', 'BOTTOM');
      htp.tableHeader ('Paper ID');
      htp.tableHeader ('Editor');
      htp.tableHeader ('Log');
      htp.tableRowClose;

      for driver_rec in driver LOOP
         htp.tableRowOpen ('LEFT', 'TOP');
         htp.tableData (driver_rec.paper_id);
         htp.tableData (common.get_user_name (driver_rec.person_id));
         common.show_abstract_log (driver_rec.abstract_id, true);
         htp.tableRowClose;
      end loop;

      htp.tableClose;
      common.footer;
   end if;

end QA_PDF;

-- ***********************************************************************************
-- ***********************************************************************************

procedure hc_nofiles is

cursor	driver is
select	a.abstract_id, sa.paper_id, a.title,
	common.get_author_profile_id (a.abstract_id, common.speaker$) spk_profile_id
from	sessions		s,
	abstract_sessions	sa,
	abstracts		a
where	a.assignable		= 0
and	exists
(
	select	1
	from	logs		l
	where	l.abstract_id	= a.abstract_id
	and	l.log_code	= common.paper_copy_received$
)
and	sa.abstract_id		= a.abstract_id
and	sa.primary_flag		= 'Y'
and	s.session_id		= sa.session_id
order by
	s.sort_order nulls last, s.session_date, s.beg_time, s.session_id, sa.session_seq;

pid	integer;

begin

   if common.header (rpt_title$) then
      htp.tableOpen ('BORDER');

      htp.tableRowOpen ('LEFT', 'BOTTOM');
      htp.tableHeader ('Paper ID');
      htp.tableHeader ('Title');
      htp.tableHeader ('Speaker/Pesenter');
      htp.tableHeader ('Email');
      htp.tableRowClose;

      for driver_rec in driver LOOP
         pid := common.decode_value ('person_id', 'profiles', 'profile_id', driver_rec.spk_profile_id);

         htp.tableRowOpen ('LEFT', 'TOP');
         htp.tableData (driver_rec.paper_id);
         htp.tableData (driver_rec.title);
         htp.tableData (common.get_user_name (pid));
         htp.tableData (common.get_user_email (pid, true));
         htp.tableRowClose;
      end loop;

      htp.tableClose;
      common.footer;
   end if;

end hc_nofiles;

-- ***********************************************************************************
-- ***********************************************************************************

procedure show_abs_contacts (abs_id integer) is

cursor	driver is
select	ctype, profile_id, person_id
from	abstract_authors
where	abstract_id	= abs_id
and	ctype		in ('Author', 'Presenter', 'Speaker')
order by ctype;

sepstr	varchar2(10)	:= null;
prev_id	integer := 0;

begin

   htp.p ('<TD>');
   for driver_rec in driver LOOP
      if prev_id <> driver_rec.person_id then
	 htp.p (sepstr);
	 htp.p (common.get_user_name_affiliation (driver_rec.profile_id, 'Y'));
	 htp.p (common.get_user_email (driver_rec.person_id, true));
	 sepstr := htf.br;
      end if;
      prev_id := driver_rec.person_id;
   end loop;
   htp.p ('</TD>');

end show_abs_contacts;

-- ***********************************************************************************

procedure unreg_papers is

cursor	driver is
select	a.abstract_id, sa.paper_id, a.title, a.editor_id
from	sessions		s,
	abstract_sessions	sa,
	abstracts		a
where	sa.abstract_id(+)	= a.abstract_id
and	s.session_id(+)		= sa.session_id
order by
	s.sort_order nulls last, s.session_date, s.beg_time, s.session_id, sa.session_seq;

cursor	C2 (abs_id integer) is
select	1
from	abstract_authors
where	abstract_id	= abs_id
and	regist_maint.is_registered (person_id) = 'Y';

rec_found	boolean;
dummy		integer;
total		integer := 0;

begin

   if common.header ('Unregistered Contributions') then
      htp.tableOpen ('BORDER');

      htp.tableRowOpen ('LEFT', 'BOTTOM');
      htp.tableHeader ('Paper ID');
      htp.tableHeader ('Editor');
      htp.tableHeader ('Contacts');
      htp.tableHeader ('Title');
      htp.tableRowClose;

      for rec in driver LOOP
         open C2 (rec.abstract_id);
         fetch C2 into dummy;
         rec_found := C2%FOUND;
         close C2;

         if NOT rec_found then
            htp.tableRowOpen ('LEFT', 'TOP');
            htp.tableData (htf.anchor ('jacow.view_abstract?abs_id=' ||
			rec.abstract_id, nvl(rec.paper_id, rec.abstract_id)), cnowrap=>'NOWRAP');
            htp.tableData (nvl (common.get_user_name (rec.editor_id), common.space$), cnowrap=>'NOWRAP');
            show_abs_contacts (rec.abstract_id);
            htp.tableData (rec.title);
            htp.tableRowClose;

            total := total + 1;
         end if;
      end loop;

      htp.tableRowOpen ('LEFT', 'TOP');
      htp.tableHeader ('Total', 'RIGHT');
      htp.tableHeader (total, 'LEFT', ccolspan=>'3');
      htp.tableRowClose;

      htp.tableClose;
      common.footer;
   end if;

end unreg_papers;

-- ***********************************************************************************
-- ***********************************************************************************

procedure poster_counts is

cursor	driver (date$ date) is
select	count(distinct abstract_id)
from	abstracts
where	presentation_code in
(	select	presentation_code
	from	presentation_codes
	where	presentation_type_code = common.poster$
)
and
(	classification_code in
	(	select	classification_code
		from	class_sessions
		where	session_id in
		(	select	session_id
			from	sessions
			where	session_date = date$
	)	)
or	eclass_code in
	(	select	eclass_code
		from	class_sessions
		where	session_id in
		(	select	session_id
			from	sessions
			where	session_date = date$
	)	)
);

this_date	date := common.conf_start_monday$;
icount		integer;
total		integer := 0;

begin

   if common.header ('Poster Counts') then
      htp.header (3, 'Poster Counts by Date Via Main Classification');
      htp.tableOpen ('BORDER');

      htp.tableRowOpen ('CENTER', 'BOTTOM');
      htp.tableHeader ('Session Date', ccolspan=>'2');
      htp.tableHeader ('Count');
      htp.tableRowClose;

      for i in 1..common.conf_len_days$ LOOP
         open driver (this_date);
         fetch driver into icount;
         close driver;

         htp.tableRowOpen ('LEFT', 'TOP');
         htp.tableData (to_char (this_date, 'Day'));
         htp.tableData (to_char (this_date, common.date_input_fmt_str$));
         htp.tableData (icount, 'RIGHT');
         htp.tableRowClose;

         this_date := this_date + 1;
         total	:= total + icount;
      end loop;

      htp.tableRowOpen ('LEFT', 'TOP');
      htp.tableHeader ('Total Posters', 'RIGHT', ccolspan=>'2');
      htp.tableHeader (total, 'RIGHT');
      htp.tableRowClose;

      htp.tableClose;
      common.footer;
   end if;

end poster_counts;

-- ***********************************************************************************
-- ***********************************************************************************

procedure by_time is

cursor	driver is
select	s.session_date, sa.paper_id, a.title,
	common.get_author_profile_id (a.abstract_id, common.speaker$) spk_profile_id,
	nvl (sa.start_time, s.beg_time) beg_time
from	abstracts		a,
	abstract_sessions	sa,
	sessions		s
where	sa.session_id		= s.session_id
and	a.abstract_id		= sa.abstract_id
order by
	s.sort_order nulls last, s.session_date, s.beg_time, s.session_id, sa.session_seq;

begin

   if common.header (rpt_title$) then
      htp.tableOpen ('BORDER');

      htp.tableRowOpen ('LEFT', 'BOTTOM');
      htp.tableHeader ('Time');
      htp.tableHeader ('Paper&nbsp;ID');
      htp.tableHeader ('Title');
      htp.tableHeader ('Speaker');
      htp.tableRowClose;

      for driver_rec in driver LOOP
         htp.tableRowOpen ('LEFT', 'TOP');
         htp.tableData (to_char (driver_rec.session_date, common.date_input_fmt_str$) || common.space2$ ||
				common.format_time (driver_rec.beg_time), cnowrap=>'NOWRAP');
         htp.tableData (driver_rec.paper_id);
         htp.tableData (driver_rec.title);
         htp.tableData (common.get_user_name_affiliation (driver_rec.spk_profile_id));
         htp.tableRowClose;
      end loop;

      htp.tableClose;
      common.footer;
   end if;

end by_time;

-- ***********************************************************************************
-- ***********************************************************************************

procedure edit_time is

cursor	driver is
select	abstract_id, log_date, person_id
from	logs			L1
where	log_code		= common.paper_assigned$
and	log_date		=
(
	select	min (log_date)
	from	logs			l2
	where	l2.abstract_id		= l1.abstract_id
	and	l2.person_id		= l1.person_id
	and	l2.log_code		= common.paper_assigned$
);

cursor	C2 (aid integer, pid integer, ldate date) is
select	status_code, log_date
from	logs
where	abstract_id	= aid
and	person_id	= pid
and	log_code	= common.edit_complete$
and	log_date	>= ldate
and	trunc(log_date)	= trunc (ldate);

cursor	C3 is
select	*
from	status_codes;

C2_rec		C2%ROWTYPE;
rec_found	boolean;

type		stype is table of integer index by status_codes.status_code%TYPE;

min_time	stype;
max_time	stype;
tot_time	stype;
cnt_time	stype;
min_person	stype;
max_person	stype;
itime		integer;
code		common.ident_arr;
descr		common.ident_arr;

begin

   if common.header ('Minutes Spent Editing') then

      for C3_rec in C3 LOOP
         min_time (C3_rec.status_code)		:= 24*60;
         max_time (C3_rec.status_code)		:= 0;
         tot_time (C3_rec.status_code)		:= 0;
         cnt_time (C3_rec.status_code)		:= 0;
         min_person (C3_rec.status_code)	:= null;
         max_person (C3_rec.status_code)	:= null;
         code (code.count+1)			:= C3_rec.status_code;
         descr (descr.count+1)			:= C3_rec.status_descr;
      end loop;

      for driver_rec in driver LOOP
         open C2 (driver_rec.abstract_id, driver_rec.person_id, driver_rec.log_date);
         fetch C2 into C2_rec;
         rec_found := C2%FOUND;
         close C2;

         if rec_found then
            itime := (C2_rec.log_date - driver_rec.log_date) * 1440;
            tot_time (C2_rec.status_code)	:= tot_time (C2_rec.status_code) + itime;
            cnt_time (C2_rec.status_code)	:= cnt_time (C2_rec.status_code) + 1;

            if itime > max_time (C2_rec.status_code) then
               max_time (C2_rec.status_code) := itime;
               max_person (C2_rec.status_code) := driver_rec.person_id;
            elsif itime < min_time (C2_rec.status_code) then
               min_time (C2_rec.status_code) := itime;
               min_person (C2_rec.status_code) := driver_rec.person_id;
            end if;
         end if;

      end loop;

      htp.tableOpen ('BORDER');
      htp.tableRowOpen ('CENTER');
      htp.tableData (common.space$);
      htp.tableHeader ('Min', ccolspan=>'2');
      htp.tableHeader ('Max', ccolspan=>'2');
      htp.tableHeader ('Avg');
      htp.tableRowClose;

      for i in 1..descr.count LOOP
         htp.tableRowOpen ('CENTER', 'TOP');
         htp.tableData (descr (i));
         htp.tableData (min_time (code(i)));
         htp.tableData (common.get_user_name (min_person(code(i)) ) );
         htp.tableData (max_time (code(i)));
         htp.tableData (common.get_user_name (max_person(code(i)) ) );

         if cnt_time (code(i)) <> 0 then
            htp.tableData ( round (tot_time (code(i)) / cnt_time (code(i)) ) );
         else
            htp.tableData (common.space$);
         end if;
         htp.tableRowClose;
      end loop;

      htp.tableClose;
      common.footer;
   end if;

end edit_time;

-- ***********************************************************************************
-- ***********************************************************************************

procedure show_data_row (data1 varchar2, data2 integer, data3 integer, data4 integer) is

begin

   htp.tableRowOpen ('RIGHT', 'TOP');
   htp.tableData (data1, 'LEFT');
   htp.tableData (data2);
   htp.tableData (data3);
   htp.tableData (data4);
   htp.tableRowClose;

end show_data_row;

-- ***********************************************************************************

function get_abs_type_count (present_type varchar2, present_code varchar2) return integer is

cursor	driver is
select	count(1)
from	abstracts
where	presentation_code	like present_code
and	presentation_code	in
(
	select	presentation_code
	from	presentation_codes
	where	presentation_type_code like present_type
);

icount	integer;

begin

   open driver;
   fetch driver into icount;
   close driver;

   return icount;

end get_abs_type_count;

-- ************************************************************************************************

function get_abs_count (this_code logs.log_code%TYPE default 'AC') return integer is

cursor	driver is
select	count(1)
from	abstracts;

icount	integer;

begin

   open driver;
   fetch driver into icount;
   close driver;

   return icount;

end get_abs_count;

-- ************************************************************************************************

function show_pct (icount1 integer, icount2 integer) return varchar2 is

begin

   if nvl (icount2, 0) = 0 then
      return '---';
   end if;

   return to_char (round (100*icount1/icount2, 2), '999,999,990.00') || '%';

end show_pct;

-- ************************************************************************************************

function get_log_date (this_id integer, this_code varchar2) return date is

cursor	driver is
select	log_date
from	logs
where	abstract_id	= this_id
and	log_code		= this_code;

driver_rec	driver%ROWTYPE;

begin

   open driver;
   fetch driver into driver_rec;
   close driver;

   return driver_rec.log_date;

end get_log_date;

-- ************************************************************************************************
-- ************************************************************************************************

procedure dup_abs is

cursor	driver is
select	title, count(1) icount
from	abstracts
group by
	title
having	count(1) > 1;

cursor	c2 (this_title varchar2) is
select	*
from	abstracts
where	title = this_title;

first	boolean;

begin

   if common.header ('Duplicate Contributions') then
      htp.tableOpen ('BORDER', cattributes=>'CELLPADDING="5"');

      htp.tableRowOpen ('LEFT', 'BOTTOM');
      htp.tableHeader ('Count');
      htp.tableHeader ('View/Edit');
      htp.tableHeader ('Contribution Title');
      htp.tableHeader ('Author(s)');
      htp.tableRowClose;

      for driver_rec in driver LOOP
         htp.tableRowOpen ('LEFT', 'TOP');
         htp.tableData (driver_rec.icount, 'CENTER', crowspan=>driver_rec.icount*2);

         first := true;

         for c2_rec in c2 (driver_rec.title) LOOP
            if not first then
               htp.tableRowOpen ('LEFT', 'TOP');
            end if;

            htp.tableData (htf.anchor2 ('JACoW.edit_abstract?abs_id=' ||
			c2_rec.abstract_id, 'Edit ' ||
				nvl (common.get_paper_id (c2_rec.abstract_id), c2_rec.abstract_id),
			ctarget=>'_blank') );
            htp.tableData (driver_rec.title);
            htp.tableData (common.get_author_list2 (c2_rec.abstract_id));
            htp.tableRowClose;

            htp.tableRowOpen ('LEFT', 'TOP');
            htp.tableData (c2_rec.description, ccolspan=>'3');
            htp.tableRowClose;

            first := false;
         end loop;

         common.show_data_sep (4, common.space$);

      end loop;

      htp.tableClose;
      common.footer;
   end if;

end dup_abs;

-- ************************************************************************************************
-- ************************************************************************************************

procedure show_invitees is

cursor	driver is
select	person_id
from	persons
where	invitee = 'Y'
order by
	sort_lname, sort_fname, sort_mname;

person_ids	common.int_arr;

begin

   if common.header (rpt_title$ || ' Invitees') then
      open driver;
      fetch driver BULK COLLECT into person_ids;
      close driver;

      htp.anchor ('profile.find_invite', 'Add Invitees');
      htp.para;
      htp.formOpen ('profile.find_invite3', 'POST');
      profile.show_profile_header (true);
      profile.show_search_results (null, null, person_ids, true);
      htp.tableClose;
      htp.para;
      htp.formSubmit;
      htp.formClose;
      common.footer;
   end if;

end show_invitees;

-- ************************************************************************************************

procedure no_status is

cursor	driver is
select	a.*, common.get_paper_id (a.abstract_id) paper_id,
	common.get_author_profile_id (a.abstract_id, common.speaker$) spk_profile_id
from	sessions		s,
	abstract_sessions	sa,
	abstracts		a
where	a.status_code		is null
and	sa.abstract_id		= a.abstract_id
and	sa.primary_flag		= 'Y'
and	s.session_id		= sa.session_id
order by
	s.sort_order nulls last, s.session_date, s.beg_time, s.session_id, sa.session_seq;

speaker		varchar2(200);
email		profiles.email_address%TYPE;
pid		integer;

begin

   if common.header (rpt_title$) then
      htp.tableOpen ('BORDER');
      htp.tableRowOpen ('LEFT', 'BOTTOM');
      htp.tableHeader ('Paper ID');
      htp.tableHeader ('Speaker');
      htp.tableHeader ('Black Dot');
      htp.tableRowClose;

      for driver_rec in driver LOOP
         pid	:= common.decode_value ('person_id', 'profiles', 'profile_id', driver_rec.spk_profile_id);
         speaker:= common.get_user_name (pid);
         email	:= common.decode_value ('email_address', 'profiles', 'profile_id', driver_rec.spk_profile_id);

         htp.tableRowOpen ('LEFT', 'TOP');
         htp.tableData (driver_rec.paper_id);
         htp.tableData (htf.mailto (email, speaker));
         htp.tableData (to_char (get_log_date (driver_rec.abstract_id, 'BD'), 'mm/dd/yyyy hh24:mi'));
         htp.tableRowClose;
      end loop;

      htp.tableClose;
      common.footer;
   end if;

end no_status;

-- ************************************************************************************************

procedure show_edits_per_day is

cursor	driver is
select	distinct trunc (common.local_time_stamp(log_date)) log_date
from	logs
where	log_code		= common.edit_complete$;

cursor	C2 (dte date) is /* Total edits per Day */
select	count(1) icount
from	logs
where	log_code		= common.edit_complete$
and	dte			= trunc (common.local_time_stamp(log_date));

cursor	C3 (dte date) is /* 1st edit per day */
select	count(1)		icount
from	logs			L1
where	log_code		= common.edit_complete$
and	dte			= trunc (common.local_time_stamp(log_date))
and	log_date		=
(
	select	min (log_date)
	from	logs			l2
	where	l2.abstract_id		= l1.abstract_id
	and	l2.log_code		= common.edit_complete$
);

total	integer := 0;
icount2	integer;
icount3	integer;

begin

   htp.para;
   htp.tableOpen ('BORDER');

   htp.tableRowOpen ('RIGHT', 'BOTTOM');
   htp.tableHeader ('Date', 'LEFT', ccolspan=>'2');
   htp.tableHeader ('1st Edit');
   htp.tableHeader ('Running<BR>1st Edit', 'CENTER', ccolspan=>'2');

   htp.tableHeader ('Re-Edits');
   htp.tableHeader ('Total<BR>Edits');
   htp.tableRowClose;

   for driver_rec in driver LOOP
      htp.tableRowOpen ('RIGHT', 'BOTTOM');
      htp.tableData (to_char (driver_rec.log_date, 'Day'), 'LEFT');
      htp.tableData (to_char (driver_rec.log_date, common.date_input_fmt_str$));

      icount2 := 0;
      open C2 (driver_rec.log_date);
      fetch C2 into icount2;
      close C2;

      icount3 := 0;
      open C3 (driver_rec.log_date);
      fetch C3 into icount3;
      close C3;

      total := total + icount3;

      htp.tableData (icount3);
      htp.tableData (total);
      htp.tableData (show_pct (total, gbl_abs_count));
      htp.tableData (icount2-icount3);
      htp.tableData (icount2);
      htp.tableRowClose;
   end loop;

   htp.tableClose;

end show_edits_per_day;

-- ************************************************************************************************
-- ************************************************************************************************

procedure black_dot_received is

cursor	driver is
select	a.abstract_id, a.title, common.get_paper_id (a.abstract_id )paper_id
from	sessions		s,
	abstract_sessions	sa,
	abstracts		a
where	a.status_code		= '~' -- common.black_dot$
and	exists
(
	select	1
	from	logs		l
	where	l.abstract_id	= a.abstract_id
	and	l.log_code	= common.paper_copy_received$
)
and	sa.abstract_id		= a.abstract_id
and	sa.primary_flag		= 'Y'
and	s.session_id		= sa.session_id
order by
	s.sort_order nulls last, s.session_date, s.beg_time, s.session_id, sa.session_seq;

icount	integer := 0;

begin

   if common.header (rpt_title$) then
      htp.tableOpen ('BORDER');

      htp.tableRowOpen ('LEFT', 'BOTTOM');
      htp.tableHeader ('Paper ID');
      htp.tableHeader ('Title');
      htp.tableHeader ('Comments');
      htp.tableRowClose;

      for driver_rec in driver LOOP
         htp.tableRowOpen ('LEFT', 'TOP');
         htp.tableData (htf.anchor('view_pac.show_abstract?this_id=' ||
			driver_rec.abstract_id || '&this_user=', driver_rec.paper_id) );
         htp.tableData (driver_rec.title);
         common.show_abstract_log (driver_rec.abstract_id, true);
         htp.tableRowClose;

         icount := icount + 1;
      end loop;

      htp.tableClose;
      htp.header (3, to_char(icount) || ' Black-dot Papers Received ');
      common.footer;
   end if;

end black_dot_received;

-- ************************************************************************************************

procedure green_dot_report (this_code varchar2) is

icount	integer := 0;

begin

   if common.header (rpt_title$) then
      htp.tableOpen ('BORDER');

      htp.tableRowOpen ('LEFT', 'BOTTOM');
      htp.tableHeader ('Paper ID');
      htp.tableHeader ('Title');
      htp.tableHeader ('Comments');
      htp.tableRowClose;

      for driver_rec in green_dots (this_code) LOOP
         htp.tableRowOpen ('LEFT', 'TOP');
         htp.tableData (htf.anchor('view_pac.show_abstract?this_id=' ||
			driver_rec.abstract_id || '&this_user=', driver_rec.paper_id) );
         htp.tableData (driver_rec.title);
         common.show_abstract_log (driver_rec.abstract_id, true);
         htp.tableRowClose;

         icount := icount + 1;
      end loop;

      htp.tableClose;

      htp.header (3, to_char(icount) || ' Green-dot Papers with no ' ||
		common.decode_value ('log_descr', 'log_codes', 'log_code', this_code) );

      common.footer;
   end if;

end green_dot_report;

-- ************************************************************************************************

procedure no_dots is

cursor	driver is
select	a.*, common.get_paper_id (a.abstract_id) paper_id
from	sessions		s,
	abstract_sessions	sa,
	abstracts		a
where	a.status_code		is null
and	sa.abstract_id		= a.abstract_id
and	sa.primary_flag		= 'Y'
and	s.session_id		= sa.session_id
order by
	s.sort_order nulls last, s.session_date, s.beg_time, s.session_id, sa.session_seq;


icount	integer := 0;

begin

   if common.header ('No-Dot Papers') then
      htp.tableOpen ('BORDER');

      htp.tableRowOpen ('LEFT', 'BOTTOM');
      htp.tableHeader ('Paper ID');
      htp.tableHeader ('Editor');
      htp.tableHeader ('Title');
      htp.tableHeader ('Comments');
      htp.tableRowClose;

      for driver_rec in driver LOOP
         htp.tableRowOpen ('LEFT', 'TOP');
         htp.tableData (htf.anchor('JACoW.view_abstract?abs_id=' ||
			driver_rec.abstract_id, driver_rec.paper_id) );
         htp.tableData (common.get_user_name (driver_rec.editor_id));
         htp.tableData (driver_rec.title);
         common.show_abstract_log (driver_rec.abstract_id, true);
         htp.tableRowClose;

         icount := icount + 1;
      end loop;

      htp.tableClose;

      htp.header (3, to_char(icount) || ' No-dot Papers');
      common.footer;
   end if;

end no_dots;

-- ************************************************************************************************

procedure red_dots is

cursor	driver is
select	a.*, common.get_paper_id (a.abstract_id) paper_id
from	sessions		s,
	abstract_sessions	sa,
	abstracts		a
where	a.status_code		= 'R'
and	sa.abstract_id		= a.abstract_id
and	sa.primary_flag		= 'Y'
and	s.session_id		= sa.session_id
order by
	s.sort_order nulls last, s.session_date, s.beg_time, s.session_id, sa.session_seq;


icount	integer := 0;

begin

   if common.header ('Red-Dot Papers') then
      htp.tableOpen ('BORDER');

      htp.tableRowOpen ('LEFT', 'BOTTOM');
      htp.tableHeader ('Paper ID');
      htp.tableHeader ('Title');
      htp.tableHeader ('Comments');
      htp.tableRowClose;

      for driver_rec in driver LOOP
         htp.tableRowOpen ('LEFT', 'TOP');
         htp.tableData (htf.anchor('JACoW.view_abstract?abs_id=' ||
			driver_rec.abstract_id, driver_rec.paper_id) );
         htp.tableData (driver_rec.title);
         common.show_abstract_log (driver_rec.abstract_id, true);
         htp.tableRowClose;

         icount := icount + 1;
      end loop;

      htp.tableClose;
      htp.header (3, to_char(icount) || ' Red-dot Papers');

      common.footer;
   end if;

end red_dots;

-- ***********************************************************************************************
-- ***********************************************************************************************

procedure show_author_counts is

cursor	driver is
select	1					sort_order,
	' Distinct Primary Authors'		descr,
	count (distinct profile_id)		icount
from	authors
where	author_code				= common.primary_author$
UNION
select	2					sort_order,
	' Distinct Accounts with Abstracts'	descr,
	count (distinct person_id)		icount
from	abstracts
UNION
select	3					sort_order,
	' Distinct Co-authors'			descr,
	count (distinct profile_id)		icount
from	authors
where	author_code				= common.co_author$
UNION
select	4					sort_order,
	' Profiles'				descr,
	count (1)				icount
from	persons
UNION
select	5					sort_order,
	' Accounts'				descr,
	count (1)				icount
from	persons
where	user_id is not null
UNION
select	6					sort_order,
	' Affiliations'				descr,
	count (1)				icount
from	affiliations;

begin

   if common.header (rpt_title$) then
      htp.tableOpen;

      for driver_rec in driver LOOP
         htp.tableRowOpen ('LEFT', 'CENTER');
         htp.tableData (to_char (driver_rec.icount, '999,999'), 'RIGHT');
         htp.tableData (driver_rec.descr);
         htp.tableRowClose;
      end loop;

      htp.tableClose;
      common.footer;
   end if;

end show_author_counts;

-- ***********************************************************************************************

function get_qa_count (this_code varchar2) return integer is

cursor	driver is
select	count(1)
from	abstracts	a
where	classification_code	= this_code
and	exists
(	select	1
	from	logs		l
	where	l.abstract_id	= a.abstract_id
	and	l.log_code	= common.abstract_QA$);

icount	integer;

begin

   open driver;
   fetch driver into icount;
   close driver;

   return (icount);

end get_qa_count;

-- ***********************************************************************************************

procedure get_types (this_type varchar2) is

cursor	driver is
select	c.classification_code,
	c.classification_descr,
	count (a.abstract_id)		abstract_count
from	classification_codes		c,
	abstracts			a
where	this_type			= common.get_presentation_type (a.presentation_code)
and	a.classification_code		= c.classification_code
group by
	c.classification_descr, c.classification_code
order by
	c.classification_descr;

qa_count	integer;
counts		common.int_arr;

begin

   counts(1) := 0; counts(2) := 0; counts(3) := 0;

   htp.tableOpen ('Border');
   htp.tableRowOpen ('RIGHT', 'TOP');
   htp.tableHeader ('Classification', 'LEFT');
   htp.tableHeader ('Contributions');
   htp.tableHeader ('QA''ed');
   htp.tableHeader ('Remaining');
   htp.tableRowClose;

   for driver_rec in driver LOOP
      qa_count := get_qa_count (driver_rec.classification_code);
      show_data_row (driver_rec.classification_descr,
			driver_rec.abstract_count,
			qa_count,
			driver_rec.abstract_count-qa_count);

      counts(1) := counts(1) + driver_rec.abstract_count;
      counts(2) := counts(2) + qa_count;
      counts(3) := counts(3) + driver_rec.abstract_count - qa_count;
   end loop;

   htp.tableRowOpen ('RIGHT', 'TOP');
   htp.tableHeader ('Totals');
   htp.tableHeader (counts(1));
   htp.tableHeader (counts(2));
   htp.tableHeader (counts(3));
   htp.tableRowClose;

   htp.tableClose;

end get_types;

-- ***********************************************************************************************

procedure show_presentation_counts is

cursor	driver is
select	*
from	presentation_type_codes;

begin

   if common.header (rpt_title$) then
      for driver_rec in driver LOOP
         htp.header (3, driver_rec.presentation_type_descr);
         get_types (driver_rec.presentation_type_code);
      end loop;

      common.footer;
   end if;

end show_presentation_counts;

-- ***********************************************************************************************
-- ***********************************************************************************************

procedure show_sub_class_counts is

cursor	driver is
select	scc.sub_class_descr, count(1) icount
from	abstracts		a,
	sub_class_codes		scc
where	a.sub_class_code	= scc.sub_class_code
group by
	scc.sub_class_descr
order by nlssort (scc.sub_class_descr, 'NLS_SORT = Latin');

begin

   if common.header ('Sub-Classification Counts') then
      gbl_abs_count := get_abs_count;
      htp.tableOpen ('BORDER', cattributes=>'CELLSPACING="3"');

      htp.tableRowOpen ('LEFT', 'BOTTOM');
      htp.tableHeader ('Sub-Classifications' || htf.br ||
			gbl_abs_count || ' Total Contributions', 'CENTER');
      htp.tableHeader ('Count');
      htp.tableHeader ('% Total', 'CENTER');
      htp.tableRowClose;

      for driver_rec in driver LOOP
         htp.tableRowOpen ('CENTER', 'TOP');
         htp.tableHeader (driver_rec.sub_class_descr, 'LEFT');
         htp.tableHeader (driver_rec.icount);
         htp.tableHeader (show_pct (driver_rec.icount, gbl_abs_count));
         htp.tableRowClose;
      end loop;

      htp.tableClose;
      common.footer;
   end if;

end show_sub_class_counts;

-- ***********************************************************************************************
-- ***********************************************************************************************

function get_class_counts (	this_code	varchar2,
				this_sub_code	varchar2,
				present_type	varchar2,
				this_session	varchar2 default '%',
				present_code	varchar2 default '%') return integer is

cursor	driver is
select	count(1)
from	abstract_sessions		sa,
	abstracts			a
where	a.classification_code		= this_code
and	nvl(a.sub_class_code,'~')	like this_sub_code
and	a.presentation_code		like present_code
and	a.presentation_code		in
(
	select	presentation_code
	from	presentation_codes
	where	presentation_type_code like present_type
)
and	sa.abstract_id(+)		= a.abstract_id
and	sa.primary_flag(+)		= 'Y'
and	nvl(sa.session_id(+),'~')	like this_session;

icount	integer;

begin

   open driver;
   fetch driver into icount;
   close driver;

   return (icount);

end get_class_counts;

-- ***********************************************************************************************

procedure show_class_sub_counts (this_code	varchar2,
				class_count	integer,
				present_type	varchar2,
				present_code	varchar2 default '%') is

cursor	driver is
select	scc.*
from	sub_class_codes		scc,
	eclass_codes		ecc
where	ecc.classification_code	= this_code
and	scc.sub_class_code	= ecc.sub_class_code
order by
	nlssort (scc.sub_class_code, 'NLS_SORT = Latin');

icount		integer;

begin

   for driver_rec in driver LOOP
      icount := get_class_counts (this_code, driver_rec.sub_class_code, present_type,
					present_code=>present_code);
      htp.tableRowOpen ('CENTER', 'TOP');
      htp.tableData (common.space2$ || driver_rec.sub_class_descr, 'LEFT');
      htp.tableData (icount);
      htp.tableData (show_pct (icount, class_count));
      htp.tableData (show_pct (icount, gbl_abs_count));
      htp.tableRowClose;
   end loop;

   common.show_data_sep (4, common.space$);

end show_class_sub_counts;

-- ***********************************************************************************************

procedure show_classification_counts2 (	present_type varchar2 default '%',
					present_code varchar2 default '%') is

cursor	driver is
select	*
from	classification_codes
order by classification_descr;

icount	integer;
buffer	presentation_codes.presentation_descr%TYPE;

begin

   if present_type||present_code = '%%' then
      gbl_abs_count := get_abs_count;
   else
      gbl_abs_count := get_abs_type_count (present_type, present_code);
   end if;

   if present_type = common.poster$ then
      buffer := 'Poster';
   elsif present_type = common.oral$ then
      buffer := 'Oral';
   elsif present_code <> '%' then
      buffer := common.decode_value ('presentation_descr', 'presentation_codes', 'presentation_code', present_code);
   end if;

   htp.tableOpen ('BORDER', cattributes=>'CELLSPACING="3"');

   htp.tableRowOpen ('LEFT', 'BOTTOM');
   htp.tableHeader (rpt_title2$ || htf.br || gbl_abs_count || ' Total ' || buffer || ' Contributions', 'CENTER', crowspan=>'2');
   htp.tableHeader ('Count', crowspan=>'2');
   htp.tableHeader ('% Total', 'CENTER', ccolspan=>'2');
   htp.tableRowClose;

   htp.tableRowOpen ('CENTER', 'BOTTOM');
   htp.tableHeader ('% In Session');
   htp.tableHeader ('% Total');
   htp.tableRowClose;

   for driver_rec in driver LOOP
      icount := get_class_counts (driver_rec.classification_code, '%', present_type,
					present_code=>present_code);

      htp.tableRowOpen ('CENTER', 'TOP');
      htp.tableHeader (driver_rec.classification_descr, 'LEFT');
      htp.tableHeader (icount);
      htp.tableHeader (show_pct (icount, gbl_abs_count), ccolspan=>'2');
      htp.tableRowClose;

      show_class_sub_counts (driver_rec.classification_code, icount, present_type, present_code);
   end loop;

   htp.tableClose;

end show_classification_counts2;

-- ***********************************************************************************************

procedure show_classification_counts (	present_type varchar2 default '%',
					present_code varchar2 default '%') is

begin

   if common.header ('Classification Counts') then
      show_classification_counts2 (present_type, present_code);
      common.footer;
   end if;

end show_classification_counts;

-- ***********************************************************************************************
-- ***********************************************************************************************

procedure show_counts_by_present_code is

cursor	driver is
select	*
from	presentation_codes
order by sort_order nulls last, presentation_descr;

sepchr	varchar2(10) := null;

begin

   if common.header ('Classification Counts by Presentation') then
      for driver_rec in driver LOOP
         htp.p (sepchr);
         show_classification_counts2 ('%', driver_rec.presentation_code);
         sepchr := htf.hr;
      end loop;
      common.footer;
   end if;

end show_counts_by_present_code;

-- ***********************************************************************************************
-- ***********************************************************************************************

procedure show_class_sub_summary (this_code	varchar2,
				class_count	integer,
				present_code	common.ident_arr) is

cursor	driver is
select	scc.*
from	sub_class_codes		scc,
	eclass_codes		ecc
where	ecc.classification_code	= this_code
and	scc.sub_class_code	= ecc.sub_class_code
order by
	nlssort (scc.sub_class_code, 'NLS_SORT = Latin');

icount	common.int_arr;
total	integer;

begin

   for driver_rec in driver LOOP
      htp.tableRowOpen ('CENTER', 'TOP');
      htp.tableData (common.space2$ || driver_rec.sub_class_descr, 'LEFT');

      total := 0;
      for i in 1..present_code.COUNT LOOP
	 icount(i) := get_class_counts (this_code, driver_rec.sub_class_code, '%', '%', present_code(i));
	 total	   := total + icount(i);
      end loop;

      htp.tableData (total);
      htp.tableData (show_pct (total, class_count));
      for i in 1..present_code.COUNT LOOP
	 htp.tableData (icount(i));
	 htp.tableData (show_pct (icount(i), total));
      end loop;

      htp.tableRowClose;
   end loop;

   common.show_data_sep (3 + present_code.COUNT*2, common.space$);

end show_class_sub_summary;

-- ***********************************************************************************************

procedure show_counts_summary is

cursor	present is
select	presentation_code, presentation_descr
from	presentation_codes
order by
	sort_order nulls last,
	nlssort (presentation_descr, 'NLS_SORT = Latin');

cursor	driver is
select	classification_code, classification_descr
from	classification_codes
order by
	classification_descr;

pcodes	common.ident_arr;
pdescr	common.ident_arr;
icount	common.int_arr;
itotal	integer;

begin

   if common.header ('Classification Counts by Presentation') then
      gbl_abs_count := get_abs_count;
      open present;
      fetch present BULK COLLECT into pcodes, pdescr;
      close present;

      htp.tableOpen ('BORDER');
      htp.tableRowOpen ('CENTER', 'BOTTOM');
      htp.tableHeader (rpt_title2$ || htf.br || gbl_abs_count || ' Total Contributions', 'CENTER');

      htp.tableHeader ('Total', ccolspan=>'2');
      for i in 1..pdescr.COUNT LOOP
         htp.tableHeader (pdescr(i), ccolspan=>'2');
      end loop;

      htp.tableRowClose;

      for driver_rec in driver LOOP
         htp.tableRowOpen ('CENTER', 'TOP');
         htp.tableHeader (driver_rec.classification_descr, 'LEFT');
         itotal := 0;

         for i in 1..pcodes.COUNT LOOP
            icount(i) := get_class_counts (driver_rec.classification_code, '%', '%', '%', pcodes(i));
            itotal := itotal + icount(i);
         end loop;

         htp.tableHeader (itotal);
         htp.tableHeader (show_pct (itotal, gbl_abs_count));

         for i in 1..pcodes.COUNT LOOP
            htp.tableHeader (icount(i));
            htp.tableHeader (show_pct (icount(i), itotal));
         end loop;

         htp.tableRowClose;

         show_class_sub_summary (driver_rec.classification_code, itotal, pcodes);
      end loop;

      htp.tableClose;
      common.footer;
   end if;

end show_counts_summary;

-- ***********************************************************************************************
-- ***********************************************************************************************

procedure show_counts_by_classification (this_session varchar2) is

--where	session_id = this_session

cursor	driver is
select	*
from	classification_codes
order by
	nlssort (classification_descr, 'NLS_SORT = Latin');

total	integer := 0;
icount	integer;

begin

   for driver_rec in driver LOOP
      icount	:= get_class_counts (driver_rec.classification_code, '%', '%', this_session);
      total	:= total + icount;

      htp.tableRowOpen ('LEFT', 'TOP');
      htp.tableData (common.space$, ccolspan=>'2');
      htp.tableData (icount, 'RIGHT');
      htp.tableData (driver_rec.classification_descr);
      htp.tableRowClose;

   end loop;

   if total <> 0 then
      htp.tableRowOpen ('LEFT', 'TOP');
      htp.tableData (common.space$, ccolspan=>'2');
      htp.tableHeader (total, 'RIGHT');
      common.show_data_sep (7, common.space$);
   end if;

end show_counts_by_classification;

-- ***********************************************************************************************
-- ***********************************************************************************************

function get_abs_session_count (this_session varchar2) return integer is

cursor	driver is
select	count(1)
from	abstract_sessions
where	session_id	= this_session;

icount	integer;

begin

   open driver;
   fetch driver into icount;
   close driver;

   return (icount);

end get_abs_session_count;

-- ***********************************************************************************************

procedure show_session_counts is

cursor	driver is
select	*
from	sessions
order by
	sort_order nulls last, session_date, beg_time, session_id;

presentation_descr	presentation_codes.presentation_descr%TYPE;
prev_date		date := to_date ('01-jan-00');

begin

   if common.header (rpt_title$) then
      htp.header (3, 'Session Counts');
      htp.tableOpen;

      for driver_rec in driver LOOP
         presentation_descr := common.decode_value ('presentation_option_descr',
			'presentation_option_codes', 'presentation_option_code',
			driver_rec.presentation_option_code);

         if prev_date <> driver_rec.session_date then
            common.show_data_sep (7);
            prev_date := driver_rec.session_date;
         end if;

         if common.decode_value ('presentation_type_code', 'presentation_option_codes',
			'presentation_option_code', driver_rec.presentation_option_code) = common.poster$ then
            common.show_data_sep (7, common.space$);
         end if;

         htp.tableRowOpen ('LEFT', 'TOP');
         htp.tableHeader (get_abs_session_count (driver_rec.session_id), 'LEFT');
         htp.tableHeader (driver_rec.session_id);
         htp.tableHeader (presentation_descr);
         htp.tableHeader (driver_rec.title);
         htp.tableHeader (to_char(driver_rec.session_date, 'mm/dd/yyyy '));
         htp.tableHeader (common.format_time(driver_rec.beg_time));
         htp.tableHeader (common.format_time(driver_rec.end_time));
         htp.tableRowCLose;

         show_counts_by_classification (driver_rec.session_id);
      end loop;

      htp.tableClose;
      common.footer;
   end if;

end show_session_counts;

-- ***********************************************************************************************
-- ***********************************************************************************************

procedure show_CR_counts is

cursor	driver is
select	*
from	abstracts;

icount		integer;
ipos		integer;
max_count	integer := 0;
max_id		integer := 0;

begin

   if common.header ('Carriage Returns') then
      for driver_rec in driver LOOP
         icount	:= 0;
         LOOP
	    ipos := instr (driver_rec.description, chr(10));
            exit when ipos is null;
            exit when ipos = 0;

            icount := icount + 1;
            driver_rec.description := substr (driver_rec.description, ipos+1);
         end loop;

         if icount > max_count then
            max_count	:= icount;
            max_id		:= driver_rec.abstract_id;
         end if;
      end loop;

      if max_id <> 0 then
         htp.header (3, 'Contribution ID ' || to_char(max_id) || ' contains ' ||
			to_char(max_count) || ' carriage returns.');

      else
         htp.header (3, 'No abstracts contain carriage returns.');
      end if;

      common.footer;
   end if;

end show_CR_counts;

-- ****************************************************************************************************
-- ****************************************************************************************************

function format_buffer (buffer varchar2) return varchar2 is

new_buff	varchar2(3000) default null;
ilen		integer;
this_byte	integer;
this_char	varchar2(10);

begin

   ilen := length (buffer);
   for i in 1..ilen LOOP
      this_char := substr (buffer, i, 1);
      this_byte := ascii (this_char);
      if NOT (this_byte in (9, 10, 13) OR this_byte between 32 and 126) then
	 new_buff := new_buff || htf.fontOpen('RED') || htf.bold(this_char) || htf.fontClose;
      else
	 new_buff := new_buff || this_char;
      end if;
   end loop;

   return (new_buff);

end format_buffer;

-- ****************************************************************************************************

procedure find_buffer (abs_id integer, buffer varchar2, label varchar2) is

ilen		constant integer := nvl(length (buffer), 0);
this_byte	integer;

begin

   for i in 1..ilen LOOP
      this_byte := ascii (substr (buffer, i, 1));

      if NOT (this_byte in (9, 10, 13) OR this_byte between 32 and 126) then
	 htp.tableRowOpen ('LEFT', 'TOP');

	 if abs_id is null then
	    htp.tableData (null);
	 else
	    htp.tableData (htf.anchor ('JACoW.view_abstract?abs_id=' || abs_id, abs_id) );
	 end if;

	 htp.tableData (label);
	 htp.tableData (format_buffer(buffer));
	 htp.tableRowClose;

	 exit;
      end if;
   end loop;

end find_buffer;

-- ***********************************************************************************************
-- ***********************************************************************************************

procedure show_sessions (this_date date, this_time integer) is

cursor	driver is
select	aa.person_id presenter_id, abs.*,
	sa.session_id, sa.session_seq, sa.paper_id
from	abstract_authors	aa,
	abstracts		abs,
	abstract_sessions	sa
where	sa.session_id		in
	(	select		session_id
		from		sessions
		where		session_date	= this_date
		and		beg_time	= this_time)
and	abs.abstract_id		= sa.abstract_id
and	aa.abstract_id		= abs.abstract_id
and	aa.author_code		= common.speaker$
order by
	1, sa.session_id, sa.session_seq;

buffer		long := null;
current		long := null;

prev_pid	integer := null;
prev_session	sessions.session_id%TYPE := null;
prev_seq	integer := null;

this_session	sessions.session_id%TYPE;
this_seq	integer;
this_pid	integer;
disjoint	boolean := false;

begin

   htp.tableRowOpen ('LEFT', 'BOTTOM');
   htp.tableHeader (to_char (this_date, 'Day MM/DD/YYYY') || to_char (this_time, '0000'), ccolspan=>'4');
   htp.tableRowClose;

   htp.tableRowOpen ('LEFT', 'BOTTOM');
   htp.tableHeader ('Contribution ID');
   htp.tableHeader ('Paper ID');
   htp.tableHeader ('Author');
   htp.tableHeader ('Registered');
   htp.tableRowClose;

   for driver_rec in driver loop
      this_session	:= driver_rec.session_id;
      this_seq		:= driver_rec.session_seq;
      this_pid		:= driver_rec.presenter_id;

      current		:=	htf.tableRowOpen ('LEFT', 'BOTTOM')		||
				htf.tableData (driver_rec.abstract_id)		||
				htf.tableData (driver_rec.paper_id)		||
				htf.tableData (common.get_user_name(this_pid))	||
				htf.tableData (regist_maint.is_registered(this_pid), 'CENTER')	||
				htf.tableRowClose;

      if this_pid = prev_pid then

	 if this_session <> prev_session then
	    disjoint := true;
	 elsif this_seq <> prev_seq + 1 then
	    disjoint := true;
	 end if;

      else

	 if disjoint then
	    htp.p (buffer);
	 end if;

	 buffer		:= null;
	 disjoint	:= false;

      end if;

      buffer		:= buffer || current;

      prev_session	:= this_session;
      prev_seq		:= this_seq;
      prev_pid		:= this_pid;
   end loop;

   if disjoint then
      htp.p (buffer);
   end if;

end show_sessions;

-- ***************************************************************************************

procedure disjointed_authors is

cursor	driver is
select	distinct session_date, beg_time
from	sessions
where	presentation_option_code	in
(	select	presentation_option_code
	from	presentation_option_codes
	where	presentation_type_code = 'P')
order by 1, 2;

begin

   if common.header (rpt_title$) then
      htp.tableOpen ('BORDER');
      for driver_rec in driver LOOP
         show_sessions (driver_rec.session_date, driver_rec.beg_time);
      end loop;
      htp.tableClose;
      common.footer;
   end if;

end disjointed_authors;

-- ***********************************************************************************************
-- ***********************************************************************************************

procedure author_index is

cursor	driver is
select	distinct p.lname, p.initials, p.person_id
from	persons			p,
	profiles		pro,
	authors			auth,
	abstracts		a
where	auth.abstract_id	= a.abstract_id
and	auth.author_code	in (common.co_author$, common.primary_author$)
and	pro.profile_id		= auth.profile_id
and	p.person_id		= pro.person_id;

begin

   htp.htmlOpen;
   htp.headOpen;
   htp.p ('<style type="text/css">HTML {font-family: "Pac Font"}</style>');
   htp.headClose;
   htp.bodyOpen;
   htp.tableOpen ('BORDER');

   htp.tableRowOpen ('LEFT', 'BOTTOM');
   htp.tableHeader ('Last Name');
   htp.tableHeader ('Initials');
   htp.tableHeader ('Paper IDs');
   htp.tableRowClose;

   for driver_rec in driver LOOP
      htp.tableRowOpen ('LEFT', 'BOTTOM');
      htp.tableData (driver_rec.lname);
      htp.tableData (driver_rec.initials);
      htp.tableData (common.get_author_paper_ids (driver_rec.person_id));
      htp.tableRowClose;
   end loop;

   htp.tableClose;
   common.footer;

end author_index;

-- *********************************************************************************

procedure abstract_book is

cursor	driver is
select	a.*, sa.paper_id,
	common.get_author_profile_id (a.abstract_id, common.speaker$) spk_profile_id
from	sessions		s,
	abstract_sessions	sa,
	abstracts		a
where	sa.abstract_id(+)	= a.abstract_id
and	s.session_id(+)		= sa.session_id
order by
	s.sort_order nulls last, s.session_date, s.beg_time, s.session_id, sa.session_seq,
	a.classification_code, a.sub_class_code, a.eclass_code, a.abstract_id;

begin

   if common.header ('Abstract Book') then
      htp.tableOpen ('BORDER');

      htp.tableRowOpen ('LEFT', 'BOTTOM');
      htp.tableHeader ('Abstract ID');
      htp.tableHeader ('Paper ID');
      htp.tableHeader ('Contribution Title');
      htp.tableHeader ('Presentation');
      htp.tableHeader ('Classification');

      if common.require_sub_class$ = 'Y' then
         htp.tableHeader ('Sub Classification');
      end if;

      htp.tableHeader ('Author List');
      htp.tableHeader ('Speaker Affiliation');
      htp.tableHeader ('Contribution');
      htp.tableHeader ('Footnote');
      htp.tableHeader ('Agency');
      htp.tableRowClose;

      for driver_rec in driver LOOP
         htp.tableRowOpen ('LEFT', 'TOP');
         htp.tableData (driver_rec.abstract_id);
         htp.tableData (driver_rec.paper_id);
         htp.tableData (driver_rec.title);
         htp.tableData (common.decode_value ('presentation_descr', 'presentation_codes', 'presentation_code', driver_rec.presentation_code) );
         htp.tableData (common.decode_value ('classification_descr', 'classification_codes', 'classification_code', driver_rec.classification_code) );

         if common.require_sub_class$ = 'Y' then
            htp.tableData (common.decode_value ('sub_class_descr', 'sub_class_codes', 'sub_class_code', driver_rec.sub_class_code) );
         end if;

         htp.tableData (common.get_author_list2 (driver_rec.abstract_id, initials_only=>common.abs_book_initials_only$));
         htp.tableData (common.get_profile_affil (driver_rec.spk_profile_id, 'Y'));
         htp.tableData (driver_rec.description);
         htp.tableData (driver_rec.footnote);
         htp.tableData (driver_rec.agency);
         htp.tableRowClose;
      end loop;

      htp.tableClose;
      common.footer;
   end if;

end abstract_book;

-- *********************************************************************************

procedure conference_guide is

cursor	driver is
select	a.*, sa.paper_id
from	sessions		s,
	abstract_sessions	sa,
	abstracts		a
where	sa.abstract_id		= a.abstract_id
and	s.session_id		= sa.session_id
order by
	s.sort_order nulls last, s.session_date, s.beg_time, s.session_id, sa.session_seq;

begin

   htp.htmlOpen;
   htp.headOpen;
   htp.p ('<style type="text/css">HTML {font-family: "Pac Font"}</style>');
   htp.headClose;
   htp.bodyOpen;
   htp.tableOpen ('BORDER');

   htp.tableRowOpen ('LEFT', 'BOTTOM');
   htp.tableHeader ('Paper ID');
   htp.tableHeader ('Contribution Title');
   htp.tableHeader ('Author List');
   htp.tableRowClose;

   for driver_rec in driver LOOP
      htp.tableRowOpen ('LEFT', 'TOP');
      htp.tableData (nvl (driver_rec.paper_id, driver_rec.abstract_id) );
      htp.tableData (driver_rec.title);
      htp.tableData (common.get_author_list2 (driver_rec.abstract_id, initials_only=>'Y'));
      htp.tableRowClose;
   end loop;

   htp.tableClose;
   common.footer;

end conference_guide;

-- *********************************************************************************

procedure extracts is

begin

   if common.header ('Data Extracts') then
      htp.header (3, 'These extracts aid in the creation of the Conference Guide ' ||
		'Contribution Book and Author Index.');

      htp.tableOpen ('BORDER');
      htp.tableRowOpen ('LEFT', 'BOTTOM');
      htp.tableData (htf.anchor ('reports.conference_guide', 'Conference Guide'));
      htp.tableData (common.server || common.vdir$ || 'reports.conference_guide');
      htp.tableRowClose;

      htp.tableRowOpen ('LEFT', 'BOTTOM');
      htp.tableData (htf.anchor ('reports.abstract_book', 'Contribution Book'));
      htp.tableData (common.server || common.vdir$ || 'reports.abstract_book');
      htp.tableRowClose;

      htp.tableRowOpen ('LEFT', 'BOTTOM');
      htp.tableData (htf.anchor ('reports.author_index', 'Author Index'));
      htp.tableData (common.server || common.vdir$ || 'reports.author_index');
      htp.tableRowClose;

      htp.tableRowOpen ('LEFT', 'BOTTOM');
      htp.tableData (htf.anchor ('reports.session_info', 'Sessions'));
      htp.tableData (common.server || common.vdir$ || 'reports.session_info');
      htp.tableRowClose;

      htp.tableRowOpen ('LEFT', 'BOTTOM');
      htp.tableData (htf.anchor ('reports.program', 'Overview of Oral Presentations'));
      htp.tableData (common.server || common.vdir$ || 'reports.program');
      htp.tableRowClose;

      htp.tableClose;
      htp.para;

      htp.p ('To load these reports into an Excel spreadsheet cut and paste');
      htp.p ('the URL above into Excel''s "New Web Query" wizard.');
      htp.para;

      common.footer;
   end if;

end extracts;

-- ***********************************************************************************************
-- ***********************************************************************************************

procedure show_session_abs (sid varchar2) is

cursor	driver is
select	sa.paper_id, a.title,
	common.get_user_name_affiliation (
		common.get_author_profile_id (
			a.abstract_id, common.speaker$)) name_affil
from	abstract_sessions	sa,
	abstracts		a
where	sa.session_id		= sid
and	a.abstract_id		= sa.abstract_id
order by sa.session_seq;

begin

   for driver_rec in driver LOOP
      htp.tableRowOpen ('LEFT', 'TOP');
      common.show_data_row (driver_rec.paper_id, driver_rec.title);
      common.show_data_row (null, driver_rec.name_affil);
  end loop;

end show_session_abs;

-- ***********************************************************************************************

procedure session_info is

cursor	driver is
select	s.*, lc.location_descr
from	location_codes		lc,
	sessions		s
where	lc.location_code	= s.location_code
order by
	s.sort_order nulls last, s.session_date, s.beg_time, s.session_id;

temp	varchar2(10);

begin

   if common.header ('Session Info') then
      htp.tableOpen;

      for driver_rec in driver LOOP
         common.show_data_row (driver_rec.session_id, htf.bold (driver_rec.title));
         common.show_data_row (null, to_char (driver_rec.session_date, common.date_fmt_str$)	||
				common.space2$ || common.format_time (driver_rec.beg_time)	||
				' - ' || common.format_time (driver_rec.end_time) );
         common.show_data_row (null, driver_rec.location_descr);

         common.show_data_sep (2, common.space$);
         show_session_abs (driver_rec.session_id);
         common.show_data_sep (2);
      end loop;

      htp.tableClose;
      common.footer;
   end if;

end session_info;

-- *********************************************************************************
-- *********************************************************************************

procedure show_abs_rec (driver_rec abstracts%ROWTYPE) is

begin

   htp.tableRowOpen ('LEFT', 'TOP');
   htp.tableData (common.get_paper_id (driver_rec.abstract_id));
   htp.tableData (driver_rec.title);
   htp.tableData (common.get_author_list2 (driver_rec.abstract_id) );
   htp.tableRowClose;

end show_abs_rec;

-- *********************************************************************************

procedure police_no_qual is

cursor	driver is
select	a.*
from	sessions		s,
	abstract_sessions	sa,
	abstracts		a,
	poster_logs		pl
where	pl.manned		= 'Y'
and	pl.posted		= 'Y'
and	pl.satisfactory		= 'N'
and	a.abstract_id		= pl.abstract_id
and	sa.abstract_id		= a.abstract_id
and	s.session_id		= sa.session_id
order by
	s.sort_order nulls last, s.session_date, s.beg_time, s.session_id, sa.session_seq;

begin

   if common.header ('Poster Police:  Posted, Manned but Quality Unsatisfactory') then
      htp.tableOpen ('BORDER');
      htp.tableRowOpen ('LEFT', 'BOTTOM');
      htp.tableHeader ('Paper ID');
      htp.tableHeader ('Title');
      htp.tableHeader ('Authors');
      htp.tableRowClose;

      for driver_rec in driver LOOP
         show_abs_rec (driver_rec);
      end loop;

      htp.tableClose;
      common.footer;
   end if;

end police_no_qual;

-- *********************************************************************************

procedure police_no_files is

cursor	driver is
select	a.*
from	sessions		s,
	abstract_sessions	sa,
	abstracts		a,
	poster_logs		pl
where	pl.manned		= 'Y'
and	pl.posted		= 'Y'
and	pl.satisfactory		= 'Y'
and	0		=
(	select	count(1)
	from	files	f
	where	f.abstract_id	= pl.abstract_id)
and	a.abstract_id		= pl.abstract_id
and	sa.abstract_id		= a.abstract_id
and	s.session_id		= sa.session_id
order by
	s.sort_order nulls last, s.session_date, s.beg_time, s.session_id, sa.session_seq;

begin

   if common.header ('Poster Police OK With No Files Uploaded') then
      htp.tableOpen ('BORDER');
      htp.tableRowOpen ('LEFT', 'BOTTOM');
      htp.tableHeader ('Paper ID');
      htp.tableHeader ('Title');
      htp.tableHeader ('Authors');
      htp.tableRowClose;

      for driver_rec in driver LOOP
         show_abs_rec (driver_rec);
      end loop;

      htp.tableClose;
      common.footer;
   end if;

end police_no_files;

-- *********************************************************************************
-- *********************************************************************************

procedure dummy is

begin

   common.public_header (rpt_title$);
   common.footer;

end dummy;

-- ***********************************************************************************************

procedure toc is

cursor	driver is
select	*
from	menu_items
where	parent_item_id	= 3
and	URL		is not null
order by
	nlssort (link_name, 'NLS_SORT = Latin');

begin

   maint.header (rpt_title$, body_open=>FALSE);

   htp.bodyOpen;
   maint.menu (3);

   htp.bodyClose;
   htp.htmlClose;

end toc;

-- ***********************************************************************************************

procedure by_page_count is

cursor	driver is
select	pc.presentation_descr, pc.max_pages,
	a.total_pages actual_pages, count(1) actual_count
from	abstracts		a,
	presentation_codes	pc
where	a.presentation_code	= pc.presentation_code
group by pc.presentation_descr, pc.max_pages, a.total_pages
order by pc.presentation_descr, pc.max_pages, a.total_pages nulls first;

begin

   common.public_header ('Page Counts by Presentation');
   htp.tableOpen ('BORDER');
   htp.tableRowOpen;
   htp.tableHeader ('Presentation Type', 'LEFT');
   htp.tableHeader ('Max Pages');
   htp.tableHeader ('Actual Pages');
   htp.tableHeader ('Count of Actual');
   htp.tableRowClose;

   for rec in driver LOOP
      htp.tableRowOpen;
      htp.tableData (rec.presentation_descr, 'LEFT');
      htp.tableData (rec.max_pages, 'CENTER');
      htp.tableData (rec.actual_pages, 'CENTER');
      htp.tableData (rec.actual_count, 'CENTER');
      htp.tableRowClose;
   end loop;

   htp.tableClose;
   common.footer;

end by_page_count;

-- ***********************************************************************************************

procedure html is

begin

   if common.header (rpt_title$, body_open=>false) then
      htp.frameSetOpen (null, '225,*');
         htp.frame ('reports.toc',  'TOC');
         htp.frame ('rpt_stats.html', 'BODY');
      htp.frameSetClose;
      htp.htmlClose;
   end if;

end html;

-- ***********************************************************************************************

end reports;

/
show error

set define on

--	Copyright (C) 2004-2008 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>
