set define off

create or replace package body repository as

rpt_title$	constant char(19)	:= 'Administrator Pages';
rpt_title2$	constant char(27)	:= 'Merge Affiliation Addresses';
date_fmt_str$	constant char(20)	:= 'dd-Mon-yy hh24:mi:ss';

cursor	dummy is select * from affiliations;

-- **********************************************************************************************
-- **********************************************************************************************

procedure rebuild_CTX2 is

cursor	driver is
select	ctx.*, nvl (ui.status, 'MISSING') status
from 	user_indexes			ui,
	SPMS_CTX_indexes		ctx
where	ui.index_name(+)		= ctx.index_name
and	ui.table_name(+)		= ctx.table_name
and	nvl (ui.status, 'MISSING')	<> 'VALID';

begin

   for driver_rec in driver LOOP
      if driver_rec.status <> 'MISSING' then
         execute immediate 'drop index ' || driver_rec.index_name || ' force';
      end if;

      execute immediate 'create index ' || driver_rec.index_name		||
			' on ' || driver_rec.table_name || ' (search_text) '	||
			'indextype is ctxsys.context '	||
			'parameters (''memory 50M storage '			||
			driver_rec.storage_name || ' stoplist SPMS_STOP_LIST'')';
   end loop;

end rebuild_CTX2;

-- **********************************************************************************************

procedure rebuild_CTX is

cursor	driver is
select	ctx.*, nvl (ui.status, 'MISSING') status
from 	user_indexes			ui,
	SPMS_CTX_indexes		ctx
where	ui.index_name(+)		= ctx.index_name
and	ui.table_name(+)		= ctx.table_name
and	nvl (ui.status, 'MISSING')	<> 'VALID';

begin

   common.get_gbl_pid;
   if common.has_privs_to_page then
      rebuild_CTX2;
      common.redirect_URL ('repository.jobs');
   end if;

end rebuild_CTX;

-- **********************************************************************************************
-- **********************************************************************************************

function get_return_URL (letter varchar2, string varchar2) return varchar2 is

begin

   if string is not null then
      return 'repository.search?string=' || string;
   else
      return 'repository.affil_list?letter=' || letter;
   end if;

end get_return_URL;

-- **********************************************************************************************

procedure back_to_search (letter varchar2, string varchar2) is

begin

   common.redirect_url (get_return_URL (letter, string) );

end back_to_search;

-- **********************************************************************************************
-- **********************************************************************************************

procedure save_errors (err_delete common.ident_arr, err_resend common.ident_arr) is

begin

   if common.header (rpt_title$) then
      for i in 2..err_resend.COUNT LOOP
         insert into mail_blats (MAIL_ID, FROM_ADDR, RECIPIENT, SUBJECT, MESSAGE,
				CC, BCC, FLAG, CREATE_DATE, PERSON_ID, ABSTRACT_ID)
         select MAIL_ID, FROM_ADDR, RECIPIENT, SUBJECT, MESSAGE,
		CC, BCC, FLAG, CREATE_DATE, PERSON_ID, ABSTRACT_ID
         from	mail_blats_errors
         where	mail_id = err_resend(i);

         delete from mail_blats_errors
         where	mail_id = err_resend(i);
      end loop;

      for i in 2..err_delete.COUNT LOOP
         delete from mail_blats_errors
         where	mail_id = err_delete(i);
      end loop;

      common.redirect_URL ('repository.show_errors');
   end if;

end save_errors;

-- **********************************************************************************************
-- **********************************************************************************************

procedure show_errors is

cursor	driver is
select	mail_id, recipient, subject, create_date, error_code, error_message
from	mail_blats_errors
order by create_date desc;

begin

   if common.header (rpt_title$) then
      htp.anchor ('repository.jobs', 'Back to Jobs page');
      htp.para;
      htp.formOpen ('repository.save_errors', 'POST');
      htp.formHidden ('err_delete', 'dummy');
      htp.formHidden ('err_resend', 'dummy');
      htp.tableOpen ('BORDER');

      htp.tableRowOpen ('LEFT', 'BOTTOM');
      htp.tableHeader ('Delete');
      htp.tableHeader ('Error Code');
      htp.tableHeader ('Error Message');
      htp.tableHeader ('Date');
      htp.tableHeader ('Subject');
      htp.tableHeader ('Recipient');
      htp.tableHeader ('Resend');
      htp.tableRowClose;

      for rec in driver LOOP
         htp.tableRowOpen ('LEFT', 'TOP');
         htp.tableData (htf.formCheckBox ('err_delete', rec.mail_id), 'CENTER');
         htp.tableData (abs (rec.error_code));
         htp.tableData (rec.error_message);
         htp.tableData (common.local_time (rec.create_date), cnowrap=>'NOWRAP');
         htp.tableData (rec.subject);
         htp.tableData (nvl (rec.recipient, common.space$));
         htp.tableData (htf.formCheckBox ('err_resend', rec.mail_id), 'CENTER');
         htp.tableRowClose;
      end loop;

      htp.tableClose;
      htp.formSubmit;
      htp.formClose;
      common.footer;
   end if;

end show_errors;

-- **********************************************************************************************
-- **********************************************************************************************

procedure status is

cursor	driver is
select	job, what, last_date, broken, failures
from	user_jobs
where	broken		= 'Y'
or	failures	> 0;

subject		mail_blats.subject%TYPE;
message		mail_blats.message%TYPE;

begin

   for driver_rec in driver LOOP
      subject	:= common.conf_title$;
      message	:= 'The batch job ' || driver_rec.what || ' running in the ' ||
			common.conf_title$ || ' SPMS database ';


      if driver_rec.broken = 'Y' then
         subject	:= subject || ' Job Broken';
         message	:= message || ' is broken.';
      else
         subject	:= subject || ' Job Failed';
         message	:= message || ' has failed ' || driver_rec.failures || ' time(s).';
      end if;

      message := message || '  Please go to the Job Queue Status page in SPMS, which is located ' ||
			'under the Overall Database Administration foler, to reset the job.';
      send_mail (common.editor_email$, subject, message);
   end loop;

end status;

-- **********************************************************************************************
-- **********************************************************************************************

procedure optimize_ctx is

begin

   rebuild_CTX2;
   ctxsys.ctx_ddl.optimize_index ('ABSTRACT_CTX', 'FULL');
   ctxsys.ctx_ddl.optimize_index ('ADDRESS_CTX', 'FULL');
   ctxsys.ctx_ddl.optimize_index ('AFFILIATION_CTX', 'FULL');

end optimize_ctx;

-- **********************************************************************************************
-- **********************************************************************************************

procedure start_batch_jobs is

cursor	driver is
select	*
from	spms_jobs
where	command NOT in
(
	select	what
	from	user_jobs
)
and	no_repo	<> decode (common.repo_db$, 'TRUE', 'Y', '~');

job	number;
startDT	date;

begin

   for rec in driver LOOP
      startDT := sysdate + rec.offset;
      if rec.command = 'sync.repo;' then
         startDT := sync.get_next_time;
      elsif rec.trunc_sysdate = 'Y' then
         startDT := trunc (sysdate) + rec.offset;
      end if;

      dbms_job.submit (job, rec.command, startDT, rec.interval);
   end loop;

end start_batch_jobs;

-- **********************************************************************************************

procedure start_jobs is

begin

   start_batch_jobs;
   common.redirect_URL ('repository.jobs');

end start_jobs;

-- **********************************************************************************************
-- **********************************************************************************************

procedure kill_job (ID integer) is

begin

   dbms_job.remove (ID);
   common.redirect_URL ('repository.jobs');

end kill_job;

-- **********************************************************************************************
-- **********************************************************************************************

function calc_time (seconds integer) return varchar2 is

beg_time	constant date := trunc(sysdate);
end_time	constant date := trunc(sysdate) + (seconds/60)/1440;
days		constant integer := trunc (end_time) - beg_time;

begin

   return to_char (days, 'FM000') || '-' || to_char (end_time, 'hh24:mi:ss');

end calc_time;

-- **********************************************************************************************

procedure show_time_zones is

cursor	driver is
select	tz_offset (dbtimezone) offset, dbtimezone
from	dual;

driver_rec	driver%ROWTYPE;

begin

   open driver;
   fetch driver into driver_rec;
   close driver;

   driver_rec.offset := replace (driver_rec.offset, chr(0), null);

   htp.tableOpen;
   common.show_data_row ('Local Time',	to_char (sysdate, date_fmt_str$) );
   common.show_data_row ('Time Zone',	driver_rec.dbtimezone);
   common.show_data_row ('Offset',	driver_rec.offset);
   common.show_data_row ('UTC Time',	to_char (common.get_UTC_time, date_fmt_str$) );
   htp.tableClose;
   htp.para;

end show_time_zones;

-- **********************************************************************************************

procedure show_unavailable_times is

cursor	driver is
select	*
from	unavailable_times
where	common.get_UTC_time < end_date
order by beg_date, end_date;

type		driver_tab_type	is table of unavailable_times%ROWTYPE index by binary_integer;
driver_tab	driver_tab_type;

begin

   open driver;
   fetch driver BULK COLLECT into driver_tab;
   close driver;

   if driver_tab.COUNT > 0 then
      htp.header (3, 'The repository database is unavailable during the following times.',
			cattributes=>common.red$);
      htp.tableOpen ('BORDER');
      htp.tableRowOpen ('LEFT', 'BOTTOM');
      htp.tableHeader ('From');
      htp.tableHeader ('Until');
      htp.tableHeader ('Reason');
      htp.tableRowClose;

      for i in 1..driver_tab.COUNT LOOP
         htp.tableRowOpen ('LEFT', 'TOP');
         htp.tableHeader (to_char (common.local_time_stamp (driver_tab(i).beg_date), date_fmt_str$) );
         htp.tableHeader (to_char (common.local_time_stamp (driver_tab(i).end_date), date_fmt_str$) );
         htp.tableHeader (driver_tab(i).reason);
         htp.tableRowClose;
      end loop;

      htp.tableClose;
      htp.para;
   end if;

end show_unavailable_times;

-- **********************************************************************************************

procedure show_email_counts is

cursor	driver is
select	count(1) icount, 'Email Pending' prompt
from	mail_blats
UNION
select	count(1) icount, 'Email Sent' prompt
from	mail_sent_blats
UNION
select	count(1) icount, 'Email Errors' prompt
from	mail_blats_errors;

link	varchar2(100);

begin

   htp.tableOpen;

   for driver_rec in driver LOOP
      link := null;
      if driver_rec.prompt = 'Email Errors' AND driver_rec.icount > 0 then
         link := common.space2$ || htf.anchor ('repository.show_errors', 'Show Errors');
      end if;
      common.show_data_row (driver_rec.prompt, driver_rec.icount || link);
   end loop;

   htp.tableClose;
   htp.para;
   common.footer;

end show_email_counts;

-- **********************************************************************************************

procedure show_CTX_status is

cursor	driver is
select	ctx.*, nvl (ui.status, 'MISSING') status
from 	user_indexes		ui,
	SPMS_CTX_indexes	ctx
where	ui.index_name(+)	= ctx.index_name
and	ui.table_name(+)	= ctx.table_name;

invalid_index	boolean := false;
style		varchar2(100);

begin

   htp.tableOpen ('BORDER');
   htp.tableRowOpen ('LEFT', 'BOTTOM');
   htp.tableHeader ('Index');
   htp.tableHeader ('Status');
   htp.tableRowClose;

   for driver_rec in driver LOOP
      style := null;
      if driver_rec.status <> 'VALID' then
         invalid_index	:= true;
         style		:= common.red$;
      end if;

      htp.tableRowOpen ('LEFT', 'TOP');
      htp.tableData (driver_rec.index_name);
      htp.tableData (driver_rec.status, cattributes=>style);
      htp.tableRowClose;

   end loop;
   htp.tableClose;

   if invalid_index then
      htp.anchor ('repository.rebuild_CTX', 'Rebuild CTX Indexes');
   end if;
   htp.para;

end show_CTX_status;

-- **********************************************************************************************

procedure jobs is

cursor	driver is
select	*
from	user_jobs;

begin

   if common.header ('Oracle Job Queue Status') then
      show_time_zones;
      show_unavailable_times;

-- **********

      htp.bold ('To kill a job click the the ID number.', cattributes=>common.red$);
      htp.p (common.space2$);
      htp.anchor ('repository.start_jobs', 'Start Jobs');

      htp.tableOpen ('BORDER');
      htp.tableRowOpen ('LEFT', 'BOTTOM');
      htp.tableHeader ('ID');
      htp.tableHeader ('Task');
      htp.tableHeader ('Last Run');
      htp.tableHeader ('Next Run');
      htp.tableHeader ('Interval');
      htp.tableHeader ('Total Time');
      htp.tableHeader ('Failures');
      htp.tableHeader ('Broken');
      htp.tableRowClose;

      for driver_rec in driver LOOP
         htp.tableRowOpen ('LEFT', 'TOP');
         htp.tableData (htf.anchor ('repository.kill_job?ID=' || driver_rec.job, driver_rec.job));
         htp.tableData (driver_rec.what);
         htp.tableData (to_char (driver_rec.last_date, date_fmt_str$), cnowrap=>'NOWRAP');
         htp.tableData (to_char (driver_rec.next_date, date_fmt_str$), cnowrap=>'NOWRAP');
         htp.tableData (driver_rec.interval);
         htp.tableData (calc_time (round (driver_rec.total_time)), 'RIGHT' );
         htp.tableData (driver_rec.failures, 'CENTER');
         htp.tableData (common.yes_no (driver_rec.broken), 'CENTER');
         htp.tableRowClose;
      end loop;

      htp.tableClose;
      htp.para;

-- **********

      show_CTX_status;
      show_email_counts;

   end if;

end jobs;

-- **********************************************************************************************
-- **********************************************************************************************

procedure show_links is

begin

   htp.ulistOpen;
   htp.listItem (htf.anchor ('repository.html', 'Back to Repository Home Page'));
   htp.listItem (htf.anchor ('repository.edit_affil?affil_id=&addr_id=&letter=&string=', 'New Affiliation'));
   htp.listItem (htf.anchor ('affil_reqs.html', 'Affiliation Requests'));
   htp.listItem (htf.anchor ('repository.affiliations', 'Affiliation Maintenance'));

   htp.ulistClose;
   htp.para;

end show_links;

-- **********************************************************************************************
-- **********************************************************************************************

procedure xls is

cursor	driver is
select	affil.affil_abbrev, affil.name1, affil.name2, affil.department,
	affil.web_site, 	affil.affil_type_code, addr.*
from	affiliations		affil,
	addresses		addr
where	affil.affiliation_id	!= 0
and	affil.affiliation_id	= addr.affiliation_id;

crlf$	constant varchar2(2) := chr(13) || chr(10);
tab$	constant varchar2(1) := chr(9);

affil_type_descr	affil_type_codes.affil_type_descr%TYPE;
country_descr		country_codes.country_descr%TYPE;
region_descr		region_codes.region_descr%TYPE;

begin

   common.get_gbl_pid;

   if common.has_privs_to_page then
      owa_util.mime_header('application/vnd.excel', TRUE);
      htp.prn ('Abbrev' || tab$ || 'Name1' || tab$ || 'Name2' || tab$ || 'Department' || tab$ ||
		'Type' || tab$ || 'Address1' || tab$ || 'Address2' || tab$ || 'Address3' || tab$ ||
		'PO Box' || tab$ || 'Town' || tab$ || 'Postal Code' || tab$ || 'Zip Code' || tab$ ||
		'Country Code' || tab$ || 'Country Name' || tab$ || 'Region' || tab$ || 'URL');
      htp.prn (crlf$);

      for driver_rec in driver LOOP
         affil_type_descr	:= common.decode_value ('affil_type_descr', 'affil_type_codes',
					'affil_type_code', driver_rec.affil_type_code);
         country_descr		:= common.decode_value ('country_descr', 'country_codes',
					'country_code', driver_rec.country_code);
         region_descr		:= common.decode_value ('region_descr', 'region_codes', 'region_code',
					common.decode_value ('region_code', 'country_codes',
							'country_code', driver_rec.country_code) );

         htp.prn (	driver_rec.affil_abbrev	|| tab$ ||
			driver_rec.name1	|| tab$ ||
			driver_rec.name2	|| tab$ ||
			driver_rec.department	|| tab$ ||
			affil_type_descr	|| tab$ ||
			driver_rec.address1	|| tab$ ||
			driver_rec.address2	|| tab$ ||
			driver_rec.address3	|| tab$ ||
			driver_rec.po_box	|| tab$ ||
			driver_rec.town		|| tab$ ||
			driver_rec.postal_code	|| tab$ ||
			driver_rec.zip_code	|| tab$ ||
			driver_rec.country_code	|| tab$ ||
			country_descr		|| tab$ ||
			region_descr		|| tab$ ||
			driver_rec.web_site	|| crlf$);
      end loop;
   end if;

end xls;

-- *******************************************************************************************
-- *******************************************************************************************

procedure copy_affil2 (aid1 integer, aid2 integer, letter varchar2, string varchar2) is

cursor	driver (AID integer) is
select	*
from	addresses
where	affiliation_id = AID;

driver_rec	driver%ROWTYPE;

begin

   if common.header (rpt_title$) then
      htp.header (3, 'Copy Complete');

      htp.tableOpen;
      htp.tableRowOpen;

      open driver (aid1);
      fetch driver into driver_rec;
      close driver;

      htp.p ('<TD>');
         htp.formOpen ('repository.edit_affil');
         htp.formHidden ('affil_id',	aid1);
         htp.formHidden ('addr_id',	driver_rec.address_id);
         htp.formHidden ('letter',		letter);
         htp.formHidden ('string',		string);
         htp.formSubmit (null, 'Edit The NEW Affiliation');
         htp.formClose;
      htp.p ('</TD>');

      open driver (aid2);
      fetch driver into driver_rec;
      close driver;

      htp.p ('<TD>');
         htp.formOpen ('repository.edit_affil');
         htp.formHidden ('affil_id',	aid2);
         htp.formHidden ('addr_id',	driver_rec.address_id);
         htp.formHidden ('letter',		letter);
         htp.formHidden ('string',		string);
         htp.formSubmit (null, 'Edit The OLD Affiliation');
         htp.formClose;
      htp.p ('</TD>');

      if letter is NOT null then
         htp.p ('<TD>');
            htp.formOpen ('repository.affil_list');
            htp.formHidden ('letter', letter);
            htp.formSubmit (null, 'Back to Previous List');
            htp.formClose;
         htp.p ('</TD>');
      end if;

      if string is NOT null then
         htp.p ('<TD>');
            htp.formOpen ('repository.search');
            htp.formHidden ('string', string);
            htp.formSubmit (null, 'Back to Previous List');
            htp.formClose;
         htp.p ('</TD>');
      end if;

      htp.tableRowClose;
      htp.tableClose;

      htp.para;
      common.footer;
   end if;

end copy_affil2;

-- *******************************************************************************************
-- *******************************************************************************************

procedure copy_affil (affil_id integer, letter varchar2, string varchar2) is

next_id		integer;

begin

   select affiliation_id_seq.nextval into next_id from dual;

   insert into affiliations
		(affiliation_id, affil_abbrev, name1, name2,
		 web_site, affil_type_code, department)
   select next_id, affil_abbrev, name1 || ' COPY', name2, web_site,
	  affil_type_code, department
   from	  affiliations
   where  affiliation_id = affil_id;

   insert into addresses (address_id, address1, address2, address3,
	  po_box, town, postal_code, zip_code, country_code, affiliation_id, update_date)
   select address_id_seq.nextval, address1, address2, address3,
	  po_box, town, postal_code, zip_code, country_code, next_id, sysdate
   from   addresses
   where  affiliation_id = affil_id;

   profile.check_CTX_index;
   common.redirect_url ('repository.copy_affil2'		||
				'?aid1='	|| next_id	||
				'&aid2='	|| affil_id	||
				'&letter='	|| letter	||
				'&string='	|| string);

end copy_affil;

-- *******************************************************************************************
-- *******************************************************************************************

procedure del_addr2 (addr_id integer, letter varchar2, string varchar2) is

integrity_constraint	exception;
pragma			exception_init (integrity_constraint, -2292);

begin

   if common.header (rpt_title$, test_only=>true) then
      delete from addresses where address_id = addr_id;
      profile.check_CTX_index;
      back_to_search (letter, string);
   else
      common.public_header (rpt_title$);
      htp.p ('Insufficient Privileges');
      htp.p (common.click_back$);
      common.footer;
   end if;

exception when integrity_constraint then
   common.public_header (rpt_title$);
   htp.header (3, 'Unable to Delete');
   htp.p ('This address record is in use on one or more profiles.');
   htp.br;
   htp.formOpen ('repository.affil_list', 'POST');
   htp.formHidden ('letter', letter);
   htp.formSubmit (null, 'Continue');
   htp.formClose;
   common.footer;

end del_addr2;

-- *******************************************************************************************
-- *******************************************************************************************

procedure del_addr (addr_id integer, letter varchar2, string varchar2) is

cursor	driver is
select	affil.*
from	affiliations		affil,
	addresses		addr
where	addr.address_id		= addr_id
and	affil.affiliation_id	= addr.affiliation_id;

driver_rec	driver%ROWTYPE;

begin

   if common.header (rpt_title$) then
      open driver;
      fetch driver into driver_rec;
      close driver;

      htp.header (3, 'Are you sure you wish to delete this address record?');

      htp.tableOpen;
      common.show_data_row ('Name1',	driver_rec.name1);

      if driver_rec.name2 is not null then
         common.show_data_row ('Name2',	driver_rec.name2);
      end if;

      common.show_data_row ('Address',	common.get_address (addr_id, htf.br) );
      htp.tableClose;

      htp.para;

      htp.tableOpen;
      htp.tableRowOpen;

      htp.p ('<TD>');
         htp.formOpen ('repository.del_addr2', 'POST');
         htp.formHidden ('addr_id', addr_id);
         htp.formHidden ('letter',  letter);
         htp.formHidden ('string',  string);
         htp.formSubmit (null, 'Yes');
         htp.formClose;
      htp.p ('</TD>');

      htp.p ('<TD>');
         htp.formOpen ('repository.affil_list', 'POST');
         htp.formHidden ('letter',  letter);
         htp.formSubmit (null, 'No');
         htp.formClose;
      htp.p ('</TD>');

      htp.tableRowClose;
      htp.tableClose;

   end if;

   common.footer;

end del_addr;

-- *******************************************************************************************
-- *******************************************************************************************

procedure del_affil2 (affil_id integer, letter varchar2) is

integrity_constraint	exception;
pragma			exception_init (integrity_constraint, -2292);

begin

   if common.header (rpt_title$, test_only=>true) then
      delete from affil_requests
      where address_id in (
	select	address_id
	from	addresses
	where	affiliation_id = affil_id);

      delete from addresses	where affiliation_id = affil_id;
      delete from affiliations	where affiliation_id = affil_id;
      profile.check_CTX_index;
      back_to_search (letter, null);
   else
      common.public_header (rpt_title$);
      htp.p ('Insufficient Privileges');
      htp.p (common.click_back$);
      common.footer;
   end if;

exception when integrity_constraint then
   common.public_header (rpt_title$);
   htp.header (3, 'Unable to Delete');
   htp.p ('This affiliation record is in use on one or more profiles.');
   htp.br;
   htp.formOpen ('repository.affil_list', 'POST');
   htp.formHidden ('letter', letter);
   htp.formSubmit (null, 'Continue');
   htp.formClose;
   common.footer;

end del_affil2;

-- *******************************************************************************************
-- *******************************************************************************************

procedure del_affil (affil_id integer, letter varchar2) is

cursor	driver is
select	*
from	affiliations
where	affiliation_id	= affil_id;

driver_rec	driver%ROWTYPE;

begin

   if common.header (rpt_title$) then
      open driver;
      fetch driver into driver_rec;
      close driver;

      htp.header (3, 'Are you sure you wish to delete this affiliation?');

      htp.tableOpen;
      common.show_data_row ('Name1',	driver_rec.name1);

      if driver_rec.name2 is not null then
         common.show_data_row ('Name2',	driver_rec.name2);
      end if;

      htp.tableClose;

      htp.para;

      htp.tableOpen;
      htp.tableRowOpen;

      htp.p ('<TD>');
         htp.formOpen ('repository.del_affil2', 'POST');
         htp.formHidden ('affil_id', affil_id);
         htp.formHidden ('letter',  letter);
         htp.formSubmit (null, 'Yes');
         htp.formClose;
      htp.p ('</TD>');

      htp.p ('<TD>');
         htp.formOpen ('repository.affil_list', 'POST');
         htp.formHidden ('letter',  letter);
         htp.formSubmit (null, 'No');
         htp.formClose;
      htp.p ('</TD>');

      htp.tableRowClose;
      htp.tableClose;

      common.footer;
   end if;

end del_affil;

-- *******************************************************************************************
-- *******************************************************************************************

procedure save_affil (letter varchar2, string varchar2, affil_id integer, addr_id integer,
			affil common.ident_arr, this_affil_type varchar2, addr common.ident_arr,
			this_country varchar2, ctxt varchar2) is

continue	boolean := true;
message		varchar2(500);
next_id		integer := affil_id;

begin

   if common.header (rpt_title$) then
      if trim (affil(2)) is null then
         message := message || htf.header (3, 'You must supply Name');
         continue := false;
      end if;

      if this_country is null then
         message := message || htf.header (3, 'You must supply the Country');
         continue := false;
      end if;

      if continue then

         if affil_id is null then -- New Affiliation
            select affiliation_id_seq.nextval into next_id from dual;

            insert into affiliations
		(affiliation_id, affil_abbrev, name1,
		name2, department, web_site, affil_type_code, admin_comments)
            values (next_id, trim (affil(1)), trim (affil(2)),
		trim (affil(3)), trim (affil(4)), trim (affil(5)), this_affil_type,
		substr (trim (ctxt), 1, 4000) );

         else
            update	affiliations
            set		affil_abbrev	= trim (affil(1)),
			name1		= trim (affil(2)),
			name2		= trim (affil(3)),
			department	= trim (affil(4)),
			web_site	= trim (affil(5)),
			affil_type_code	= this_affil_type,
			admin_comments	= substr (trim (ctxt), 1, 4000)
            where	affiliation_id	= next_id;

         end if;

-- **********

         if addr_id is null then -- new address
            insert into addresses (address_id, address1, address2, address3,
			po_box, town, postal_code, zip_code, country_code, affiliation_id)
            values (address_id_seq.nextval, trim(addr(1)), trim(addr(2)), trim(addr(3)), trim(addr(4)),
			trim(addr(5)), trim(addr(6)), trim(addr(7)), this_country, next_id);
         else
            update	addresses
            set		address1	= trim (addr(1)),
			address2	= trim (addr(2)),
			address3	= trim (addr(3)),
			po_box		= trim (addr(4)),
			town		= trim (addr(5)),
			postal_code	= trim (addr(6)),
			zip_code	= trim (addr(7)),
			country_code	= this_country
            where	address_id	= addr_id;
         end if;

-- **********

         profile.check_CTX_index;
         back_to_search (letter, string);
      else
         common.public_header (rpt_title$);
         htp.p (message);
         htp.p (common.click_back$);
         common.footer;
      end if;

   end if;

end save_affil;

-- *******************************************************************************************

function req (buffer varchar2) return varchar2 is

begin

   if common.has_privs_to_page then
      return buffer;
   end if;

   return common.red (buffer || '*');

end req;

-- *******************************************************************************************

procedure affil_addr (addr_id integer) is

cursor	driver is
select	*
from	addresses
where	address_id = addr_id;

driver_rec	driver%ROWTYPE;

begin

   open driver;
   fetch driver into driver_rec;
   close driver;

   common.show_data_row ('Address 1',			htf.formText ('addr', 40, 40, driver_rec.address1) );
   common.show_data_row ('Address 2',			htf.formText ('addr', 40, 40, driver_rec.address2) );
   common.show_data_row ('Address 3',			htf.formText ('addr', 40, 40, driver_rec.address3) );
   common.show_data_row ('PO Box',			htf.formText ('addr', 40, 40, driver_rec.po_box) );
   common.show_data_row (common.red('Town*'),		htf.formText ('addr', 40, 40, driver_rec.town) );
   common.show_data_row (common.red('Postal Code*'),	htf.formText ('addr', 40, 40, driver_rec.postal_code) );
   common.show_data_row (common.red('Zip Code*'),	htf.formText ('addr', 40, 40, driver_rec.zip_code) );

   common.show_lookup_table ('country', common.red('Country*'), driver_rec.country_code);

end affil_addr;

-- *******************************************************************************************

procedure affil (affil_id integer, include_comment boolean default true) is

cursor	driver is
select	*
from	affiliations
where	affiliation_id = affil_id;

driver_rec	driver%ROWTYPE;

begin

   open driver;
   fetch driver into driver_rec;
   close driver;

   common.show_data_row ('Acronym',	htf.formText ('affil', 30, 25,  driver_rec.affil_abbrev) );
   common.show_data_row (common.red('Name1*'),	htf.formText ('affil',  50, 200, driver_rec.name1) );
   common.show_data_row ('Name2',	htf.formText ('affil',  50, 200, driver_rec.name2) );
   common.show_data_row ('Department',	htf.formText ('affil',   50, 200,  driver_rec.department) || ' Optional');
   common.show_data_row ('Web Site',	htf.formText ('affil',    50, 100, driver_rec.web_site) );

   common.show_lookup_table ('affil_type', common.red('Type'),	driver_rec.affil_type_code);

   if include_comment then
      common.show_data_row ('Comments',	htf.formTextAreaOpen ('ctxt', 6, 100)	||
					driver_rec.admin_comments		||
					htf.formTextAreaClose);
   end if;

end affil;

-- *******************************************************************************************

procedure edit_affil (affil_id integer, addr_id integer, letter varchar2, string varchar2) is

begin

   if common.header (rpt_title$) then
      htp.formOpen ('repository.save_affil', 'POST');
      htp.formHidden ('letter', letter);
      htp.formHidden ('string', string);
      htp.formHidden ('affil_id', affil_id);
      htp.formHidden ('addr_id',  addr_id);

      htp.tableOpen;
      affil (affil_id);
      affil_addr (addr_id);
      common.show_data_row (null, htf.formSubmit (null, 'Save'));
      htp.tableClose;

      if affil_id is not null then
         htp.para;
         htp.anchor ('repository.copy_affil?affil_id=' || affil_id ||
			'&letter=' || letter || '&string=' || string,
			'Duplicate This Affiliation');
      end if;

      htp.para;
      common.footer;
   end if;

end edit_affil;

-- *******************************************************************************************
-- *******************************************************************************************

procedure merge_addr3b (addrs common.ident_arr, merge_into_id integer) is

-- This routine runs on the repository only; never the conference database.

cursor	driver (old_id integer) is
select	p1.profile_id	old_profile_id,
	p2.profile_id	new_profile_id
from	profiles	p2,
	profiles	p1
where	p1.address_id	= old_id
and	p2.address_id	= merge_into_id
and	p2.person_id	= p1.person_id;

type		rec_type is table of driver%ROWTYPE index by binary_integer;
recs		rec_type;
sync_merge_id	integer;

begin

   for i in 2..addrs.COUNT LOOP
      if addrs(i) <> merge_into_id then

--	Change all address to the new one if the person doesn't already have that address in a profile
         update	profiles	p1
         set	p1.address_id	= merge_into_id
         where	address_id	= addrs(i)
         and	not exists
		(	select	1
			from	profiles	p2
			where	p2.person_id	= p1.person_id
			and	p2.address_id	= merge_into_id);

         update	affil_requests
         set	address_id	= merge_into_id
         where	address_id	= addrs(i);

--	Record profiles that have both the old and new address for conferences to update during sync

         open driver (addrs(i));
         fetch driver BULK COLLECT into recs;
         close driver;

         for j in 1..recs.COUNT loop
            select sync_id_seq.nextval into sync_merge_id from dual;

            insert into sync_data (sync_id, table_name, action)
            values (sync_merge_id, 'PROFILES', 'MERGE');

            insert into sync_merge_profiles (sync_id, old_profile_id, new_profile_id)
            values (sync_merge_id, recs(j).old_profile_id, recs(j).new_profile_id);

            sync.merge_profile (recs(j).old_profile_id, recs(j).new_profile_id);
         end loop;

--	Remove all profiles with the old address; either merged above or already exists
         delete from profiles	where address_id = addrs(i);
         delete from addresses	where address_id = addrs(i);
      end if;
   end loop;

   profile.check_CTX_index;

end merge_addr3b;

-- *******************************************************************************************

procedure merge_addr3 (letter varchar2, string varchar2,
			addrs common.ident_arr, merge_into_id integer) is

begin

   merge_addr3b (addrs, merge_into_id);
   back_to_search (letter, string);

end merge_addr3;

-- *******************************************************************************************

procedure merge_addr2 (affil_id integer, letter varchar2,
			string varchar2, addr_id common.ident_arr) is

checked	varchar2(8) := 'checked';

begin

   if common.header (rpt_title$) then
      htp.header (3, rpt_title2$);
      htp.bold ('All addresses on this screen will be merged into the selected address.');
      htp.para;
      htp.bold (common.get_affil_name (affil_id, htf.br));

      htp.formOpen ('repository.merge_addr3', 'POST');
      htp.formHidden ('letter', letter);
      htp.formHidden ('string', string);
      common.hidden_array ('addrs', addr_id);

      htp.tableOpen;
      for i in 2..addr_id.COUNT LOOP
         htp.tableRowOpen ('LEFT', 'TOP');
         htp.tableData (htf.formRadio ('merge_into_id', addr_id(i), cchecked=>checked));
         htp.tableData (common.get_address (addr_id(i), htf.br));
         htp.tableRowClose;

         checked := null;
      end loop;
      htp.tableClose;

      htp.formSubmit;
      htp.formClose;
      htp.para;
      htp.anchor (get_return_URL (letter, string), 'Back to Affiliation Search');
      common.footer;
   end if;

end merge_addr2;

-- *******************************************************************************************

procedure merge_addr (affil_id integer, letter varchar2, string varchar2) is

cursor	driver is
select	address_id
from	addresses
where	affiliation_id = affil_id
order by address_id;

addrs	common.ident_arr;

begin

   if common.header (rpt_title$, test_only=>true) then
      open driver;
      fetch driver BULK COLLECT into addrs;
      close driver;

      if addrs.COUNT > 2 then
         common.public_header (rpt_title$);
         htp.header (3, rpt_title2$);
         htp.bold (common.get_affil_name (affil_id, htf.br));
         htp.para;

         htp.formOpen ('repository.merge_addr2', 'POST');
         htp.formHidden ('affil_id', affil_id);
         htp.formHidden ('letter', letter);
         htp.formHidden ('string', string);
         htp.formHidden ('addr_id', 'dummy');

         htp.tableOpen;
         for driver_rec in driver LOOP
            htp.tableRowOpen ('LEFT', 'TOP');
            htp.tableData (htf.formCheckBox ('addr_id', driver_rec.address_id));
            htp.tableData (common.get_address (driver_rec.address_id, htf.br));
            htp.tableRowClose;

            common.show_data_sep (2);
         end loop;
         htp.tableClose;

         htp.formSubmit;
         htp.formClose;
         htp.para;
         htp.anchor (get_return_URL (letter, string), 'Back to Affiliation Search');
         common.footer;

      else -- there are only two address, no need to ask which ones to merge
         addrs(3) := addrs(2);
         addrs(2) := addrs(1);
         addrs(1) := 'dummy';

         merge_addr2 (affil_id, letter, string, addrs);
      end if;
   end if;

end merge_addr;

-- *******************************************************************************************
-- *******************************************************************************************

procedure show_addresses (affil_id integer, letter varchar2, string varchar2) is

cursor	driver is
select	*
from	addresses
where	affiliation_id	= affil_id
order by
	address1	nulls first,
	address2	nulls first,
	address3	nulls first,
	po_box		nulls first,
	town		nulls first,
	postal_code	nulls first,
	zip_code	nulls first,
	update_date	desc;

URL	varchar2(256);
icount	integer := 0;

begin

   htp.p ('<TD>');

   htp.tableOpen;
   for driver_rec in driver LOOP
      htp.tableRowOpen ('LEFT', 'TOP');
      htp.tableData (htf.anchor ('repository.edit_affil'	||
					'?affil_id='		|| affil_id			||
					'&addr_id='		|| driver_rec.address_id	||
					'&letter='		|| letter			||
					'&string='		|| common.sc (string), 'EDIT')		||
		     htf.br ||
		     htf.anchor ('repository.del_addr'		||
					'?addr_id='		|| driver_rec.address_id	||
					'&letter='		|| letter			||
					'&string='		|| common.sc (string), 'Delete') );
      htp.tableData (common.get_address (driver_rec.address_id, htf.br) || htf.br ||
			'Last Updated: ' || to_char (driver_rec.update_date, common.date_fmt_str$) || htf.br);
      htp.tableRowClose;
      common.show_data_sep (2);
      icount := icount + 1;
   end loop;

   if icount > 1 then
      URL := 			common.space2$ || common.space2$ ||
		  htf.anchor ('repository.merge_addr'	||
				'?affil_id='		|| affil_id	||
				'&letter='		|| letter	||
				'&string='		|| common.sc (string), 'Merge Address');
   end if;

   htp.tableRowOpen ('LEFT', 'TOP');
   htp.tableData (htf.anchor ('repository.edit_affil'	||
				'?affil_id='		|| affil_id	||
				'&addr_id='		||
				'&letter='		|| letter	||
				'&string='		|| common.sc (string), 'New Address') ||
			URL, ccolspan=>'2');
   htp.tableRowClose;

   htp.tableClose;
   htp.p ('</TD>');

end show_addresses;

-- *******************************************************************************************

procedure show_list_hdr (show_score boolean) is

begin

   htp.tableOpen ('BORDER');
   htp.tableRowOpen ('LEFT', 'BOTTOM');

   if show_score then
      htp.tableHeader ('Score');
   end if;

   htp.tableHeader ('Merge');
   htp.tableHeader ('Acronym');
   htp.tableHeader ('Name');
   htp.tableHeader ('Mailing Addresses');
   htp.tableHeader ('Comments');
   htp.tableRowClose;

end show_list_hdr;

-- *******************************************************************************************

procedure show_list_data (affil_rec dummy%ROWTYPE, letter varchar2 default null,
			iscore integer default null, checkBox boolean default true,
			style varchar2 default null) is

type_descr	affil_type_codes.affil_type_descr%TYPE;

begin

   type_descr := common.decode_value ('affil_type_descr', 'affil_type_codes',
					'affil_type_code', affil_rec.affil_type_code);
   if iscore is not null then
      htp.tableData (iscore, 'CENTER');
   end if;

   if checkBox then
      htp.tableData (htf.formCheckBox ('merge', affil_rec.affiliation_id), 'CENTER');
   else
      htp.tableData (htf.formRadio ('merge', affil_rec.affiliation_id, 'SELECTED'), 'CENTER');
   end if;

   htp.tableData (nvl (affil_rec.affil_abbrev, '** No Acronym') || htf.br ||
			htf.anchor ('repository.del_affil' ||
					'?affil_id='	|| affil_rec.affiliation_id	||
					'&letter='	|| letter,
					'Delete') );
   htp.tableData (htf.anchor (affil_rec.web_site, affil_rec.name1) || htf.br ||
			common.ifnotnull (affil_rec.name2, affil_rec.name2 || htf.br) ||
			common.ifnotnull (affil_rec.department, affil_rec.department || htf.br) ||
			common.ifnotnull (affil_rec.affil_type_code,
				htf.bold ('Type: ') || type_descr || htf.br ||
			'Last Updated: ' || to_char (affil_rec.update_date, common.date_fmt_str$) ),
			cattributes=>style );

end show_list_data;

-- *******************************************************************************************

function same_text (text1 varchar2, text2 varchar2) return boolean is

begin

   if	nlssort (upper(nvl(text1,' ')), 'NLS_SORT = Latin') =
	nlssort (upper(nvl(text2,' ')), 'NLS_SORT = Latin') then
      return true;
   end if;

   return false;

end same_text;

-- *******************************************************************************************

procedure show_affil_list (letter varchar2) is

cursor	driver is
select	*
from	affiliations
where	name1	like letter || '%'
order by
	common.sort_text (name1),
	common.sort_text (name2),
	update_date desc;

type	rec_type is table of driver%ROWTYPE index by binary_integer;
recs	rec_type;
style$	varchar2(40);

begin

   if common.header (rpt_title$) then
      show_links;

      htp.formOpen ('repository.merge', 'POST');
      htp.formHidden ('merge', 'dummy');
      htp.formHidden ('letter', letter);
      htp.formHidden ('string', '');
      show_list_hdr (false);

      open driver;
      fetch driver BULK COLLECT into recs;
      close driver;

      for i in 1..recs.COUNT LOOP
         style$ := null;

         if i <> 1 then
            if same_text (recs(i).name1, recs(i-1).name1)	AND
		same_text (recs(i).name2, recs(i-1).name2)	AND
		same_text (recs(i).department, recs(i-1).department) then
               style$ := common.highlight$;
            end if;
         end if;

         if i <> recs.COUNT then
            if same_text (recs(i).name1, recs(i+1).name1)	AND
		same_text (recs(i).name2, recs(i+1).name2)	AND
		same_text (recs(i).department, recs(i+1).department) then
               style$ := common.highlight$;
            end if;
         end if;

         htp.tableRowOpen ('LEFT', 'TOP');
         show_list_data (recs(i), letter, null, true, style$);
         show_addresses (recs(i).affiliation_id, letter, null);
         htp.tableData (recs(i).admin_comments);
         htp.tableRowClose;
      end loop;

      htp.tableClose;
      htp.para;
      htp.formSubmit (null, 'Merge');
      htp.formClose;
      common.footer;
   end if;

end show_affil_list;

-- *******************************************************************************************

procedure affil_list (letter varchar2) is

begin

   if letter is null then
      affiliations;
   else
      show_affil_list (letter);
   end if;

end affil_list;

-- *******************************************************************************************
-- *******************************************************************************************

procedure merge2 (merge integer, list common.ident_arr, letter varchar2, string varchar2,
			req_id integer default null) is

begin

   common.get_gbl_pid;

   if common.has_privs_to_page then
      for i in 1..list.count LOOP
         if merge <> list(i) then
            update	addresses
            set		affiliation_id = merge
            where	affiliation_id = list(i);

            delete	from affiliations where affiliation_id = list(i);

            profile.check_CTX_index;
         end if;
      end loop;
   end if;

   if req_id is not null then
      delete from affil_requests where affil_request_id = req_id;
      common.redirect_URL ('repository.merge?merge=dummy&merge=' || merge ||
			'&letter=&string=req_id=');
   else
--    back_to_search (letter, string);
      common.redirect_URL ('repository.merge_addr?affil_id=' || merge ||
			'&letter=' || letter || '&string=' || string);
   end if;

end merge2;

-- *******************************************************************************************
-- *******************************************************************************************

procedure merge (merge common.ident_arr, letter varchar2, string varchar2, req_id integer default null) is

cursor	driver (ID integer) is
select	*
from	affiliations
where	affiliation_id = ID;

driver_rec	driver%ROWTYPE;

begin

   if common.header (rpt_title$) then
      show_links;

      if merge.count = 2 then
         htp.header (3, 'Merge Addresses (Optional)');
      else
         htp.header (3, 'Merge these affiliations into the selected one.');
      end if;

      htp.formOpen ('repository.merge2', 'POST');
      htp.formHidden ('req_id', req_id);

      for i in 2..merge.count LOOP
         htp.formHidden ('list', merge(i));
      end loop;

      htp.formHidden ('letter', letter);
      htp.formHidden ('string', string);
      show_list_hdr (false);

      for i in 2..merge.count LOOP
         open driver (merge(i));
         fetch driver into driver_rec;
         close driver;

         htp.tableRowOpen ('LEFT', 'TOP');
         show_list_data (driver_rec, letter, null, false);
         show_addresses (driver_rec.affiliation_id, string, null);
         htp.tableData (driver_rec.admin_comments);
         htp.tableRowClose;
      end loop;

      htp.tableClose;
      htp.para;

      if merge.count > 2 then
         htp.formSubmit (null, 'Merge');
      end if;
      htp.formClose;
      common.footer;
   end if;

end merge;

-- *******************************************************************************************
-- *******************************************************************************************

procedure search (string varchar2) is

cursor	driver is
select	distinct score(1) score, affil.*
from	addresses		addr,
	affiliations		affil
where	addr.affiliation_id	= affil.affiliation_id
and  (	contains (affil.search_text, string, 1) > 0
     OR	contains (addr.search_text,  string, 2) > 0 )
and	affil.affiliation_id	>= 0
order by score(1) desc, affil.name1, affil.name2 nulls first;

affil_rec	dummy%ROWTYPE;

begin

   if common.header (rpt_title$) then
      show_links;

      htp.formOpen ('repository.merge', 'POST');
      htp.formHidden ('merge', 'dummy');
      htp.formHidden ('letter', '');
      htp.formHidden ('string', string);

      show_list_hdr (true);

      for driver_rec in driver LOOP
         affil_rec.affiliation_id	:= driver_rec.affiliation_id;
         affil_rec.affil_abbrev		:= driver_rec.affil_abbrev;
         affil_rec.web_site		:= driver_rec.web_site;
         affil_rec.name1		:= driver_rec.name1;
         affil_rec.name2		:= driver_rec.name2;
         affil_rec.department		:= driver_rec.department;
         affil_rec.affil_type_code	:= driver_rec.affil_type_code;
         affil_rec.update_date		:= driver_rec.update_date;
         affil_rec.admin_comments	:= driver_rec.admin_comments;

         htp.tableRowOpen ('LEFT', 'TOP');
         show_list_data (affil_rec, null, driver_rec.score, true);
         show_addresses (driver_rec.affiliation_id, null, string);
         htp.tableData (affil_rec.admin_comments);
         htp.tableRowClose;
      end loop;

      htp.tableClose;
      htp.para;
      htp.formSubmit (null, 'Merge');
      htp.formClose;
      common.footer;
   end if;

end search;

-- *******************************************************************************************
-- *******************************************************************************************

procedure re_index_prompt is

cursor	C1 is
select	count(1)
from	ctxsys.dr$pending
where	exists
(	select	1
	from	affiliations
	where	rowid = pnd_rowid);

cursor	c2 is
select	count(1)
from	ctxsys.dr$pending
where	exists
(	select	1
	from	addresses
	where	rowid = pnd_rowid);

icount1		integer;
icount2		integer;

begin

   open C1;
   fetch C1 into icount1;
   close C1;

   open C2;
   fetch C2 into icount2;
   close C2;

   if icount1+icount2 > 0 then
      htp.anchor ('repository.affil_indx', icount1+icount2 ||
		' Entries in the Affiliation Index Needs Rebuilding');
      htp.para;
   end if;

end re_index_prompt;

-- *******************************************************************************************

procedure affiliations is

cursor	driver is
select	distinct substr (name1, 1, 1)	letter
from	affiliations
where	affiliation_id	> 0
order by
	common.sort_text (substr (name1, 1, 1));

begin

   if common.header (rpt_title$) then
      show_links;

      htp.header (3, 'Find an Affiliation');
      htp.p ('Click on a letter for a list of affiliations whose name begins with that letter.');
      htp.para;

      htp.fontOpen (csize=>'+1');
      for driver_rec in driver LOOP
         htp.anchor ('repository.affil_list?letter=' || driver_rec.letter, driver_rec.letter);
         htp.p (common.space2$);
      end loop;
      htp.fontClose;

      htp.para;

      if NOT common.repo_is_remote then
         re_index_prompt;
      end if;

      htp.formOpen ('repository.search', 'POST');
      htp.formText ('string', 50, 200);
      htp.formSubmit (null, 'Search Affiliations');
      htp.formClose;

      htp.para;
      common.footer;
   end if;

end affiliations;

-- *******************************************************************************************
-- *******************************************************************************************

procedure affil_indx is

begin

   profile.check_CTX_index;
   common.redirect_URL ('repository.affiliations');

end affil_indx;

-- *******************************************************************************************
-- *******************************************************************************************

procedure html is

begin

   common.get_gbl_pid;
   if common.logged_in then
      if common.has_privs_to_page then
         maint.header (rpt_title$);
         maint.menu (0);
         common.footer;
      else
         common.redirect_url (common.server || common.vdir$ || 'jacow.html');
      end if;
   end if;

end html;

-- *******************************************************************************************

end repository;
/
show error

set define on

--	Copyright (C) 2004 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>
