set define off

create or replace package body search as

rpt_title$	constant varchar2(20) := 'Search';
shade$		constant varchar2(35)	:= 'STYLE="background-color:#D3D3D3"';

show_header	boolean := true;
Q$		constant varchar2(1)	:= chr(39);
Q2$		constant varchar2(2)	:= chr(39) || chr(39);

type		priority_recs_type is table of priority_codes%ROWTYPE index by binary_integer;
type		proposal_recs_type is table of proposal_status_codes%ROWTYPE index by binary_integer;

OK_edit_abstract_editor		boolean;
OK_sort_file_page		boolean;
OK_editor_page			boolean;
OK_abstract_maint_edit		boolean;
OK_editor_qa_paper		boolean;
OK_email_one_email		boolean;
OK_JACoW_abs_log		boolean;
OK_jacow_additional_sessions	boolean;
OK_JACoW_change_owner		boolean;
OK_JACoW_copyright		boolean;
OK_JACoW_edit_abstract_editor	boolean;
OK_JACoW_file_info		boolean;
OK_JACoW_print_abstract		boolean;
OK_JACoW_set_editor		boolean;
OK_JACoW_withdraw_abstract	boolean;
OK_profile_find_repo3		boolean;
OK_referee_edit			boolean;
OK_search_abs_QA		boolean;
OK_set_proposal_page		boolean;

-- ******************************************************************************************
-- ******************************************************************************************

procedure withdraw_nopriority2 is

cursor	driver is
select	abstract_id
from	abstracts
where	abstract_id NOT in (
	select	distinct abstract_id
	from	abstract_proposals
	where	proposal_status_code = prog_cmtte.proposal_accepted$);

IDs	common.ident_arr;

begin

   if common.header ('Withdraw Proposals with no Priority') then
      if common.PC_mode$ = 'Y' then
         open driver;
         fetch driver BULK COLLECT into IDs;
         close driver;

         for i in 1..IDs.COUNT LOOP
            jacow.journal_abstract (IDs(i));
         end loop;

         update	abstracts a
         set	person_id = nvl (
			(	select	person_id
				from	abstract_authors	aa
				where	aa.abstract_id		= a.abstract_id
				and	aa.is_speaker		= 'Y'),
			a.person_id);

         htp.header (3, IDs.COUNT || ' Proposals Withdrawn');
         htp.formOpen ('search.html', 'POST');
         htp.formSubmit (null, 'Continue');
         htp.formClose;

      else
         htp.header (3, 'SPMS is not in Abstract Proposal mode.', cattributes=>common.red$);
         htp.bold ('You cannot withdraw abstracts with this procedure at this time.');
      end if;

      common.footer;
   end if;

end withdraw_nopriority2;

-- ******************************************************************************************

procedure withdraw_nopriority is

cursor	driver is
select	a.abstract_id, a.title,
	cc.classification_descr
from	classification_codes	cc,
	abstracts		a
where	a. abstract_id not in
	(	select	distinct abstract_id
		from	abstract_proposals
		where	proposal_status_code is not null)
and	cc.classification_code	= a.classification_code
order by cc.classification_descr, a.abstract_id;

type	recs_type is table of driver%ROWTYPE index by binary_integer;
recs	recs_type;

begin

   if common.header ('Withdraw Proposals with no Priority') then
      if common.PC_mode$ = 'Y' then
         open driver;
         fetch driver BULK COLLECT into recs;
         close driver;

         htp.header (3, 'Are you sure you wish to withdraw all proposals with no ' ||
			htf.bold ('Accepted Priority?') );
         htp.italic ('Note: many abstracts may have priorities set as preferences by committee members,');
         htp.italic ('however this routine withdraws the proposal if the chair has');
         htp.italic (htf.bold ('not accepted'));
         htp.italic ('the proposal. In addition, all abstract owners will be changed to the primary author.');
         htp.para;

         htp.bold ('You will withdraw ' || recs.COUNT || ' proposals.');

         htp.tableOpen;
         htp.tableRowOpen;
         htp.tableData (htf.formOpen ('search.withdraw_nopriority2', 'POST')	||
			htf.formSubmit (null, 'Yes')				||
			htf.formClose);
         htp.tableData (common.space2$);
         htp.tableData (common.space2$);
         htp.tableData (htf.formOpen ('search.html', 'POST')			||
			htf.formSubmit (null, 'No')				||
			htf.formClose);
         htp.tableRowClose;
         htp.tableClose;

         htp.tableOpen ('BORDER');
         htp.tableRowOpen ('LEFT', 'TOP');
         htp.tableHeader ('ID');
         htp.tableHeader ('Classification');
         htp.tableHeader ('Title');
         htp.tableRowClose;

         for i in 1..recs.COUNT loop
            htp.tableRowOpen ('LEFT', 'TOP');
            htp.tableData (recs(i).abstract_id);
            htp.tableData (recs(i).classification_descr);
            htp.tableData (recs(i).title);
            htp.tableRowClose;
         end loop;

         htp.tableClose;

      else
         htp.header (3, 'SPMS is not in Abstract Proposal mode.', cattributes=>common.red$);
         htp.bold ('You cannot withdraw abstracts with this procedure at this time.');
      end if;

      common.footer;
   end if;

end withdraw_nopriority;

-- ******************************************************************************************
-- ******************************************************************************************

procedure JS is

begin

   htp.p ('function schedule() {');
   htp.p ('   window.opener.location.href="' || common.server || common.vdir$ || 'search.schedule";');
   htp.p ('   window.setTimeout("window.close()",1000);');
   htp.p ('}');

end JS;

-- ******************************************************************************************
-- ******************************************************************************************

function add_URL (abstract_id integer) return varchar2 is

begin

   return htf.anchor('javascript:newWindow(''search.add?aid=' ||
			abstract_id || ''')', 'Add');

end add_URL;

-- ******************************************************************************************
-- ******************************************************************************************

function already_scheduled (abs_id integer) return boolean is

cursor	driver is
select	1
from	person_schedules
where	person_id	= common.gbl_pid
and	abstract_id	= abs_id;

dummy		integer;
rec_found	boolean;

begin

   open driver;
   fetch driver into dummy;
   rec_found := driver%FOUND;
   close driver;

   return rec_found;

end already_scheduled;

-- ******************************************************************************************
-- ******************************************************************************************

function requires_QA (abs_id integer) return varchar2 is

cursor	driver is
select	log_code, person_id
from	logs
where	abstract_id	= abs_id
and	log_code	in (common.abstract_created$, common.abstract_updated$, common.abstract_QA$)
order by
	log_date	desc;

begin

-- If a QA was performed after the last update then no QA is required.

   for driver_rec in driver LOOP
      if driver_rec.log_code = common.abstract_QA$ then
	 return 'N';
      elsif NOT common.privileged_user (driver_rec.person_id) then
	 return 'Y';
      end if;
   end loop;

   return 'Y';

end requires_QA;

-- ******************************************************************************************
-- ******************************************************************************************

function highlight (descr varchar2, this_text varchar2) return varchar2 is

ilen		integer;
icount		integer;
ipos		integer;
epos		integer;
stop_count	integer := 1; -- failsafe to prevent infinite loop

new_descr	long		:= descr;
new_text	varchar2(100)	:= trim (this_text);
word		varchar2(50);

begin

   LOOP
      exit when stop_count > 100;
      exit when new_text is null;

      ipos := instr (new_text, ' ');
      if ipos = 0 then
	 word		:= new_text;
	 new_text	:= null;
      else
	 word		:= substr (new_text, 1, ipos-1);
	 new_text	:= trim (substr (new_text, ipos+1));
      end if;

      exit when word is null;
      ilen	:= length (word);
      icount	:= 1;

      LOOP
	 exit when stop_count > 100;
	 ipos := instr (upper (new_descr), upper (word), 1, icount);
	 exit when ipos = 0;

	 epos		:= ipos + ilen;
	 new_descr	:= substr (new_descr, 1, ipos-1) ||
				htf.cite (substr (new_descr, ipos, ilen)) ||
				substr (new_descr, epos);
	 icount		:= icount + 1;
	 stop_count	:= stop_count + 1;
      end loop;

      stop_count	:= stop_count + 1;
   end loop;

   return new_descr;

end highlight;

-- ******************************************************************************************
-- ******************************************************************************************

procedure show_schedule (printer_friendly boolean, sdate varchar2, show_abstract boolean) is

cursor	driver is
select	s.session_date, s.session_date session_date2,
	sa.start_time abs_start_time, s.beg_time start_time, s.end_time, sa.paper_id, sa.duration,
	s.session_id, a.abstract_id, a.title, a.description, s.presentation_option_code,
	s.location_code, sa.presentation_code,
	common.get_user_name_affiliation (
		common.get_author_profile_id (a.abstract_id, common.speaker$) ) speaker
from	sessions		s,
	abstract_sessions	sa,
	abstracts		a,
	person_schedules	ps
where	sdate			is null
and	ps.person_id		= nvl (common.gbl_priv_pid, common.gbl_pid)
and	a.abstract_id		= ps.abstract_id
and	sa.abstract_id		= a.abstract_id
and	s.session_id		= sa.session_id
UNION
select	s.session_date, s.session_date session_date2,
	sa.start_time abs_start_time, s.beg_time start_time, s.end_time, sa.paper_id, sa.duration,
	s.session_id, a.abstract_id, a.title, a.description, s.presentation_option_code,
	s.location_code, sa.presentation_code,
	common.get_user_name_affiliation (
		common.get_author_profile_id (a.abstract_id, common.speaker$) ) speaker
from	abstracts		a,
	abstract_sessions	sa,
	sessions		s
where	s.session_date		like decode (sdate,	null,'01-jan-0',
							'%','%',
							to_date (sdate, 'mmddyyyy'))
and	sa.session_id		= s.session_id
and	a.abstract_id		= sa.abstract_id;

time_fmt_str$	constant varchar2(20) := substr (common.date_fmt_str$, instr(common.date_fmt_str$, ' ') + 1);
conf_date	date := common.conf_start_monday$;  -- may not be a Monday.
buffer		varchar2(200);
prev_date	date := common.conf_start_monday$ - 20;

begin

   if NOT printer_friendly then
      htp.formOpen ('search.print_schedule', 'POST', 'PRINT');
      htp.bold ('Display');
      htp.formSelectOpen ('sdate');
      htp.formSelectOption ('* My Schedule *', 'SELECTED', cattributes=>'VALUE=""');
      htp.formSelectOption ('* ALL *', cattributes=>'VALUE="%"');
      for i in 1..common.conf_len_days$ LOOP
	 htp.formSelectOption (to_char (conf_date, 'Day'),
				cattributes=>'VALUE="' || to_char (conf_date, 'mmddyyyy') || '"');
	 conf_date := conf_date + 1;
      end loop;
      htp.formSelectClose;
      htp.p (common.space2$);
      htp.p ('Include Abstract');
      htp.formHidden ('show_abstract', 'DUMMY');
      htp.formCheckBox ('show_abstract', 'DUMMY', 'CHECKED');
      htp.formSubmit;
      htp.formClose;
   end if;

   htp.tableOpen;

   for driver_rec in driver LOOP

      if driver_rec.abs_start_time is not null then  -- Oral
         driver_rec.session_date := common.add_time (driver_rec.session_date, driver_rec.abs_start_time);
         driver_rec.session_date2 := common.add_time (driver_rec.session_date, driver_rec.duration);
      else
         driver_rec.session_date2 := common.add_time (driver_rec.session_date2, driver_rec.end_time);
      end if;

      buffer	:= null;
      if NOT printer_friendly then
	 buffer := htf.anchor ('search.remove?aid=' || driver_rec.abstract_id, 'Remove') || common.space$;
      end if;

      htp.tableRowOpen ('LEFT', 'TOP');
      htp.tableHeader (nvl (driver_rec.paper_id, driver_rec.abstract_id));
      htp.tableData (buffer || htf.italic (driver_rec.title) );
      htp.tableRowClose;

      common.show_data_row (null, htf.bold (common.decode_value ('presentation_descr',
		'presentation_codes', 'presentation_code', driver_rec.presentation_code) ) );

      htp.tableRowOpen ('LEFT', 'TOP');
      htp.tableHeader ('Location', 'RIGHT');
      htp.tableData (common.decode_value ('location_descr', 'location_codes',
			'location_code', driver_rec.location_code) );
      htp.tableRowClose;

      buffer := to_char (driver_rec.session_date, 'FMDay,') || ' ' ||
			to_char (driver_rec.session_date, common.date_fmt_str$)	|| ' - ' ||
			to_char (driver_rec.session_date2, time_fmt_str$);

      htp.tableRowOpen ('LEFT', 'TOP');
      htp.tableHeader ('Time', 'RIGHT');
      if driver_rec.presentation_option_code = common.oral$ AND
		prev_date between driver_rec.session_date and driver_rec.session_date2 then
	 htp.tableHeader (buffer, cattributes=>common.red$);
      else
	 htp.tableData (buffer);
      end if;
      htp.tableRowClose;

      htp.tableRowOpen ('LEFT', 'TOP');
      if common.get_presentation_type (driver_rec.presentation_code) = common.oral$ then
	 htp.tableHeader ('Speaker', 'RIGHT');
      else
	 htp.tableHeader ('Presenter', 'RIGHT');
      end if;

      htp.tableData (driver_rec.speaker);
      htp.tableRowClose;

      if show_abstract then
	 htp.tableRowOpen ('LEFT', 'TOP');
	 htp.tableHeader ('Abstract', 'RIGHT');
	 htp.tableData (driver_rec.description);
	 htp.tableRowClose;
      end if;

      common.show_data_sep (5, common.space$);
      prev_date := driver_rec.session_date;
   end loop;

   htp.tableClose;

end show_schedule;

-- ******************************************************************************************

procedure schedule is

begin

   if common.header ('Schedule') then
      show_schedule (false, null, true);
      common.footer;
   end if;

end schedule;

-- ******************************************************************************************

procedure print_schedule (sdate varchar2, show_abstract common.ident_arr) is

begin

   if common.header ('Schedule') then
      show_schedule (true, sdate, show_abstract.count=2);

      htp.bodyClose;
      htp.htmlClose;
   end if;

end print_schedule;

-- ******************************************************************************************
-- ******************************************************************************************

procedure remove (aid integer) is

begin

   common.get_gbl_pid;

   delete from person_schedules
   where  person_id	= nvl (common.gbl_priv_pid, common.gbl_pid)
   and	  abstract_id	= aid;

   schedule;

end remove;

-- ******************************************************************************************
-- ******************************************************************************************

procedure add (aid integer) is

pid$	integer;

begin

   common.get_gbl_pid;
   pid$	:= nvl (common.gbl_priv_pid, common.gbl_pid);

   htp.htmlOpen;
   htp.headOpen;
   htp.print ('<script language="JavaScript" src="' ||
		common.server || common.vdir$ || 'search.js"></script>');
   htp.headClose;
   htp.bodyOpen;

   insert into person_schedules (person_id, abstract_id)
   select pid$, aid
   from   dual
   where  NOT exists
	(	select	1
		from	person_schedules
		where	person_id	= pid$
		and	abstract_id	= aid
	);

   htp.header (3, 'Scheduled Updated');
   htp.anchor ('javascript:close();', 'Continue');
   htp.p (common.space2$);
   htp.anchor ('javascript:schedule()', 'Show Schedule');

   htp.bodyClose;
   htp.htmlClose;

end add;

-- ******************************************************************************************
-- ******************************************************************************************

procedure abs_QA (abs_id integer) is

begin

   if common.header (rpt_title$) then
      insert into logs (abstract_id, log_code) values (abs_id, common.abstract_QA$);
      htp.header (3, common.decode_value ('log_descr', 'log_codes', 'log_code', common.abstract_QA$) ||
		' Recorded');
      htp.anchor ('javascript:window.close()', 'Continue');
      common.footer;
   end if;

end abs_QA;

-- ******************************************************************************************

procedure new_search (URL varchar2) is

begin

   htp.para;
   htp.formOpen (URL, 'POST');
   htp.formSubmit (null, 'New Search');
   htp.formClose;
   htp.para;

end new_search;

-- ******************************************************************************************

procedure check_CTX_index is

-- Update the index.

cursor	driver is
select	count(1)
from	ctxsys.dr$pending
where	exists
(	select	1
	from	abstracts
	where	rowid = pnd_rowid);

dummy	integer;

begin

   open driver;
   fetch driver into dummy;
   close driver;

   if dummy > 0 then
      common.rebuild_CTX ('ABSTRACT_CTX');
   end if;

end check_CTX_index;

-- ******************************************************************************************

function get_user_sort_name (prof_id integer, name_flag integer default 1) return varchar2 is

cursor	driver is
select	*
from	persons
where	person_id	=
(	select	person_id
	from	profiles
	where	profile_id = prof_id);

driver_rec	driver%ROWTYPE;

begin

   open driver;
   fetch driver into driver_rec;
   close driver;

   if name_flag = 1 then
      return driver_rec.sort_lname;
   end if;

   if name_flag = 2 then
      return driver_rec.sort_fname;
   end if;

   if name_flag = 3 then
      return driver_rec.sort_mname;
   end if;

   return null;

end get_user_sort_name;

-- ******************************************************************************************

function get_session_sort_string (abs_id integer) return varchar2 is

cursor	driver is
select	to_char (nvl (s.sort_order, 0), '0000000000')	||
	to_char (s.session_date, 'yyyymmdd')		||
	to_char (s.beg_time, '0000')			||
	to_char (sa.session_seq, '00000')		sort_order
from	sessions		s,
	abstract_sessions	sa
where	sa.abstract_id		= abs_id
and	s.session_id		= sa.session_id
order by primary_flag desc,
	 s.sort_order nulls last,
	 s.session_date, s.beg_time, sa.session_seq;

driver_rec	driver%ROWTYPE;

begin

   open driver;
   fetch driver into driver_rec;
   close driver;

   return driver_rec.sort_order;

end get_session_sort_string;

-- ******************************************************************************************

procedure fmt_sort (this_search_sort common.ident_arr) is

sep_char	varchar2(2) := null;

begin

   for i in 1..this_search_sort.count LOOP
      if this_search_sort(i) is not null then

	 if this_search_sort(i) = sort_AUTH$ then
	    sql_stmt := sql_stmt || sep_char || '13, 14, 15 ';

	 elsif this_search_sort(i) = sort_MC$ then
	    sql_stmt := sql_stmt || sep_char || '11 ';

	 elsif this_search_sort(i) = sort_SC$ then
	    sql_stmt := sql_stmt || sep_char || '12 ';

	 elsif this_search_sort(i) = sort_AID$ then
	    sql_stmt := sql_stmt || sep_char || '1 ';

	 elsif this_search_sort(i) = sort_PID$ then
	    sql_stmt := sql_stmt || sep_char || 'search.get_session_sort_string (abstract_id) ';

	 elsif this_search_sort(i) = sort_PAFF$ then
	    sql_stmt := sql_stmt || sep_char ||
		'common.get_profile_affil (common.get_author_profile_id (a.abstract_id, ' ||
			Q$ || common.speaker$ || Q$ || ')) ';

	 elsif this_search_sort(i) = sort_poster$ then
	    sql_stmt := sql_stmt || sep_char || 'paper_ids.gen_sort_order (abstract_id, 0) ';

	 end if;

	 sep_char := ', ';
      end if;
   end loop;

   if sep_char is null then
      sql_stmt := sql_stmt || '2, 1 '; -- no sort order specified so sort the paper_id, abstract_id
   end if;

end fmt_sort;

-- ******************************************************************************************

function build_sql_stmt (rec search_type) return varchar2 is

search_name$	constant varchar2(100)	:= replace (rec.this_name,Q$,Q2$);
search_text$	constant varchar2(100)	:= replace(rec.this_text,Q$,Q2$);

buffer		varchar2(6)		:= 'where ';
pcode		integer;
attrib2		common.ident_arr;
test		varchar2(20);
attrib_in_list	varchar2(1000);

begin

   sql_stmt := 'select abstract_id, common.get_paper_id (abstract_id) paper_id, title, description, footnote, '											||
		'agency, status_code, referee_code, '														||
		'common.get_author_profile_id (a.abstract_id, ' || Q$ || common.speaker$	|| Q$ || '), '							||
		'common.decode_value (''presentation_descr'', ''presentation_codes'', ''presentation_code'', presentation_code) presentation_code, '		||
		'common.decode_value (''classification_descr'', ''classification_codes'', ''classification_code'', classification_code) classification_code, '	||
		'common.decode_value (''sub_class_descr'', ''sub_class_codes'', ''sub_class_code'', sub_class_code) sub_class_code, '				||
		'search.get_user_sort_name (common.get_author_profile_id (a.abstract_id, ' || Q$ || common.primary_author$ || Q$ || '), 1) sort_lname, '	||
		'search.get_user_sort_name (common.get_author_profile_id (a.abstract_id, ' || Q$ || common.primary_author$ || Q$ || '), 2) sort_fname, '	||
		'search.get_user_sort_name (common.get_author_profile_id (a.abstract_id, ' || Q$ || common.primary_author$ || Q$ || '), 3) sort_mname, '	||
		'person_id, editor_id, classification_code from abstracts a ';
   buffer := 'where ';

-- **********

   if rec.AID(1) is not null OR rec.AID(2) is not null then
      if common.validate_number (rec.AID(1)) AND common.validate_number (rec.AID(2)) then
	 sql_stmt := sql_stmt || buffer || 'abstract_id between ' ||
				nvl (rec.AID(1),rec.AID(2)) || ' and ' || nvl (rec.AID(2),rec.AID(1)) || ' ';
	 buffer := 'and ';
      else
	 return htf.header (3, 'Contribution ID is not numeric.');
      end if;
   end if;

-- **********

   if rec.PID is not null then
      sql_stmt := sql_stmt || buffer || 'abstract_id in '		||
				'(select abstract_id '			||
				'from	abstract_sessions '		||
				'where	paper_id in (' ||
			Q$ || upper (rec.PID) || Q$ || ', ' || Q$ || rec.PID || Q$ || ') ) ';
      buffer := 'and ';
   end if;

-- **********

   if rec.this_text is not null then
      if rec.exact.count = 2 then -- exact text match
	 sql_stmt := sql_stmt || buffer || 'title = ' || Q$ || rec.this_text || Q$ || ' ';
      else
	 check_CTX_index;
	 sql_stmt := sql_stmt || buffer || 'contains (search_text, ' || Q$ ||
		common.fmt_search_string (search_text$, ' and ') || Q$ || ', 1) > 0 ';
      end if;

      buffer := 'and ';
   end if;

-- **********

   if rec.this_name is not null then
      sql_stmt := sql_stmt || buffer || 'abstract_id in '		||
				'(select abstract_id '			||
				'from	abstract_authors '		||
				'where	sort_lname = common.sort_text ('||
					Q$ || search_name$ || Q$ || ') ) ';
      buffer := 'and ';
   end if;

-- **********

   if rec.this_classification is not null then
      sql_stmt := sql_stmt || buffer || 'classification_code = ' || Q$ || rec.this_classification || Q$ || ' ';
      buffer := 'and ';
   end if;

-- **********

   if rec.this_sub_class is not null then
      sql_stmt := sql_stmt || buffer || 'sub_class_code = ' || Q$ || rec.this_sub_class || Q$ || ' ';
      buffer := 'and ';
   end if;

-- **********

   if rec.this_session is not null then
      sql_stmt := sql_stmt || buffer || 'abstract_id in '	||
				'(select abstract_id '		||
				'from	abstract_sessions '	||
				'where	session_id = '		||
					Q$ || rec.this_session || Q$ || ') ';
      buffer := 'and ';
   end if;

-- **********

   if rec.this_presentation is not null then
      sql_stmt := sql_stmt || buffer || 'presentation_code = ' || Q$ || rec.this_presentation || Q$ || ' ';
      buffer := 'and ';
   end if;

-- **********

   if rec.this_presentation_type is not null then
      sql_stmt := sql_stmt || buffer || 'presentation_code in '			||
		'(select presentation_code from presentation_codes '	||
		'where presentation_type_code = ' || Q$ || rec.this_presentation_type || Q$ || ') ';
      buffer := 'and ';
   end if;

-- **********

   if rec.this_affil is not null then
      sql_stmt := sql_stmt || buffer || 'abstract_id in '					||
		'(select abstract_id from abstract_authors where profile_id in '		||
		'(select /*+ INDEX(p) */ profile_id from profiles p where address_id in '	||
		'(select /*+ INDEX(addr) */ address_id '					||
			'from addresses addr, affiliations affil '				||
		'where addr.affiliation_id = affil.affiliation_id '				||
		'and   contains (affil.search_text, ' || Q$					||
			replace (common.fmt_search_string (rec.this_affil), Q$, Q2$) || Q$ || ', 1) > 0 ) ) ';

      if rec.xcoauth.count = 2 then
	 sql_stmt := sql_stmt || 'and ctype <> ''Co-Author'') ';
      else
	 sql_stmt := sql_stmt || ') ';
      end if;

      buffer := 'and ';
   end if;

-- **********

   if rec.this_status is not null then
      if rec.this_status = common.search_for_null$ then
	 sql_stmt := sql_stmt || buffer || 'status_code is null ';
      else
         sql_stmt := sql_stmt || buffer || 'status_code = ' || Q$ || rec.this_status || Q$ || ' ';
      end if;
      buffer := 'and ';
   end if;

-- **********

   if rec.this_referee is not null then
      if rec.this_referee = common.search_for_null$ then
	 sql_stmt := sql_stmt || buffer || 'referee_code is null ';
      else
         sql_stmt := sql_stmt || buffer || 'referee_code = ' || Q$ || rec.this_referee || Q$ || ' ';
      end if;
      buffer := 'and ';
   end if;

-- **********

   if rec.QA is not null then
      sql_stmt := sql_stmt || buffer || 'search.requires_QA (abstract_id) = ' || Q$ || rec.QA || Q$ || ' ';
      buffer := 'and ';
   end if;

-- **********

   if rec.QAED is not null then
      sql_stmt := sql_stmt || buffer || 'common.final_QA (abstract_id) = ' || Q$ || rec.QAED || Q$ || ' ';
      buffer := 'and ';
   end if;

-- **********

   if rec.posted is not null then
     test := Q$ || rec.posted || Q$;

     sql_stmt := sql_stmt || buffer || 'exists '		||
		'(select	1 '				||
			'from	poster_logs l '			||
			'where	l.abstract_id = a.abstract_id '	||
			'and	l.posted = ' || test || ') ';
      buffer := 'and ';
   end if;

-- **********

   if rec.manned is not null then
     test := Q$ || rec.manned || Q$;

     sql_stmt := sql_stmt || buffer || 'exists '		||
		'(select	1 '				||
			'from	poster_logs l '			||
			'where	l.abstract_id = a.abstract_id '	||
			'and	l.manned = ' || test || ') ';
      buffer := 'and ';
   end if;

-- **********

   if rec.qualok is not null then
     test := Q$ || rec.qualok || Q$;

     sql_stmt := sql_stmt || buffer || 'exists '		||
		'(select	1 '				||
			'from	poster_logs l '			||
			'where	l.abstract_id = a.abstract_id '	||
			'and	l.satisfactory = ' || test || ') ';
      buffer := 'and ';
   end if;

-- **********

   if rec.accept is not null then
     test := Q$ || upper (rec.accept) || Q$;

     sql_stmt := sql_stmt || buffer || 'exists '		||
		'(select	1 '				||
			'from	dual '				||
			'where	common.paper_OK_to_publish '	||
				'(a.abstract_id) = ' || test	|| ') ';
      buffer := 'and ';
   end if;

-- **********

   if rec.files is not null then
      if rec.files = 'Y' then
	 test := '> 0 ';
      else
	 test := '= 0 ';
      end if;

      sql_stmt := sql_stmt || buffer || 'nvl (assignable,0) ' || test || ' ';
      buffer := 'and ';
   end if;

-- **********

   if rec.this_platform is not null then
      sql_stmt := sql_stmt || buffer || 'exists '	||
		'(select	1 '			||
		'from	dual '				||
		'where	common.get_abs_platform '	||
				'(a.abstract_id) = '	||
			Q$ || rec.this_platform || Q$	|| ') ';
      buffer := 'and ';
   end if;

-- **********

   if rec.PDF is not null then
      sql_stmt := sql_stmt || buffer				||
		'common.file_type_uploaded (a.abstract_id, '	||
			Q$ || common.PDF_file$		|| Q$	|| ') = '	||
			Q$ || common.yes_no (rec.PDF)	|| Q$	|| ' ';
      buffer := 'and ';
   end if;

-- **********

   if rec.SLIDES is not null then
      sql_stmt := sql_stmt || buffer					||
		'common.file_type_uploaded (a.abstract_id, '		||
			Q$ || common.transp_file$	|| Q$ || ') = '	||
			Q$ || common.yes_no (rec.SLIDES)	|| Q$ || ' ';
      buffer := 'and ';
   end if;

-- **********

   if rec.NEW_AFFILS = 'Y' then
      sql_stmt := sql_stmt || buffer || 'abstract_id in '				||
		'(select abstract_id from abstract_authors where profile_id in '	||
		'(select profile_id from profiles where address_id = 0 ))';
      buffer := 'and ';
   end if;

-- **********

   if rec.CRF is not null then
     if rec.CRF = 'Y' then
	test := 'exists ';
     else
	test := 'not exists ';
     end if;

     sql_stmt := sql_stmt || buffer || test			||
		'(select	1 '				||
			'from	logs l '			||
			'where	l.abstract_id = a.abstract_id '	||
			'and	log_code = ' || Q$		||
				common.copyright_received$	|| Q$ || ') ';
      buffer := 'and ';
   end if;

-- **********

   if rec.attrib.count > 1 then
      for i in 2..rec.attrib.COUNT LOOP
	 attrib2(i-1) := rec.attrib(i);
      end loop;

      attrib_in_list := ' and ' || common.build_IN_list (attrib2, 'attribute_code');

      sql_stmt := sql_stmt || buffer							||
			'a.abstract_id in ('						||
				'select abstract_id '					||
				'from abs_attributes '					||
				'where 1 = 1 ' || attrib_in_list 			||
			'UNION '							||
				'select /*+ INDEX (auth) */ abstract_id '		||
				'from	authors auth '					||
				'where	profile_id in ' 				||
					'(select profile_id'				||
					' from profile_attributes'			||
					' where 1 = 1 ' || attrib_in_list || ') '	||
			') ';
      buffer := 'and ';
   end if;

-- **********

   if rec.this_priority is not null AND rec.this_proposal_status is not null then
      pcode := nvl (rec.this_proposal_status, -999);

      sql_stmt := sql_stmt || buffer || 
			'abstract_id in '		||
			'(select distinct abstract_id'	||
			' from	abstract_proposals'	||
			' where nvl(priority_code,''~'')	LIKE nvl(' || Q$ || rec.this_priority || Q$ || ',''%'')' ||
			' and	nvl(proposal_status_code,-999)	= ' || pcode ||
			') ';

   elsif rec.this_priority is not null then
      sql_stmt := sql_stmt || buffer || 
			'abstract_id in '		||
			'(select distinct abstract_id'	||
			' from	abstract_proposals'	||
			' where priority_code = ' || Q$ || rec.this_priority || Q$ ||
			') ';

   elsif rec.this_proposal_status is not null then
      pcode := nvl (rec.this_proposal_status, -999);

      sql_stmt := sql_stmt || buffer || 
			'abstract_id in '		 ||
			'(select distinct abstract_id'	 ||
			' from	abstract_proposals'	 ||
			' where proposal_status_code = ' || pcode ||
			') ';

   end if;


-- **********

   if rec.sdte is not null then
      sql_stmt := sql_stmt || buffer || 'abstract_id in ' ||
		'(	 select sa.abstract_id '	||
			'from abstract_sessions sa, sessions s where s.session_date = ' ||
		'to_date(' || Q$ || rec.sdte || Q$ || ', ' || Q$ || common.date_input_fmt_str$ || Q$ || ') ' ||
		'and sa.session_id = s.session_id) ';
      buffer := 'and ';
   end if;

-- **********

   sql_stmt := sql_stmt || 'order by ';
   fmt_sort (rec.this_search_sort);

   return null;

end build_sql_stmt;

-- ******************************************************************************************
-- ******************************************************************************************

function get_priority_recs return priority_recs_type is

cursor	driver is
select	*
from	priority_codes
order by priority_descr;

recs	priority_recs_type;

begin

   open driver;
   fetch driver BULK COLLECT into recs;
   close driver;

   return recs;

end get_priority_recs;

-- ******************************************************************************************

function get_proposal_recs return proposal_recs_type is

cursor	driver is
select	*
from	proposal_status_codes
order by proposal_status_code;

recs	proposal_recs_type;

begin

   open driver;
   fetch driver BULK COLLECT into recs;
   close driver;

   return recs;

end get_proposal_recs;

-- ******************************************************************************************

function get_referee_name (abs_id integer) return varchar2 is

cursor	driver is
select	distinct
	p.sort_lname, p.sort_fname, p.sort_mname, p.lname
from	persons			p,
	referee_abstracts	x
where	x.abstract_id		= abs_id
and	p.person_id		= x.person_id
order by
	p.sort_lname, p.sort_fname, p.sort_mname;

buffer	long		:= null;

begin

   for driver_rec in driver LOOP
      buffer := buffer || htf.br || driver_rec.lname;
   end loop;

   return substr (buffer, length(htf.br)+1 );

end get_referee_name;

-- ******************************************************************************************

procedure show_admin_info (abs_rec abs_record) is

cursor	poster is
select	*
from	poster_logs
where	abstract_id = abs_rec.abstract_id;

poster_rec	poster_logs%ROWTYPE;
editor_lname	persons.lname%TYPE	:= null;

begin

   open poster;
   fetch poster into poster_rec;
   close poster;

   if abs_rec.status_code is null then
      editor_lname := common.decode_value ('lname', 'persons', 'person_id', abs_rec.editor_id);
   end if;

   htp.tableOpen ('BORDER=0');

   common.show_data_row ('Requires QA',	common.yes_no (search.requires_QA (abs_rec.abstract_id) ) );
   common.show_data_row ('Editor',	common.get_dot_image (abs_rec.status_code, common.editor_dot$) || editor_lname);

   if common.referee_conf$ = 'Y' then
      common.show_data_row ('Referee',	nvl (common.get_dot_image (abs_rec.referee_code, common.referee_dot$),
						get_referee_name (abs_rec.abstract_id) ) );
   end if;

   if common.use_poster_police$ = 'Y' then
      if common.get_presentation_type (abs_rec.abstract_id) = common.poster$ then
	 common.show_data_row ('Posted',	  common.yes_no (poster_rec.posted) );
	 common.show_data_row ('Manned',	  common.yes_no (poster_rec.manned) );
	 common.show_data_row ('Quality OK', common.yes_no (poster_rec.satisfactory) );
      end if;
   end if;

   common.show_data_row ('Final QA',	common.final_QA (abs_rec.abstract_id) );
   common.show_data_row ('Publishable',	common.paper_OK_to_publish (abs_rec.abstract_id) );
   common.show_data_row ('PDF',		common.file_type_uploaded (abs_rec.abstract_id, common.PDF_file$) );
   common.show_data_row ('Slides',	common.file_type_uploaded (abs_rec.abstract_id, common.transp_file$) );

   htp.tableClose;

end show_admin_info;

-- ******************************************************************************************

function get_users_priority_code (abs_id integer) return varchar2 is

cursor	driver is
select	priority_code
from	abstract_proposals
where	abstract_id		= abs_id
and	person_id		= nvl (common.gbl_priv_pid, common.gbl_pid)
and	proposal_status_code	is null;

code	proposal_status_codes.proposal_status_code%TYPE;

begin

   open driver;
   fetch driver into code;
   close driver;

   return code;

end get_users_priority_code;

-- ******************************************************************************************

function get_accepted_priority_code (abs_id integer) return varchar2 is

cursor	driver is
select	priority_code
from	abstract_proposals
where	abstract_id		= abs_id
and	proposal_status_code	is NOT null;

code	proposal_status_codes.proposal_status_code%TYPE;

begin

   open driver;
   fetch driver into code;
   close driver;

   return code;

end get_accepted_priority_code;

-- ******************************************************************************************

procedure show_proposals (abs_id integer, class_code varchar2, irows in out integer) is

cursor	driver (code1$ varchar2) is
select	*
from	abstract_proposals
where	abstract_id		= abs_id
and	priority_code		= code1$
order by
	proposal_status_code	nulls last;

buffer			long;
priority_recs		priority_recs_type := get_priority_recs;
proposal_recs		proposal_recs_type := get_proposal_recs;
users_priority		priority_codes.priority_code%TYPE;
accepted_priority	priority_codes.priority_code%TYPE;
buffer1			varchar2(20);
buffer2			varchar2(21);

begin

   users_priority	:= get_users_priority_code (abs_id);
   accepted_priority	:= get_accepted_priority_code (abs_id);

   for i in 1..priority_recs.COUNT loop
      htp.tableRowOpen ('LEFT', 'TOP');
      htp.tableHeader (priority_recs(i).priority_descr, 'RIGHT', cnowrap=>'NOWRAP');
      htp.tableData (common.space2$);
--
      buffer := null;
      if accepted_priority is null then

         if users_priority is null or users_priority <> priority_recs(i).priority_code then
            buffer := buffer || htf.anchor ('javascript:popUp(''prog_cmtte.set_priority'||
					'?aid='	 || abs_id				||
					'&code=' || priority_recs(i).priority_code	||
				 ''')', 'Set as my preference') || common.space2$;
         end if;

         if users_priority = priority_recs(i).priority_code then
            buffer := buffer || htf.anchor ('javascript:popUp(''prog_cmtte.set_priority'||
					'?aid='	 || abs_id				||
					'&code='					||
				 ''')', 'Clear my preference') || common.space2$;
         end if;

      else
         buffer := buffer || common.space$ || 'No longer accepting preferences.' || common.space$;
      end if;

      if prog_cmtte.is_coordinator (class_code) then
         if accepted_priority is null or accepted_priority <> priority_recs(i).priority_code then
            buffer := buffer || htf.anchor ('javascript:popUp(''prog_cmtte.set_proposal'||
					'?aid='	 || abs_id				||
					'&flag=' || prog_cmtte.proposal_accepted$	||
					'&code=' || priority_recs(i).priority_code	||
				 ''')', 'Accept')|| common.space2$;

         elsif accepted_priority = priority_recs(i).priority_code then
            buffer := buffer || htf.anchor ('javascript:popUp(''prog_cmtte.set_proposal'||
					'?aid='	 || abs_id				||
					'&flag=' 					||
					'&code=' || priority_recs(i).priority_code	||
				 ''')', 'Clear Accepted Priority');
         end if;

      end if;

      htp.tableData (buffer);
      htp.tableRowClose;
      irows := irows + 1;
--
      buffer	:= null;
      buffer1	:= htf.italic ('Accepted by: ');
      buffer2	:= htf.italic ('Preferred by: ');

      for rec in driver (priority_recs(i).priority_code) LOOP
         if rec.proposal_status_code is not null then
            buffer := buffer || buffer1;
         else
            buffer := buffer || buffer2;
         end if;

         buffer := buffer || htf.bold (common.get_user_name (rec.person_id)) || ' at ' ||
				to_char (rec.proposal_status_date, common.date_fmt_str$);

         buffer1 := ', ';
         buffer2 := ', ';
      end loop;

      if buffer is not null then
         htp.tableRowOpen ('LEFT', 'TOP');
         htp.tableData (common.space$);
         htp.tableData (common.space2$);
         htp.tableData (buffer);
         irows := irows + 1;
      end if;

   end loop;

end show_proposals;

-- ******************************************************************************************

procedure show_extra_fields (display common.ident_arr, driver_rec abs_record,
				this_text varchar2, this_name varchar2, this_affil varchar2) is

pid		integer;
irows		integer		:= 0;
show_primary	boolean		:= true;

admin_info$	boolean;
name		varchar2(500);
date_str	varchar2(30);
curr_status	integer := 4;
can_set_prop$	boolean;

begin

   if common.PC_mode$ = 'Y' then
      show_primary := false;
   end if;

   htp.tableOpen;

   for i in 2..display.count LOOP

      if i = 2 then

         can_set_prop$ := false;
	 if common.PC_mode$ = 'Y' then
            can_set_prop$	:= OK_set_proposal_page or prog_cmtte.is_coordinator (driver_rec.classification_code);

            if can_set_prop$ then
               show_proposals (driver_rec.abstract_id, driver_rec.classification_code, irows);
            end if;
	 end if;

	 if common.gbl_pid is not null and driver_rec.paper_id is not null then
	    if show_links and NOT already_scheduled (driver_rec.abstract_id) then
	       htp.tableRowOpen ('LEFT', 'TOP');
	       htp.tableHeader ('My Schedule', 'RIGHT', cnowrap=>'NOWRAP');
	       htp.tableData (common.space2$);
	       htp.tableData (add_URL (driver_rec.abstract_id));
	       htp.tableRowClose;
	       irows := irows + 1;
	    end if;
	 end if;

      end if;

-- **********

      if display(i) = display_CO$ then
	 if common.PC_mode$ = 'Y' then
	    htp.tableRowOpen ('LEFT', 'TOP');
	    htp.tableHeader ('Proposed By', 'RIGHT', cnowrap=>'NOWRAP');
	    htp.tableData (common.space2$);
	    htp.tableData (highlight (common.get_user_name (driver_rec.person_id), this_name) || ' (' ||
			highlight (common.get_user_affil_abbrev_list (driver_rec.person_id, 'Y'), this_affil) || ')');
	    htp.tableRowClose;
	    irows := irows + 1;
	 end if;

	 htp.tableRowOpen ('LEFT', 'TOP');
	 if common.get_presentation_type (driver_rec.abstract_id) = common.oral$ then
	    htp.tableHeader (jacow.set_label ('Preferred Speaker', 'Speaker'), 'RIGHT', cnowrap=>'NOWRAP');
	 else
	    htp.tableHeader ('Presenter', 'RIGHT', cnowrap=>'NOWRAP');
	 end if;

	 pid := common.decode_value ('person_id', 'profiles', 'profile_id', driver_rec.spk_profile_id);
	 htp.tableData (common.space2$);
         name := highlight (common.get_user_name (pid), this_name) || ' (' ||
		 highlight (common.get_affil_by_abs_person (driver_rec.abstract_id, pid), this_affil) || ')';
         name := replace (trim (name), '()', null);
	 htp.tableData (name);
	 htp.tableRowClose;

-- **********

	 htp.tableRowOpen ('LEFT', 'TOP');
	 htp.tableHeader (jacow.set_label ('Alternative Speakers', 'Authors'), 'RIGHT', cnowrap=>'NOWRAP');
	 htp.tableData (common.space2$);
	 htp.tableData (highlight (highlight (common.get_author_list2 (driver_rec.abstract_id, show_primary), this_name), this_affil) );
	 htp.tableRowClose;

	 if OK_editor_page and common.pc_mode$ = 'N' then
	    htp.tableRowOpen ('LEFT', 'TOP');
	    htp.tableHeader ('Registered', 'RIGHT', cnowrap=>'NOWRAP');
	    htp.tableData (common.space2$);
	    htp.tableData (regist_maint.person_list (driver_rec.abstract_id));
	    htp.tableRowClose;
	    irows := irows + 1;
	 end if;

-- **********

	 irows := irows + 2;

      elsif display(i) = display_AT$ then
	 htp.tableRowOpen ('LEFT', 'TOP');
	 htp.tableHeader ('Abstract', 'RIGHT', cnowrap=>'NOWRAP');
	 htp.tableData (common.space2$);
	 htp.tableData (highlight (common.convert_text (driver_rec.description), this_text));
	 htp.tableRowClose;

	 if driver_rec.footnote is not null then
	    htp.tableRowOpen ('LEFT', 'TOP');
	    htp.tableHeader ('Footnotes', 'RIGHT', cnowrap=>'NOWRAP');
	    htp.tableData (common.space2$);
	    htp.tableData (replace (driver_rec.footnote, chr(10), htf.br));
	    htp.tableRowClose;

	    irows := irows + 1;
	 end if;

	 irows := irows + 1;

      elsif display(i) = display_FA$ and common.PC_mode$ = 'N' then
	 htp.tableRowOpen ('LEFT', 'TOP');
	 htp.tableHeader ('Funding Agency', 'RIGHT', cnowrap=>'NOWRAP');
	 htp.tableData (common.space2$);
	 htp.tableData (driver_rec.agency);
	 htp.tableRowClose;

	 irows := irows + 1;

      elsif display(i) = display_PRET$ then
	 htp.tableRowOpen ('LEFT', 'TOP');
	 htp.tableHeader ('Type of Presentation', 'RIGHT', cnowrap=>'NOWRAP');
	 htp.tableData (common.space2$);
	 htp.tableData (driver_rec.presentation_descr);
	 htp.tableRowClose;

	 irows := irows + 1;

      elsif display(i) = display_SPK$ then
	 htp.tableRowOpen ('LEFT', 'TOP');
	 if common.get_presentation_type (driver_rec.abstract_id) = common.oral$ then
	    htp.tableHeader ('Speaker', 'RIGHT', cnowrap=>'NOWRAP');
	 else
	    htp.tableHeader ('Presenter', 'RIGHT', cnowrap=>'NOWRAP');
	 end if;

	 htp.tableData (common.space2$);
	 htp.tableData (common.get_user_name_affiliation (driver_rec.spk_profile_id));
	 htp.tableRowClose;

	 irows := irows + 1;

      elsif display(i) = display_MC$ then
	 htp.tableRowOpen ('LEFT', 'TOP');
	 htp.tableHeader ('Main Classification', 'RIGHT', cnowrap=>'NOWRAP');
	 htp.tableData (common.space2$);
	 htp.tableData (driver_rec.classification_descr);
	 htp.tableRowClose;

	 irows := irows + 1;

      elsif display(i) = display_SC$ then
         if common.require_sub_class$ = 'Y' or common.user_is_JACoW_admin then
            htp.tableRowOpen ('LEFT', 'TOP');
            htp.tableHeader ('Sub Classification', 'RIGHT', cnowrap=>'NOWRAP');
            htp.tableData (common.space2$);
            htp.tableData (driver_rec.sub_class_descr);
            htp.tableRowClose;

            irows := irows + 1;
         end if;
      end if;

   end loop;

   htp.tableClose;

end show_extra_fields;

-- ******************************************************************************************

procedure show_admin_links (abs_id integer, paper_id varchar2) is

pid	integer;
IDs	common.ident_arr := common.get_paper_ID_list (abs_id);

begin

   if OK_edit_abstract_editor then
      htp.anchor2 ('JACoW.edit_abstract_editor?abs_id=' || abs_id,	'Edit', ctarget=>'_blank');
      htp.p (common.space2$);
   elsif common.OK_to_edit_abstract (abs_id) then
      if common.PC_mode$ = 'Y' AND common.PC_edit$ = 'Y' then
         htp.anchor2 ('prog_cmtte.edit_abs?abs_id=' || abs_id,		'Edit', ctarget=>'_blank');
         htp.p (common.space2$);
      end if;
   end if;

   if OK_JACoW_withdraw_abstract then
      htp.anchor2 ('JACoW.withdraw_abstract?abs_id=' || abs_id,		'Withdraw', ctarget=>'_blank');
      htp.p (common.space2$);
   end if;

   if OK_JACoW_change_owner then
      htp.anchor2 ('JACoW.change_owner?abs_id=' || abs_id,		'Owner', ctarget=>'_blank');
      htp.p (common.space2$);
   end if;

   if common.PC_mode$ = 'N' then
      if OK_JACoW_set_editor then
	 htp.anchor2 ('JACoW.set_editor?abs_id=' || abs_id,		'Editor', ctarget=>'_blank');
         htp.p (common.space2$);
      end if;

      if OK_search_abs_QA then
         htp.anchor2 ('search.abs_QA?abs_id=' || abs_id, 'QA', ctarget=>'_blank');
         htp.p (common.space2$);
      end if;

      if common.copyright_required$ = 'Y' then
         if OK_JACoW_copyright then
            htp.anchor2 ('JACoW.copyright?this_id=' || abs_id,		'Copyright', ctarget=>'_blank');
            htp.p (common.space2$);
         end if;
      end if;

      if common.referee_conf$ = 'Y' then
         if OK_referee_edit then
            htp.anchor ('referee.edit?abs_id=' || abs_id, 'Referee');
            htp.p (common.space2$);
         end if;
      end if;

      if OK_JACoW_file_info and
	 common.editor_download_script$ is not null then
         if IDs.COUNT = 1 then
            htp.p (common.paper_anchor_tag (IDs(1)));
            htp.p (common.space2$);
         else
            htp.prn (htf.bold('Download ('));
            for i in 1..IDs.COUNT loop
               htp.prn (common.paper_anchor_tag (IDs(i), IDs(i)));
               if i <> IDs.COUNT then
                  htp.prn (common.space2$);
               end if;
            end loop;
            htp.bold (')');
            htp.p (common.space2$);
         end if;
      end if;

      if OK_abstract_maint_edit then
	 htp.anchor2 ('abstract_maint.edit?ID=' || abs_id, 'Sort/File/Edit', ctarget=>'_blank');
         htp.p (common.space2$);
      end if;

      if OK_editor_qa_paper then
	 htp.anchor2 ('editor.QA_paper?abs_id=' || abs_id, 'Final QA', ctarget=>'_blank');
         htp.p (common.space2$);
      end if;
   end if;

   if OK_JACoW_print_abstract then
      htp.anchor2 ('JACoW.print_abstract?abs_id=' || abs_id,	'Print', ctarget=>'_blank');
      htp.p (common.space2$);
   end if;

   if OK_JACoW_abs_log then
      htp.anchor2 ('JACoW.abs_log?abs_id=' || abs_id,		'Log',	 ctarget=>'_blank');
      htp.p (common.space2$);
   end if;

   if OK_JACoW_file_info then
      if common.OK_to_upload_files (abs_id) then
         if IDs.COUNT = 1 then
            htp.anchor2 ('JACoW.file_info?abs_id=' || abs_id ||
				'&paper_id=' || IDs(1),'Upload', ctarget=>'_blank');
            htp.p (common.space2$);
         else
            htp.prn (htf.bold('Upload ('));
            for i in 1..IDs.COUNT loop
               htp.prn (htf.anchor2 ('JACoW.file_info?abs_id=' || abs_id ||
				'&paper_id=' || IDs(i), IDs(i), ctarget=>'_blank'));
               if i <> IDs.COUNT then
                  htp.prn (common.space2$);
               end if;
            end loop;
            htp.bold (')');
            htp.p (common.space2$);
         end if;
      end if;
   end if;

   if common.OK_to_edit_abstract (abs_id) then
      htp.anchor2 ('JACoW.abs_authors?abs_id=' || abs_id, 'Authors', ctarget=>'_blank');
      htp.p (common.space2$);
   end if;

   if OK_email_one_email then
      htp.anchor2 ('email.one_email?abs_id=' || abs_id, 'Email', ctarget=>'_blank');
      htp.p (common.space2$);
   end if;

   if OK_profile_find_repo3 then
      pid := common.get_author_profile_id (abs_id, common.primary_author$);
      pid := common.decode_value ('person_id', 'profiles', 'profile_id', pid);
      htp.anchor2 ('profile.find_repo3?pid=' || pid, 'Profile', ctarget=>'_blank');
      htp.p (common.space2$);
   end if;

   if OK_jacow_additional_sessions then
      htp.anchor2 ('jacow.additional_sessions?aid=' || abs_id, 'Sessions', ctarget=>'_blank');
      htp.p (common.space2$);
   end if;

end show_admin_links;

-- ******************************************************************************************

procedure results_summary (display common.ident_arr, this_text varchar2,
				this_name varchar2, this_affil varchar2,
				new_search_URL varchar2 default 'search.html') is

paper_id		varchar2(4000);
spacer			integer := 3;
page_break		varchar2(200) := null;

begin

   OK_edit_abstract_editor		:= common.has_privs_to_page ('JACoW.edit_abstract_editor', false);
   OK_abstract_maint_edit		:= common.has_privs_to_page ('abstract_maint.edit', false);
   OK_editor_page			:= common.has_privs_to_page (common.editor_page$, false);
   OK_editor_qa_paper			:= common.has_privs_to_page ('editor.qa_paper', false);
   OK_email_one_email			:= common.has_privs_to_page ('email.one_email', false);
   OK_JACoW_abs_log			:= common.has_privs_to_page ('JACoW.abs_log', false);
   OK_jacow_additional_sessions		:= common.has_privs_to_page ('jacow.additional_sessions', false);
   OK_JACoW_change_owner		:= common.has_privs_to_page ('JACoW.change_owner', false);
   OK_JACoW_copyright			:= common.has_privs_to_page ('JACoW.copyright', false);
   OK_JACoW_edit_abstract_editor	:= common.has_privs_to_page ('JACoW.edit_abstract_editor', false);
   OK_JACoW_file_info			:= common.has_privs_to_page ('JACoW.file_info', false);
   OK_JACoW_print_abstract		:= common.has_privs_to_page ('JACoW.print_abstract', false);
   OK_JACoW_set_editor			:= common.has_privs_to_page ('JACoW.set_editor', false);
   OK_JACoW_withdraw_abstract		:= common.has_privs_to_page ('JACoW.withdraw_abstract', false);
   OK_profile_find_repo3		:= common.has_privs_to_page ('profile.find_repo3', false);
   OK_referee_edit			:= common.has_privs_to_page ('referee.edit', false);
   OK_search_abs_QA			:= common.has_privs_to_page ('search.abs_QA', false);
   OK_sort_file_page			:= common.has_privs_to_page (common.sort_file_page$, false);
   OK_set_proposal_page			:= common.has_privs_to_page (common.set_proposal_page$, false);

   htp.p (profile.js$);
   htp.tableOpen;

   for i in 1..abs_recs.COUNT LOOP
      htp.p (page_break);
      page_break := htf.tableRowOpen || htf.tableData(common.page_break$) || htf.tableRowClose;

      paper_id := null;
      if abs_recs(i).paper_id is not null then
	 if common.hide_program_codes$ = 'N' OR common.user_is_JACoW_admin OR
		sysdate > common.conf_start_monday$ + common.conf_len_days$ then
	    paper_id := ' - ' || common.get_paper_id (abs_recs(i).abstract_id, 'N', 'Y');
	 end if;
      end if;

      htp.tableRowOpen ('LEFT', 'TOP');
      htp.tableHeader ('ID: ' || abs_recs(i).abstract_id || paper_id ||
			common.space2$ || highlight (common.convert_text (abs_recs(i).title), this_text),
		ccolspan=>'2');
      htp.tableRowClose;

      if show_links and (OK_sort_file_page OR OK_editor_page OR common.PC_mode$ = 'Y' OR referee.is_a_referee) then
         htp.tableRowOpen ('LEFT', 'TOP');
         htp.p ('<TD COLSPAN="2">');
            show_admin_links (abs_recs(i).abstract_id, abs_recs(i).paper_id);
         htp.p ('</TD>');
         htp.tableRowClose;
      end if;

      htp.tableRowOpen ('LEFT', 'TOP');
      if show_admin_data and (OK_sort_file_page or OK_editor_page) then
          htp.p ('<TD ' || shade$ || '>');
            show_admin_info (abs_recs(i));
         htp.p ('</TD>');
      end if;

      htp.p ('<TD>');
         show_extra_fields (display, abs_recs(i), this_text, this_name, this_affil);
      htp.p ('</TD>');
      htp.tableRowClose;
      common.show_data_sep (spacer, common.space$);
   end loop;

   htp.tableClose;

   if show_header then
      if abs_recs.COUNT = 0 then
	 htp.header (3, 'No abstracts match your query.');
      elsif abs_recs.COUNT = 1 then
	 htp.header (3, '1 abstract matched your query.');
      else
	 htp.header (3, abs_recs.COUNT || ' abstracts matched your query.');
      end if;

      new_search (new_search_URL);
   end if;

end results_summary;

-- ******************************************************************************************

procedure load_abstracts is

type	cursor_type is REF CURSOR;
driver	cursor_type;

begin

   open driver for sql_stmt;
   fetch driver BULK COLLECT into abs_recs;
   close driver;

end load_abstracts;

-- ******************************************************************************************

procedure results2 (rec search_type, new_search_URL varchar2 default 'search.html') is

msg	varchar2(200);

begin

   msg := build_sql_stmt (rec);
   load_abstracts;

   if msg is null then
      if common.PC_mode$ = 'Y' then
	 common.js$;
      end if;

      if rec.exact.count = 2 then
	 results_summary (rec.display, null, rec.this_name, rec.this_affil, new_search_URL);
      else
	 results_summary (rec.display, rec.this_text, rec.this_name, rec.this_affil, new_search_URL);
      end if;
   else
      htp.p (msg);
   end if;

end results2;

-- ******************************************************************************************

function search_params_to_rec (name_array owa.vc_arr, value_array owa.vc_arr) return search_type is

rec	search_type;

begin

   for i in 1..name_array.COUNT loop

      if lower(name_array(i)) = 'aid' then
         rec.aid(rec.aid.COUNT+1) := trim(value_array(i));
      end if;

      if lower(name_array(i)) = 'pid' then
         rec.pid := trim(value_array(i));
      end if;

      if lower(name_array(i)) = 'this_text' then
         rec.this_text := trim(value_array(i));
      end if;

      if lower(name_array(i)) = 'exact' then
         rec.exact(rec.exact.COUNT+1) := trim(value_array(i));
      end if;

      if lower(name_array(i)) = 'this_name' then
         rec.this_name := trim(value_array(i));
      end if;

      if lower(name_array(i)) = 'this_affil' then
         rec.this_affil := trim(value_array(i));
      end if;

      if lower(name_array(i)) = 'xcoauth' then
         rec.xcoauth(rec.xcoauth.COUNT+1) := trim(value_array(i));
      end if;

      if lower(name_array(i)) = 'this_classification' then
         rec.this_classification := trim(value_array(i));
      end if;

      if lower(name_array(i)) = 'this_sub_class' then
         rec.this_sub_class := trim(value_array(i));
      end if;

      if lower(name_array(i)) = 'this_session' then
         rec.this_session := trim(value_array(i));
      end if;

      if lower(name_array(i)) = 'this_presentation' then
         rec.this_presentation := trim(value_array(i));
      end if;

      if lower(name_array(i)) = 'this_search_sort' then
         rec.this_search_sort(rec.this_search_sort.COUNT+1) := trim(value_array(i));
      end if;

      if lower(name_array(i)) = 'display' then
         rec.display(rec.display.COUNT+1) := trim(value_array(i));
      end if;

      if lower(name_array(i)) = 'this_status' then
         rec.this_status := trim(value_array(i));
      end if;

      if lower(name_array(i)) = 'this_referee' then
         rec.this_referee := trim(value_array(i));
      end if;

      if lower(name_array(i)) = 'qa' then
         rec.qa := trim(value_array(i));
      end if;

      if lower(name_array(i)) = 'qaed' then
         rec.qaed := trim(value_array(i));
      end if;

      if lower(name_array(i)) = 'posted' then
         rec.posted := trim(value_array(i));
      end if;

      if lower(name_array(i)) = 'manned' then
         rec.manned := trim(value_array(i));
      end if;

      if lower(name_array(i)) = 'qualok' then
         rec.qualok := trim(value_array(i));
      end if;

      if lower(name_array(i)) = 'accept' then
         rec.accept := trim(value_array(i));
      end if;

      if lower(name_array(i)) = 'files' then
         rec.files := trim(value_array(i));
      end if;

      if lower(name_array(i)) = 'this_platform' then
         rec.this_platform := trim(value_array(i));
      end if;

      if lower(name_array(i)) = 'pdf' then
         rec.pdf := trim(value_array(i));
      end if;

      if lower(name_array(i)) = 'slides' then
         rec.slides := trim(value_array(i));
      end if;

      if lower(name_array(i)) = 'new_affils' then
         rec.new_affils := trim(value_array(i));
      end if;

      if lower(name_array(i)) = 'crf' then
         rec.crf := trim(value_array(i));
      end if;

      if lower(name_array(i)) = 'attrib' then
         rec.attrib(rec.attrib.COUNT+1) := trim(value_array(i));
      end if;

      if lower(name_array(i)) = 'this_priority' then
         rec.this_priority := trim(value_array(i));
      end if;

      if lower(name_array(i)) = 'this_proposal_status' then
         rec.this_proposal_status := trim(value_array(i));
      end if;

      if lower(name_array(i)) = 'sdte' then
         rec.sdte := trim(value_array(i));
      end if;

      if lower(name_array(i)) = 'this_presentation_type' then
         rec.this_presentation_type := trim(value_array(i));
      end if;

   end loop;

   for i in 1..2 LOOP
      if NOT rec.aid.EXISTS(i) then
         rec.aid(i) := null;
      end if;
   end loop;

   return rec;

end search_params_to_rec;

-- ***********************************************************************************************

procedure results1 (rec search_type) is

msg	varchar2(200);
abs_id	integer;
dummy	varchar2(1);

begin

   common.get_gbl_pid;

   if common.has_privs_to_page (common.sort_file_page$, false) then
      if rec.PID is not null then
         abs_id := common.decode_value ('abstract_id', 'abstract_sessions', 'paper_id', upper(trim(rec.PID)));
      end if;
   end if;

   if abs_id is not null and NOT common.user_is_JACoW_admin then
      abstract_maint.edit (abs_id);
   else
      common.public_header (rpt_title$, print=>true);
      results2 (rec);
      common.footer;
   end if;

end results1;

-- ******************************************************************************************

procedure results (name_array owa.vc_arr, value_array owa.vc_arr) is

rec	search_type;

begin

   rec := search_params_to_rec (name_array, value_array);
   results1(rec);

end results;

-- ******************************************************************************************
-- ******************************************************************************************

procedure simple (aid integer, pcode varchar2) is

rec	search_type;

begin

   select search_display_code
   BULK COLLECT into rec.display
   from search_display_codes;

   rec.display(rec.display.count+1) := null;
   for i in reverse 2..rec.display.count LOOP
      rec.display(i) := rec.display(i-1);
   end loop;

   rec.aid(1) := aid;
   rec.aid(2) := aid;
   rec.this_presentation := pcode;

   if show_header then
      results1 (rec);
   else
      results2 (rec);
   end if;

end simple;

-- ******************************************************************************************

procedure pcode (pcode varchar2) is

begin

   show_header := true;
   simple (null, pcode);

end pcode;

-- ******************************************************************************************

procedure abs_id (aid integer) is

begin

   show_header := true;
   simple (aid, null);

end abs_id;

-- ******************************************************************************************

procedure abstract_id (aid integer) is

begin

   show_header := false;
   simple (aid, null);

end abstract_id;

-- ******************************************************************************************
-- ******************************************************************************************

function load_search_display_codes return common.ident_arr is

cursor	driver is
select	search_display_code
from	search_display_codes
order	by sort_order;

codes	common.ident_arr;

begin

   codes(1) := 'dummy';
   for driver_rec in driver LOOP
      codes(codes.COUNT+1) := driver_rec.search_display_code;
   end loop;

   return codes;

end load_search_display_codes;

-- ******************************************************************************************

procedure author_list (letter varchar2) is

cursor	driver is
select	a.abstract_id, common.get_paper_id (a.abstract_id) paper_id,
	a.title, a.description, a.footnote, a.agency, a.status_code, a.referee_code,
	common.get_author_profile_id (a.abstract_id, common.speaker$),
	common.decode_value ('presentation_descr', 'presentation_codes',
				'presentation_code', a.presentation_code),
	common.decode_value ('classification_descr', 'classification_codes',
				'classification_code', a.classification_code),
	common.decode_value ('sub_class_descr', 'sub_class_codes',
				'sub_class_code', a.sub_class_code),
	search.get_user_sort_name (common.get_author_profile_id (a.abstract_id, common.primary_author$), 1) sort_lname,
	search.get_user_sort_name (common.get_author_profile_id (a.abstract_id, common.primary_author$), 2) sort_fname,
	search.get_user_sort_name (common.get_author_profile_id (a.abstract_id, common.primary_author$), 3) sort_mname,
	a.person_id, a.editor_id, a.classification_code
from	abstract_authors	aa,
	abstract_sessions	sa,
	abstracts		a
where	aa.soundex_lname	like letter || '%'
and	aa.ctype		in ('Author', 'Co-Author', 'Proposed Author')
and	a.abstract_id		= aa.abstract_id
and	sa.abstract_id		= a.abstract_id
and	sa.primary_flag		is not null
order by
	nvl (sa.session_id || trim (to_char (sa.session_seq, '0000000000')),
	to_char (a.abstract_id, '999999'));

codes	common.ident_arr := load_search_display_codes;

begin

   if common.header (rpt_title$, print=>true) then
      open driver;
      fetch driver BULK COLLECT into abs_recs;
      close driver;

      results_summary (codes, null, null, null);
      common.footer;
   end if;

end author_list;

-- *********************************************************************************************************
-- *********************************************************************************************************

procedure show_session_times_header (this_id varchar2) is

cursor	driver is
select	presentation_type_code
from	presentation_option_codes
where	presentation_option_code =
(	select	presentation_option_code
	from	sessions
	where	session_id = this_id);

driver_rec	driver%ROWTYPE;
buffer		varchar2(20) := 'Presenter';

begin

   open driver;
   fetch driver into driver_rec;
   close driver;

   htp.tableRowOpen ('LEFT', 'BOTTOM');
   htp.tableHeader ('Session #', cnowrap=>'NOWRAP');

   if driver_rec.presentation_type_code = common.oral$ then
      htp.tableHeader ('Time');
      buffer := 'Speaker';
   end if;

   if common.gbl_pid is not null then
      htp.tableHeader ('Schedule');
   end if;

   htp.tableHeader (buffer);
   htp.tableHeader ('Abstract Title');
   htp.tableHeader ('Classification');
   htp.tableHeader ('Author(s)');
   htp.tableRowClose;

end show_session_times_header;

-- *********************************************************************************************************

procedure show_session_items (this_id varchar2) is

cursor	driver is
select	a.*, sa.paper_id, sa.duration, sa.start_time
from	abstracts		a,
	abstract_sessions	sa
where	sa.session_id		= this_id
and	a.abstract_id		= sa.abstract_id
order by sa.session_seq;

paper_URL	varchar2(200);
sched_URL	varchar2(200);
end_time	date;
end_str		varchar2(10);
buffer		varchar2(1000);

begin

   htp.tableOpen ('BORDER');
   show_session_times_header (this_id);

   for driver_rec in driver LOOP
      paper_URL := htf.anchor('JACoW.view_abstract?abs_id=' ||
			to_char(driver_rec.abstract_id), driver_rec.paper_id);
      buffer	:= common.decode_value ('classification_descr', 'classification_codes',
			'classification_code', driver_rec.classification_code);
      if driver_rec.sub_class_code is not null then
	 buffer := buffer || ' / ' || common.decode_value ('sub_class_descr',
			'sub_class_codes', 'sub_class_code', driver_rec.sub_class_code);
      end if;

      if common.gbl_pid is null then
	 null;
      elsif already_scheduled (driver_rec.abstract_id) then
	 sched_URL := 'Scheduled';
      else
	 sched_URL := add_URL (driver_rec.abstract_id);
      end if;

      htp.tableRowOpen ('LEFT', 'TOP');
      htp.tableData (paper_URL, 'RIGHT');

      if common.get_presentation_type (driver_rec.presentation_code) = common.oral$ then
	 end_str := null;
	 if driver_rec.duration is not null then
	    end_time := common.add_time (trunc (sysdate), driver_rec.start_time);
	    end_time := common.add_time (end_time, driver_rec.duration);
	    end_str  := ' - ' || to_char (end_time, 'hh24:mi');
	 end if;
	 htp.tableData (common.format_time (driver_rec.start_time) || end_str, cnowrap=>'NOWRAP' );
      end if;

      if common.gbl_pid is not null then
	 htp.tableData (sched_URL, 'CENTER');
      end if;

      htp.tableData (common.get_user_name_affiliation (common.get_author_profile_id (driver_rec.abstract_id, common.speaker$)));
      htp.tableData (driver_rec.title);
      htp.tableData (buffer);
      htp.tableData (common.get_author_list2 (driver_rec.abstract_id));
      htp.tableRowClose;
   end loop;

   htp.tableClose;

end show_session_items;

-- *********************************************************************************************************

procedure show_session_HDR (this_session varchar2) is

cursor	driver is
select	s.*,
	common.get_user_name (pro.person_id)		name,
	common.get_affil_name (addr.affiliation_id)	affiliation
from	addresses		addr,
	profiles		pro,
	sessions		s
where	s.session_id		= this_session
and	pro.profile_id(+)	= s.chair_profile_id
and	addr.address_id(+)	= pro.address_id;

driver_rec	driver%ROWTYPE;

begin

   open driver;
   fetch driver into driver_rec;
   close driver;

   htp.tableOpen;
   common.show_data_row ('Session',	driver_rec.session_id);
   common.show_data_row ('Title',	driver_rec.title);
   common.show_data_row ('Date',	to_char(driver_rec.session_date,'mm/dd/yyyy') || ' ' ||
					common.format_time(driver_rec.beg_time) || ' - ' ||
					common.format_time(driver_rec.end_time) );
   common.show_data_row ('Chair',	common.convert_text (driver_rec.name, common.html_fmt$) ||
					htf.br || htf.italic (driver_rec.affiliation) );
   common.show_data_row ('Type',	common.decode_value ('presentation_option_descr',
					'presentation_option_codes', 'presentation_option_code',
					driver_rec.presentation_option_code) );
   htp.tableClose;

end show_session_HDR;

-- *********************************************************************************************************

procedure show_session (this_session varchar2) is

begin

   if common.header ('[' || this_session || '] ' ||
		common.decode_value ('title', 'sessions', 'session_id', this_session)) then
      common.js$;
      show_session_HDR (this_session);
      htp.para;
      show_session_items (this_session);
      common.footer;
   end if;

end show_session;

-- ******************************************************************************************
-- ******************************************************************************************

procedure list is

cursor	driver is
select	*
from	sessions
order by
	sort_order, session_date, beg_time, end_time;

begin

   if common.header ('Program Guide') then
      htp.tableOpen ('BORDER');

      htp.tableRowOpen ('LEFT', 'BOTTOM');
      htp.tableHeader ('Session');
      htp.tableHeader ('Title');
      htp.tableHeader ('Day');
      htp.tableHeader ('Start');
      htp.tableHeader ('End');
      htp.tableHeader ('Location');
      htp.tableHeader ('Chair');
      htp.tableRowClose;

      for driver_rec in driver LOOP
         htp.tableRowOpen ('LEFT', 'TOP');
         htp.tableData (htf.anchor ('search.show_session?this_session=' ||
			driver_rec.session_id, driver_rec.session_id) );
         htp.tableData (driver_rec.title);
         htp.tableData (to_char (driver_rec.session_date, 'FMDay'));
         htp.tableData (common.format_time (driver_rec.beg_time) );
         htp.tableData (common.format_time (driver_rec.end_time) );
         htp.tableData (common.decode_value ('location_descr', 'location_codes', 'location_code', driver_rec.location_code));
         htp.tableData (common.get_user_name_affiliation (driver_rec.chair_profile_id));
         htp.tableRowClose;
      end loop;

      htp.tableClose;
      common.footer;
   end if;

end list;

-- ******************************************************************************************
-- ******************************************************************************************

procedure list2 is

rec	search_type;

begin

   select search_display_code
   BULK COLLECT into rec.display
   from search_display_codes;

   rec.display(rec.display.count+1) := null;
   for i in reverse 2..rec.display.count LOOP
      rec.display(i) := rec.display(i-1);
   end loop;

   rec.this_search_sort(1) := sort_PID$;
   rec.this_search_sort(2) := sort_PID$;
   rec.this_search_sort(3) := sort_PID$;

   results1 (rec);

end list2;

-- ******************************************************************************************
-- ******************************************************************************************

procedure session_search_form is

cursor	driver is
select	session_id, title
from	sessions		s
where	exists
(	select	1
	from	abstract_sessions	a
	where	a.session_id	= s.session_id)
order by
	sort_order, session_date, beg_time, session_id;

type	rec_type is table of driver%ROWTYPE;
recs	rec_type;

begin

   open driver;
   fetch driver BULK COLLECT into recs;
   close driver;

   if recs.COUNT > 0 then
      htp.formOpen ('search.show_session', 'POST');
      htp.tableOpen;
      htp.tableRowOpen ('TOP');
      htp.tableHeader ('Session', 'RIGHT');
      htp.p ('<TD>');
      htp.formSelectOpen ('this_session');
      htp.formSelectOption ('');

      for i in 1..recs.COUNT LOOP
         htp.formSelectOption (recs(i).session_id || ' - ' || recs(i).title,
			cattributes=>'VALUE="' || recs(i).session_id || '"');
      end loop;

   end if;

   if recs.COUNT > 0 then
      htp.formSelectClose;
      htp.p ('</TD>');
      htp.tableRowClose;
      htp.tableClose;

      htp.formSubmit;
      htp.formClose;
   end if;

end;

-- ******************************************************************************************

procedure session_search is

begin

   if common.header ('Session Search') then
      session_search_form;
      common.footer;
   end if;

end session_search;

-- ******************************************************************************************

procedure show_display_columns is

cursor	driver is
select	search_display_code, search_display_descr
from	search_display_codes
order by sort_order;

type	rec_type is table of driver%ROWTYPE index by binary_integer;
recs	rec_type;
checked	varchar2(7);

begin

   open driver;
   fetch driver BULK COLLECT into recs;
   close driver;

   htp.bold ('Show Abstract Title and...');
   htp.tableOpen;
   htp.tableRowOpen ('LEFT', 'TOP');

   htp.p ('<TD>');
   htp.formHidden ('display', 'dummy');

   for i in 1..recs.count LOOP
      checked := 'CHECKED';
      if lower (owa_util.get_cgi_env ('PATH_INFO')) <> '/search.html' then
         if recs(i).search_display_code in ('CO', 'AT', 'FA') then
            checked := null;
         end if;
      end if;

      htp.formCheckBox ('display', recs(i).search_display_code, checked);
      htp.p (recs(i).search_display_descr || htf.br);

      if i = round (recs.count/2) then
	 htp.p ('</TD>');
	 htp.p ('<TD>');
      end if;
   end loop;

   htp.p ('</TD>');
   htp.tableRowClose;
   htp.tableClose;

end show_display_columns;

-- ******************************************************************************************

procedure show_session_dates_popup is

cursor	driver is
select	distinct session_date
from	sessions
order by session_date;

begin

   htp.tableRowOpen ('LEFT', 'TOP');
   htp.tableHeader ('Session Date', 'RIGHT');
   htp.p ('<TD>');
      htp.formSelectOpen ('sdte');
      htp.formSelectOption ('', 'SELECTED');
      for driver_rec in driver LOOP
	 htp.formSelectOption (
		to_char (driver_rec.session_date, 'FMDay')	|| ', ' ||
		to_char (driver_rec.session_date, common.date_input_fmt_str$),
		cattributes=>'VALUE="' ||
		to_char (driver_rec.session_date, common.date_input_fmt_str$) || '"');
      end loop;
      htp.formSelectClose;
   htp.p ('</TD>');
   htp.tableRowClose;

end show_session_dates_popup;

-- ******************************************************************************************

procedure show_common_fields is

buffer		varchar2(15)		:= 'Author Name';
accept_iorals$	constant boolean	:= common.has_privs_to_page (common.iorals_page$, false);
show_sub_class	boolean			:= true;
where_stmt	varchar2(1000)		:= null;

begin

   htp.tableOpen;
   common.show_data_row ('Contribution ID',	htf.formText ('AID', 5, 10) || ' through ' ||
						htf.formText ('AID', 5, 10));

   if common.PC_mode$ = 'N' then
      common.show_data_row ('Paper ID',		htf.formText ('PID', 13, 13) || ' ID assigned by editor.');
   else
      buffer := 'Speaker Name';
   end if;

   if common.PC_mode$ = 'N' then
      show_session_dates_popup;
   end if;

   common.show_data_row ('Word in Title or Abstract',
						htf.formText ('this_text', 50, 100)	||
						htf.formHidden ('exact', '')		||
						htf.formCheckBox ('exact', 'Y')		||
						'Exact Title Match');
   common.show_data_row (buffer,		htf.formText ('this_name', 25, 100));
   common.show_lookup_table ('CLASSIFICATION', 'Main Classification',
				onChange=>'onChange="load_sub_class(this.value);"');

   if common.PC_mode$ = 'Y' AND common.require_sub_class$ = 'N' then
      show_sub_class := false;
   end if;

   if show_sub_class then
      jacow.show_sub_class_lookup ('Sub Classification', null, null);
   end if;

   if common.PC_mode$ = 'Y' then
      where_stmt := 'where propose_oral = ''Y''';
   end if;

   common.show_lookup_table ('PRESENTATION',	  'Type of Presentation', where_claws=>where_stmt);
   common.show_lookup_table ('PRESENTATION_TYPE', 'Presentation Group');

   if common.PC_mode$ = 'Y' then
      if common.has_privs_to_page (common.set_proposal_page$, false) then
         common.show_lookup_table ('priority', 'Priority');
      end if;

      if common.has_privs_to_page (common.set_proposal_page$, false) then
         common.show_lookup_table ('proposal_status', 'Proposal Status');
      end if;
   end if;

   common.show_data_row ('Affiliation',		htf.formText ('this_affil', 50, 200) || ' Any part of name. ' ||
						htf.formHidden ('xcoauth', 'dummy') || htf.br ||
						htf.formCheckBox ('xcoauth', 'Y') ||
						'Exclude Affiliation of Co-author' );
   htp.tableClose;

end show_common_fields;

-- ******************************************************************************************

procedure show_admin_fields is

begin

   htp.tableOpen;
   common.show_lookup_table ('status',	'Editor Status', search_nulls=>true);
   if common.referee_conf$ = 'Y' then
      common.show_lookup_table ('referee', 'Referee Status', search_nulls=>true);
   end if;

   common.show_data_row ('Requires Initial QA',	common.popup_yes_no ('QA', ''));
   common.show_data_row ('Final QA''ed',	htf.formSelectOpen ('QAED')		||
						htf.formSelectOption ('', 'SELECTED')	||
						htf.formSelectOption ('Passed')		||
						htf.formSelectOption ('Failed')		||
						htf.formSelectOption ('Pending')	||
						htf.formSelectClose);

   if common.use_poster_police$ = 'Y' then
      common.show_data_row ('Posted',		common.popup_yes_no ('POSTED', ''));
      common.show_data_row ('Manned',		common.popup_yes_no ('MANNED', ''));
      common.show_data_row ('Quality OK',	common.popup_yes_no ('QUALOK', ''));
      common.show_data_row ('Accepted',		common.popup_yes_no ('ACCEPT', ''));
   end if;

   common.show_data_row ('Source Files Uploaded',common.popup_yes_no ('FILES', ''));
   common.show_lookup_table ('platform',	'Source Platform');
   common.show_data_row ('PDF Uploaded',	common.popup_yes_no ('PDF', ''));
   common.show_data_row ('Slides Uploaded',	common.popup_yes_no ('SLIDES', ''));
   common.show_data_row ('New Affiliations',	common.popup_yes_no ('NEW_AFFILS', ''));
   common.show_data_row ('Copyright Form',	common.popup_yes_no ('CRF', ''));
   htp.tableClose;

end show_admin_fields;

-- ******************************************************************************************

procedure show_sort_fields is

type	vtype is varray(3) of varchar2(20);
labels	vtype := vtype ('First Sort', 'Second Sort', 'Third Sort');
sorts	vtype := vtype (sort_PID$, sort_MC$, sort_SC$);
wclaws	varchar2(50);

begin

   wclaws := 'where search_flag like ' || q$ || search_flag || q$;     

   htp.tableOpen;
   for i in 1..labels.COUNT loop
      common.show_lookup_table ('search_sort', labels(i), sorts(i), wclaws, show_null=>false);
   end loop;
   htp.tableClose;

end show_sort_fields;

-- ******************************************************************************************

procedure html_form (target varchar2) is

begin

   OK_editor_page := common.has_privs_to_page (common.editor_page$, false);
   htp.p (jacow.js$);
   htp.formOpen (target, 'POST', cattributes=>'NAME="SPMS"');
   htp.formHidden ('attrib', 'dummy');

   if common.referee_conf$ = 'N' OR not OK_editor_page then
      htp.formHidden ('this_referee', '');
   end if;

   if common.PC_mode$ = 'Y' then
      htp.formHidden ('PID',		'');
      htp.formHidden ('display',	'dummy');
      htp.formHidden ('display',	display_AT$);
      htp.formHidden ('display',	display_CO$);
      htp.formHidden ('display',	display_MC$);
      htp.formHidden ('display',	display_SC$);
      htp.formHidden ('display',	display_PRET$);
   end if;

   htp.bold ('Enter Search Criteria.');
   htp.p (common.space2$);
   htp.p ('Search matches all criteria specified below.');
   htp.p (common.space2$);
   htp.bold ('Note:');
   htp.p ('To view all abstracts submit a search request with no search criteria.');

   htp.tableOpen;

-- **********

   htp.tableRowOpen ('LEFT', 'TOP');

   htp.p ('<TD COLSPAN="2">');
      show_common_fields;
   htp.p ('</TD>');

   if OK_editor_page then
      htp.p ('<TD ROWSPAN="5">');
      show_admin_fields;
      htp.p ('</TD>');

      htp.p ('<TD ROWSPAN="5">');
	 htp.tableOpen;
	 jacow.abs_attrib_fields;
	 htp.tableClose;
      htp.p ('</TD>');

   else
      htp.tableData (common.space$, crowspan=>'5');
   end if;

   htp.tableRowClose;

-- **********

   htp.tableRowOpen ('LEFT', 'TOP');
   htp.p ('<TD ALIGN="RIGHT">');
      show_sort_fields;
   htp.p ('</TD>');

   if common.PC_mode$ = 'N' then
      htp.p ('<TD>');
	 show_display_columns;
      htp.p ('</TD>');
   end if;

   htp.tableRowClose;

-- **********

   htp.tableRowOpen ('CENTER', 'TOP');

   htp.p ('<TD COLSPAN="2">');
   htp.formSubmit (null, 'Search');
   htp.p (common.space2$);
   htp.p (common.space2$);
   htp.formReset;
   htp.p ('</TD>');
   htp.tableRowClose;
   htp.tableClose;

   htp.formClose;

   if common.PC_mode$ = 'Y' then
      common.form_focus ('SPMS', 'this_CLASSIFICATION');
   else
      common.form_focus ('SPMS', 'PID');
   end if;


-- **********

   htp.hr;
   htp.header (3, 'Search by Session');
   session_search_form;

end html_form;

-- ******************************************************************************************

procedure html is

continue	boolean;

begin

   common.public_header (rpt_title$);
   common.get_gbl_pid;

   continue := common.has_privs_to_page ('search.html', false);
   OK_editor_page := common.has_privs_to_page (common.editor_page$, false);

   if NOT continue then
      if common.PC_mode$ = 'Y' then
         continue := common.has_privs_to_page (common.set_proposal_page$, false)
			OR prog_cmtte.is_coordinator ('%');
      end if;
   end if;

   if continue then
      html_form('!search.results');
   else
      htp.header (3, 'The search feature is not available at this time.');
   end if;
   common.footer;

end html;

-- ******************************************************************************************

end search;

/
show error

set define on

--	Copyright (C) 2004-2008 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>
