set define off

create or replace package body stakeholder as

rpt_title$	constant char(23)	:= 'Stakeholder Maintenance';
date_fmt$	constant char(9)	:= 'dd-Mon-yy';
min_date$	constant date		:= to_date('01-01-1800', 'mm-dd-yyyy');
max_date$	constant date		:= to_date('01-01-3000', 'mm-dd-yyyy');

type	dte_type is record (
		dte	date,
		str	varchar2(20) );

type	form_rec_type is record (
		code		stakeholder_group_codes.stakeholder_group_code%TYPE,
		UID		persons.user_id%TYPE,
		affil_id	integer,
		affil_name	varchar2(4000),
		conf_series	conference_codes.conference_code%TYPE,
		conf_id		integer,
		starting	dte_type,
		ending		dte_type,
		inserting	char(1) default 'Y',
		msg		long);

form_rec	form_rec_type;

type	persons_type is table of persons%ROWTYPE index by pls_integer;

-- ***************************************************************************************************************
-- ***************************************************************************************************************

procedure js is

begin

   htp.prn (
'function newWindow (URL) {

   w = parseInt(screen.width*.5);
   h = parseInt(screen.height*.5);
   newWin = window.open (URL, "newWin' || common.time$ || '", "menubar=0,height=" + h + ",width=" + w + ",scrollbars=1,location=1,toolbar=1,status=1,resizable=1")
   newWin.focus()

} // end newWindow
');

   htp.print ('');

   htp.prn (
'function setValue (affil_id) {

   opener.document.SPMS.affil_id.value=affil_id;
   opener.document.SPMS.affil_name.value=affils[affil_id].name.replace (/<BR>/, "\n");
   opener.focus();
   window.close();

} // end setValue
');

   htp.print ('');

   htp.prn (
'function ImageConstruct (name, address) {

   this.name = name;
   this.address = address;

} // end ImaegConstruct
');

end js;

-- ***************************************************************************************************************

function get_conf_data (cid integer) return conf_conferences%ROWTYPE is

cursor	driver is
select	*
from	conf_conferences
where	conference_id = cid;

rec		driver%ROWTYPE;
rec_found	boolean;

begin

   open driver;
   fetch driver into rec;
   rec_found := driver%FOUND;
   close driver;

   if NOT rec_found then
      return null;
   end if;

   return rec;

end get_conf_data;

-- ***************************************************************************************************************
-- ***************************************************************************************************************

procedure XML is

begin

   XML2.header;
   XML2.print_xml_fom_sql (
'select	S.*, sc.stakeholder_descr, sgc.stakeholder_group_descr,
	common.get_user_name (s.person_id)		stakeholder_name,
	common.get_user_affil_abbrev_list(s.person_id)	stakeholder_affil,
	common.get_user_email(s.person_id)		stakeholder_email,
	common.get_affil_name (s.affiliation_id)	affiliation_name,
	common.decode_value (	''conference_descr'', ''conference_codes'',
				''conference_code'', s.conference_code) conference_name,
	cc.conference_title, cc.location conference_location,
	cc.start_date conference_start_date, cc.end_date conference_end_date
from	conf_conferences		cc,
	stakeholder_group_codes		sgc,
	stakeholder_codes		sc,
	stakeholders			s
where	sc.stakeholder_code		= s.stakeholder_code
and	sgc.stakeholder_group_code	= sc.stakeholder_group_code
and	cc.conference_id(+)		= s.conference_id', 'stakeholders');

   XML2.footer;

end XML;

-- ***************************************************************************************************************
-- ***************************************************************************************************************

procedure init_variables is

begin

   repo_email.init_variables;
   email.variables(1) := null;
   email.variables(2) := null;

end init_variables;

-- ***************************************************************************************************************
-- ***************************************************************************************************************

function get_persons (	gcode	common.ident_arr,
			code	common.ident_arr,
			curr	common.ident_arr) return persons_type is

cursor	driver (curr$ integer) is
select	*
from	persons
where	person_id in
(	select	person_id
	from	stakeholders
	where	stakeholder_code in
	(	select	stakeholder_code
		from	stakeholder_codes_temp
	)
	and	curr$ = case 
		when trunc(sysdate) between nvl(start_date, to_date('1000-01-01', 'yyyy-mm-dd') )
					and nvl(end_date,   to_date('4000-01-01', 'yyyy-mm-dd') )
		then 1 else 0 end
);

recs	persons_type;
current	integer;

begin

   for i in 1..gcode.COUNT loop
      insert into stakeholder_codes_temp
      select	stakeholder_code
      from	stakeholder_codes
      where	stakeholder_group_code = gcode(i);
   end loop;

   for i in 1..code.COUNT loop
      insert into stakeholder_codes_temp values (code(i));
   end loop;

   if curr.COUNT > 0 then
      current := 0;
   else
      current := 1;
   end if;

   open driver (current);
   fetch driver BULK COLLECT into recs;
   close driver;

   return recs;

end get_persons;

-- ***************************************************************************************************************
-- ***************************************************************************************************************

procedure mail_send2 (	gcode common.ident_arr default common.mt_ident_arr$,
			code  common.ident_arr default common.mt_ident_arr$,
			subject varchar2, msg varchar2,
			curr  common.ident_arr default common.mt_ident_arr$) is

recs	persons_type;

begin

   if common.header (rpt_title$) then
      init_variables;
      recs := get_persons (gcode, code, curr);

      for i in 1..recs.COUNT LOOP
         repo_email.driver_rec := recs(i);

         send_mail (	common.get_user_email (recs(i).person_id),
			repo_email.replace_variables(subject),
			repo_email.replace_variables(msg),
			pid=>recs(i).person_id );
      end loop;

      htp.header (3, 'Email Sent: ' || recs.COUNT);

      htp.anchor ('stakeholder.mail', 'Back to Stakeholder Email Utility');
      common.footer;
   end if;

end mail_send2;

-- ***************************************************************************************************************
-- ***************************************************************************************************************

procedure show_sample_email (subject varchar2, msg varchar2) is

cursor	driver is
select	*
from	persons
where	person_id = nvl(common.gbl_priv_pid, common.gbl_pid);

begin

   htp.header (3, 'Sample Email');

   init_variables;
   open driver;
   fetch driver into repo_email.driver_rec;
   close driver;

   htp.bold ('Subject');
   htp.p (repo_email.replace_variables(subject));
   htp.para;
   htp.bold ('Message');
   htp.br;
   htp.p (replace (repo_email.replace_variables(msg), chr(10), htf.br) );

end show_sample_email;

-- ***************************************************************************************************************

procedure mail_send (	gcode common.ident_arr default common.mt_ident_arr$,
			code  common.ident_arr default common.mt_ident_arr$,
			subject varchar2, msg varchar2,
			curr  common.ident_arr default common.mt_ident_arr$) is

recs	persons_type;

begin

   if common.header (rpt_title$) then
      show_sample_email (subject, msg);

      htp.formOpen ('stakeholder.mail_send2', 'POST');
      common.hidden_array ('gcode', gcode);
      common.hidden_array ('code', code);
      common.hidden_array ('curr', curr);
      htp.formHidden ('subject', subject);
      htp.formHidden ('msg', msg);
      htp.formSubmit (null, 'Send Email');
      htp.formClose;

      recs := get_persons (gcode, code, curr);
      htp.header (3, recs.COUNT || ' email will be sent.');
      common.footer;
   end if;

end mail_send;

-- ***************************************************************************************************************
-- ***************************************************************************************************************

function get_counts (gcode varchar2, code varchar2) return varchar2 is

cursor	driver is
select	sum(icount) icount, sum(active_count) acount
from	(
		select	count(1) icount,
			SUM (case 
			when trunc(sysdate) between nvl(start_date, to_date('1000-01-01', 'yyyy-mm-dd') )
						and nvl(end_date,   to_date('4000-01-01', 'yyyy-mm-dd') )
			then 1
			else 0
			end) active_count
		from	stakeholders
		where	stakeholder_code in
		(	select	stakeholder_code
			from	stakeholder_codes
			where	stakeholder_group_code = gcode
		)
		UNION
		select	count(1) icount,
			SUM (case 
			when trunc(sysdate) between nvl(start_date, to_date('1000-01-01', 'yyyy-mm-dd') )
						and nvl(end_date,   to_date('4000-01-01', 'yyyy-mm-dd') )
			then 1
			else 0
			end) active_count
		from	stakeholders
		where	stakeholder_code = code
	);

rec	driver%ROWTYPE;

begin

   open driver;
   fetch driver into rec;
   close driver;

   return ' (' || rec.icount || ', ' || nvl(rec.acount,0) || ')';

end get_counts;

-- ***************************************************************************************************************

procedure email_recipients is

cursor	driver is
select	*
from	stakeholder_group_codes
order by stakeholder_group_descr;

cursor	C2 (code$ varchar2) is
select	*
from	stakeholder_codes
where	stakeholder_group_code = code$
order by stakeholder_descr;

begin

   htp.header (3, 'Select Recipients (Total, Active)');
   for rec in driver LOOP
      htp.formCheckbox ('gcode', rec.stakeholder_group_code);
      htp.p (rec.stakeholder_group_descr);
      htp.p (get_counts (rec.stakeholder_group_code, null));
      htp.br;

      htp.ulistOpen;
      for C2_rec in C2(rec.stakeholder_group_code) LOOP
         htp.listItem (	htf.formCheckbox ('code', C2_rec.stakeholder_code) ||
			C2_rec.stakeholder_descr ||
			get_counts (null, C2_rec.stakeholder_code) );
      end loop;
      htp.ulistClose;
   end loop;

end email_recipients;

-- ***************************************************************************************************************
-- ***************************************************************************************************************

procedure mail is

begin

   if common.header (rpt_title$, js_src=>common.server||common.vdir$||'stakeholder.js') then
      init_variables;
      htp.formOpen ('stakeholder.mail_send', 'POST', cattributes=>'SPMS');

      htp.tableOpen;
         htp.tableRowOpen;
            htp.p ('<TD ALIGN="LEFT">');
               email_recipients;
            htp.p ('</TD ALIGN="LEFT">');

            htp.p ('<TD ALIGN="LEFT">');
               htp.tableOpen;
                  common.show_data_row ('Subject', htf.formText ('subject', 50, 100) );
                  common.show_data_row ('Message', htf.formTextArea ('msg', 10, 50) );
                  common.show_data_row (null, htf.formCheckbox ('curr', 'Y', 'CHECKED') ||
						' Send to ' || htf.bold ('Active') || ' members only' );
                  common.show_data_row (null, htf.formReset || common.space2$ || htf.formSubmit);

                  htp.tableRowOpen;
                     htp.tableHeader (common.space$);
                     htp.p ('<TD ALIGN="LEFT">');
                        email.show_variables(false);
                     htp.p ('</TD>');
                  htp.tableRowClose;
               htp.tableClose;
            htp.p ('</TD ALIGN="LEFT">');
         htp.tableRowClose;
      htp.tableClose;
      common.footer;
   end if;

end mail;

-- ***************************************************************************************************************
-- ***************************************************************************************************************

procedure del (code varchar2, pid integer) is

begin

   common.get_gbl_pid;
   if common.has_privs_to_page then
      delete from stakeholders where stakeholder_code = code and person_id = pid;
      common.redirect_URL (common.sserver || common.vdir$ || 'stakeholder.html');
   end if;

end del;

-- ***************************************************************************************************************
-- ***************************************************************************************************************

procedure new_form (form_rec form_rec_type) is

cursor	driver is
select	sc.*, sgc.stakeholder_group_descr
from	stakeholder_group_codes		sgc,
	stakeholder_codes		sc
where	sc.stakeholder_code		= form_rec.code
and	sgc.stakeholder_group_code	= sc.stakeholder_group_code;

rec	driver%ROWTYPE;
buffer	varchar2(7);

begin

   if form_rec.inserting = 'Y' then
      buffer := 'Add ';
   else
      buffer := 'Update ';
   end if;

   open driver;
   fetch driver into rec;
   close driver;

   htp.header (3, buffer || rec.stakeholder_group_descr || common.mdash$ || rec.stakeholder_descr);

   if form_rec.msg is not null then
      htp.p (form_rec.msg);
   end if;

   htp.formOpen ('stakeholder.save', 'POST', cattributes=>'NAME="SPMS"');
   htp.formHidden ('code', form_rec.code);
   htp.formHidden ('affil_id', '');
   htp.formHidden ('inserting', form_rec.inserting);
   htp.tableOpen;

   common.show_data_row ('<INPUT TYPE="button" VALUE="Search for Username" ' ||
				'onClick=''return newWindow ("profile.find_account")''>',
				htf.formText ('uid', 30, 30, form_rec.UID) );

   if rec.require_affiliation = 'Y' then
      common.show_data_row ('<INPUT TYPE="button" VALUE="Search for Affiliation" ' ||
				'onClick=''return newWindow ("stakeholder.affil_search")''>',
				htf.formText ('affil_name', 50, 50, form_rec.affil_name, cattributes=>'DISABLED') );
   end if;

   if rec.require_series = 'Y' then
      common.show_popup_table_row ('conference_code', 'conference_descr', 'conference_codes',
				'Conference Series', form_rec.conf_series, var_name=>'conf_series');

   end if;

   if rec.require_conference = 'Y' then
      common.show_popup_table_row ('conference_id', 'conference_title', 'conf_conferences',
				'Conference Name' || htf.formHidden ('starting', '')
						  || htf.formHidden ('ending', ''),
				form_rec.conf_id, var_name=>'conf_id',
				where_claws=>'where conference_code <> ''TM'' ' ||
					'and conference_id not in ' ||
					'(select conference_id from stakeholders ' ||
					'where stakeholder_code = ''' || form_rec.code || ''')',
				order_by=>'order by to_char(start_date, ''yyyy'') desc, conference_title');
   else
      common.show_data_row ('Start Date',  htf.formText ('starting', 10, 10, form_rec.starting.str) || ' Format: MM/DD/YYYY');
      common.show_data_row ('Ending Date', htf.formText ('ending', 10, 10, form_rec.ending.str) || ' Format: MM/DD/YYYY');
   end if;

   common.show_data_row (null, htf.formSubmit (null, buffer));

   htp.tableClose;
   htp.formClose;

end new_form;

-- ***************************************************************************************************************
-- ***************************************************************************************************************

procedure edit (pid integer, code varchar2) is

cursor	driver is
select	*
from	stakeholders
where	person_id		= pid
and	stakeholder_code	= code;

rec		driver%ROWTYPE;
form_rec	form_rec_type;

begin

   if common.header (rpt_title$) then
      open driver;
      fetch driver into rec;
      close driver;

      form_rec.code		:= code;
      form_rec.UID		:= common.decode_value ('user_id', 'persons', 'person_id', rec.person_id);
      form_rec.affil_id		:= rec.affiliation_id;
      form_rec.affil_name	:= common.get_affil_name (rec.affiliation_id);
      form_rec.conf_series	:= rec.conference_code;
      form_rec.conf_id		:= rec.conference_id;
      form_rec.starting.dte	:= rec.start_date;
      form_rec.starting.str	:= to_char(rec.start_date, 'mm/dd/yyyy');
      form_rec.ending.dte	:= rec.end_date;
      form_rec.ending.str	:= to_char(rec.end_date, 'mm/dd/yyyy');
      form_rec.inserting	:= 'N';

      new_form (form_rec);

      common.footer;
   end if;

end edit;

-- ***************************************************************************************************************
-- ***************************************************************************************************************

procedure set_editor_dates (conf_id integer) is

cursor	driver is
select	add_months (start_date, -12)	start_date,
	add_months (end_date, 12)	end_date
from	conf_conferences
where	conference_id	= conf_id;

begin

   open driver;
   fetch driver into form_rec.starting.dte, form_rec.ending.dte;
   close driver;

   form_rec.starting.str	:= to_char(form_rec.starting.dte, 'mm/dd/yyyy');
   form_rec.ending.str		:= to_char(form_rec.ending.dte, 'mm/dd/yyyy');

end set_editor_dates;

-- ***************************************************************************************************************

procedure save (code		varchar2,
		UID		varchar2,
		affil_id	integer	 default null,
		conf_series	varchar2 default null,
		conf_id		integer  default null,
		starting	varchar2,
		ending		varchar2,
		inserting	char) is

cursor	driver is
select	*
from	stakeholder_codes
where	stakeholder_code = code;

cursor	C2 (bdate date, edate date) is
select	1
from	stakeholders
where	person_id = (select person_id from persons where user_id = upper(UID))
and	stakeholder_code	= code
and	nvl(conference_id,0)	= nvl(conf_id,0)
and	nvl(bdate,sysdate)	between nvl(start_date,min_date$) and nvl(end_date,max_date$)
and	nvl(edate,sysdate)	between nvl(start_date,min_date$) and nvl(end_date,max_date$);

rec		driver%ROWTYPE;
dummy		integer;
rec_found	boolean;

begin

   open driver;
   fetch driver into rec;
   close driver;

   form_rec.code		:= code;
   form_rec.UID			:= UID;
   form_rec.affil_id		:= affil_id;
   form_rec.affil_name		:= common.get_affil_name (affil_id, htf.br);
   form_rec.conf_series		:= conf_series;
   form_rec.conf_id		:= conf_id;
   form_rec.starting.str	:= trim(starting);
   form_rec.ending.str		:= trim(ending);

   if UID is null then
      form_rec.msg := form_rec.msg || htf.header (3, 'You must enter a username.', cattributes=>common.red$);
   end if;

   if rec.require_affiliation = 'Y' then
      if nvl(affil_id,0) = 0 then
         form_rec.msg := form_rec.msg || htf.header (3, 'You must enter an affiliation.', cattributes=>common.red$);
      end if;
   end if;

   if rec.require_series = 'Y' then
      if conf_series is null then
         form_rec.msg := form_rec.msg || htf.header (3, 'You must enter a conference series.', cattributes=>common.red$);
      end if;
   end if;

   if rec.require_conference = 'Y' then
      if conf_id is null then
         form_rec.msg := form_rec.msg || htf.header (3, 'You must enter a conference name.', cattributes=>common.red$);
      end if;
   end if;

   if form_rec.conf_id is not null then
      set_editor_dates (form_rec.conf_id);
   end if;

   if rec.require_dates = 'Y' then
      if form_rec.starting.str is null then
         form_rec.msg := form_rec.msg || htf.header (3, 'Starting date is required.', cattributes=>common.red$);
      end if;

      if form_rec.ending.str is null then
         form_rec.msg := form_rec.msg || htf.header (3, 'Ending date is required.', cattributes=>common.red$);
      end if;
   end if;

   if form_rec.starting.str is not null then
      if common.validate_date (form_rec.starting.str) then
         form_rec.starting.dte := common.convert_date (form_rec.starting.str);
      else
         form_rec.msg := form_rec.msg || htf.header (3, 'Invalid starting date.', cattributes=>common.red$);
      end if;
   end if;

   if form_rec.ending.str is not null then
      if common.validate_date (form_rec.ending.str) then
         form_rec.ending.dte := common.convert_date (form_rec.ending.str);
      else
         form_rec.msg := form_rec.msg || htf.header (3, 'Invalid ending date.', cattributes=>common.red$);
      end if;
   end if;

   if inserting = 'Y' AND form_rec.msg is null then
      open C2 (form_rec.starting.dte, form_rec.ending.dte);
      fetch C2 into dummy;
      rec_found := C2%FOUND;
      close C2;

      if rec_found then
         form_rec.msg := form_rec.msg || htf.header (3, 'This User is already in the specified role.', cattributes=>common.red$);
      end if;
   end if;

   common.get_gbl_pid;
   if common.has_privs_to_page then

      if form_rec.msg is null then

         if inserting = 'Y' then
            insert into stakeholders (STAKEHOLDER_CODE, PERSON_ID, AFFILIATION_ID, CONFERENCE_ID,
					CONFERENCE_CODE, START_DATE, END_DATE)
            select	form_rec.code, person_id, form_rec.affil_id, form_rec.conf_id, form_rec.conf_series,
			form_rec.starting.dte, form_rec.ending.dte
            from	persons
            where	user_id = upper(UID);

         else
            update	stakeholders
            set		AFFILIATION_ID		= form_rec.affil_id,
			CONFERENCE_ID		= form_rec.conf_id,
			CONFERENCE_CODE		= form_rec.conf_series,
			START_DATE		= form_rec.starting.dte,
			END_DATE		= form_rec.ending.dte
            where	PERSON_ID		= common.get_user_pid (form_rec.UID)
            and		STAKEHOLDER_CODE	= form_rec.code;
         end if;

         common.redirect_URL (common.sserver || common.vdir$ || 'stakeholder.html');

      else
         common.public_header (rpt_title$);
         new_form (form_rec);
         common.footer;
      end if;

   end if;

end save;

-- ***************************************************************************************************************
-- ***************************************************************************************************************

procedure affil_search2 (text varchar2) is

cursor driver (txt varchar2) is
select	affil.affiliation_id,
	common.get_affil_name (affil.affiliation_id, htf.br) affil_name
from	affiliations affil
where	affil.affiliation_id >= 0
and	contains (affil.search_text, txt, 1) > 0
order by score(1) desc, 1;

TYPE rec_type is table of driver%ROWTYPE index by binary_integer;
recs rec_type;

begin

   if common.header (rpt_title$, js_src=>common.server||common.vdir$||'stakeholder.js') then
      profile.check_CTX_index;

      open driver (common.fmt_search_string (text, ' NEAR '));
      fetch driver BULK COLLECT into recs;
      close driver;

      if recs.count = 0 then
         open driver (common.fmt_search_string (text, ' OR ') );
         fetch driver BULK COLLECT into recs;
         close driver;
      end if;

      htp.p ('<SCRIPT TYPE="text/javascript" language="Javascript" type="text/javascript">');
      htp.p ('<!--');
      htp.p ('var affils = new Array();');

      for i in 1..recs.count LOOP
         recs(i).affil_name := trim (recs(i).affil_name);
         recs(i).affil_name := replace (recs(i).affil_name, '"', null);

         htp.prn ('affils[');
         htp.prn (recs(i).affiliation_id);
         htp.prn ('] = new ImageConstruct ("');
         htp.prn (recs(i).affil_name);
         htp.prn ('");');
         htp.print('');
      end loop;

      htp.p ('-->');
      htp.p ('</SCRIPT>');

      htp.tableOpen ('BORDER');
      for i in 1..recs.count LOOP
         htp.tableRowOpen ('LEFT', 'TOP');
         htp.tableData (htf.anchor ('javascript:setValue(' || recs(i).affiliation_id || ')', 'Select') );
         htp.tableData (recs(i).affil_name);
         htp.tableRowClose;
      end loop;
      htp.tableClose;

      if recs.count = 0 then
         htp.header (3, 'No Affiliations Matched Your Search String', cattributes=>common.red$);
         htp.bold ('You many use your browswer''s back button to try again.');
      end if;

      common.footer;
   end if;

end affil_search2;

-- ***************************************************************************************************************

procedure affil_search is

begin

   if common.header (rpt_title$) then
      htp.formOpen ('stakeholder.affil_search2', cattributes=>'NAME="ASEARCH"');
      htp.header (4, 'Search for an Affiliation');
      htp.p ('Enter any portion of an Affiliation''s name then click Search.');
      htp.br;
      htp.formText ('text', 50, 50);
      htp.br;
      htp.formSubmit (null, 'Search');
      htp.formClose;
      common.form_focus ('ASEARCH', 'text');
      common.footer;
   end if;

end affil_search;

-- ***************************************************************************************************************

procedure new (code varchar2) is

begin

   if common.header (rpt_title$, js_src=>common.server||common.vdir$||'stakeholder.js') then
      form_rec.code := code;
      new_form (form_rec);
      common.footer;
   end if;

end new;

-- ***************************************************************************************************************

procedure new_stakeholder is

begin

   htp.para;
   htp.formOpen ('stakeholder.new', 'POST');
   common.show_popup_table ('stakeholder_code', 'stakeholder_descr', 'stakeholder_codes',
				htf.bold ('Add a new stakeholder: '), var_name=>'code');
   htp.formSubmit;
   htp.formClose;
   htp.para;

end new_stakeholder;

-- ***************************************************************************************************************

function get_conf_info (cid integer) return varchar2 is

rec	conf_conferences%ROWTYPE;

begin

   rec := get_conf_data (cid);
   if rec.conference_id is null then
      return null;
   end if;

   return '<DIV STYLE="margin-left: 15px; font-style: italic">' ||
		rec.conference_title	|| common.mdash$ ||
		rec.location		|| htf.br ||
		to_char(rec.start_date,date_fmt$) || common.mdash$ ||
		to_char(rec.end_date,date_fmt$)	  || '</DIV>';

end get_conf_info;

-- ***************************************************************************************************************

procedure show_filters is

begin

   htp.formOpen ('stakeholder.show_stakeholders', 'POST');
   htp.p ('Display Expired Terms');
   htp.formCheckBox ('include_expired', 'Y');
   htp.formSubmit;
   htp.formClose;
   htp.para;

end show_filters;

-- ***************************************************************************************************************

procedure show_stakeholders (include_expired char default 'N') is

cursor	driver is
select	s.*, sc.stakeholder_descr, sgc.stakeholder_group_descr,
	htf.bold (common.get_user_name(s.person_id)) || ', ' ||
	htf.italic(common.get_user_affil_abbrev_list(s.person_id))	stakeholder_name,
	common.decode_value ('conference_descr', 'conference_codes', 'conference_code', conference_code) conference_series
from	stakeholder_group_codes		sgc,
	stakeholder_codes		sc,
	stakeholders			s
where	sc.stakeholder_code		= s.stakeholder_code
and	sgc.stakeholder_group_code	= sc.stakeholder_group_code
and (  (include_expired	= 'N' 	AND
	sysdate		<= nvl(end_date,max_date$)
       )
    OR	include_expired = 'Y'
    )
order by sc.sort_order,
	to_date (common.decode_value ('start_date', 'conf_conferences', 'conference_id', conference_id), 'dd-Mon-yy') desc,
	sc.stakeholder_descr,
	common.get_user_name (s.person_id, 'Y'),
	s.start_date desc;

style		varchar2(32);
highlight$	constant varchar2(32)	:= 'STYLE="BACKGROUND-COLOR:#DDDDDD"';

type	rec_type is table of driver%ROWTYPE index by pls_integer;
recs	rec_type;

begin

   if common.header (rpt_title$) then
      new_stakeholder;
      show_filters;

      htp.header (2, 'Joint Accelerator Conference Website (JACoW) Stakeholders');
      htp.tableOpen ('BORDER', cattributes=>common.table_hover$);

      htp.tableRowOpen;
      htp.tableHeader (common.space$, ccolspan=>'2');
      htp.tableHeader ('Stakeholder Name', 'LEFT');
      htp.tableHeader ('Stakeholder Type', 'CENTER', ccolspan=>'2');
      htp.tableHeader ('Affiliation', 'LEFT');
      htp.tableHeader ('Conference Series', 'LEFT');
      htp.tableHeader ('Start Date', 'LEFT');
      htp.tableHeader ('End Date', 'LEFT');
      htp.tableRowClose;

      open driver;
      fetch driver BULK COLLECT into recs;
      close driver;

      for i in 1..recs.COUNT loop
         style := null;
         if sysdate NOT between nvl(recs(i).start_date,sysdate-1) and nvl(recs(i).end_date,sysdate+1) then
            style := highlight$;
         end if;

         htp.tableRowOpen ('LEFT', 'TOP', cattributes=>style);
         htp.tableData (htf.anchor ('stakeholder.edit'	||
					'?pid=' || recs(i).person_id ||
					'&code=' || recs(i).stakeholder_code, 'Edit') );
         htp.tableData (htf.anchor ('stakeholder.del'	||
					'?pid=' || recs(i).person_id ||
					'&code=' || recs(i).stakeholder_code, 'Delete') );
         htp.tableData (recs(i).stakeholder_name);
         htp.tableData (recs(i).stakeholder_group_descr);
         htp.tableData (recs(i).stakeholder_descr || get_conf_info(recs(i).conference_id));
         htp.tableData (common.get_affil_name(recs(i).affiliation_id));
         htp.tableData (recs(i).conference_series);
         htp.tableData (to_char(recs(i).start_date, date_fmt$));
         htp.tableData (to_char(recs(i).end_date, date_fmt$));
         htp.tableRowClose;
      end loop;

      htp.tableClose;
      htp.p (recs.COUNT);
      htp.p ('Stakeholders.');
      htp.p ('Rows highlighted in gray are either expired or not yet active terms.');
      common.footer;
   end if;

end show_stakeholders;

-- ***************************************************************************************************************

procedure html is

begin

   show_stakeholders;

end html;

-- ***************************************************************************************************************

end stakeholder;
/
show error

set define on
