set define off

create or replace package body status_code_matrix as

rpt_title$	constant char(41)	:= 'Upload Editor/Referee Status Code Matrix';

-- ******************************************************************************
-- ******************************************************************************

procedure save (eCode common.ident_arr, rCode common.ident_arr, upload common.ident_arr) is

begin

   if common.header (rpt_title$, test_only=>true) then
      for i in 1..upload.COUNT LOOP
         update	upload_status_codes
         set	uploadable		= upload(i)
         where	editor_status_code	= eCode(i)
         and	referee_status_code	= rCode(i)
         and	uploadable		<> upload(i);
      end loop;

      common.redirect_URL ('status_code_matrix.html');
   end if;

end save;

-- ******************************************************************************
-- ******************************************************************************

procedure html is

cursor	C1 is
select	*
from	editor_status_codes
order by decode (editor_status_code,	common.unassigned$, 0,
					common.assigned$,   1,
					common.uploadable$, 2, 3);
cursor	C2 is
select	*
from	referee_status_codes
order by decode (referee_status_code,	common.unassigned$, 0,
					common.assigned$,   1,
					common.declined$,   4,
					common.uploadable$, 2, 3);

cursor	C3 (eCode varchar2, rCode varchar2) is
select	uploadable
from	upload_status_codes
where	editor_status_code	= eCode
and	referee_status_code	= rCode;

type	eRec_type is table of C1%ROWTYPE index by binary_integer;
type	rRec_type is table of C2%ROWTYPE index by binary_integer;
eRecs	eRec_type;
rRecs	rRec_type;

C3_rec	C3%ROWTYPE;

begin

   if common.header (rpt_title$) then
      open C1;
      fetch C1 BULK COLLECT into eRecs;
      close C1;

      open C2;
      fetch C2 BULK COLLECT into rRecs;
      close C2;

      htp.para;

      htp.formOpen ('status_code_matrix.save', 'POST');
      htp.tableOpen ('BORDER');
      htp.tableCaption ('The Yes/No drop-down lists determine if a '	||
		'user may upload files based on the status of the  '	||
		'Editor and Referee status codes. Every  combination  ' ||
		'is covered in the rows and columns.  Set the values  ' ||
		'of each combination then click save.',
		cattributes=>'style="text-align: left;"');

      htp.tableRowOpen ('LEFT', 'BOTTOM');
         htp.tableHeader ('Editor Status', crowspan=>'2');
         htp.tableHeader ('Referee Status', 'CENTER', ccolspan=>'5');
      htp.tableRowClose;

      htp.tableRowOpen;
         for i in 1..rRecs.COUNT LOOP
            htp.tableHeader (rRecs(i).referee_status_descr);
         end loop;
      htp.tableRowClose;

      for i in 1..eRecs.COUNT LOOP
         htp.tableRowOpen ('CENTER', 'TOP');
         htp.tableHeader (eRecs(i).editor_status_descr, 'LEFT');
         for j in 1..rRecs.COUNT LOOP
            open C3 (eRecs(i).editor_status_code, rRecs(j).referee_status_code);
            fetch C3 into C3_rec;
            close C3;

            htp.tableData (htf.formHidden ('eCode', eRecs(i).editor_status_code)  ||
			   htf.formHidden ('rCode', rRecs(j).referee_status_code) ||
		common.popup_yes_no ('upload', C3_rec.uploadable, false));
         end loop;
         htp.tableRowClose;
      end loop;

      htp.tableClose;
      htp.para;
      htp.formSubmit (null, 'Save');
      htp.formClose;

      common.footer;
   end if;

end html;

-- ******************************************************************************

end status_code_matrix;
/
show error

set define on

--	Copyright (C) 2004 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>