set define off

create or replace package body sync_initial as

rpt_title$	constant char(28)	:= 'Initial SPMS Synchronization';
dte_fmt$	constant char(10)	:= 'mm/dd/yyyy';

-- **************************************************************************************************

function get_admins (admin_email varchar2) return varchar2 is

cursor	driver is
select	prof.email_address
from	profiles		prof,
	user_functions		uf
where	uf.function_code	in ('CC', 'DBA')
and	prof.person_id		= uf.person_id
and	prof.email_address	<> admin_email
and	prof.email_address	is not null;

buffer	varchar2(4000);
sepstr	varchar2(2);

begin

   for rec in driver LOOP
      buffer := buffer || sepstr || rec.email_address;
      sepstr := ', ';
   end loop;

   return buffer;

end get_admins;

-- **************************************************************************************************

procedure send_initial_email is

cursor	driver is
select	person_id, common.get_user_name (person_id) name
from	profiles
where	email_address = lower (common.editor_email$);

admin		driver%ROWTYPE;
rec		mail_blats%ROWTYPE;
pid		integer;
region_name	varchar2(4);

type	region_rec is record (
		name	varchar2(20),
		URL	varchar2(50) );

type	region_type is table of region_rec index by varchar2(4);
region	region_type;

begin

   common.params.DELETE;
   region_name := common.decode_value ('value', 'parameters', 'ID', '-102');
   delete from parameters where ID = -102;
   
   region('CERN').name	:= 'European';
   region('CERN').URL	:= 'https://oraweb.cern.ch';
   region('FNAL').name	:= 'North American';
   region('FNAL').URL	:= 'https://appora.fnal.gov';
   region('KEK').name	:= 'Asian';
   region('KEK').URL	:= 'https://spms.kek.jp';

   open driver;
   fetch driver into admin;
   close driver;

   rec.subject		:= 'JACoW SPMS instance for ' || common.conf_title$ || ' is active';
   rec.recipient	:= common.editor_email$;
   rec.cc		:= get_admins (rec.recipient);

   rec.message  := 'Dear ' || admin.name || ','								|| common.eol2$ ||
		'We are pleased to inform you that the SPMS database for the ' || common.conf_title$	||
		' conference, and its web accessibility, has been successfully set up at the '	||
		'JACoW ' || region(region_name).name || ' Regional Support Center, hosted at '		||
		region_name || '. We provided all necessary privileges as "Conference '			||
		'Coordinator" to you. To set up the additional parameters for the conference in the '	||
		'SPMS system please proceed via the following URLs: '					|| common.eol2$ ||
		'This is the "standard" URL that you can publish on your website for authors, '		||
		'workshop attendees, etc.: ' || region(region_name).URL || common.vdir_dflt$		||
		'profile.html This is your URL to use, as it gives more detailed information in case '	||
		'of service malfunctioning. ' || region(region_name).URL || common.vdir_debug$		||
		'profile.html'										|| common.eol2$	||
		'For any questions or advice with respect to this parameterization, feel free to '	||
		'send email to jacow-spms-users@lists.sourceforge.net.'					|| common.eol2$	||
		'Wishing you all the best for your conference.'						|| common.eol2$	||
		'The JACoW Team';

   send_mail (rec.recipient, rec.subject, rec.message, null, rec.cc);

end send_initial_email;

-- **************************************************************************************************

procedure load_data is

cursor	driver (conf_code$ varchar2) is
select	conference_id
from	repo_conf_conferences
where	conference_code = conf_code$
and	start_date	=
	(	select	max (start_date)
		from	repo_conf_conferences
		where	conference_code = conf_code$
	);


icount		integer;
conf_code	conference_codes.conference_code%TYPE;
conf_id		integer;
next_seq_id	integer;
max_seq_id	integer;
dummy		integer;

begin

   dbms_application_info.set_client_info ('SYNC to Repository');
   if NOT common.repo_is_remote then
      raise_application_error(-20004, 'This IS the repository database or a standalone DB. No need to SYNC.');
   end if;

   update control set last_sync_date = common.get_UTC_time;

-- **********

   select conference_code into conf_code from control;

   open driver (conf_code);
   fetch driver into conf_id;
   close driver;

-- **********

   insert into CLASSIFICATION_CODES (classification_code, classification_descr)
   select class_code, class_descr
   from   repo_conf_classifications
   where  conference_id = conf_id;

   select count(1) into icount from classification_codes;
   if icount = 0 then
      insert into classification_codes select * from repo_classification_codes;
   end if;

-- **********

   insert into SUB_CLASS_CODES (sub_class_code, sub_class_descr)
   select sub_class_code, sub_class_descr
   from	  repo_conf_sub_classes
   where  conference_id = conf_id;

   select count(1) into icount from sub_class_codes;
   if icount = 0 then
      insert into sub_class_codes select * from repo_sub_class_codes;
   end if;

-- **********

   insert into keyword_codes (keyword_code, keyword_descr, regular_expression)
   select keyword_code, keyword_descr, regular_expression
   from	  repo_conf_keywords
   where  conference_id = conf_id;

-- **********

   insert into ACTIVITY_CODES			select * from repo_ACTIVITY_CODES;
   insert into affil_type_codes			select * from repo_affil_type_codes;
   insert into BYTE_CODES			select * from repo_BYTE_CODES;
   insert into SUPER_REGION_CODES		select * from repo_SUPER_REGION_CODES;
   insert into REGION_CODES			select * from repo_REGION_CODES;
   insert into country_codes			select * from repo_country_codes;
   insert into DATE_FORMAT_CODES		select * from repo_DATE_FORMAT_CODES;
   insert into FILE_TYPE_CODES			select * from repo_FILE_TYPE_CODES;
   insert into STATUS_CODES			select * from repo_STATUS_CODES;
   insert into LOG_CODES			select * from repo_LOG_CODES;
   insert into MEMBERSHIP_CODES			select * from repo_MEMBERSHIP_CODES;
   insert into PLATFORM_CODES			select * from repo_PLATFORM_CODES;
   insert into presentation_type_codes		select * from repo_presentation_type_codes;
   insert into presentation_codes		select * from repo_presentation_codes;
   insert into presentation_option_codes	select * from repo_presentation_option_codes;
   insert into ERROR_CODES			select * from repo_ERROR_CODES;
   insert into SOURCE_CODES			select * from repo_SOURCE_CODES;
   insert into search_sort_codes		select * from repo_search_sort_codes;
   insert into search_display_codes		select * from repo_search_display_codes;
   insert into data_type_codes			select * from repo_data_type_codes;
   insert into display_codes			select * from repo_display_codes;
   insert into payment_codes			select * from repo_payment_codes;
   insert into SPMS_jobs			select * from repo_SPMS_jobs;
   insert into time_zone_codes			select * from repo_time_zone_codes;
   insert into ical_chars			select * from repo_ical_chars;
   insert into DST_codes			select * from repo_DST_codes;

   delete from PRESENTATION_FILE_CODES;
   insert into PRESENTATION_FILE_CODES		select * from repo_PRESENTATION_FILE_CODES;

   insert into payment_vendor_codes		select * from repo_payment_vendor_codes;
   insert into SPMS_var_codes			select * from repo_SPMS_var_codes;
   insert into var_type_codes			select * from repo_var_type_codes;
   insert into payment_vendor_vars		select * from repo_payment_vendor_vars;

   insert into no_login_URLs			select * from repo_no_login_URLs;
   insert into proposal_status_codes		select * from repo_proposal_status_codes;
   insert into priority_codes			select * from repo_priority_codes;

   insert into editor_status_codes		select * from repo_editor_status_codes;
   insert into referee_status_codes		select * from repo_referee_status_codes;
   insert into upload_status_codes		select * from repo_upload_status_codes;
   insert into transparency_codes		select * from repo_transparency_codes;

-- **********

   insert into affiliations			select * from repo_affiliations;
   insert into addresses			select * from repo_addresses;
   insert into persons				select * from repo_persons;
   insert into profiles				select * from repo_profiles;
   insert into person_memberships		select * from repo_person_memberships;
   insert into mailing_lists			select * from repo_mailing_lists;
   insert into unavailable_times		select * from repo_unavailable_times;
   insert into author_types			select * from repo_author_types;
   insert into author_codes			select * from repo_author_codes;
   insert into program_sort_fields		select * from repo_program_sort_fields;
   insert into bypass_var_codes			select * from repo_bypass_var_codes;
   insert into SPMS_CTX_indexes			select * from repo_SPMS_CTX_indexes;
   insert into CTX_reserved_words		select * from repo_CTX_reserved_words;
   insert into maint_yes_no_flags		select * from repo_maint_yes_no_flags;

   insert into FUNCTION_CODES			select * from repo_FUNCTION_CODES	where function_code  <> 'RM';
   insert into PRIVILEGE_CODES			select * from repo_PRIVILEGE_CODES	where privilege_code <> 'RM';
   insert into FUNCTION_PRIVILEGES		select * from repo_FUNCTION_PRIVILEGES	where 'RM' not in (function_code, privilege_code);
   insert into APPLICATIONS			select * from repo_APPLICATIONS;
   insert into APPLICATION_PRIVS		select * from repo_APPLICATION_PRIVS	where privilege_code <> 'RM';

-- ********** Setup initial access

   insert into user_functions		-- The actual conference coordinator
	select	distinct pro.person_id, 'CC'
	from	profiles		pro,
		parameters		param
	where	param.id		= 19
	and	pro.email_address	= lower(param.value);


   insert into user_functions		-- The person who ran the initial setup
	select	distinct pro.person_id, 'DBA'
	from	profiles		pro,
		parameters		param
	where	param.id		= -101
	and	pro.email_address	= lower(param.value);
   delete from parameters where ID = -101;

   insert into user_functions		-- The principle SPMS developer
	select	1, 'DBA'
	from	dual
	where	not exists (	select	1
				from	user_functions
				where	person_id = 1);

   insert into user_functions		-- Default authorized users from the repository
	select	*
	from	repo_user_functions	repo
	where	function_code		= 'CC'
	and	NOT exists
		(	select	1
			from	user_functions	local
			where	local.person_id	= repo.person_id
		);

   insert into function_privileges
	select	'CC', 'ADMN'
	from	dual
	where	not exists
		(select 1 from function_privileges where function_code = 'CC' and privilege_code = 'ADMN');

-- ********** Prevent past merges from executing as we already have the most current data.

   insert into merge_logs			select * from repo_merge_logs;
   update merge_logs set merged_conf		= common.get_UTC_time;

   insert into merged_addresses			select * from repo_merged_addresses;
   update merged_addresses set timestamp	= common.get_UTC_time;

-- **********

   select application_ID_seq.nextval into next_seq_ID from dual;
   select max (application_ID) into max_seq_id from applications;

   for i in next_seq_ID..max_seq_id LOOP
      select application_ID_seq.nextval into dummy from dual;
   end loop;

-- ********** Set the URLs to the regional default values

   update status_codes	local
   set	image_URL	= (	select	image_URL
				from	repo_status_codes	repo
				where	repo.status_code	= local.status_code);

   update referee_codes	local
   set	image_URL	= (	select	image_URL
				from	repo_referee_codes	repo
				where	repo.referee_code	= local.referee_code);

   send_initial_email;
   repository.start_batch_jobs;

end load_data;

-- **************************************************************************************************
-- **************************************************************************************************

procedure header is

begin

   htp.p ('<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN" "http://www.w3.org/TR/1998/REC-html40-19980424/loose.dtd">');
   htp.htmlOpen;
   htp.headOpen;
   htp.p ('<META HTTP-EQUIV="PRAGMA" CONTENT="NO-CACHE">');
   htp.title (rpt_title$);
   htp.headClose;
   htp.bodyOpen;
   htp.header (2, rpt_title$);

end header;

-- **************************************************************************************************

procedure show_data_row (label varchar2, data_value varchar2) is

begin

   htp.tableRowOpen ();
   htp.tableHeader (label, 'RIGHT', cnowrap=>'NOWRAP');
   htp.tableData(data_value);
   htp.tableRowClose;

end show_data_row;

-- **************************************************************************************************

procedure footer is

begin

   htp.bodyClose;
   htp.htmlClose;

end footer;

-- **************************************************************************************************
-- **************************************************************************************************

procedure done is

begin

   header;
   htp.header (3, 'The initialization parameters were saved.');
   htp.p ('The initialization process will begin in 1 minute.');
   htp.p ('Monitor the initial synchronization job in SQL*Plus. When it completes,');
   htp.p ('you may continue with additional setup steps.');
   htp.p ('Once the initial sync completes, you may');
   htp.anchor ('repository.html', 'log into the conference repository');
   htp.p ('with the username JACOW and the master password you entered on the previous page.');
   footer;

end done;

-- **************************************************************************************************
-- **************************************************************************************************

function get_value (ID integer, param_id common.ident_arr, param_value common.ident_arr) return varchar2 is

begin

   for i in 1..param_id.COUNT loop
      if ID = param_id(i) then
         return param_value(i);
      end if;
   end loop;

   return null;

end get_value;

-- **************************************************************************************************

procedure save (this_payment_code varchar2, ver varchar2, pwd varchar2, this_user varchar2,
		param_id common.ident_arr, param_type common.ident_arr,
		param_value common.ident_arr) is

cursor	driver (code$ varchar2, bdate$ date) is
select	conference_id, last_sync_id
from	repo_conf_conferences
where	conference_code			= code$
and	to_char (start_date, 'yyyy')	= to_char (bdate$, 'yyyy');

rec		driver%ROWTYPE;
msg		long := null;
conf_rec	repo_conf_conferences%ROWTYPE;
job_id		number;
missing		boolean := false;
region_name	varchar2(20)  := owa_util.get_cgi_env ('SERVER_NAME');

begin

   header;

   if trim (this_payment_code) is null then
      msg := msg || htf.header (3, 'You must supply a Confernece Series.');
   end if;

   if trim (ver) is null then
      msg := msg || htf.header (3, 'You must supply the SPMS version number.');
   end if;

   if trim (pwd) is null then
      msg := msg || htf.header (3, 'You supply a master password.');
   end if;

   for i in 1..param_id.COUNT loop

      if trim (param_value(i)) is null then
         missing := true;
      end if;

      if param_type(i) = params.dtype_date$ then
         if trim (param_value(i)) is NOT null then
            if NOT common.validate_date (trim(param_value(i)), dte_fmt$) then
               msg := msg || htf.header (3, 'The Date Entered is Invalid. ' || param_value(i) );
            end if;
         end if;
      end if;

      if param_type(i) = params.dtype_number$ then
         if NOT common.validate_number (param_value(i)) then
            msg := msg || htf.header (3, 'The Value Entered is not a Number. ' || param_value(i) );
         end if;
      end if;

   end loop;

   if missing then
      msg := msg || htf.header (3, 'You must supply All Parameter Values');
   end if;

-- **********

   if msg is null then
      conf_rec.start_date := to_date (get_value(5,param_id,param_value), dte_fmt$);

      open driver (this_payment_code, conf_rec.start_date);
      fetch driver into rec;
      close driver;

      if rec.last_sync_id is not null then
         msg := msg || htf.header (3, 'This confernece is already being synced.');
      end if;
   end if;

-- ********** Save the data

   if msg is null then
      conf_rec.conference_id	:= rec.conference_id;
      conf_rec.conference_code	:= this_payment_code;
      conf_rec.end_date		:= conf_rec.start_date + get_value(45,param_id,param_value) - 1;
      conf_rec.location		:= get_value(43,param_id,param_value);
      conf_rec.conference_title	:= this_payment_code || ' ' || to_char (conf_rec.start_date, 'yyyy');

      insert into conference_codes	select * from repo_conference_codes;
      insert into param_categories	select * from repo_PARAM_CATEGORIES;
      insert into parameters		select * from repo_parameters where ID <> 0;
      insert into menu_items		select * from repo_menu_items;
      insert into menu_item_subs	select * from repo_menu_item_subs;

--	Temporary values to pass to the intial sync.
      insert into parameters (ID, value, dtype, long_text) values (-101, this_user, 'N', 'N'); -- the ID of the DBA
      region_name := substr (region_name, instr(region_name,'.')+1);
      region_name := substr (region_name, 1, instr(region_name,'.')-1);
      region_name := upper (region_name);
      insert into parameters (ID, value, dtype, long_text) values (-102, region_name, 'T', 'N');

      update	parameters	local
      set	value	= (
				select	value
				from	repo_initial_parameters repo
				where	repo.ID		= local.ID
				and	repo.region	= region_name)
      where	ID in (select ID from repo_initial_parameters);

      for i in 1..param_id.COUNT LOOP
         update	parameters
         set	value	= trim (param_value(i))
         where	ID	= param_id(i);
      end loop;

      update parameters set value = conf_rec.conference_title where ID = 1;

      repo_sync.initialize (conf_rec);

      insert into control (conference_code, conference_id, withdrawn_count, jacow_pwd, version)
      values (conf_rec.conference_code, conf_rec.conference_id, 0, trim(pwd), trim(ver) );

      dbms_job.submit (job_id, 'sync_initial.load_data;', sysdate + (1/1440));

      common.redirect_URL ('sync_initial.done');
   else
      htp.p (msg);
      htp.p (common.click_back$);
   end if;

   footer;

end save;

-- **************************************************************************************************

procedure get_initial_parameters is

cursor	driver (code varchar2) is
select	i.id, p.category_code, p.description, i.value, p.dtype
from	repo_parameters		p,
	repo_initial_parameters	i
where	i.region		= upper (code)
and	i.id			= p.id
and	(i.id = 6 OR p.category_code <> 'IURL')
order by p.category_code, p.description;

region	varchar2(20)  := owa_util.get_cgi_env ('SERVER_NAME');
vdir	varchar2(100);
L1	integer;
L2	integer;
buffer	varchar2(40);

begin

   vdir		:= lower (owa_util.get_cgi_env ('SCRIPT_NAME')) || '/';
   vdir		:= replace (vdir, '_debug', null);
   region	:= substr (region, instr(region,'.')+1);
   region	:= substr (region, 1, instr(region,'.')-1);

   htp.formOpen ('sync_initial.save', 'POST');
   htp.tableOpen;

   common.show_popup_table_row ('conference_code', 'conference_descr', 'repo_conference_codes', 'Conference Series');
   show_data_row ('SPMS Version',	htf.formText ('ver',	10, 10) );
   show_data_row ('Master Password',	htf.formText ('pwd',	20, 20) || ' Remember this password!');
   show_data_row (htf.bold ('Your') || ' email address',	htf.formText ('this_user', 20, 100));

   for rec in driver (region) LOOP
      if rec.id = 34 then
         rec.value := vdir;
      elsif rec.id = 76 then
         rec.value := substr (vdir, 1, length(vdir)-1) || '_debug/';
      end if;

      L1 := 100;
      L2 := 2000;
      buffer := null;

      if rec.dtype = params.dtype_date$ then
         L1 := 11;
         L2 := 11;
         buffer := ' (format: ' || dte_fmt$ || ')';
      elsif rec.dtype = params.dtype_number$ then
         L1 := 5;
         L2 := 5;
      end if;

      if rec.dtype = params.dtype_boolean$ then
         show_data_row (rec.description,	htf.formHidden ('param_id', rec.id) ||
						htf.formHidden ('param_type', rec.dtype) ||
						common.popup_yes_no ('param_value', rec.value, false));
      else
         show_data_row (rec.description,	htf.formHidden ('param_id', rec.id) ||
						htf.formHidden ('param_type', rec.dtype) ||
						htf.formText ('param_value', L1, L2, rec.value) || buffer);
      end if;
   end loop;

   show_data_row ('Note: All fields are required', htf.formSubmit);
   htp.tableClose;
   htp.formClose;

end get_initial_parameters;

-- **************************************************************************************************

procedure html is

cursor	driver is
select	count(1)
from	control;

icount	integer;

begin

   open driver;
   fetch driver into icount;
   close driver;

   header;
   if icount = 0 then
      get_initial_parameters;
   else
      htp.header (3, 'The initial sync process already ran.');
      htp.p ('If you need to rerun this process you must first drop all the objects in this');
      htp.p ('conference instance and rerun the create respository script.');
   end if;

   footer;

end html;

-- **************************************************************************************************

end sync_initial;
/
show error

set define on

--	Copyright (C) 2004 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>
