set define off

create or replace package body transparencies as

rpt_title$		constant char(24)	:= 'Transparency Maintenance';
prev_duration		integer := 0;
present_datetime	date;
present_time		integer;
now$			constant date := trunc (sysdate);

active_papers$		constant char(1) := '~';
all_papers$		constant char(1) := '%';

-- *******************************************************************************
-- *******************************************************************************

procedure show_buttons is

begin

   htp.para;

   htp.tableOpen;
   htp.tableRowOpen ('LEFT', 'TOP');

   htp.tableData (
	htf.formOpen ('transparencies.get_next', 'POST')	||
	htf.formSubmit (null, 'Get Next Transparency')		||
	htf.formClose);

   htp.tableData (common.space2$);

   htp.tableData (
	htf.formOpen ('transparencies.list', 'POST')		||
	htf.formSubmit (null, 'List My Active Transparencies')	||
	htf.formClose);

   htp.tableData (common.space2$);

   htp.tableData (
	htf.formOpen ('transparencies.list_all', 'POST')	||
	htf.formSubmit (null, 'List All My Transparencies')	||
	htf.formClose);

   htp.tableRowClose;
   htp.tableClose;

end show_buttons;

-- *******************************************************************************
-- *******************************************************************************

procedure show_count (icount integer, idx in out integer, totals in out common.int_arr) is

begin

   htp.tableData (icount);
   idx := idx + 1;
   totals(idx) := totals(idx) + icount;

end show_count;

-- *******************************************************************************

procedure show_trans_graph (this_date$ date default null) is

cursor	C1 is
select	distinct session_date, to_char (session_date, 'Day')	dow
from	sessions
where	presentation_option_code	in
	(	select	presentation_option_code
		from	presentation_option_codes
		where	presentation_type_code	= common.oral$)
order by 1;

cursor	C2 is
select	presentation_code, presentation_descr
from	presentation_codes
where	presentation_type_code	= common.oral$
order by sort_order nulls last, presentation_descr;

-- Total by date/presentation type

cursor	C3 (date$ date, ptype varchar2) is
select	count(1), count(trans_editor_id) assigned
from	abstracts
where	abstract_id	in
(	select	abstract_id
	from	abstract_sessions
	where	session_id	in
	(	select	session_id
		from	sessions
		where	session_date	= date$
)	)
and	presentation_code	= ptype;

-- Total by date/presentation type with files uploaded
cursor	C4 (date$ date, ptype varchar2) is
select	count(1)
from	abstracts	abs
where	abstract_id	in
(	select	abstract_id
	from	abstract_sessions
	where	session_id	in
	(	select	session_id
		from	sessions
		where	session_date	= date$
)	)
and	presentation_code	= ptype
and	exists
	(	select	1
		from	files		f
		where	f.abstract_id	= abs.abstract_id
		and	file_type_code	in
			(	select	file_type_code
				from	file_type_codes
				where	transparency_flag	= 'Y'
	)		);

cursor	C5 is
select	transparency_code, image_URL, edot_bg_color
from	transparency_codes
order by sort_order nulls last, transparency_descr;

cursor	C6 (date$ date, ptype varchar2, tcode$ varchar2) is
select	count(1)
from	abstracts
where	abstract_id	in
(	select	abstract_id
	from	abstract_sessions
	where	session_id	in
	(	select	session_id
		from	sessions
		where	session_date	= date$
)	)
and	presentation_code	= ptype
and	transparency_code	= tcode$;

type	C2_type is table of C2%ROWTYPE index by binary_integer;
pres	C2_type;

type	C5_type is table of C5%ROWTYPE index by binary_integer;
tcodes	C5_type;

icount		integer;
assigned	integer;
assignable	integer;
idx		integer;
totals		common.int_arr;
URL		varchar2(1000);
buffer1		varchar2(100);
buffer2		varchar2(100);

begin

   open C2;
   fetch C2 BULK COLLECT into pres;
   close C2;

   open C5;
   fetch C5 BULK COLLECT into tcodes;
   close C5;

-- Total number of columns (with numbers) in the table.
   for i in 1 .. (pres.COUNT * tcodes.COUNT) + (3 * pres.count) LOOP
      totals(i) := 0;
   end loop;

   htp.tableOpen ('BORDER');
   htp.tableRowOpen ('CENTER', 'BOTTOM');

   htp.tableHeader ('Day', 'LEFT', crowspan=>'2');
   for i in 1..pres.COUNT LOOP
      htp.tableHeader (pres(i).presentation_descr, ccolspan=>tcodes.COUNT+3);
   end loop;
   htp.tableRowClose;

   htp.tableRowOpen ('CENTER', 'BOTTOM');
   for i in 1..pres.COUNT LOOP
      htp.tableHeader ('Total');
      htp.tableHeader ('Assignable');
      htp.tableHeader ('Assigned');

      for j in 1..tcodes.COUNT LOOP
         htp.tableHeader (htf.img (tcodes(j).image_URL));
      end loop;
   end loop;
   htp.tableRowClose;

   for rec in C1 LOOP
      idx := 0;
      htp.tableRowOpen ('CENTER', 'TOP');

      if this_date$ = rec.session_date then
         htp.tableData (rec.dow, 'LEFT');
      else
         htp.tableData (htf.anchor ('transparencies.dow?this_date=' || rec.session_date, rec.dow), 'LEFT');
      end if;

      for i in 1..pres.COUNT LOOP
         open C4 (rec.session_date, pres(i).presentation_code);
         fetch C4 into assignable;
         close C4;

         open C3 (rec.session_date, pres(i).presentation_code);
         fetch C3 into icount, assigned;
         close C3;

         show_count (icount, idx, totals);
         show_count (assignable, idx, totals);
         show_count (assigned, idx, totals);

         for j in 1..tcodes.COUNT LOOP
            open C6 (rec.session_date, pres(i).presentation_code, tcodes(j).transparency_code);
            fetch C6 into icount;
            close C6;
            show_count (icount, idx, totals);
         end loop;

      end loop;

      htp.tableRowClose;
   end loop;

   htp.tableRowClose;

   htp.tableRowOpen ('CENTER', 'TOP');
   htp.tableHeader ('Totals', 'RIGHT');
   for i in 1..totals.COUNT LOOP
      htp.tableHeader (totals(i));
   end loop;

-- Display the pie chart

   htp.tableRowOpen ('CENTER', 'BOTTOM');
   htp.tableData (common.space$);

   idx := 3;
   for i in 1..pres.COUNT LOOP  -- for each presentation type
      if totals(idx) = 0 then
         htp.tableData (common.space$, ccolspan=>tcodes.COUNT+3);
      else
         icount	 := 0;
         URL	 := URL || 'http://chart.apis.google.com/chart?cht=p3&chs=150x75&chds=0,' || totals(idx);
         buffer1 := null;
         buffer2 := null;

         for j in 1..tcodes.COUNT LOOP  -- for each dot color
            idx	:= idx + 1;
            icount	:= icount + totals(idx);
            buffer1:= buffer1 || totals(idx) || ',';
            buffer2:= buffer2 || tcodes(j).edot_bg_color || ',';
         end loop;

         buffer1	:= '&chd=t:' || buffer1 || totals(idx-tcodes.COUNT);
         buffer2	:= '&chco='  || buffer2 || 'EEEEEE';

         htp.tableData (htf.img (url || buffer1 || buffer2), ccolspan=>tcodes.COUNT+3);
      end if;

      idx := idx + 3;  -- skip to the total amount
   end loop;

   htp.tableRowClose;
   htp.tableClose;

end show_trans_graph;

-- *******************************************************************************
-- *******************************************************************************

procedure save_status (abs_id integer, this_code varchar2, code varchar2,
			this_comment varchar2, this_note varchar2) is

cursor	driver (prof_id$ integer) is
select	-999 sort_order, email_address
from	profiles
where	profile_id = prof_id$
and	email_address is not null
UNION
select	nvl (sort_order, 999), email_address
from	profiles
where	profile_id	in
	(	select	person_id
		from	profiles
		where	profile_id = prof_id$
	)
and	email_address is not null
order by 1;

rec	driver%ROWTYPE;
msg	transparency_codes.email_msg%TYPE;
prof_id	integer;
pid	integer;

begin

   if common.header (rpt_title$) then
      update	abstracts
      set	transparency_code	= code
      where	abstract_id		= abs_id;

      insert into logs (abstract_id, log_code, transparency_code, comments, editor_notes)
      values (abs_id, common.trans_complete$, code, trim(this_comment), trim(this_note) );

      prof_id	:= common.get_speaker_profile_id (abs_id);
      pid	:= common.decode_value ('person_id', 'profiles', 'profile_id', prof_id);

      open driver (prof_id);
      fetch driver into rec;
      close driver;

      if rec.email_address is not null then
         msg := common.decode_value ('email_msg', 'transparency_codes', 'transparency_code', code);
         send_mail (rec.email_address, common.conf_title$ || ' Transparency Processing Status',
			msg || ' ' || trim (this_comment), pid=>pid, abs_id=>abs_id);
      end if;

      if this_code = active_papers$ then
         common.redirect_URL ('transparencies.list');
      else
         common.redirect_URL ('transparencies.list_all');
      end if;

   end if;

end save_status;

-- *******************************************************************************
-- *******************************************************************************

procedure set_status (abs_id integer, this_code varchar2, code varchar2) is

cursor	C1 is
select	transparency_descr, email_msg, image_URL
from	transparency_codes
where	transparency_code = code;

cursor	C2 is
select	common.get_paper_id (abstract_id) paper_id, title
from	abstracts
where	abstract_id = abs_id;

rec1	C1%ROWTYPE;
rec2	C2%ROWTYPE;

begin

   if common.header (rpt_title$) then

      if code is null then
         htp.header (3, 'You must select a Status Code', cattributes=>common.red$);
         htp.bold (common.click_back$);
      else
         open C1;
         fetch C1 into rec1;
         close C1;

         open C2;
         fetch C2 into rec2;
         close C2;

         if rec1.image_URL is null then
            htp.bold (rec1.transparency_descr);
         else
            htp.img (rec1.image_URL);
         end if;

         htp.bold (rec2.paper_id);
         htp.bold (common.mdash$);
         htp.bold (rec2.title);

         htp.formOpen ('transparencies.save_status', 'POST', cattributes=>'NAME="SPMS"');
         htp.formHidden ('abs_id', abs_id);
         htp.formHidden ('this_code', this_code);
         htp.formHidden ('code', code);
         htp.para;
         htp.tableOpen;

         htp.tableRowOpen ('LEFT', 'BOTTOM');
         htp.tableHeader ('Comments to Speaker');
         htp.tableHeader ('Editor''s Notes');
         htp.tableRowClose;

         htp.tableRowOpen ('LEFT', 'BOTTOM');
         htp.tableHeader (htf.formTextArea ('this_comment', 5, 40) );
         htp.tableHeader (htf.formTextArea ('this_note',    5, 40) );
         htp.tableRowClose;

         htp.tableClose;
         htp.para;

         if rec1.email_msg is not null then
            htp.p ('Your comments will be emailed to the speaker along with the following default message.');
            htp.italic (rec1.email_msg);
            htp.para;
         end if;

         htp.formSubmit (null, 'Save');
         htp.formClose;
         common.form_focus ('SPMS', 'this_comment');
      end if;

      common.footer;
   end if;

end set_status;

-- *******************************************************************************
-- *******************************************************************************

procedure assign (abs_id integer) is

the_code	log_codes.log_code%TYPE;

begin

   common.get_gbl_pid;
   if common.has_privs_to_page then
      if common.decode_value ('trans_editor_id', 'abstracts', 'abstract_id', abs_id) is null then
         the_code := common.trans_assigned$;
      else
         the_code := common.trans_reassigned$;
      end if;

      update	abstracts
      set	trans_editor_id = common.gbl_priv_pid
      where	abstract_id	= abs_id;

      insert into logs (abstract_id, log_code) values (abs_id, the_code);

      common.redirect_URL ('transparencies.list');
   end if;

end assign;

-- *******************************************************************************
-- *******************************************************************************

function trans_files_exist (abs_id integer) return varchar2 is

cursor	C1 is
select	max (uploaded_date)
from	files
where	abstract_id	 = abs_id
and	file_type_code	in
	(	select	file_type_code
		from	file_type_codes
		where	transparency_flag = 'Y');

cursor	C2 is
select	max (log_date)
from	logs
where	abstract_id		= abs_id
and	transparency_code	is not null;

file_date	date;
log_date	date;

begin

   open C1;
   fetch C1 into file_date;
   close C1;

   if file_date is null then
      return 'No';
   end if;

   open C2;
   fetch C2 into log_date;
   close C2;

   if log_date is null then
      return 'Yes';
   end if;

   if file_date >= log_date then
      return 'New';
   end if;

   return 'Yes';

end trans_files_exist;

-- *******************************************************************************

procedure show_dow (the_date$ date) is

cursor	C1 is
select	s.session_id, s.title, l.location_descr,
	common.get_user_name_affiliation (s.chair_profile_id) chair,
	s.beg_time, s.end_time
from	location_codes			l,
	sessions			s
where	s.session_date			= the_date$
and	s.presentation_option_code	in
	(	select	presentation_option_code
		from	presentation_option_codes
		where	presentation_type_code = common.oral$)
		
and	l.location_code			= s.location_code
order by s.sort_order nulls last,
	 s.beg_time, s.end_time, s.title;

cursor	C2 (session_id$ varchar2) is
select	abs.abstract_id, tc.image_URL, abs.title, sa.duration,
	common.get_paper_id (abs.abstract_id) paper_id,
	common.get_user_name_affiliation (
		common.get_speaker_profile_id (abs.abstract_id)) speaker,
	common.get_user_name (trans_editor_id) editor_name,
	trans_editor_id
from	transparency_codes	tc,
	abstracts		abs,
	abstract_sessions	sa
where	sa.session_id		= session_id$
and	sa.primary_flag		= 'Y'
and	abs.abstract_id		= sa.abstract_id
and	tc.transparency_code(+)	= abs.transparency_code
order by sa.session_seq;

files_exist		varchar2(100);
URL			varchar2(1000);

begin

   htp.header (2, to_char (the_date$, 'FMDay, DD-Month-YYYY') );
   htp.tableOpen ('BORDER');

   for rec in C1 LOOP
      htp.tableRowOpen ('LEFT', 'TOP');
      htp.tableHeader ( common.format_time (rec.beg_time) || '-' ||
			common.format_time (rec.end_time), ccolspan=>'5');
      htp.tableHeader (rec.title	|| htf.br ||
		common.mdash$ || 'Location: '	|| rec.location_descr || htf.br ||
		common.mdash$ || 'Chair: '	|| rec.chair, ccolspan=>'3');
      htp.tableRowClose;

      htp.tableRowOpen ('LEFT', 'BOTTOM');
      htp.tableHeader ('Time');
      htp.tableHeader ('Paper ID', CNOWRAP=>'NOWRAP');
      htp.tableHeader ('Status');
      htp.tableHeader ('Files?');
      htp.tableHeader ('Editor');
      htp.tableHeader ('Title/Speaker');
      htp.tableHeader ('Contribution Log');
      htp.tableRowClose;

      present_datetime	:= now$;
      present_datetime	:= common.add_time (present_datetime, rec.beg_time);
      prev_duration	:= 0;

      for rec2 in C2 (rec.session_id) LOOP
         present_datetime := common.add_time (present_datetime, prev_duration);
         present_time	  := to_char (present_datetime, 'hh24mi');

         htp.tableRowOpen ('LEFT', 'TOP');
         htp.tableData (common.format_time (present_time));
         htp.tableData (rec2.paper_id);

         if rec2.image_URL is null then
            htp.tableData (common.space$);
         else
            htp.tableData (htf.img (rec2.image_URL), 'CENTER');
         end if;

         files_exist	:= trans_files_exist (rec2.abstract_id);
         if files_exist = 'Yes' then
            files_exist	:= '<span ' || common.highlight$ || '>' || files_exist || '</span>';
         end if;
         htp.tableData (files_exist, 'CENTER');

         URL := null;
         if nvl (rec2.trans_editor_id, 0) <> common.gbl_priv_pid then
            if rec2.editor_name is not null then
                URL := htf.br;
            end if;
            URL := URL || htf.anchor ('transparencies.assign?abs_id=' || rec2.abstract_id, 'Assign To Me');
         end if;

         htp.tableData (rec2.editor_name || URL, cnowrap=>'NOWRAP');
         htp.tableData (rec2.title || htf.br || rec2.speaker);
         common.show_abstract_log (rec2.abstract_id, true);
         htp.tableRowClose;

         prev_duration := rec2.duration;
         common.show_data_sep (8, common.space$);
      end loop;

      htp.tableRowClose;
   end loop;

   htp.tableClose;

end show_dow;

-- *******************************************************************************
-- *******************************************************************************

procedure get_next is

cursor	driver is
select	sa.abstract_id
from	sessions		s,
	abstract_sessions	sa,
	abstracts		a
where	sa.presentation_code	in
	(	select	presentation_code
		from	presentation_codes
		where	presentation_type_code = common.oral$)
and	a.abstract_id		= sa.abstract_id
and	a.trans_editor_id	is null
and	exists
	(	select	1
		from	file_type_codes		ftc,
			files			f
		where	f.abstract_id		= a.abstract_id
		and	ftc.file_type_code	= f.file_type_code
		and	ftc.transparency_flag	= 'Y')
and	s.session_id	= sa.session_id
order by
	s.session_date, sa.session_seq;

abs_id	integer := null;

begin

   common.get_gbl_pid;
   if common.has_privs_to_page then

      open driver;
      fetch driver into abs_id;
      close driver;

      if abs_id is null then
         common.public_header (rpt_title$);
         show_trans_graph;
         show_buttons;
         htp.header (3, 'There are no Tranparencies remaining to process.',
			cattributes=>common.red$);
         common.footer;
      else
         assign (abs_id);
      end if;

   end if;

end get_next;

-- *******************************************************************************
-- *******************************************************************************

function calc_start_time (sid varchar2, sdate date,
			btime integer, abs_id integer) return varchar2 is

cursor	driver is
select	abstract_id, duration
from	abstract_sessions
where	session_id	= sid
order by
	session_seq;

cursor	C2 is
select	replace (substr (date_format_code, instr (date_format_code, ' ')+1), ':ss', null)
from	persons
where	person_id = common.gbl_priv_pid;

time_fmt_str	varchar2(30);

begin

   present_datetime := sdate;
   present_datetime := common.add_time (present_datetime, btime);

   for rec in driver LOOP
      exit when rec.abstract_id = abs_id;
      present_datetime := common.add_time (present_datetime, rec.duration);
   end loop;

   open C2;
   fetch C2 into time_fmt_str;
   close C2;

   time_fmt_str := nvl (time_fmt_str, 'HH24:MI');
   return to_char (present_datetime, 'FMDay, ') || to_char (present_datetime, time_fmt_str);

end calc_start_time;

-- *******************************************************************************

procedure list2 (this_code varchar2 default null) is

cursor	driver is
select	abs.abstract_id, abs.transparency_code, sa.paper_id,
	tc.image_URL, s.session_id, s.session_date, s.beg_time
from	transparency_codes		tc,
	sessions			s,
	abstract_sessions		sa,
	abstracts			abs
where	abs.trans_editor_id		= common.gbl_priv_pid
and	nvl(abs.transparency_code,'~')	like this_code
and	sa.abstract_id			= abs.abstract_id
and	sa.primary_flag			= 'Y'
and	s.session_id			= sa.session_id
and	tc.transparency_code(+)		= abs.transparency_code
order by
	s.session_date, sa.session_seq;

type	rec_type is table of driver%ROWTYPE index by binary_integer;
recs	rec_type;
msg	varchar2(7)	:= 'active ';

begin

   if common.header (rpt_title$) then
      open driver;
      fetch driver BULK COLLECT into recs;
      close driver;

      show_trans_graph;
      show_buttons;

      if recs.COUNT = 0 then
         if this_code = all_papers$ then
            msg := null;
         end if;
         htp.header (3, 'There are no ' || msg || 'transparencies assigned to you.',
			cattributes=>common.red$);
      else
         htp.tableOpen ('BORDER');
         htp.tableRowOpen ('LEFT', 'TOP');
         htp.tableHeader ('Paper ID');
         htp.tableHeader ('Time');

         if this_code <> active_papers$ then
            htp.tableHeader ('Status');
         end if;

         htp.tableHeader ('Set Status');
         htp.tableHeader ('Upload');
         htp.tableHeader ('Download');
         htp.tableHeader ('Log');
         htp.tableRowClose;
      end if;

      for i in 1..recs.COUNT LOOP
         htp.tableRowOpen ('LEFT', 'TOP');
         htp.tableData (recs(i).paper_id, 'LEFT');
         htp.tableData (calc_start_time (recs(i).session_id, recs(i).session_date,
					 recs(i).beg_time, recs(i).abstract_id));

         if this_code <> active_papers$ then
            if recs(i).image_URL is null then
               htp.tableData (common.space$);
            else
               htp.tableData (htf.img (recs(i).image_URL), 'CENTER');
            end if;
         end if;

         htp.p ('<TD>');
            htp.formOpen ('transparencies.set_status', 'POST');
            htp.formHidden ('abs_id', recs(i).abstract_id);
            htp.formHidden ('this_code', this_code);
            common.status_code_popup (recs(i).transparency_code, '%', '%', 'Y');
            htp.br;
            htp.formSubmit (null, 'Set Status');
            htp.formClose;
         htp.p ('</TD>');

         if common.author_upload_script$ is null then
            htp.tableData ('No Script', CNOWRAP=>'NOWRAP');
         elsif recs(i).paper_id is null then
            htp.tableData ('No Paper ID', CNOWRAP=>'NOWRAP');
         else
            htp.tableData (htf.anchor2 ('JACoW.file_info'	||
						'?abs_id='	|| recs(i).abstract_id	||
						'&paper_id='	|| recs(i).paper_id,
				'Upload', ctarget=>'_blank') );
         end if;

         if common.editor_download_script$ is null then
            htp.tableData ('No Script', CNOWRAP=>'NOWRAP');
         elsif recs(i).paper_id is null then
            htp.tableData ('No Paper ID', CNOWRAP=>'NOWRAP');
         else
            htp.tableData (common.paper_anchor_tag (recs(i).paper_id));
         end if;

         common.show_abstract_log (recs(i).abstract_id, true);

         htp.tableRowClose;
      end loop;

      if recs.COUNT > 0 then
         htp.tableClose;
      end if;

      common.footer;
   end if;

end list2;

-- *******************************************************************************
-- *******************************************************************************

procedure list is

begin

   list2 (active_papers$);

end list;

-- *******************************************************************************
-- *******************************************************************************

procedure list_all is

begin

   list2 (all_papers$);

end list_all;

-- *******************************************************************************
-- *******************************************************************************

procedure dow (this_date varchar2) is

the_date$	constant date := to_date (this_date);

begin

   if common.header (rpt_title$) then
      htp.p (profile.js$);
      show_trans_graph (the_date$);
      show_buttons;
      show_dow (the_date$);
      common.footer;
   end if;

end dow;

-- *******************************************************************************
-- *******************************************************************************

procedure display is

cursor	driver is
select	a.*, s.session_date, s.beg_time, sa.paper_id, sa.start_time,
	common.get_author_profile_id (a.abstract_id, common.speaker$) spk_profile_id,
	(	select	DISTINCT 1
		from	files		f
		where	f.abstract_id	= a.abstract_id
		and	file_type_code	in
			(	select	file_type_code
				from	file_type_codes
				where	transparency_flag	= 'Y'
	)		) files_exist
from	sessions		s,
	abstracts		a,
	abstract_sessions	sa
where	sa.presentation_code	in
(	select	presentation_code
	from	presentation_codes
	where	presentation_type_code = common.oral$)
and	a.abstract_id		= sa.abstract_id
and	s.session_id		= sa.session_id
order by
	s.sort_order nulls last, s.session_date, s.beg_time, s.session_id, sa.session_seq;

download_URL	varchar2(4000);
email_URL	varchar2(4000);
can_email	boolean;

begin

   if common.header ('Contributions With Transparencies') then
      can_email := common.has_privs_to_page ('email.one_email', false);
      htp.tableOpen ('BORDER', cattributes=>'CELLSPACING="3"');

      htp.tableRowOpen ('LEFT', 'BOTTOM');
      htp.tableHeader ('Paper ID');
      htp.tableHeader ('When');
      htp.tableHeader ('Contribution Title');
      htp.tableHeader ('Speaker');
      htp.tableRowClose;

      for rec in driver LOOP

         if rec.files_exist = 1 then
            download_URL := common.paper_anchor_tag (rec.paper_id);
         else
            download_URL := 'No Files';
         end if;

         email_URL := null;
         if can_email then
            email_URL := common.space2$ || htf.anchor2 ('email.one_email?abs_id=' || rec.abstract_id, 'Email', ctarget=>'_new');
         end if;

         htp.tableRowOpen ('LEFT', 'TOP');
         htp.tableData (
		htf.anchor2 ('search.abs_id?aid=' || rec.abstract_id, rec.paper_id, ctarget=>'_blank')	|| htf.br ||
		download_URL || common.space2$ ||
		htf.anchor2 ('JACoW.file_info'	||
						'?abs_id='	|| rec.abstract_id	||
						'&paper_id='	|| rec.paper_id,
				'Upload', ctarget=>'_blank')	|| email_URL, cnowrap=>'NOWRAP');

         htp.tableData (to_char (rec.session_date, 'Day') || ' ' || common.format_time (rec.start_time), cnowrap=>'NOWRAP' );
         htp.tableData (rec.title);
         htp.tableData (common.get_user_name_affiliation (rec.spk_profile_id));
         htp.tableRowClose;
      end loop;

      htp.tableClose;
      common.footer;
   end if;

end display;

-- ************************************************************************************************
-- ************************************************************************************************

procedure html is

begin

   list2 (active_papers$);

end html;

-- *******************************************************************************
-- *******************************************************************************

end transparencies;
/
show error

--	Copyright (C) 2004 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>

set define off
