set define off

create or replace package body visa as

rpt_title$	constant varchar2(50) := 'VISA Invitation';
VISA$		constant activity_codes.activity_code%TYPE := 'VISA';

-- **********************************************************************************
-- **********************************************************************************

procedure put (buffer varchar2, newLine boolean default false) is

begin

   htp.prn (buffer);
   if newLine then
      htp.prn (common.crlf$);
   else
      htp.prn (common.tab$);
   end if;

end put;

-- **********************************************************************************

procedure header is

begin

   htp.prn (common.crlf$);
   put ('Name');
   put ('Affiliation');
   put ('Address');
   put ('Email');
   put ('PO Box');
   put ('Fax');
   put ('Phone', true);

end header;

-- **********************************************************************************

function first_letter (pid integer) return boolean is

cursor	driver is
select	1
from	activities
where	person_id	= pid
and	activity_code	= VISA$;

dummy		integer;
rec_found	boolean;

begin

   open driver;
   fetch driver into dummy;
   rec_found := driver%FOUND;
   close driver;

   return NOT rec_found;

end first_letter;

-- **********************************************************************************

procedure letters (extract_all boolean) is

cursor	driver is
select	p.person_id, pro.email_address, pro.po_box, pro.fax_number, pro.phone_number,
	common.get_user_name (p.person_id)			name,
	common.get_affil_name (addr.affiliation_id, ' ', 'Y')	affiliation,
	common.get_address (addr.address_id, ', ')		address
from	addresses		addr,
	profiles		pro,
	persons			p
where	p.requires_invitation	= 'Y'
and	pro.person_id		= p.person_id
and	addr.address_id		= pro.address_id
order by
	p.person_id, pro.current_address nulls last;

prev_id	integer := 0;

begin

   common.get_gbl_pid;
   owa_util.mime_header('application/vnd.ms-excel', TRUE);
   header;
   for driver_rec in driver LOOP
      if (extract_all and prev_id <> driver_rec.person_id) OR
		first_letter (driver_rec.person_id) then

         common.gbl_pid := driver_rec.person_id;
         profile.log_activity (VISA$);

         put (driver_rec.name);
         put (driver_rec.affiliation);
         put (driver_rec.address);
         put (driver_rec.email_address);
         put (driver_rec.po_box);
         put (driver_rec.fax_number);
         put (driver_rec.phone_number, true);
      end if;

      prev_id := driver_rec.person_id;
   end loop;

end letters;

-- **********************************************************************************
-- **********************************************************************************

procedure all_letters is

begin

   letters (true);

end all_letters;

-- **********************************************************************************
-- **********************************************************************************

procedure new_letters is

begin

   letters (false);

end new_letters;

-- **********************************************************************************
-- **********************************************************************************

procedure html is

begin

   if common.header (rpt_title$) then
      htp.anchor ('visa.all_letters', 'Extract All VISA letter requests.');
      htp.para;
      htp.anchor ('visa.new_letters', 'Extract New VISA letter requests.');
      common.footer;
   end if;

end html;

-- *********************************************************************************

end visa;
/
show error

--	Copyright (C) 2004 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>

set define on
