create or replace package body validate_spms is

procedure person_data is

cursor	C1 is
select	person_id, lname, user_id
from	repo_persons
order by person_id;

cursor	C2 is
select	person_id, lname, user_id
from	persons
order by person_id;

type		person_type	is table of C2%ROWTYPE index by binary_integer;
rpersons	person_type;
lpersons	person_type;
persons		person_type;
j		integer;

begin

   open C1;
   fetch C1 bulk collect into rpersons;
   close C1;

   open C2;
   fetch C2 bulk collect into lpersons;
   close C2;

   for i in 1..lpersons.COUNT loop
      j := lpersons(i).person_id;
      persons(j) := lpersons(i);
   end loop;

   dbms_output.put_line ('Validating Person Records');
   dbms_output.put_line (lpersons.count || ' ' || rpersons.count);

   for i in 1..rpersons.count LOOP
      j := rpersons(i).person_id;
      if NOT persons.exists(j) then
         dbms_output.put_line ('Missing ' || rpersons(i).person_id);
      elsif persons(j).lname <> rpersons(i).lname then
         dbms_output.put_line (j || ' Lname');
      end if;
   end loop;

-- **********

   persons.delete;
   for i in 1..rpersons.COUNT loop
      j := rpersons(i).person_id;
      persons(j) := rpersons(i);
   end loop;

   for i in 1..lpersons.count LOOP
      j := lpersons(i).person_id;
      if NOT persons.exists(j) then
         dbms_output.put_line ('Not in REPO ' || lpersons(i).person_id);
      elsif persons(j).lname <> lpersons(i).lname then
         dbms_output.put_line (j || ' Lname');
      end if;
   end loop;

end person_data;

-- ************************************************************************************

procedure profile_data is

cursor	C1 is
select	profile_id, email_address, address_id
from	repo_profiles
order by profile_id;

cursor	C2 is
select	profile_id, email_address, address_id
from	profiles
order by profile_id;

type		profile_type	is table of C2%ROWTYPE index by binary_integer;
rprofiles	profile_type;
lprofiles	profile_type;
profiles	profile_type;
j		integer;

found		boolean;

begin

   open C1;
   fetch C1 bulk collect into rprofiles;
   close C1;

   open C2;
   fetch C2 bulk collect into lprofiles;
   close C2;

   for i in 1..lprofiles.COUNT loop
      j := lprofiles(i).profile_id;
      profiles(j) := lprofiles(i);
   end loop;

   dbms_output.put_line ('Validating Profile Records');
   dbms_output.put_line (lprofiles.count || ' ' || rprofiles.count);

   for i in 1..rprofiles.count LOOP
      j := rprofiles(i).profile_id;
      if NOT profiles.exists(j) then
         dbms_output.put_line ('Missing ' || rprofiles(i).profile_id);
      elsif nvl (rprofiles(i).email_address, '~') <>
	    nvl (profiles(j).email_address, '~') then
         dbms_output.put_line (j || ' email address');
      elsif rprofiles(i).address_id <> profiles(j).address_id then
         dbms_output.put_line (j || ' address ID');
      end if;
   end loop;

-- **********

   profiles.delete;
   for i in 1..rprofiles.COUNT loop
      j := rprofiles(i).profile_id;
      profiles(j) := rprofiles(i);
   end loop;

   for i in 1..lprofiles.count LOOP
      j := lprofiles(i).profile_id;
      if NOT profiles.exists(j) then
         dbms_output.put_line ('Not in REPO ' || lprofiles(i).profile_id);
      elsif nvl (lprofiles(i).email_address, '~') <>
	    nvl (profiles(j).email_address, '~') then
         dbms_output.put_line (j || ' email address');
      elsif lprofiles(i).address_id <> profiles(j).address_id then
         dbms_output.put_line (j || ' address ID');
      end if;
   end loop;

end profile_data;

end validate_SPMS;
/
show error
