set define off

create or replace package body vendor as

rpt_title$	constant varchar2(20) := 'Vendor Registration';

-- ********************************************************************************************
-- ********************************************************************************************

procedure rpt is

cursor	driver is
select	common.get_user_affil_list (d.vendor_id) vendor_name,
	common.get_user_name (d.vendor_id, 'N') delegate_name,
	r.function_descr
from	function_codes		r,
	vendor_delegates	d
where	r.function_code		= d.function_code
order by 1, 2;

prev_vname	varchar2(500) := '~';

begin

   if common.header (rpt_title$) then
      htp.tableOpen ('BORDER');

      htp.tableRowOpen ('LEFT', 'BOTTOM');
      htp.tableHeader ('Vendor');
      htp.tableHeader ('Delegate');
      htp.tableHeader ('Role');
      htp.tableRowClose;

      for driver_rec in driver LOOP
         htp.tableRowOpen ('LEFT', 'TOP');

         if prev_vname <> driver_rec.vendor_name then
            htp.tableData (driver_rec.vendor_name);
         else
            htp.tableData (common.space$);
         end if;

         htp.tableData (driver_rec.delegate_name,	cnowrap=>'NOWRAP');
         htp.tableData (driver_rec.function_descr,	cnowrap=>'NOWRAP');

         htp.tableRowClose;
         prev_vname := driver_rec.vendor_name;
      end loop;

      htp.tableClose;
      common.footer;
   end if;

end rpt;

-- ********************************************************************************************
-- ********************************************************************************************

procedure del (pid integer, vid integer, code varchar2) is

begin

   delete from vendor_delegates
   where person_id	= pid
   and   vendor_id	= vid
   and	 function_code	= code;

   common.redirect_URL ('vendor.html');

end del;

-- ********************************************************************************************
-- ********************************************************************************************

procedure new2 (UID varchar2, this_function varchar2) is

dup_role	exception;
pragma		exception_init (dup_role, -1);

vid		constant integer := common.decode_value ('person_id', 'persons',
					'user_id', upper (UID) );

begin

   if common.header (rpt_title$) then
      if vid is not null then
         insert into vendor_delegates
			(person_id, vendor_id, function_code)
		values	(common.gbl_pid, vid, this_function);
         common.redirect_URL ('vendor.html');

      else
         common.public_header (rpt_title$);
         htp.header (3, 'No Such User');
         htp.p ('There is no user in the database with that user ID.');
         htp.p (common.click_back$);
         htp.para;
         common.footer;
      end if;
   end if;

exception when dup_role then
      common.redirect_URL ('vendor.html');

end new2;

-- ********************************************************************************************
-- ********************************************************************************************

procedure new is

begin

   if common.header (rpt_title$) then
      htp.p (profile.js$);
      htp.header (3, 'Add a New Associate');

      htp.anchor ('javascript:newWindow(''profile.find_account'')', 'Search for an Account');

      htp.formOpen ('vendor.new2', 'POST', cattributes=>'NAME="SPMS"');
      htp.tableOpen;

      common.show_data_row ('Enter a User ID', htf.formText ('uid', 30, 30) );
      common.show_lookup_table ('function', 'Role', where_claws=>'where VENDOR_FLAG=''Y'''); 
      common.show_data_row (null, htf.formSubmit (null, 'Add User') );

      htp.tableClose;
      htp.formClose;
      common.form_focus ('SPMS', 'uid');

      htp.para;
      common.footer;
   end if;

end new;

-- ********************************************************************************************
-- ********************************************************************************************

procedure html is

cursor	driver is
select	v.vendor_id, fr.function_code, fr.function_descr
from	persons			p,
	vendor_delegates	v,
	function_codes		fr
where	v.function_code		= fr.function_code
and	v.person_id		= common.gbl_pid
and	v.vendor_id		= p.person_id
order by
	p.sort_lname, p.sort_fname, p.sort_mname, fr.function_descr;

begin

   if common.header (rpt_title$) then
      htp.tableOpen ('BORDER', cattributes=>'CELLPADDING="3"');

      htp.tableRowOpen ('LEFT', 'BOTTOM');
      htp.tableData (htf.anchor ('vendor.new', 'New') );
      htp.tableHeader ('Contact');
      htp.tableHeader ('Name');
      htp.tableRowClose;

      for driver_rec in driver LOOP
         htp.tableRowOpen ('LEFT', 'BOTTOM');
         htp.tableData (htf.anchor ('vendor.del?pid='	|| common.gbl_pid		||
					'&vid='		|| driver_rec.vendor_id	||
					'&code='	|| driver_rec.function_code, 'Remove') );
         htp.tableData (driver_rec.function_descr);
         htp.tableData (common.get_user_name (driver_rec.vendor_id, 'Y') );
         htp.tableRowClose;
      end loop;

      htp.tableClose;
      htp.para;
      common.footer;
   end if;

end html;

-- ********************************************************************************************
-- ********************************************************************************************

end vendor;
/
show error

set define on

--	Copyright (C) 2004 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>
