set define off

create or replace package body XML2 as

attrib_name	common.ident_arr;
attrib_val	common.ident_arr;
full_extract	boolean := false;

-- **********************************************************************************
-- **********************************************************************************

procedure push_xml_data (conf_id number, tname varchar2, sql_stmt varchar2) is

xml_data	clob;
xml_array	repo_xml_pkg.xml_data_type;
i		integer := 0;

begin

   xml_data := sys.dbms_xmlgen.getxml(sql_stmt);

   LOOP
      exit when xml_data is null;
      exit when i > 10000;
      i := i + 1;

      xml_array(i) := substr(xml_data,1,5000);
      xml_data := substr(xml_data,5001);
   end loop;

   repo_xml_pkg.load_xml_data (conf_id, tname, xml_array);

end push_xml_data;

-- **********************************************************************************
-- **********************************************************************************

procedure print_xml_fom_sql (sql_stmt varchar2, label varchar2) is

string		varchar2(4000);
ipos		integer;
offset		integer;
n		integer := 2;

qryctx		dbms_xmlgen.ctxhandle;
result		clob;
outer_tag	varchar2(100)	:= upper (label) || '_GROUP';
inner_tag	varchar2(100)	:= upper (label) || '_ITEM';

begin

   qryctx := dbms_xmlgen.newcontext (sql_stmt);
   dbms_xmlgen.setRowSetTag (qryctx, outer_tag);
   dbms_xmlgen.setRowTag (qryctx, inner_tag);
   result := dbms_xmlgen.getxml (qryctx);
   dbms_xmlgen.closecontext (qryctx);

-- Skip the 1st tag.  It is an <?xml > tag that is not necessary
   ipos		:= dbms_lob.instr (result, chr(10), 1, 1);
   offset	:= ipos + 1;

   LOOP
      exit when result is null;
      ipos := dbms_lob.instr (result, chr(10), 1, n);
      exit when ipos = 0;

      string := dbms_lob.substr (result, ipos-offset, offset);
      htp.p (string);

      offset	:= ipos + 1;
      n		:= n + 1;
   end LOOP;

   if result is null then
      htp.p ('<'  || outer_tag || '>');
      htp.p ('</' || outer_tag  || '>');
   end if;

end print_xml_fom_sql;

-- *******************************************************************************
-- *******************************************************************************

procedure header is

begin

   owa_util.mime_header('text/xml', true);
   htp.p ('<?xml version="1.0" encoding="iso-8859-1" ?>');
   htp.p ('<conference xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance' ||
		'" xsi:noNamespaceSchemaLocation="http://bel.gsi.de/docs/xml/conference.xsd' ||
		'" name="' || common.conf_title$ || ', ' || common.conf_location$ || '">');

end header;

-- **********************************************************************************

procedure footer is

begin

   htp.p ('</conference>');

end footer;

-- **********************************************************************************
-- **********************************************************************************

procedure attrib (name  varchar2, value varchar2) is

begin

   attrib_name(attrib_name.count+1)	:= name;
   attrib_val(attrib_val.count+1)	:= htf.escape_sc (value);

end attrib;

-- **********************************************************************************

procedure tag (name varchar2, value varchar2) is

begin

   if value is not null then
      htp.prn ('<');
      htp.prn (name);

      for i in 1..attrib_name.count LOOP
         if attrib_val(i) is not null then
            htp.prn (' ');
            htp.prn (attrib_name(i));
            htp.prn ('="');
            htp.prn (attrib_val(i));
            htp.prn ('"');
         end if;
      end loop;

      htp.prn ('>');

      htp.prn (htf.escape_sc (value));

      htp.prn ('</');
      htp.prn (name);
      htp.p   ('>');
   end if;

   attrib_name.delete;
   attrib_val.delete;

end tag;

-- **********************************************************************************
-- **********************************************************************************

procedure files (abs_id integer) is

cursor	driver is
select	*
from	files
where	abstract_id	= abs_id
and	deleted		is null
order by uploaded_date;

first		boolean := true;
platform_descr	platform_codes.platform_descr%TYPE;
file_type_descr	file_type_codes.file_type_descr%TYPE;

begin

   for rec in driver LOOP
      if first then
         htp.p ('<files>');
         first := false;
      end if;

      htp.p ('<file>');

      platform_descr	:= common.decode_value ('platform_descr', 'platform_codes',
						'platform_code', rec.platform_code);
      file_type_descr	:= common.decode_value ('file_type_descr', 'file_type_codes',
							'file_type_code', rec.file_type_code);

      attrib ('abbrev', rec.platform_code);
      tag ('platform',	platform_descr);

      attrib ('abbrev',		rec.file_type_code);
      tag ('file_type',		file_type_descr);

      tag ('name', rec.file_name);

      htp.p ('</file>');
   end loop;

   if not first then
      htp.p ('</files>');
   end if;

end files;

-- **********************************************************************************
-- **********************************************************************************

procedure keywords (abs_id integer) is

cursor	driver is
select	c.*
from	keyword_codes		c,
	abstract_keywords	k
where	k.abstract_id		= abs_id
and	c.keyword_code		= k.keyword_code
order by
	c.keyword_descr;

first	boolean := true;

begin

   for rec in driver LOOP
      if first then
         htp.p ('<keywords>');
      end if;

      tag ('keyword', rec.keyword_descr);
      first := false;
   end loop;

   if not first then
      htp.p ('</keywords>');
   end if;

end keywords;

-- **********************************************************************************
-- **********************************************************************************

procedure emails (abs_id integer, pid integer) is

cursor	driver is
select	distinct email_address
from	profiles		prof,
	authors			auth
where	auth.abstract_id	= abs_id
and	prof.profile_id		= auth.profile_id
and	prof.person_id		= pid;

begin

   htp.p ('<emails>');
   for rec in driver LOOP
      tag ('email', rec.email_address);
   end loop;
   htp.p ('</emails>');

end emails;

-- **********************************************************************************

procedure affiliation (prof_id integer) is

cursor	driver is
select	affil.*, addr.address1, addr.address2, addr.address3,
	addr.po_box, addr.town, addr.postal_code, addr.zip_code, addr.country_code
from	affiliations		affil,
	addresses		addr,
	profiles		prof
where	profile_id		= prof_id
and	addr.address_id		= prof.address_id
and	affil.affiliation_id	= addr.affiliation_id;

rec			driver%ROWTYPE;
country_descr		country_codes.country_descr%TYPE;
affil_type_descr	affil_type_codes.affil_type_descr%TYPE;

begin

   open driver;
   fetch driver into rec;
   close driver;

   country_descr	:= common.decode_value ('country_descr', 'country_codes',
					'country_code', rec.country_code);
   affil_type_descr	:= common.decode_value ('affil_type_descr', 'affil_type_codes',
					'affil_type_code', rec.affil_type_code);

   htp.p ('<institute>');
   attrib ('abbrev',	rec.affil_abbrev);
   attrib ('type',	affil_type_descr);
   tag ('full_name',	common.get_affil_name (rec.affiliation_id, incl_abbrev=>'N'));

   tag ('name1',	rec.name1);
   tag ('name2',	rec.name2);
   tag ('department',	rec.department);
   tag ('URL',		rec.web_site);

   tag ('address1',	rec.address1);
   tag ('address2',	rec.address2);
   tag ('address3',	rec.address3);
   tag ('po_box',	rec.po_box);

   tag ('town',		rec.town);
   tag ('postal_code',	rec.postal_code);
   tag ('zip_code',	rec.zip_code);

   attrib ('abbrev',	rec.country_code);
   tag ('country_code',	country_descr);

   htp.p ('</institute>');

end affiliation;

-- **********************************************************************************
-- **********************************************************************************

procedure institutions (abs_id integer, pid integer, acode varchar2) is

cursor	driver is
select	profile_id
from	authors
where	abstract_id		= abs_id
and	author_code		= acode
and	profile_id		in
(	select	profile_id
	from	profiles
	where	person_id	= pid);
begin

--   htp.p ('<institutions>');
   htp.p ('<institutions ckey="' || acode || '">');
   for rec in driver LOOP
      affiliation (rec.profile_id);
   end loop;
   htp.p ('</institutions>');

end institutions;

-- **********************************************************************************

procedure name (lname varchar2, fname varchar2, mname varchar2, iname varchar2, pid integer) is

begin

   tag ('lname', lname);
   tag ('fname', fname);
   tag ('mname', mname);
   tag ('iname', iname);
   tag ('author_id', pid);

end name;

-- **********************************************************************************

procedure institutions (pid integer) is

cursor	driver is
select	profile_id
from	profiles
where	person_id = pid;
begin

   htp.p ('<institutions>');
   for rec in driver LOOP
      affiliation (rec.profile_id);
   end loop;
   htp.p ('</institutions>');

end institutions;

-- **********************************************************************************

procedure contributors (abs_id integer) is

cursor	driver is
select	distinct ctype, lname, fname, mname, iname, person_id, author_code
from	abstract_authors
where	abstract_id = abs_id
order by
	ctype, lname, fname, mname;

begin

   htp.p ('<contributors>');
   for rec in driver LOOP
      htp.p ('<contributor type="' || rec.ctype || '">');
      name (rec.lname, rec.fname, rec.mname, rec.iname, rec.person_id);

      if rec.ctype = 'Owner' then
         institutions (rec.person_id);
      else
         institutions (abs_id, rec.person_id, rec.author_code);
      end if;
      emails (abs_id, rec.person_id);
      htp.p ('</contributor>');
   end loop;
   htp.p ('</contributors>');

end contributors;

-- **********************************************************************************
-- **********************************************************************************

procedure emails (pid integer) is

cursor	driver is
select	email_address
from	profiles
where	person_id = pid;

begin

   htp.p ('<emails>');
   for rec in driver LOOP
      tag ('email', rec.email_address);
   end loop;
   htp.p ('</emails>');

end emails;

-- **********************************************************************************

procedure coordinators (code varchar2) is

cursor	driver is
select	person_id, lname, fname, mname, initials
from	persons
where	person_id	in
(
	select	
	person_id
	from	experts
	where	classification_code	= code
	and 	coordinator		= 'Y'
)
order by
	sort_lname, sort_fname, sort_mname;

rec_found	boolean := false;

begin

   for rec in driver LOOP
      if NOT rec_found then
         htp.p ('<coordinators>');
         rec_found := true;
      end if;

      htp.p ('<coordinator>');
      name (rec.lname, rec.fname, rec.mname, rec.initials, rec.person_id);
      institutions (rec.person_id);
      emails (rec.person_id);
      htp.p ('</coordinator>');
   end loop;

   if rec_found then
      htp.p ('</coordinators>');
   end if;

end coordinators;

-- **********************************************************************************

procedure paper_URLs (abs_id integer, paper_id varchar2) is

cursor	driver is
select	f.*, ft.file_type_descr
from	file_type_codes	ft,
	files			f
where	f.abstract_id		= abs_id
and	f.deleted		is null
and	ft.file_type_code	= f.file_type_code;

begin

   htp.p ('<fileURLs>');
   for rec in driver LOOP
      attrib ('type',		rec.file_type_descr);
      tag ('fileURL',		common.editor_download_script$		||
					'?paper_id='	|| paper_id	||
					'&wanted_file='	|| rec.file_name);
   end loop;
   htp.p ('</fileURLs>');

end paper_URLs;

-- **********************************************************************************

procedure final_qa_person (abs_id integer) is

cursor	driver is
select	/*+ INDEX(l) */
	common.get_user_name (person_id) name
from	logs		l
where	abstract_id	= abs_id
and	log_code	in (common.final_QA_passed$, common.final_QA_failed$)
order by
	log_date desc;

rec	driver%ROWTYPE;

begin

   open driver;
   fetch driver into rec;
   close driver;

   tag ('final_qa', rec.name);

end final_qa_person;

-- **********************************************************************************

procedure abstract_log (abs_id integer) is

cursor	driver is
select	/*+ INDEX(l) */
	l.*, lc.log_descr,
	common.get_user_name (person_id) name,
	common.decode_value ('status_descr', 'status_codes', 'status_code', l.status_code) editor_status,
	common.decode_value ('referee_descr', 'referee_codes', 'referee_code', l.referee_code) referee_status,
	common.decode_value ('transparency_descr', 'transparency_codes', 'transparency_code', l.transparency_code) transparency_status
from	log_codes	lc,
	logs		l
where	l.abstract_id	= abs_id
and	lc.log_code	= l.log_code
order by log_date;

rec	driver%ROWTYPE;

begin

   htp.p ('<abstract_logs>');
   for rec in driver LOOP
      htp.p ('<abstract_log>');

      attrib ('code', rec.log_code);
      tag ('log_status', rec.log_descr);

      tag ('timestamp', to_char (rec.log_date, 'dd-Mon-yyyy hh24:mi:ss'));

      attrib ('code', rec.status_code);
      tag ('editor_status', rec.editor_status);

      attrib ('code', rec.referee_code);
      tag ('referee_status', rec.referee_status);

      attrib ('code', rec.transparency_code);
      tag ('transparency_status', rec.transparency_status);

      tag ('comments', rec.comments);
      tag ('editor_notes', rec.editor_notes);

      htp.p ('</abstract_log>');
   end loop;
   htp.p ('</abstract_logs>');

end abstract_log;

-- **********************************************************************************
-- **********************************************************************************

procedure program_codes (abs_id integer) is

cursor	driver is
select	*
from	abstract_sessions
where	abstract_id = abs_id;

presentation	presentation_codes.presentation_descr%TYPE;
present_tcode	presentation_type_codes.presentation_type_code%TYPE;
present_type	presentation_type_codes.presentation_type_descr%TYPE;

begin

   htp.p ('<program_codes>');
   for rec in driver LOOP
      presentation	:= common.decode_value ('presentation_descr', 'presentation_codes',
						'presentation_code', rec.presentation_code);
      present_tcode	:= common.decode_value ('presentation_type_code', 'presentation_codes',
						'presentation_code', rec.presentation_code);
      present_type	:= common.decode_value ('presentation_type_descr', 'presentation_type_codes',
						'presentation_type_code', present_tcode);

      htp.p ('<program_code>');

      attrib ('primary',	rec.primary_flag);
      tag ('code',		rec.paper_id);

      attrib ('type',		present_type);
      tag ('presentation',	presentation);

      tag ('start_time',	rec.start_time);
      tag ('duration',		rec.duration);

      attrib ('type',	'Streaming Video');
      tag ('fileURL',	rec.streaming_video_URL);

      htp.p ('</program_code>');
   end loop;
   htp.p ('</program_codes>');

end program_codes;

-- **********************************************************************************

procedure one_paper (rec abstracts%ROWTYPE) is

class_descr	classification_codes.classification_descr%TYPE;
sub_class_descr	sub_class_codes.sub_class_descr%TYPE;

status_descr	status_codes.status_descr%TYPE;

begin

   htp.p ('<paper>');
   coordinators (rec.classification_code);
   tag ('abstract_id', rec.abstract_id);

   if full_extract then
      class_descr	:= common.decode_value ('classification_descr', 'classification_codes',
					'classification_code', rec.classification_code);
      sub_class_descr:= common.decode_value ('sub_class_descr', 'sub_class_codes',
					'sub_class_code', rec.sub_class_code);
      status_descr	:= common.decode_value ('status_descr', 'status_codes',
					'status_code', rec.status_code);

      tag ('pages',		rec.total_pages);
      tag ('toc',		rec.toc_page_num);
      tag ('main_class',	class_descr);
      tag ('sub_class',		sub_class_descr);
      tag ('publishable',	common.paper_OK_to_publish (rec.abstract_id));

      program_codes (rec.abstract_id);

      tag ('dot',		status_descr);
      tag ('title',		rec.title);
      tag ('abstract',		rec.description);
      tag ('agency',		rec.agency);
      tag ('footnote',		rec.footnote);

      contributors (rec.abstract_id);
      keywords (rec.abstract_id);
      files (rec.abstract_id);

      paper_URLs (rec.abstract_id, common.get_paper_id (rec.abstract_id, 'Y'));

      tag ('editor', common.get_user_name (rec.editor_id));
      final_qa_person (rec.abstract_id);
      abstract_log (rec.abstract_id);

   end if;

   htp.p ('</paper>');

end one_paper;

-- **********************************************************************************

procedure paper (AID integer) is

cursor	driver is
select	*
from	abstracts
where	abstract_id = AID;

rec	driver%ROWTYPE;

begin

   open driver;
   fetch driver into rec;
   close driver;

   full_extract := true;
   header;
   one_paper (rec);
   footer;

end paper;

-- **********************************************************************************

procedure papers (SID varchar2 default null) is

cursor	driver is
select	a.*
from	abstracts		a,
	abstract_sessions	sa
where (	sa.session_id		= SID OR SID is null)
and	a.abstract_id		= sa.abstract_id
order by
	sa.session_id, sa.session_seq, sa.abstract_id;

begin

   htp.p ('<papers>');
   for rec in driver loop
      one_paper (rec);
   end loop;
   htp.p ('</papers>');

end papers;

-- **********************************************************************************

procedure all_papers is

cursor	driver is
select	a.*
from	abstracts		a,
	abstract_sessions	sa
order by
	sa.session_id, sa.session_seq, sa.abstract_id;

begin

   full_extract := true;
   header;

   htp.p ('<papers>');
   for rec in driver loop
      one_paper (rec);
   end loop;
   htp.p ('</papers>');

   footer;

end all_papers;

-- **********************************************************************************
-- **********************************************************************************

procedure session_location (ID$ varchar2) is

cursor	driver is
select	pc.presentation_option_descr, s.title, lc.location_descr
from	presentation_option_codes	pc,
	location_codes			lc,
	sessions			s
where	s.session_id			= ID$
and	lc.location_code		= s.location_code
and	pc.presentation_option_code	= s.presentation_option_code;

rec	driver%ROWTYPE;

begin

   open driver;
   fetch driver into rec;
   close driver;

   attrib ('type', rec.presentation_option_descr);
   tag ('location', rec.location_descr);

end session_location;

-- **********************************************************************************

procedure one_session (rec driver%ROWTYPE) is

cursor	chair (prof_id integer) is
select	*
from	persons
where	person_id =
(	select	person_id
	from	profiles
	where	profile_id = prof_id);

cursor	C2 is
select	presentation_type_descr
from	presentation_type_codes
where	presentation_type_code =
	(	select	presentation_type_code
		from	presentation_option_codes
		where	presentation_option_code = rec.presentation_option_code);

chair_rec	chair%ROWTYPE;
pt_descr	presentation_type_codes.presentation_type_descr%TYPE;

begin

   open c2;
   fetch C2 into pt_descr;
   close c2;

   htp.p ('<session type="' || pt_descr || '">');
   attrib ('abbr', rec.session_id);
   tag ('name', rec.title);

   if full_extract then
      chair_rec := null;
      open chair (rec.chair_profile_id);
      fetch chair into chair_rec;
      close chair;

      attrib ('btime', to_char (rec.beg_time, 'FM0000'));
      attrib ('etime', to_char (rec.end_time, 'FM0000'));
      tag ('date', to_char (rec.session_date, 'dd-Mon-yy'));
      session_location (rec.session_id);

      if rec.chair_profile_id is NOT null then
         htp.p ('<chairs>');
         htp.p ('<chair>');
         name (chair_rec.lname, chair_rec.fname, chair_rec.mname, chair_rec.initials, chair_rec.person_id);

         htp.p ('<institutions>');
         affiliation (rec.chair_profile_id);
         htp.p ('</institutions>');

         emails (chair_rec.person_id);
         htp.p ('</chair>');
         htp.p ('</chairs>');
      end if;

      papers (rec.session_id);
   end if;

   htp.p ('</session>');

end one_session;

-- **********************************************************************************

procedure session (sid varchar2) is

cursor	driver is
select	*
from	sessions
where	session_id = sid;

rec	driver%ROWTYPE;

begin

   open driver;
   fetch driver into rec;
   close driver;

   full_extract := true;
   header;
   one_session (rec);
   footer;

end session;

-- **********************************************************************************

procedure sessions is

cursor	driver is
select	*
from	sessions
order by
	sort_order nulls last, session_date, beg_time, end_time, session_id;

begin

   for rec in driver LOOP
      one_session (rec);
   end loop;

end sessions;

-- **********************************************************************************
-- **********************************************************************************

procedure full (full_export boolean) is

cursor	driver is
select	count(1)
from	abstract_sessions
where	session_id is not null;

icount	integer;

begin

   full_extract := full_export;

   open driver;
   fetch driver into icount;
   close driver;

   header;

   if icount > 0 then
      sessions;
   else
      papers;
   end if;

   footer;

end full;

-- *********************************************************************************
-- *********************************************************************************

procedure author (fn varchar2, ln varchar2) is

cursor	driver is
select	1 sort_order, 'Exact' match_type,
	lname, fname, mname, initials, person_id
from	persons
where	sort_lname	= common.sort_text (ln)
and	sort_fname	= common.sort_text (fn)
UNION
select	2 sort_order, 'Last' match_type,
	lname, fname, mname, initials, person_id
from	persons
where	sort_lname	= common.sort_text (ln)
and	sort_fname	<> common.sort_text (fn)
UNION
select	3 sort_order, 'Fuzzy' match_type,
	lname, fname, mname, initials, person_id
from	persons
where	sort_lname	<> common.sort_text (ln)
and	soundex_lname	= soundex (ln)
order by 1;

begin

   header;
   htp.p ('<authors>');

   for rec in driver LOOP
      htp.p ('<author match="' || rec.match_type || '">');
      name (rec.lname, rec.fname, rec.mname, rec.initials, rec.person_id);
      institutions (rec.person_id);
      emails (rec.person_id);
      htp.p ('</author>');
   end loop;

   htp.p ('</authors>');
   footer;

end author;

-- *********************************************************************************

end XML2;
/
show error

--	Copyright (C) 2004 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>

set define on
