-- SPMS - Scientific Programme Management System
-- see at the bottom of this file for licensing
-- and copyright information

set define off

Create or Replace package BODY Xtract AS

-- ***********************************************************************************************

procedure regstats is

cursor driver  is
select	to_char(trunc(r.create_date), 'YYYY-MM-DD') || ', ' || count(trunc(r.create_date)) || ', ' ||
	decode(r.SPONSORED_BY,null,'D','S') STATS
from	registrants	r,
	pages		pg
where	'Y'		= nvl (pg.include_participant_rpt, 'Y')
and	pg.page_id	= r.page_id
group by trunc(r.create_date), decode(r.SPONSORED_BY,null,'D','S')
order by trunc(r.create_date), decode(r.SPONSORED_BY,null,'D','S');

begin
    owa_util.mime_header('text/csv',false, common.charset$);
    htp.p('Content-Disposition: filename="regstats.csv"');
    owa_util.http_header_close;
    for driver_rec in driver LOOP
	htp.p(driver_rec.STATS);
    end loop;
end regstats;

-- ***********************************************************************************************

procedure attendees is

cursor	driver is
Select q.lname,q.fname,common.GET_AFFIL_NAME(q.affiliation_id,' ','N')affiliation , q1.country_descr country, decode(q.sponsored_by,null,'D','S') type
from
	(Select distinct r.PERSON_ID, p.LNAME, p.FNAME, r.SPONSORED_BY,
		common.GET_USER_PRIMARY_AFFIL_ID(p.PERSON_ID) affiliation_id
	from	registrants r
	inner join persons p on r.PERSON_ID=p.PERSON_ID
	where page_id in (
	    SELECT page_id
	    FROM pages
	    where upper(nvl (include_participant_rpt, 'Y')) = 'Y'
      )
	) q
	inner join
		(Select distinct a.AFFILIATION_ID, cc.COUNTRY_DESCR
		 from addresses a
		 inner join country_codes cc on cc.COUNTRY_CODE=a.COUNTRY_CODE) q1
	    on	q1.AFFILIATION_ID=q.affiliation_id
order by type, country ,q.lname,q.fname;

begin

    owa_util.mime_header('text/csv',false, common.charset$);
    htp.p('Content-Disposition: filename="attendes.csv"');
    owa_util.http_header_close;
    for driver_rec in driver loop
	htp.prn('"' || driver_rec.lname ||'","' || driver_rec.fname ||'","' || driver_rec.affiliation ||'","' || driver_rec.country ||'","' ||driver_rec.type || '"' || common.crlf$);
    end loop;

end attendees;

-- ***********************************************************************************************

procedure abstractsubmissions ( chk varchar2 default '-', format varchar2 default 'csv' ) is

cursor	driver is
select	to_char(trunc(log_date), 'YYYY-MM-DD') || ', ' || count(1)  STATS
from	logs
where	log_code = 'AC'
group by to_char(trunc(log_date), 'YYYY-MM-DD')
order by 1;

begin
    if common.xtract_chk$ is not null and chk != common.xtract_chk$ then
      htp.p( 'access denied!' );
      return;
    end if;

    if format = 'csv' then
      owa_util.mime_header('text/csv',false, common.charset$);
      htp.p('Content-Disposition: filename="abstractsubmissions.csv"');
      owa_util.http_header_close;
    end if;

    htp.p('#abstractsubmissions:begin');
    for driver_rec in driver LOOP
	htp.p(driver_rec.STATS);
    end loop;
    htp.p('#abstractsubmissions:end');
end abstractsubmissions;

-- ***********************************************************************************************

procedure filesuploaded ( chk varchar2 default '-', what varchar2 default 'papercode' ) is

cursor	driver_total is
  select	count(1)
  from	abstracts;

cursor	driver_papercode is
select to_char(trunc(up_date), 'YYYY-MM-DD') || ', ' || count(abstract_id) STATS
from (
  select abstract_id, min(trunc(uploaded_date)) up_date
  from files
  group by abstract_id
)
group by up_date
order by up_date;

total_abstracts integer := 0;

begin
    if common.xtract_chk$ is not null and chk != common.xtract_chk$ then
      htp.p( 'access denied!' );
      return;
    end if;

    open driver_total;
      fetch driver_total into total_abstracts;
    close driver_total;

    if what = 'papercode' then
      owa_util.mime_header('text/csv',false, common.charset$);
      htp.p('Content-Disposition: filename="filesuploaded_papercode.csv"');
      owa_util.http_header_close;

      htp.p('#filesuploaded_papercode:begin');
      htp.p('#total_abstracts:' || total_abstracts);
      for driver_rec in driver_papercode loop
	      htp.p(driver_rec.stats);
      end loop;
      htp.p('#filesuploaded_papercode:end');

    end if;


end filesuploaded;

-- ***********************************************************************************************

procedure dates( chk varchar2 default '-', format varchar2 default 'csv' ) is

ids	assoc_arr;

lbl	varchar2(64);

begin

   if common.xtract_chk$ is not null and chk != common.xtract_chk$ then
      htp.p( 'access denied!' );
      return;
   end if;

   ids('abstracts_start')	:= common.abs_submission_start_date$;
   ids('abstracts_end')		:= common.abs_submission_end_date$;
   ids('conference_start')	:= to_char(common.conf_start_monday$,'MM/DD/YYYY');
   ids('contributions_end')	:= common.contributions_end_date$;
   ids('delegates_reg_start')	:= common.delegates_reg_start$;
   ids('delegates_reg_early')	:= common.delegates_reg_early$;

   htp.p('#dates:begin');

   lbl :=ids.FIRST;
   while lbl is not NULL loop
      if ids(lbl) is not null then
	 htp.p (lbl || ', ' || to_char(to_date(ids(lbl), 'MM/DD/YYYY'), 'YYYY-MM-DD') );
      end if;

      lbl :=ids.NEXT(lbl);
   end loop;

   htp.p('conference_end, ' ||	to_char(common.conf_start_monday$ +
				to_number(common.conf_len_days$),'YYYY-MM-DD') );
   htp.p('#dates:end');

end dates;


-- ***********************************************************************************************
-- getBoothStatus
-- returns a text list of booth numbers already booked according to a given meta data code (item)
-- optional parameters are bn (base number, from which number to start counting) and the
-- passphrase to be able to export the data
-- ***********************************************************************************************

procedure getBoothStatus(mdc varchar2, chk varchar2 default '-') is

cursor driver (mdcode number) is
select unique(p.poplist_item_id || ';' || p.item_value) item_value
from registrations r join poplist_items p on (r.poplist_item_id = p.poplist_item_id)
where r.meta_data_code=mdcode
order by item_value;

driver_rec    driver%ROWTYPE;
found	      boolean := false;

begin

    if common.xtract_chk$ is not null and chk != common.xtract_chk$ then
      htp.p( 'access denied!' );
      return;
    end if;

    for driver_rec in driver(mdc) loop
      htp.p(driver_rec.item_value);
    end loop;

end getBoothStatus;


-- ***********************************************************************************************

function yn_txt (code varchar2, txt varchar2) return varchar2 is

begin

   if txt = 'Picture Taken' then
      if code = 'Y' then
	 return ', ' || txt || '.';
      end if;
      return '.';
   end if;

   if code = 'N' then
      return 'Poster Not ' || txt;
   end if;

   return 'Poster ' || txt;

end yn_txt;

-- *********************************************************************************************

function status_txt (PP varchar2, PM varchar2, PS varchar2, PT varchar2) return varchar2 is

begin

   return	yn_txt (PP, 'Posted')		|| ', ' ||
		yn_txt (PM, 'Manned')		|| ', ' ||
		yn_txt (PS, 'Satisfactory')	||
		yn_txt (PT, 'Picture Taken');

end status_txt;

-- *********************************************************************************************



procedure posterpolicesetstatus ( chk varchar2 default '-',  pid number,
    aid number,
    pp poster_logs.posted%type default 'Y',
    pm poster_logs.manned%type default 'Y',
    ps poster_logs.satisfactory%type default 'Y',
    pt poster_logs.picture%type default 'Y',
    co poster_logs.comments%type default '' ) is

-- code from poster_police.save

cursor	driver (a integer) is
select	1
from	poster_logs
where	abstract_id	= A;

dummy		integer;
txt		varchar2(300);
rec_found	boolean;
add_rec		boolean;

begin
    if common.xtract_chk$ is not null and chk != common.xtract_chk$ then
      htp.p( 'access denied!' );
      return;
    end if;

    open driver (aid);
    fetch driver into dummy;
    rec_found := driver%FOUND;
    close driver;

    add_rec := false;
    if rec_found then
       update	poster_logs
       set	manned			= pm,
	    posted			= pp,
	    satisfactory	= ps,
	    picture			= pt,
	    comments		= trim (co)
       where	abstract_id		= aid
	   and (nvl(manned,'~')		<> nvl(pm,'~')
	   or	nvl(posted,'~')		<> nvl(pp,'~')
	   or	nvl(satisfactory,'~')	<> nvl(ps,'~')
	   or	nvl(picture,'~')	<> nvl(pt,'~')
	   or	nvl(comments,'~')	<> nvl(trim(co),'~') );

       if SQL%ROWCOUNT > 0 then -- poster police status changed
	  add_rec := true;
       end if;
    else
       add_rec := true;
    end if;

    if add_rec then
       insert into poster_logs (abstract_id, manned, posted, satisfactory, picture, comments)
       values (aid, pm, pp, ps, pt, trim(co) );

       txt := status_txt (pp, pm, ps, pt);
       insert into logs (abstract_id, log_code, comments, editor_notes, person_id)
       values (aid, common.poster_police$, trim(txt), trim(co), pid);
    end if;

    htp.p('OK');


end PosterPoliceSetStatus;

-- ***********************************************************************************************




end Xtract;

/
show error

set define on

--	Copyright (C) 2004-2008 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>
